package gov.nih.nci.codegen.util;

import gov.nih.nci.ncicb.xmiinout.domain.UMLAssociation;
import gov.nih.nci.ncicb.xmiinout.domain.UMLAttribute;
import gov.nih.nci.ncicb.xmiinout.domain.UMLClass;
import gov.nih.nci.ncicb.xmiinout.domain.UMLDatatype;
/**
 * EVSTransformerUtils provides functionality to process EVS specific tag values
 * defined in the EVS model.
 * @author Shaziya Muhsin
 */
public class EVSTransformerUtils extends TransformerUtils
{
//   private static Logger log = Logger.getLogger(EVSTransformerUtils.class);
   
   /**
    * Identifies the otherend datatypes.
    * 
    * @param assoc the assoc
    * 
    * @return the other end data type
    */   
   public static String getOtherEndDataType(UMLAssociation assoc){        
        String dataType = null;
        if(assoc.getTaggedValue("otherend-type")!=null){
            dataType = assoc.getTaggedValue("otherend-type").getValue();            
        }        
        return dataType;
    }
   
   /**
    * Annotates associations with @LazyLoad annotation.
    * 
    * @param assoc the assoc
    * 
    * @return true, if checks if is lazy load
    */
   public static boolean isLazyLoad(UMLAssociation assoc){        
       boolean lazyLoad = false;
       if(assoc.getTaggedValue("evs_lazy_load")!=null){
           String lazy = assoc.getTaggedValue("evs_lazy_load").getValue();
           if(lazy.equalsIgnoreCase("true")){
               lazyLoad = true;
           }
       }        
       return lazyLoad;
   }
   
   /**
    * Annotate attributes with @LazyLoad annotation.
    * 
    * @param attr the attr
    * 
    * @return true, if checks if is lazy load
    */
   public static boolean isLazyLoad(UMLAttribute attr){        
       boolean lazyLoad = false;
       if(attr.getTaggedValue("evs_lazy_load")!=null){
           String lazy = attr.getTaggedValue("evs_lazy_load").getValue();
           if(lazy.equalsIgnoreCase("true")){
               lazyLoad = true;
           }
       }        
       return lazyLoad;
   }
   
   /**
    * Supports addtional datatypes like Vectors.
    * 
    * @param attr the attr
    * 
    * @return the data type
    */
	public static String getDataType(UMLAttribute attr)
	{
        UMLDatatype dataType = attr.getDatatype();
		String name = dataType.getName();
		if(dataType instanceof UMLClass)
			name = getFQCN((UMLClass)dataType);

		if(name.startsWith("java.lang."))
			name = name.substring("java.lang.".length());

		if("int".equalsIgnoreCase(name) || "integer".equalsIgnoreCase(name))
			return "Integer";
		if("double".equalsIgnoreCase(name))
			return "Double";
		if("float".equalsIgnoreCase(name))
			return "Float";
		if("long".equalsIgnoreCase(name))
			return "Long";
		if("string".equalsIgnoreCase(name))
			return "String";
		if("char".equalsIgnoreCase(name) || "character".equalsIgnoreCase(name))
			return "Character";
		if("boolean".equalsIgnoreCase(name) )
			return "Boolean";

		if("date".equalsIgnoreCase(name) || "java.util.date".equalsIgnoreCase(name))
			return "java.util.Date";
        if(name.startsWith("java.util.")){
            return name;
        }
		return name;
	}


}