package gov.nih.nci.evs.util;

import gov.nih.nci.evs.domain.*;
import org.LexGrid.concepts.CodedEntry;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2001-2004 SAIC. Copyright 2001-2003 SAIC. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by the SAIC and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "SAIC" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or SAIC-Frederick.
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* SAIC, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */
/**
  * @author EVS Team
  * @version 4.1
 */
/**
 * Implements methods for supporting Quick Sort of DescLogicConcept, MetaThesaurusConcept, and CodedEntry by name or code.
 */


public class SortUtils {

    public static int SORT_BY_NAME = 1;
    public static int SORT_BY_CODE = 2;


    public SortUtils() {

	}


	 /**
	 * Swaps two elements in a list.
	 *
	 * @param A the list
	 * @param x the position of the first element in the list
	 * @param y the position of the second element in the list
	 */

    public static void swap (List A, int x, int y)
    {
        Object temp = A.get(x);
        A.set(x, A.get(y));
        A.set(y, temp);
    }


	 /**
	 * Gets the key of the object.
	 *
	 * @param c the instance of the Object
	 *
	 * @return the key (a String) for the object
	 *
	 */
    private static String getKey(Object c)
    {
        return getKey(c, SORT_BY_NAME);
    }


	 /**
	 * Gets the key of the object.
	 *
	 * @param c the instance of the Object
	 * @param sort_option, an integer; 1, if sort by name; 2: if sort by code
	 *
	 * @return the key (a String) for the object
	 *
	 */
    private static String getKey(Object c, int sort_option)
    {
	    if (c instanceof DescLogicConcept)
	    {
		    DescLogicConcept concept = (DescLogicConcept) c;
		    if (sort_option == SORT_BY_NAME) return concept.getName();
		    return concept.getCode();
	    }
	    else if (c instanceof MetaThesaurusConcept)
	    {
			MetaThesaurusConcept concept = (MetaThesaurusConcept) c;
			if (sort_option == SORT_BY_NAME) return concept.getName();
			return concept.getCui();
	    }
	    else if (c instanceof CodedEntry)
	    {
			CodedEntry concept = (CodedEntry) c;
			if (sort_option == SORT_BY_NAME) return concept.getEntityDescription().getContent();
			return concept.getConceptCode();
		}
	    return "NA";
    }


	 /**
	 * Finds the partition position.
	 *
	 * @param A an instance of List
	 * @param i the first position
	 * @param j the second position
	 *
	 * @return the pivoting position.
	 *
	 */
    private static int partition(List A, int i, int j)
    {
	    return partition(A, i, j, SORT_BY_NAME);
    }



	 /**
	 * Finds the pivoting position.
	 *
	 * @param A an instance of List
	 * @param i the first position
	 * @param j the second position
	 * @param sort_option, an integer; 1, if sort by name; 2: if sort by code
	 *
	 * @return the pivoting position for quick sort.
	 *
	 */
    private static int partition(List A, int i, int j, int sort_option)
    {
 	    Object c0 = A.get(i);
 	    String key_0 = getKey(c0, sort_option);

 	    Object c1 = A.get(i);
 	    String key_1 = getKey(c1, sort_option);

 	    Object c2 = A.get(j);
        String key_2 = getKey(c2, sort_option);

        while (i < j)
        {
 	  	   c1 = (Object) A.get(i);
 		   c2 = (Object) A.get(j);
  		   key_1 = getKey(c1, sort_option);
 		   key_2 = getKey(c2, sort_option);

           while (key_1.compareToIgnoreCase(key_0) < 0)
           {
 			  i++;
 			  c1 = A.get(i);
 			  key_1 = getKey(c1, sort_option);
 		   }

           while (key_2.compareToIgnoreCase(key_0) > 0)
           {
 			  j--;
 			  c2 = A.get(j);
 			  key_2 = getKey(c2, sort_option);
 		  }
           if (i != j) swap (A, i, j);
       }
       return i;
   }

	 /**
	 * Performs quick sort of a List given two positions.
	 *
	 * @param A an instance of List
	 * @param i the starting position
	 * @param j the end position
	 *
	 */
   private static void quickSort(List A, int i, int j)
   {
       quickSort(A, i, j, SORT_BY_NAME);
   }

	 /**
	 * Performs quick sort of a List given two positions.
	 *
	 * @param A an instance of List
	 * @param i the starting position
	 * @param j the end position
	 * @param sort_option, an integer; 1, if sort by name; 2: if sort by code
	 *
	 */
   private static void quickSort(List A, int i, int j, int sort_option)
   {
       if (i >= j) return;
       int k = partition(A, i, j, sort_option);
       quickSort(A, i, k, sort_option);
       quickSort(A, k+1, j, sort_option);
   }


	 /**
	 * Performs quick sort of a List by name.
	 *
	 * @param A an instance of List
	 */

   public static void quickSort(List A)
   {
	   quickSort(A, SORT_BY_NAME);
   }


	 /**
	 * Performs quick sort of a List by a specified sort option.
	 *
	 * @param A an instance of List
	 * @param sort_option, an integer; 1, if sort by name; 2: if sort by code
	 */
   public static void quickSort(List A, int sort_option)
   {
	   if(A.size() <= 1) return;
	   quickSort(A, 0, A.size()-1, sort_option);
   }

}

