package gov.nih.nci.lexbig.ext;

import java.io.Serializable;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */

/** EditActionDate
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */
@SuppressWarnings("serial")
public class EditActionDate implements Serializable {


	//010904
	/** Constant string for Create action
	 */
	public final static String EDIT_ACTION_CREATE_STR = "create";
	/** Constant string for Merge action
	 */
	public static final String EDIT_ACTION_MERGE_STR = "merge";
	/** Constant string for Modify action
	 */
	public static final String EDIT_ACTION_MODIFY_STR = "modify";
	/** Constant string for Retire action
	 */
	public static final String EDIT_ACTION_RETIRE_STR = "retire";
	/** Constant string for Split action
	 */
	public static final String EDIT_ACTION_SPLIT_STR = "split";


	/** Constant string for Create action
	 */
	public final static int EDIT_ACTION_CREATE = 0;
	/** Constant string for Merge action
	 */
	public static final int EDIT_ACTION_MERGE = 2;
	/** Constant string for Modify action
	 */
	public static final int EDIT_ACTION_MODIFY = 4;
	/** Constant string for Retire action
	 */
	public static final int EDIT_ACTION_RETIRE = 1;
	/** Constant string for Split action
	 */
	public static final int EDIT_ACTION_SPLIT = 3;


	/** Edit date
	 */
	//java.util.Date Date editdate;
	java.util.Date editdate;
	/** Edit action indexAction
	 */
	int action;


	/** Default constructor
	 */
	public EditActionDate() {
		this.action = 0;
	}

	/**
	 * Converts action to String.
	 * 
	 * @param iAction action id
	 * 
	 * @return the string
	 */
	public static String actionToString(int iAction)
	{
		if (iAction == EDIT_ACTION_CREATE) return EDIT_ACTION_CREATE_STR;
		else if (iAction == EDIT_ACTION_CREATE) return EDIT_ACTION_CREATE_STR;
		else if (iAction == EDIT_ACTION_MERGE) return EDIT_ACTION_MERGE_STR;
		else if (iAction == EDIT_ACTION_MODIFY) return EDIT_ACTION_MODIFY_STR;
		else if (iAction == EDIT_ACTION_RETIRE) return EDIT_ACTION_RETIRE_STR;
		else if (iAction == EDIT_ACTION_SPLIT) return EDIT_ACTION_SPLIT_STR;
		return "Unidentifiable action";
	}

	/**
	 * Converts Action to int.
	 * 
	 * @param action the action
	 * 
	 * @return the int
	 */
	public static int actionToId(String action)
	{
		if (action.equals(EDIT_ACTION_CREATE_STR)) return EDIT_ACTION_CREATE;
		else if (action.equals(EDIT_ACTION_MERGE_STR)) return EDIT_ACTION_MERGE;
		else if (action.equals(EDIT_ACTION_MODIFY_STR)) return EDIT_ACTION_MODIFY;
		else if (action.equals(EDIT_ACTION_RETIRE_STR)) return EDIT_ACTION_RETIRE;
		else if (action.equals(EDIT_ACTION_SPLIT_STR)) return EDIT_ACTION_SPLIT;
		return -1;
	}

	/**
	 * Sets the edit date.
	 * 
	 * @param editdate the new edit date
	 */
	public void setEditDate(java.util.Date editdate) {
		this.editdate = editdate;
	}

	/** Assigns edit action index.
	 *
	 * @param action edit action index
	 */
	public void setAction(int action) {
		this.action = action;
	}

	/** Gets editdate
	 *
	 * @return editdate
	 */
	public java.util.Date getEditDate() {
		return this.editdate;
	}

	/** Gets action
	 *
	 * @return action
	 */
	public int getAction() {
		return this.action;
	}

	/** Gets a XML representation of EditActionDate object in String
	 *
	 * @return a XML representation of EditActionDate object in String
	 */
	public String toXML() {
		return writeDocument(toDOM());
	}

	/** Converts a XML Document to a string
	 *
	 * @param doc a XML Document
	 * @return a String representation of a XML Document
	 */
	private String writeDocument(Document doc)
	{
	    try {
	        OutputFormat format = new OutputFormat(doc);
			StringWriter stringOut = new StringWriter();
	        XMLSerializer serial = new XMLSerializer(stringOut, format);
	        serial.asDOMSerializer();
	        serial.serialize(doc.getDocumentElement());
	        return stringOut.toString();
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
	    return null;
	}

	/** Gets a String representation of an EditActionDate object
	 *
	 * @return a String representation of an EditActionDate object
	 */
	public String toString() {
		return toXML();
	}

	/** Converts an EditActionDate object to a XML Document
	 *
	 * @return a XML Document representation of an EditActionDate object
	 */
	public Document toDOM()
	{
		Document doc= new DocumentImpl();
		try {
			Element root = doc.createElement("EditActionDate");
			Element element;
			element = doc.createElement("editdate");
			Date date_editdate = getEditDate();
			element.appendChild(doc.createTextNode(dateToString(date_editdate)));
			root.appendChild(element);

			element = doc.createElement("action");
			int int_action = getAction();
			element.appendChild(doc.createTextNode(intToString(int_action)));
			root.appendChild(element);

			doc.appendChild(root);
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		return doc;
	}

	/** Converts a Date to a String
	 *
	 * @param aDate a Date object
	 * @return a String representation of a Date object
	 */
	private String dateToString(java.util.Date aDate)
	{
		if (aDate == null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String theDate = sdf.format(aDate);
		return theDate;
	}


	/** Converts an int value to a String
	 *
	 * @param i an int value
	 * @return a String representation of an int value
	 */
	private String intToString(int i)
	{
		return "" + i;
	}


} // end of EditActionDate
