package gov.nih.nci.lexbig.ext;

import java.io.Serializable;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */

/** HistoryRecord
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version DLBExt 1.0
 */
@SuppressWarnings("serial")
public class HistoryRecord implements Serializable {

	/** Concept code
	 */
	String code;

	/** Namespace Id
	 */
	int namespaceId;

	/** edit action
	 */
	String editaction;

	/** edit date
	 */
	java.util.Date editdate;

	/** referenced code
	 */
	String reference;


	/** Default constructor
	 *
	 * @since DLBExt 1.0
	 */
	public HistoryRecord() {
		this.code = "";
		this.namespaceId = 0;
		this.editaction = "";
		this.editdate = null;
		this.reference = "";
	}

	/** Constructor
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 * @param namespaceId namespace id
	 * @param editaction edit action
	 * @param editdate edit date
	 * @param reference reference code
	 */
	public HistoryRecord(String code,
	                     int namespaceId,
	                     String editaction,
	                     java.util.Date editdate,
	                     String reference) {
		this.code = code;
		this.namespaceId = namespaceId;
		this.editaction = editaction;
		this.editdate = editdate;
		this.reference = reference;
	}


/////////////////////////////////////////////////////////////////////////////////////

	/** Gets code value
	 *
	 * @since DLBExt 1.0
	 * @return concept code
	 */
	public String getCode() {
		return code;
	}

	/** Gets namespaceId value
	 *
	 * @since DLBExt 1.0
	 * @return namespace id
	 */
	public int getNamespaceId() {
		return namespaceId;
	}

	/** Gets edit action value
	 *
	 * @since DLBExt 1.0
	 * @return edit action
	 */
	public String getEditAction() {
		return editaction;
	}


	/** Gets creation date value
	 *
	 * @since DLBExt 1.0
	 * @return edit date
	 */
	public java.util.Date getEditDate() {
		return editdate;
	}

	/** Gets referenced code value
	 *
	 * @since DLBExt 1.0
	 * @return reference code
	 */
	public String getReference() {
		return reference;
	}

/////////////////////////////////////////////////////////////////////////////////////

	/** Assigns code value
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 */
	public void setCode(String code) {
		this.code = code;
	}

	/** Assigns namespaceId value
	 *
	 * @since DLBExt 1.0
	 * @param namespaceId namespace id
	 */
	public void setNamespaceId(int namespaceId) {
		this.namespaceId = namespaceId;
	}

	/** Assigns edit action value
	 *
	 * @since DLBExt 1.0
	 * @param editaction edit action
	 */
	public void setEditAction(String editaction) {
		this.editaction = editaction;
	}


	/** Assigns edit date value
	 *
	 * @since DLBExt 1.0
	 * @param editdate edit date of a concept
	 */
	public void setEditDate(java.util.Date editdate) {
		this.editdate = editdate;
	}

	/** Assigns referenced code value
	 *
	 * @since DLBExt 1.0
	 * @param reference referenced code
	 */
	public void setReference(String reference) {
		this.reference = reference;
	}


/////////////////////////////////////////////////////////////////////////////////



	/** Gets a String representation of a HistoryRecord object in XML format.
	 *
	 * @return a String representation of a HistoryRecord object in XML format
	 */

	 /*
	public String toXML() {
		return writeDocument(toDOM());
	}
	*/

	/** Converts a Document to a String
	 *
	 * @param doc an XML Document
	 * @return a String representation of an XML Document
	 */

/*
	private String writeDocument(Document doc)
	{
	    try {
	        OutputFormat format = new OutputFormat(doc);
			StringWriter stringOut = new StringWriter();
	        XMLSerializer serial = new XMLSerializer(stringOut, format);
	        serial.asDOMSerializer();
	        serial.serialize(doc.getDocumentElement());
	        return stringOut.toString();
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
	    return null;
	}
*/

	/** Gets a String representation of the HistoryRecord object.
	 *
	 * @return a String representation of the HistoryRecord object
	 */

	 /*
	public String toString() {
		return toXML();
	}
	*/

	/** Gets a XML Document representation of the HistoryRecord object.
	 *
	 * @return a XML Document representation of the HistoryRecord object
	 */

/*
	public Document toDOM()
	{
		Document doc= new DocumentImpl();
		try {
			Element root = doc.createElement("HistoryRecord");
			Element element;
			element = doc.createElement("code");
			element.appendChild(doc.createTextNode(code));
			root.appendChild(element);

			element = doc.createElement("name");
			element.appendChild(doc.createTextNode(name));
			root.appendChild(element);

			element = doc.createElement("creationDate");
			Date date_creationdate = getCreationDate();
			element.appendChild(doc.createTextNode(dateToString(date_creationdate)));
			root.appendChild(element);

			element = doc.createElement("isRetired");
			boolean boolean_isretired = getIsRetired();
			element.appendChild(doc.createTextNode(booleanToString(boolean_isretired)));
			root.appendChild(element);

			Element parentElement;
			Element childElement;

			parentElement = doc.createElement("EditActionDates");
			for (int i=0; i<editactiondate_vec.size(); i++)
			{
				EditActionDate e = (EditActionDate) editactiondate_vec.elementAt(i);
				Element editactiondate = doc.createElement("EditActionDate");

				childElement = doc.createElement("editdate");
				Date date_editdate = e.getEditDate();
				childElement.appendChild(doc.createTextNode(dateToString(date_editdate)));
				editactiondate.appendChild(childElement);

				childElement = doc.createElement("action");
				int int_action = e.getAction();
				childElement.appendChild(doc.createTextNode(intToString(int_action)));
				editactiondate.appendChild(childElement);

				parentElement.appendChild(editactiondate);
			}
			root.appendChild(parentElement);
			doc.appendChild(root);
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		return doc;
	}
*/

//	/** Converts a Date to a String.
//	 *
//	 * @since DLBExt 1.0
//	 * @param aDate a Date object
//	 * @return a String representation of a Date object
//	 */
//	private String dateToString(java.util.Date aDate)
//	{
//		if (aDate == null) return null;
//		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//		String theDate = sdf.format(aDate);
//		return theDate;
//	}


//	/** Converts an int value to a String.
//	 *
//	 * @since DLBExt 1.0
//	 * @param i an int value
//	 * @return a String representation of an int value
//	 */
//	private String intToString(int i)
//	{
//		return "" + i;
//	}


} // end of HistoryRecord
