package gov.nih.nci.lexbig.ext;


import java.io.Serializable;
import java.io.StringWriter;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** Qualifier
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version DLBExt 1.0
 */

public class Qualifier implements Serializable {

	/** namespace id
	 */
	int namespaceId;
	/** qualifier name
	 */
	String name;
	/** qualifier value
	 */
	String value;


	/** Default constructor
	 */
	public Qualifier() {
		this.namespaceId = -1;
		this.name = "";
		this.value = "";
	}

	/** Constructor
	 *
	 * @since DLBExt 1.0
	 * @param name qualifier name
	 * @param value qualifier value
	 */
	public Qualifier(String name, String value) {
		this.namespaceId = -1;
		this.name = name;
		this.value = value;
	}

	/** Constructor
	 *
	 * @since DLBExt 1.0
	 * @param name qualfier name
	 * @param value qualfier value
	 * @param namespaceId namespace id
	 */
	public Qualifier(String name, String value, int namespaceId) {
		this.namespaceId = namespaceId;
		this.name = name;
		this.value = value;
	}

	/** Sets value to namespaceId
	 *
	 * @since DLBExt 1.0
	 * @param namespaceId namespace id
	 */
	public void setNamespaceId(int namespaceId) {
		this.namespaceId = namespaceId;
	}

	/** Sets value to qualifier name
	 *
	 * @since DLBExt 1.0
	 * @param name qualifier name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/** Sets value to qualifier value
	 *
	 * @since DLBExt 1.0
	 * @param value qualifier value
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/** Gets namespaceId
	 *
	 * @since DLBExt 1.0
	 * @return namespaceId
	 */
	public int getNamespaceId() {
		return this.namespaceId;
	}

	/** Gets name
	 *
	 * @since DLBExt 1.0
	 * @return name
	 */
	public String getName() {
		return this.name;
	}

	/** Gets value
	 *
	 * @return value
	 */
	public String getValue() {
		return this.value;
	}

	/** Gets XML formatted string
	 *
	 * @since DLBExt 1.0
	 * @return a XML formatted string
	 */
	public String toXML() {
		return writeDocument(toDOM());
	}

	/** Writes a XML document
	 *
	 * @since DLBExt 1.0
	 * @param doc XML Document
	 * @return a XML formatted string
	 */
	private String writeDocument(Document doc)
	{
	    try {
	        OutputFormat format = new OutputFormat(doc);
			StringWriter stringOut = new StringWriter();
	        XMLSerializer serial = new XMLSerializer(stringOut, format);
	        serial.asDOMSerializer();
	        serial.serialize(doc.getDocumentElement());
	        return stringOut.toString();
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
	    return null;
	}

	/** Converts to a String
	 *
	 * @since DLBExt 1.0
	 * @return a String
	 */
	public String toString() {
		return this.name + "$" + this.value;
	}

	/** Converts to a XML document
	 *
	 * @since DLBExt 1.0
	 * @return a XML document
	 */
	public Document toDOM()
	{
		Document doc= new DocumentImpl();
		try {
			Element root = doc.createElement("Qualifier");
			Element element;
			element = doc.createElement("namespaceId");
			element.appendChild(doc.createTextNode(intToString(namespaceId)));
			root.appendChild(element);

			element = doc.createElement("name");
			element.appendChild(doc.createTextNode(name));
			root.appendChild(element);

			element = doc.createElement("value");
			element.appendChild(doc.createTextNode(value));
			root.appendChild(element);

			doc.appendChild(root);
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		return doc;
	}

	/** Converts an int value Date to a String
	 *
	 * @since DLBExt 1.0
	 * @param i an int value
	 * @return a String representation of an int value
	 */
	private String intToString(int i)
	{
		return "" + i;
	}

} // end of Qualifier
