package gov.nih.nci.lexbig.ext;

import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.client.ApplicationServiceProvider;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.ConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.LocalNameList;
import org.LexGrid.LexBIG.DataModel.Collections.NameAndValueList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.SortOptionList;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeSummary;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ConceptReference;
import org.LexGrid.LexBIG.DataModel.Core.NameAndValue;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.CodingSchemeRendering;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Impl.LexBIGServiceImpl;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeGraph;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.SearchDesignationOption;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.LexBIG.Utility.Iterators.ResolvedConceptReferencesIterator;
import org.LexGrid.LexBIG.Utility.LBConstants.MatchAlgorithms;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.naming.SupportedAssociation;
import org.LexGrid.naming.SupportedProperty;

import org.LexGrid.naming.SupportedPropertyQualifier;
import org.apache.log4j.Logger;

/**
 * The Class SearchUtil.
 */
public class SearchUtil {

	/** The Constant SEARCH_STARTS_WITH. */
	public static final int SEARCH_STARTS_WITH = 1;

	/** The Constant SEARCH_ENDS_WITH. */
	public static final int SEARCH_ENDS_WITH = 2;

	/** The Constant SEARCH_CONTAINS. */
	public static final int SEARCH_CONTAINS = 3;

	/** The Constant SEARCH_EXACT_MATCH. */
	public static final int SEARCH_EXACT_MATCH = 4;


	private static Logger log = Logger.getLogger("SearchUtil");

	static final List<String> STOP_WORDS = Arrays.asList(new String[] {"a", "an", "of", "the"});


//	private ResolvedConceptReferencesIterator searchResultIterator = null;

	private LexBIGService lbSvc;
//	private String url;
	private String codingScheme_Name;
	private String version;
	private CodingScheme codingScheme;
	
	//GF# 14545
	private String _service = "EvsServiceInfo";
	
//	private String tag = null;

	//DLBAdapter adapter = null;

	/**
	 * Instantiates a new search util.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param version the version
	 * @param lurl the url
	 */
	public SearchUtil(String codingSchemeName, String version, String lurl )
	{
//		this.url = lurl;
		this.lbSvc = createLexBIGService(lurl);
		this.codingScheme_Name= codingSchemeName;
		this.version = version;
		try {
			this.codingScheme = getCodingScheme(codingSchemeName, version);
	    } catch (Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 * Instantiates a new search util.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param version the version
	 * @param lbSvc the lb svc
	 */
	public SearchUtil(String codingSchemeName, String version, LexBIGService lbSvc )
	{
//		this.url = url;
		this.lbSvc = lbSvc;
		this.codingScheme_Name= codingSchemeName;
		this.version = version;
		try {
			this.codingScheme = getCodingScheme(codingSchemeName, version);
	    } catch (Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 * Creates the lex big service.
	 *
	 * @param lurl the url
	 *
	 * @return the lex big service
	 */
	public LexBIGService createLexBIGService(String lurl)
    {
//		LexBIGService lbSvc = null;
		try {
			if (lurl == null)
			{
				return new LexBIGServiceImpl();
			}
			else if (lurl.compareTo("") == 0)
			{
				return new LexBIGServiceImpl();
			}

			try {
				return (EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(lurl, _service);
			} catch (Exception e) {
				return null;
		    }
		} catch (Exception e) {
			e.printStackTrace();
            return null;
        }
	}

    /**
     * Gets the supported associations.
     *
     * @return the supported association names
     */
    public Vector<String> getSupportedAssociations()
	{
		if (codingScheme == null) return null;
        Vector<String> v = new Vector<String>();
	    SupportedAssociation[] associations = codingScheme.getMappings().getSupportedAssociation();
		for (int i=0; i<associations.length; i++)
		{
		     SupportedAssociation asso = (SupportedAssociation) associations[i];
		     v.add(asso.getLocalId());
		}
        return v;
	}


	/**
	 * Gets the coding scheme.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 *
	 * @return the coding scheme
	 *
	 * @throws Exception the exception
	 */
	public CodingScheme getCodingScheme(String codingSchemeName, String vers) throws Exception {
		try {
			CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalname = css.getFormalName();
				if (formalname.compareTo(codingSchemeName) == 0) {
					CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
					vt.setVersion(vers);
					CodingScheme scheme = lbSvc.resolveCodingScheme(formalname, vt);
					return scheme;
				}
				String localName = css.getLocalName();
				if (localName.compareTo(codingSchemeName) == 0) {
					CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
					vt.setVersion(vers);
					CodingScheme scheme = lbSvc.resolveCodingScheme(formalname, vt);
					return scheme;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			//System.out.println("*** getCodingScheme exception.");
		}
		return null;
	}

	/**
	 * Restrict to source.
	 *
	 * @param cns the CodedNodeSet
	 * @param source the source
	 *
	 * @return the coded node set
	 */
	protected CodedNodeSet restrictToSource(CodedNodeSet cns, String source)
	{
		if (source == null || source.compareTo("*") == 0) return cns;

		//TODO - there should be a more efficient way to do this.
		Vector<String> v = new Vector<String>();
		v.add(source);
	    LocalNameList sourceList = vector2LocalNameList(v);
	    //

		CodedNodeSet.PropertyType[] types = new CodedNodeSet.PropertyType[] {CodedNodeSet.PropertyType.PRESENTATION};
		try {

			cns = cns.restrictToProperties(null, types, sourceList, null, null);

		} catch (Exception e) {
   			System.out.println("restrictToSource error. Source: " + source);
		    return null;
		}
		return cns;
	}


	/**
	 * Gets the searchable properties for coding scheme.
	 *
	 * @return the String array of property names
	 */
	public String[] getPropertyForCodingSchemeSearch(){
		int count = codingScheme.getMappings().getSupportedPropertyCount();
		ArrayList<String> lst = new ArrayList<String>();
//		Mappings mappings = codingScheme.getMappings();
		SupportedProperty[] sp = codingScheme.getMappings().getSupportedProperty();
		for(int i=0;i<count ;i++){
			SupportedProperty prop = sp[i];
			if( prop!=null && prop.getLocalId()!=null && !prop.getLocalId().equalsIgnoreCase("textualPresentation")){
				lst.add(prop.getLocalId());
			}
		}
		String[] str = new String[lst.size()];
		for(int j=0;j<lst.size();j++){
			str[j] = lst.get(j).toString();
		}

		return str;
	}

	private String replacePeriod(String s){
		String temp = "";
		if(s.indexOf(".") < 0)
			return s;
		while(s.indexOf(".") >= 0){
			temp = temp + s.substring(0,s.indexOf(".")) + "\\.";
			s = s.substring(s.indexOf(".") +1);
		}
		temp += s;
		return temp;
	}

	/**
	 * Converts a Vector to a LocalNameList
	 *
	 * @param v the Vector
	 *
	 * @return the local name list
	 */
	public LocalNameList vector2LocalNameList(Vector v)
	{
		if (v == null) return null;
		LocalNameList list = new LocalNameList();
		for (int i=0; i<v.size(); i++)
		{
			String vEntry = (String) v.elementAt(i);
			list.addEntry(vEntry);
		}
        return list;
	}


	/**
	 * Creates the name and value list from arrays of names and values
	 *
	 * @param names the names array
	 * @param values the values array
	 *
	 * @return the NameAndValueList
	 */
	private NameAndValueList createNameAndValueList(String[] names, String[] values)
	{
		NameAndValueList nvList = new NameAndValueList();
		for (int i=0; i<names.length; i++)
		{
			NameAndValue nv = new NameAndValue();
			nv.setName(names[i]);
			if (values != null)
			{
				nv.setContent(values[i]);
			}
			nvList.addNameAndValue(nv);
		}
		return nvList;
	}


    private ConceptReferenceList resolvedConceptReferenceListToConceptReferenceList(ResolvedConceptReferenceList rcrl)
    {
		if (rcrl == null) return null;
		ResolvedConceptReference[] rcr_array =	rcrl.getResolvedConceptReference();
		if (rcr_array == null)
		{
			return null;
		}

		if (rcr_array.length == 0) return null;
		ConceptReferenceList crl = new ConceptReferenceList();

		for (int i=0; i<rcr_array.length; i++)
		{
			ResolvedConceptReference rcr = rcr_array[i];
//			CodedEntry ce =rcr.getReferencedEntry();
			crl.addConceptReference(rcr);
		}
		return crl;
    }

	/**
	 * Search nodes for definition.
	 *
	 * @param names the list of search terms
	 * @param soundsLike use sounds like search
	 * @param searchInactive whether to include inactive concept
	 *
	 * @return the resolved concept references iterator
	 */
	public ResolvedConceptReferencesIterator searchNodesForDefinition(List names,  boolean soundsLike, boolean searchInactive){
		ResolvedConceptReferencesIterator matchIterator = null;
		try{
			String scheme = codingScheme.getRegisteredName();
			CodedNodeSet nodes = lbSvc.getCodingSchemeConcepts(scheme, null);
			String name = null;
			Iterator namesIte = names.iterator();
			String matchAlgorithm = "RegExp";
			if(soundsLike)
				matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

			while(namesIte.hasNext()){
				name = (String)namesIte.next();
				name = name.toLowerCase();
				name = replacePeriod(name);
				if(name.indexOf(" ") > 0){
					name = "\"" + name + "\"";
					matchAlgorithm = MatchAlgorithms.LuceneQuery.name();
				}
				else{

					if(!soundsLike){
						name = ".*" + name + ".*";
						matchAlgorithm = "RegExp";

					}
				}

				if(soundsLike)
					matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

/*
 CodedNodeSet 	restrictToMatchingProperties(LocalNameList propertyNames,
                                             CodedNodeSet.PropertyType[] propertyTypes,
                                             java.lang.String matchText,
                                             java.lang.String matchAlgorithm,
                                             java.lang.String language)
 */
				nodes = nodes.restrictToMatchingProperties(
					null,
					new PropertyType[] {PropertyType.DEFINITION},
					name,
					soundsLike?MatchAlgorithms.DoubleMetaphoneLuceneQuery.name():matchAlgorithm,
					null);

				// GF9215
				boolean activeOnly = !searchInactive;
				nodes = restrictToActiveStatus(nodes, activeOnly);
			}

			SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
			matchIterator =
				nodes.resolve(sortCriteria,null,null);
		}catch(Exception e){
			e.printStackTrace();
		}
		return matchIterator;

	}


	/**
	 * Search nodes for definition.
	 *
	 * @param names the list of search terms
	 * @param count - parameter not used
	 * @param soundsLike use sounds like search
	 * @param searchInactive whether to include inactive concept
	 *
	 * @return the resolved concept references iterator
	 * @deprecated - use searchNodesForDefinition(List,boolean,boolean)
	 */@Deprecated
	public ResolvedConceptReferencesIterator searchNodesForDefinition(List names,@SuppressWarnings("unused") int count,
			boolean soundsLike, boolean searchInactive){
		ResolvedConceptReferencesIterator matchIterator = null;
		try{
			String scheme = codingScheme.getRegisteredName();
			CodedNodeSet nodes = lbSvc.getCodingSchemeConcepts(scheme, null);
			String name = null;
			Iterator namesIte = names.iterator();
			String matchAlgorithm = "RegExp";
			if(soundsLike)
				matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

			while(namesIte.hasNext()){
				name = (String)namesIte.next();
				name = name.toLowerCase();
				name = replacePeriod(name);
				if(name.indexOf(" ") > 0){
					name = "\"" + name + "\"";
					matchAlgorithm = MatchAlgorithms.LuceneQuery.name();
				}
				else{

					if(!soundsLike){
						name = ".*" + name + ".*";
						matchAlgorithm = "RegExp";

					}
				}

				if(soundsLike)
					matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

/*
 CodedNodeSet 	restrictToMatchingProperties(LocalNameList propertyNames,
                                             CodedNodeSet.PropertyType[] propertyTypes,
                                             java.lang.String matchText,
                                             java.lang.String matchAlgorithm,
                                             java.lang.String language)
 */
				nodes = nodes.restrictToMatchingProperties(
					null,
					new PropertyType[] {PropertyType.DEFINITION},
					name,
					soundsLike?MatchAlgorithms.DoubleMetaphoneLuceneQuery.name():matchAlgorithm,
					null);

				// GF9215
				boolean activeOnly = !searchInactive;
				nodes = restrictToActiveStatus(nodes, activeOnly);
			}

			SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
			matchIterator =
				nodes.resolve(sortCriteria,null,null);
		}catch(Exception e){
			e.printStackTrace();
		}
		return matchIterator;

	}


	/**
	 * Search nodes for properties.
	 *
	 * @param names the list of search terms
	 * @param count the count
	 * @param soundsLike whether to use the sounds like search
	 * @param searchInactive whether to include inactive concepts
	 *
	 * @return the resolved concept references iterator
	 */
	public ResolvedConceptReferencesIterator searchNodesForProperties(List names, int count,boolean soundsLike,
		boolean searchInactive){
        return searchNodesForProperties(names, count, soundsLike, searchInactive, null, null, null);
	}


	/**
	 * Search nodes for properties.
	 *
	 * @param names the list of search terms
	 * @param soundsLike whether to use the sounds like search
	 * @param searchInactive whether to include inactive concepts
	 * @param searchSource the search source
	 * @param roleToSearchIn the role to search in
	 * @param associationToSearchIn the association to search in
	 *
	 * @return the resolved concept references iterator
	 */
	public ResolvedConceptReferencesIterator searchNodesForProperties(List names, boolean soundsLike,
		boolean searchInactive, String searchSource, String roleToSearchIn, String associationToSearchIn){
		ResolvedConceptReferencesIterator matchIterator = null;
		try{
			String scheme = codingScheme.getRegisteredName();
			CodedNodeSet nodes1 = null;
			CodedNodeSet nodes2 = lbSvc.getCodingSchemeConcepts(scheme, null);

			String name = null;
			Iterator namesIte = names.iterator();
			String[] propList = getPropertyForCodingSchemeSearch();
			if(propList.length > 0)
				nodes1 = lbSvc.getCodingSchemeConcepts(scheme, null);
			String matchAlgorithm = "RegExp";
			if(soundsLike)
				matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

			while(namesIte.hasNext()){
				name = (String)namesIte.next();
				name = name.toLowerCase();
				if(name.indexOf(" ") > 0){
					name = "\"" + name + "\"";
					matchAlgorithm = MatchAlgorithms.LuceneQuery.name();
				}
				else{

					if(!soundsLike){
						name = ".*" + name + ".*";
						matchAlgorithm = "RegExp";
					}
				}
				if(soundsLike)
					matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

				if(propList.length > 0){

					nodes1 = nodes1.restrictToMatchingProperties(
						Constructors.createLocalNameList(
						propList),
						null,
						name,
						matchAlgorithm,
						null);
				}
				nodes2 = nodes2.restrictToMatchingDesignations(
					name,
					SearchDesignationOption.NON_PREFERRED_ONLY,
					matchAlgorithm,
					null);
			}

			CodedNodeSet nodes = null;
			if(propList.length > 0)
				nodes = nodes1.union(nodes2);
			else
				nodes = nodes2;


			if (searchSource != null && searchSource.compareTo("") != 0)
			{
				nodes = restrictToSource(nodes, searchSource);
			}


            matchIterator = null;
            //SearchNodeResultBean searchBean = null;
            ResolvedConceptReferenceList lst = null;
            CodedNodeSet nodes5 = null;
            CodedNodeSet nodes6 = null;

			if (roleToSearchIn != null && roleToSearchIn.compareTo("") != 0)
			{
				CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
				CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, "roles");

				NameAndValueList asso_list =
					createNameAndValueList(
					new String[] {roleToSearchIn}, null);

				cng = cng.restrictToAssociations(asso_list, null);

				cng = cng.restrictToTargetCodes(nodes);

				lst =
					cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
					1, 1, null, null, null, 1000);

				nodes5 = lbSvc.getCodingSchemeConcepts(scheme, null);
				nodes5 = nodes5.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
			}

			if (associationToSearchIn != null && associationToSearchIn.compareTo("") != 0)
			{
				CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
				CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, null);

				NameAndValueList asso_list =
					createNameAndValueList(
					new String[] {associationToSearchIn}, null);

				cng = cng.restrictToAssociations(asso_list, null);
				cng = cng.restrictToTargetCodes(nodes);

				lst =
					cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
					1, 1, null, null, null, 1000);

				nodes6 = lbSvc.getCodingSchemeConcepts(scheme, null);
				nodes6 = nodes6.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
			}

			if (nodes5 != null && nodes6 != null)
			{
				nodes = nodes5.union(nodes6);
			}
			else if (nodes5 == null && nodes6 != null)
			{
				nodes = nodes6;
			}
			else if (nodes5 != null && nodes6 == null)
			{
				nodes = nodes5;
			}

			// GF9215
			boolean activeOnly = !searchInactive;
			nodes = restrictToActiveStatus(nodes, activeOnly);

			SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
			matchIterator =
				nodes.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);

	 }catch(Exception e){
		 e.printStackTrace();
	 }
	 return matchIterator;
  }


	/**
	 * Search nodes for properties.
	 *
	 * @param names the list of search terms
	 * @param count - parameter not used
	 * @param soundsLike whether to use the sounds like search
	 * @param searchInactive whether to include inactive concepts
	 * @param searchSource the search source
	 * @param roleToSearchIn the role to search in
	 * @param associationToSearchIn the association to search in
	 *
	 * @return the resolved concept references iterator
	 *
	 */


  //GF#10952
	public ResolvedConceptReferencesIterator searchNodesForProperties(List names,@SuppressWarnings("unused") int count,
			boolean soundsLike,
		boolean searchInactive, String searchSource, String roleToSearchIn, String associationToSearchIn){
		ResolvedConceptReferencesIterator matchIterator = null;
		try{
			String scheme = codingScheme.getRegisteredName();
			CodedNodeSet nodes1 = lbSvc.getCodingSchemeConcepts(scheme, null);
			CodedNodeSet nodes2 = lbSvc.getCodingSchemeConcepts(scheme, null);

			String name = null;
			Iterator namesIte = names.iterator();

			//String matchAlgorithm = "RegExp";
			String matchAlgorithm = "contains";

			if(soundsLike)
				matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

			while(namesIte.hasNext()){
				name = (String)namesIte.next();
				name = name.toLowerCase();

				if (nodes1 != null) {
					try {
						nodes1 = nodes1.restrictToMatchingProperties(null,
						   new PropertyType[] {PropertyType.GENERIC, PropertyType.DEFINITION}, name, matchAlgorithm, null);
				    } catch (Exception ex) {
						nodes1 = null;
					}
				}


				if (nodes2 != null) {
					try {
					    nodes2 = nodes2.restrictToMatchingDesignations(
						name,
						SearchDesignationOption.NON_PREFERRED_ONLY,
						matchAlgorithm,
						null);
				    } catch (Exception ex) {
						nodes2 = null;
					}

				}
			}

			CodedNodeSet nodes = null;
            if (nodes1 != null && nodes2 != null)
            {
				nodes = nodes1.union(nodes2);
			}
            else if (nodes1 != null && nodes2 == null)
            {
				nodes = nodes2;
			}
            if (nodes1 == null && nodes2 != null)
            {
				nodes = nodes1;
			}

 			if (nodes == null) return null;
			if (searchSource != null && searchSource.compareTo("") != 0)
			{
				nodes = restrictToSource(nodes, searchSource);
			}

            matchIterator = null;
            ResolvedConceptReferenceList lst = null;
            CodedNodeSet nodes5 = null;
            CodedNodeSet nodes6 = null;

			if (roleToSearchIn != null && roleToSearchIn.compareTo("") != 0)
			{
				CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
				CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, null);

				NameAndValueList asso_list =
					createNameAndValueList(
					new String[] {roleToSearchIn}, null);

				cng = cng.restrictToAssociations(asso_list, null);

				cng = cng.restrictToTargetCodes(nodes);

				lst =
					cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
					1, 1, null, null, null, 1000);

				nodes5 = lbSvc.getCodingSchemeConcepts(scheme, null);
				nodes5 = nodes5.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
			}

			if (associationToSearchIn != null && associationToSearchIn.compareTo("") != 0)
			{
				CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
				CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, null);

				NameAndValueList asso_list =
					createNameAndValueList(
					new String[] {associationToSearchIn}, null);

				cng = cng.restrictToAssociations(asso_list, null);
				cng = cng.restrictToTargetCodes(nodes);

				lst =
					cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
					1, 1, null, null, null, 1000);

				nodes6 = lbSvc.getCodingSchemeConcepts(scheme, null);
				nodes6 = nodes6.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
			}



			if (nodes5 != null && nodes6 != null)
			{
				nodes = nodes5.union(nodes6);
			}
			else if (nodes5 == null && nodes6 != null)
			{
				nodes = nodes6;
			}
			else if (nodes5 != null && nodes6 == null)
			{
				nodes = nodes5;
			}

			if (nodes == null)
			{
				return null;
			}

			// GF9215
			boolean activeOnly = !searchInactive;
			nodes = restrictToActiveStatus(nodes, activeOnly);


			SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});

			matchIterator = null;
			try {
				matchIterator = nodes.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);
			} catch (Exception ex) {

			}

	 }catch(Exception e){
		 e.printStackTrace();
	 }
	 return matchIterator;
  }


	  /**
	   * Search nodes for properties.
	   *
	   * @param names the names
	   * @param algorithm the algorithm
	   * @param properties the properties
	   * @param soundsLike the sounds like
	   * @param includeObsolete the include obsolete
	   *
	   * @return the resolved concept references iterator
	   */
	  public ResolvedConceptReferencesIterator searchNodesForProperties(List names, int algorithm, String[] properties,boolean soundsLike, boolean includeObsolete){
	 		ResolvedConceptReferencesIterator matchIterator = null;
			try{
				String scheme = codingScheme.getRegisteredName();
				CodedNodeSet nodes1 = null;
//				CodedNodeSet nodes2 = lbSvc.getCodingSchemeConcepts(scheme, null);

				String name = null;
				Iterator namesIte = names.iterator();
				String[] propList = properties;
				if(propList.length > 0)
						nodes1 = lbSvc.getCodingSchemeConcepts(scheme, null);
				String matchAlgorithm = "RegExp";
				if(soundsLike)
							matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

				while(namesIte.hasNext()){
					    name = (String)namesIte.next();
					    name = name.toLowerCase();
						if(!soundsLike){
							name = replacePeriod(name);
							switch(algorithm){
								case SEARCH_STARTS_WITH:name = name + ".*";
									break;
								case SEARCH_ENDS_WITH:name = ".*" + name;
									break;
								case SEARCH_CONTAINS:name = ".*" + name + ".*";
									break;
								case SEARCH_EXACT_MATCH:
									break;
							}
						}

						if(propList.length > 0){

							nodes1 = nodes1.restrictToMatchingProperties(
										Constructors.createLocalNameList(
											propList),
											null,
											name,
											matchAlgorithm,
											null);
						}

				}

				CodedNodeSet nodes = null;
				if(propList.length > 0)
					nodes = nodes1;//.union(nodes2);


				// GF9215
				boolean activeOnly = !includeObsolete;
				nodes = restrictToActiveStatus(nodes, activeOnly);


				SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
				// Analyze the result ...
				matchIterator =
					nodes.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);

		 }catch(Exception e){
		 	e.printStackTrace();
	     }
	     return matchIterator;
	  }

  /**
   * Search nodes for properties.
   *
   * @param names the names
   * @param count parameter not used
   * @param algorithm the algorithm
   * @param properties the properties
   * @param soundsLike the sounds like
   * @param includeObsolete the include obsolete
   *
   * @return the resolved concept references iterator
   * @deprecated - use searchNodesForProperties(List,int,String[],boolean,boolean)
   */@Deprecated
  public ResolvedConceptReferencesIterator searchNodesForProperties(List names,@SuppressWarnings("unused") int count,
		  int algorithm, String[] properties,boolean soundsLike, boolean includeObsolete){
 		ResolvedConceptReferencesIterator matchIterator = null;
		try{
			String scheme = codingScheme.getRegisteredName();
			CodedNodeSet nodes1 = null;
//			CodedNodeSet nodes2 = lbSvc.getCodingSchemeConcepts(scheme, null);

			String name = null;
			Iterator namesIte = names.iterator();
			String[] propList = properties;
			if(propList.length > 0)
					nodes1 = lbSvc.getCodingSchemeConcepts(scheme, null);
			String matchAlgorithm = "RegExp";
			if(soundsLike)
						matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

			while(namesIte.hasNext()){
				    name = (String)namesIte.next();
				    name = name.toLowerCase();
					if(!soundsLike){
						name = replacePeriod(name);
						switch(algorithm){
							case SEARCH_STARTS_WITH:name = name + ".*";
								break;
							case SEARCH_ENDS_WITH:name = ".*" + name;
								break;
							case SEARCH_CONTAINS:name = ".*" + name + ".*";
								break;
							case SEARCH_EXACT_MATCH:
								break;
						}
					}

					if(propList.length > 0){

						nodes1 = nodes1.restrictToMatchingProperties(
									Constructors.createLocalNameList(
										propList),
										null,
										name,
										matchAlgorithm,
										null);
					}

			}

			CodedNodeSet nodes = null;
			if(propList.length > 0)
				nodes = nodes1;//.union(nodes2);


			// GF9215
			boolean activeOnly = !includeObsolete;
			nodes = restrictToActiveStatus(nodes, activeOnly);


			SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
			// Analyze the result ...
			matchIterator =
				nodes.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);

	 }catch(Exception e){
	 	e.printStackTrace();
     }
     return matchIterator;
  }


   /**
    * Search nodes for associations.
    *
    * @param names the names
    * @param soundsLike the sounds like
    * @param includeObsolete the include obsolete
    * @param roleToSearchIn the role to search in
    *
    * @return the resolved concept references iterator
    */


    // GF#10877
    public ResolvedConceptReferencesIterator searchNodesForAssociations(List names, int count, boolean soundsLike, boolean includeObsolete, String roleToSearchIn)
    {
   	    ResolvedConceptReferencesIterator matchIterator = null;
        try{
  		  String scheme = codingScheme.getRegisteredName();
  		  //CodedNodeSet nodes3 = lbSvc.getCodingSchemeConcepts(scheme, null);
  		  CodedNodeSet nodes3 = null;
  		  CodedNodeSet nodes4 = lbSvc.getCodingSchemeConcepts(scheme, null);
  		  String name = "";
  		  Iterator namesIte = names.iterator();
  		  String name1 = "";
  		  String temp="";
  		  String temp1="";
  		  String matchAlgorithm = "contains";

  		  if(soundsLike)
  		  {
  			  matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();
  		  }
  		  int counter = 0;
  		  while(namesIte.hasNext()){
  			  name = (String)namesIte.next();
  			  name = name.toLowerCase();
              if (nodes3 != null)
              {
				  try {
					  nodes3 = nodes3.restrictToMatchingProperties(
						  Constructors.createLocalNameList(
						  "conceptCode"),null,
						  name,
						  matchAlgorithm,
						  null);
				  } catch (Exception ex) {
					  nodes3 = null;
				  }
		      }

              if (nodes4 != null)
              {
				  try {
					  nodes4 = nodes4.restrictToMatchingDesignations(
						  name,
						  true,
						  matchAlgorithm,//soundsLike?MatchAlgorithms.DoubleMetaphoneLuceneQuery.name():"RegExp",
						  null);
				  } catch (Exception ex) {
					  nodes4 = null;
				  }
		      }
  		  }

  		  CodedNodeSet nodes = null;
  		  if (nodes3 == null && nodes4 == null) return null;
  		  else if (nodes3 == null && nodes4 != null) nodes = nodes4;
  		  else if (nodes3 != null && nodes4 == null) nodes = nodes3;
  		  else if (nodes3 != null && nodes4 != null) nodes = nodes3.union(nodes4);

   		  SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});

  		  ResolvedConceptReferenceList temp_list = null;
  		  try {
  		  	  temp_list = nodes.resolveToList(sortCriteria, null, null, 1000);
  		  } catch (Exception e) {

  		  }
  		  if (temp_list == null || temp_list.getResolvedConceptReferenceCount() == 0)
  		  {
  			   System.out.println("No matches. temp_list.getResolvedConceptReferenceCount() == 0 ");
  			   return null;
  		  }

  		  CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
  		  CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, null);
  		  NameAndValueList asso_list =
  			  createNameAndValueList(
  			  new String[] {roleToSearchIn}, null);

  		  cng = cng.restrictToAssociations(asso_list, null);
  		  cng = cng.restrictToTargetCodes(nodes);

  		  ResolvedConceptReferenceList lst = null;
  		  try {
  		  	lst = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
                  1, 1, null, null, null, 1000);
		  } catch (Exception e) {
			  e.printStackTrace();
		  }

  		  if (lst == null)
  		  {
  			  return null;
  		  }
  		  CodedNodeSet nodes5 = lbSvc.getCodingSchemeConcepts(scheme, null);
  		  ConceptReferenceList crl = resolvedConceptReferenceListToConceptReferenceList(lst);
  		  if (crl == null) return null;

  		  nodes5 = nodes5.restrictToCodes(crl);

  		  // GF9215
  		  boolean activeOnly = !includeObsolete;
  		  nodes5 = restrictToActiveStatus(nodes5, activeOnly);

  		  matchIterator =
  			  nodes5.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);

  	  }catch(Exception e){
  		  e.printStackTrace();
  	  }
  	  return matchIterator;
    }




   /**
    * Search nodes for name.
    *
    * @param names the names
    * @param algorithm the algorithm
    * @param soundsLike the sounds like
    * @param includeObsolete the include obsolete
    *
    * @return the resolved concept references iterator
    */
   public ResolvedConceptReferencesIterator searchNodesForName(List names, int algorithm, boolean soundsLike, boolean includeObsolete){
 	ResolvedConceptReferencesIterator matchIterator = null;
 	try{
 		String scheme = codingScheme.getRegisteredName();
 		CodedNodeSet nodes4 = lbSvc.getCodingSchemeConcepts(scheme, null);
// 		CodedNodeSet nodes2 = null;
// 		CodedNodeSet nodes1 = null;
 		String name = "";
 		Iterator namesIte = names.iterator();
// 		String name1 = "";
// 		String temp="";
// 		String temp1="";
 		String matchAlgorithm = "RegExp";

 		if(soundsLike)
 					matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

// 		int counter = 0;

 		while(namesIte.hasNext()){
 			name = (String)namesIte.next();
 			name = name.toLowerCase();

 			if(!soundsLike){
 				name = replacePeriod(name);
 				switch(algorithm){
 					case SEARCH_STARTS_WITH:name = name + ".*";
 						break;
 					case SEARCH_ENDS_WITH:name = ".*" + name;
 						break;
 					case SEARCH_CONTAINS:name = ".*" + name + ".*";
 						break;
 					case SEARCH_EXACT_MATCH:
 						break;
 				}

 				matchAlgorithm = "RegExp";
 			}

// 			nodes4 = nodes4.restrictToMatchingDesignations(
// 			name,
// 			true,
// 			matchAlgorithm,
// 			null);

 		    nodes4 = nodes4.restrictToMatchingDesignations(name,
 				  SearchDesignationOption.PREFERRED_ONLY,
 				  matchAlgorithm,
 				  null);
	    }
 		CodedNodeSet nodes = nodes4;

 		// GF9215
 		boolean activeOnly = !includeObsolete;
 		nodes = restrictToActiveStatus(nodes, activeOnly);

 		// Sort by search engine recommendation & code ...
 		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
 		// Analyze the result ...
 		matchIterator =
 			nodes.resolve(sortCriteria,null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);
 	 }catch(Exception e){
 	 	e.printStackTrace();
      }
      return matchIterator;

   }


  /**
   * Search nodes for name.
   *
   * @param names the names
   * @param count parameter not used
   * @param algorithm the algorithm
   * @param soundsLike the sounds like
   * @param includeObsolete the include obsolete
   *
   * @return the resolved concept references iterator
   * @deprecated - use searchNodesForName(List,int,boolean,boolean)
   */@Deprecated
  public ResolvedConceptReferencesIterator searchNodesForName(List names,@SuppressWarnings("unused") int count,
		  int algorithm, boolean soundsLike, boolean includeObsolete){
	ResolvedConceptReferencesIterator matchIterator = null;
	try{
		String scheme = codingScheme.getRegisteredName();
		CodedNodeSet nodes4 = lbSvc.getCodingSchemeConcepts(scheme, null);
//		CodedNodeSet nodes2 = null;
//		CodedNodeSet nodes1 = null;
		String name = "";
		Iterator namesIte = names.iterator();
//		String name1 = "";
//		String temp="";
//		String temp1="";
		String matchAlgorithm = "RegExp";

		if(soundsLike)
					matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

//		int counter = 0;

		while(namesIte.hasNext()){
			name = (String)namesIte.next();
			name = name.toLowerCase();

			if(!soundsLike){
				name = replacePeriod(name);
				switch(algorithm){
					case SEARCH_STARTS_WITH:name = name + ".*";
						break;
					case SEARCH_ENDS_WITH:name = ".*" + name;
						break;
					case SEARCH_CONTAINS:name = ".*" + name + ".*";
						break;
					case SEARCH_EXACT_MATCH:
						break;
				}

				matchAlgorithm = "RegExp";
			}

//			nodes4 = nodes4.restrictToMatchingDesignations(
//			name,
//			true,
//			matchAlgorithm,
//			null);
		    nodes4 = nodes4.restrictToMatchingDesignations(name,
				  SearchDesignationOption.PREFERRED_ONLY,
				  matchAlgorithm,
				  null);
	    }
		CodedNodeSet nodes = nodes4;

		// GF9215
		boolean activeOnly = !includeObsolete;
		nodes = restrictToActiveStatus(nodes, activeOnly);

		// Sort by search engine recommendation & code ...
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
		// Analyze the result ...
		matchIterator =
			nodes.resolve(sortCriteria,null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);
	 }catch(Exception e){
	 	e.printStackTrace();
     }
     return matchIterator;

  }




  /**
   * Search nodes for name.
   *
   * @param names the names
   * @param count parameter not used
   * @param soundsLike the sounds like
   * @param searchInactive the search inactive
   * @param searchSource the search source
   * @param roleToSearchIn the role to search in
   * @param associationToSearchIn the association to search in
   *
   * @return the resolved concept references iterator
   * @deprecated - use searchNodesForName(List,boolean,boolean,String,String,String)
   */@Deprecated

  public ResolvedConceptReferencesIterator searchNodesForName(List names,@SuppressWarnings("unused") int count,
	  boolean soundsLike, boolean searchInactive,
	  String searchSource, String roleToSearchIn, String associationToSearchIn){

      String algorithm = "contains";
      return searchNodesForName(names, count, soundsLike, searchInactive, searchSource, roleToSearchIn, associationToSearchIn, algorithm);
  }


     /**
      * Gets the next.
      *
      * @param iterator the iterator
      *
      * @return the next
      */
     public ResolvedConceptReferenceList getNext(ResolvedConceptReferencesIterator iterator)
     {
		 return iterator.getNext();
	 }

	 /**
 	 * Dump_matches to output, for debug purposes
 	 *
 	 * @param iterator the iterator
 	 * @param maxToReturn the max to return
 	 */
 	public void dump_matches(ResolvedConceptReferencesIterator iterator, int maxToReturn)
	 {
		 if (iterator == null)
		 {
			 System.out.println("No match.");
			 return;
		 }
		 try {
			 int iteration = 0;
			 while (iterator.hasNext())
			 {
				 iteration++;
				 iterator = iterator.scroll(maxToReturn);
				 ResolvedConceptReferenceList rcrl = iterator.getNext();
				 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
				 for (int i=0; i<rcra.length; i++)
				 {
					 ResolvedConceptReference rcr = rcra[i];
					 CodedEntry ce = rcr.getReferencedEntry();
					 System.out.println("Iteration " + iteration + " " + ce.getConceptCode() + " " + ce.getEntityDescription().getContent());
				 }
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
	 }

///////////////////////////////////////////////////////////////////////////////////////////////////

	/**
 * Search nodes.
 *
 * @param scheme the scheme
 * @param vers the version
 * @param ltag the tag
 * @param searchTerm the search term
 * @param limit parameter not used
 * @param source the source
 * @param cui parameter not used
 * @param shortResult parameter not used
 * @param score parameter not used
 *
 * @return the vector
 * @deprecated - use searchNodes(String,String,String,String,String,boolean)
 */@Deprecated
public Vector<CodedEntry> searchNodes(String scheme, String vers, String ltag, String searchTerm,
		@SuppressWarnings("unused")int limit, String source,
		@SuppressWarnings("unused")boolean cui, @SuppressWarnings("unused")boolean shortResult,
		@SuppressWarnings("unused")boolean score)
	{
		 Vector v = new Vector();
		 /*
		 if (cui)
		 {
			CodedEntry concept  = getCodedEntry(scheme, version, tag, searchTerm);
			if (concept != null)
			{
				v.add(concept);
				return v;
			}
		 }
     	 else if (source != null && source.compareTo("*") != 0) // search by source
		 {
			 return findCodedEntriesWithSourceCodeMatching(scheme, version, tag, source, searchTerm, limit);
		 }
		 */

         CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
         csvt.setVersion(vers);
         csvt.setTag(ltag);

         Vector<CodedEntry> w = new Vector<CodedEntry>();
         HashSet<String> hset = new HashSet<String>();
         try {
         	 //lbSvc = new LexBIGServiceImpl();
			 v = matchSynonyms(searchTerm, lbSvc, scheme, csvt, source);
			 for (int i=0; i<v.size(); i++)
			 {
				 CodedEntry ce = (CodedEntry) v.elementAt(i);
				 if (!hset.contains(ce.getConceptCode()))
				 {
					 w.add(ce);
					 hset.add(ce.getConceptCode());
				 }
			 }
			 v = matchSpell(searchTerm, lbSvc, scheme, csvt, source);
			 for (int i=0; i<v.size(); i++)
			 {
				 CodedEntry ce = (CodedEntry) v.elementAt(i);
				 if (!hset.contains(ce.getConceptCode()))
				 {
					 w.add(ce);
					 hset.add(ce.getConceptCode());
				 }
			 }

			 v = matchTermCompletion(searchTerm, lbSvc, scheme, csvt, source);
			 for (int i=0; i<v.size(); i++)
			 {
				 CodedEntry ce = (CodedEntry) v.elementAt(i);
				 if (!hset.contains(ce.getConceptCode()))
				 {
					 w.add(ce);
					 hset.add(ce.getConceptCode());
				 }
			 }

			 v = matchWordCompletion(searchTerm, lbSvc, scheme, csvt, source);
			 for (int i=0; i<v.size(); i++)
			 {
				 CodedEntry ce = (CodedEntry) v.elementAt(i);
				 if (!hset.contains(ce.getConceptCode()))
				 {
					 w.add(ce);
					 hset.add(ce.getConceptCode());
				 }
			 }

			 v = matchSubquery(searchTerm, lbSvc, scheme, csvt, source);
			 for (int i=0; i<v.size(); i++)
			 {
				 CodedEntry ce = (CodedEntry) v.elementAt(i);
				 if (!hset.contains(ce.getConceptCode()))
				 {
					 w.add(ce);
					 hset.add(ce.getConceptCode());
				 }
			 }
		 } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return w;
	}


/**
* Search nodes. Searches for the given term using a variety of search techniques
*
* @param scheme the scheme
* @param vers the version
* @param ltag the tag
* @param searchTerm the search term
* @param source the source qualifier
*
* @return the vector
*/
public Vector<CodedEntry> searchNodes(String scheme, String vers, String ltag, String searchTerm,
	String source)
{
	 Vector v = new Vector();
	 /*
	 if (cui)
	 {
		CodedEntry concept  = getCodedEntry(scheme, version, tag, searchTerm);
		if (concept != null)
		{
			v.add(concept);
			return v;
		}
	 }
 	 else if (source != null && source.compareTo("*") != 0) // search by source
	 {
		 return findCodedEntriesWithSourceCodeMatching(scheme, version, tag, source, searchTerm, limit);
	 }
	 */

     CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
     csvt.setVersion(vers);
     csvt.setTag(ltag);

     Vector<CodedEntry> w = new Vector<CodedEntry>();
     HashSet<String> hset = new HashSet<String>();
     try {
     	 //lbSvc = new LexBIGServiceImpl();
		 v = matchSynonyms(searchTerm, lbSvc, scheme, csvt, source);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 if (!hset.contains(ce.getConceptCode()))
			 {
				 w.add(ce);
				 hset.add(ce.getConceptCode());
			 }
		 }
		 v = matchSpell(searchTerm, lbSvc, scheme, csvt, source);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 if (!hset.contains(ce.getConceptCode()))
			 {
				 w.add(ce);
				 hset.add(ce.getConceptCode());
			 }
		 }

		 v = matchTermCompletion(searchTerm, lbSvc, scheme, csvt, source);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 if (!hset.contains(ce.getConceptCode()))
			 {
				 w.add(ce);
				 hset.add(ce.getConceptCode());
			 }
		 }

		 v = matchWordCompletion(searchTerm, lbSvc, scheme, csvt, source);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 if (!hset.contains(ce.getConceptCode()))
			 {
				 w.add(ce);
				 hset.add(ce.getConceptCode());
			 }
		 }

		 v = matchSubquery(searchTerm, lbSvc, scheme, csvt, source);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 if (!hset.contains(ce.getConceptCode()))
			 {
				 w.add(ce);
				 hset.add(ce.getConceptCode());
			 }
		 }
	 } catch (Exception e) {
		 e.printStackTrace();
	 }
	 return w;
}

	/**
	 * Display concepts and related text strings matching the given string.
	 *
	 * @param s The test string.
	 * @param l_lbSvc the LexBIGService
	 * @param scheme the scheme
	 * @param csvt the CodingSchemeVersionOrTag
	 * @param source the source
	 *
	 * @return the vector
	 *
	 * @throws LBException the LB exception
	 */
	protected Vector<CodedEntry> matchSynonyms(String s, LexBIGService l_lbSvc, String scheme, CodingSchemeVersionOrTag csvt, String source)
		throws LBException
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
		// Add restriction to narrow the scope.
		// Q: How does Metaphrase determine raw term for search (words
		//    are pre-normalized? is match dependent on order of words
		//    provided in phrase)?
		// Q: Best algorithm to use (exact match, contains, logical stemmed)?
		// Q: What punctuation is to be ignored on search?
		// Q: Do we always need to return a synonym match?   If not where
		//    boundary defined to know what is 'close enough' to match?
		//    Do all non-stop words in the search string need to be
		//    in the matched concept?  Can the concept text be a superset?
		//
		// An attempt is made to use a partial normalized form on search by
		// removing stop words and utilizing a stemmed query algorithm.
		// Query is performed over text representations that contain all
		// non-stop words for the given string.
		//
		// On resolve, we sort by lucene score ('matchToQuery' sort algorithm).

		StringBuffer searchPhrase = new StringBuffer();
		String[] words = toWords(s, true);
		for (int i = 0; i < words.length; i++) {
			if (i > 0)
				searchPhrase.append(" AND ");
			searchPhrase.append(words[i]);
		}

		// Define the code set and add restrictions ...
		CodedNodeSet nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		if (source != null && source.compareTo("*") != 0)
		{
			nodes = restrictToSource(nodes, source);
		}
		nodes = nodes.restrictToMatchingDesignations(
			searchPhrase.toString(), SearchDesignationOption.ALL, "StemmedLuceneQuery", null);
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
		PropertyType[] fetchTypes =
			new PropertyType[] {PropertyType.PRESENTATION};

		// Resolve and analyze the result ...
		ResolvedConceptReferenceList matches =
			nodes.resolveToList(sortCriteria, null, fetchTypes, -1);

		// Found a match?  If so, sort according to relevance.
		if (matches.getResolvedConceptReferenceCount() > 0) {
			// NOTE that a match so far only indicates that all of the words in the
			// passed in string also exist in a term assigned to the resolved
			// concepts.  It does not, however, exclude results where the terms
			// also have additional words.  This code currently processes only
			// full matches by calling getReferenceWeight and taking only those
			// values with weight '1'.

			// List concept references with exact correlation of at least one
			// concept term to the compare string.

			//final List<String> matchWords = Arrays.asList(words);

			final List<String> matchWords = Arrays.asList(words);


			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				if (getReferenceWeight(ref, matchWords) == 1)
				{
				    CodedEntry ce = ref.getReferencedEntry();
				    v.add(ce);
				}
			}
		}
		return v;
	}

	/**
	 * Attempt to approximate spelling suggestions.
	 *
	 * @param s The test string.
	 * @param l_lbSvc the LexBIGService
	 * @param scheme the scheme
	 * @param csvt the CodingSchemeVersionOrTag
	 * @param source the source
	 *
	 * @return the vector
	 *
	 * @throws LBException the LB exception
	 */
	protected Vector matchSpell(String s, LexBIGService l_lbSvc, String scheme, CodingSchemeVersionOrTag csvt, String source)
		throws LBException
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
		// Spell suggestion is based off entire phrase in comparison
		// to content stored in the ontology; no external dictionary
		// is utilized.  Two different algorithms are highlighted
		// for consideration. The first utilizes the lucene-based
		// Levenshtein distance algorithm.  The second utilizes the
		// DoubleMetaphoneLuceneQuery (sounds-like) algorithm.
		//
		// Q: Do we want to search all terms or just primary text?
		// Queries are currently made against all text representations,
		// but this could be narrowed.
		//
		// On resolve, we sort by lucene score ('matchToQuery' sort algorithm).

		// Levenshtein distance processing ...
		String[] words = toWords(s, true);
		String prefix = words[0].substring(0, 1);

		CodedNodeSet nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		if (source != null && source.compareTo("*") != 0)
		{
			nodes = restrictToSource(nodes, source);
		}
		StringBuffer searchPhrase = new StringBuffer(s).append('~');
		nodes = nodes.restrictToMatchingDesignations(
			searchPhrase.toString(), SearchDesignationOption.ALL,
			MatchAlgorithms.LuceneQuery.toString(), null);
		nodes = nodes.restrictToMatchingProperties(null, new PropertyType[] {PropertyType.PRESENTATION},
			prefix, "startsWith", null);
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});

		// Resolve and analyze the result ...
		ResolvedConceptReferenceList matches =
			nodes.resolveToList(sortCriteria, null, new PropertyType[] {}, 5);

		// Found a match?
		if (matches.getResolvedConceptReferenceCount() > 0) {
			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				// OK, so we have it narrowed to a concept at this point but not a
				// specific term.  Narrow the choices to those that include the same
				// starting letter and number of words as our test phrase.
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				CodedEntry ce = ref.getReferencedEntry();
				v.add(ce);
			}
		}

		// Sounds-like processing ...
		nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		nodes = nodes.restrictToMatchingDesignations(
			s, SearchDesignationOption.ALL,
			MatchAlgorithms.DoubleMetaphoneLuceneQuery.toString(), null);
		nodes = nodes.restrictToMatchingProperties(null, new PropertyType[] {PropertyType.PRESENTATION},
			prefix, "startsWith", null);

		// Resolve and analyze the result ...
		matches = nodes.resolveToList(sortCriteria, null, new PropertyType[] {}, 5);

		// Found a match?
		if (matches.getResolvedConceptReferenceCount() > 0) {
			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				// Once again we have it narrowed to a concept at this point but not
				// a specific term.  Narrow the choices to those that include the same
				// starting letter and number of words as our test phrase.
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				CodedEntry ce = ref.getReferencedEntry();
				v.add(ce);
			}
		}
        return v;
	}

	/**
	 * Attempt to approximate term completion.
	 * @param s The test string.
	 * @param l_lbSvc
	 * @param scheme
	 * @param csvt
	 * @throws LBException
	 */
	@SuppressWarnings("unchecked")
	protected Vector<CodedEntry> matchTermCompletion(String s, LexBIGService l_lbSvc, String scheme, CodingSchemeVersionOrTag csvt, String source)
		throws LBException
	{
		//Util.displayMessage("\n*** Term completion ...");

		// Term completion is basically 'startsWith'.  Two different
		// queries are tried to match.  The first utilizes the standard
		// startsWith match algorithm.  The second utilizes regular
		// expressions to try and accomplish short-circuiting more
		// efficiently.
		//
		// Queries are currently made against all text representations,
		// but this could be narrowed.
		//
		// On resolve, we sort by lucene score ('matchToQuery' sort algorithm).

        Vector<CodedEntry> v = new Vector<CodedEntry>();

		String prefix = s.toLowerCase();
//		String[] words = toWords(s, false);
		CodedNodeSet nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		if (source != null && source.compareTo("*") != 0)
		{
			nodes = restrictToSource(nodes, source);
		}
		nodes = nodes.restrictToMatchingDesignations(
			prefix, SearchDesignationOption.ALL,
			"startsWith", null);
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});

		// Resolve and analyze the result ...
		ResolvedConceptReferenceList matches =
			nodes.resolveToList(sortCriteria, null, new PropertyType[] {}, 5);

		// Found a match?
		//Util.displayMessage("\tstartsWith (limited to 5 hits); short circuit after query...");
//		if (matches.getResolvedConceptReferenceCount() > 0) {
//			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
//				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
//				//printText(ref, false, prefix, words.length + 1);
//			}
//		} else {
//			//Util.displayMessage("\tNo match found.");
//		}

		// Try again with regular expression.
		nodes = lbSvc.getCodingSchemeConcepts(scheme, csvt);
		String regex = '^' + prefix + "\\W*\\w*\\Z";
		nodes.restrictToMatchingDesignations(
				regex, SearchDesignationOption.ALL,
				"RegExp", null);

		// Resolve and analyze the result ...
		matches = nodes.resolveToList(sortCriteria, null, new PropertyType[] {}, 5);

		// Found a match?
		//Util.displayMessage("\n\tRegExp match (limited to 5 hits); short circuit on query...");
		if (matches.getResolvedConceptReferenceCount() > 0) {
			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				//printText(ref, false, prefix, words.length + 1);
				CodedEntry ce = ref.getReferencedEntry();
				v.add(ce);

			}
		} else {
			//Util.displayMessage("\tNo match found.");
		}

		return v;
	}



	/**
	 * Attempt to approximate word completion.
	 *
	 * @param s The test string.
	 * @param l_lbSvc the LexBIGService
	 * @param scheme the scheme
	 * @param csvt the CodingSchemeVersionOrTag
	 * @param source the source
	 *
	 * @return the vector
	 *
	 * @throws LBException the LB exception
	 */
	//
	protected Vector<CodedEntry> matchWordCompletion(String s, LexBIGService l_lbSvc, String scheme, CodingSchemeVersionOrTag csvt, String source)
		throws LBException
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
		// For word completion, we just use a regular expression and
		// substitute any non-whitespace character for the substitution
		// character ('.').
		//
		// Q: Do we want to search all terms or just primary text?
		// Queries are currently made against all text representations,
		// but this could be narrowed.
		//
		// On resolve, we sort by lucene score ('matchToQuery' sort algorithm).

//		String[] words = toWords(s, false);
		String prefix = s;
		CodedNodeSet nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		if (source != null && source.compareTo("*") != 0)
		{
			nodes = restrictToSource(nodes, source);
		}
		String regex = '^' + s.replaceAll("\\.", "\\\\S*") + "\\Z";
		nodes = nodes.restrictToMatchingDesignations(
				regex, SearchDesignationOption.ALL,
				"RegExp", null);
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});

		// Resolve and analyze the result ...
		ResolvedConceptReferenceList matches =
			nodes.resolveToList(sortCriteria, null, new PropertyType[] {}, 5);

		if (matches.getResolvedConceptReferenceCount() > 0) {
			int dotIndex = prefix.indexOf('.');
			if (dotIndex < 0)
				dotIndex = prefix.length();
			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				CodedEntry ce = ref.getReferencedEntry();
				v.add(ce);
			}
		}
		return v;

	}

	/**
	 * Attempt to approximate compositional or sub-query match.
	 *
	 * @param s The test string.
	 * @param l_lbSvc the LexBIGService
	 * @param scheme the scheme
	 * @param csvt the CodingSchemeVersionOrTag
	 * @param source the source
	 *
	 * @return the vector
	 *
	 * @throws LBException the LB exception
	 */
	protected Vector<CodedEntry> matchSubquery(String s, LexBIGService l_lbSvc, String scheme, CodingSchemeVersionOrTag csvt, String source)
		throws LBException
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();

		// This is basically just the default lucene match; no special syntax
		// required.
		//
		// Q: Do we want to search all terms or just primary text?
		// Queries are currently made against preferred text representations,
		// but this could be widened.
		//
		// On resolve, we sort by lucene score ('matchToQuery' sort algorithm).

		CodedNodeSet nodes = l_lbSvc.getCodingSchemeConcepts(scheme, csvt);
		if (source != null && source.compareTo("*") != 0)
		{
			nodes = restrictToSource(nodes, source);
		}
		nodes = nodes.restrictToMatchingDesignations(
				s, SearchDesignationOption.PREFERRED_ONLY,
				"LuceneQuery", null);

		// Resolve and analyze the result ...
		ResolvedConceptReferenceList matches = nodes.resolveToList(null, null, new PropertyType[] {}, -1);

		// Found a match?
		if (matches.getResolvedConceptReferenceCount() > 0) {
			for (int i = 0; i < matches.getResolvedConceptReferenceCount(); i++) {
				ResolvedConceptReference ref = matches.getResolvedConceptReference(i);
				CodedEntry ce = ref.getReferencedEntry();
				v.add(ce);
			}
		}
		return v;
	}

	/**
	 * Return a relative weight between 0 and 1 that indicates how well the
	 * given reference maps to a set of words.
	 *
	 * @param ref
	 * @param matchWords
	 */
	//protected float getReferenceWeight(ResolvedConceptReference ref, List<String> matchWords) {
	protected float getReferenceWeight(ResolvedConceptReference ref, List<String> matchWords) {
		float weight = 0;
		CodedEntry ce = ref.getReferencedEntry();
		int presCount = ce.getPresentationCount();
		for (int i = 0; i < presCount; i++) {
			weight = Math.max(weight, getTextWeight(ce.getPresentation(i).getText().getContent(), matchWords));
		}
		return weight;
	}

	/**
	 * Return a relative weight between 0 and 1 that indicates how well the
	 * given string maps to a set of words.
	 *
	 * @param text
	 * @param matchWords
	 */
	protected float getTextWeight(String text, List<String> matchWords) {
		int matchedInTextWords = 0;
		int matchedInCompareWords = 0;
		String[] textWords = toWords(text, true);
		Set<String> textWordSet = new HashSet<String>();
		for (int i = 0; i < textWords.length; i++) {
			String word = textWords[i];
			textWordSet.add(word);
			if (matchWords.contains(word))
				matchedInCompareWords++;
		}
		for (Iterator<String> compareWords = matchWords.iterator(); compareWords.hasNext(); ) {
			String word = compareWords.next();
			if (textWordSet.contains(word))
				matchedInTextWords++;
		}
		float textToCompareRatio = matchedInCompareWords/textWordSet.size();
		float compareToTextRatio = matchedInTextWords/matchWords.size();
		return (textToCompareRatio + compareToTextRatio)/2;
	}



    /**
     * Creates the concept reference list.
     *
     * @param codes the codes
     * @param codeSystem the code system
     *
     * @return the concept reference list
     */
    public static ConceptReferenceList createConceptReferenceList(String[] codes, String codeSystem)
    {
        if (codes == null)
        {
            return null;
        }
        ConceptReferenceList list = new ConceptReferenceList();
        for (int i = 0; i < codes.length; i++)
        {
            ConceptReference cr = new ConceptReference();
            cr.setCodingScheme(codeSystem);
            cr.setConceptCode(codes[i]);
            list.addConceptReference(cr);
        }
        return list;
    }



	/**
	 * Returns an array containing the individual white-space delimited words
	 * contained by the given string, removing stop words and normalizing to
	 * lower case.
	 *
	 * @param s
	 */

	protected String[] toWords(String s, boolean removeStopWords) {
		String[] words = s.split("\\s");
		List<String> adjusted = new ArrayList<String>();
		for (int i = 0; i < words.length; i++) {
			if (!removeStopWords || !STOP_WORDS.contains(words[i]))
				adjusted.add(words[i].toLowerCase());
		}
		return adjusted.toArray(new String[adjusted.size()]);
	}





     /** Runs a regression test of the search utility
      * Run.
      */
     public void run()
     {
		 String[] propList = getPropertyForCodingSchemeSearch();
		 for (int i=0; i<propList.length; i++)
		 {
			 System.out.println(propList[i]);
		 }

		 String s = "....This is a test.";


		 System.out.println("\n\nString: " + s);

		 s = replacePeriod(s);

		 System.out.println("Replacement: " + s);

		 ArrayList<String> names = new ArrayList<String>();
		 names.add("Anatomy");
		 //names.add("Target");
		 int count = 100;
		 boolean soundsLike = true;
		 boolean searchInactive = true;

		 int maxToReturn = 100;
		 ResolvedConceptReferencesIterator rcri = null;

         // Definition:
         try {
			 rcri = searchNodesForDefinition(names, count, soundsLike, searchInactive);
			 dump_matches(rcri, 5);
	     } catch (Exception e) {
			 e.printStackTrace();
		 }


         // Properties:
		 names = new ArrayList<String>();
		 names.add("Anatomy");
		 count = 100;
		 soundsLike = true;
		 searchInactive = true;

		 maxToReturn = 100;
		 String searchSource = null;
		 String roleToSearchIn = null;
		 String associationToSearchIn = null;


         try {
			 rcri = searchNodesForProperties(names, count, soundsLike, searchInactive, searchSource, roleToSearchIn, associationToSearchIn);
			 rcri = rcri.scroll(maxToReturn);
			 ResolvedConceptReferenceList rcrl = rcri.getNext();
			 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
			 for (int i=0; i<rcra.length; i++)
			 {
				 ResolvedConceptReference rcr = rcra[i];

				 CodedEntry ce = rcr.getReferencedEntry();
				 System.out.println(ce.getConceptCode() + " " + ce.getEntityDescription().getContent());

			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }

		 names = new ArrayList<String>();
		 names.add("fdsafdafd");
		 count = 100;
		 soundsLike = true;
		 searchInactive = true;

		 maxToReturn = 100;

		 // Roles:
         associationToSearchIn = "Technique_Has_Sample_Or_Specimen_Anatomy";

         try {
			 rcri = searchNodesForAssociations(names, count, soundsLike, searchInactive, associationToSearchIn);
			 if (rcri == null)
			 {
				 System.out.println("No match.");
			 }
			 else
			 {
				 rcri = rcri.scroll(maxToReturn);
				 ResolvedConceptReferenceList rcrl = rcri.getNext();
				 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
				 for (int i=0; i<rcra.length; i++)
				 {
					 ResolvedConceptReference rcr = rcra[i];

					 CodedEntry ce = rcr.getReferencedEntry();
					 System.out.println(ce.getConceptCode() + " " + ce.getEntityDescription().getContent());

				 }
		     }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }

		 names = new ArrayList<String>();
		 names.add("Resources");
		 count = 100;
		 soundsLike = true;
		 searchInactive = true;

		 maxToReturn = 100;
         associationToSearchIn = "Concept_In_Subset";

		 // Associations:
         try {
			 rcri = searchNodesForAssociations(names, count, soundsLike, searchInactive, associationToSearchIn);

			 rcri = rcri.scroll(maxToReturn);
			 ResolvedConceptReferenceList rcrl = rcri.getNext();

			 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
			 for (int i=0; i<rcra.length; i++)
			 {
				 ResolvedConceptReference rcr = rcra[i];

				 CodedEntry ce = rcr.getReferencedEntry();
				 System.out.println(ce.getConceptCode() + " " + ce.getEntityDescription().getContent());

			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }

		 // MetaMatch:
         String searchTerm = "Target";
         int limit = 100;
         String source = null;
         boolean cui = false;
         boolean shortResult = false;
         boolean score = false;

	     Vector v = searchNodes(codingScheme_Name, version, null, searchTerm, limit, source,
						  cui, shortResult, score);
		 for (int i=0; i<v.size(); i++)
		 {
			 CodedEntry ce = (CodedEntry) v.elementAt(i);
			 System.out.println(ce.getConceptCode() + " " + ce.getEntityDescription().getContent());

		 }

	 }


	 //GF9215
	 private CodedNodeSet restrictToActiveStatus(CodedNodeSet cns, boolean activeOnly)
     {
		 if (cns == null) return null;
		 if (!activeOnly) return cns; // no restriction, do nothing
		 try {
		 	 cns = cns.restrictToStatus(CodedNodeSet.ActiveOption.ACTIVE_ONLY, null);
		 	 return cns;
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }


  /**
   * Search nodes for name.
   *
   * @param names the names
   * @param count parameter not used
   * @param soundsLike the sounds like
   * @param searchInactive the search inactive
   * @param searchSource the search source
   * @param roleToSearchIn the role to search in
   * @param associationToSearchIn the association to search in
   * @param algorithm matching algorithm
   *
   * @return the resolved concept references iterator
   * @deprecated - use searchNodesForName(List,boolean,boolean,String,String,String)
   */@Deprecated
  public ResolvedConceptReferencesIterator searchNodesForName(List names,@SuppressWarnings("unused") int count,
	  boolean soundsLike, boolean searchInactive,
	  String searchSource, String roleToSearchIn, String associationToSearchIn, String algorithm){
      ResolvedConceptReferencesIterator matchIterator = null;

      if (searchSource != null && roleToSearchIn == null && associationToSearchIn == null)
      {
		  String code = (String) names.get(0);
		  return findCodedEntriesWithSourceCodeMatching(searchSource, code, count, searchInactive);
	  }

      try{
		  java.lang.String formalName = codingScheme.getFormalName();

		  CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		  versionOrTag.setVersion(codingScheme.getRepresentsVersion());

		  String scheme = codingScheme.getRegisteredName();
		  CodedNodeSet nodes3 = lbSvc.getCodingSchemeConcepts(formalName, versionOrTag);

		  CodedNodeSet nodes4 = lbSvc.getCodingSchemeConcepts(formalName, versionOrTag);

		  String name = "";
		  Iterator namesIte = names.iterator();
		  String name1 = "";
		  String matchAlgorithm = algorithm;
		  if(soundsLike)
			  matchAlgorithm = MatchAlgorithms.DoubleMetaphoneLuceneQuery.name();

  		  int counter = 0;
  		  while(namesIte.hasNext()){
  			  name = (String)namesIte.next();
  			  name = name.toLowerCase();
              if (nodes3 != null)
              {
				  try {
					  nodes3 = nodes3.restrictToMatchingProperties(
						  Constructors.createLocalNameList(
						  "conceptCode"),null,
						  name,
						  matchAlgorithm,
						  null);
				  } catch (Exception ex) {
					  nodes3 = null;
				  }
		      }

              if (nodes4 != null)
              {
				  try {
					  nodes4 = nodes4.restrictToMatchingDesignations(
						  name,
						  true,
						  matchAlgorithm,//soundsLike?MatchAlgorithms.DoubleMetaphoneLuceneQuery.name():"RegExp",
						  null);
				  } catch (Exception ex) {
					  nodes4 = null;
				  }
		      }
  		  }

  		  CodedNodeSet nodes = null;

  		  if (nodes3 == null && nodes4 == null)
  		  {
			  if (roleToSearchIn != null && roleToSearchIn.compareTo("") != 0)
			  {
			  	  return null;
			  }
			  if (associationToSearchIn != null && associationToSearchIn.compareTo("") != 0)
			  {
				  return null;
			  }
		  }

  		  if (nodes3 == null && nodes4 != null) nodes = nodes4;
  		  else if (nodes3 != null && nodes4 == null) nodes = nodes3;
  		  else if (nodes3 != null && nodes4 != null) nodes = nodes3.union(nodes4);

          if (nodes != null)
          {
			  if (searchSource != null && searchSource.compareTo("") != 0)
			  {
				  nodes = restrictToSource(nodes, searchSource);
			  }
	      }

		  matchIterator = null;
		  ResolvedConceptReferenceList lst = null;
		  CodedNodeSet nodes5 = null;
		  CodedNodeSet nodes6 = null;

		  if (roleToSearchIn != null && roleToSearchIn.compareTo("") != 0)
		  {
			  CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, "roles");
			  NameAndValueList asso_list =
				  createNameAndValueList(
				  new String[] {roleToSearchIn}, null);

			  cng = cng.restrictToAssociations(asso_list, null);
			  if (nodes != null) cng = cng.restrictToTargetCodes(nodes);
			  lst = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				  1, 1, null, null, null, 1000);

			  nodes5 = lbSvc.getCodingSchemeConcepts(scheme, null);
			  try {
			  	 nodes5 = nodes5.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
			  } catch (Exception ex) {

			  }
		  }

		  if (associationToSearchIn != null && associationToSearchIn.compareTo("") != 0)
		  {
			  versionOrTag = new CodingSchemeVersionOrTag();
			  CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme_Name, versionOrTag, null);

			  NameAndValueList asso_list =
				  createNameAndValueList(
				  new String[] {associationToSearchIn}, null);

			  cng = cng.restrictToAssociations(asso_list, null);
			  if (nodes != null) cng = cng.restrictToTargetCodes(nodes);
			  lst = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				  1, 1, null, null, null, 1000);

			  nodes6 = lbSvc.getCodingSchemeConcepts(scheme, null);
			  nodes6 = nodes6.restrictToCodes(resolvedConceptReferenceListToConceptReferenceList(lst));
		  }

		  if (nodes5 != null && nodes6 != null)
		  {
			  nodes = nodes5.union(nodes6);
		  }
		  else if (nodes5 == null && nodes6 != null)
		  {
			  nodes = nodes6;
		  }
		  else if (nodes5 != null && nodes6 == null)
		  {
			  nodes = nodes5;
		  }

	      // GF9215
  		  boolean activeOnly = !searchInactive;
		  nodes = restrictToActiveStatus(nodes, activeOnly);

		  SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery"});
		  // Analyze the result .

		  //matchIterator = null;
		  try {
			  matchIterator = nodes.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);
	      } catch (Exception ex) {

		  }

	 }catch(Exception e){
		 e.printStackTrace();
     }
     return matchIterator;

  }




   public ResolvedConceptReferencesIterator findCodedEntriesWithSourceCodeMatching(String sourceAbbr, String code, int maxToReturn, boolean searchInactive)
   {
		String scheme = codingScheme.getRegisteredName();

        LocalNameList contextList = null;
        NameAndValueList qualifierList = null;

 		Vector<String> v = new Vector<String>();
        SupportedPropertyQualifier[] supported_qualifiers = codingScheme.getMappings().getSupportedPropertyQualifier();
        for (int i=0; i<supported_qualifiers.length; i++)
        {
			v.add(supported_qualifiers[i].getLocalId());
		}

        if (v.contains("source-code"))
        {
			if (code != null && code.compareTo("") != 0)
			{
				qualifierList = new NameAndValueList();
			    NameAndValue nv = new NameAndValue();
			    nv.setName("source-code");
		        nv.setContent(code);

				qualifierList.addNameAndValue(nv);
			}
		}

        LocalNameList propertyLnL = null;
     // sourceLnL
        Vector<String> w2 = new Vector<String>();
        w2.add(sourceAbbr);
        LocalNameList sourceLnL = vector2LocalNameList(w2);

        if (sourceAbbr.compareTo("*") == 0 || sourceAbbr.compareToIgnoreCase("ALL") == 0)
        {
			sourceLnL = null;
		}

        ResolvedConceptReferencesIterator matchIterator = null;
		SortOptionList sortCriteria = null;//Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
		try {
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(scheme, null);
			if (cns == null) return null;
			CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			cns = cns.restrictToProperties(propertyLnL, types, sourceLnL, contextList, qualifierList);

  		    boolean activeOnly = !searchInactive;
		    cns = restrictToActiveStatus(cns, activeOnly);

			try {
			    matchIterator = cns.resolve(sortCriteria, null,null);//ConvenienceMethods.createLocalNameList(getPropertyForCodingScheme(cs)),null);
			} catch (Exception ex) {

		    }

		} catch (Exception e) {
			 //getLogger().error("ERROR: Exception in findCodedEntriesWithSourceCodeMatching.");
			 return null;
		}
		return matchIterator;
    }



	 /**
 	 * The main method.
 	 *
 	 * @param args the arguments
 	 */
// 	public static void main(String[] args)
//	 {
//		 String name = "NCI_Thesaurus";
//		 String version = "06.12d";
//		 String url = null;
//
//		 SearchUtil test = new SearchUtil(name, version, url);
//		 test.run();
//
//	 }
}

