package gov.nih.nci.system.applicationservice.impl;

import gov.nih.nci.evs.query.EVSQuery;
import gov.nih.nci.evs.query.EVSQueryImpl;
import gov.nih.nci.system.applicationservice.ApplicationException;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.dao.DAO;
import gov.nih.nci.system.dao.Request;
import gov.nih.nci.system.dao.Response;
import gov.nih.nci.system.dao.properties.EVSProperties;
import gov.nih.nci.system.util.ClassCache;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.ExtensionDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.ModuleDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.SortDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.ValueDomainRenderingList;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ValueDomainVersionOrTag;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.types.SortContext;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Exceptions.LBInvocationException;
import org.LexGrid.LexBIG.Extensions.Generic.GenericExtension;
import org.LexGrid.LexBIG.Extensions.Query.Filter;
import org.LexGrid.LexBIG.Extensions.Query.Sort;
import org.LexGrid.LexBIG.History.HistoryService;
import org.LexGrid.LexBIG.Impl.LexBIGServiceImpl;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeGraph;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceManager;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.LexBIG.LexBIGService.ValueDomainEntryNodeSet;
import org.LexGrid.LexBIG.LexBIGService.ValueDomainNodeSet;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.valueDomains.ValueDomain;
import org.apache.log4j.Logger;
import org.springframework.util.ClassUtils;

import gov.nih.nci.evs.security.SecurityToken;


import java.util.HashMap;


/**
 * ApplicationService interface for EVS.
 *
 * @author <a href="mailto:muhsins@mail.nih.gov">Shaziya Muhsin</a>
 * @author <a href="mailto:rokickik@mail.nih.gov">Konrad Rokicki</a>
 */@SuppressWarnings("serial")
public class EVSApplicationServiceImpl extends ApplicationServiceImpl implements
        EVSApplicationService {


    /** The log. */
    private static Logger log = Logger.getLogger(EVSApplicationServiceImpl.class);

    /** The Constant LEXBIG_SYSPROPERTY. */
    private static final String LEXBIG_SYSPROPERTY = "LG_CONFIG_FILE";

    /** The config. */
    private final Hashtable<String,String> config = new Hashtable<String,String>();

    /** The evs class cache. */
    private final ClassCache evsClassCache;

    /** The lbs. */
    private final LexBIGService lbs;

    /**
     * Instantiates a new EVSApplicationService impl.
     *
     * @param classCache the ClassCache object
     * @param systemProperties the Properties object
     *
     * @throws ApplicationException the application exception
     */
    public EVSApplicationServiceImpl(ClassCache classCache,
                Properties systemProperties) throws ApplicationException {

        super(classCache, systemProperties);

        this.evsClassCache = classCache;

        for (Iterator it = systemProperties.keySet().iterator(); it.hasNext();) {
            String key = (String) it.next();
            String value = systemProperties.getProperty(key);
            config.put(key, value);
        }

        try {
            EVSProperties evsProperties = EVSProperties.getInstance();
            if (evsProperties.getLexBigConfigFileLocation() != null) {
                System.setProperty(LEXBIG_SYSPROPERTY,
                    evsProperties.getLexBigConfigFileLocation());
            log.info("LexBig system property '"+LEXBIG_SYSPROPERTY+"' set to: "
                    + System.getProperty(LEXBIG_SYSPROPERTY));
            }
            else {
                log.error("LexBig system property '"+LEXBIG_SYSPROPERTY+
                    "' undefined.");
            }

            this.lbs = new LexBIGServiceImpl();
        }
        catch (Exception e) {
            throw new ApplicationException("LexBIG initialization failed",e);
        }
    }

    public HashMap getSecurityToken_map()
    {
		return new HashMap();
	}

    /* (non-Javadoc)
     * @see gov.nih.nci.system.applicationservice.EVSApplicationService#evsSearch(gov.nih.nci.evs.query.EVSQuery)
     */@SuppressWarnings("unchecked")
    public List<Object> evsSearch(EVSQuery evsQuery)
            throws ApplicationException {

        Request request = new Request(evsQuery);
        request.setDomainObjectName(evsQuery.getClass().getName());
        request.setConfig(config);
        List<Object> results = new ArrayList<Object>();

        request.setClassCache(evsClassCache);
        String domainObjectName = request.getDomainObjectName();
        if (domainObjectName == null || domainObjectName.equals("")) {
            throw new ApplicationException(
                    "No Domain Object name specified in the request; " +
                    "unable to locate corresponding DAO");
        }
        DAO dao = evsClassCache.getDAOForClass(request.getDomainObjectName());

        if (dao == null)
            throw new ApplicationException("Could not obtain DAO for: "
                    + request.getDomainObjectName());

        try {
            Response r = dao.query(request);
            if (r != null) {
                results = (List)r.getResponse();
            }
        }
        catch (Exception e) {
            throw new ApplicationException("Error: " + e.getMessage(), e);
        }

        return results;
    }



    /* (non-Javadoc)
     * @see gov.nih.nci.system.applicationservice.impl.ApplicationServiceImpl#getAssociation(java.lang.Object, java.lang.String)
     */
    //public List<Object> getAssociation(Object source, String associationName)

    public List<Object> getAssociation(Object source, String associationName, SecurityToken securityToken)
             throws ApplicationException {
        EVSQuery evsQuery = new EVSQueryImpl();

        String methodName = "get"
                + associationName.substring(0, 1).toUpperCase()
                + associationName.substring(1) + "byC";
        Method[] methods = null;
        List<Object> results = new ArrayList<Object>();
        try {
            methods = EVSQueryImpl.class.getMethods();
        }
        catch (Exception e) {
            throw new ApplicationException(
                "Failed to get EVSQueryImpl methods",e);
        }

        Method queryMethod = null;
        for (int m = 0; m < methods.length; m++) {
            Method method = methods[m];
            if (method.getName().startsWith(methodName)) {
                queryMethod = method;
            }
        }

        if (queryMethod == null) {
            throw new ApplicationException(
                "Failed to find EVSQueryImpl method starting with: "+methodName);
        }

        try {
            if (queryMethod.getName().endsWith("byCode")) {
                gov.nih.nci.evs.domain.DescLogicConcept dlc = (gov.nih.nci.evs.domain.DescLogicConcept) source;
                String code = dlc.getCode();
                String vocabName = null;
                if(dlc.getVocabulary()!=null){
                    gov.nih.nci.evs.domain.Vocabulary vocab = dlc.getVocabulary();
                    vocabName = vocab.getName();
                }
                if(vocabName == null){
                    throw new Exception("Vocabulary information not available");
                }

				if (securityToken != null)
				{
					evsQuery.addSecurityToken(vocabName, securityToken);
				}

                queryMethod.invoke(evsQuery, new Object[] { vocabName, code });
            }
            else if (queryMethod.getName().endsWith("byCui")) {
                gov.nih.nci.evs.domain.MetaThesaurusConcept mtc =
                    (gov.nih.nci.evs.domain.MetaThesaurusConcept) source;
                String code = mtc.getCui();

 				if (securityToken != null)
				{
					evsQuery.addSecurityToken("NCI MetaThesaurus", securityToken);
				}
				queryMethod.invoke(evsQuery, new Object[] { code });
            }

            results = evsSearch(evsQuery);
        }
        catch (Exception e) {
            throw new ApplicationException("EVSApplicationService error: " + e);
        }

        return results;
    }

    /* (non-Javadoc)
     * @see gov.nih.nci.system.applicationservice.impl.ApplicationServiceImpl#search(java.lang.String, java.lang.Object)
     */@SuppressWarnings("unchecked")
    public List<Object> search(String searchClass, Object evsCriteria)
            throws ApplicationException {
        Request request = new Request(evsCriteria);
        request.setDomainObjectName(evsCriteria.getClass().getName());
        if (searchClass == null) {
            searchClass = evsCriteria.getClass().getName();
        }
        config.put("searchClass", searchClass);
        request.setConfig(config);
        List<Object> results = new ArrayList<Object>();

        request.setClassCache(evsClassCache);
        String domainObjectName = request.getDomainObjectName();
        if (domainObjectName == null || domainObjectName.equals("")) {
            throw new ApplicationException(
                    "No Domain Object name specified in the request; " +
                    "unable to locate corresponding DAO");
        }
        DAO dao = evsClassCache.getDAOForClass(request.getDomainObjectName());

        if (dao == null)
            throw new ApplicationException("Could not obtain DAO for: "
                    + request.getDomainObjectName());

        try {
            Response r = dao.query(request);
            if (r != null) {
                results = (List)r.getResponse();
            }

        }
        catch (Exception e) {
            throw new ApplicationException("EVSApplicationServiceImpl Error", e);
        }

        return results;
    }

    /**
     * Execute the given method on the specified LexBig object.
     *
     * @param object - Object
     * @param methodName - String
     * @param parameterClasses -String[]
     * @param args - Object[]
     *
     * @return the object
     *
     * @throws ApplicationException the application exception
     */
    public Object executeRemotely(Object object, String methodName,
            String[] parameterClasses, Object[] args) throws ApplicationException {

        log.info("executeRemotely: object=" + object + ", method=" + methodName);

        if (!object.getClass().getName().startsWith("org.LexGrid")) {
            log.warn("executeRemotely called for non-LexBig object: "
                    + object.getClass().getName());
            throw new SecurityException(
                    "Cannot execute method on non-LexBig object");
        }

        try {
            int i = 0;
            Class[] parameterTypes = new Class[parameterClasses.length];
            for (String paramClass : parameterClasses) {
                parameterTypes[i++] = ClassUtils.forName(paramClass);
            }
            Method objMethod = object.getClass().getMethod(methodName,
                parameterTypes);
            return objMethod.invoke(object, args);
        }
        catch (Exception e) {
            throw new ApplicationException(
                "Failed to execute LexBig method remotely",e);
        }
    }

    /**
     * Gets the coding scheme concepts.
     *
     * @param codingScheme the coding scheme String
     * @param versionOrTag the version or tag String
     * @param activeOnly the active only boolean
     *
     * @return the coding scheme concepts as a CodedNodeSet
     *
     * @throws LBException the LB exception
     *
     * @deprecated Not implemented here since it is deprecated in the
     * LexBIGService interface.
     */@Deprecated
    public CodedNodeSet getCodingSchemeConcepts(
    		@SuppressWarnings("unused")String codingScheme,
    		@SuppressWarnings("unused")CodingSchemeVersionOrTag versionOrTag,
    		@SuppressWarnings("unused")boolean activeOnly)
            throws LBException {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getCodingSchemeConcepts(java.lang.String, org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag)
     */
    public CodedNodeSet getCodingSchemeConcepts(String codingScheme,
            CodingSchemeVersionOrTag versionOrTag) throws LBException {
        return lbs.getCodingSchemeConcepts(codingScheme, versionOrTag);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getCodingSchemeConcepts(org.LexGrid.LexBIG.LexBIGService.ValueDomainEntryNodeSet)
     */
    public CodedNodeSet getCodingSchemeConcepts(ValueDomainEntryNodeSet nodeSet)
            throws LBException {
        return lbs.getCodingSchemeConcepts(nodeSet);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getFilter(java.lang.String)
     */
    public Filter getFilter(String name) throws LBException {
        return lbs.getFilter(name);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getFilterExtensions()
     */
    public ExtensionDescriptionList getFilterExtensions() {
        return lbs.getFilterExtensions();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getGenericExtension(java.lang.String)
     */
    public GenericExtension getGenericExtension(String name) throws LBException {
        return lbs.getGenericExtension(name);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getGenericExtensions()
     */
    public ExtensionDescriptionList getGenericExtensions() {
        return lbs.getGenericExtensions();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getHistoryService(java.lang.String)
     */
    public HistoryService getHistoryService(String codingScheme)
            throws LBException {
        return lbs.getHistoryService(codingScheme);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getLastUpdateTime()
     */
    public Date getLastUpdateTime() throws LBInvocationException {
        return lbs.getLastUpdateTime();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getMatchAlgorithms()
     */
    public ModuleDescriptionList getMatchAlgorithms() {
        return lbs.getMatchAlgorithms();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getNodeGraph(java.lang.String, org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag, java.lang.String)
     */
    public CodedNodeGraph getNodeGraph(String codingScheme,
            CodingSchemeVersionOrTag versionOrTag, String relationsName)
            throws LBException {
        return lbs.getNodeGraph(codingScheme, versionOrTag, relationsName);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getServiceManager(java.lang.Object)
     */
    public LexBIGServiceManager getServiceManager(Object credentials)
            throws LBException {
        return lbs.getServiceManager(credentials);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getServiceMetadata()
     */
    public LexBIGServiceMetadata getServiceMetadata() throws LBException {
        return lbs.getServiceMetadata();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getSortAlgorithm(java.lang.String)
     */
    public Sort getSortAlgorithm(String name) throws LBException {
        return lbs.getSortAlgorithm(name);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getSortAlgorithms(org.LexGrid.LexBIG.DataModel.InterfaceElements.types.SortContext)
     */
    public SortDescriptionList getSortAlgorithms(SortContext context) {
        return lbs.getSortAlgorithms(context);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getSupportedCodingSchemes()
     */
    public CodingSchemeRenderingList getSupportedCodingSchemes()
            throws LBInvocationException {
        return lbs.getSupportedCodingSchemes();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getSupportedValueDomains()
     */
    public ValueDomainRenderingList getSupportedValueDomains()
            throws LBInvocationException {
        return lbs.getSupportedValueDomains();
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getValueDomainEntries(org.LexGrid.LexBIG.LexBIGService.ValueDomainNodeSet)
     */
    public ValueDomainEntryNodeSet getValueDomainEntries(
            ValueDomainNodeSet nodeSet) throws LBException {
        return lbs.getValueDomainEntries(nodeSet);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#getValueDomains(boolean)
     */
    public ValueDomainNodeSet getValueDomains(boolean activeOnly)
            throws LBException {
        return lbs.getValueDomains(activeOnly);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#resolveCodingScheme(java.lang.String, org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag)
     */
    public CodingScheme resolveCodingScheme(String codingScheme,
            CodingSchemeVersionOrTag versionOrTag) throws LBException {
        return lbs.resolveCodingScheme(codingScheme, versionOrTag);
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.LexBIGService.LexBIGService#resolveValueDomain(java.lang.String, org.LexGrid.LexBIG.DataModel.Core.ValueDomainVersionOrTag)
     */
    public ValueDomain resolveValueDomain(String valueDomain,
            ValueDomainVersionOrTag versionOrTag) throws LBException {
        return lbs.resolveValueDomain(valueDomain, versionOrTag);
    }

}
