package gov.nih.nci.system.client.proxy;


import gov.nih.nci.evs.query.EVSQueryImpl;
import gov.nih.nci.evs.security.SecurityToken;
import gov.nih.nci.system.applicationservice.ApplicationService;
import gov.nih.nci.system.applicationservice.EVSApplicationService;


import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;

import org.aopalliance.intercept.MethodInvocation;
import org.apache.log4j.Logger;
import org.springframework.aop.framework.Advised;

import java.util.Iterator;


import net.sf.cglib.proxy.Enhancer;
import gov.nih.nci.evs.domain.*;
import gov.nih.nci.evs.query.*;

/**
 * Application Service proxy for EVS.
 * Certain methods are overridden to
 * provide EVS-specific proxying functionality.
 *
 * @author <a href="mailto:rajasimhah@mail.nih.gov">Harsha K Rajasimha</a>
 * @author <a href="mailto:ongki@mail.nih.gov">Kim L. Ong</a>
 */
public class EVSApplicationServiceProxy extends ApplicationServiceProxy {

    private ThreadLocal threadLocal = new ThreadLocal();

    private static Logger log = Logger.getLogger(EVSApplicationServiceProxy.class.getName());

    private EVSApplicationService eas;

	protected HashMap<String, SecurityToken> securityToken_map = new HashMap<String, SecurityToken>();

    @Override
	public void setApplicationService(ApplicationService as) {
    	//System.out.println("In method: setApplicationService...");
		eas = (EVSApplicationService) as;
		// TODO Auto-generated method stub
		super.setApplicationService(eas);
	}

    public HashMap getSecurityToken_map()
    {
		return securityToken_map;
	}

	@Override
	public Object invoke(MethodInvocation invocation) throws Throwable {
		Object bean = invocation.getThis();
        Method method = invocation.getMethod();

        String methodName = method.getName();

		if(methodName.equals("getSecurityToken_map")) {
			return securityToken_map;
		}

        Object[] args = invocation.getArguments();
        Class implClass = bean.getClass();

        SecurityToken securityToken = null;
        String vocabulary = null;
        HashMap tokenCollection = new HashMap();
        String vocabularyName = null;

	    // Check if the invocation.methodName is evsSearch
		if(methodName.equals("evsSearch")) {

		   /* User needs to determine which vocabulary to register
		    * User needs to pass vocabulary name and token string.
		    * registerSecurityToken(String vocabulary, String token)
		    * First argument is assumed to be vocabulary name
		    */
		   EVSQueryImpl criteria = (EVSQueryImpl) args[0];

           if (criteria.getSecurityTokenCollection().size() > 0
                    && criteria.getSecurityTokenCollection() != null) {
                 tokenCollection = criteria.getSecurityTokenCollection();
           }

           HashMap mapValues = new HashMap();
           if(((HashMap)criteria.getClass().getField("descLogicValues").get(criteria)).size()>0){
                mapValues = (HashMap)criteria.getClass().getField("descLogicValues").get(criteria);
           }else if(((HashMap)criteria.getClass().getField("metaThesaurusValues").get(criteria)).size()>0){
                mapValues = (HashMap)criteria.getClass().getField("metaThesaurusValues").get(criteria);
           }

	       Iterator it = tokenCollection.keySet().iterator();
	       SecurityToken st = null;
	       while (it.hasNext())
	       {
			   String key = (String) it.next();
			   st = (SecurityToken) tokenCollection.get(key);
			   // KLO, 05/26/2008
			   securityToken_map.put(key, st);
	       	   break;
		   }

           /*
           int k = 0;
           Iterator ite = mapValues.keySet().iterator();
           while (ite.hasNext())
           {
			   k++;
			   String key = (String) ite.next();
			   Object value = mapValues.get(key);
			   if (k == 1)
			   {
				   if (st != null)
				   {
					    vocabularyName = (String) value;
				   		securityToken_map.put((String) value, st);
				   }
			   }
		   }
		   */

		}

		return super.invoke(invocation);
	}

     /**
      * Returns a list of class names that are parameters to the given method.
      * @param methodImpl
      * @return list of fully-qualified class names
      */
     private String[] getParameterTypes(Method methodImpl) {
         String[] paramClasses =
             new String[methodImpl.getParameterTypes().length];
         int i = 0;
         for(Class paramClass : methodImpl.getParameterTypes()) {
             if (paramClass == null) continue;
             paramClasses[i++] = paramClass.getName();
         }
         return paramClasses;
     }


   /**
    * Returns the underlying object that the specified proxy is advising.
    *
    * @param proxy the proxy
    *
    * @return the object
    *
    * @throws Exception the exception
    */
   private Object unwrap(Object proxy) throws Exception {
       Object interceptor = null;
       int i = 0;
       while (true) {
           Field field = proxy.getClass().getDeclaredField("CGLIB$CALLBACK_"+i);
           field.setAccessible(true);
           Object value = field.get(proxy);
           if (value.getClass().getName().contains("EqualsInterceptor")) {
               interceptor = value;
               break;
           }
           i++;
       }

       Field field = interceptor.getClass().getDeclaredField("advised");
       field.setAccessible(true);
       Advised advised = (Advised)field.get(interceptor);
       Object realObject = advised.getTargetSource().getTarget();
       return realObject;
   }

}
