package gov.nih.nci.system.dao.evs;

import gov.nih.nci.evs.domain.Association;
import gov.nih.nci.evs.domain.Atom;
import gov.nih.nci.evs.domain.Definition;
import gov.nih.nci.evs.domain.DescLogicConcept;
import gov.nih.nci.evs.domain.EdgeProperties;
import gov.nih.nci.evs.domain.MetaThesaurusConcept;
import gov.nih.nci.evs.domain.SemanticType;
import gov.nih.nci.evs.domain.Source;
import gov.nih.nci.evs.domain.Vocabulary;
import gov.nih.nci.evs.query.EVSQueryImpl;
import gov.nih.nci.evs.security.SecurityToken;
import gov.nih.nci.lexbig.ext.ConceptAssociation;
import gov.nih.nci.lexbig.ext.DLBAdapter;
import gov.nih.nci.lexbig.ext.PropertyType;

import gov.nih.nci.system.dao.DAO;
import gov.nih.nci.system.dao.DAOException;
import gov.nih.nci.system.dao.Request;
import gov.nih.nci.system.dao.Response;
import gov.nih.nci.system.dao.cache.EVSCacheManager;
import gov.nih.nci.system.dao.security.DAOSecurity;
import gov.nih.nci.system.dao.security.SecurityKey;
import gov.nih.nci.system.dao.security.UserCredentials;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import org.LexGrid.concepts.CodedEntry;

import org.apache.log4j.Logger;

import gov.nih.nci.evs.util.SortUtils;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2001-2004 SAIC. Copyright 2001-2003 SAIC. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by the SAIC and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "SAIC" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or SAIC-Frederick.
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* SAIC, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */
/**
  * @author EVS Team
  * @version 1.0.1
 */
/**
 * Defines and implements methods to query the Enterprise Vocabulary Service.
 * Results are returned as a Response object.
 */
@SuppressWarnings("unused")
public class EVSQueryDAOImpl implements DAO
{
	private static Logger log = Logger.getLogger(EVSQueryDAOImpl.class.getName());
    private static HashMap <String, DLBAdapter> adapters = new HashMap<String, DLBAdapter>();

    private static HashMap <Integer, String> namespaceId2VocabularyName_Map = new HashMap<Integer, String>();

    private static HashMap <String, Vocabulary>vocabularies = new HashMap<String, Vocabulary>();
    private final static String defaultVocabularyName = "NCI_Thesaurus";
    private final static String metaVocabularyName = "NCI MetaThesaurus";
    // KLO, 022008 GF5381
    private static SimpleDateFormat dateFormat = new SimpleDateFormat();
    private ThreadLocal threadLocal = new ThreadLocal();
//    private static Properties properties = new Properties();
//    private static String fileName = null;
    private Hashtable configs = new Hashtable();

    Properties systemProperties = null;
    /**
     * Instantiates an EVSQueryDAOImpl instance.
     */
   public EVSQueryDAOImpl(){
       log.debug("Loading EVSQueryDAOImpl...");
   }

   /**
    * Instantiates a new EVSQueryDAOImpl instance.
    *
    * @param file the filename String
    */
   public EVSQueryDAOImpl(String file){
//       fileName = file;
       log.debug("Loading EVSQueryDAOImpl..." + file);
   }

   /**
    * Instantiates a new EVSQueryDAOImpl instance.
    *
    * @param file the filename String
    * @param systemProperties Properties object
    */
   public EVSQueryDAOImpl(String file, Properties systemProperties){

       //GF#10439
       this.systemProperties = systemProperties;

//       fileName = file;
       log.debug("Loading EVSQueryDAOImpl..." + file + ", " + systemProperties.getProperty("MedDRA"));
       for(Iterator it =systemProperties.keySet().iterator();it.hasNext();){
           String key = (String)it.next();
           String value = systemProperties.getProperty(key);
           log.debug("System config:  :"+ key +"\tvalue: "+ value);
           configs.put(key, value);
       }
   }


    /**
     * Creates a connection to the EVS datasource and generates a query based on the request.
     * The results are wrapped in a response object
     *
     * @param r - Request object (This object holds the evs query)
     *
     * @return - Response object The results are stored in a list and returned as a response object
     *
     * @throws DAOException the DAO exception
     *
     * */
    public Response query(Request r) throws DAOException {
        Object obj = r.getRequest();
//        List resultList = new ArrayList();
        HashMap tokenCollection = new HashMap();
        Response response = new Response();
        try {
			/*
			// GForge 10439, KLO 040208
            configs.putAll(r.getConfig());
            for(Iterator it = configs.keySet().iterator(); it.hasNext();){
                String key = (String)it.next();
                String value = (String)configs.get(key);
                log.debug("Config value: "+ key +"\t"+ value);
            }
            */
            EVSQueryImpl criteria = (EVSQueryImpl) obj;
            if (criteria.getSecurityTokenCollection().size() > 0
                    && criteria.getSecurityTokenCollection() != null) {
                    tokenCollection = criteria.getSecurityTokenCollection();
            }
            HashMap mapValues = new HashMap();
            if(((HashMap)criteria.getClass().getField("descLogicValues").get(criteria)).size()>0){
                mapValues = (HashMap)criteria.getClass().getField("descLogicValues").get(criteria);
            }else if(((HashMap)criteria.getClass().getField("metaThesaurusValues").get(criteria)).size()>0){
                mapValues = (HashMap)criteria.getClass().getField("metaThesaurusValues").get(criteria);
            }

            try {
                response = queryEVS(configs, tokenCollection, mapValues);

            } catch (Exception ex) {
                DAOException e = null;
                if(ex.getMessage() == null) {
                    if(threadLocal.get() == null){
                        e = new DAOException("Exception in method: "+getMethodName(mapValues) +" - unable to execute query") ;
                    }else{
                        e = (DAOException)threadLocal.get();
                    }
                } else {
                    e = new DAOException(ex.getMessage());
                }
                threadLocal.remove();
                log.error(e.getMessage());
                throw new DAOException(e.getMessage());
            }

        } catch (Exception e) {
            throw new DAOException(e.getMessage());
        }
        return response;
    }

    /**
     * Returns the vocabulary name specified
     * @param mapValues
     * @return vocabulary name
     */
    private String getVocabularyName(HashMap mapValues){
        String vocabularyName = null;
        for(Iterator it = mapValues.keySet().iterator(); it.hasNext();){
            String key = (String)it.next();
            String name = key.substring(key.indexOf("$") + 1, key.length());
            Object value = mapValues.get(key);
            if (name.equalsIgnoreCase("VocabularyName")){
                vocabularyName = (String)value;
                break;
            }
        }

        return vocabularyName;
    }

    /**
     * Returns the method name specified in the hashmap
     * @param mapValues
     * @return Method name specified in the map
     */
    private String getMethodName(HashMap mapValues){
        String methodName = null;
        Iterator iter = mapValues.keySet().iterator();
        String keyMethod = (String) iter.next();
        if(keyMethod != null){
            methodName = keyMethod.indexOf("$") > 0 ? keyMethod.substring(0, keyMethod.indexOf("$")): keyMethod;
        }
        return methodName;
    }

    /**
     * Generates a cache key based on the method name and map values
     * @param methodName
     * @param mapValues
     * @return cache key
     */
    private String getCacheKey(String methodName,HashMap mapValues){
        String key = methodName;
        for(Iterator it = mapValues.keySet().iterator(); it.hasNext();){
            String name = (String)it.next();
            Object value = mapValues.get(name);
            key += "_"+String.valueOf(value);
        }
        return key;
    }

    /**
     * Returns true if the enableCache value in the config file is set to true
     * @param config
     * @return true if enableCache is set to true
     */
    private boolean enableCache(Hashtable config){
        boolean enableCache = true;
        if(config.get("enableCache")!=null){
            try{
                enableCache = Boolean.valueOf((String)config.get("enableCache")).booleanValue();
            }catch(Exception ex){}
        }
        return enableCache;
    }

    /**
     * Executes the query
     * @param config - specifies the values set in the config file
     * @param tokenCollection - specifies the vocabulary tokens
     * @param mapValues - specifies the search method and parameters
     * @return result list is returned in a Response object
     */
    private Response queryEVS(Hashtable config, HashMap tokenCollection, HashMap mapValues) throws Exception{
        Response response = null;
        String vocabularyName = getVocabularyName(mapValues);
        String methodName = getMethodName(mapValues);
        Method method = this.getClass().getDeclaredMethod(methodName, new Class[] { HashMap.class });
        if (method == null) {
            log.error("Invalid method name");
            throw new DAOException(getException("Invalid method name"));
        }
        String key = getCacheKey(methodName, mapValues);
        boolean checkCache = false;
        EVSCacheManager evsCache = null;
        //Check if a security token is required to access the vocabulary
        DAOSecurity security = null;

        //GF#9397 KLO, 120607
        if (methodName.compareTo("getAllLicenses") != 0)
        {
			if(vocabularyName != null){
				if(getSecurityAdapter(vocabularyName)!= null){
					security = getSecurityAdapter(vocabularyName);
				}
			}else{

				try{
					DLBAdapter adapter = getDLBAdapterForMeta();
					//if(adapters.get(metaVocabularyName)!= null){
					if(adapter != null){
						key += "_"+ metaVocabularyName;
					}else{
						key += "_"+ defaultVocabularyName;
					}
				}catch(Exception ex){
					ex.printStackTrace();
					key += "_"+ defaultVocabularyName;
				}
			}
    	}

        if(security != null){
            SecurityToken securityToken = null;
            if(tokenCollection.size()>0){
                securityToken = (SecurityToken)tokenCollection.get(vocabularyName);
            }
            if(securityToken==null){
                Exception ex = new Exception("Permission denied - Please set SecurityToken for "+ vocabularyName);
                log.error(ex.getMessage());
                throw new Exception(getException(ex));
            }

            SecurityKey securityKey = getSecurityKey(security, securityToken);
        }else{
            //Security token is not required to access the Vocabulary
            //Check if cache needs to be enabled
            if(enableCache(config)){
                evsCache = EVSCacheManager.getInstance();
                if(evsCache != null){
                    if(evsCache.isCacheNameValid(methodName)){
                        checkCache = true;
                        try{
                            //read data from cache
                            Object results = evsCache.get(key, methodName);
                            if(results != null){
                                return (Response)results;
                            }
                        }catch(Exception ex){}
                    }
                 }
            }
        }


        try{
            //Query EVS Servers
            if(adapters.size()<1){
                makeRemoteConnection(config);
            }
            response = (Response) method.invoke(this,new Object[] { mapValues });
            //update cache
            if(checkCache){
                evsCache.put(key, response, methodName);
            }
        }catch(Exception ex){
           log.error( methodName + " throws Exception ");
            throw new DAOException(getException(ex.getMessage()));
        }
        return response;
    }

    /**
     * Creates a connection to the EVS Servers
     * @param configs specifies the values set in the config file
     * @param vocabularyName specifies the vocabulary name
     */
    private void makeRemoteConnection(Hashtable configs)throws Exception{
        try{
            if(adapters.size()<1){
                DLBAdapter adapter = null;
                Vector vocabs = new Vector();
                try{
                    try{

                        adapter = new DLBAdapter();

                    }catch(Exception e){
                        throw new Exception("DLBAdapter exception: "+ e);
                    }
 					adapter.setVocabulary(defaultVocabularyName);
                    vocabs = adapter.getVocabularyNames();


                 for(int i=0; i<vocabs.size(); i++){
                	 //loops through vocabs to see if vocabName can be retrieved.
                	 //If any vocab fails to connect, will be thrown into catch block.
                     String vocabName = (String)vocabs.get(i);
                 }

                    //vocabs = adapter.getLocalNames();
                }catch(Exception ex){
                    log.error("Unable to connect to LexBig Server - check server log for details\n"+ ex.getMessage() );
                    throw new Exception("Unable to connect to LexBig Server - check server log for details\n"+ ex );
                }

                try {
					adapters = DLBAdapter.getAdapterHashMap(null);

					Iterator it = adapters.keySet().iterator();
					while (it.hasNext())
					{
						String vocabName = (String) it.next();
						adapter = getDLBAdapter(vocabName);
						Vocabulary vocabulary = new Vocabulary();
						vocabulary.setName(vocabName);
						vocabulary.setDescription(adapter.getVocabularyDescription(vocabName));
						vocabulary.setNamespaceId(adapter.getNamespaceId(vocabName));
						vocabularies.put(vocabName, vocabulary);
					}
				}catch(Exception ex) {
					log.error("Unable to create Adapters");
					//continue;
				}
            }
        }catch(Exception ex){
            throw new DAOException(getException(ex));
        }
     }


     private String getFormalName(String localname)
     {
		 DLBAdapter adapter = new DLBAdapter();
		 return adapter.getFormalCodingSchemeName(localname);
	 }

    /**
     * Returns an instance of DLBAdapter for the specified vocabulary.
     *
     * @param vocabularyName the vocabulary name
     *
     * @return the DLB adapter object
     *
     * @throws DAOException
     */
    private DLBAdapter getDLBAdapter(String vocabularyName) throws DAOException{
        if(adapters==null)
        {
			return null;
		}

		if (adapters.keySet().contains(vocabularyName))
		{
			return adapters.get(vocabularyName);
		}

//		DLBAdapter adapter = null;
        String formalName = getFormalName(vocabularyName);
        return adapters.get(formalName);
    }

    /**
     * Returns a DLBAdapter instance for the MetaThesaurus.
     *
     * @return the DLB adapter object for meta
     *
     * @throws Exception the exception
     */
    private DLBAdapter getDLBAdapterForMeta() throws Exception{
		DLBAdapter adapter = getDLBAdapter(metaVocabularyName);
		if (adapter == null)
		{
            adapter = new DLBAdapter();
            adapter.setVocabulary(metaVocabularyName);
            //adapters.put(metaVocabularyName, adapter);
		}

        return adapter;
    }

     /**
 	 * Gets the descendant concept codes for the specified concept
 	 * @param map - Holds the input parameters
 	 * @return - Descendant concept codes are returned in a Response list
 	 * @throws Exception
 	 */
 	private Response getDescendants(HashMap map) throws Exception
 	{
 		String vocabularyName = null;
 		String conceptCode = null;
 		boolean flag = false;
 		String iBaseLineDate = null;
 		String fBaseLineDate = null;
 		Vector descendants = new Vector();
 		ArrayList list = new ArrayList();
 		try
 		{
 			//parse hashmap
 			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
 			{
 				String key = (String)iter.next();
 				String name = key.substring(key.indexOf("$")+1, key.length());

 				if(name.equalsIgnoreCase("vocabularyName"))
 					vocabularyName = (String)map.get(key);
 				else if(name.equalsIgnoreCase("conceptCode"))
 					conceptCode = (String)map.get(key);
 				else if(name.equalsIgnoreCase("flag"))
 					flag = ((Boolean)map.get(key)).booleanValue();
 				else if(name.equalsIgnoreCase("iBaseLineDate"))
 					iBaseLineDate = (String)map.get(key);
 				else if(name.equalsIgnoreCase("fBaseLineDate"))
 					fBaseLineDate = (String)map.get(key);
 			}

 			DLBAdapter adapter = getDLBAdapter(vocabularyName);
 			descendants = adapter.getDescendantCodes(conceptCode, flag, stringToDate(iBaseLineDate), stringToDate(fBaseLineDate));
 			if(descendants == null){
 				//log.info("No descendants found");
 			}
 			else{
 		 		for(int i=0; i<descendants.size(); i++)
 		 		{
 		 			list.add(descendants.get(i)); // codes
 		 		}
 			}
 		}
 		catch(Exception e)
 		{
 			log.error(e.getMessage());
 			throw new DAOException (getException(e.getMessage()));
 		}
 		return new Response(list);
 	}

      /**
     * Validates vocabulary security token
     * @param vocabularyName
     * @param token
     * @return response
     */
    private Response validateToken(HashMap map)throws SecurityException{
        List validList = new ArrayList();
        String vocabularyName = null;
        SecurityToken securityToken = null;
        boolean valid = true;
        for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
        {
            String key = (String)iter.next();
            String name = key.substring(key.indexOf("$")+1, key.length());

            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("token"))
                securityToken = (SecurityToken)map.get(key);
        }
        DAOSecurity security = getSecurityAdapter(vocabularyName);
        if(security != null){
            if(securityToken == null){
                throw new SecurityException(getException("Please specify security token to access "+ vocabularyName));
            }
            try{
                valid = validateToken(security, securityToken);
            }
            catch(Exception ex){
                valid = false;
            }
            validList.add(valid);
        }else{
            throw new SecurityException(getException("Token not required to access " + vocabularyName ));
        }
        return new Response(valid);
    }

    /**
     * Validates token
     * @param security
     * @param securityToken
     * @return valid
     */
    private boolean validateToken(DAOSecurity security, SecurityToken securityToken) throws SecurityException{
        boolean valid = false;
        try{
            if(getSecurityKey(security, securityToken) != null){
                valid = true;
            }
        }catch(Exception ex){
        }
        return valid;
    }

    /**
     * Returns the SecurityKey for the specified securityToken.
     *
     * @param security the security
     * @param securityToken the security token
     *
     * @return the security key
     */
    private SecurityKey getSecurityKey(DAOSecurity security, SecurityToken securityToken){
        SecurityKey key = null;
        try{
            key = security.getSecurityKey(new UserCredentials(securityToken.getAccessToken()));
        }catch(Exception ex){
            throw new SecurityException(getException(ex.getMessage()));
        }
        return key;
    }

    /**
     * Returns the Security implementation class for the specified vocabulary.
     *
     * @param vocabularyName the vocabulary name
     *
     * @return the security adapter
     */
    private DAOSecurity getSecurityAdapter(String vocabularyName){
        DAOSecurity security = null;
        try{
            //GF#10439
            //security = (DAOSecurity)Class.forName((String)configs.get(vocabularyName)).newInstance();
            if (systemProperties == null) return null;
            String value = systemProperties.getProperty(vocabularyName);
            if (value != null)
            {
				// GForge 10439, KLO 040208
	            //security = (DAOSecurity)Class.forName((String)configs.get(vocabularyName)).newInstance();
	            security = (DAOSecurity)Class.forName(value).newInstance();
			}
       }catch(Exception ex){
            return null;
        }
        return security;
    }

    /**
     * Returns all vocabularies.
     *
     * @param map parameter not used
     *
     * @return the list of all vocabularies as a Response object
     *
     * @throws DAOException the DAO exception
     */
    private Response getAllVocabularies(HashMap map) throws DAOException{
        List vocabList = new ArrayList();
        for(Iterator i= vocabularies.keySet().iterator(); i.hasNext();){
        	try{
                vocabList.add(vocabularies.get(i.next()));
			}
			catch(Exception ex){}
        }
        return new Response(vocabList);
    }


    private Vocabulary getVocabulary(String vocabularyName)
    {
		if(vocabularyName != null){
			if (vocabularies.keySet().contains(vocabularyName))				{
				Vocabulary vocab = (Vocabulary)vocabularies.get(vocabularyName);
				return vocab;
			}
			else
			{
				String formalName = getFormalName(vocabularyName);
				if(vocabularies.get(formalName)!= null){
					Vocabulary vocab = (Vocabulary)vocabularies.get(formalName);
					return vocab;
				}
			}
		}
		return null;
	}



    /**
     * Returns the vocabulary for the name specifed.
     *
     * @param map specifies the vocabulary name
     *
     * @return the vocabulary by name
     *
     * @throws Exception the exception
     */
    private Response getVocabularyByName(HashMap map) throws Exception{
        String vocabularyName = null;
        Vocabulary vocab = new Vocabulary();
        List vocabList = new ArrayList();

        try{
            for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
            {
                String key = (String)iter.next();
                String name = key.substring(key.indexOf("$")+1, key.length());
                if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
            }
            if(vocabularyName != null){
				if (vocabularies.keySet().contains(vocabularyName))				{
                    vocab = (Vocabulary)vocabularies.get(vocabularyName);
                    vocabList.add(vocab);
				}
				else
				{
					String formalName = getFormalName(vocabularyName);
					if(vocabularies.get(formalName)!= null){
						vocab = (Vocabulary)vocabularies.get(formalName);
						vocabList.add(vocab);
					}
			    }
            }

        }catch(Exception ex){
            throw new DAOException(getException(ex.getMessage()));
        }
        return new Response(vocabList);
    }

	/**
	 * Performs a search for a DescLogicConcept in the LexBIG server and returns a response
	 * @param map Specifies the search criteria values
	 * @return Returns a response that holds a list of DescLogicConcept Objects
	 * @throws Exception
	 */
	private Response getDescLogicConcept(HashMap map) throws Exception	{
		String vocabularyName = null;
		String conceptName = null;
//		boolean inputFlag =  false;
//		Vector v = new Vector();
		ArrayList list = new ArrayList();
		try
        {
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("VocabularyName"))
					vocabularyName = (String)map.get(key);
				else if(name.equalsIgnoreCase("conceptName"))
					conceptName = (String)map.get(key);
//				else if(name.equalsIgnoreCase("inputFlag"))
//					inputFlag = ((Boolean)map.get(key)).booleanValue();
			}

//			boolean valid = true;
			DescLogicConcept dlc = new DescLogicConcept();
			//Concept concept = new Concept();

/*
			if(!inputFlag){
	       		validateConceptName(conceptName, vocabularyName);
	       	}
			else{
	       		validateDLConceptCode(conceptName, vocabularyName);
	       	}
*/


            DLBAdapter adapter = getDLBAdapter(vocabularyName);
// 			CodedEntry ce = adapter.getConcept(conceptName, inputFlag, 1);
 			CodedEntry ce = adapter.getConcept(conceptName);

	        //dlc = buildDescLogicConcept(ce, 0, vocabularyName);
	        //KLO, 04/30/08
	        //[#13841] Collections retrieved from NCI Thesaurus are empty
	        dlc = buildDescLogicConcept(ce, 1, vocabularyName);

	        list.add(dlc);
		}catch(Exception e){
			log.error("Error: getDescLogicConcept  "+e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return new Response(list);
	}

	/**
	 * Performs a search for DescLogicConcepts in the LexBig server and returns a response
	 * @param map Specifies the search criteria values
	 * @return Returns a response that holds a list of DescLogicConcept Objects
	 * @throws Exception
	 */
	 /*
	private Response searchDescLogicConcepts(HashMap map) throws Exception
	{
		String vocabularyName = null;
		String searchTerm = null;
		int limit = 1;
		int matchOption = 0;
		String matchType = "";
		int ASDIndex = 0;
//		Vector v = new Vector();
		ArrayList list = new ArrayList();
		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());
				if(name.equalsIgnoreCase("VocabularyName"))
				{
					vocabularyName = (String)map.get(key);
					vocabularyName = vocabularyName.trim();
				}
				else if(name.equalsIgnoreCase("SearchTerm"))
				{
					searchTerm = (String)map.get(key);
					searchTerm = searchTerm.trim();
				}
				else if(name.equalsIgnoreCase("matchLimit"))
					limit = ((Integer)map.get(key)).intValue();
				else if(name.equalsIgnoreCase("matchOption"))
					matchOption = ((Integer)map.get(key)).intValue();
				else if(name.equalsIgnoreCase("matchType"))
					matchType = (String)map.get(key);
				else if(name.equalsIgnoreCase("ASDIndex"))
					ASDIndex = ((Integer)map.get(key)).intValue();
			}

	        if(searchTerm.length()<0){
	        	log.warn("searchTerm cannot be null");
	        	throw new DAOException(getException(" searchTerm cannot be null"));
	        }

	        CodedEntry[] concepts = null;
	        DLBAdapter adapter = getDLBAdapter(vocabularyName);

	        // [#13260] searchDescLogicConcept() not returning results if search for an ID
	        boolean found = false;
	        try {
				CodedEntry ce = adapter.getConcept(searchTerm);
				if (ce != null)
				{
					concepts = new CodedEntry[1];
					concepts[0] = ce;
					found = true;
				}

			} catch (Exception ex) {

			}

	        if (!found)
	        {
				try{
					concepts = (getDLBAdapter(vocabularyName)).searchConcepts(searchTerm, limit, matchOption, matchType,ASDIndex,"contains");

				}catch(Exception ex){

					if(matchOption==0){
						try{
							concepts = adapter.searchConcepts(searchTerm, limit,"contains");
						}catch(Exception e){
							if(e.getMessage() != null){
								throw new DAOException(getException(ex));
							}
						}
					}
					else{
						throw new DAOException(getException(ex));
					}
				}
		    }
            if(concepts.length > 0){
            	DescLogicConcept[] dlc = new DescLogicConcept[concepts.length];

    	        for(int x=0; x<concepts.length; x++){
    	            //dlc[x] = new DescLogicConcept();
    	            dlc[x] = buildDescLogicConcept(concepts[x], ASDIndex, vocabularyName);
    	            list.add(dlc[x]);
    	        }
            }

		}catch(Exception e){

            if(e.getMessage() != null){
                log.error(e.getMessage());
                throw new DAOException(getException(e.getMessage()));
            }
		}
		return new Response(list);
	}
	*/


		private Response searchDescLogicConcepts(HashMap map) throws Exception
		{
			String vocabularyName = null;
			String searchTerm = null;
			int limit = 1;
			int matchOption = 0;
			String matchType = "";
			int ASDIndex = 0;
	//		Vector v = new Vector();
			ArrayList list = new ArrayList();
			try
			{
				for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
				{
					String key = (String)iter.next();
					String name = key.substring(key.indexOf("$")+1, key.length());
					if(name.equalsIgnoreCase("VocabularyName"))
					{
						vocabularyName = (String)map.get(key);
						//[#13213] Trim of leading and trailing spaces
						vocabularyName = vocabularyName.trim();
					}
					else if(name.equalsIgnoreCase("SearchTerm"))
					{
						searchTerm = (String)map.get(key);
						//[#13213] Trim of leading and trailing spaces
						searchTerm = searchTerm.trim();
					}
					else if(name.equalsIgnoreCase("matchLimit"))
						limit = ((Integer)map.get(key)).intValue();
					else if(name.equalsIgnoreCase("matchOption"))
						matchOption = ((Integer)map.get(key)).intValue();
					else if(name.equalsIgnoreCase("matchType"))
						matchType = (String)map.get(key);
					else if(name.equalsIgnoreCase("ASDIndex"))
						ASDIndex = ((Integer)map.get(key)).intValue();
				}
		        if(searchTerm.length()<0){
		        	log.warn("searchTerm cannot be null");
		        	throw new DAOException(getException(" searchTerm cannot be null"));
		        }

		        CodedEntry[] concepts = null;
		        DLBAdapter adapter = getDLBAdapter(vocabularyName);

		        // [#13260] searchDescLogicConcept() not returning results if search for an ID
		        boolean found = false;

				if (matchOption == 0)
				{
					if (searchTerm.indexOf('*') == -1)
					{
						try {
							CodedEntry ce = adapter.getConcept(searchTerm);
							if (ce != null)
							{
								concepts = new CodedEntry[1];
								concepts[0] = ce;
								found = true;
							}

						} catch (Exception ex) {

						}
				    }

					//GF#13212 [KLO, 040308]
					if (!found && searchTerm.indexOf('*') == -1)
					{
						try {
							String matchAlgorithm = "exactMatch";
							concepts = adapter.searchConcepts(searchTerm, limit, matchAlgorithm);
							if (concepts != null && concepts.length == 1)
							{
								found = true;
							}

						} catch (Exception ex) {

						}
					}
				}

		        if (!found)
		        {
					//[GF#13212] Make default search ExactMatch
					String matchAlgorithm = "contains";

					// Exact match properties
					if (searchTerm.indexOf('*') == -1)
					{
						matchAlgorithm = "exactMatch";
					}
					else
					{
						if (searchTerm.startsWith("*"))
						{
							searchTerm = searchTerm.substring(1, searchTerm.length());
						}
						else if (searchTerm.endsWith("*"))
						{
							searchTerm = searchTerm.substring(0, searchTerm.length()-1);
							matchAlgorithm = "startsWith";
						}
						else if (searchTerm.startsWith("*") && searchTerm.endsWith("*"))
						{
							searchTerm = searchTerm.substring(1, searchTerm.length()-1);
						}
				    }

					try{
						concepts = adapter.searchConcepts(searchTerm, limit, matchOption, matchType, ASDIndex, matchAlgorithm);

					} catch(Exception ex) {

						if(matchOption==0){
							try{
								//concepts = adapter.searchConcepts(searchTerm, limit, "contains");
								concepts = adapter.searchConcepts(searchTerm, limit, matchAlgorithm);
							}catch(Exception e){
								if(e.getMessage() != null){
									throw new DAOException(getException(ex));
								}
							}
						}
						else{
							throw new DAOException(getException(ex));
						}
					}
			    }
	            if(concepts.length > 0){
	            	DescLogicConcept[] dlc = new DescLogicConcept[concepts.length];

	    	        for(int x=0; x<concepts.length; x++){
	    	            //dlc[x] = new DescLogicConcept();
	    	            dlc[x] = buildDescLogicConcept(concepts[x], ASDIndex, vocabularyName);
	    	            list.add(dlc[x]);
	    	        }
	            }

			}catch(Exception e){

	            if(e.getMessage() != null){
	                log.error(e.getMessage());
	                throw new DAOException(getException(e.getMessage()));
	            }
			}
			return new Response(list);
	}



	/**
	 * get a DefaultMutableTreeNode for a given DescLogicConcept
	 * @param map - Specifies the input parameters
	 * @return - Returns a Response that holds a DefaultMutableTreeNode in a list
	 * @throws Exception
	 */
	public Response getTree(HashMap map) throws Exception
	{
		List list = new ArrayList();
		String vocabularyName = null;
		String rootName = null;
		boolean direction = false;
		boolean isaFlag = false;
		String levels = null;
		Vector roles = null;
		int attributes = 1;
		boolean direction_role = false;
		DefaultMutableTreeNode tree = null;
		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("VocabularyName"))
					vocabularyName = (String)map.get(key);
				else if(name.equalsIgnoreCase("RootName"))
					rootName = (String)map.get(key);
				else if(name.equalsIgnoreCase("direction"))
					direction = ((Boolean)map.get(key)).booleanValue();
				else if(name.equalsIgnoreCase("isaFlag"))
					isaFlag = ((Boolean)map.get(key)).booleanValue();
				else if(name.equalsIgnoreCase("attributes"))
					attributes = ((Integer)map.get(key)).intValue();
				else if(name.equalsIgnoreCase("levels"))
					levels = (String)map.get(key);
				else if(name.equalsIgnoreCase("roles"))
					roles = (Vector)map.get(key);
				else if(name.equalsIgnoreCase("direction_role"))
					direction_role = ((Boolean)map.get(key)).booleanValue();

			}
			tree = getTree(vocabularyName, rootName, direction, isaFlag, attributes, levels, roles, direction_role);

			if(tree != null){
				list.add(tree);
			}


		}catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException(getException(e.getMessage()));
		}

		return new Response(list);
	}

	/**
	 * Generates a DefaultMutableTreeNode for the specified values
	 * @param vocabularyName
	 * @param rootName
	 * @param direction
	 * @param isaFlag
	 * @param atributes
	 * @param levels
	 * @param roles
	 * @return Returns a DefaultMutableTreeNode object
	 * @throws Exception
	 */
    private DefaultMutableTreeNode getTree(String vocabularyName, String rootName, boolean direction,
			                           boolean isaFlag, int attributes, String levels, Vector roles, boolean direction_role) throws Exception
    {
		DefaultMutableTreeNode evsTree = null;
        //DefaultMutableTreeNode evsTree = new DefaultMutableTreeNode();
        if(vocabularyName == null){
        	log.error("vocabularyName cannot be null");
        	throw new DAOException(" vocabularyName cannot be null");
        }

        if(rootName == null){
        	log.error("Root Node cannot be null");
        	throw new DAOException(" Root Node cannot be null");
        }
        DefaultMutableTreeNode tree = null;
        try
        {
        	int depthLevel = -1;
			if(levels != null)
			{
				if(levels.equalsIgnoreCase("ALL"))
					depthLevel = -1;
                else
                	depthLevel = (new Integer(levels)).intValue();
            }
		  	try
			{
				DLBAdapter adapter = getDLBAdapter(vocabularyName);
				tree = adapter.getTree(rootName, direction, depthLevel, attributes, isaFlag, roles, direction_role);
				if(tree == null){
                    throw new DAOException("Unable to generate Tree  - LexBIG Exception");
                }
                evsTree = convertTree(tree, attributes, vocabularyName);
			}
		  	catch(Exception e)
			{
		  		log.error(e.getMessage());
		  		throw new DAOException (getException( e.getMessage()));
			}

		}
		catch (Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException( e.getMessage()));
        }
		return evsTree;
    }


	/**
	 * Generates a DefaultMutableTreeNode for the specified values
	 * @param vocabularyName
	 * @param rootName
	 * @param direction
	 * @param isaFlag
	 * @param atributes
	 * @param levels
	 * @param roles
	 * @return Returns a DefaultMutableTreeNode object
	 * @throws Exception
	 */
    private DefaultMutableTreeNode getTree(String vocabularyName, String rootName, boolean direction,
			                           boolean isaFlag, int attributes, int levels, Vector roles, boolean direction_role) throws Exception
    {
		DefaultMutableTreeNode evsTree = null;
        //DefaultMutableTreeNode evsTree = new DefaultMutableTreeNode();
        if(vocabularyName == null){
        	log.error("vocabularyName cannot be null");
        	throw new DAOException(" vocabularyName cannot be null");
        }

        if(rootName == null){
        	log.error("Root Node cannot be null");
        	throw new DAOException(" Root Node cannot be null");
        }
        DefaultMutableTreeNode tree = null;
        try
        {
        	int depthLevel = -1;
			
				if(levels <= 0)
					depthLevel = -1;
                else
                	depthLevel = levels;
            
		  	try
			{
				DLBAdapter adapter = getDLBAdapter(vocabularyName);
				tree = adapter.getTree(rootName, direction, depthLevel, attributes, isaFlag, roles, direction_role);
				if(tree == null){
                    throw new DAOException("Unable to generate Tree  - LexBIG Exception");
                }
                evsTree = convertTree(tree, attributes, vocabularyName);
			}
		  	catch(Exception e)
			{
		  		log.error(e.getMessage());
		  		throw new DAOException (getException( e.getMessage()));
			}

		}
		catch (Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException( e.getMessage()));
        }
		return evsTree;
    }

    private DefaultMutableTreeNode convertTree(DefaultMutableTreeNode dlbtree, int attributes, String vocabularyName)
    {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)dlbtree.getRoot();
		gov.nih.nci.lexbig.ext.TreeNode treenode = (gov.nih.nci.lexbig.ext.TreeNode) root.getUserObject();

		CodedEntry ce = (CodedEntry) treenode.getConcept();
		DescLogicConcept dlc = buildDescLogicConcept(ce, attributes, vocabularyName);
		root.setUserObject(null);

        //GF#14304: [KLO, 05/18/2008]
		//dlc.setEdgeProperties(convertEdgeProperties(treenode));
		//dlc.setEdgeProperties(treenode);

		dlc.setEdgeProperties(convertEdgeProperties(treenode));
		//dlc.setTreeNode(convertTreeNode(treenode));

		DefaultMutableTreeNode dlTree = new DefaultMutableTreeNode(dlc, true);

		if(root.getChildCount()>0){
			dlTree = getChildNodes(dlTree, root, attributes, vocabularyName);
		}
		return dlTree;
	}


/**
 * Generates and returns a DefaultMutableTreeNode.
 *
 * @param dlNode the dl node
 * @param dlbNode the dlb node
 * @param attributes the attributes
 * @param vocabularyName the vocabulary name
 *
 * @return the child nodes
 */
	private DefaultMutableTreeNode getChildNodes(DefaultMutableTreeNode dlNode, DefaultMutableTreeNode dlbNode, int attributes, String vocabularyName){
		int childCounter = dlbNode.getChildCount();
		for(int i=0; i<childCounter; i++){
			DefaultMutableTreeNode node = new DefaultMutableTreeNode();
			DefaultMutableTreeNode childnode = (DefaultMutableTreeNode) dlbNode.getChildAt(i);
			gov.nih.nci.lexbig.ext.TreeNode treenode = (gov.nih.nci.lexbig.ext.TreeNode) childnode.getUserObject();
			DescLogicConcept dlc = buildDescLogicConcept((CodedEntry) treenode.getConcept(), attributes, vocabularyName);
			childnode.setUserObject(null);
		    dlc.setEdgeProperties(convertEdgeProperties(treenode));
			node.setUserObject(dlc);
			if(childnode.getChildCount()>0){
				node = getChildNodes(node, childnode, attributes, vocabularyName);
			}
			dlNode.add(node);
		}
		return dlNode;
	}



/**
 * Gets all the concept names for a given property
 * @param map - Specifies the input parameters
 * @return Returns a Response with a list of concept names
 * @throws Exception
 */
private Response getConceptWithPropertyMatching(HashMap map) throws Exception
{
	String vocabularyName = null;
	String propertyName = null;
	String propertyValue = null;
	int limit = 1;
	Vector v = new Vector();
	ArrayList list = new ArrayList();

	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("propertyName"))
				propertyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("propertyValue"))
				propertyValue = (String)map.get(key);
			else if(name.equalsIgnoreCase("matchLimit"))
				limit = ((Integer)map.get(key)).intValue();
		}

		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		v=adapter.findConceptsWithPropertyMatching(propertyName, propertyValue, limit);
		if(v != null){
			for(int i=0; i<v.size(); i++)
			{
				CodedEntry ce = (CodedEntry) v.elementAt(i);
				DescLogicConcept dlc = buildDescLogicConcept(ce, 0, vocabularyName);
				list.add(dlc);
			}
		}

	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}


	return (new Response(list));
}

/**
 * Seaches for a concept in the specified silo.  Currently will return always and empty vector
 *
 * @param map parameter not used
 *
 * @return the concept with silo matching
 *
 * @throws Exception the exception
 *
 * @deprecated Silos are not supported in the LexBIG
 */@Deprecated
private Response getConceptWithSiloMatching(HashMap map) throws Exception
{

//	String vocabularyName = null;
//	String searchTerm = null;
//	String siloName = null;
//	int limit = 10;
//	Vector v = new Vector();
	ArrayList list = new ArrayList();
//
//	try
//	{
//		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
//		{
//			String key = (String)iter.next();
//			String name = key.substring(key.indexOf("$")+1, key.length());
//
//			if(name.equalsIgnoreCase("vocabularyName"))
//				vocabularyName = (String)map.get(key);
//			else if(name.equalsIgnoreCase("searchTerm"))
//				searchTerm = (String)map.get(key);
//			else if(name.equalsIgnoreCase("siloName"))
//				siloName = (String)map.get(key);
//			else if(name.equalsIgnoreCase("matchLimit"))
//				limit = ((Integer)map.get(key)).intValue();
//		}

//        DLBAdapter adapter = getDLBAdapter(vocabularyName);
/*
		v=adapter.findConceptsWithSiloMatching(searchTerm, limit);

		if(v != null){

			for(int i=0; i<v.size(); i++)
			{
				list.add((String)v.get(i));
			}

		}
*/
//	}
//	catch(Exception e)
//	{
//		log.error(e.getMessage());
//		throw new DAOException (getException(e.getMessage()));
//	}


	return (new Response(list));
}


/**
 * gets the concept name for the given concept code
 * @param map
 * @return Response object that holds the concept name in a list
 * @throws Exception
 */

private Response getDescLogicConceptNameByCode(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptCode = null;
	String conceptName = null;
	List conceptList = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);
		}

        DLBAdapter adapter = getDLBAdapter(vocabularyName);

		conceptName = adapter.getConceptNameByCode(conceptCode);
		if(conceptName != null){
			conceptList.add(conceptName);
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}


	return (new Response(conceptList));
}


/**
 * Checks if the first concept is a subconcept of the second concept
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a Boolean value in a list
 * @throws Exception
 */
private Response isSubConcept(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName1 = null;
	String conceptName2 = null;
	Boolean isSub = Boolean.FALSE;
	List isSubList = new ArrayList();
	try
	{

		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode1"))
				conceptName1 = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode2"))
				conceptName2 = (String)map.get(key);
		}


		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		isSub=adapter.isChild(conceptName1,conceptName2);
        if(isSub==null){
            isSub = false;
        }
		isSubList.add(isSub);
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
	return (new Response(isSubList));
}


/**
 * Checks if the specified concept is retired or not.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a Boolean value in a list
 * @throws Exception
 */
private Response isRetired(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptCode = null;
	Boolean retired = Boolean.FALSE;
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);
		}
        DLBAdapter adapter = getDLBAdapter(vocabularyName);
		retired = adapter.isRetired(conceptCode);
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
	List isList = new ArrayList();
	isList.add(retired);
	return (new Response(isList));

}


/**
 * Gets the values of a property owned by a concept
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a the property values in a list
 * @throws Exception
 */
private Response getPropertyValues(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName = null;
	String propertyName = null;
	Vector values = new Vector();
	ArrayList list = new ArrayList();

	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
			else if(name.equalsIgnoreCase("propertyName"))
				propertyName = (String)map.get(key);
		}


        DLBAdapter adapter = getDLBAdapter(vocabularyName);
		values = adapter.getPropertyValues(conceptName, propertyName);

		if(values != null){
			for(int i=0; i<values.size(); i++)
			{
				list.add(values.get(i));
			}
			}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}



	return (new Response(list));

}


/**
 * Gets the ancestor concept codes of the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of concept codes
 * @throws Exception
 */
private Response getAncestors(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptCode = null;
	boolean flag = false;
	String iBaseLineDate = null;
	String fBaseLineDate = null;
	Vector ancestors = new Vector();
	ArrayList list = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);
			else if(name.equalsIgnoreCase("flag"))
				flag = ((Boolean)map.get(key)).booleanValue();
			else if(name.equalsIgnoreCase("iBaseLineDate"))
				iBaseLineDate = (String)map.get(key);
			else if(name.equalsIgnoreCase("fBaseLineDate"))
				fBaseLineDate = (String)map.get(key);
		}

        DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ancestors = adapter.getAncestorCodes(conceptCode, flag, stringToDate(iBaseLineDate), stringToDate(fBaseLineDate));

		if(ancestors.size()> 0){
			for(int i=0; i<ancestors.size(); i++)
			{
				list.add(ancestors.get(i));
			}

		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}




	return (new Response(list));
}



/**
 * Gets the codes or the names, of all subconcepts of the specified concept
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of conceptNames or codes
 * @throws Exception
 */
private Response getSubConcepts(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName = null;
	Boolean inputFlag = null;
	Boolean outputFlag = null;
	ArrayList list = new ArrayList();

	Vector subConcepts = new Vector();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
			else if(name.equalsIgnoreCase("inputFlag"))
				inputFlag = (Boolean)map.get(key);
			else if(name.equalsIgnoreCase("outputFlag"))
				outputFlag = (Boolean)map.get(key);
		}

		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		subConcepts = adapter.getSubConcepts(conceptName, inputFlag, outputFlag);
		if(subConcepts != null){
			for(int i=0; i<subConcepts.size(); i++)
			{
				//CodedEntry ce = (CodedEntry) subConcepts.elementAt(i);
				//DescLogicConcept dlc = buildDescLogicConcept(ce, 0, vocabularyName);
				//list.add(dlc);
				String s = (String) subConcepts.elementAt(i);
				list.add(s);
			}

		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}

	return (new Response(list));

}


/**
 * Gets the codes (or the names) of all superconcepts of the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of super concept names/codes
 * @throws Exception
 */
private Response getSuperConcepts(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName = null;
	Boolean inputFlag = null;
	Boolean outputFlag = null;

	Vector superConcepts = new Vector();
	ArrayList list = new ArrayList();
	try
	{

		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
			else if(name.equalsIgnoreCase("inputFlag"))
				inputFlag = (Boolean)map.get(key);
			else if(name.equalsIgnoreCase("outputFlag"))
				outputFlag = (Boolean)map.get(key);
		}

		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		superConcepts = adapter.getSuperConcepts(conceptName, inputFlag, outputFlag);

		if(superConcepts != null){
			for(int i=0; i<superConcepts.size(); i++)
			{
				//CodedEntry ce = (CodedEntry) superConcepts.elementAt(i);
				//DescLogicConcept dlc = buildDescLogicConcept(ce, 0, vocabularyName);
				//list.add(dlc);
				String s = (String) superConcepts.elementAt(i);
				list.add(s);

			}

		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}




	return (new Response(list));

}


/**
 * Gets the roles owned by the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of roles
 * @throws Exception
 */
private Response getRolesByConceptName(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName = null;

	Vector roles = new Vector();
	ArrayList list = new ArrayList();

	try
	{

		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
		}
		if(conceptName==null)
        	throw new DAOException("conceptName cannot be null");
        DLBAdapter adapter = getDLBAdapter(vocabularyName);
		roles = adapter.getRolesByConceptName(conceptName);
		if(roles != null) {
			for(int i=0; i<roles.size(); i++)
			{
				//String elem = (String) roles.get(i);
				//StringTokenizer st = new StringTokenizer(elem, "$");
				gov.nih.nci.lexbig.ext.Role r = (gov.nih.nci.lexbig.ext.Role) roles.elementAt(i);
				gov.nih.nci.evs.domain.Role role = new gov.nih.nci.evs.domain.Role();
				role.setName(r.getName());
				role.setValue(r.getValue());
				list.add(role);
			}
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}


	return (new Response(list));

}

/**
 * Gets the names and values of properties owned by the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of properties
 * @throws Exception
 */
private Response getPropertiesByConceptName(HashMap map) throws Exception
{

	String vocabularyName = null;
	String conceptName = null;
	ArrayList list = new ArrayList();

	Vector lproperties = new Vector();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
		}
		if(conceptName==null)
        	throw new DAOException("conceptName cannot be null");
        DLBAdapter adapter = getDLBAdapter(vocabularyName);
        lproperties = adapter.getPropertiesByConceptName(conceptName);
		if(lproperties == null){
			log.info("No properties found....");
		} else{
			for(int i=0; i<lproperties.size(); i++)
			{
				gov.nih.nci.lexbig.ext.Property p = (gov.nih.nci.lexbig.ext.Property) lproperties.elementAt(i);
				gov.nih.nci.evs.domain.Property property = new gov.nih.nci.evs.domain.Property();
				property.setName(p.getName());
				property.setValue(p.getValue());
				Vector qualifiers = p.getQualifierCollection();
				Vector w = new Vector();
				for(int j=0; j<qualifiers.size(); j++)
				{
					gov.nih.nci.lexbig.ext.Qualifier q = (gov.nih.nci.lexbig.ext.Qualifier) qualifiers.elementAt(j);
					gov.nih.nci.evs.domain.Qualifier qualifier = new gov.nih.nci.evs.domain.Qualifier();
					qualifier.setName(q.getName());
					qualifier.setValue(q.getValue());

					w.add(qualifier);
			    }
				list.add(property);
			}
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}

	return (new Response(list));

}



/**
 * Gets vocabulary names from EVS.
 *
 * @param map paramter not used
 *
 * @return - Returns a response that holds a list of vocabulary names
 *
 * @throws Exception the exception
 */
private Response getVocabularyNames(HashMap map) throws Exception
{
    ArrayList list = new ArrayList();
	try
	{
		for(Iterator i= vocabularies.keySet().iterator(); i.hasNext();)
		{
            String name = (String)i.next();

			list.add(name);
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
	return (new Response(list));
}



/**
 * Gets the code of the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of codes
 * @throws Exception
 *
 * @deprecated - evsapi4 does not support name as an identifier
 */@Deprecated
private Response getConceptCodeByName(HashMap map) throws Exception
{

	String vocabularyName = null;
	String conceptName = null;
	String conceptCode = null;
	List conceptList = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);
		}
		if(conceptName== null){
			String msg = "conceptName cannot be null";
			log.error(msg);
			throw new DAOException(getException(msg));
		}
		conceptCode = getDLBAdapter(vocabularyName).getConceptCodeByName(conceptName);

		if(conceptCode != null){
			conceptList.add(conceptCode);

		}
		else{
			//log.info("No conceptCode found for "+conceptName);
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}



	return (new Response(conceptList));

}



/**
 * Gets the host of a vocabulary server
 * @param map - parameter not used
 * @return - Returns a response
 * @throws Exception
 * @deprecated - lexbig does not have host
 */@Deprecated
private Response getVocabularyHost(HashMap map) throws Exception
{
//	String vocabularyName = null;
	String host = null;
/*
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
		}
		host = getDLBAdapter(vocabularyName).getVocabularyHost(vocabularyName);
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}
*/
	List hostList = new ArrayList();
	hostList.add(host);
	return (new Response(hostList));

}



/**
 * Gets the port of a vocabulary server
 * @param map parameter not used
 * @return returns a response that holds the vocabulary server port number
 * @throws Exception
 * @deprecated - lexbig does not have port
 */@Deprecated
private Response getVocabularyPort(HashMap map) throws Exception
{
//	String vocabularyName = null;
	String port = null;
/*
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
		}
		port = getDLBAdapter(vocabularyName).getVocabularyPort(vocabularyName);
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}
*/
	List portList = new ArrayList();
	portList.add(port);

	return (new Response(portList));
}



/**
 * Returns a list of versions for the given vocabulary in the knowledgebase.
 *
 * @param map the map
 *
 * @return the vocabulary version
 *
 * @throws Exception the exception
 */
public Response getVocabularyVersion(HashMap map) throws Exception{
    String vocabularyName = null;
    List version = new ArrayList();

    try
    {
        for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
        {
            String key = (String)iter.next();
            String name = key.substring(key.indexOf("$")+1, key.length());

            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
        }
       version.add(getDLBAdapter(vocabularyName).getVocabularyVersion(vocabularyName));
    }
    catch(Exception e)
    {
        log.error(e.getMessage());
        throw new DAOException (getException( e.getMessage()));
    }

    return (new Response(version));

}


/**
 * Gets all EditAction of the specified concept. A EditAction contains a edit date
 * and an editing action. The method searches for all editing actions and the corresponding
 * editing dates on the specified concept.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of edit actions and dates
 * @throws Exception
 */
private Response getConceptEditAction(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptCode = null;
	Vector editActions = new Vector();
	ArrayList list = new ArrayList();
    Date editActionDate = null;
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);
            else if(name.equalsIgnoreCase("editActionDate"))
                editActionDate = (Date)map.get(key);
		}

        if(editActionDate == null){
            editActions = getDLBAdapter(vocabularyName).getConceptEditAction(conceptCode);
        }
        else{
            editActions = getDLBAdapter(vocabularyName).getConceptEditAction(conceptCode, editActionDate);
        }

		if(editActions != null) {
			for(int i=0; i<editActions.size(); i++)
			{
				gov.nih.nci.lexbig.ext.EditActionDate editactiondate
				   = (gov.nih.nci.lexbig.ext.EditActionDate) editActions.elementAt(i);
                int iAction = editactiondate.getAction();
                String action = gov.nih.nci.lexbig.ext.EditActionDate.actionToString(iAction);
                //Date date = stringToDate(editactiondate.getEditDate());
                Date date = editactiondate.getEditDate();
                gov.nih.nci.evs.domain.EditActionDate ead = new gov.nih.nci.evs.domain.EditActionDate();
                ead.setAction(gov.nih.nci.lexbig.ext.EditActionDate.actionToId(action));
                ead.setEditDate(date);
                list.add(ead);
			}
		}

	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}


	return (new Response(list));

}




/**
 * Gets EditActionDate that match with the given concept code and edit action.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of edit action dates
 * @throws Exception
 */
private Response getConceptEditActionDates(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptCode = null;
	String action = null;
	Vector editActions = new Vector();
	ArrayList list = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);
			else if(name.equalsIgnoreCase("action"))
				action = (String)map.get(key);
		}
		editActions = getDLBAdapter(vocabularyName).getConceptEditActionDates(conceptCode, action);
		if(editActions != null){
			for(int i=0; i<editActions.size(); i++)
			{
				list.add(editActions.get(i));
			}
		}

	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
	return (new Response(list));

}


/**
 * Retrieves all root concepts.
 * @param map - Specifies the input parameters
 * @return - Returns a response that holds a list of DescLogicConcept objects
 * @throws Exception
 */
private Response getRootConcepts(HashMap map) throws Exception
{
	String vocabularyName = null;
	boolean allAttributes = true;

	CodedEntry[] concepts = null;
	ArrayList list = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("allAttributes"))
				allAttributes = ((Boolean)map.get(key)).booleanValue();
		}

		concepts = getDLBAdapter(vocabularyName).getRootConcepts(allAttributes);
		if(concepts.length >0) {

		    for(int i=0; i<concepts.length; i++)
			{
				// [#13395] getHasChildren throwing error
				// list.add(buildDescLogicConcept(concepts[i], 1, vocabularyName));
				if (allAttributes)
				{
					list.add(buildDescLogicConcept(concepts[i], 1, vocabularyName));
				}
				else
				{
					list.add(buildDescLogicConcept(concepts[i], 0, vocabularyName));
				}
			}

			//GF13917
			//SortUtils.quickSort(list, 0, list.size()-1, SortUtils.SORT_BY_NAME);
			SortUtils.quickSort(list);
		}
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}
	return (new Response(list));
}


/**
 * Converts LexBIG property objects to evs property objects
 * @param nciProps - vector of LexBIG property object
 * @return Returns a vector of evs property objects
 */
private Vector convertProperties(Vector nciProps)
{
	Vector lproperties = new Vector();
	for(int i=0 ;i<nciProps.size(); i++)
	{
        gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) nciProps.get(i);
		gov.nih.nci.evs.domain.Property p = new gov.nih.nci.evs.domain.Property();
		p.setName(property.getName());
		p.setValue(property.getValue());
		p.setQualifierCollection(convertQualifiers(property.getQualifiers()));


		lproperties.add(p);
	}
	return lproperties;
}


private Vector convertQualifiers(Vector dtsrpcQualifiers){
 	Vector qualifierVector = new Vector();

 	for(int i=0; i<dtsrpcQualifiers.size(); i++){
 	   gov.nih.nci.evs.domain.Qualifier qualifier = new gov.nih.nci.evs.domain.Qualifier();
 	   gov.nih.nci.lexbig.ext.Qualifier dtsQualifier = (gov.nih.nci.lexbig.ext.Qualifier)dtsrpcQualifiers.get(i);
 	   qualifier.setName(dtsQualifier.getName());
 	   qualifier.setValue(dtsQualifier.getValue());
 	   qualifierVector.add(qualifier);
 	    }
 	return qualifierVector;
}
/**
 * Converts dtsrpc role objects to evs role objects
 * @param nciRoles vector of dtsrpc roles
 * @return Returns a vector of evs roles
 */
private Vector convertRoles(Vector nciRoles)
{
	Vector roles = new Vector();
	for(int i=0 ;i<nciRoles.size(); i++)
	{
		gov.nih.nci.lexbig.ext.Role dtsRole = (gov.nih.nci.lexbig.ext.Role)nciRoles.get(i);
		gov.nih.nci.evs.domain.Role role = new gov.nih.nci.evs.domain.Role();
		role.setName(dtsRole.getName());
		role.setValue(dtsRole.getValue());
		roles.add(role);
	}
	return roles;
}


/**
 * Gets DescLogicConcept object for the given conceptname
 * @param map
 * @return Returns a response that holds a DescLogicConcept in a list
 *
 * @deprecated -evsapi4 does not support name as an identifier
 * @throws Exception
 */@Deprecated
private Response getConceptByName(HashMap map) throws Exception
{
	String vocabularyName = null;
	String conceptName = null;
//	ArrayList list = new ArrayList();
	List dlcList = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);

		}
        DescLogicConcept dlc = buildDescLogicConcept(getDLBAdapter(vocabularyName).findConcept(conceptName),
            0, vocabularyName);
        dlcList.add(dlc);
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(dlcList));
	}

/**
 * Searches MetaThesaurus
 * @param map - specifies the input parameters
 * @return Returns a response that holds a list of MetaThesaurusConcepts
 * @throws Exception
 */
private Response searchMetaThesaurus(HashMap map) throws Exception
{
	String searchTerm = null;
	int limit = 10;
	String source = "*";
	boolean cui = true;
//	boolean shortResult = false;
//	boolean score = false;

	ArrayList list = new ArrayList();
//	List uList = new ArrayList();
	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
			if(name.equalsIgnoreCase("searchTerm"))
			{
				//[#13213] Trim of leading and trailing spaces
				searchTerm = (String)map.get(key);
				searchTerm = searchTerm.trim();
			}
			else if(name.equalsIgnoreCase("limit"))
				limit = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("source")) {
			    if(!(map.get(key)==null)){
                    source = (String)map.get(key);
			    }
				if (source.toLowerCase().equals("all sources")) {
					source = "*";
				}
			}
			else if(name.equalsIgnoreCase("cui"))
				cui = ((Boolean)map.get(key)).booleanValue();
//			else if(name.equalsIgnoreCase("shortResult"))
//				shortResult = ((Boolean)map.get(key)).booleanValue();
//			else if(name.equalsIgnoreCase("score"))
//				score = ((Boolean)map.get(key)).booleanValue();
		}

//		boolean checkSource = false;
		if(!source.equals("*")){
			if(!(source.length() == 0 || source.indexOf("*") > 0)){
                if(!validateSource(source)){
				throw new DAOException("Invalid source "+ source);
				}
//				checkSource = true;
			}
		}


		//[GF#13212] Make default search ExactMatch
		String matchAlgorithm = "contains";
		if (searchTerm.startsWith("*"))
		{
			searchTerm = searchTerm.substring(1, searchTerm.length());
		}
		else if (searchTerm.endsWith("*"))
		{
			searchTerm = searchTerm.substring(0, searchTerm.length()-1);
			matchAlgorithm = "startsWith";
		}
		else if (searchTerm.startsWith("*") && searchTerm.endsWith("*"))
		{
			searchTerm = searchTerm.substring(1, searchTerm.length()-1);
		}

        DLBAdapter adapter = getDLBAdapterForMeta();
//      CodedEntry[] concepts = adapter.searchConcepts("NCI_MetaThesaurus", searchTerm, limit, source, cui);
        CodedEntry[] concepts = adapter.searchConcepts("NCI_MetaThesaurus", searchTerm, limit, source, cui, matchAlgorithm);

        if (concepts != null)
        {
			for (int i=0; i<concepts.length; i++)
			{
				CodedEntry ce = (CodedEntry) concepts[i];
	            MetaThesaurusConcept metaConcept = buildMetaThesaurusConcept(ce, 0);
				list.add(metaConcept);
			}
		}

    }
	catch(Exception e)
	{
        if(e != null){
            log.error("Error: searchMetaThesaurus - "+e.getMessage());
            throw new DAOException (getException( e.getMessage()));
        }
	}

	return new Response(list);
}


/**
 * Validate source.
 *
 * @param source the source
 *
 * @return true, if validate source
 *
 * @throws Exception the exception
 */
private boolean validateSource(String source) throws Exception{
		boolean valid = false;
		HashMap map = new HashMap();

		try{
			List metaSources = (List)getMetaSources(map).getResponse();
			for(int i=0; i<metaSources.size(); i++){
			String sourceName = ((Source)metaSources.get(i)).getAbbreviation();
			if(source.equalsIgnoreCase(sourceName)){
				valid = true;
				}
			}
			}catch(Exception e){
			    String msg = null;
			    if(e.getMessage() == null){
			        msg = "Invalid source - "+ source;
			        }
			    else{
			        source = e.getMessage();
			        }
			throw new DAOException(getException(msg));
			}
		return valid;

	}




  /**
   * Gets sources from metaphrase for the given metaphraseConcept
   * @param metaphraseConcept
   * @return List of Sources
   * @throws Exception
   */
  private ArrayList getSources(CodedEntry metaphraseConcept)
  throws Exception
  {
    ArrayList sourceList = new ArrayList();
  	try
	{
       sourceList = buildMetaThesaurusConcept(metaphraseConcept).getSourceCollection();
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException("MetaThesaurus exception - while getting sources \n"+ e.getMessage());
	}
	return sourceList;
  }



  /**
   * Gets synonyms for the given concept from metaphrase
   * @param metaConcept
   * @return Returns a list of Synonyms
   * @throws Exception
   */
  private ArrayList getSynonyms(CodedEntry metaConcept) throws Exception
  {
    ArrayList synonyms = new ArrayList();
  	try
	{
       synonyms = buildMetaThesaurusConcept(metaConcept).getSynonymCollection();
	}
	catch(Exception e)
	{
		log.error(e.getMessage());
		throw new DAOException(getException( e.getMessage()));
	}
	return synonyms;
  }


	/**
	 * Search by LoincId in the specified source.
	 *
	 * @param map the map
	 *
	 * @return the response
	 *
	 * @throws Exception the exception
	 * @deprecated - use searchSourceByAtomCode
	 */@Deprecated
	private Response searchByLoincId(HashMap map) throws Exception
	{
		return searchMetaThesaurusConceptByAtomCode(map);
	}


	/**
	 * Search by atom code in the specified source.
	 *
	 * @param map the map
	 *
	 * @return the response object
	 *
	 * @throws Exception the exception
	 */
	private Response searchSourceByAtomCode(HashMap map) throws Exception
	{
		String code = null;
		String sourceAbbr = "*";
        int limit = 100;

		ArrayList list = new ArrayList();
		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("code"))
					code = (String)map.get(key);
				else if(name.equalsIgnoreCase("sourceAbbr")){
					if(map.get(key)!=null){
						sourceAbbr = (String)map.get(key);
					}
				}else if(name.equalsIgnoreCase("limit"))
					if(map.get(key)!=null){
						try{
							limit = ((Integer)map.get(key)).intValue();
						}catch(Exception ex){}
					}
				}

            DLBAdapter adapter = getDLBAdapterForMeta();
            Vector metaConcepts = adapter.findConceptsWithSourceCodeMatching("NCI MetaThesaurus", sourceAbbr, code, limit);
            if (metaConcepts == null)
            {
				return new Response(list);
			}

            for (int i=0; i<metaConcepts.size(); i++)
            {
				CodedEntry ce = (CodedEntry) metaConcepts.elementAt(i);
            	MetaThesaurusConcept mtc = buildMetaThesaurusConcept(ce);
            	list.add(mtc);
			}

		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return new Response(list);
	}

	/**
	 * Search by atom code in the specified source.
	 *
	 * @param map the map
	 *
	 * @return the response object
	 *
	 * @throws Exception the exception
	 * @deprecated - use searchSourceByAtomCode
	 */@Deprecated
	private Response searchMetaThesaurusConceptByAtomCode(HashMap map) throws Exception
	{
		String code = null;
		String sourceAbbr = "*";
        int limit = 100;

		ArrayList list = new ArrayList();
		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("code"))
					code = (String)map.get(key);
				else if(name.equalsIgnoreCase("sourceAbbr")){
					if(map.get(key)!=null){
						sourceAbbr = (String)map.get(key);
					}
				}else if(name.equalsIgnoreCase("limit"))
					if(map.get(key)!=null){
						try{
							limit = ((Integer)map.get(key)).intValue();
						}catch(Exception ex){}
					}
				}

            DLBAdapter adapter = getDLBAdapterForMeta();
            Vector metaConcepts = adapter.findConceptsWithSourceCodeMatching("NCI MetaThesaurus", sourceAbbr, code, limit);
            if (metaConcepts == null)
            {
				return new Response(list);
			}

            for (int i=0; i<metaConcepts.size(); i++)
            {
				CodedEntry ce = (CodedEntry) metaConcepts.elementAt(i);
            	MetaThesaurusConcept mtc = buildMetaThesaurusConcept(ce);
            	list.add(mtc);
			}

		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return new Response(list);
	}

	/**
	 * Gets all semantic types from metaphrase
	 * @param map parameter not used
	 * @return Returns a response that holds a list of SemanticTypes
	 * @throws Exception
	 */
	private Response getSemanticTypes(HashMap map) throws Exception
	{
		ArrayList list = new ArrayList();
		try
		{
            DLBAdapter adapter = getDLBAdapterForMeta();
          if(adapter.getSemanticTypes() != null){
              HashMap types = adapter.getSemanticTypes();
            for(Iterator i = types.keySet().iterator(); i.hasNext();){
                  String key = (String) i.next();
                SemanticType semanticType = new SemanticType();
                  semanticType.setId(key);
                  semanticType.setName((String)types.get(key));
                  list.add(semanticType);
              }
          }
		}
		catch(Exception e)
		{
			log.error("Exception occured on getSemanticTypes"+ e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return (new Response(list));
	}



	/**
	 * Gets MetaThesaurusConcepts for the specified source
	 * @param map
	 * @return Returns a response that holds a list of MetaThesaurusConcepts
	 * @throws Exception
	 */
	private Response getConceptsBySource(HashMap map) throws Exception
	{
		ArrayList list = new ArrayList();
		String sourceAbbr = null;
        String conceptCode = null;
		Vector metaConcepts = new Vector();
		try
		{

			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("sourceAbbr"))
					sourceAbbr = (String)map.get(key);
                if(name.equalsIgnoreCase("conceptCode"))
                    conceptCode = (String)map.get(key);
			}

			if(sourceAbbr== null)
				  throw new DAOException(" Invalid Source");

            if(conceptCode == null){
                throw new Exception("Please specify conceptCode");
            }
     /** check this ***/
            log.info("findConceptWithSourceCodeMatching..."+ sourceAbbr +"\t"+ conceptCode);
			metaConcepts = getDLBAdapterForMeta().findConceptsWithSourceCodeMatching(sourceAbbr, conceptCode, 1);
            for(int i=0; i<metaConcepts.size(); i++){
                CodedEntry concept = (CodedEntry)metaConcepts.get(i);
                list.add(this.buildMetaThesaurusConcept(concept));
            }
		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return (new Response(list));
	}


	/**
	 * Gets MetaThesaurusConcept name
	 * @param map
	 * @return Returns a response that holds a MetaThesaurusConcept in a list
	 * @throws Exception
	 */
	private Response getMetaConceptNameByCode(HashMap map) throws Exception
	{
//		ArrayList list = new ArrayList();
		String conceptCode = null;
//		CodedEntry metaConcept = null;
//		String conceptName = null;
		List conceptList = new ArrayList();

		try
		{

			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String)map.get(key);
			}

			if(conceptCode==null)
				  throw new DAOException(" Invalid concept code");


			//metaConcept = getDLBAdapterForMeta().findConceptByCode(conceptCode, 1);
			//conceptList.add(metaConcept.getName());
//            conceptList.add(getDLBAdapterForMeta().findConceptNameByCode("NCI_MetaThesaurus", null, conceptCode));
            conceptList.add(getDLBAdapterForMeta().getConceptName("NCI_MetaThesaurus", null, conceptCode));


		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}



		return (new Response(conceptList));
	}

	private List getMetaSources() throws Exception{
		 List list = new ArrayList();
			try
			{
		 		Vector sources= getDLBAdapterForMeta().getSupportedSources();
		  		for(int i=0; i< sources.size(); i++ )
				{
	                Source source =  new Source();
	                source.setAbbreviation((String)sources.get(i));
					list.add(source);
				}

			}
			catch(Exception e)
			{
				log.error(e.getMessage());
				throw new DAOException (getException( e.getMessage()));
			}

			return list;
	}


	/**
	 * Gets all MetaThesaurus sources.
	 *
	 * @param map parameter not used
	 *
	 * @return the meta sources
	 *
	 * @throws Exception the exception
	 */
	private Response getMetaSources(HashMap map) throws Exception
	{
		return (new Response(getMetaSources()));
	}


	/**
	 * Gets children of the specified concept in the specified source
	 * @param map Specifies the input parameters
	 * @return Returns a response that holds a list of MetaThesaurusConcepts
	 * @throws Exception
	 */
	private Response getChildren(HashMap map) throws Exception
	{
		ArrayList list = new ArrayList();
		String conceptCode = null;
		String sourceAbbr = null;

		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String)map.get(key);
				else if(name.equalsIgnoreCase("sourceAbbr"))
					sourceAbbr = (String)map.get(key);

			}

            // list = getRelatedConcepts(conceptCode, sourceAbbr, "RN");
            list = getRelatedConcepts(conceptCode, true, DLBAdapter.DEFAULT_ASSOC_FWD, sourceAbbr);
            if (list == null || list.size() == 0)
            {
				list = getRelatedConcepts(conceptCode, false, DLBAdapter.DEFAULT_ASSOC_REV, sourceAbbr);
			}
		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException( e.getMessage()));
		}
		return new Response(list);
	}


	/**
	 * Gets  Parent concepts for the specified concept in a given source
	 * @param map Specifies the input parameters
	 * @return Returns a response that holds a list of MetaThesaurusConcepts
	 * @throws Exception
	 */
	private Response getParent(HashMap map) throws Exception
	{
		ArrayList list = new ArrayList();
		String conceptCode = null;
		String sourceAbbr = null;
		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String)map.get(key);
				else if(name.equalsIgnoreCase("sourceAbbr"))
					sourceAbbr = (String)map.get(key);
			}

            //list = getRelatedConcepts(conceptCode, sourceAbbr, "RB");
            list = getRelatedConcepts(conceptCode, false, DLBAdapter.DEFAULT_ASSOC_FWD, sourceAbbr);
            if (list == null || list.size() == 0)
            {
				list = getRelatedConcepts(conceptCode, true, DLBAdapter.DEFAULT_ASSOC_REV, sourceAbbr);
			}
		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return new Response(list);
	}


	 /**
 	 * Retrievs MetaThesaurusConcepts based on the specified code and relation in a given source.
 	 *
 	 * @param conceptCode the concept code
 	 * @param sourceAbbr the source abbr
 	 *
 	 * @return the related concepts
 	 *
 	 * @throws Exception the exception
 	 */
     private ArrayList getRelatedConcepts(String conceptCode, String sourceAbbr) throws Exception
	 {
		 return getRelatedConcepts(conceptCode, sourceAbbr, null);
	 }


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//To be modified:

	 /**
 * Retrievs MetaThesaurusConcepts based on the specified code and relation in a given source.
 *
 * @param relation the relation
 * @param conceptCode the concept code
 * @param sourceAbbr the source abbr
 *
 * @return the related concepts
 *
 * @throws Exception the exception
 */
     private ArrayList getRelatedConcepts(String conceptCode, String sourceAbbr, String relation) throws Exception
	 {
		 String[] assocs = null;
		 if (relation == null)
		 {
			 assocs = new String[1];
			 assocs[0] = relation;
		 }
		 return getRelatedConcepts(conceptCode, true, assocs, sourceAbbr);
     }

     private ArrayList getRelatedConcepts(String conceptCode, boolean direction, String[] assocs, String sourceAbbr) throws Exception
	 {
		 ArrayList uniqueList = new ArrayList();
         Set list = new HashSet();
	  	 try
	  	 {
			if(conceptCode==null)
			{
				throw new DAOException(" Invalid code, set concept code");
			}

			DLBAdapter adapter = getDLBAdapterForMeta();
			Vector v = adapter.getRelatedConcepts(conceptCode, direction, assocs, sourceAbbr, 0);
            for(int i=0; i< v.size(); i++){
				CodedEntry ce = (CodedEntry) v.elementAt(i);
				if (!list.contains(ce.getConceptCode()))
				{
					list.add(ce.getConceptCode());
            	    uniqueList.add(this.buildMetaThesaurusConcept(ce.getConceptCode(), 0));
			    }
            }
	  	 }
		 catch(Exception e)
		 {
			log.error(e.getMessage());
			throw new DAOException(getException(e.getMessage()));
		 }
         return uniqueList;
	 }


   /**
    * Gets concepts more general than the specified concept
    * @param map Specifies the input parameters
    * @return Returns a response that holds a list of MetaThesaurusConcepts
    * @throws Exception
    * @deprecated - use getParent(map)
    */@Deprecated
   private Response getBroaderConcepts(HashMap map) throws Exception
   {
	   return getParent(map);
   }


   /**
    * Gets concepts more specific than the specified concept
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of MetaThesaurusConcepts
    * @throws Exception
    * @deprecated - use getChildren(map)
    */@Deprecated
   private Response getNarrowerConcepts(HashMap map) throws Exception
   {
        return getChildren(map);
   }


   /**
    * Gets MetaThesaurusConcepts related to the specified concept
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of MetaThesaurusConcepts
    * @throws Exception
    */@SuppressWarnings("all")
   private Response getRelatedConcepts(HashMap map) throws Exception
   {
		ArrayList uniqueList = new ArrayList();
        Set list = new HashSet();

		String conceptCode = null;
		String sourceAbbr = null;
		Vector rc = new Vector();
		String relation = null;//"*";

		try
		{

			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String)map.get(key);
				else if(name.equalsIgnoreCase("sourceAbbr"))
					sourceAbbr = (String)map.get(key);
				else if(name.equalsIgnoreCase("relation"))
					relation = (String)map.get(key);
			}

			uniqueList = this.getRelatedConcepts(conceptCode, sourceAbbr, relation);
		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return new Response(uniqueList);
   }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


   /**
    * Gets the concepts of specified category  for the given concept
    * category can be "Medications", "Procedures", "Laboratory", "Diagnosis"
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of MetaThesaurusConcepts
    * @throws Exception
    * @deprecated
    */@Deprecated
   private Response getConceptsByCategories(HashMap map) throws Exception
   {
		ArrayList list = new ArrayList();
		String conceptCode = null;
		String category = null;
		CodedEntry[] metaConcepts = null;

		try
		{
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String)map.get(key);
				else if(name.equalsIgnoreCase("category"))
					category = (String)map.get(key);
			}


			if(conceptCode==null)
				  throw new DAOException(" invalid concept code ");

			if(category==null)
				  throw new DAOException(" Invalid Category - set value");

/**** check code ****/
            metaConcepts = getDLBAdapterForMeta().searchConcepts(conceptCode, 100, 1, category, 1);


/*
			Cooccurrence[] categoryValues = null;
			if(category.equalsIgnoreCase("Medications"))
				categoryValues = metaConcept.cooccurs(Cooccurrence.MED, metaphrase.defaultCooccurClassifier);
			else if(category.equalsIgnoreCase("Procedures"))
				categoryValues = metaConcept.cooccurs(Cooccurrence.PROC, metaphrase.defaultCooccurClassifier);
			else if(category.equalsIgnoreCase("Laboratory"))
				categoryValues = metaConcept.cooccurs(Cooccurrence.LAB, metaphrase.defaultCooccurClassifier);
			else if(category.equalsIgnoreCase("Diagnosis"))
				categoryValues = metaConcept.cooccurs(Cooccurrence.DIAG, metaphrase.defaultCooccurClassifier);
*/


			for(int i=0; i<metaConcepts.length; i++)
			{
				list.add(buildMetaThesaurusConcept(metaConcepts[i]));
			}

		}
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException( e.getMessage()));
		}

		return new Response(list);
   }


   /**
    * Returns a Boolean value if the specified concept contains inverse roles
    * @param map Specifies the input parameters
	* @return Returns a response that holds a Boolean value in a list
	* @throws Exception
    */
   public Response containsInverseRole(HashMap map) throws Exception{

	String vocabularyName = null;
   	String roleName = null;
   	String roleValue = null;
   	String conceptName = null;

	Boolean inverseRole = Boolean.FALSE;

	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("roleName"))
				roleName = (String)map.get(key);
			else if(name.equalsIgnoreCase("roleValue"))
				roleValue = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);

		}
		inverseRole = getDLBAdapter(vocabularyName).containsInverseRole(roleName, roleValue, conceptName);
	}catch(Exception e){
		log.error(e.getMessage());
		throw new DAOException(getException(e.getMessage()));
		}
	List booleanList = new ArrayList();
	booleanList.add(inverseRole);

   	return new Response(booleanList);

   	}


   /**
    * Returns a Boolean value if the specified concept contains  roles
    * @param map Specifies the input parameters
	* @return Returns a response that holds a Boolean value in a list
	* @throws Exception
    */
   public Response containsRole(HashMap map) throws Exception{

	String vocabularyName = null;
   	String roleName = null;
   	String roleValue = null;
   	String conceptName = null;

	Boolean inverseRole = Boolean.FALSE;

	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("roleName"))
				roleName = (String)map.get(key);
			else if(name.equalsIgnoreCase("roleValue"))
				roleValue = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptName"))
				conceptName = (String)map.get(key);

		}
		inverseRole = getDLBAdapter(vocabularyName).containsRole(roleName, roleValue, conceptName);
	}catch(Exception e){
		log.error(e.getMessage());
		throw new DAOException(getException(e.getMessage()));
		}
	List booleanList = new ArrayList();
	booleanList.add(inverseRole);

   	return new Response(booleanList);

   	}


   /**
    * Fetches the properties of a given concept that belongs to the given namespace
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of properties
	* @throws Exception
    */
   private Response fetchDTSProperties(HashMap map) throws Exception
   {
   	String vocabularyName = null;
   	String term = null;
   	Vector ret_vec = new Vector();
   	List list = new ArrayList();
   	try
   	{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("term"))
   				term = (String)map.get(key);
   		}
   		ret_vec = getDLBAdapter(vocabularyName).getPropertiesByConceptName(term);
		if (ret_vec != null) {
			Vector evs_vec = convertPropertyCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
   	}
   	catch(Exception e)
   	{
   		log.error(e.getMessage());
   		throw new DAOException (getException(e.getMessage()));
   	}


   	return (new Response(list));
   }


   /**
    * Fetches term association data for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */

   private Response fetchTermAssociations(HashMap map) throws Exception
   {
   	String vocabularyName = null;
   	String term = null;
   	Vector associations = new Vector();
   	List list = new ArrayList();
   	try
   	{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("term"))
   				term = (String)map.get(key);
   		}
        DLBAdapter adapter = getDLBAdapter(vocabularyName);
        String code = term;
        try{
            validateDLConceptCode(code, vocabularyName);
        }catch(Exception ex){
//            try{
//                if(validateConceptName(code, vocabularyName)){
//                    code = adapter.getConceptCodeByName(term);
//                }
//            }catch(Exception exc){
                throw new DAOException("Invalid concept identifier "+ code);
//            }
        }
        associations = adapter.fetchTermAssociations(code);
        for(int i=0; i<associations.size(); i++)
   	   	{
   	   		list.add(associations.get(i));
   	   	}
   	}
   	catch(Exception e)
   	{
   		log.error(e.getMessage());
   		throw new DAOException(getException(e.getMessage()));
   	}


   	return (new Response(list));
   }


   /**
    * Gets all association types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */

   public Response getAllAssociationTypes(HashMap map) throws Exception{

   	String vocabularyName = null;
   	Vector associations = new Vector();
   	List associationList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);

   		}
   		//associations = getDLBAdapter(vocabularyName).getAllAssociationTypes();
   		associations = getDLBAdapter(vocabularyName).getSupportedConceptAssociationNames();

   		for(int i=0 ;i<associations.size(); i++)
   		{
   			associationList.add((String)associations.get(i));
   		}
   		}catch(Exception e){
   			log.error(e.getMessage());
   			throw new DAOException(getException( e.getMessage()));
		}

	return new Response(associationList);
   	}




   /**
    * Gets all concept association qualifier types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllConceptAssociationQualifierTypes(HashMap map)throws Exception{

   	String vocabularyName = null;
   	Vector types = new Vector();
   	List typeList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);

   		}
   		types = getDLBAdapter(vocabularyName).getAllConceptAssociationQualifierTypes();
   		for(int i=0 ;i<types.size(); i++)
   		{
   			typeList.add(types.get(i));
   		}
	}catch(Exception e){
		log.error(e.getMessage());
		throw new DAOException(getException(e.getMessage()));
	}

	return new Response(typeList);
   }



   /**
    * Gets all concept association types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllConceptAssociationTypes(HashMap map)throws Exception{
      	String vocabularyName = null;
      	Vector types = new Vector();
      	List typeList = new ArrayList();
      	try{
      		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
      		{
      			String key = (String)iter.next();
      			String name = key.substring(key.indexOf("$")+1, key.length());

      			if(name.equalsIgnoreCase("vocabularyName"))
      				vocabularyName = (String)map.get(key);

      		}
//      		types = getDLBAdapter(vocabularyName).getAllConceptAssociationTypes();
      		types = getDLBAdapter(vocabularyName).getConceptAssociationTypes();

      		for(int i=0 ;i<types.size(); i++)
      		{
      			typeList.add(types.get(i));
      		}
      		}catch(Exception e){
      			log.error(e.getMessage());
      			throw new DAOException(getException( e.getMessage()));
   		}

   	return new Response(typeList);
      }



   /**
    * Gets all concept property qualifier types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllConceptPropertyQualifierTypes(HashMap map)throws Exception{
     	String vocabularyName = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);

     		}
     		types = getDLBAdapter(vocabularyName).getAllConceptPropertyQualifierTypes();
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException( e.getMessage()));
  		}

  	return new Response(typeList);
     }



   /**
    * Gets all concept property types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllConceptPropertyTypes(HashMap map)throws Exception{

     	String vocabularyName = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);

     		}
     		types = getDLBAdapter(vocabularyName).getAllConceptPropertyTypes();
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException(e.getMessage()));
  		}

  	return new Response(typeList);
     }



   /**
    * Gets all licenses for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllLicenses(HashMap map)throws Exception{
     	String vocabularyName = null;
     	String condition = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);
     			if(name.equalsIgnoreCase("condition"))
     				condition = (String)map.get(key);

     		}
     		types = getDLBAdapter(vocabularyName).getAllLicenses();
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException( e.getMessage()));
  		}

  	return new Response(typeList);
     }



   /**
    * Gets all qualifier types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllQualifierTypes(HashMap map)throws Exception{
     	String vocabularyName = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);


     		}
     		types = getDLBAdapter(vocabularyName).getAllQualifierTypes();
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
		}catch(Exception e){
			log.error(e.getMessage());
			throw new DAOException(getException(e.getMessage()));
  		}

  	    return new Response(typeList);
     }




   /**
    * Gets all the Role names for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllRoleNames(HashMap map)throws Exception{
     	String vocabularyName = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);


     		}
     		types = getDLBAdapter(vocabularyName).getAllRoleNames();
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException(e.getMessage()));
  		}

  	return new Response(typeList);
     }

   /**
    * Gets the all silos.
    *
    * @param map the map
    *
    * @return the silos
    *
    * @throws Exception the exception
    * @deprecated - Silos not supported in evsapi4
    */@Deprecated
   public Response getAllSilos(HashMap map)throws Exception{
	   List siloList = new ArrayList();
	   return new Response(siloList);
   }


   /**
    * Gets all sub-concept codes for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllSubConceptCodes(HashMap map)throws Exception{
     	String vocabularyName = null;
     	String conceptCode = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);
     			if(name.equalsIgnoreCase("conceptCode"))
     				conceptCode = (String)map.get(key);


     		}
     		types = getDLBAdapter(vocabularyName).getSubConcepts(conceptCode, Boolean.TRUE, Boolean.TRUE);
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add(types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException( e.getMessage()));
  		}

  	return new Response(typeList);
     }




   /**
    * Gets all sub-concept names for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
	*
	* @deprecated - evsapi4 does not support names as an identifier. Use getAllSubConceptCodes(HashMap)
    */@Deprecated
   public Response getAllSubConceptNames(HashMap map)throws Exception{
     	String vocabularyName = null;
     	String conceptName = null;
     	Vector types = new Vector();
     	List typeList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);
     			if(name.equalsIgnoreCase("conceptName"))
     				conceptName = (String)map.get(key);


     		}
     		types = getDLBAdapter(vocabularyName).getAllSubConceptNames(conceptName);
     		for(int i=0 ;i<types.size(); i++)
     		{
     			typeList.add((String)types.get(i));
     		}
     		}catch(Exception e){
     			log.error(e.getMessage());
     			throw new DAOException(getException( e.getMessage()));
  		}

  	    return new Response(typeList);
     }




   /**
    * Gets all synonym types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllSynonymTypes(HashMap map)throws Exception{
	        	String vocabularyName = null;
	        	Vector types = new Vector();
	        	List typeList = new ArrayList();
	        	try{
	        		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
	        		{
	        			String key = (String)iter.next();
	        			String name = key.substring(key.indexOf("$")+1, key.length());

	        			if(name.equalsIgnoreCase("vocabularyName"))
	        				vocabularyName = (String)map.get(key);

	        		}
	        		// no longer supported
	        		/*
	        		types = getDLBAdapter(vocabularyName).getAllSynonymTypes();
	        		for(int i=0 ;i<types.size(); i++)
	        		{
	        			typeList.add(types.get(i));
	        		}
	        		*/
	        	}catch(Exception e){
	        			log.error(e.getMessage());
	        			throw new DAOException(getException(e.getMessage()));
	     		}

	     	return new Response(typeList);
   }



   /**
    * Gets all synonym types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
/*
   public Response getAllSilos(HashMap map)throws Exception{

	        	String vocabularyName = null;
	        	List siloList = new ArrayList();
	        	try{
	        		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
	        		{
	        			String key = (String)iter.next();
	        			String name = key.substring(key.indexOf("$")+1, key.length());

	        			if(name.equalsIgnoreCase("vocabularyName"))
	        				vocabularyName = (String)map.get(key);

	        		}
	        		setVocabulary(vocabularyName);
	        		gov.nih.nci.dtsrpc.client.Silo[] silos = dtsrpc.getAllSilos();
	        		for(int i=0 ;i<silos.length; i++)
	        		{
	        			siloList.add(convertSilo(silos[i]));
	        		}
	        		}catch(Exception e){
	        			log.error(e.getMessage());
	        			throw new DAOException(getException(e.getMessage()));
	     		}

	     	return new Response(siloList);
   }
   */

   /*
   //private gov.nih.nci.evs.domain.Silo convertSilo(gov.nih.nci.dtsrpc.client.Silo dtsSilo){
   private gov.nih.nci.evs.domain.Silo convertSilo(gov.nih.nci.lexbig.ext.Silo dtsSilo){
       gov.nih.nci.evs.domain.Silo silo = new gov.nih.nci.evs.domain.Silo();
       silo.setId(dtsSilo.getId());
       silo.setName(dtsSilo.getName());
       //log.info("Silo name = "+ silo.getName() +"\t"+ silo.getId());
       return silo;
       }

    */

   /**
    * Gets all term association qualifier types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllTermAssociationQualifierTypes(HashMap map)throws Exception{
	        	String vocabularyName = null;
	        	Vector types = new Vector();
	        	List typeList = new ArrayList();
	        	try{
	        		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
	        		{
	        			String key = (String)iter.next();
	        			String name = key.substring(key.indexOf("$")+1, key.length());

	        			if(name.equalsIgnoreCase("vocabularyName"))
	        				vocabularyName = (String)map.get(key);

	        		}
	        		types = getDLBAdapter(vocabularyName).getAllConceptAssociationQualifierTypes();
	        		for(int i=0 ;i<types.size(); i++)
	        		{
	        			typeList.add(types.get(i));
	        		}
	        		}catch(Exception e){
	        			log.error(e.getMessage());
	        			throw new DAOException(getException( e.getMessage()));
	     		}

	     	return new Response(typeList);
   }




   /**
    * Gets all term property qualifier types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllTermPropertyQualifierTypes(HashMap map)throws Exception{
   	String vocabularyName = null;
   	Vector types = new Vector();
   	List typeList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);

   		}
   		types = getDLBAdapter(vocabularyName).getAllQualifierTypes();
   		for(int i=0 ;i<types.size(); i++)
   		{
   			typeList.add(types.get(i));
   		}
   		}catch(Exception e){
   			log.error(e.getMessage());
   			throw new DAOException(getException( e.getMessage()));
		}

	return new Response(typeList);
}



   /**
    * Gets all term property types for the specified concept from a given namespace
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   public Response getAllTermPropertyTypes(HashMap map)throws Exception{
   	String vocabularyName = null;
   	Vector types = new Vector();
   	List typeList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);

   		}

   		//types = adapter.getAllPropertyTypes();
   		//gov.nih.nci.lexrpx.client.PropertyType[] properties = adapter.getAllPropertyTypes();
   		PropertyType[] lproperties = getDLBAdapter(vocabularyName).getAllPropertyTypes();

   		for(int i=0 ;i< lproperties.length; i++)
   		{
   			gov.nih.nci.evs.domain.Property pro = new gov.nih.nci.evs.domain.Property();
   			pro.setName(lproperties[i].getName());
   			//pro.setValue(properties[i].getValue());
   			typeList.add(pro);
   		}
   		}catch(Exception e){
   			log.error(e.getMessage());
   		throw new DAOException(getException( e.getMessage()));
		}

	return new Response(typeList);
}



   /**
    * Convert dtsrpc Association's to ConceptAssociation's.
    *
    * @param dtsrpcAssociations the dtsrpc Associations Vector
    *
    * @return the ConceptAssociation vector
    */
   public Vector convertAssociations(Vector dtsrpcAssociations){
       Vector associationVector = new Vector();
       for(int i=0; i<dtsrpcAssociations.size(); i++){
           gov.nih.nci.evs.domain.Association association = new Association();
           ConceptAssociation dtsAssociation = (ConceptAssociation)dtsrpcAssociations.get(i);
           association.setName(dtsAssociation.getName());
           association.setValue(dtsAssociation.getValue());
           //log.info("Number of qualifiers = "+ dtsAssociation.getQualifiers().size());
           association.setQualifierCollection(convertQualifiers(dtsAssociation.getQualifiers()));
           associationVector.add(association);
           }
       return associationVector;
       }


   /**
    * Convert a TreeNode object to an EdgeProperties object
    *
    * @param treenode
    *
    * @return the EdgeProperties object
    */
   public EdgeProperties convertEdgeProperties(gov.nih.nci.lexbig.ext.TreeNode treenode){
       EdgeProperties edgeProperties = new EdgeProperties();
       //edgeProperties.setName(treenode.getName());
       edgeProperties.setIsA(treenode.getIsA());
       edgeProperties.setLinks(treenode.getLinks());
       edgeProperties.setTraverseDown(treenode.getTraverseDown());
       return edgeProperties;
       }


   /**
    * Retrievs all the parent concepts for a given concept
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of DescLogicConcepts
	* @throws Exception
    */

   private Response getParentConcepts(HashMap map)throws Exception{

   	String vocabularyName = null;
   	String conceptName = null;
   	boolean inputFlag = false;
   	int asdIndex = 1;

   	List typeList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("conceptName"))
 				conceptName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("inputFlag"))
				inputFlag = ((Boolean)map.get(key)).booleanValue();
   			else if(name.equalsIgnoreCase("ASDIndex"))
				asdIndex = ((Integer)map.get(key)).intValue();

   		}

   		/*
   		Vector conceptNames = new Vector();
   		CodedEntry[] superConcepts =  null;
   		if(!inputFlag){
   		   	if(validateConceptName(conceptName, vocabularyName)){
   		   		superConcepts = getDLBAdapter(vocabularyName).getDirectSups(conceptName, false, asdIndex);
   		   	    }
   		   	}
   		else{
   		    if(validateDLConceptCode(conceptName, vocabularyName)){
   		    	superConcepts = getDLBAdapter(vocabularyName).getDirectSups(conceptName, false, asdIndex);
		   	    }
		   	}
        */

        DLBAdapter adapter = getDLBAdapter(vocabularyName);
		String code = conceptName;
		if (!inputFlag) {
		    code = adapter.getConceptCodeByName(conceptName);
		}

		validateDLConceptCode(code, vocabularyName);
		//boolean direction = false;
		//Vector v = adapter.getRelatedConcepts(code, direction, DLBAdapter.DEFAULT_ASSOC_FWD, null, 0);
		Vector v = adapter.getParentConcepts(code, null, 0);

		CodedEntry[] concepts = new CodedEntry[v.size()];
		for (int k=0; k<v.size(); k++)
		{
			concepts[k] = (CodedEntry) v.elementAt(k);
		}

		for(int i=0; i<concepts.length; i++) {
			DescLogicConcept dlc = new DescLogicConcept();
			//dlc = buildDescLogicConcept(concepts[i]);
			dlc = buildDescLogicConcept(concepts[i], asdIndex, vocabularyName);
			typeList.add(dlc);
		}


	}catch(Exception e){
		log.error(e.getMessage());
	 throw new DAOException(getException(e.getMessage()));
	}

	return new Response(typeList);
}




   /**
	* Retrievs all the child DescLogicConcepts for a given conceptName
    * @param map Specifies the input parameters
	* @return Returns a response that holds a list of DescLogicConcepts
	* @throws Exception
    */

   private Response getChildConcepts(HashMap map)throws Exception{
   	String vocabularyName = null;
   	String conceptName = null;
   	boolean inputFlag = false;
   	int asdIndex = 1;

   	List typeList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("conceptName"))
 				conceptName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("inputFlag"))
				inputFlag = ((Boolean)map.get(key)).booleanValue();
   			else if(name.equalsIgnoreCase("ASDIndex"))
				asdIndex = ((Integer)map.get(key)).intValue();

   		}
   		CodedEntry[] concepts = null;
        DLBAdapter adapter = getDLBAdapter(vocabularyName);
        /*
		if(!inputFlag){
			validateConceptName(conceptName, vocabularyName);
			concepts = adapter.getDirectSubs(conceptName,false, asdIndex);
		}
		else{
			validateDLConceptCode(conceptName, vocabularyName);
			concepts = adapter.getDirectSubs(conceptName,true, asdIndex);
		}
		*/

		String code = conceptName;
		if (!inputFlag) {
		    code = adapter.getConceptCodeByName(conceptName);
		}
		validateDLConceptCode(code, vocabularyName);

		//boolean direction = true;
		//Vector v = adapter.getRelatedConcepts(code, direction, DLBAdapter.DEFAULT_ASSOC_FWD, null, 0);
		Vector v = adapter.getChildConcepts(code, null, 0);

		concepts = new CodedEntry[v.size()];
		for (int k=0; k<v.size(); k++)
		{
			concepts[k] = (CodedEntry) v.elementAt(k);
		}

		for(int i=0; i<concepts.length; i++) {
			DescLogicConcept dlc = new DescLogicConcept();
			//dlc = buildDescLogicConcept(concepts[i]);
			dlc = buildDescLogicConcept(concepts[i], asdIndex, vocabularyName);
			typeList.add(dlc);
		}

	}catch(Exception e){
		log.error(e.getMessage());
		throw new DAOException(getException(e.getMessage()));
	}

	return new Response(typeList);
}


   /**
    * Returns a Boolean value if the specified concept has a child concept
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
   	private Response hasChildren(HashMap map)throws Exception{
   	   	String vocabularyName = null;
   	   	String conceptName = null;
   	   	List hasList = new ArrayList();

   	   	Boolean found = new Boolean(false);
   	   	try{
   	   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   	   		{
   	   			String key = (String)iter.next();
   	   			String name = key.substring(key.indexOf("$")+1, key.length());

   	   			if(name.equalsIgnoreCase("vocabularyName"))
   	   				vocabularyName = (String)map.get(key);
   	   			else if(name.equalsIgnoreCase("conceptName"))
   	 				conceptName = (String)map.get(key);

   	   		}
   	   		if(validateConceptName(conceptName, vocabularyName)){
   	   			found  = getDLBAdapter(vocabularyName).hasChildren(conceptName);
   	   			hasList.add(found);
   	   		}
   	   		}catch(Exception e){
   	   		log.error(e.getMessage());
   	   		throw new DAOException(getException(e.getMessage()));
   			}

   		return new Response(hasList);

   	}


    /**
     * Returns a Boolean value if the given concept has a Parent concept
     * @param map Specifies the input parameters
 	 * @return Returns a response
 	 * @throws Exception
     */
   	private Response hasParents(HashMap map)throws Exception{
   	   	String vocabularyName = null;
   	   	String conceptName = null;
   	   	List hasList = new ArrayList();

   	   	Boolean found = new Boolean(false);
   	   	try{
   	   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   	   		{
   	   			String key = (String)iter.next();
   	   			String name = key.substring(key.indexOf("$")+1, key.length());

   	   			if(name.equalsIgnoreCase("vocabularyName"))
   	   				vocabularyName = (String)map.get(key);
   	   			else if(name.equalsIgnoreCase("conceptName"))
   	 				conceptName = (String)map.get(key);

   	   		}
   	   		if(validateConceptName(conceptName, vocabularyName)){
   	   			found  = getDLBAdapter(vocabularyName).hasParents(conceptName);
   	   			hasList.add(found);
   	   		}
   	   		}catch(Exception e){
   	   		log.error(e.getMessage());
   	   			throw new DAOException(getException(e.getMessage()));
   			}

   		return new Response(hasList);

   	}


    /**
     * Checks if the concept name is valid
     * @param conceptName
 	 * @return boolean value
 	 * @throws Exception
 	 *
 	 * @deprecated - evsapi4 does not support name as an identifier
     */@Deprecated
   private boolean validateConceptName (String conceptName, String vocabularyName)throws Exception{

   	boolean valid = true;

   	try{
   	if(conceptName == null){
   		valid = false;
   		throw new DAOException(" Concept name cannot be a null value");
   			}
   	else if(getDLBAdapter(vocabularyName).getConceptCodeByName(conceptName) == null){
   			valid = false;
   			throw new DAOException(" Invalid concept name - "+conceptName);
   			}
	}
   	catch(Exception e){
   		throw new DAOException(e.getMessage());
   	}
   		return valid;
   	}




   /**
    * Checks if the relationship type is valid
    * @param relation
	* @return boolean value
	* @throws Exception
    */
   private boolean validateRelation(String relation) throws Exception{
   	boolean valid = false;
   	String[] relationType = {"RB","RN","RO","RL","PAR","CHD","SIB","AQ","QB"};

   	for(int i=0; i<relationType.length && !valid; i++){
   		if(relation.equalsIgnoreCase(relationType[i])){
   			valid = true;
   			}
   		}
   	if(!valid){
   		throw new DAOException(" Invalid relation attribute "+ relation);
   		}
   	return valid;

   	}

   /**
    * Gets the concept name for the specified code
    * @param conceptCode - specifies the concept code
    * @return Returns the concept name
    */

   private Response getConceptNameByCode(HashMap map)throws Exception{
   	String vocabularyName = null;
	String conceptName = null;
	String conceptCode = null;
	List list = new ArrayList();

	try
	{
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

			if(name.equalsIgnoreCase("vocabularyName"))
				vocabularyName = (String)map.get(key);
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String)map.get(key);

		}
		if(conceptCode == null){
			log.error(" Concept code cannot be a null value");
			throw new DAOException (" Invalid concept code");
			}
		conceptName = getDLBAdapter(vocabularyName).getConceptNameByCode(conceptCode);
		list.add(conceptName);
	}
	catch(Exception e)
	{
		log.error("Exception - getConceptNameByCode "+e.getMessage());
		throw new DAOException (getException( e.getMessage()));
	}
    return (new Response(list));
   	}

   /**
    * Gets the dlc semantic types.
    *
    * @param dlc the Description Logic Concept
    *
    * @return the dlc's semantic types
    */
   public Vector getDLSemanticTypes(DescLogicConcept dlc){
   	Vector semanticTypes = new Vector();
   	Vector lproperties = new Vector();
   	if(dlc.getSemanticTypeVector() == null){
   		lproperties = dlc.getPropertyCollection();
   	   	for(int i=0; i< lproperties.size(); i++){
   	   		gov.nih.nci.evs.domain.Property p = (gov.nih.nci.evs.domain.Property)lproperties.get(i);
   	   		if(p.getName().equalsIgnoreCase("Semantic_Type")){
   	   			semanticTypes.add(p.getValue());
   	   			}
   	   		}
   	}
   	return semanticTypes;
   	}

   /**
    * Gets all property types.
    *
    * @param map the map
    *
    * @return the list of property types as a Response object
    *
    * @throws Exception the exception
    */
   public Response getAllPropertyTypes(HashMap map)throws Exception{

   	String vocabularyName = null;

   	List propertyList = new ArrayList();
   	try{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);

   		}
   		PropertyType[] propertyTypes = getDLBAdapter(vocabularyName).getAllPropertyTypes();
   		for(int i=0 ;i<propertyTypes.length; i++)
   		{
   			propertyList.add(propertyTypes[i].getName());
   		}
   		}catch(Exception e){
   			log.error(e.getMessage());
   			throw new DAOException(getException( e.getMessage()));
		}

	return new Response(propertyList);

   	}
   private Exception getException(String msg){
       DAOException ex = null;
       if(threadLocal.get() != null){
           ex = (DAOException)threadLocal.get();
       }else if(msg != null){
           ex = new DAOException(msg);
           threadLocal.set(ex);
       }
   	return ex;
   	}

   private Exception getException(Exception ex){
   	return getException(ex.getMessage());
   	}

   private boolean validateMetaConceptCode(String code, DLBAdapter adapter)throws Exception {
       boolean valid = false;
       try{
           if(adapter.getConceptNameByCode(code)== null){
               throw new DAOException("Invalid concept code "+ code);
           }else{
               valid = true;
           }
       }catch(Exception ex){
           throw new DAOException(ex.getMessage());
       }
       return valid;
   }

   private boolean validateDLConceptCode(String code, String vocabularyName) throws Exception{

	boolean valid = true;

   	try{
   	if(code == null){
   		valid = false;
   		throw new DAOException (" Concept code cannot be a null value");
   			}
   	else if(getDLBAdapter(vocabularyName).getConceptNameByCode(code) == null){
   			valid = false;
   			throw new DAOException (" Invalid concept code - "+ code);
   			}
	}
   	catch(Exception e){
   		throw new DAOException(e.getMessage());
   	}
   		return valid;
   	}

   /**
 * Gets the history records by concept code.
 *
 * @param map the list of codes
 *
 * @return the list of history records as a response object
 *
 * @throws Exception the exception
 * @deprecated - replaced by getHistoryRecords(HashMap)
 */@Deprecated
public Response getHistoryRecordbyCode(HashMap map) throws Exception{
       return getHistoryRecords(map);
   }

   /**
    * Gets the history records.
    *
    * @param map the list of codes
    *
    * @return the list of history records as a response object
    *
    * @throws Exception the exception
    */
   public Response getHistoryRecords(HashMap map) throws Exception{
        String vocabularyName = null;
        String conceptCode = null;
        Date initialDate = null;
        Date finalDate = null;
        int namespaceId = -1;
        dateFormat.applyPattern("MM/dd/yyyy");
        Vector v = new Vector();
        List<gov.nih.nci.evs.domain.HistoryRecord> list = new ArrayList<gov.nih.nci.evs.domain.HistoryRecord>();

        try
        {
            for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
            {
                String key = (String)iter.next();
                String name = key.substring(key.indexOf("$")+1, key.length());

                if(name.equalsIgnoreCase("VocabularyName"))
                    vocabularyName = (String)map.get(key);
                else if(name.equalsIgnoreCase("conceptCode"))
                    conceptCode = (String)map.get(key);
                else if(name.equalsIgnoreCase("initialDate"))
//                    initialDate = (Date)map.get(key);
                  initialDate = dateFormat.parse((String)map.get(key));
                else if(name.equalsIgnoreCase("finalDate"))
//                    finalDate = (Date)map.get(key);
                	finalDate = dateFormat.parse((String)map.get(key));
                else if(name.equalsIgnoreCase("namespaceId"))
                    namespaceId = ((Integer)map.get(key)).intValue();
             }
            if(vocabularyName == null ){
                if(namespaceId >-1){
                    vocabularyName = namespaceId2VocabularyName(namespaceId);
                }
            }
            if(conceptCode != null){
                validateDLConceptCode(conceptCode, vocabularyName);
            }
            DLBAdapter adapter = getDLBAdapter(vocabularyName);
            if(conceptCode == null){
            	v = adapter.getHistoryDates();
            }
            else{
                v = adapter.getConceptEditAction(conceptCode);
                if(v!=null && v.size()>0){


                    gov.nih.nci.evs.domain.HistoryRecord hr = new gov.nih.nci.evs.domain.HistoryRecord();
                    hr.setDescLogicConceptCode(conceptCode);
                    Vector<gov.nih.nci.evs.domain.History> historyVector = new Vector<gov.nih.nci.evs.domain.History>();

                    for(int i=0; i<v.size(); i++)
                    {
                        gov.nih.nci.lexbig.ext.EditActionDate e = (gov.nih.nci.lexbig.ext.EditActionDate) v.elementAt(i);
                        Date aDate = e.getEditDate();
                        int action_int = e.getAction();
                        String action = gov.nih.nci.lexbig.ext.EditActionDate.actionToString(action_int);
                        gov.nih.nci.evs.domain.History h = new gov.nih.nci.evs.domain.History();
                        h.setEditAction(action);
                        h.setEditActionDate(aDate);
                        Vector<String> ref = new Vector<String>();

                        if(action.equalsIgnoreCase("retire") ||action.equalsIgnoreCase("merge")){
//                                ref = adapter.getCodeActionChildren(conceptCode, aDate);
                                ref = adapter.getDescendantCodes(conceptCode, aDate);
                        }else if(action.equalsIgnoreCase("split")){
//                            	ref = adapter.getCodeActionParents(conceptCode, aDate);
                            	ref = adapter.getAncestorCodes(conceptCode, aDate);
                        }
						String refCodes = "";
						for(int r=0; r<ref.size(); r++){
							refCodes += (String)ref.get(r);
						}
						if(refCodes.length()>0){
							h.setReferenceCode(refCodes);
						}
						if(initialDate != null && finalDate == null){
							if(initialDate.equals(aDate) || aDate.after(initialDate)){
								historyVector.add(h);
							}
						}else if(initialDate == null && finalDate != null){
							if(finalDate.equals(aDate) || aDate.before(finalDate)){
								historyVector.add(h);
							}
						}else if(initialDate != null && finalDate != null ){
							if(initialDate.equals(aDate) ||(aDate.after(initialDate) && aDate.before(finalDate)) || aDate.equals(finalDate)){
								historyVector.add(h);
							}
						}else if(initialDate == null && finalDate == null){
							historyVector.add(h);
						}
					//}
                        hr.setHistoryCollection(historyVector);
                    }
                    list.add(hr);
                }
            }



        }
        catch(Exception e){
            if(e != null){
                log.error("Error: getHistoryRecords - "+e.getMessage());
                throw new DAOException (getException( e.getMessage()));
            }
//            throw new DAOException(getException(e));
        }
 return new Response(list);

   }


   /**
    * Returns edit action string for a given code.
    *
    * @param code the code
    *
    * @return the edit action string
    *
    * @throws Exception     */
   private String getEditActionString(int code){
       String action = null;
       if(code == 0){
           action = "create";
       }else if(code == 1){
           action = "merge";
       }else if(code == 2){
           action = "modify";
       }else if(code == 3){
           action = "retire";
       }else if(code == 4){
           action = "split";
       }
       return action;
   }
   private Response getHistoryStartDate(HashMap map) throws Exception{
	   String vocabularyName = null;
	   	List dateList = new ArrayList();
	   	try{
	   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
	   		{
	   			String key = (String)iter.next();
	   			String name = key.substring(key.indexOf("$")+1, key.length());

	   			if(name.equalsIgnoreCase("vocabularyName"))
	   				vocabularyName = (String)map.get(key);

	   		}
	   		Date startDate = getDLBAdapter(vocabularyName).getHistoryStartDate();
	   		dateList.add(startDate);
	   		}catch(Exception e){
	   			log.error(e.getMessage());
	   			throw new DAOException(getException( e.getMessage()));
			}

		return new Response(dateList);
   }

   private Response getHistoryEndDate(HashMap map)throws Exception{
	   String vocabularyName = null;
	   	List dateList = new ArrayList();
	   	try{
	   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
	   		{
	   			String key = (String)iter.next();
	   			String name = key.substring(key.indexOf("$")+1, key.length());

	   			if(name.equalsIgnoreCase("vocabularyName"))
	   				vocabularyName = (String)map.get(key);
	   		}
	   		Date endDate = getDLBAdapter(vocabularyName).getHistoryEndDate();
	   		dateList.add(endDate);
	   		}catch(Exception e){
	   			log.error(e.getMessage());
	   			throw new DAOException(getException( e.getMessage()));
			}
		return new Response(dateList);
   }
   private Response getHistoryDates(HashMap map) throws Exception {
       List dateList = new ArrayList();
       String vocabularyName = null;
       try{
            for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
            {
                String key = (String)iter.next();
                String name = key.substring(key.indexOf("$")+1, key.length());

                if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
            }
            Vector dates = getDLBAdapter(vocabularyName).getHistoryDates();
            for(int i=0; i< dates.size(); i++){
                dateList.add(dates.get(i));
            }
        }catch(Exception e){
                log.error(e.getMessage());
                throw new DAOException(getException( e.getMessage()));
        }
       return new Response(dateList);
   }

   private Response getHistoryEndBaseLineDate(HashMap map) throws Exception {
       List dateList = new ArrayList();
       String vocabularyName = null;
       try{
           for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
           {
               String key = (String)iter.next();
               String name = key.substring(key.indexOf("$")+1, key.length());

               if(name.equalsIgnoreCase("vocabularyName"))
                   vocabularyName = (String)map.get(key);
           }
//           Date date = getDLBAdapter(vocabularyName).getHistoryEndBaselineDate();
           Date date = getDLBAdapter(vocabularyName).getHistoryEndDate();
           if(date!=null){
               dateList.add(date);
           }
       }catch(Exception e){
               log.error(e.getMessage());
               throw new DAOException(getException( e.getMessage()));
       }

       return new Response(dateList);
   }
   private Date stringToDate(String aString) throws Exception
   {
      Date theDate = null;
       try
       {
           SimpleDateFormat sdf = null;
           if(aString.indexOf("-")>3){
               sdf = new SimpleDateFormat("yyyy-MM-dd");
           }
           else if(aString.indexOf("/")>3){
               sdf = new SimpleDateFormat("yyyy/MM/dd");
           }
           else{
               sdf = new SimpleDateFormat("MM/dd/yyyy");
           }

           theDate = sdf.parse(aString);
       }
       catch(Exception e)
       {
           log.error("Exception: " + e.getMessage());
           throw new Exception(getException(e.getMessage()));
       }
       return theDate;

   }
   /**
    * Gets all child concept codes for the specified concept based on the action.
    * @param map - Specifies the input parameters
    * @return - Returns a response that holds a list of concept codes
    * @throws Exception
    */
   private Response getCodeActionChildren(HashMap map) throws Exception
   {
   	String vocabularyName = null;
   	String conceptCode = null;
   	String action = null;
    Date baseLineDate = null;
    Vector children = new Vector();
   	ArrayList list = new ArrayList();
   	try
   	{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("conceptCode"))
   				conceptCode = (String)map.get(key);
            else if(name.equalsIgnoreCase("baseLineDate"))
                baseLineDate = stringToDate((String)map.get(key));
            else if(name.equalsIgnoreCase("action"))
                action = (String)map.get(key);
   		}
   		if(action == null){
   //			children = getDLBAdapter(vocabularyName).getCodeActionChildren(conceptCode, baseLineDate);
   			children = getDLBAdapter(vocabularyName).getDescendantCodes(conceptCode, baseLineDate);
   		}

   		for(int i=0; i<children.size(); i++)
			{
				list.add(children.get(i));
			}
   	}
   	catch(Exception e)
   	{
   		log.error(e.getMessage());
   		throw new DAOException (getException( e.getMessage()));
   	}
   	return (new Response(list));

   }

   /**
    * Gets all parent concept codes for the specified concept.
    * @param map - Specifies the input parameters
    * @return - Returns a response that holds a list of concept codes
    * @throws Exception
    */
   private Response getCodeActionParents(HashMap map) throws Exception
   {
   	String vocabularyName = null;
   	String conceptCode = null;
    Date baseLineDate = null;
    Vector parents = new Vector();
   	ArrayList list = new ArrayList();
   	try
   	{
   		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
   		{
   			String key = (String)iter.next();
   			String name = key.substring(key.indexOf("$")+1, key.length());

   			if(name.equalsIgnoreCase("vocabularyName"))
   				vocabularyName = (String)map.get(key);
   			else if(name.equalsIgnoreCase("conceptCode"))
   				conceptCode = (String)map.get(key);
            else if(name.equalsIgnoreCase("baseLineDate"))
                baseLineDate = stringToDate((String)map.get(key));
   		}
//   		parents = getDLBAdapter(vocabularyName).getCodeActionParents(conceptCode, baseLineDate);
   		parents = getDLBAdapter(vocabularyName).getAncestorCodes(conceptCode, baseLineDate);
   		for(int i=0; i<parents.size(); i++)
			{
				list.add(parents.get(i));
			}
   	}
   	catch(Exception e)
   	{
   		log.error(e.getMessage());
   		throw new DAOException (getException( e.getMessage()));
   	}
   	return (new Response(list));
   }





private Response getDefinitionCollectionbyCui(HashMap map) throws Exception
{
	String cui = null;
	ArrayList list = new ArrayList();
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("cui"))
				cui = (String) map.get(key);
		}
		DLBAdapter adapter = getDLBAdapterForMeta();
		Vector ret_obj = adapter.getDefinitionCollectionbyCode(metaVocabularyName, cui);
		if (ret_obj != null) {
			Vector evs_obj = convertDefinitionCollection(ret_obj);
			for (int i=0; i<evs_obj.size(); i++)
			{
				list.add(evs_obj.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}

private Response getSourceCollectionbyCui (HashMap map) throws Exception
{
	String cui = null;
	ArrayList list = new ArrayList();
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("cui"))
				cui = (String) map.get(key);
		}
		DLBAdapter adapter = getDLBAdapterForMeta();
		//Vector ret_obj = adapter.getSourceCollectionbyCode(metaVocabularyName, cui);
		Vector ret_obj = adapter.getAtomCollectionbyCode(metaVocabularyName, cui);
		Vector source_vec = new Vector();
		for (int i=0; i<ret_obj.size(); i++)
		{
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) ret_obj.elementAt(i);
			Vector u = property.getQualifiers();
			if (u != null)
			{
				for (int j=0; j<u.size(); j++)
				{
					gov.nih.nci.lexbig.ext.Qualifier qualifier = (gov.nih.nci.lexbig.ext.Qualifier) u.elementAt(j);
					if (qualifier.getName().equalsIgnoreCase("SOURCE"))
					{
						source_vec.add(qualifier.getValue());
					}
				}
		    }
		}

		if (ret_obj != null) {
			//Vector evs_obj = convertSourceCollection(ret_obj);
			Vector evs_obj = convertSourceCollection(source_vec);
			for (int i=0; i<evs_obj.size(); i++)
			{
				list.add(evs_obj.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getSemanticTypeVectorbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String code = null;
	ArrayList list = new ArrayList();
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("code"))
				code = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		Vector ret_obj = adapter.getSemanticTypeCollectionbyCode(vocabularyName, code);

		if (ret_obj != null) {
			Vector evs_obj = convertSemanticTypeCollection(ret_obj);
			for (int i=0; i<evs_obj.size(); i++)
			{
				list.add(evs_obj.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getSemanticTypeCollectionbyCui (HashMap map) throws Exception
{
	String cui = null;
	ArrayList list = new ArrayList();
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("cui"))
            {
				cui = (String) map.get(key);
			}
		}

		DLBAdapter adapter = getDLBAdapterForMeta();
		Vector ret_obj = adapter.getSemanticTypeCollectionbyCode(metaVocabularyName, cui);

		if (ret_obj != null) {
			Vector evs_obj = convertSemanticTypeCollection(ret_obj);
			for (int i=0; i<evs_obj.size(); i++)
			{
				list.add(evs_obj.elementAt(i));
			}
		}

	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getSynonymCollectionbyCui (HashMap map) throws Exception
{
	//return getAtomCollectionbyCui (map);
	String cui = null;
	ArrayList list = new ArrayList();
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("cui"))
				cui = (String) map.get(key);
		}
		DLBAdapter adapter = getDLBAdapterForMeta();
		Vector ret_obj = adapter.getAtomCollectionbyCode(metaVocabularyName, cui);

		if (ret_obj != null) {
			Vector evs_obj = convertAtomCollection(ret_obj);
			for (int i=0; i<evs_obj.size(); i++)
			{
				Atom atom = (Atom) evs_obj.elementAt(i);
				list.add(atom.getName());
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getAtomCollectionbyCui (HashMap map) throws Exception
{
	String cui = null;
	ArrayList list = new ArrayList();
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("cui"))
				cui = (String) map.get(key);
		}
		DLBAdapter adapter = getDLBAdapterForMeta();
		Vector ret_obj = adapter.getAtomCollectionbyCode(metaVocabularyName, cui);

		if (ret_obj != null) {
			Vector evs_obj = convertAtomCollection(ret_obj);
			for (int i=0; i<evs_obj.size(); i++)
			{
				//list.add(evs_obj.elementAt(i));
				Atom atom = (Atom) evs_obj.elementAt(i);
				//list.add(atom.getName());
				//GF#11889
				list.add(atom);
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}

private Response getHistoryCollectionbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ret_vec = adapter.getHistoryCollectionbyCode(vocabularyName, conceptCode);
		if (ret_vec != null) {
			evs_vec = convertHistoryCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getRoleCollectionbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;

	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ret_vec = adapter.getRoleCollectionbyCode(vocabularyName, conceptCode);
		if (ret_vec != null) {
			evs_vec = convertRoleCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getInverseRoleCollectionbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ret_vec = adapter.getInverseRoleCollectionbyCode(vocabularyName, conceptCode);
		if (ret_vec != null) {
			evs_vec = convertRoleCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}

private Response getInverseAssociationCollectionbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ret_vec = adapter.getInverseAssociationCollectionbyCode(vocabularyName, conceptCode);
		if (ret_vec != null) {
			// GForge 13294, KLO 032808
			//evs_vec = convertRoleCollection(ret_vec);
			evs_vec = convertAssociationCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}


private Response getAssociationCollectionbyCode (HashMap map) throws Exception
{
	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());
            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);
		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		ret_vec = adapter.getAssociationCollectionbyCode(vocabularyName, conceptCode);
		if (ret_vec != null) {
			evs_vec = convertAssociationCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}

private Response getPropertyCollectionbyCode (HashMap map) throws Exception
{

	int namespaceId = -1;
	String conceptCode = null;
	ArrayList list = new ArrayList();
	Vector ret_vec = null;
	Vector evs_vec = null;
    String vocabularyName = null;
	try {
		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
		{
			String key = (String)iter.next();
			String name = key.substring(key.indexOf("$")+1, key.length());

            if(name.equalsIgnoreCase("vocabularyName"))
                vocabularyName = (String)map.get(key);
            else if(name.equalsIgnoreCase("namespaceId"))
				namespaceId = ((Integer)map.get(key)).intValue();
			else if(name.equalsIgnoreCase("conceptCode"))
				conceptCode = (String) map.get(key);
		}
		//String vocabularyName = namespaceId2VocabularyName(namespaceId);

		DLBAdapter adapter = getDLBAdapter(vocabularyName);
		try{
            ret_vec = adapter.getPropertyCollectionbyCode(vocabularyName, conceptCode);
        }catch(Exception ex){
            log.error("Unable to query Properties for "+ conceptCode);
        }
		if (ret_vec != null) {
			evs_vec = convertPropertyCollection(ret_vec);
			for (int i=0; i<evs_vec.size(); i++)
			{
				list.add(evs_vec.elementAt(i));
			}
		}
	} catch(Exception e) {
		log.error(e.getMessage());
		throw new DAOException (getException(e.getMessage()));
	}
    return (new Response(list));
}






private Vocabulary convertVocabulary (String vocabulary) {
	Vocabulary vocab = new Vocabulary();
	vocab.setName(vocabulary);
	return vocab;

}


private Vector convertAssociationCollection (Vector dtsrpcAssociations) {
	return convertAssociations(dtsrpcAssociations);
}


private Vector convertPropertyCollection (Vector nciProps) {
	return convertProperties(nciProps);
}

/*
private Vector convertSiloCollection (Vector v) {
	Vector u = new Vector();
	for (int i=0; i<v.size(); i++)
	{
		gov.nih.nci.evs.domain.Silo silo = convertSilo((gov.nih.nci.lexbig.ext.Silo) v.elementAt(i));
		u.add(silo);
	}
    return u;
}
*/

private Vector convertQualifierCollection (Vector dtsrpcQualifiers) {
    return convertQualifiers(dtsrpcQualifiers);
}


private Vector convertDefinitionCollection (Vector v) {
	Vector u = new Vector();
	for (int i=0; i<v.size(); i++)
	{
		gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) v.elementAt(i);
		Definition definition = new Definition();
		definition.setDefinition(property.getValue());
		gov.nih.nci.evs.domain.Source source = new gov.nih.nci.evs.domain.Source();
		Vector qCollection = property.getQualifierCollection();
		for(int q=0; q< qCollection.size(); q++){
			gov.nih.nci.lexbig.ext.Qualifier qualifier = (gov.nih.nci.lexbig.ext.Qualifier)qCollection.get(q);
			if(qualifier.getName().toLowerCase().equalsIgnoreCase("source")){
				source.setAbbreviation(qualifier.getValue());
			}else if(qualifier.getName().toLowerCase().equalsIgnoreCase("source-code")){
				source.setCode(qualifier.getValue());
			}
		}
		definition.setSource(source);
		u.add(definition);
	}
	return u;
 }

/*
private Vector convertSourceCollection (Vector v) {
	Vector u = new Vector();
	for (int i=0; i<v.size(); i++)
	{
		gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) v.elementAt(i);
		if(property.getName().toUpperCase().equalsIgnoreCase("SOURCE")) {
			Source source = new Source();
			source.setAbbreviation(property.getValue());
			u.add(source);
	    }
	}
	return u;
}
*/

private Vector convertSourceCollection (Vector v) {
	Vector u = new Vector();
	for (int i=0; i<v.size(); i++)
	{
		String s = (String) v.elementAt(i);
		Source source = new Source();
		source.setAbbreviation(s);
		u.add(source);
	}
	return u;
}

private Vector convertSemanticTypeCollection (Vector v) {
	Vector u = new Vector();
	DLBAdapter adapter = new DLBAdapter();
	//Vector codes = adapter.getSemanticTypeCodes(v);

	//Vector name_vec = new Vector();

	for (int i=0; i<v.size(); i++)
	{
		gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) v.elementAt(i);
		if(property.getName().toUpperCase().equalsIgnoreCase("SEMANTIC_TYPE")) {
			SemanticType sType = new SemanticType();
			String semanticTypeCode = adapter.getSemanticTypeCode(property.getValue());
			sType.setName(property.getValue());
			sType.setId(semanticTypeCode);
			u.add(sType);
	    }
	}
	return u;
}


private Vector convertAtomCollection (Vector v) {
	Vector u = new Vector();
	for (int i=0; i<v.size(); i++)
	{
		gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) v.elementAt(i);
		//if(property.getName().toUpperCase().equalsIgnoreCase("FULL_SYN")) {
		gov.nih.nci.evs.domain.Atom atom = new gov.nih.nci.evs.domain.Atom();
		atom.setName(property.getValue());
		//atom.setCode(metaConcept.getCode());
		Vector qCollection = property.getQualifierCollection();
		gov.nih.nci.evs.domain.Source source = null;

		for(int q=0; q< qCollection.size(); q++){
			gov.nih.nci.lexbig.ext.Qualifier qualifier = (gov.nih.nci.lexbig.ext.Qualifier)qCollection.get(q);
			if(qualifier.getName().toLowerCase().equalsIgnoreCase("source")){
				source = new gov.nih.nci.evs.domain.Source();
				source.setAbbreviation(qualifier.getValue());
			}
			if(qualifier.getName().toLowerCase().equalsIgnoreCase("source-code")){
				atom.setCode(qualifier.getValue());
			}
		}

		if(source !=null) {
			atom.setSource(source);
			atom.setOrigin(source.getAbbreviation());
		}

		u.add(atom);
	    //}
	}
	return u;
}


/*
private Vector convertHistoryCollection (Vector v) {
	Vector historyVector = new Vector();
	for (int i=0; i<v.size(); i++) {
		String actionDate = (String) v.get(i);
		StringTokenizer st = new StringTokenizer(actionDate, "|");
		while(st.hasMoreTokens()){
			String action = st.nextToken();
			Date aDate = stringToDate(st.nextToken());
			gov.nih.nci.evs.domain.History h = new gov.nih.nci.evs.domain.History();
			h.setEditAction(action);
			h.setEditActionDate(aDate);
			Vector ref = new Vector();
			if(action.equalsIgnoreCase("retire") ||action.equalsIgnoreCase("merge")){
				ref = adapter.getCodeActionChildren(conceptCode, aDate);
			}else if(action.equalsIgnoreCase("split")){
				ref = adapter.getCodeActionParents(conceptCode, aDate);
			}
			String refCodes = "";
			for(int r=0; r<ref.size(); r++){
				refCodes += (String)ref.get(r);
			}
			if(refCodes.length()>0){
				h.setReferenceCode(refCodes);
			}

			if(initialDate != null && finalDate == null){
				if(initialDate.equals(aDate) || aDate.after(initialDate)){
					historyVector.add(h);
				}
			}else if(initialDate == null && finalDate != null){
				if(finalDate.equals(aDate) || aDate.before(finalDate)){
					historyVector.add(h);
				}
			}else if(initialDate != null && finalDate != null ){
				if(initialDate.equals(aDate) ||(aDate.after(initialDate) && aDate.before(finalDate)) || aDate.equals(finalDate)){
					historyVector.add(h);
				}
			}else if(initialDate == null && finalDate == null){
				historyVector.add(h);
			}

			historyVector.add(h);

		}
	}
	return historyVector;
}
*/

// Place holders
private Vector convertHistoryCollection (Vector v) {
	Vector historyVector = new Vector();
	return historyVector;
}


private SecurityToken convertSecurityToken (String token) {
	return new SecurityToken();
}


private Vector convertRoleCollection (Vector nciRoles) {
	return convertRoles(nciRoles);

}

/*
private EdgeProperties convertEdgeProperties (gov.nih.nci.lexbig.ext.TreeNode node) {
    return new EdgeProperties();
}
*/

private Source convertSource (String sab)  {
	try {
		HashMap map = new HashMap();
		List metaSources = (List)getMetaSources(map).getResponse();
		for(int i=0; i<metaSources.size(); i++) {
			Source source = (Source)metaSources.get(i);
			String sourceName = source.getAbbreviation();
			if(sourceName.equalsIgnoreCase(sab)) {
				return source;
			}
		}
    } catch (Exception e) {
		//e.printStackTrace();
		log.error(e.getMessage());
	}
	return null;
}


private String namespaceId2VocabularyName(int namespaceId)
{
	Integer id = new Integer(namespaceId);
	if (namespaceId2VocabularyName_Map.containsKey(id))
	{
		return (String) namespaceId2VocabularyName_Map.get(id);
	}
	try {
		String vocabularyName = getDLBAdapter(defaultVocabularyName).getFormalCodingSchemeName(id.toString());
		if (vocabularyName != null)
		{
			namespaceId2VocabularyName_Map.put(id, vocabularyName);
			return vocabularyName;
		}
    } catch (Exception e) {
		//e.printStackTrace();
		log.error(e.getMessage());
	}
	return null;
}


   	private Response getHasParentsbyCode(HashMap map)throws Exception{
		int namespaceId = -1;
		String conceptCode = null;
		ArrayList list = new ArrayList();
		Vector ret_vec = null;
		Vector evs_vec = null;
        String vocabularyName = null;
		try {
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());
                 if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
                 else if(name.equalsIgnoreCase("namespaceId"))
					namespaceId = ((Integer)map.get(key)).intValue();
                 else if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String) map.get(key);
			}
			//String vocabularyName = namespaceId2VocabularyName(namespaceId);
			DLBAdapter adapter = getDLBAdapter(vocabularyName);
			Boolean retval = adapter.hasParents(vocabularyName, conceptCode);
			list.add(retval);
		} catch(Exception e) {
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return (new Response(list));
   	}

   	private Response getHasChildrenbyCode(HashMap map)throws Exception{
		int namespaceId = -1;
		String conceptCode = null;
		ArrayList list = new ArrayList();
		Vector ret_vec = null;
		Vector evs_vec = null;
        String vocabularyName = null;
		try {
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());
                if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
                else if(name.equalsIgnoreCase("namespaceId"))
					namespaceId = ((Integer)map.get(key)).intValue();
				else if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String) map.get(key);
			}
			//String vocabularyName = namespaceId2VocabularyName(namespaceId);
			DLBAdapter adapter = getDLBAdapter(vocabularyName);
			Boolean retval = adapter.hasChildren(vocabularyName, conceptCode);
			list.add(retval);
		} catch(Exception e) {
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return (new Response(list));
   	}

   	private Response getIsRetiredbyCode(HashMap map)throws Exception{
		int namespaceId = -1;
		String conceptCode = null;
		ArrayList list = new ArrayList();
		Vector ret_vec = null;
		Vector evs_vec = null;
        String vocabularyName = null;
		try {
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());
                if(name.equalsIgnoreCase("vocabularyName"))
                    vocabularyName = (String)map.get(key);
                else if(name.equalsIgnoreCase("namespaceId"))
					namespaceId = ((Integer)map.get(key)).intValue();
				else if(name.equalsIgnoreCase("conceptCode"))
					conceptCode = (String) map.get(key);
			}
			//String vocabularyName = namespaceId2VocabularyName(namespaceId);
			DLBAdapter adapter = getDLBAdapter(vocabularyName);
			Boolean retval = adapter.isRetired(conceptCode);
			list.add(retval);
		} catch(Exception e) {
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return (new Response(list));
   	}

   	/* (non-Javadoc)
	    * @see gov.nih.nci.system.dao.DAO#getAllClassNames()
	    */
	   public List getAllClassNames(){
        List classNames = new ArrayList();
        classNames.add("gov.nih.nci.evs.query.EVSQueryImpl");
        return classNames;
    }


 //========================================================================================================================

    /**
    * Converts a dtsrpc Concept object to a DescLogicConcept object
    * @param concept dtsrpc concept object
	* @return Returns a response
	* @throws Exception
    */

   private DescLogicConcept buildDescLogicConcept(CodedEntry concept, int asdIndex, String vocabularyName){
   		DescLogicConcept dlc = new DescLogicConcept();
        DLBAdapter adapter = null;
        try {
            adapter = getDLBAdapter(vocabularyName);
        } catch (Exception e) {
            //e.printStackTrace();
            log.error("Unable to connect to "+ vocabularyName);
        }

        //Vocabulary v = (Vocabulary)vocabularies.get(vocabularyName);
        Vocabulary v = getVocabulary(vocabularyName);

        dlc.setVocabulary(v);
        int namespaceId = v.getNamespaceId();
        String code = concept.getConceptCode();
        String name = adapter.getConceptNameByCode(code);
   		dlc.setName(name);
   		dlc.setCode(code);
   		dlc.setNamespaceId(namespaceId);


   		if (asdIndex == 0) return dlc;

   		if (asdIndex == 1 || asdIndex == 3) // Properties
   		{
			//dlc.setPropertyCollection(convertProperties(adapter.getAtomCollectionbyCode(vocabularyName, code)));
   		    //dlc.setPropertyCollection(convertProperties(adapter.getPropertyCollectionbyCode(vocabularyName, code)));
//			Vector v1 = convertProperties(adapter.getAtomCollectionbyCode(vocabularyName, code));
			Vector v1 = convertProperties(adapter.getPropertyCollectionbyCode(vocabularyName, code));
/*
			for (int i=0; i<v2.size(); i++)
			{
				v1.add(v2.elementAt(i));
			}
*/
			dlc.setPropertyCollection(v1);
   		    dlc.setSemanticTypeVector(convertProperties(adapter.getSemanticTypeCollectionbyCode(vocabularyName, code)));
	    }

   		if (asdIndex == 1 || asdIndex == 2) // Roles
   		{
   		    dlc.setRoleCollection(convertRoles(adapter.getRoleCollectionbyCode(vocabularyName, code)));
            dlc.setInverseRoleCollection(convertRoles(adapter.getInverseRoleCollectionbyCode(vocabularyName, code)));
	    }

		if (asdIndex == 4) // Associations
		{
		    dlc.setInverseAssociationCollection(convertAssociations(adapter.getInverseAssociationCollectionbyCode(vocabularyName, code)));
			dlc.setAssociationCollection(convertAssociations(adapter.getAssociationCollectionbyCode(vocabularyName, code)));
	    }
/*
		if(concept.getTreeNode()!=null){
			dlc.setEdgeProperties(convertEdgeProperties(concept));
			dlc.setTreeNode(convertTreeNode(concept));
		}

   		if (asdIndex == 1)
   		{
			dlc.setHasParents(concept.getHasParents());
			dlc.setHasChildren(concept.getHasChildren());
			dlc.setIsRetired(concept.getIsRetired());
	    }
*/

   		return dlc;
   	}


	/**
	 * Converts CodedEntry to gov.nih.nci.evs.domain.MetaThesaurusConcept
	 * @param metaConcept
	 * @return Returns a MetaThesaurusConcept object
	 */

	private MetaThesaurusConcept buildMetaThesaurusConcept(CodedEntry metaConcept) throws Exception
	{
		return buildMetaThesaurusConcept(metaConcept, 0);
	}


	private MetaThesaurusConcept buildMetaThesaurusConcept(CodedEntry metaConcept, int asdIndex) throws Exception
	{
		String code = metaConcept.getConceptCode();
		return buildMetaThesaurusConcept(code, asdIndex);
    }

	private MetaThesaurusConcept buildMetaThesaurusConcept(String code, int asdIndex) throws Exception
	{
		MetaThesaurusConcept metaThesaurusConcept = new MetaThesaurusConcept();
		try
		{
			String name = getDLBAdapter(metaVocabularyName).getConceptNameByCode(code);
			metaThesaurusConcept.setCui(code);
			metaThesaurusConcept.setName(name);
			ArrayList<String> synonymCollection = new ArrayList();

			DLBAdapter adapter = getDLBAdapter(metaVocabularyName);

			Vector<String> syn_vec = adapter.getPropertyValues(metaVocabularyName, null, null, code, "textualPresentation");

			for(int i=0; i<syn_vec.size(); i++){
				String syn = (String) syn_vec.elementAt(i);
				synonymCollection.add(syn);
			}
			metaThesaurusConcept.setSynonymCollection(synonymCollection);

			if (asdIndex == 0) return metaThesaurusConcept;

			ArrayList<Definition> definitionsCollection = new ArrayList();
			ArrayList<SemanticType> semanticTypesCollection = new ArrayList();
			ArrayList<Atom> atomCollection = new ArrayList();
			ArrayList<Source> sourceCollection = new ArrayList();
			HashSet synonyms = new HashSet();

			HashMap sourceMap = new HashMap();
			HashMap atoms = new HashMap();

			HashMap semanticTypes = new HashMap();
			HashMap definitions = new HashMap();
			synonymCollection = new ArrayList();

			//DLBAdapter adapter = getDLBAdapter(metaVocabularyName);
			Vector propertyCollection = convertProperties(adapter.getPropertyCollectionbyCode(metaVocabularyName, code));
			try{
				for(int p=0; p<propertyCollection.size(); p++){
					gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) propertyCollection.get(p);
					Vector qCollection = property.getQualifierCollection();
					if(property.getName().toUpperCase().equalsIgnoreCase("FULL_SYN")){
						gov.nih.nci.evs.domain.Atom atom = new gov.nih.nci.evs.domain.Atom();
						atom.setName(property.getValue());
						atom.setCode(code);
						for(int q=0; q< qCollection.size(); q++){
							gov.nih.nci.lexbig.ext.Qualifier qualifier = (gov.nih.nci.lexbig.ext.Qualifier)qCollection.get(q);
							gov.nih.nci.evs.domain.Source source = new gov.nih.nci.evs.domain.Source();
							if(qualifier.getName().toLowerCase().equalsIgnoreCase("source")){
								source.setAbbreviation(qualifier.getValue());
							}else if(qualifier.getName().toLowerCase().equalsIgnoreCase("source-code")){
								source.setCode(qualifier.getValue());
							}
							if(source.getAbbreviation()!=null){
								sourceMap.put(source.getAbbreviation(), source);
								atom.setSource(source);
								atom.setOrigin(source.getAbbreviation());
								atoms.put(atom.getName() + source.getAbbreviation(),atom);
							}
						}
						synonyms.add(property.getValue());

					}else if(property.getName().toUpperCase().equalsIgnoreCase("SEMANTIC_TYPE")){
						SemanticType sType = new SemanticType();
						sType.setName(property.getValue());
						semanticTypes.put(property.getValue(), sType);
					}else if(property.getName().toUpperCase().equalsIgnoreCase("DEFINITION")){
						Definition definition = new Definition();
						definition.setDefinition(property.getValue());
						gov.nih.nci.evs.domain.Source source = new gov.nih.nci.evs.domain.Source();
						for(int q=0; q< qCollection.size(); q++){
							gov.nih.nci.lexbig.ext.Qualifier qualifier = (gov.nih.nci.lexbig.ext.Qualifier)qCollection.get(q);
							if(qualifier.getName().toLowerCase().equalsIgnoreCase("source")){
								source.setAbbreviation(qualifier.getValue());
							}else if(qualifier.getName().toLowerCase().equalsIgnoreCase("source-code")){
								source.setCode(qualifier.getValue());
							}
						}
						definition.setSource(source);
						definitions.put(property.getValue(),definition);
						//sourceCollection.add(source);
						sourceMap.put(source.getAbbreviation(), source);
					}else if(property.getName().toUpperCase().equalsIgnoreCase("SOURCE")){
						Source source = new Source();
						source.setAbbreviation(property.getValue());
						//sourceCollection.add(source);
						sourceMap.put(source.getAbbreviation(), source);
					}

				}

			}catch(Exception ex){
				throw new DAOException("\nException: buildMetaConcept: "+ ex.getMessage());
			}

			for(Iterator it = semanticTypes.keySet().iterator(); it.hasNext();){
				semanticTypesCollection.add((SemanticType)semanticTypes.get(it.next()));
			}
			metaThesaurusConcept.setSemanticTypeCollection(semanticTypesCollection);

		    for(Iterator it = synonyms.iterator(); it.hasNext();){
				String synonym = (String) it.next();
				synonymCollection.add(synonym);
			}
			//metaThesaurusConcept.setSynonymCollection((ArrayList)synonymCollection);
			metaThesaurusConcept.setSynonymCollection(synonymCollection);

			for(Iterator it = definitions.keySet().iterator(); it.hasNext();){
				definitionsCollection.add((Definition)definitions.get(it.next()));
			}
			metaThesaurusConcept.setDefinitionCollection(definitionsCollection);
			for(Iterator it = sourceMap.keySet().iterator(); it.hasNext();){
				sourceCollection.add((Source)sourceMap.get(it.next()));
			}
			metaThesaurusConcept.setSourceCollection(sourceCollection);
			for(Iterator it = atoms.keySet().iterator(); it.hasNext();){
				atomCollection.add((Atom)atoms.get(it.next()));
			}
			metaThesaurusConcept.setAtomCollection(atomCollection);

		}catch(Exception e)
		{
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}

		return metaThesaurusConcept;
	}



	//GF8678 User wishes to retrieve all Synonyms for a set of concepts
	private Response fetchPropertyCollectionByCodes (HashMap map) throws Exception
	{
		int namespaceId = -1;
		String conceptCode = null;
		ArrayList list = new ArrayList();
		Vector ret_vec = null;
		String vocabularyName = null;

		ArrayList propertyNameList = null;
		ArrayList codeList = null;

		try {
			for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
			{
				String key = (String)iter.next();
				String name = key.substring(key.indexOf("$")+1, key.length());

				if(name.equalsIgnoreCase("vocabularyName"))
					vocabularyName = (String)map.get(key);
				else if(name.equalsIgnoreCase("propertyNames"))
					propertyNameList = (ArrayList) map.get(key);
				else if(name.equalsIgnoreCase("conceptCodes"))
					codeList = (ArrayList) map.get(key);
			}

			DLBAdapter adapter = getDLBAdapter(vocabularyName);
			try{
				ret_vec = adapter.fetchPropertyCollectionByCodes(vocabularyName, propertyNameList, codeList);
			}catch(Exception ex){
				log.error("Unable to query Properties for "+ conceptCode);
			}
			if (ret_vec != null) {
				int asdIndex = 3;
				for (int i=0; i<ret_vec.size(); i++)
				{
					CodedEntry ce = (CodedEntry) ret_vec.elementAt(i);
					DescLogicConcept concept = buildDescLogicConcept(ce, vocabularyName);
					list.add(concept);
				}
			}
		} catch(Exception e) {
			log.error(e.getMessage());
			throw new DAOException (getException(e.getMessage()));
		}
		return (new Response(list));
	}


    /**
    * Converts a dtsrpc Concept object to a DescLogicConcept object
    * @param concept dtsrpc concept object
	* @return Returns a response
	* @throws Exception
    */

   private DescLogicConcept buildDescLogicConcept(CodedEntry ce, String vocabularyName){
   		DescLogicConcept dlc = new DescLogicConcept();
        DLBAdapter adapter = null;
        try {
            adapter = getDLBAdapter(vocabularyName);
        } catch (Exception e) {
            //e.printStackTrace();
            log.error("Unable to connect to "+ vocabularyName);
        }

        Vocabulary vocab = getVocabulary(vocabularyName);

        dlc.setVocabulary(vocab);
        int namespaceId = vocab.getNamespaceId();
        String code = ce.getConceptCode();
        String name = adapter.getConceptNameByCode(code);
   		dlc.setName(name);
   		dlc.setCode(code);
   		dlc.setNamespaceId(namespaceId);

   		Vector v = convertProperties(adapter.getProperties(ce));
   		dlc.setPropertyCollection(v);
        return dlc;
   }



   /**
    * Gets all local names of the specified vocabulary
    * @param map Specifies the input parameters
	* @return Returns a response
	* @throws Exception
    */
    // GF#13786 Expose New Method via EVSQuery
    public Response getLocalNames(HashMap map)throws Exception {
     	String vocabularyName = null;
     	Vector names = new Vector();
     	List nameList = new ArrayList();
     	try{
     		for(Iterator iter=map.keySet().iterator(); iter.hasNext();)
     		{
     			String key = (String)iter.next();
     			String name = key.substring(key.indexOf("$")+1, key.length());

     			if(name.equalsIgnoreCase("vocabularyName"))
     				vocabularyName = (String)map.get(key);
     		}
     		names = getDLBAdapter(vocabularyName).getLocalNames(vocabularyName);
     		for(int i=0 ;i<names.size(); i++)
     		{
     			nameList.add(names.get(i));
     		}
		}catch(Exception e){
			log.error(e.getMessage());
			throw new DAOException(getException(e.getMessage()));
  		}
  	    return new Response(nameList);
    }
}
