package gov.nih.nci.system.dao.properties;

import java.io.File;
import java.io.FileInputStream;
import java.util.Iterator;
import java.util.Properties;

import org.apache.log4j.Logger;


/**
 * Singleton for accessing EVS Properties.
 * 
 * @author <a href="mailto:muhsins@mail.nih.gov">Shaziya Muhsin</a>
 * @author <a href="mailto:rokickik@mail.nih.gov">Konrad Rokicki</a>
 */
public class EVSProperties {
    
    private static Logger log = Logger.getLogger(EVSProperties.class);
    
	private static EVSProperties evsProperties;

    private static String lexBigConfigFileLocation = null;
	
    /**
     * Private constructor for singleton pattern.
     */
	private EVSProperties() {}
	
	/**
	 * Gets the single instance of EVSProperties.
	 * 
	 * @return single instance of EVSProperties
	 * 
	 * @throws Exception the exception
	 */
	public static EVSProperties getInstance() throws Exception{
		if(evsProperties == null) {
			synchronized(EVSProperties.class) {
				if(evsProperties == null) {
					evsProperties = new EVSProperties();
					loadProperties();
				}
			}
		}
		return evsProperties;
	}
	
	
    /**
     * Gets the lex big config file location.
     * 
     * @return the lex big config file location
     * 
     * @throws Exception the exception
     */
    public String getLexBigConfigFileLocation() throws Exception{
        return lexBigConfigFileLocation;
    }
    
	private static void loadProperties() throws Exception{
		String propertyFile = System.getProperty("gov.nih.nci.cacore.evsProperties");
		Properties properties = new Properties();
        log.info("evsPropertiesFileLocation="+ propertyFile);

		if(propertyFile != null && propertyFile.length() > 0){
			FileInputStream fis = new FileInputStream(new File(propertyFile));
			properties.load(fis);				
		}

		for(Iterator i = properties.keySet().iterator(); i.hasNext();){
			String key = (String)i.next();
			String value  = properties.getProperty(key);
            log.debug("KEY: "+ key +"\t - "+value);
		}
        
		lexBigConfigFileLocation = properties.getProperty("LG_CONFIG_FILE");
	} 

}
