package gov.nih.nci.system.webservice;

import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.util.ClassCache;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.server.ServiceLifecycle;

import org.apache.axis.MessageContext;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.log4j.Logger;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

//GF#9397 KLO, 120607
import gov.nih.nci.evs.domain.*;
import gov.nih.nci.evs.query.*;

/*
 * Created on Aug 8, 2007
 * Shaziya Muhsin
 *
 */
/**
 * The Class EVSWSQuery.
 */
public class EVSWSQuery implements ServiceLifecycle{

    private static Logger log = Logger.getLogger(EVSWSQuery.class);
    private EVSApplicationService applicationService;
    private ClassCache classCache;
//    private static int resultCountPerQuery = 1000;


    /* (non-Javadoc)
     * @see javax.xml.rpc.server.ServiceLifecycle#init(java.lang.Object)
     */
    public void init(Object obj) throws ServiceException {
        ServletContext context;
        if(obj instanceof ServletContext)
        {
            log.debug("Obtained servlet context directly as init parameter");
            context= (ServletContext) obj;
        }
        else
        {
            log.debug("Obtaining servlet context indirectly from message context");
            HttpServlet srv = (HttpServlet) MessageContext.getCurrentContext().getProperty(HTTPConstants.MC_HTTP_SERVLET);
            context = srv.getServletContext();
            log.debug("Obtained servlet context from message context");
        }
        WebApplicationContext ctx =  WebApplicationContextUtils.getWebApplicationContext(context);
        classCache = (ClassCache)ctx.getBean("ClassCache");
        applicationService = (EVSApplicationService)ctx.getBean("EVSApplicationServiceImpl");

        Properties systemProperties = (Properties) ctx.getBean("SystemProperties");

        try {
            String count = systemProperties.getProperty("resultCountPerQuery");
            log.debug("resultCountPerQuery: " + count);
//            if (count != null) {
//                resultCountPerQuery = Integer.parseInt(count);
//            }
        } catch (Exception ex) {
            log.error("Exception initializing resultCountPerQuery: ", ex);
            throw new ServiceException("Exception initializing resultCountPerQuery: ", ex);
        }
    }
    public void destroy() {
        applicationService = null;
        classCache = null;
//        resultCountPerQuery = 0;
    }

    /**
     * Query.
     *
     * @param targetClassName the target class name
     * @param criteria the criteria
     * @param startIndex the start index
     *
     * @return the list
     *
     * @throws Exception the exception
     */
    public List query(String targetClassName, Object criteria, int startIndex) throws Exception
    {
        List results = new ArrayList();
        results = getResultSet(targetClassName, criteria, startIndex);
        return results;
    }

    /**
     * Query object.
     *
     * @param targetClassName the target class name
     * @param criteria the criteria
     *
     * @return the list
     *
     * @throws Exception the exception
     */
    public List queryObject(String targetClassName, Object criteria) throws Exception
    {
        return query(targetClassName,criteria,0);
    }

    private List getResultSet(String targetClassName, Object searchCriteria, int startIndex) throws Exception{

        List results = new ArrayList();
        String searchClassName = null;
        if(targetClassName == null){
            searchClassName = searchCriteria.getClass().getName();
        }else{
            searchClassName = classCache.getQualifiedClassName(targetClassName);
        }
        try
        {
            results = applicationService.search(searchClassName,searchCriteria);

        }
        catch(Exception e)
        {
            log.error("WSQuery caught an exception: ", e);
            throw e;
        }
        return results;
    }


    // GF#9397 KLO, 120607
    /**
     * Query.
     * 
     * @param methodName the method name
     * @param criteria the criteria
     * 
     * @return the list
     * 
     * @throws Exception the exception
     */
    public List query(String methodName, Object criteria) throws Exception
    {
		List results = new ArrayList();
		if (methodName.equals("getLicense") || methodName.equals("getAllLicenses"))
		{
			 Vocabulary vocabulary = (Vocabulary) criteria;
			 String vocabularyName = vocabulary.getName();
			 String condition = "";
			 EVSQuery evsQuery = new EVSQueryImpl();
			 evsQuery.getAllLicenses(vocabularyName, condition);
			 try
			 {
				results = applicationService.evsSearch(evsQuery);
			 }
			 catch(Exception e)
			 {
				log.error("WSQuery caught an exception: ", e);
				throw e;
			 }
		}
		return results;
    }

}
