package gov.nih.nci.evs.domain;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Vector;

import gov.nih.nci.evs.EVSQueryServiceTestCase;

import org.junit.Test;

public class DescLogicConceptTests extends EVSQueryServiceTestCase{
    
	final static String testID = "domain_DescLogicConceptTest";
	DescLogicConcept ldlc = EVSQueryServiceTestCase.getDescLogicConcept();
	DescLogicConcept setLdlc = new DescLogicConcept();
	
    @Override
    protected String getTestID()
    {
        return testID;
    }

	@Test
	public final void testGetCode() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		String code = ldlc.getCode();
		assertNotNull("Code should exist", code);
		assertEquals("Code should be C3113", code, DOMAIN_CONCEPT_CODE);

	}

	@Test
	public final void testSetCode() {
		String setCode="OA1234";
		setLdlc.setCode(setCode);
		String code = "";
		code = setLdlc.getCode();
		assertTrue("Code set",code.equals(setCode));
	}

	@Test
	public final void testGetHasChildren() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		boolean has_child = ldlc.getHasChildren();
		assertTrue("Concept has children", has_child);
	}

	@Test
	public final void testSetHasChildren() {
		boolean setHasChildren = true;
		setLdlc.setHasChildren(setHasChildren);
		Boolean hasChildren = false;
		hasChildren = setLdlc.getHasChildren();
		assertTrue("hasChildren set", hasChildren.booleanValue());
	}

	@Test
	public final void testGetHasParents() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		boolean has_parent = ldlc.getHasParents();
		assertTrue("Concept has parent", has_parent);
	}

	@Test
	public final void testSetHasParents() {
		boolean setHasParents = true;
		setLdlc.setHasParents(setHasParents);
		Boolean hasParents = false;
		hasParents = setLdlc.getHasParents();
		assertNotNull("hasParents returned",hasParents);
		assertTrue("hasParents set", hasParents.booleanValue());
	}

	@Test
	public final void testGetIsRetired() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Boolean retired = ldlc.getIsRetired();
		assertNotNull("Value should exist", retired);
		assertFalse("Chosen concept is not retired", retired.booleanValue());
	}

	@Test
	public final void testSetIsRetired() {
		boolean setIsRetired = true;
		setLdlc.setIsRetired(setIsRetired);
		Boolean isRetired = false;
		isRetired = setLdlc.getIsRetired();
		assertNotNull("isRetired returned", isRetired);
		assertTrue("isRetired set", isRetired.booleanValue());
	}

	@Test
	public final void testGetName() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		String name = ldlc.getName();
		assertNotNull("Name should exist", name);
		assertEquals("Name should be Hyperplasia", name, DOMAIN_CONCEPT_NAME);
	}

	@Test
	public final void testSetName() {
		String setName="ConceptName";
		setLdlc.setName(setName);
		String name = "";
		name = setLdlc.getName();
		assertNotNull("Name returned", name);
		assertTrue("name set",name.equals(setName));
	}

	@Test
	public final void testGetNamespaceId() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Integer namespace = ldlc.getNamespaceId();
		String namespaceId = namespace.toString();
		assertNotNull("Name should exist", namespaceId);
		assertEquals("NamespaceId should be 40010", namespaceId, "40010");
	}

	@Test
	public final void testSetNamespaceId() {
		Integer setNsid=4444;
		setLdlc.setNamespaceId(setNsid);
		Integer nsid = new Integer(0);
		nsid = setLdlc.getNamespaceId();
		assertNotNull("NamespaceId returned", nsid);
		assertTrue("NamespaceId set",nsid.equals(setNsid));
	}

	@Test
	public final void testGetSemanticTypeVector() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Vector semanticType= ldlc.getSemanticTypeVector();
		assertNotNull("SemanticTypes should be returned", semanticType);
		assertTrue("SemanticTypes list should have values", semanticType.size()>0);
	}

	@Test
	public final void testSetSemanticTypeVector() {
		SemanticType sem1 = new SemanticType();
		String sem1Id = "Sem1_ID";
		sem1.setId(sem1Id);
		String sem1Name = "sem1_Name";
		sem1.setName(sem1Name);
		SemanticType sem2 = new SemanticType();
		String sem2Id = "Sem2_ID";
		sem2.setId(sem2Id);
		String sem2Name = "Sem2_Name";
		sem2.setName(sem2Name);
		Vector semTypeVec = new Vector();
		semTypeVec.add(0, sem1);
		semTypeVec.add(1, sem2);
		
		setLdlc.setSemanticTypeVector(semTypeVec);
		Vector getVector = new Vector();
		getVector = setLdlc.getSemanticTypeVector();
		assertNotNull("Semantic Type vector should be returned", getVector);
		assertTrue("Semantic Type Vector should have 2 entries", getVector.size()==2);
		SemanticType getSem = (SemanticType)getVector.elementAt(0);
		assertTrue("First SemanticType code matches", getSem.getId().equals(sem1Id));
		assertTrue("First SemanticType name matches", getSem.getName().equals(sem1Name));
		getSem = (SemanticType)getVector.elementAt(1);
		assertTrue("First SemanticType code matches", getSem.getId().equals(sem2Id));
		assertTrue("First SemanticType name matches", getSem.getName().equals(sem2Name));

	}

	@Test
	public final void testGetTreeNode() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		TreeNode node= ldlc.getTreeNode();
		assertNotNull("TreeNode should be returned", node);
		Boolean hierarchy = node.getIsA();
		assertNotNull("Hierarchy should be returned", hierarchy);
		assertTrue("Hierarchy list should be true",hierarchy.booleanValue());

	}

	@Test
	public final void testSetTreeNode() {
		TreeNode setNode = new TreeNode();
		String name = "nodeName";
		setNode.setName(name);
		Boolean isA = new Boolean(true);
		setNode.setIsA(isA);
		Boolean traverseDown = new Boolean(true);
		setNode.setTraverseDown(traverseDown);
		
		setLdlc.setTreeNode(setNode);
		
		TreeNode getNode = new TreeNode();
		getNode = setLdlc.getTreeNode();
		assertTrue("Node names match", getNode.getName().equals(name));
		assertTrue("isA is true", getNode.getIsA().booleanValue());
		assertTrue("traverse down in true", getNode.getTraverseDown().booleanValue());
	}

	@Test
	public final void testGetRoleCollection() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Vector<Role> roles= ldlc.getRoleCollection();
		assertNotNull("Roles should be returned", roles);
		assertTrue("Roles list should have values", roles.size()>0);
		boolean found = false;
		Role role = new Role();
		for (int i=0;i<roles.size();i++)
		{
			role = (Role)roles.elementAt(i);
			String name = role.getName();
			if (name.equals(DOMAIN_CONCEPT_ROLE)) found = true;
		}
		assertTrue("Role Disease_Has_Finding found in list", found);
	}

	@Test
	public final void testSetRoleCollection() {
		Role role1 = new Role();
		String name = "Name_of_role1";
		String value = "Value of Role1";
		role1.setName(name);
		role1.setValue(value);
		
		Role role2 = new Role();
		name = "Name_of_role2";
		value = "Value of role2";
		role2.setName(name);
		role2.setValue(value);
		
		Vector<Role> setRoles = new Vector<Role>();
		setRoles.add(0, role1);
		setRoles.add(1,role2);
		
		setLdlc.setRoleCollection(setRoles);
		
		Vector<Role> roles = setLdlc.getRoleCollection();
		assertNotNull("Roles set", roles);
		assertTrue("Roles correct", roles.elementAt(1).getName().equals(name));
	}

	@Test
	public final void testGetAssociationCollection() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Vector<Association> assocs= ldlc.getAssociationCollection();
		assertNotNull("Associations should be returned", assocs);
		assertTrue("Associations list should have values", assocs.size()>0);
		boolean found = false;
		Association assoc = new Association();
		for (int i=0;i<assocs.size();i++)
		{
			assoc = (Association)assocs.elementAt(i);
			String name = assoc.getName();
			if (name.equals(DOMAIN_CONCEPT_ASSOC)) found = true;
		}
		assertTrue("Association found in list", found);
	}

	@Test
	public final void testSetAssociationCollection() {
		Association assoc1 = new Association();
		String name = "Name of Association1";
		String value = "Value of Association1";
		assoc1.setName(name);
		assoc1.setValue(value);
		
		Association assoc2 = new Association();
		name = "Name of Association2";
		value = "Vale of Association2";
		assoc2.setName(name);
		assoc2.setValue(value);
		
		Vector<Association> setAssoc = new Vector<Association>();
		setAssoc.add(0, assoc1);
		setAssoc.add(1,assoc2);
		setLdlc.setAssociationCollection(setAssoc);
		
		Vector<Association> assocs = new Vector<Association>();
		assocs = setLdlc.getAssociationCollection();
		assertNotNull("Assocs returned", assocs);
		assertTrue("Association value correct", assocs.get(1).getValue().equals(value));
	}

	@Test
	public final void testGetInverseAssociationCollection() {
		DescLogicConcept ldlc_ia = EVSQueryServiceTestCase.getDescLogicConcept_inverse_association();
		Vector<Association> assocs = ldlc_ia.getInverseAssociationCollection();
		assertNotNull("Associations retuned", assocs);
		assertTrue("Associations has values", assocs.size()>0);
	}

	@Test
	public final void testSetInverseAssociationCollection() {
		Association assoc1 = new Association();
		String name = "Name of Association1";
		String value = "Value of Association1";
		assoc1.setName(name);
		assoc1.setValue(value);
		
		Association assoc2 = new Association();
		name = "Name of Association2";
		value = "Vale of Association2";
		assoc2.setName(name);
		assoc2.setValue(value);
		
		Vector<Association> setAssoc = new Vector<Association>();
		setAssoc.add(0, assoc1);
		setAssoc.add(1,assoc2);
		setLdlc.setInverseAssociationCollection(setAssoc);
		
		Vector<Association> assocs = new Vector<Association>();
		assocs = setLdlc.getInverseAssociationCollection();
		assertNotNull("Assocs returned", assocs);
		assertTrue("Association value correct", assocs.get(1).getValue().equals(value));
	}

	@Test
	public final void testGetInverseRoleCollection() {
		DescLogicConcept ldlc_ir = EVSQueryServiceTestCase.getDescLogicConcept_inverse_role();
		Vector<Role> roles = ldlc_ir.getInverseRoleCollection();
		assertNotNull("Roles returned", roles);
		assertTrue("Roles has values", roles.size()>0);
	}

	@Test
	public final void testSetInverseRoleCollection() {
		Role role1 = new Role();
		String name = "Name_of_role1";
		String value = "Value of Role1";
		role1.setName(name);
		role1.setValue(value);
		
		Role role2 = new Role();
		name = "Name_of_role2";
		value = "Value of role2";
		role2.setName(name);
		role2.setValue(value);
		
		Vector<Role> setRoles = new Vector<Role>();
		setRoles.add(0, role1);
		setRoles.add(1,role2);
		
		setLdlc.setInverseRoleCollection(setRoles);
		
		Vector<Role> roles = setLdlc.getInverseRoleCollection();
		assertNotNull("Roles set", roles);
		assertTrue("Roles correct", roles.elementAt(1).getName().equals(name));
	}

	@Test
	public final void testGetEdgeProperties() {
		EdgeProperties getEP = new EdgeProperties();
		getEP = ldlc.getEdgeProperties();
		assertNotNull("EdgeProperties returned", getEP);
		String edgeName = getEP.getName();
		assertNotNull("Name returned", edgeName);
		Boolean getIsA = getEP.getIsA();
		assertNotNull("isA returned", getIsA);
		Boolean getTD = getEP.getTraverseDown();
		assertNotNull("traverseDown returned", getTD);
		HashSet getLinks = getEP.getLinks();
		assertNotNull("Links returned", getLinks);
	}

	@Test
	public final void testSetEdgeProperties() {
		EdgeProperties edge1 = new EdgeProperties();
		String name = "edge1";
		Boolean isa = new Boolean(true);
		Boolean td = new Boolean(false);
		edge1.setIsA(isa);
		edge1.setName(name);
		edge1.setTraverseDown(td);
		
		setLdlc.setEdgeProperties(edge1);
		
		EdgeProperties getEdge = setLdlc.getEdgeProperties();
		assertNotNull("EdgeProperties returned",getEdge);
		assertTrue("EdgeProperties isA is true", getEdge.getIsA());
		assertFalse("EdgeProperties traversedown is false", getEdge.getTraverseDown());
	}

	@Test
	public final void testGetPropertyCollection() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		Vector<Property> properties= ldlc.getPropertyCollection();
		assertNotNull("Properties should be returned", properties);
		assertTrue("Properties list should have values", properties.size()>0);
		boolean found = false;
		Property property = new Property();
		for (int i=0;i<properties.size();i++)
		{
			property = (Property)properties.elementAt(i);
			String name = property.getName();
			if (name.equals("DEFINITION")) found = true;
		}
		assertTrue("Property DEFINITION found in list", found);

	}

	@Test
	public final void testSetPropertyCollection() {
		Property prop1 = new Property();
		String name = "Property1";
		String value1 = "Value of property1";
		prop1.setName(name);
		prop1.setValue(value1);
		
		Property prop2 = new Property();
		name = "Property2";
		String value2 = "Value of property2";
		prop2.setName(name);
		prop2.setValue(value2);
		
		Vector<Property> setProps = new Vector<Property>();
		setProps.add(0, prop1);
		setProps.add(1, prop2);
		setLdlc.setPropertyCollection(setProps);
		
		Vector<Property> props = setLdlc.getPropertyCollection();
		assertNotNull("Props returned", props);
		assertTrue("Properties are correct", props.get(0).getValue().equals(value1));
	}

	@Test
	public final void testGetHistoryRecord() {
		assertNotNull("DescLogicConcept should be populated", ldlc);
		HistoryRecord record= ldlc.getHistoryRecord();
		assertNotNull("HistoryRecord should be returned", record);
		Vector<History> events = record.getHistoryCollection();
		assertNotNull("History events should be returned", events);
		assertTrue("History events list should have values", events.size()>0);

	}

	@Test
	public final void testSetHistoryRecord() {
		String code = "AHist01";
		HistoryRecord setHistRecord = new HistoryRecord();
		History hist1 = new History();
		String editAction1 = "Modify";
		Date date1 = new Date();
		date1.setMonth(6);
		date1.setDate(5);
		date1.setYear(2004);
		hist1.setEditAction(editAction1);
		hist1.setEditActionDate(date1);
	
		Vector<History> histCollection = new Vector<History>();
		histCollection.add(hist1);
		
		setHistRecord.setHistoryCollection(histCollection);
		setHistRecord.setDescLogicConceptCode(code);
		
		setLdlc.setHistoryRecord(setHistRecord);
		
		HistoryRecord histRecord = setLdlc.getHistoryRecord();
		assertNotNull("History Record returned", histRecord);
		assertTrue("History Record is correct",histRecord.getDescLogicConceptCode().equals(code));
		assertTrue("History Record collection is correct", histRecord.getHistoryCollection().elementAt(0).getEditAction().equals(editAction1));
	}

	@Test
	public final void testGetVocabulary() {
		Vocabulary vocab = ldlc.getVocabulary();
		String vocabName = vocab.getName();
		assertNotNull("Name should exist", vocabName);
		assertEquals("Vocab name should be NCI Thesaurus", vocabName, "NCI Thesaurus");
	}

	@Test
	public final void testSetVocabulary() {
		Vocabulary setVocab = new Vocabulary();
		String name = "setVocabName testing";
		setVocab.setName(name);
		String description = "A test description for dlc vocab";
		setVocab.setDescription(description);
		Integer namespaceId = 111223;
		setVocab.setNamespaceId(namespaceId);
		setLdlc.setVocabulary(setVocab);
		
		Vocabulary vocab = setLdlc.getVocabulary();
		assertTrue("Vocab name returned", vocab.getName().equals(name));
		assertTrue("Vocab namespace id returned", vocab.getNamespaceId().equals(namespaceId));
		assertTrue("Vocab description returned", vocab.getDescription().equals(description));
	}

}
