package gov.nih.nci.evs.domain;

//import static org.junit.Assert.*;
import java.util.ArrayList;
import java.util.Iterator;

import gov.nih.nci.evs.EVSQueryServiceTestCase;

import org.junit.Test;

public class MetathesaurusTest extends EVSQueryServiceTestCase{
    
	final static String testID = "domain_MetathesaurusTest";
	MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
    @Override
    protected String getTestID()
    {
        return testID;
    }
	@Test
	public final void testGetCui() {
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		String cui = lmtc.getCui();
		assertNotNull("Cui should exist", cui);
		assertEquals("Cui should be C0007952", cui, "C0007952");
	}

	@Test
	public final void testSetCui() {
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		String cuiSet= "C0011111";
		meta.setCui(cuiSet);
		String cui = meta.getCui();
		assertEquals("setting cui should work", cuiSet, cui);
		
		cuiSet= "X33333";
		meta.setCui(cuiSet);
		cui = meta.getCui();
		assertEquals("Should be able to reset cui", cuiSet, cui);
	}

	@Test
	public final void testGetName() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		String name = lmtc.getName();
		assertNotNull("Name should exist", name);
		assertEquals("Name should be Character", name, "Character");
	}

	@Test
	public final void testSetName() {
		
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		String nameSet= "concept test";
		meta.setName(nameSet);
		String name = meta.getName();
		assertEquals("setting name should work", nameSet, name);
		
		nameSet= "concept_test";
		meta.setName(nameSet);
		name = meta.getName();
		assertEquals("Should be able to reset name", nameSet, name);

		nameSet= "Concept_Test";
		meta.setName(nameSet);
		name = meta.getName();
		assertEquals("Capitalization should be preserved", nameSet, name);

	}

	@Test
	public final void testGetSynonymCollection() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		ArrayList synonyms = lmtc.getSynonymCollection();
		assertNotNull("synonyms should be returned", synonyms);
		assertTrue("synonym list should have values", synonyms.size()>0);
		boolean found = false;
		String synonym = "";
		for (int i=0;i<synonyms.size();i++)
		{
			synonym = (String)synonyms.get(i);
			if (synonym.equals("Characters")) found = true;
		}
		assertTrue("Synonym Characters found in list", found);
	}

	@Test
	public final void testSetSynonymCollection() {
		ArrayList synonymCollection = new ArrayList();
		synonymCollection.add("Synonym1");
		synonymCollection.add("Syn2");
		synonymCollection.add("synonym3");
		
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		meta.setSynonymCollection((ArrayList)synonymCollection);

		ArrayList synonyms = meta.getSynonymCollection();
		assertNotNull("synonyms should be returned", synonyms);
		assertTrue("synonym list should have values", synonyms.size()>0);
		assertEquals("first synonym matches", "Synonym1", synonyms.get(0));
		assertEquals("second synonym matches", "Syn2", synonyms.get(1));
		assertEquals("third synonym matches", "synonym3", synonyms.get(2));
	}

	@Test
	public final void testGetSemanticTypeCollection() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		ArrayList<SemanticType> semanticTypes = lmtc.getSemanticTypeCollection();
		assertNotNull("semanticTypes should be returned", semanticTypes);
		assertTrue("semanticTypes list should have values", semanticTypes.size()>0);
		boolean found = false;
		SemanticType semanticType = new SemanticType();
		for (int i=0;i<semanticTypes.size();i++)
		{
			semanticType = (SemanticType)semanticTypes.get(i);
			String name = semanticType.getName();
			if (name.equals("Mental Process")) found = true;
		}
		assertTrue("SemanticType Mental Process found in list", found);

	}

	@Test
	public final void testSetSemanticTypeCollection() {
		
        SemanticType sem1 = new SemanticType();
        SemanticType sem2 = new SemanticType();
        
        sem1.setId("1");
        sem1.setName("semtype1");
        
        sem2.setId("2");
        sem2.setName("semtype2");
        
        ArrayList<SemanticType> semanticTypes = new ArrayList<SemanticType>();
        semanticTypes.add(sem1);
        semanticTypes.add(sem2);
        
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		meta.setSemanticTypeCollection(semanticTypes);

		ArrayList<SemanticType> semTypes = meta.getSemanticTypeCollection();
		assertNotNull("semantic type should be returned", semTypes);
		assertTrue("semantic type list should have values", semTypes.size()>0);
		assertEquals("first semantic type id matches", "1", semTypes.get(0).getId());
		assertEquals("first semantic type name matches", "semtype1", semTypes.get(0).getName());
		assertEquals("second semantic type id matches", "2", semTypes.get(1).getId());
		assertEquals("second semantic type name matches", "semtype2", semTypes.get(1).getName());
	}

	@Test
	public final void testGetAtomCollection() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		ArrayList<Atom> Atoms = lmtc.getAtomCollection();
		assertNotNull("Atoms should be returned", Atoms);
		assertTrue("Atoms list should have values", Atoms.size()>0);
		boolean found = false;
		Atom atom = new Atom();
		for (int i=0;i<Atoms.size();i++)
		{
			atom = (Atom)Atoms.get(i);
			String name = atom.getName();
			if (name.equals("Personality Character")) found = true;
		}
		assertTrue("Atom Personality Character found in list", found);

	}

	@Test
	public final void testSetAtomCollection() {
        Atom atom1 = new Atom();
        Atom atom2 = new Atom();
        Source source1 = new Source();
        Source source2 = new Source();
        
        atom1.setCode("1");
        atom1.setLui("lui1");
        atom1.setName("atom1");
        atom1.setOrigin("origin1");
        source1.setAbbreviation("short1");
        source1.setCode("source1");
        source1.setDescription("this is source 1");
        atom1.setSource(source1);
        
        atom2.setCode("2");
        atom2.setLui("lui2");
        atom2.setName("atom2");
        atom2.setOrigin("origin2");
        source2.setAbbreviation("short2");
        source2.setCode("source2");
        source2.setDescription("this is source 2");
        atom2.setSource(source2);
        
        ArrayList<Atom> setAtoms = new ArrayList<Atom>();
        setAtoms.add(atom1);
        setAtoms.add(atom2);
        
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		meta.setAtomCollection(setAtoms);

		ArrayList<Atom> atoms = meta.getAtomCollection();
		assertNotNull("atom list should be returned", atoms);
		assertTrue("atom list should have values", atoms.size()>0);
		
		assertEquals("first atom code matches", "1", atoms.get(0).getCode());
		assertEquals("first atom lui matches", "lui1", atoms.get(0).getLui());
		assertEquals("first atom name matches", "atom1", atoms.get(0).getName());
		assertEquals("first atom origin matches", "origin1", atoms.get(0).getOrigin());
		assertEquals("first atom source abbreviation matches", "short1", atoms.get(0).getSource().getAbbreviation());
		assertEquals("first atom source code matches", "source1", atoms.get(0).getSource().getCode());
		assertEquals("first atom source description matches", "this is source 1", atoms.get(0).getSource().getDescription());

		assertEquals("second atom code matches", "2", atoms.get(1).getCode());
		assertEquals("second atom lui matches", "lui2", atoms.get(1).getLui());
		assertEquals("second atom name matches", "atom2", atoms.get(1).getName());
		assertEquals("second atom origin matches", "origin2", atoms.get(1).getOrigin());
		assertEquals("second atom source abbreviation matches", "short2", atoms.get(1).getSource().getAbbreviation());
		assertEquals("second atom source code matches", "source2", atoms.get(1).getSource().getCode());
		assertEquals("second atom source description matches", "this is source 2", atoms.get(1).getSource().getDescription());
	}

	@Test
	public final void testGetSourceCollection() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		ArrayList<Source> sources = lmtc.getSourceCollection();
		assertNotNull("sources should be returned", sources);
		assertTrue("sources list should have values", sources.size()>0);
		boolean found = false;
		Source source = new Source();
		for (int i=0;i<sources.size();i++)
		{
			source = (Source)sources.get(i);
			String abrev = source.getAbbreviation();
			if (abrev.equals("MSH")) found = true;
		}
		assertTrue("Source MSH found in list", found);
		
	}

	@Test
	public final void testSetSourceCollection() {
        Source source1 = new Source();
        Source source2 = new Source();
        
        source1.setAbbreviation("short1");
        source1.setCode("source1");
        source1.setDescription("this is source 1");
        
        source2.setAbbreviation("short2");
        source2.setCode("source2");
        source2.setDescription("this is source 2");
        
        ArrayList<Source> setSources = new ArrayList<Source>();
        setSources.add(source1);
        setSources.add(source2);
        
		MetaThesaurusConcept meta = new MetaThesaurusConcept();
		meta.setSourceCollection(setSources);

		ArrayList<Source> sources = meta.getSourceCollection();
		assertNotNull("source list should be returned", sources);
		assertTrue("source list should have values", sources.size()>0);
		assertEquals("first source abbreviation matches", "short1", sources.get(0).getAbbreviation());
		assertEquals("first source code matches", "source1", sources.get(0).getCode());
		assertEquals("first source description matches", "this is source 1", sources.get(0).getDescription());
		assertEquals("second source abbreviation matches", "short2", sources.get(1).getAbbreviation());
		assertEquals("second source code matches", "source2", sources.get(1).getCode());
		assertEquals("second source description matches", "this is source 2", sources.get(1).getDescription());
	}

	@Test
	public final void testGetDefinitionCollection() {
//		MetaThesaurusConcept lmtc = EVSQueryServiceTestCase.getMetaThesaurusConcept();
		assertNotNull("Metathesaurus concept should be populated", lmtc);
		ArrayList<Definition> Definitions = lmtc.getDefinitionCollection();
		assertNotNull("Definitions should be returned", Definitions);
		assertTrue("Definitions list should have values", Definitions.size()>0);
		Definition definition = new Definition();
		definition = (Definition)Definitions.get(0);
		String def = definition.getDefinition();
		assertTrue("Definition has text", def.length()>0);
	}

	@Test
	public final void testSetDefinitionCollection() {
	        Definition def1 = new Definition();
	        Definition def2 = new Definition();
	        Source source1 = new Source();
	        Source source2 = new Source();
	        
	        def1.setDefinition("This is definition1");
	        source1.setAbbreviation("short1");
	        source1.setCode("1");
	        source1.setDescription("source1");
	        def1.setSource(source1);

	        def2.setDefinition("This is definition2");
	        source2.setAbbreviation("short2");
	        source2.setCode("2");
	        source2.setDescription("source2");
	        def2.setSource(source2);
	        
	        ArrayList<Definition> setDef = new ArrayList<Definition>();
	        setDef.add(def1);
	        setDef.add(def2);
	        
			MetaThesaurusConcept meta = new MetaThesaurusConcept();
			meta.setDefinitionCollection(setDef);

			ArrayList<Definition> definitions = meta.getDefinitionCollection();
			assertNotNull("definition list should be returned", definitions);
			assertTrue("definition list should have values", definitions.size()>0);
			assertEquals("first definition matches", "This is definition1", definitions.get(0).getDefinition());
			assertEquals("first definition source abbreviation matches", "short1", definitions.get(0).getSource().getAbbreviation());
			assertEquals("first definition source code matches", "1", definitions.get(0).getSource().getCode());
			assertEquals("first definition source description matches", "source1", definitions.get(0).getSource().getDescription());
			assertEquals("second definition matches", "This is definition2", definitions.get(1).getDefinition());
			assertEquals("second definition source abbreviation matches", "short2", definitions.get(1).getSource().getAbbreviation());
			assertEquals("second definition source code matches", "2", definitions.get(1).getSource().getCode());
			assertEquals("second definition source description matches", "source2", definitions.get(1).getSource().getDescription());	}

}
