package gov.nih.nci.evs.history;

import gov.nih.nci.evs.EVSQueryServiceHolder;
import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.evs.domain.EditActionDate;
import gov.nih.nci.evs.domain.HistoryRecord;
import gov.nih.nci.evs.query.EVSQuery;
import gov.nih.nci.evs.query.EVSQueryImpl;
import gov.nih.nci.lexbig.ext.DLBAdapter;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.Test;

/**
 * The Class HistoryTest.
 */
public class HistoryTest extends EVSQueryServiceTestCase {

    final static String testID = "HistoryTest";

    @Override
    protected String getTestID()
    {
        return testID;
    }

    /**
     * This method will test getAncestors.
     * 
     * @throws Exception the exception
     */
    public void testgetAncestors() throws Exception {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
//        String conceptCode = "C45253";
        String conceptCode = "C68805";
		List evsResults = new ArrayList();
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getAncestors(vocabName,conceptCode,true,"07/27/2007","09/27/2007");
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0); 
            String x = (String)evsResults.get(0);
            assertTrue("ancestor is C45253", x.compareTo("C45253") == 0);

            } 
        catch (Exception ex) {
            System.out.println("testgetAncestors throws Exception = " + ex);
            fail("Exception in testgetAncestors " + ex.getMessage() );
         }
         
    }
    
    /**
     * This method will test getConceptEditAction
     *  
     */
    public void testgetConceptEditAction_StringString() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C618";
		List evsResults = new ArrayList();
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptEditAction(vocabName,conceptCode);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>10); 
            EditActionDate editDate = (EditActionDate)evsResults.get(5);
            Integer x = (Integer)editDate.getAction();
            assertNotNull("editDate has values", x);
            } catch (Exception ex) {
                System.out.println("testgetConceptEditAction throws Exception = " + ex);
                fail("Exception in testgetConceptEditAction " + ex.getMessage() );
            }
    }
    
    /**
     * This method will test getConceptEditActionDates
     *  
     */
    public void testgetConceptEditActionDates_StringStringString() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C618";
		String action = "modify";
		List evsResults = new ArrayList();
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptEditActionDates(vocabName,conceptCode,action);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0); 
            Date editDate = (Date)evsResults.get(0);
            String x = (String)editDate.toString();
            assertNotNull("editDate has values", x);
            } catch (Exception ex) {
                System.out.println("testgetConceptEditActionDates throws Exception = " + ex);
                fail("Exception in EVSQueryServiceTestCase " + ex.getMessage() );
            }
    }
    
	@Test
	public void testGetConceptEditAction_StringStringDate() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C618";
		String action = "modify";
		Date editActionDate = new Date(103,11,5);

		List evsResults = new ArrayList();
		try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptEditAction(vocabName, conceptCode, editActionDate);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            EditActionDate editDate = (EditActionDate)evsResults.get(0);
            String x = (String)editDate.toString();
            assertNotNull("editDate has values", x);
            } catch (Exception ex) {
                System.out.println("testgetConceptEditActionDates throws Exception = " + ex);
                fail("Exception in EVSQueryServiceTestCase " + ex.getMessage() );
            }
	}

    
    /**
     * This method will test getDescendants
     *  
     */
    public void testgetDescendants() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String conceptCode = "C60648";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescendants(vocabName,conceptCode,false,"07/27/2007","09/27/2007");
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0); 
            String x = (String)evsResults.get(0);
            assertTrue("ancestor is C25967", x.compareTo("C25967")== 0);
            } catch (Exception ex) {
                System.out.println("testgetDescendants throws Exception = " + ex);
                fail("Exception in testgetDescendants " + ex.getMessage() );
            }
    }
    
    /**
     * This method will test isRetired
     *  
     */
    public void testisRetired() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String conceptCode = "C12756";
		List evsResults = new ArrayList();
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.isRetired(vocabName, conceptCode);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsQuery);
                assertTrue("size",evsResults.size()>0);
                Boolean testValue = (Boolean)evsResults.get(0);
                assertFalse("answer is false", testValue.booleanValue());
                } catch (Exception ex) {
                    System.out.println("testisRetired throws Exception = " + ex);
                    fail("Exception in testisRetired " + ex.getMessage() );
                }
    }
    
    /**
     * This method will test getHistoryRecords
     *  
     */
    public void testgetHistoryRecords_NameCode()
    {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String conceptCode = "C12756";
		List evsResults = new ArrayList();
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getHistoryRecords(vocabName, conceptCode);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsResults);
                assertTrue("size",evsResults.size()>0);
                HistoryRecord histRecord = (HistoryRecord)evsResults.get(0);
                String x = histRecord.getDescLogicConceptCode();
                assertNotNull("editDate has values", x);
                } catch (Exception ex) {
                    System.out.println("testgetHistoryRecords_NameCode throws Exception = " + ex);
                    fail("Exception in testgetHistoryRecords_NameCode " + ex.getMessage() );
                }
    }
    
    /**
     * This method will test getHistoryRecords
     *  Note, this methods fails because HistoryRecord requires conceptCode
     *  @deprecated - a concept code is required to get a HistoryRecord object
     */@Deprecated
    public void testgetHistoryRecords_NameDateDate()
    {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String initialDate = "05/25/2007";
        String finalDate = "07/27/2007";
		List evsResults = new ArrayList();
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getHistoryRecords(vocabName, initialDate,finalDate);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
//                assertNotNull("null",evsResults);
//                assertTrue("size",evsResults.size()>0);
//                HistoryRecord histRecord = (HistoryRecord)evsResults.get(0);
//                String x = histRecord.getDescLogicConceptCode();
//                assertNotNull("editDate has values", x);
                } catch (Exception ex) {
                    System.out.println("testgetHistoryRecords_NameDateDate throws Exception = " + ex);
                    fail("Exception in testgetHistoryRecords_NameDateDate " + ex.getMessage() );
                }
    }
    
    /**
     * This method will test getHistoryRecords
     *  
     */
    public void testgetHistoryRecords_NameDateDateCode()
    {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String initialDate = "05/25/2007";
        String finalDate = "07/27/2007";
        String conceptCode = "C68637";
		List evsResults = new ArrayList();
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getHistoryRecords(vocabName, initialDate,finalDate,conceptCode);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsResults);
                assertTrue("size",evsResults.size()>0);
                HistoryRecord histRecord = (HistoryRecord)evsResults.get(0);
                String x = histRecord.getDescLogicConceptCode();
                assertNotNull("editDate has values", x);
                } catch (Exception ex) {
                    System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
                    fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
                }
    }
    


    @Test
    public void testGetHistoryRecordbyCode() {
    	String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
    	String conceptCode = "C618";
    	List evsResults = new ArrayList();
    	try{
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getHistoryRecordbyCode(vocabName, conceptCode);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null",evsResults);
    		assertTrue("size",evsResults.size()>0);
    		HistoryRecord histRecord = (HistoryRecord)evsResults.get(0);
    		String x = histRecord.getDescLogicConceptCode();
    		assertNotNull("editDate has values", x);
    	} catch (Exception ex) {
    		System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
    		fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
    	}
    }

	@Test
	public void testGetHistoryStartDate() {
		List evsResults = new ArrayList();
		try{
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getHistoryStartDate(EVSQueryServiceTestCase.THES_SCHEME);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null",evsResults);
    		assertTrue("size",evsResults.size()>0);
    		Date histRecord = (Date)evsResults.get(0);
    		String x = histRecord.toString();
    		assertNotNull("editDate has values", x);
    	} catch (Exception ex) {
    		System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
    		fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
    	}
	}

	@Test
	public void testGetHistoryEndDate() {
		List evsResults = new ArrayList();
		try{
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getHistoryEndDate(EVSQueryServiceTestCase.THES_SCHEME);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null",evsResults);
    		assertTrue("size",evsResults.size()>0);
    		Date histRecord = (Date)evsResults.get(0);
    		String x = histRecord.toString();
    		assertNotNull("editDate has values", x);
    	} catch (Exception ex) {
    		System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
    		fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
    	}
	}

	@Test
	public void testGetCodeActionChildren() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C20889";
		String baselineDate = "09/29/2003";
		java.util.Date iBaselineDate_in = DLBAdapter.stringToDate(baselineDate);

		String action = "Merge";
		List evsResults = new ArrayList();
		try{
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getCodeActionChildren(vocabName, conceptCode, baselineDate, null);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null_1",evsResults);
    		assertTrue("size_1",evsResults.size()>0);
    		String histRecord = (String)evsResults.get(0);
    		assertNotNull("editDate has values 1", histRecord);
    		
    		//pass in an actual action.  Reported as error LexEVS GF#14601
    		evsQuery.getCodeActionChildren(vocabName, conceptCode, baselineDate, action);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null_2",evsResults);
    		assertTrue("size_2",evsResults.size()>0);
    		histRecord = (String)evsResults.get(0);
    		assertNotNull("editDate has values 2", histRecord);
    	} catch (Exception ex) {
    		System.out.println("testgetCodeActionChildren throws Exception = " + ex);
    		fail("Exception in testgetCodeActionChildren " + ex.getMessage() );
    	}
	}

	@Test
	public void testGetCodeActionParents() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C20889";
		String baselineDate = "09/29/2003";
		List evsResults = new ArrayList();
		try{
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getCodeActionParents(vocabName, conceptCode, baselineDate);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null",evsResults);
    		assertTrue("size",evsResults.size()>0);
    		String histRecord = (String)evsResults.get(0);
    		assertNotNull("editDate has values", histRecord);
    	} catch (Exception ex) {
    		System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
    		fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
    	}
	}

	@Test
	public void testGetHistoryCollectionbyDescLogicConceptCode() {
    	String conceptCode = "C618";
    	List evsResults = new ArrayList();
    	try{//reported as error in LexEVS GF#14602
    		EVSQuery evsQuery = new EVSQueryImpl();
    		evsQuery.getHistoryCollectionbyDescLogicConceptCode(conceptCode);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    		assertNotNull("null",evsResults);
    		assertTrue("size",evsResults.size()>0);
    		HistoryRecord histRecord = (HistoryRecord)evsResults.get(0);
    		String x = histRecord.getDescLogicConceptCode();
    		assertNotNull("editDate has values", x);
    	} catch (Exception ex) {
//    		System.out.println("testgetHistoryRecords_NameDateDateCode throws Exception = " + ex);
    		fail("Exception in testgetHistoryRecords_NameDateDateCode " + ex.getMessage() );
    	}
	}

}
