//This tests Description Logic queries


package gov.nih.nci.evs.query;

import static org.junit.Assert.fail;
import gov.nih.nci.evs.EVSQueryServiceHolder;
import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.evs.domain.DescLogicConcept;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

/**
 * The Class DLQuery.
 */
public class DLQuery extends EVSQueryServiceTestCase{
    final static String testID = "DLQuery";

    @Override
    protected String getTestID()
    {
        return testID;
    }

    @Test
	public void testContainsInverseRole() {
		
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		String role = "" ;
		String roleValue = "";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.containsInverseRole(vocabName, role, roleValue,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}


    @Test
	public void testContainsRole() {
    	
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		String role = "" ;
		String roleValue = "";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.containsRole(vocabName,role, roleValue, conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}
     @Test
	public void testFetchDTSProperties() {
		
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.fetchDTSProperties(vocabName, conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

    @Test
	public void testFetchPropertyCollectionByCodes() {
    	
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();
		List propNames = new ArrayList();
		propNames.add("Synonym");
		List conceptCodes = new ArrayList();
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.fetchPropertyCollectionByCodes(vocabName, propNames, conceptCodes);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

    @Test
	public void testFetchTermAssociations() {
    	//There are no term associations in any of our vocabularies.
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();
		List propNames = new ArrayList();
		List conceptCodes = new ArrayList();
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.fetchTermAssociations(vocabName, conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	      //There are no term associations in any of our vocabularies.
	        assertTrue("size",evsResults.size()==0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	
	}
     /**
     * This method will test getAllSilos
     *  @deprecated - evsapi4 no longer supports silos
     */@Deprecated
    public void testgetAllSilos() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getAllSilos(vocabName);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()==0);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testgetAllSilos throws Exception = " + ex);
                }
    }

    @Test
	public void testGetAssociationCollectionbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C61686"; //"Clopidogrel";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getAssociationCollectionbyCode(vocabName, conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

    /**
     * This method will test getConceptByName
     *  @deprecated
     */@Deprecated
    public void testgetConceptByName() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptByName(vocabName,conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);

            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetConceptByName throws Exception = " + ex);
            }
    }

    /**
     * This method will test getConceptCodeByName
     *
     *  @deprecated
     */@Deprecated
    public void testgetConceptCodeByName() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptCodeByName(vocabName,conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
             } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetConceptCodeByName throws Exception = " + ex);
            }
     }

    /**
     * This method will test getConceptNameByCode
     *
     *  @deprecated
     */@Deprecated
    public void testgetConceptNameByCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C618";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptNameByCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);

            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetConceptNameByCode throws Exception = " + ex);
            }
    }
     /**
     * This method will test getConceptWithPropertyMatching
     *
     */
    public void testgetConceptWithPropertyMatching_MGED() {
		String vocabName = EVSQueryServiceTestCase.MGED_SCHEME;
        String searchIn = "unique_identifier";
        String searchValue = "MO_71";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptWithPropertyMatching(vocabName, searchIn, searchValue, 10);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetConceptWithPropertyMatching_MGED throws Exception = " + ex);
            }
     }


    /**
     * This method will test getConceptWithPropertyMatching
     *
     */
    public void testgetConceptWithPropertyMatching_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String searchIn = "Synonym";
        String searchValue = "Tobacco Smoking";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptWithPropertyMatching(vocabName, searchIn, searchValue, 10);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetConceptWithPropertyMatching_NCI throws Exception = " + ex);
            }
    }

    /**
     * This method will test getConceptWithSiloMatching
     *
     *  @deprecated
     */@Deprecated
    public void testgetConceptWithSiloMatching1() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String searchTerm = "brain*";
        String siloName = "Terms";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getConceptWithSiloMatching(vocabName,searchTerm,5,siloName);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()==0);
                 } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testgetConceptWithSiloMatching1 throws Exception = " + ex);
                }
    }
    /**
     * This method will test getConceptWithSiloMatching
     *
     *  @deprecated
     */@Deprecated
    public void testgetConceptWithSiloMatching2() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String searchTerm = "Blood";
        String siloName = "Terms";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getConceptWithSiloMatching(vocabName,searchTerm,5,siloName);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()==0);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testgetConceptWithSiloMatching2 throws Exception = " + ex);
                }
    }

    /**
     * This method will test getConceptWithSiloMatching
     *
     *  @deprecated
     */@Deprecated
    public void testgetConceptWithSiloMatching3() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String searchTerm = "Blood";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.getConceptWithSiloMatching(vocabName,searchTerm,5);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()==0);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testgetConceptWithSiloMatching3 throws Exception = " + ex);
                }
    }

    /**
     * This method will test getDescLogicConceptbyCode
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptbyCode_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConcept(vocabName,conceptCode,true);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptbyCode_NCI throws Exception = " + ex);
            }
    }

    /**
     * This method will test getDescLogicConceptbyCode
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptbyCode_Snomed() {
		String vocabName = EVSQueryServiceTestCase.SNOMED_SCHEME;
		String conceptCode = "87612001"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConcept(vocabName,conceptCode,true);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptbyCode_Snomed throws Exception = " + ex);
            }
    }


    /**
     * This method will test getDescLogicConceptbyName
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptbyName_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConcept(vocabName,conceptName,false);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptbyName_NCI throws Exception = " + ex);
            }
    }

    /**
     * This method will test getDescLogicConceptbyName
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptbyName_Snomed() {
		String vocabName = EVSQueryServiceTestCase.SNOMED_SCHEME;
		String conceptName = "87612001"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConcept(vocabName,conceptName,true);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptbyName_Snomed throws Exception = " + ex);
            }
    }

    /**
     * This method will test getDescLogicConceptNameByCode
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptNameByCode_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C618";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConceptNameByCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptNameByCode_NCI throws Exception = " + ex);
            }
    }
    /**
     * This method will test getDescLogicConceptNameByCode
     *
     *  @deprecated
     */@Deprecated
    public void testgetDescLogicConceptNameByCode_Snomed() {
		String vocabName = EVSQueryServiceTestCase.SNOMED_SCHEME;
		String conceptCode = "87612001"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getDescLogicConceptNameByCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetDescLogicConceptNameByCode_Snomed throws Exception = " + ex);
            }
    }

    @Test
    //reported as error in LexEVS gf#14651
	public void testGetEdgePropertiesbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getEdgePropertiesbyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}


    @Test
	public void testGetInverseAssociationCollectionbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12909"; //"Hematopoietic system";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getInverseAssociationCollectionbyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}
    

	@Test
	public void testGetInverseRoleCollectionbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getInverseRoleCollectionbyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}

	public void testGetLocalNames()
    {
	    try {
		    EVSQuery query = new EVSQueryImpl();
		    String vocabularyName = EVSQueryServiceTestCase.THES_SCHEME;

		    query.getLocalNames(vocabularyName);

		    List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
		    assertNotNull("should not be null",evsResults);
		    assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetPropertiesByConceptCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getPropertiesByConceptCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}

	/**
     * This method will test getPropertiesByConceptName
     *
     *  @deprecated
     */@Deprecated
    public void testgetPropertiesByConceptName_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getPropertiesByConceptName(vocabName,conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
    }

	@Test
	public void testGetPropertyCollectionbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getPropertyCollectionbyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}

	@Test
	public void testGetPropertyCollectionbyName() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getPropertyCollectionbyName(conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertiesByConceptName_NCI throws Exception = " + ex);
            }
	}

	/**
     * This method will test getPropertyValues
     *
     */
    public void testgetPropertyValues() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		String property = "DEFINITION";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getPropertyValues(vocabName,conceptName,property);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertyValues throws Exception = " + ex);
            }
    }

	@Test
	public void testGetQualifierCollectionbyName() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getQualifierCollectionbyName(conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetPropertyValues throws Exception = " + ex);
            }
	}

	@Test
	public void testGetRoleCollectionbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getRoleCollectionbyCode(vocabName, conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	@Test
	public void testGetRolesByConceptCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getRolesByConceptCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	/**
     * This method will test getRolesByConceptName
     *
     *  @deprecated
     */@Deprecated
    public void testgetRolesByConceptName() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getRolesByConceptName(vocabName,conceptName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
    }


	@Test
	public void testGetSecurityTokenbyName() {
		try {
			EVSQuery evsQuery = new EVSQueryImpl();
			List evsResults = new ArrayList();
			evsQuery.getSecurityTokenbyName(THES_SCHEME);

			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
			assertNotNull("Results returned", evsResults);
			assertTrue("Results empty", evsResults.size()==0);
			
			evsQuery.getSecurityTokenbyName(MEDDRA_SCHEME);
			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
			assertNotNull("Results returned", evsResults);
			assertTrue("Results not empty", evsResults.size()>0);
		} catch (Exception e) {
			System.out.println("testValidateToken throws Exception = " + e);
		}
	}

	@Test
	public void testGetSemanticTypeVectorbyCode() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSemanticTypeVectorbyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	@Test
	public void testGetSiloCollectionbyName() {
		//there are no silos in any of the EVS vocabs
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptName = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSiloCollectionbyName(vocabName);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	@Test
	public void testGetSourcebyCode() {
		//TODO Metathesaurus method?
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSourcebyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	@Test
	public void testGetVocabularybyCode() {
		//TODO Metathesaurus method?
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C12434"; //"Blood";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getVocabularybyCode(vocabName,conceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);
            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testgetRolesByConceptName throws Exception = " + ex);
            }
	}

	/**
     * Test h l7.
     */
    public void testHL7() {
		List evsResults = new ArrayList();
		String vocabName = "HL7";
		String serType = "OID";  //OID      P111
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptWithPropertyMatching(vocabName, serType, "2.16.840.1.113883.5.6#1186-6", 100);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);

            } catch (Exception ex) {fail("Exception thrown");
                System.out.println("testHL7 throws Exception = " + ex);
            }

    }

	//This tests Meddra security token
    /**
     * Test meddra security.
     */
    public void testMeddraSecurity() {
		try {
			EVSQuery evsQuery = new EVSQueryImpl();
			List evsResults = new ArrayList();

			//evsQuery.searchDescLogicConcepts(EVSQueryServiceTestCase.MEDDRA_SCHEME,"gene*",10,0,"",1);
			evsQuery.searchDescLogicConcepts(EVSQueryServiceTestCase.MEDDRA_SCHEME,"gene*",100);
			//get the results
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("should not be null",evsResults);
	        assertTrue("should be empty", evsResults.size()==0);

			gov.nih.nci.evs.security.SecurityToken token = new gov.nih.nci.evs.security.SecurityToken();
			//make sure token is null
			token.setAccessToken(null);

	        //set bad token
			token.setAccessToken("123123123");
			//evsQuery.searchDescLogicConcepts(EVSQueryServiceTestCase.MEDDRA_SCHEME,"gene*",10,0,"",1);
			evsQuery.searchDescLogicConcepts(EVSQueryServiceTestCase.MEDDRA_SCHEME,"gene*",100);
			//get the results
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("should not be null",evsResults);
	        assertTrue("should be empty", evsResults.size()==0);

	        //now test after adding the security token
			token.setAccessToken(EVSQueryServiceTestCase.MEDDRA_TOKEN);
			evsQuery.addSecurityToken(EVSQueryServiceTestCase.MEDDRA_SCHEME,token);
			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
			assertNotNull("should not be null",evsResults);
	        assertTrue("size",evsResults.size()>0);

		}

    	catch (Exception ex)
    	{
            System.out.println("testMeddraSecurity throws Exception = " + ex);
    	}
    }

	/**
     * This method will test searchDescLogicConcepts
     *
     */
    public void testsearchDescLogicConcepts_GO() {
		String vocabName = EVSQueryServiceTestCase.GO_SCHEME;
        String searchTerm = "apoptosis*";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,10);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()>0);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testsearchDescLogicConcepts_GO throws Exception = " + ex);
                }
    }

	/**
     * This method will test searchDescLogicConcepts
     *
     */
    public void testsearchDescLogicConcepts_NCI() {
		String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
        String searchTerm = "apoptosis*";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,10);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()>0);
    	        DescLogicConcept concept = (DescLogicConcept)evsResults.get(0);
    	        assertNotNull("concept returned",concept.getCode());
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testsearchDescLogicConcepts_NCI throws Exception = " + ex);
                }
    }

	/**
     * This method will test searchDescLogicConcepts
     *
     */
    public void testsearchDescLogicConcepts_Snomed() {
		String vocabName = EVSQueryServiceTestCase.SNOMED_SCHEME;
        String searchTerm = "apoptosis";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,10);
    	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
    	        assertNotNull("null",evsResults);
    	        assertTrue("size",evsResults.size()>0);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testsearchDescLogicConcepts_Snomed throws Exception = " + ex);
                }
    }

	@Test
	public void testValidateToken() {
		try {
			EVSQuery evsQuery = new EVSQueryImpl();
			List evsResults = new ArrayList();

			gov.nih.nci.evs.security.SecurityToken token = new gov.nih.nci.evs.security.SecurityToken();
			//make sure token is null
			token.setAccessToken(null);
			evsQuery.validateToken(MEDDRA_SCHEME,token );

			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
			assertNotNull("Results returned", evsResults);
			Boolean valid = (Boolean)evsResults.get(0);
			assertFalse("token not valid", valid.booleanValue());
			
			token.setAccessToken(MEDDRA_TOKEN);
			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
			assertNotNull("Results returned", evsResults);
			valid = (Boolean)evsResults.get(0);
			assertTrue("token valid", valid.booleanValue());


		} catch (Exception e) {
			System.out.println("testValidateToken throws Exception = " + e);
		}
	}

}
