package gov.nih.nci.evs.query;

import gov.nih.nci.evs.EVSQueryServiceHolder;
import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.evs.domain.MetaThesaurusConcept;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;


/**
 * The Class MetathesaurusQuery.
 */@SuppressWarnings("deprecation")
public class MetathesaurusQuery extends EVSQueryServiceTestCase{

    /** The Constant testID. */
    final static String testID = "MetathesaurusQuery";

    /* (non-Javadoc)
     * @see gov.nih.nci.evs.EVSQueryServiceTestCase#getTestID()
     */
    @Override
    protected String getTestID()
    {
        return testID;
    }

    /**
     * Test meta sources.
     *
     * @throws Exception the exception
     */
    public  void testMetaSources() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {

    	  metaQuery.getMetaSources();
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testMetaSources : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test semantic types.
     *
     * @throws Exception the exception
     */
    public  void testSemanticTypes() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {

    	  metaQuery.getSemanticTypes();
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testSemanticTypes : " + e.toString());
        fail(e.toString());
      }
    }


    /**
     * Test meta search.
     *
     * @throws Exception the exception
     */@SuppressWarnings("unused")
    public  void testMetaSearch_overLimit() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
          metaQuery.searchMetaThesaurus("blo*", 50000, "all sources", false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);

         System.out.println(metaResults.size());
      }
      catch(Exception e)
      {
        return;
      }
      fail("exception expected in testMetaSearch_overLimit");
    }

    /**
     * Test meta search.
     *
     * @throws Exception the exception
     */
    public  void testMetaSearch_withinLimit() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
          metaQuery.searchMetaThesaurus("blood*", 400, "all sources", false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);

      }
      catch(Exception e)
      {
    	  fail("exception occured in testMetaSearch_withinLimit");
      }

    }

  /**
   * Test meddra text.
   *
   * @throws Exception the exception
   */
  public void testMeddraText() throws Exception
    {//This works if the version specific name is passed in (MDR90) currently
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchMetaThesaurus("Blood alcohol", 100, EVSQueryServiceTestCase.meddraMeta, false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testMeddraText : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test meddra code.
     *
     * @throws Exception the exception
     */
    public  void testMeddraCode() throws Exception
    {//This works if the version specific name is passed in (MDR90) currently
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchMetaThesaurus("10049127", 10, EVSQueryServiceTestCase.meddraMeta, false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
//        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testMeddraCode : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test round trip.
     *
     * @throws Exception the exception
     */
    public  void testRoundTrip() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
          EVSQuery aSearch = new EVSQueryImpl();
          aSearch.getMetaConceptNameByCode("C0006104");  //("C0699870");   //("CL051858");
          List aResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(aSearch);
          assertNotNull("null1",aResults);
          assertTrue("size1",aResults.size()>0);

          String sMeta = (String)aResults.get(0);
//          System.out.println("sMeta : " + sMeta + " :");
          metaQuery.searchMetaThesaurus(sMeta, 10, "All Sources", false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null2",metaResults);
        assertTrue("size2",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testRoundTrip : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test all sources query.
     *
     * @throws Exception the exception
     */
    public  void testAllSourcesQuery() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchMetaThesaurus("CL111010", 10, "All Sources", true, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testAllSourcesQuery : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test all sources search.
     *
     * @throws Exception the exception
     */
    public  void testAllSourcesSearch() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchMetaThesaurus("pizza", 10, "All Sources", false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testAllSourcesSearch : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test empty params.
     *
     * @throws Exception the exception
     */@SuppressWarnings("unused")
    public  void testEmptyParams() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchMetaThesaurus("", 2, "", false, false, false);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("Meta results returned", metaResults);
        assertFalse("Meta results set is empty", metaResults.size()>0);
        
      }
      catch(Exception e)
      {
        return;
      }
      
    }

    /**
     * Test source by code_ loinc.
     *
     * @throws Exception the exception
     */
    public  void testSourceByCode_LOINC() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchSourceByCode("10834-0", EVSQueryServiceTestCase.loincMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testSourceByCode_LOINC : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test search by loincid.
     *
     * @throws Exception the exception
     */
    public  void testSearchByLOINCID() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchByLoincId("10834-0", EVSQueryServiceTestCase.loincMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testSearchByLOINCID : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test source by code_ nc it.
     *
     * @throws Exception the exception
     */
    public  void testSourceByCode_NCIt() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.searchSourceByCode("C40565", EVSQueryServiceTestCase.ncitMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testSourceByCode_NCIt : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * Test concepts by source_ nc it.
     *
     * @throws Exception the exception
     * @deprecated - need to pass in concept code
     */@Deprecated
    public  void testConceptsBySource_NCIt() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getConceptsBySource(EVSQueryServiceTestCase.ncitMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
      }
      catch(Exception e)
      {
        return;
      }
      fail("Exception expected when no concept code passed in for testConceptsBySource_NCIt");
    }

    /**
     * Test concepts by source_ snomed.
     *
     * @throws Exception the exception
     */
    public  void testConceptsBySource_Snomed() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getConceptsBySource(EVSQueryServiceTestCase.snomedMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        return;
      }
      fail("Exception expected when no concept code passed in for testConceptsBySource_Snomed");

    }

    /**
     * Test get concept name by code.
     *
     * @throws Exception the exception
     */
    public void testGetConceptNameByCode() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getMetaConceptNameByCode("C0017337");

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);

        metaQuery.getMetaConceptNameByCode("C0024109");
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null2",metaResults);
        assertTrue("size2",metaResults.size()>0);

      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testGetConceptNameByCode : " + e.toString());
        fail(e.toString());
      }
    }

    /**
     * This method will test getConceptsByCategories.
     *
     * @deprecated - lexbig does not support categories
     */@Deprecated
    public void testGetConceptsByCategories1() {
		String conceptCode = "C0017337";
		String category = "Procedures";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getConceptsByCategories(conceptCode,category);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
//	        assertNotNull("null",evsResults);
//	        assertTrue("size",evsResults.size()>0);
            } catch (Exception e) {
                System.out.println("testGetConceptsByCategories1 throws Exception = " + e.toString());
                fail(e.toString());
            }
    }

    /**
     * Test get concepts by categories2.
     *
     * @throws Exception the exception
     *
     * @deprecated - lexbig does not support categories
     */@Deprecated
    public void testGetConceptsByCategories2() throws Exception
    {//Categories "Medications", "Procedures", "Laboratory", "Diagnosis"
    	//Note:  these may not be applicable for LexBIG.  Need to check with Kim.
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getConceptsByCategories("C0013990","Procedures");

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
//        assertNotNull("null",metaResults);
//        assertFalse("size",metaResults.size()>0);

        metaQuery.getConceptsByCategories("C0013990", "Diagnosis");
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
//        assertNotNull("null2",metaResults);
//        assertTrue("size2",metaResults.size()>0);

      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testGetConceptsByCategories2 : " + e.toString());
        fail(e.toString());
      }
    }



    /**
     * This method will test searchMetaThesaurus.
     */
    public void testsearchUsingAsterisk() {
		String searchTerm = "lung";
		List evsResults = new ArrayList();

//        System.out.println("*** Bug Test...");
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.searchMetaThesaurus(searchTerm, 2, "*", false, false, false);
                assertNotNull("null",evsResults);
                assertFalse("size",evsResults.size()>0);

    			evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                } catch (Exception ex) {fail("Exception thrown");
                    System.out.println("testsearchUsingAsterisk throws Exception = " + ex.toString());
                }
    }

    /**
     * This method will test searchMetaThesaurus.
     */
    public void testsearchUsingDoubleAsterisk() {
		String searchTerm = "blood*";
		List evsResults = new ArrayList();

//        System.out.println("*** Bug Test...");
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.searchMetaThesaurus(searchTerm, 100, "*", false, false, false);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null", evsResults);
                assertTrue("size",evsResults.size()>0);
                } catch (Exception e) {
                    System.out.println("testsearchUsingDoubleAsterisk throws Exception = " + e);
                    fail(e.toString());
                }
     }

    /**
     * Test search by source.
     */
    public void testsearchBySource() {
//    	ApplicationService evsService = ApplicationService.getRemoteInstance("http://cabio-stage.nci.nih.gov:80/cacore32/http/remoteService");
    	EVSQuery evsQuery = new EVSQueryImpl();
    	String termStr = "agent";
    	int iMetaLimit = 100;
    	String sMetaSource = ncitMeta;
    	try {
    		evsQuery.searchMetaThesaurus(termStr, iMetaLimit, sMetaSource, false,
					false, false);

			List metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null", metaResults);
            assertTrue("size",metaResults.size()>0);
//			System.out.println("ResultSet size is:" + metaResults.size());
			for (int i =0; i<metaResults.size(); i++)
			{//This checks that the concepts returned have valid info.  Will throw exception if fail
				MetaThesaurusConcept metaCon =(MetaThesaurusConcept)metaResults.get(i);
				if (metaCon != null)
				{
					String sConName = metaCon.getName();
					String sConID = metaCon.getCui();
					ArrayList semanticTypes = metaCon.getSemanticTypeCollection();
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail("error occured in testSearchBySource");
		}

    }
    public  void testsearchSourceByAtomCode_StringString() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
       metaQuery.searchSourceByAtomCode("C12434", EVSQueryServiceTestCase.ncitMeta);         //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);
      }
      catch(Exception e)
      {
        System.out.println("Error Occured in testsearchSourceByAtomCode_StringString : " + e.toString());
        fail(e.toString());
      }
    }

    public void testGetLocalNames()
    {
	    try {
		    EVSQuery query = new EVSQueryImpl();
		    String vocabularyName = EVSQueryServiceTestCase.META_SCHEME;

		    query.getLocalNames(vocabularyName);

		    List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
		    assertNotNull("should not be null",evsResults);
		    assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}
    

	@Test
	public void testSearchMetaThesaurus_StringIntStringBooleanBooleanBoolean() {
	    try {
		    EVSQuery query = new EVSQueryImpl();

		    query.searchMetaThesaurus("BLOOD", 10,"NCI",false,true,true);

		    List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
		    assertNotNull("should not be null",evsResults);
		    assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testSearchMetaThesaurus_String() {
	    try {
		    EVSQuery query = new EVSQueryImpl();

		    query.searchMetaThesaurus(DOMAIN_CUI);

		    List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
		    assertNotNull("should not be null",evsResults);
		    assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetMetaConceptsForAllSources() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getMetaConceptsForAllSources();

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}


	@Test
	public void testGetAtomCollectionbyCui() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getAtomCollectionbyCui(DOMAIN_CUI);

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetSynonymCollectionbyCui() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getSynonymCollectionbyCui(DOMAIN_CUI);

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetSourceCollectionbyCui() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getSourceCollectionbyCui(DOMAIN_CUI);

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetSourcebyDefinition() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getSourcebyDefinition("");

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetDefinitionCollectionbyCui() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getDefinitionCollectionbyCui(DOMAIN_CUI);

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}

	@Test
	public void testGetSemanticTypeCollectionbyCui() {
		try{
			EVSQuery query = new EVSQueryImpl();

			query.getSemanticTypeCollectionbyCui(DOMAIN_CUI);

			List evsResults = EVSQueryServiceHolder.instance().getAppService().evsSearch(query);
			assertNotNull("should not be null",evsResults);
			assertTrue("size",evsResults.size()>0);

		} catch (Exception e) {
			System.out.println("testGetLocalNames throws Exception = " + e);
		}
	}


}
