package gov.nih.nci.evs.tree;

import java.util.ArrayList;
import java.util.List;

import gov.nih.nci.evs.EVSQueryServiceHolder;
import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.evs.query.EVSQuery;
import gov.nih.nci.evs.query.EVSQueryImpl;

/**
 * The Class MetathesaurusTree.
 */
public class MetathesaurusTree extends EVSQueryServiceTestCase{

    final static String testID = "MetathesaurusTree";

    @Override
    protected String getTestID()
    {
        return testID;
    }
    
    /**
     * Test parent concept.
     * 
     * @throws Exception the exception
     * @deprecated
     */@Deprecated
    public  void testParentConcept() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getParentConcepts(EVSQueryServiceTestCase.META_SCHEME, "C0009393", true);
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testParentConcept : " + e.toString());
        fail(e.toString());
      }
    }
 
    /**
     * Test parent concept.
     * 
     * @throws Exception the exception
     */
    public  void testParentConcept2() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getParentConcepts(EVSQueryServiceTestCase.META_SCHEME, "C0009393");
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testParentConcept2 : " + e.toString());
        fail(e.toString());
      }
    }
    
    /**
     * Testget parent by code and vocab_ nci.
     * 
     * @throws Exception the exception
     */
    public  void testgetParentByCodeAndVocab_NCI() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      String conceptCode = "C0009393";// "C37927"; //"Color";
      try
      {
    	  metaQuery.getParent(conceptCode, EVSQueryServiceTestCase.ncitMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testgetParentByCodeAndVocab_NCI : " + e.toString());
        fail(e.toString());
      }
    } 
    /**
     * This method will test getParent by sourceAbbr
     *  
     */
    public void testgetParentByCodeAndVocab_Snomed() {
    	String metaConceptCode="C0017337";
    	String sourceAbbr = EVSQueryServiceTestCase.snomedMeta;
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getParent(metaConceptCode,sourceAbbr);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("testgetParentByCodeAndVocab_Snomed throws Exception = " + e);
                fail(e.toString());
            }
     }

    /**
     * Test broader by code and vocab_ mesh.
     * 
     * @throws Exception the exception
     */
    public  void testGetBroaderConcepts_StringString_MESH() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      String conceptCode="C0006087";//"D001916"
      try
      {
    	  metaQuery.getBroaderConcepts(conceptCode, EVSQueryServiceTestCase.meshMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testBroaderByCodeAndVocab_MESH : " + e.toString());
        fail(e.toString());
      }
    } 

    /**
     * This method will test getBroaderConcepts by concept code and vocab
     *  
     */
    public void testGetBroaderConcepts_StringString_Snomed() {
    	String metaConceptCode = "C1278908"; //"181216001", "Entire lung"
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getBroaderConcepts(metaConceptCode,EVSQueryServiceTestCase.snomedMeta);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      

            } catch (Exception e) {
                System.out.println("testBroaderByCodeAndVocab_Snomed throws Exception = " + e);
                fail(e.toString());
            }
    }
 
    /**
     * Test related.
     * 
     * @throws Exception the exception
     */
    public  void testRelated() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getRelatedConcepts("C0017337");
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testRelated : " + e.toString());
        fail(e.toString());
      }
    } 

    /**
     * Test related by source.
     * 
     * @throws Exception the exception
     */
    public  void testRelatedBySource() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getRelatedConcepts("C0017337", EVSQueryServiceTestCase.snomedMeta);
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testRelatedBySource : " + e.toString());
        fail(e.toString());
      }
    } 

    /**
     * Test related by source and type.
     * 
     * @throws Exception the exception
     */
    public  void testRelatedBySourceAndType() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getRelatedConcepts("C0017337", EVSQueryServiceTestCase.snomedMeta,"RN");
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testRelatedBySourceAndType : " + e.toString());
        fail(e.toString());
      }
    } 
    
    /**
     * Test child concept.
     * 
     * @throws Exception the exception
     */@SuppressWarnings("deprecation")
    public  void testGetChildConcepts_StringStringboolean() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getChildConcepts(EVSQueryServiceTestCase.META_SCHEME, "C0009393", true);
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testChildConcept : " + e.toString());
        fail(e.toString());
      }
    } 
    
    /**
     * Test child concept.
     * 
     * @throws Exception the exception
     */
    public  void testGetChildConcepts_StringString() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getChildConcepts(EVSQueryServiceTestCase.META_SCHEME, "C0009393");
        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testChildConcept2 : " + e.toString());
        fail(e.toString());
      }
    } 
    
    /**
     * This method will test getChildren by concept code
     *  
     */
    public void testGetChildren_StringString_novocab() {
		String sourceAbbr = "";
		String metaConceptCode="C0017337";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getChildren(metaConceptCode,sourceAbbr);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("testgetChildrenByCode throws Exception = " + e);
                fail(e.toString());
            }
    }
    
    /**
     * This method will test getChildren by concept code and vocab name
     *  
     */
    public void testGetChildren_StringString() {
		String sourceAbbr = "NCI";
		String metaConceptCode="C0017337";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getChildren(metaConceptCode,sourceAbbr);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("testgetChildrenByCodeAndVocab throws Exception = " + e);
                fail(e.toString());
            }
    }    
   
    /**
     * This method will test getBroaderConcepts by concept code
     *  
     */
    public void testGetBroaderConcepts_String() {
		String metaConceptCode="C0017337";
		List evsResults = new ArrayList();
//		long startTime = System.currentTimeMillis(),endTime =0;

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
//			evsQuery.getMetaSources();
			evsQuery.getBroaderConcepts(metaConceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("getBroaderConcepts throws Exception = " + e);
                fail(e.toString());
            }

    }
   
    
 
    
    /**
     * This method will test getNarrowerConcepts
     *  
     */
    public void testNarrowerByCode() {
		String metaConceptCode="C0017337";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getNarrowerConcepts(metaConceptCode);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("testNarrowerByCode throws Exception = " + e);
                fail(e.toString());
            }
    }
     /**
     * This method will test getNarrowerConcepts by sourceAbbr
     *  
     */
    public void testNarrowerByCodeAndVocab_NCI() {
    	String metaConceptCode="C0017337";
    	String sourceAbbr = EVSQueryServiceTestCase.ncitMeta;
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getNarrowerConcepts(metaConceptCode,sourceAbbr);
	        evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        assertTrue("size",evsResults.size()>0);      
            } catch (Exception e) {
                System.out.println("testNarrowerByCodeAndVocab_NCI throws Exception = " + e);
                fail(e.toString());
            }
    }
    
    /**
     * Test narrower by code and vocab_ snomed.
     * 
     * @throws Exception the exception
     */
    public  void testNarrowerByCodeAndVocab_Snomed() throws Exception
    {
      EVSQuery metaQuery = new EVSQueryImpl();
      List metaResults  = new ArrayList();
      try
      {
    	  metaQuery.getNarrowerConcepts("C0017337", EVSQueryServiceTestCase.snomedMeta);

        //get the results
        metaResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(metaQuery);
        assertNotNull("null",metaResults);
        assertTrue("size",metaResults.size()>0);      
      }      
      catch(Exception e)
      {
        System.out.println("Error Occured in testNarrowerByCodeAndVocab_Snomed : " + e.toString());
        fail(e.toString());
      }
    } 

    
    
}
