package gov.nih.nci.evs.tree;

import gov.nih.nci.evs.EVSQueryServiceHolder;
import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.evs.query.EVSQuery;
import gov.nih.nci.evs.query.EVSQueryImpl;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.junit.Test;

/**
 * The Class TreeTest.
 */
public class TreeTest extends EVSQueryServiceTestCase{

    final static String testID = "TreeTest";

    @Override
    protected String getTestID()
    {
        return testID;
    }

    /**
     * This method will test getAllSubConceptCodes
     *  
     */
    public void testgetAllSubConceptCodes() {
    	String vocabName = EVSQueryServiceTestCase.THES_SCHEME;
		String conceptCode = "C17934";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getAllSubConceptCodes(vocabName,conceptCode);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
         
            } catch (Exception e) {
                System.out.println("testgetAllSubConceptCodes throws Exception = " + e);
                fail(e.toString());
            }
    }
    

     /**
     * This method will test getParentConcepts by concept code
     *  
     */
    public void testgetParentConcepts_StringStringbool() {
		String vocabName = THES_SCHEME;
		String conceptCode = "C618";
		List evsResults = new ArrayList();

        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getParentConcepts(vocabName,conceptCode,true);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
			evsQuery.getParentConcepts(vocabName,conceptCode,false);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetParentConceptsByCode throws Exception = " + e);
                fail(e.toString());
            }
    }
    
    /**
     * This method will test getRootConcepts
     *  
     */
    public void testgetRootConcepts_NCI() {
		String vocabName = THES_SCHEME;         
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getRootConcepts(vocabName,true);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetRootConcepts throws Exception = " + e);
                fail(e.toString());
            }
            
    }
    
    
    /**
     * This method will test getRootConcepts
     *  
     */
    public void testgetRootConcepts_Snomed() {
		String vocabName = SNOMED_SCHEME;         
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getRootConcepts(vocabName,true);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetRootConcepts throws Exception = " + e);
                fail(e.toString());
            }
            
    }
    
    /**
     * This method will test getSubConcepts
     *  
     */
    public void testgetSubConcepts_StringStringBoolBool() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSubConcepts(vocabName,rootName,Boolean.FALSE,Boolean.FALSE);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
    }
    /**
     * This method will test getSuperConcepts
     *  
     */
    public void testgetSuperConcepts_StringStringBoolBool() {
		String vocabName = THES_SCHEME;
		String conceptName = "C12434";//"Blood";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSuperConcepts(vocabName,conceptName,Boolean.FALSE,Boolean.FALSE);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSuperConcepts throws Exception = " + e);
                fail(e.toString());
            }
        
    }
    
    /**
     * This method will test getTree
     *  
     */
    public void testgetTree_NCI() {
		String vocabName = THES_SCHEME;
        String conceptName = "C12727";//"Heart";
		List evsResults = new ArrayList();
		
        try {
        	
			Vector<String> v = new Vector<String>();
			v.add("Anatomic_Structure_Has_Location");
			v.add("Anatomic_Structure_is_Physical_Part_of");
			v.add("part_of");

			//When isFlag is true system exception
        	//evsQuery.getTree("NCI_Thesaurus", "Heart", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, true, 1, "4", v);
			EVSQuery evsQuery = new EVSQueryImpl();
        	evsQuery.getTree(vocabName, conceptName, true, false, 1, "4", v);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetTree_NCI throws Exception = " + e);
                fail(e.toString());
            }
         
    }
     /**
     * This method will test getTree
     *  
     */
    public void testgetTree_GO_isa() {
		String vocabName = GO_SCHEME;
        String conceptName = "GO_0003673";//"Gene_Ontology";
		List evsResults = new ArrayList();
		
        try {
        	
			Vector<String> v = new Vector<String>();
			v.add("is_a");
			v.add("part_of");

			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getTree(vocabName, conceptName, true, false, 1, "4", v);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
        } catch (Exception e) {
                System.out.println("testgetTree_GO_isa throws Exception = " + e);
                fail(e.toString());
        }
       
    }
     /**
     * This method will test getTree
     *  
     */
    public void testgetTree_GO_partof() {
		String vocabName = GO_SCHEME;
        String conceptName = "GO_0003673";//"Gene_Ontology";
		List evsResults = new ArrayList();
		
        try {
        	
			Vector<String> v = new Vector<String>();
			v.add("is-a");
			v.add("part_of");

			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getTree(vocabName, conceptName, true, true, 1, "4", v);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetTree_GO_partof throws Exception = " + e);
                fail(e.toString());
            }
            
    }
    
    /**
     * This method will test isSubConcept
     *  
     */
    public void testisSubConcept() {
		String vocabName = THES_SCHEME;
        String mainConcept = "TP73_Gene";
        String testConcept = "Apoptosis_Regulation_Gene";
		List evsResults = new ArrayList();
		
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.isSubConcept(vocabName, mainConcept, testConcept);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsResults);
                assertTrue("size",evsResults.size()>0);
                
            } catch (Exception e) {
                    System.out.println("testisSubConcept throws Exception = " + e);
                    fail(e.toString());
            }
               
    }
    /**
     * This method will test hasChildren
     *  
     */
    public void testhasChildren() {
		String vocabName = THES_SCHEME;
        String conceptName = "C13236";//"Body_Fluid_or_Substance";
		List evsResults = new ArrayList();
		
            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.hasChildren(vocabName, conceptName);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsResults);
                assertTrue("size",evsResults.size()>0);
                
                } catch (Exception e) {
                    System.out.println("testhasChildren throws Exception = " + e);
                    fail(e.toString());
                }
                
    }
    /**
     * This method will test hasParents
     *  
     */
    public void testhasParents() {
		String vocabName = THES_SCHEME;
        String conceptName = "C12434";//"Blood";
		List evsResults = new ArrayList();

            try {
    			EVSQuery evsQuery = new EVSQueryImpl();
    			evsQuery.hasParents(vocabName, conceptName);
        		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
                assertNotNull("null",evsResults);
                assertTrue("size",evsResults.size()>0);
               } catch (Exception e) {
                    System.out.println("testhasParents throws Exception = " + e);
                    fail(e.toString());
               }
               
    }
    

	@Test
	public void testGetTree_StringStringBooleanBooleanIntStringVectorBoolean() {
		String vocabName = THES_SCHEME;
        String conceptCode = "C12727";//"Heart";
		List evsResults = new ArrayList();
		
        try {
        	
			Vector<String> v = new Vector<String>();
			v.add("Anatomic_Structure_Has_Location");
			v.add("Anatomic_Structure_is_Physical_Part_of");
			v.add("part_of");

			//When isFlag is true system exception
        	//evsQuery.getTree("NCI_Thesaurus", "Heart", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, true, 1, "4", v);
			EVSQuery evsQuery = new EVSQueryImpl();
        	evsQuery.getTree(vocabName, conceptCode, true, false, 1, "3", v, true);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetTree throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetTree_StringStringBooleanBooleanIntIntVector() {
		String vocabName = THES_SCHEME;
        String conceptCode = "C12727";//"Heart";
		List evsResults = new ArrayList();
		
		try {
        	
			Vector<String> v = new Vector<String>();
			v.add("Anatomic_Structure_Has_Location");
			v.add("Anatomic_Structure_is_Physical_Part_of");
			v.add("part_of");

			//When isFlag is true system exception
        	//evsQuery.getTree("NCI_Thesaurus", "Heart", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, false, 1, "4", v);
			//evsQuery.getTree("GO", "Gene_Ontology", true, true, 1, "4", v);
			EVSQuery evsQuery = new EVSQueryImpl();
        	evsQuery.getTree(vocabName, conceptCode, true, false, 1, 3, v);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetTree throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetSubConcepts_StringString() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSubConcepts(vocabName,rootName);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetSuperConcepts_StringString() {
		String vocabName = THES_SCHEME;
		String conceptName = "C12434";//"Blood";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getSuperConcepts(vocabName,conceptName);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSuperConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetAllSubConceptNames() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getAllSubConceptNames(vocabName, rootName);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetParentConcepts_StringStringBooleanInt() {
		String vocabName = THES_SCHEME;
		String conceptName = "C12434";//"Blood";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getParentConcepts(vocabName, conceptName, false,1);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSuperConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetParentConcepts_StringStringInt() {
		String vocabName = THES_SCHEME;
		String conceptName = "C12434";//"Blood";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getParentConcepts(vocabName, conceptName, 1);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
            } catch (Exception e) {
                System.out.println("testgetSuperConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetChildConcepts_StringStringBooleanInt() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getChildConcepts(vocabName, rootName, true, 1);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            
			evsQuery.getChildConcepts(vocabName, rootName, false, 1);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);
            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetChildConcepts_StringStringInt() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getChildConcepts(vocabName, rootName, 1);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);

            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}

	@Test
	public void testGetTreeNodebyCode() {
		String vocabName = THES_SCHEME;
		String rootName = "C12219";//"Anatomic_Structure_System_or_Substance";
		List evsResults = new ArrayList();
		
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.getTreeNodebyCode(vocabName, rootName);
    		evsResults = (List)EVSQueryServiceHolder.instance().getAppService().evsSearch(evsQuery);
            assertNotNull("null",evsResults);
            assertTrue("size",evsResults.size()>0);

            } catch (Exception e) {
                System.out.println("testgetSubConcepts throws Exception = " + e);
                fail(e.toString());
            }
	}



}
