package gov.nih.nci.lexbig.ext.objects;

import static org.junit.Assert.*;

import java.util.Vector;

import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.lexbig.ext.ConceptAssociation;
import gov.nih.nci.lexbig.ext.DLBAdapter;
import gov.nih.nci.lexbig.ext.Qualifier;

import org.LexGrid.concepts.CodedEntry;
import org.junit.Test;

public class ConceptAssociationTest extends ServiceTestCase {

	ConceptAssociation setAssoc = new ConceptAssociation();
	final static String testID = "ConceptAssociationTest";
	DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();
	private String associationCode = "C192";
	CodedEntry ce = new CodedEntry();
	ConceptAssociation ca = new ConceptAssociation();
	

	public ConceptAssociationTest()
	{
		ce = adapter.getCodedEntryByCode(THES_SCHEME, THES_VERSION, THES_TAG, associationCode );
		Vector<ConceptAssociation> assocs = adapter.getConceptAssociations(ce);
		ca = assocs.firstElement();
	}
	
    /* (non-Javadoc)
     * @see gov.nih.nci.evs.EVSQueryServiceTestCase#getTestID()
     */
    @Override
    protected String getTestID()
    {
        return testID;
    }

	@Test
	public void testSetName() {
		String setName="ConceptName";
		setAssoc.setName(setName);
		String name = "";
		name = setAssoc.getName();
		assertNotNull("Name returned", name);
		assertTrue("name set",name.equals(setName));
	}

	@Test
	public void testSetValue() {
		String setValue="ConceptName";
		setAssoc.setValue(setValue);
		String value = "";
		value = setAssoc.getValue();
		assertNotNull("Name returned", value);
		assertTrue("name set",value.equals(setValue));
	}

	@Test
	public void testGetName() {
		assertNotNull("Concept association is returned",ca);
		String name = ca.getName();
		assertNotNull("Name returned", name);
		assertTrue("Name has value", name.length()>0);
	}

	@Test
	public void testGetValue() {
		String value = ca.getValue();
		assertNotNull("Value returned", value);
		assertTrue("Value has value",value.length()>0);
	}

	@Test
	public void testAddQualifier_StringString() {
		String name = "QualName";
		String value = "QualValue";
		setAssoc.addQualifier(name, value);
		
		assertTrue("Now has qualifiers", setAssoc.hasQualifier());
		Vector<Qualifier> quals = setAssoc.getQualifiers();
		assertNotNull("Qualifiers returned", quals);
		assertTrue("Qualifiers has values", quals.size()>0);
		boolean found = false;
		for (int i = 0; i< quals.size();i++)
		{
			Qualifier qual = quals.elementAt(i);
			if (qual.getName().equals(name)) found = true;
		}
		assertTrue("Qualifier name is found", found);
		
		quals = setAssoc.getQualifierCollection();
		assertNotNull("Qualifier collection returned", quals);
		assertTrue("Qualifier collection has values", quals.size()>0);
		found = false;
		for (int i = 0; i< quals.size();i++)
		{
			Qualifier qual = quals.elementAt(i);
			if (qual.getName().equals(name)) found = true;
		}
		assertTrue("Qualifier name is found in collection", found);
	}

	@Test
	public void testAddQualifier_Qualifier() {
		String name = "QualName2";
		String value = "QualValue2";
		Qualifier setQual = new Qualifier();
		setQual.setName(name);
		setQual.setValue(value);
		setAssoc.addQualifier(setQual);
		
		assertTrue("Now has qualifiers", setAssoc.hasQualifier());
		Vector<Qualifier> quals = setAssoc.getQualifiers();
		assertNotNull("Qualifiers returned", quals);
		assertTrue("Qualifiers has values", quals.size()>0);
		boolean found = false;
		for (int i = 0; i< quals.size();i++)
		{
			Qualifier qual = quals.elementAt(i);
			if (qual.getName().equals(name)) found = true;
		}
		assertTrue("Qualifier name is found", found);
		
		quals = setAssoc.getQualifierCollection();
		assertNotNull("Qualifier collection returned", quals);
		assertTrue("Qualifier collection has values", quals.size()>0);
		found = false;
		for (int i = 0; i< quals.size();i++)
		{
			Qualifier qual = quals.elementAt(i);
			if (qual.getName().equals(name)) found = true;
		}
		assertTrue("Qualifier name is found in collection", found);
	}

	@Test
	public void testGetQualifiers() {
		assertNull("No associations have qualifiers", ca.getQualifiers());
	}

	@Test
	public void testGetQualifierCollection() {
		assertNull("No associations have qualifier collections", ca.getQualifierCollection());
	}

	@Test
	public void testHasQualifier() {
		assertFalse("No associations have qualifiers yet",ca.hasQualifier());
	}

}
