package gov.nih.nci.lexbig.ext.properties;

import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.lexbig.ext.DLBAdapter;
import gov.nih.nci.lexbig.ext.PropertyType;

import java.util.Vector;

/**
 * The Class PropertiesTest.
 */
public class PropertiesTest extends ServiceTestCase{
    final static String testID = "PropertiesTest";
    //KLO, 10/19/2007 (not for by-code mode)
    //String conceptName = "Adaptor_Signaling_Protein_Gene";

    String conceptName = "C192";

    String code = "C0005767";
    String propertyname = "Semantic_Type";

    DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();
    String csName = ServiceTestCase.META_SCHEME;
    String version = ServiceTestCase.META_VERSION;

    @Override
    protected String getTestID()
    {
    	return testID;
    }

    /**
     * Test properties.
     * 
     * @throws Exception the exception
     */
    public void testProperties() throws Exception
    {

    	Boolean retval = adapter.setVocabulary(csName, version, null);
    	if (retval == null || !retval.equals(Boolean.TRUE))
    	{
    		System.out.println("Unable to connect - program aborts.");
    		return;
    	}

    }
//  These methods all test against the MetaThesaurus


    // Vector getDefinitionCollectionbyCode(String vocabularyName, String code)
    /**
     * Test getDefinitionCollectionbyCode(String vocabularyName, String code)
     */
    public void testgetDefinitionCollectionbyCode()
    {
    	//code = "C191";
    	Vector v = new Vector();
    	v = adapter.getDefinitionCollectionbyCode(csName, code);
    	assertNotNull("getDefinitionCollectionbyCode(String,String)", v);
    	assertTrue("getDefinitionCollectionbyCode(String,String) has values",v.size()>0);
    }
    //Vector getSourceCollectionbyCode(String vocabularyName, String code)
    /**
     * Test getSourceCollectionbyCode(String vocabularyName, String code)
     */
    public void testgetSourceCollectionbyCode()
    {
    	Vector v = adapter.getSourceCollectionbyCode(csName, code);
    	assertNotNull("getSourceCollectionbyCode(String,String)", v);
    	assertTrue("getSourceCollectionbyCode(String,String) has values",v.size()>0);
    }
    //Vector getSemanticTypeCollectionbyCode(String vocabularyName, String code) {
    /**
     * Test getSemanticTypeCollectionbyCode(String vocabularyName, String code)
     */
    public void testgetSemanticTypeCollectionbyCode()
    {
    	Vector v = adapter.getSemanticTypeCollectionbyCode(csName, code);
    	assertNotNull("getSemanticTypeCollectionbyCode(String,String)", v);
    	assertTrue("getSemanticTypeCollectionbyCode(String,String) has values",v.size()>0);
    }
    //Vector getPropertyCollectionbyCode(String vocabularyName, String code)
    /**
     * Test getPropertyCollectionbyCode(String vocabularyName, String code)
     */
    public void testgetPropertyCollectionbyCode()
    {
    	Vector v = adapter.getPropertyCollectionbyCode(csName, code);
    	assertNotNull("getPropertyCollectionbyCode(String,String)", v);
    	assertTrue("getPropertyCollectionbyCode(String,String) has values",v.size()>0);
    }
    //Vector getAtomCollectionbyCode(String vocabularyName, String code)
    /**
     * Test getAtomCollectionbyCode(String vocabularyName, String code)
     */
    public void testgetAtomCollectionbyCode()
    {
    	Vector v = adapter.getAtomCollectionbyCode(csName, code);
    	assertNotNull("getAtomCollectionbyCode(String,String)", v);
    	assertTrue("getAtomCollectionbyCode(String,String) has values",v.size()>0);
    }

//  These methods all test against the NCI_Thesaurus

    //adapter.setVocabulary(ServiceTestCase.THES_SCHEME);
    /**
     * Test getPropertyTypes().
     */
    public void testgetPropertyTypes()
    {try{
    	adapter = AdapterServiceHolder.instance().getDLBAdapter();
    	csName = ServiceTestCase.THES_SCHEME;
    	version = ServiceTestCase.THES_VERSION;

    	Boolean retval = adapter.setVocabulary(csName, version, null);

    	if (retval == null || !retval.equals(Boolean.TRUE))
    	{
    		System.out.println("Unable to connect - program aborts.");
    		return;
    	}

    	// Vector getPropertyTypes() // a CodingScheme
    	Vector v = adapter.getPropertyTypes();
    	assertNotNull("getPropertyTypes()", v);
    	assertTrue("getPropertyTypes() has values",v.size()>0);
    }
    catch(Exception e){
    	fail("Exception thrown :" + e.getMessage());
    }}

    //Vector getPropertiesByConceptName(String conceptname)
    /**
     * Test getPropertiesByConceptName(String conceptname)
     */
    public void testgetPropertiesByConceptName()
    {
    	Vector v = adapter.getPropertiesByConceptName(conceptName);
    	assertNotNull("getPropertiesByConceptName(String)", v);
    	assertTrue("getPropertiesByConceptName(String) has values",v.size()>0);
    }

    
    //Vector getPropertiesByConceptCode(String conceptcode)
    /**
     * Test getPropertiesByConceptName(String conceptcode)
     */
    public void testgetPropertiesByCode_Meta()
    {
    	DLBAdapter l_adapter = new DLBAdapter(AdapterServiceHolder.getEVSsvc());
    	l_adapter.setVocabulary(META_SCHEME);
    	Vector v = l_adapter.getPropertiesByCode(code);
    	assertNotNull("getPropertiesByConceptCode(String)", v);
    	assertTrue("getPropertiesByConceptCode(String) has values",v.size()>0);
    }
    
    //Vector getPropertyValues(String conceptname, String propertyname)
    /**
     * Test getPropertyValues(String conceptname, String propertyname)
     */
    public void testgetPropertyValues()
    {
    	Vector v = adapter.getPropertyValues(conceptName, "Semantic_Type");
    	assertNotNull("getPropertyValues(String)", v);
    	assertTrue("getPropertyValues(String) has values",v.size()>0);
    }
    //Vector getAllConceptPropertyTypes()
    /**
     * Test getAllConceptPropertyTypes()
     */
    public void testgetAllConceptPropertyTypes()
    {try{
    	Vector v = adapter.getAllConceptPropertyTypes();
    	assertNotNull("getAllConceptPropertyTypes()", v);
    	assertTrue("getAllConceptPropertyTypes() has values",v.size()>0);
    }
    catch(Exception e){
    	fail("Exception thrown :" + e.getMessage());
    }}
    // PropertyType[] getAllPropertyTypes()
    /**
     * Test getAllPropertyTypes()
     */
    public void testgetAllPropertyTypes()
    {try{
    	PropertyType[] properties = adapter.getAllPropertyTypes();
//  	if (properties != null)
//  	{
//  	for (int i=0; i<properties.length; i++)
//  	{
//  	PropertyType property = properties[i];
////	System.out.println("getAllPropertyTypes " + property.getName());
//  	}
//  	}
    	assertNotNull("getAllPropertyTypes()", properties);
    	assertTrue("getAllPropertyTypes() has values",properties.length>0);
    }
    catch(Exception e){
    	fail("Exception thrown :" + e.getMessage());
    }}

    // String getConceptStatus(String conceptname)
    /**
     * Test getConceptStatus(String conceptname)
     */
    public void testgetConceptStatus()
    {
    	String status = adapter.getConceptStatus(conceptName);
    	assertNotNull("getConceptStatus(String)", status);
    	assertTrue("getConceptStatus(String) has values",status.length()>0);
    }

    // Boolean hasStatus(String conceptname)
    /**
     * Test hasStatus(String conceptname)
     */
    public void testhasStatus()
    {
    	Boolean bool_obj = adapter.hasStatus(conceptName);
    	assertNotNull("hasStatus(String)", bool_obj);

    	//KLO 101907 (does not necessarily have a status.)
//  	assertTrue("hasStatus(String) has values",bool_obj);
    }

    //Vector getPropertyNames(String conceptname)
    /**
     * Test getPropertyNames(String conceptname)
     */
    public void testgetPropertyNames()
    {
    	Vector v = adapter.getPropertyNames(conceptName);
    	assertNotNull("getPropertyNames()", v);
    	assertTrue("getPropertyNames() has values",v.size()>0);
    }

    // Vector getValidPropertyNames(Vector propertynames)
    /**
     * Test getValidPropertyNames(Vector propertynames)
     */
    public void testgetValidPropertyNames()
    {try{
    	Vector<String> propertynames = new Vector<String>();
    	propertynames.add(propertyname);
    	propertynames.add("CONCEPT_NAME");
    	Vector v = adapter.getValidPropertyNames(propertynames);
    	assertNotNull("getValidPropertyNames(Vector<String>)", v);
    	assertTrue("getValidPropertyNames(Vector<String>) has values",v.size()>0);
    }
    catch(Exception e){
    	fail("Exception thrown :" + e.getMessage());
    }}
}


