/*
 * Tests Queries against the DLBAdapter
 */
package gov.nih.nci.lexbig.ext.query;

import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.lexbig.ext.DLBAdapter;

import java.util.Vector;

import org.LexGrid.concepts.CodedEntry;


/**
 * The Class QueryTest.
 */@SuppressWarnings("deprecation")
 public class QueryTest extends ServiceTestCase{
	 final static String testID = "QueryTest";
	 DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();
	 String csName = ServiceTestCase.THES_SCHEME;
	 String version = ServiceTestCase.THES_SCHEME;
	 String tag = ServiceTestCase.THES_SCHEME;
	 String metaScheme = ServiceTestCase.META_SCHEME;
	 String pattern = "Target";
	 int searchLimit = 100;
	 String superconceptCode = "C25997";
	 String subconceptCode = "C24819";
	 String cuiSearch = "C0017237";
	 String conceptForCui = "Gelatin";
	 String superconceptName = "Adaptor_Signaling_Protein_Gene";
	 String subconceptName = "FRS2_Gene";

	 @Override
	 protected String getTestID()
	 {
		 return testID;
	 }

	 //Boolean codeExists(String code)
	 public void testCodeExists_String()
	 {
		 Boolean bool_obj = adapter.codeExists(superconceptCode);
		 assertTrue("codeExists(String)", bool_obj);
	 }

	 public void testFindCodedEntriesWithPropertyMatching_StringStringStringStringVectorVectorStringint()
	 {
		 //Vector findCodedEntriesWithPropertyMatching(
		 //                    codingSchemeName,
		 //                    version,
		 //                    matchText,
		 //                    property_vec,
		 //                    restrictToProperties_vec,
		 //                    matchAlgorithm,
		 //                    maxToReturn);
		 String matchText = "Treatment";
		 int maxToReturn = 100;
		 Vector<String> property_vec = new Vector<String>();
		 property_vec.add("Semantic_Type");
		 property_vec.add("ALT_DEFINITION");
		 String displayPropertyName = "Semantic_Type";
		 Vector<String> restrictToProperties_vec = new Vector<String>();
		 restrictToProperties_vec.add("Semantic_Type");
		 restrictToProperties_vec.add("ALT_DEFINITION");
		 String matchAlgorithm = "contains";
		 try {
			 Vector v = adapter.findConceptsWithPropertyMatching(csName, matchText, property_vec, restrictToProperties_vec,
					 displayPropertyName, matchAlgorithm, searchLimit);
			 assertNotNull("findConceptsWithPropertyMatching(String, String, Vector<String>, Vector<String>, String, String, int)", v);
			 assertTrue("findConceptsWithPropertyMatching(String, String, Vector<String>, Vector<String>, String, String, int) has values", v.size()>0);
		 } catch (Exception ex) {fail("Exception thrown");
		 System.out.println("Error: Unexpected Error");
		 }
	 }

	 public void testFindCodedEntriesWithSourceCodeMatching_Meta_StringStringStringint()
	 {
		 DLBAdapter l_adapter = new DLBAdapter(AdapterServiceHolder.getEVSsvc());
		 l_adapter.setVocabulary("NCI MetaThesaurus");
//		 String code = "MTHU000096";
//		 String source = "LNC";
		 //Vector v = adapter.getPropertiesByCode("CL347198");
		 //     Vector v = adapter.findConceptsWithSourceCodeMatching(source, code, 1);
		 Vector v = l_adapter.findCodedEntriesWithSourceCodeMatching("NCI MetaThesaurus", "LNC", "MTHU000096", 1);
		 assertNotNull("findCodedEntriesWithSourceCodeMatching(String,String,String,int) vs Meta", v);
		 assertTrue("findCodedEntriesWithSourceCodeMatching has values", v.size()>0);

	 }

	 public void testFindConcept_String()
	 {
		 // CodedEntry findConcept(String conceptname)
		 CodedEntry ce = adapter.findConcept(subconceptCode);
		 assertNotNull("findConcept(String)", ce);
	 }

	 public void testFindConcept_Stringboolean()
	 {
		 // CodedEntry findConcept(String conceptname, boolean allAttributes)
		 CodedEntry ce = adapter.findConcept(subconceptCode, true);
		 assertNotNull("findConcept(String, boolean)", ce);
	 }

	 public void testFindConceptByCode_String()
	 {
		 // CodedEntry findConceptByCode(String code)
		 CodedEntry ce = adapter.findConceptByCode(superconceptCode);
		 assertNotNull("findConceptByCode(String)", ce);
	 }

	 public void testFindConceptByCode_Stringbool()
	 {
		 // CodedEntry findConceptByCode(String conceptcode, boolean allAttributes)
		 CodedEntry ce = adapter.findConceptByCode(superconceptCode, true);
		 assertNotNull("findConceptByCode(String,boolean)", ce);
	 }

	 public void testFindConceptByCode_Stringint()
	 {
		 //CodedEntry findConceptByCode(String code, int ASD_index)
		 CodedEntry ce = adapter.findConceptByCode(superconceptCode, 1);
		 assertNotNull("findConceptByCode(String, int)", ce);
	 }

	 public void testFindConceptByCode_StringString()
	 {
		 // CodedEntry findConceptByCode(String code, String displayPropertyName)
		 CodedEntry ce = adapter.findConceptByCode(superconceptCode, "Semantic_Type");
		 assertNotNull("findConceptByCode(String, String)", ce);
	 }


	 public void testFindConceptByName_Stringint()
	 {
		 // CodedEntry findConceptByName(String conceptname, int ASD_index)
		 CodedEntry ce = adapter.findConceptByName(subconceptCode, 1);
		 assertNotNull("findConceptByName(String, int)", ce);
	 }

	 public void testFindConceptByName_StringString()
	 {
		 // CodedEntry findConceptByName(String conceptname, String displayPropertyName)
		 CodedEntry ce = adapter.findConceptByName(subconceptCode, "Semantic_Type");
		 assertNotNull("findConceptByName(String, String)", ce);
	 }

	 public void testFindConceptsByCode_Vector()
	 {
		 Vector<String> u = new Vector<String>();
		 u.add(subconceptCode);
		 u.add(superconceptCode);
		 Vector v = adapter.findConceptsByCode(u);
		 assertNotNull("findConceptsByCode(Vector<CodedEntry>)", v);
		 assertTrue("findConceptsByCode(Vector<CodedEntry>) has values",v.size()>0);
	 }

	 public void testFindConceptsWithAssociationMatching_StringStringStringintStringVectorString()
	 {
//		 TODO research further to see if Association needs to be specified by code
		 //public Vector findConceptsWithAssociationMatching(String codingSchemeName,
		 //                                            String associationName,
		 //                        String matchText,
		 //                        int maxToReturn,
		 //                        String matchAlgorithm,
		 //                        Vector restricted_property_vec,
		 //                        String relationName)
		 String associationName = "Concept_In_Subset";
		 String matchText = "Structured";
		 int maxToReturn = 100;
		 String matchAlgorithm = "contains";
		 Vector restrictToProperties_vec = new Vector<String>();
		 restrictToProperties_vec.add("Synonym");
		 restrictToProperties_vec.add("DEFINITION");
		 String relationName = "associations";
		 Vector v = adapter.findConceptsWithAssociationMatching(csName, associationName, matchText, searchLimit, matchAlgorithm,
				 restrictToProperties_vec, relationName);
		 assertNotNull("findConceptsWithAssociationMatching(String,String,String, int, String, Vector<String>, String)", v);
		 assertTrue("findConceptsWithAssociationMatching(String,String,String, int, String, Vector<String>, String) has values", v.size()>0);
	 }

	 public void testFindConceptsWithDesignationMatching_StringStringbooleanVectorVectorStringint()
	 {
		 // Vector findConceptsWithDesignationMatching(
		 //                                            String matchText,
		 //                                            String displayPropertyName,
		 //                        boolean preferredOnly,
		 //                                            Vector sortByProperty_vec,
		 //                                            Vector source_vec,
		 //                                            String matchAlgorithm,
		 //                                            int maxToReturn)
		 //matchText = "Role Target";
		 String matchText = "Target";
		 String displayPropertyName = "Textual Presentation";
		 boolean preferredOnly = false;
		 int maxToReturn = 100;
		 Vector<String> sortByProperty_vec = new Vector<String>();
		 sortByProperty_vec.add("Semantic_Type");
		 Vector<String> source_vec = null;
		 String matchAlgorithm = "contains";
		 Vector v = null;
		 try {
		 	v = adapter.findConceptsWithDesignationMatching(matchText, displayPropertyName, preferredOnly, sortByProperty_vec,
				 source_vec, matchAlgorithm, searchLimit);
		 	assertNotNull("findConceptsWithDesignationMatching(String,String, boolean, Vector<String>, Vector<String>, String, int)", v);
		 	assertTrue("findConceptsWithDesignationMatching(String,String, boolean, Vector<String>, Vector<String>, String, int) has values", v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }

	 }

	 public void testFindConceptsWithInverseAssociationMatching_StringStringStringintStringVectorString()
	 {
		 //public Vector findConceptsWithInverseAssociationMatching(
		 //              String codingSchemeName,
		 //              String roleType_in,
		 //        String pattern,
		 //        int searchLimit,
		 //        String matchAlgorithm,
		 //        Vector restricted_property_vec,
		 //        String relationName)
		 String associationName = "Concept_In_Subset";
		 String matchText = "Chemotherapy";
		 int maxToReturn = 100;
		 String matchAlgorithm = "contains";
		 Vector restrictToProperties_vec = new Vector<String>();
		 restrictToProperties_vec.add("Semantic_Type");
		 restrictToProperties_vec.add("ALT_DEFINITION");
		 String relationName = "associations";

		 Vector v = null;
		 try {
			 v = adapter.findConceptsWithInverseAssociationMatching(csName, associationName, matchText, searchLimit, matchAlgorithm,
					 restrictToProperties_vec, relationName);
			 assertNotNull("findConceptsWithInverseAssociationMatching(String,String,String, int, String, Vector<String>, String)", v);
			 assertTrue("findConceptsWithInverseAssociationMatching(String,String,String, int, String, Vector<String>, String) has values", v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }

	 public void testFindConceptsWithNameMatching_Stringint()
	 {
		 //adapter.setVocabulary(ServiceTestCase.THES_SCHEME);
		 //Vector findConceptsWithNameMatching(String pattern, int searchLimit)
		 Vector v = null;
		 try {
			 v = adapter.findConceptsWithNameMatching("findConceptsWithNameMatching(" + pattern + ")", searchLimit);
			 assertNotNull("findConceptsWithNameMatching(String,int)", v);
			 assertTrue("findConceptsWithNameMatching(String,int) has values",v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }

	 // Vector findConceptsWithNameMatching(String matchText,
	 //         int maxToReturn,
	 //         boolean synonyms)
	 public void testFindConceptsWithNameMatching_Stringintboolean()
	 {
		 String matchText = "Target";
		 int maxToReturn = 100;
		 Vector v = null;
		 try {
			 v = adapter.findConceptsWithNameMatching(matchText, maxToReturn, true);
			 assertNotNull("findConceptsWithNameMatching(String,int, boolean)", v);
			 assertTrue("findConceptsWithNameMatching(String, int,boolean) has values", v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }



	 // Vector findConceptsWithNameMatching(String matchText,
	 //       int maxToReturn,
	 //       boolean synonyms
	 //       String matchAlgorithm)
	 public void testFindConceptsWithNameMatching_StringintbooleanString()
	 {
		 String matchText = "Target";
		 int maxToReturn = 100;
		 String matchAlgorithm = "contains";
		 Vector v = null;
		 try {
			 v = adapter.findConceptsWithNameMatching(matchText, maxToReturn, true, matchAlgorithm);
			 assertNotNull("findConceptsWithNameMatching(String,int, boolean, String)", v);
			 assertTrue("findConceptsWithNameMatching(String, int, boolean, String) has values", v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }


	 // Vector findConceptsWithNameMatching(String matchText,
	 //       int maxToReturn,
	 //       String matchAlgorithm)
	 public void testFindConceptsWithNameMatching_StringintString()
	 {
		 String matchText = "Target";
		 int maxToReturn = 100;
		 String matchAlgorithm = "contains";
		 Vector v = null;
		 try {
		 	v = adapter.findConceptsWithNameMatching(matchText, maxToReturn, matchAlgorithm);
			 assertNotNull("findConceptsWithNameMatching(String,int, String)", v);
			 assertTrue("findConceptsWithNameMatching(String, int, String) has values", v.size()>0);
	     } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }

	 public void testFindConceptsWithPropertyContaining_StringStringint()
	 {
		 //public Vector findConceptsWithPropertyContaining(String propertyName,
		 //        String matchText,
		 //        int searchLimit)
		 String propertyName = "ALT_DEFINITION";
		 String matchText = "Treatment";
		 int maxToReturn = 100;
		 try {
			 Vector v = adapter.findConceptsWithPropertyContaining(propertyName, matchText, searchLimit);
			 assertNotNull("findConceptsWithPropertyContaining(String,String,int)", v);
			 assertTrue("findConceptsWithPropertyContaining(String,String,int) has values", v.size()>0);
		 } catch (Exception ex) {fail("Exception thrown");
		 System.out.println("Error: Unexpected Error");
		 }

	 }

	 public void testfindConceptsWithSourceCodeMatching_StringStringStringint()
	 {
		 //public CodedEntry[] findConceptWithSourceCodeMatching(String source, String code, int maxToReturn)
		 adapter.setVocabulary(ServiceTestCase.META_SCHEME);
		 String code = "MTHU000096";
		 String source = "LNC";


		 Vector<CodedEntry> a = adapter.findConceptsWithSourceCodeMatching(META_SCHEME, source, code, searchLimit);
		 adapter.setVocabulary(csName);
		 assertNotNull("findConceptWithSourceCodeMatching(String,String,int)", a);
		 assertTrue("findConceptWithSourceCodeMatching(String,String,int) has values", a.size()>0);
	 }

	 public void testfindConceptWithSourceCodeMatching_Meta_StringStringint()
	 {
		 //public CodedEntry[] findConceptWithSourceCodeMatching(String source, String code, int maxToReturn)
		 adapter.setVocabulary(ServiceTestCase.META_SCHEME);
		 String code = "MTHU000096";
		 String source = "LNC";

		 CodedEntry[] a = adapter.findConceptWithSourceCodeMatching(source, code, searchLimit);
		 adapter.setVocabulary(csName);
		 assertNotNull("findConceptWithSourceCodeMatching(String,String,int)", a);
		 assertTrue("findConceptWithSourceCodeMatching(String,String,int) has values", a.length>0);
	 }
	 public void testGetConcept_Stringbooleanint()
	 {
		 // CodedEntry getConcept(String inputvalue,
		 CodedEntry ce = adapter.getConcept(subconceptCode, false, 1);
		 assertNotNull("getConcept(String, boolean, int)", ce);

		 ce = adapter.getConcept(subconceptCode, true, 1);
		 assertNotNull("getConcept(String, boolean, int)", ce);
	 }

	 public void testGetConcept_StringbooleanString()
	 {
		 // CodedEntry getConcept(String inputvalue,
		 CodedEntry ce = adapter.getConcept(subconceptCode, false, "Semantic_Type");
		 assertNotNull("getConcept(String, boolean, String)", ce);

		 // CodedEntry getConcept(String inputvalue,
		 ce = adapter.getConcept(subconceptCode, true, "Semantic_Type");
		 assertNotNull("getConcept(String, boolean, String)", ce);
	 }

	 // CodedEntry getConcept(String codingSchemeName, String code, boolean isCode, int ASD_index)
//	 ce = adapter.getConcept(csName, superconceptCode, true, 1);
	 public void testGetConcept_StringStringint()
	 {
		 CodedEntry ce = adapter.getConcept(csName, superconceptCode, 1);
		 assertNotNull("findConceptByCode(String, String, boolean, int)", ce);
	 }

	 public void testGetConcepts_Stringboolean()
	 {
		 // CodedEntry[] getConcepts(Vector conceptNames_in, boolean allAttributes)
		 Vector conceptNames_in = new Vector<String>();
		 conceptNames_in.add(subconceptName);
		 conceptNames_in.add(superconceptName);
		 CodedEntry[] a = adapter.getConcepts(conceptNames_in, true);
		 assertNotNull("getConcepts(String,boolean)", a);
		 assertTrue("getConcepts(String,boolean) has values", a.length>0);
	 }


	 public void testGetConcepts_Stringint()
	 {
		 //CodedEntry[] getConcepts(Vector conceptNames_in, int ASD_index)
		 Vector<String> conceptNames_in = new Vector<String>();
		 conceptNames_in.add(subconceptName);
		 conceptNames_in.add(superconceptName);
		 CodedEntry[] a = adapter.getConcepts(conceptNames_in, 1);
		 assertNotNull("getConcepts(String, int)", a);
		 assertTrue("getConcepts(String, int) has values",a.length>0);
	 }

	 public void testGetConcepts_Vector()
	 {
		 // CodedEntry[] getConcepts(Vector conceptNames_in)
		 Vector conceptNames_in = new Vector<String>();
		 conceptNames_in.add(subconceptName);
		 conceptNames_in.add(superconceptName);
		 CodedEntry[] a = adapter.getConcepts(conceptNames_in);
		 assertNotNull("getConcepts(String)", a);
		 assertTrue("getConcepts(String) has values", a.length>0);
	 }


//	 public void testSearchConcepts_StringStringintStringbooleanbooleanboolean() throws Exception
//	 {
//
////		 Boolean retval = adapter.setVocabulary(csName, version, null);
////		 if (retval == null || !retval.equals(Boolean.TRUE))
////		 {
////			 System.out.println("Unable to connect - program aborts.");
////			 return;
////		 }
//
//
//		 String source = "*";//ServiceTestCase.
//
//
//		 String associationName = "hasSubtype";
//
//		 try {
//			 CodedEntry[] a = null;
//			 try {
//				 a = adapter.searchConcepts(metaScheme, "Target", 100, source, false, false, false);
//				 assertNotNull("searchConcepts(String, String, int, String, boolean, boolean, boolean)", a);
//				 assertTrue("searchConcepts(String, String, int, String, boolean, boolean, boolean) has values",a.length>0);
//		     } catch (Exception ex) {
//				 ex.printStackTrace();
//			 }
//		 } catch (Exception e) {
//			 e.printStackTrace();
//		 }
//		 //}
//
//	 }
	 
	 /**
	  * Test searchConcepts(String, String, int, String, boolean, boolean, boolean)
	  *
	  * @throws Exception the exception
	  */
	 public void testSearchConcepts_StringStringintStringbooleanbooleanboolean()
	 {
		 //public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
		 //             boolean cui, boolean shortResult, boolean score)
		 String matchText = "Information";
		 CodedEntry [] a = null;
		 try {
		 	a = adapter.searchConcepts(THES_SCHEME, matchText, searchLimit, null, false, false, false);
		 	assertNotNull("searchConcepts(String,String,int, String, boolean,boolean,boolean)", a);
		 	assertTrue("searchConcepts(String,String,int, String, boolean,boolean,boolean) has values", a.length>0);
		 } catch (Exception e) {
			 e.printStackTrace();
		 }
	  }

	 public void testSearchConcepts_StringStringintStringbooleanbooleanbooleanint_Meta()
	 {
		 //public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
		 //           boolean cui, boolean shortResult, boolean score, int ASDIndex)

		 String source = "NCI";
		 boolean cui = true;
		 boolean shortResult = false;
		 boolean score = false;
		 int ASDindex = 0;

		 DLBAdapter l_adapter = new DLBAdapter(AdapterServiceHolder.getEVSsvc());
		 l_adapter.setVocabulary(META_SCHEME);
		 CodedEntry[] a = null;
		 try {
			 a = l_adapter.searchConcepts(META_SCHEME, cuiSearch, searchLimit,source, cui, shortResult, score, ASDindex);
			 assertNotNull("searchConcepts(String, String, int, String, boolean, boolean, boolean, int)", a);
			 assertTrue("searchConcepts(String, String, int, String, boolean, boolean, boolean, int) has values",a.length>0);
		 } catch (Exception ex) {
			 ex.printStackTrace();
		 }

		 cui = false;
		 try {
		 	a = l_adapter.searchConcepts(META_SCHEME, conceptForCui, searchLimit,source, cui, shortResult, score, ASDindex);
		 	assertNotNull("searchConcepts(String, String, int, String, boolean, boolean, boolean, int)", a);
		 	assertTrue("searchConcepts(String, String, int, String, boolean, boolean, boolean, int) has values",a.length>0);
		 } catch (Exception ex) {
			 ex.printStackTrace();
		 }
	 }
 }
