package gov.nih.nci.lexbig.ext.tree.traversal;

import java.util.Vector;

import org.LexGrid.concepts.CodedEntry;

import gov.nih.nci.evs.EVSQueryServiceTestCase;
import gov.nih.nci.lexbig.ext.DLBAdapter;
import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.client.ApplicationServiceProvider;

/**
 * The Class TreeTraversalTest.
 */
public class TreeTraversalTest extends ServiceTestCase{

	final static String testID = "TreeHierarchyTest";
	String rootCode = "C12727";  //Heart
	String superconceptCode = "C25997";//Adaptor_Signaling_Protein_Gene
	String subconceptCode = "C24819";//FRS2_Gene
	String associationName = "hasSubtype";
//	String conceptassociationname = "Concept_In_Subset";
	String roleName = "Anatomic_Structure_Is_Physical_Part_Of";
	String csName = ServiceTestCase.THES_SCHEME;
	String version = ServiceTestCase.THES_VERSION;
	Vector v = new Vector();
	boolean bool_val;
	Boolean bool_obj;
	String _service = "EvsServiceInfo";
	
	DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();

	@Override
	protected String getTestID()
	{
		return testID;
	}

	/**
	 * Test tree traversal.
	 * 
	 * @throws Exception the exception
	 */
	public void testTreeTraversal() throws Exception
	{

//		String tag = ServiceTestCase.THES_TAG;

		Boolean retval = adapter.setVocabulary(csName,
				version, null);
		if (retval == null || !retval.equals(Boolean.TRUE))
		{
			System.out.println("Unable to connect - program aborts.");
			return;
		}

	}
	public void testGetRootConcepts(){
		CodedEntry[] a = adapter.getRootConcepts();
		assertNotNull("getRootConcepts() not null", a);
		assertTrue("getRootConcepts() has values", a.length>0);
	}
	
	public void testGetRootConcepts_int(){
		// CodedEntry[] getRootConcepts(int ASD_index)
		CodedEntry[] a = adapter.getRootConcepts(1);
		assertNotNull("getRootConcepts(int)", a);
		assertTrue("getRootConcepts(int) has values", a.length>0);
	}
	
	public void testGetRootConcepts_boolean(){
		// CodedEntry[] getRootConcepts(boolean allAttributes)
		CodedEntry[] a = adapter.getRootConcepts(true);
		assertNotNull("getRootConcepts(boolean)", a);
		assertTrue("getRootConcepts(boolean) has values", a.length>0);
	}
	
	
		//Boolean areCodesRelated(
		//            String code1, // source
		//            String code2, // target
		//            String associationName)
	public void testAreCodesReleated(){
		Boolean bool_obj = adapter.areCodesRelated(subconceptCode, superconceptCode, associationName );
		assertTrue("areCodesRelated(String, String, String)", bool_obj);
	}

	// Vector findRelatedCodes(String code,
	//                          String relation,
	//                          boolean direction)
public void testFindRelatedCodes(){
		Vector v = adapter.findRelatedCodes(superconceptCode, "hasSubtype", true);
		assertNotNull("findRelatedCodes(String,String,boolean)", v);
		assertTrue("findRelatedCodes(String,String,boolean) has values", v.size()>0);
}

public void testGetRelation(){
		// String getRelation(String codingSchemeName, String associationName)
		String s = adapter.getRelation(csName, "Concept_In_Subset");
		assertNotNull("getRelation(String,String)", s);
		assertTrue("getRelation(String,String) has values", s.length()>0);

		s = adapter.getRelation(csName, "EO_Disease_Has_Associated_Cell_Type");
		assertNotNull("getRelation(String,String) 2", s);
		assertTrue("getRelation(String,String) 2 has values", s.length()>0);
}

public void testGetRelatedConcepts_StringString(){
		v = adapter.getRelatedConcepts(rootCode, roleName);
		assertNotNull("getRelatedConcepts(String,String)", v);
		assertTrue("getRelatedConcepts(String,String) has values", v.size()>0);
	}



	public void testHasSubconcepts(){
		// boolean hasSubconcepts(String conceptname)
		bool_val = adapter.hasSubconcepts(superconceptCode);
		assertTrue("hasSubconcepts(String)", bool_val);
	}
	
	public void testHasChildren(){
		// Boolean hasChildren(String conceptname)
		bool_obj = adapter.hasChildren(superconceptCode);
		assertTrue("hasChildren(String)", bool_obj);
	}
	
	public void testIsSubconcept(){
		bool_val = adapter.isSubConcept(subconceptCode, superconceptCode);
		assertTrue("isSubConcept(String,String)", bool_val);

		bool_val = adapter.isSubConcept(superconceptCode, subconceptCode);
		assertFalse("hasSubconcepts(String, String) false values", bool_val);
	}
	
	public void testIsSubconceptByCode() {

		bool_val = adapter.isSubConceptByCode(subconceptCode, superconceptCode);
		assertTrue("isSubConceptByCode(String, String)", bool_val);

		bool_val = adapter.isSubConceptByCode(superconceptCode, subconceptCode);
		assertFalse("isSubConceptByCode(String, String) false values", bool_val);

		bool_obj = adapter.isSubConceptByCode(subconceptCode, superconceptCode);
		assertTrue("isSubConceptByCode(String,String)", bool_obj);
	}
	
	public void testIsChildByCode() {
		bool_obj = adapter.isChildByCode(subconceptCode,  superconceptCode);
		assertTrue("isChildByCode(String,String) false values", bool_obj);
	}
	
	public void testGetAllSubConceptNames(){
		// Vector getAllSubConceptNames(java.lang.String conceptName)
		v = adapter.getAllSubConceptNames(superconceptCode);
		assertNotNull("getAllSubConceptNames(String)", v);
		assertTrue("getAllSubConceptNames(String) has values", v.size() > 0);
	}
	
	public void testGetSubconcepts(){
		// Vector getSubConcepts(String inputvalue, Boolean inputcodeObj, Boolean outputcodeObj)
		v = adapter.getSubConcepts(superconceptCode, Boolean.TRUE, Boolean.TRUE);
		assertNotNull("getSubConcepts(String,Boolean,Boolean)", v);
		assertTrue("getSubConcepts(String,Boolean,Boolean) has values", v.size() > 0);

		// Vector getSubConcepts(String inputvalue, boolean inputcode, boolean outputcode)
		v = adapter.getSubConcepts(superconceptCode, true, true);
		assertNotNull("getSubConcepts(String,boolean,boolean)", v);
		assertTrue("getSubConcepts(String,boolean,boolean) has values", v.size() > 0);
	}
	
	public void testHasSuperconcepts_String(){
		//boolean hasSuperconcepts(String conceptname)
		bool_val = adapter.hasSuperconcepts(subconceptCode);
		assertTrue("hasSuperconcepts(String)", bool_val);
	}
	
	public void testHasSuperconcepts_Stringbool(){
		// boolean hasSuperconcepts(String inputvalue, boolean isCode)
		bool_val = adapter.hasSuperconcepts(subconceptCode, true);
		assertTrue("hasSuperconcepts(String,boolean)", bool_val);
	}
	
	public void testHasParents(){
		// Boolean hasParents(String conceptname)
		bool_obj = adapter.hasParents(subconceptCode);
		assertTrue("hasParents(String)", bool_obj);
		
		// Boolean hasParents(String inputvalue, boolean isCode)
		bool_obj = adapter.hasParents(subconceptCode, false);
		assertTrue("hasParents(String,boolean)", bool_obj);
	}


    public void testGetSuperconcepts(){
		// Vector getSuperConcepts(String inputvalue, Boolean inputcodeObj, Boolean outputcodeObj)
		v = adapter.getSuperConcepts(subconceptCode, Boolean.TRUE, Boolean.TRUE);
		assertNotNull("getSuperConcepts(String,Boolean,Boolean)", v);
		assertTrue("getSuperConcepts(String,Boolean,Boolean) has values", v.size() > 0);

		// Vector getSuperConcepts(String inputvalue, boolean inputcode, boolean outputcode)
		v = adapter.getSuperConcepts(subconceptCode, true, true);
		assertNotNull("getSuperConcepts(String,boolean,boolean)", v);
		assertTrue("getSuperConcepts(String,boolean,boolean) has values", v.size() > 0);
    }
    
    public void testIsSubconceptByName(){
		//Boolean isSubConceptByName(String concept1,
		bool_obj = adapter.isSubConceptByName(subconceptCode, superconceptCode);
		assertTrue("isSubConceptByName(String,String)", bool_obj);
    }
    
    public void testGetConceptChildRoots(){
		//CodedEntry[] getConceptChildRoots()
		CodedEntry[] a = adapter.getConceptChildRoots();
		assertNotNull("getConceptChildRoots()", a);
		assertTrue("getConceptChildRoots() has values",a.length>0);
    }
    
    public void testGetConceptChildRoots_int(){
		//CodedEntry[] getConceptChildRoots()
		CodedEntry[] a = adapter.getConceptChildRoots(1);
		assertNotNull("getConceptChildRoots(int)", a);
		assertTrue("getConceptChildRoots(int) has values",a.length>0);
    }   
    
    public void testGetConceptChildRoots_intStringString()
    {	//CodedEntry[] getConceptChildRoots()
		CodedEntry[] a = adapter.getConceptChildRoots(1, "roles", "hasSubtype");
		assertNotNull("getConceptChildRoots(int,String,String)", a);
		assertTrue("getConceptChildRoots(int,String,String) has values",a.length>0);
    }
    public void testGetDirectSubs_String(){
		// CodedEntry[] getDirectSubs(String conceptname)
    	CodedEntry[] a = adapter.getDirectSubs(superconceptCode);
		assertNotNull("getDirectSubs(String)", a);
		assertTrue("getDirectSubs(String) has values",a.length>0);

    }
    public void testGetDirectSubs_Stringint(){

    	 //CodedEntry[] getDirectSubs(String conceptname, int ASD_index)
    	CodedEntry[] a = adapter.getDirectSubs(superconceptCode, 1);
		assertNotNull("getDirectSubs(String,int)", a);
		assertTrue("getDirectSubs(String,int) has values",a.length>0);

    }
    public void testGetDirectSubs_Stringboolean(){

		// CodedEntry[] getDirectSubs(String inputvalue,boolean)
    	CodedEntry[] a = adapter.getDirectSubs(superconceptCode, true);
		assertNotNull("getDirectSubs(String,boolean)", a);
		assertTrue("getDirectSubs(String,boolean) has values",a.length>0);
    }
    
    public void testGetDirectSubs_Stringbooleanint(){

		// CodedEntry[] getDirectSubs(String inputvalue,boolean)
    	CodedEntry[] a = adapter.getDirectSubs(superconceptCode, true,1);
		assertNotNull("getDirectSubs(String,boolean)", a);
		assertTrue("getDirectSubs(String,boolean) has values",a.length>0);
    }
   
    public void testGetDirectSups_Stringbool(){
		//CodedEntry[] getDirectSups(String conceptname)
    	CodedEntry[] a = adapter.getDirectSups(subconceptCode, true);
		assertNotNull("getDirectSups(String,boolean)", a);
		assertTrue("getDirectSups(String,boolean) has values",a.length>0);

    }
    public void testGetDirectSups_Stringint(){
		//CodedEntry[] getDirectSups(String conceptname, int ASD_index)
    	CodedEntry[] a = adapter.getDirectSups(subconceptCode, 1);
		assertNotNull("getDirectSups(String,int)", a);
		assertTrue("getDirectSups(String,int) has values",a.length>0);
    }
    public void testGetDirectSups_Stringboolint(){
		//CodedEntry[] getDirectSups(String conceptname, int ASD_index)
    	CodedEntry[] a = adapter.getDirectSups(subconceptCode, true,1);
		assertNotNull("getDirectSups(String,boolean)", a);
		assertTrue("getDirectSups(String,boolean) has values",a.length>0);

    }
    public void testGetDirectSups_String(){
		//CodedEntry[] getDirectSups(String conceptname)
    	CodedEntry[] a = adapter.getDirectSups(subconceptCode);
		assertNotNull("getDirectSups(String,int)", a);
		assertTrue("getDirectSups(String,int) has values",a.length>0);
    } 
    
    public void testIsChild(){
		bool_obj = adapter.isChild(superconceptCode, subconceptCode);
		assertFalse("isChild(String,String) false values", bool_obj);
		
		bool_obj = adapter.isChild(subconceptCode, superconceptCode);
		assertTrue("isChild(String,String) false values", bool_obj);
	}


    /**
     * Test parent and child_ zebrafish.
     */
    public void testParentAndChild_Zebrafish()
    {
		//Test Hierarchy in Zebrafish

		// Need to pass in both coding scheme name and version to switch to a new ontology;
		String codingScheme = ServiceTestCase.ZEBRAFISH_SCHEME;
		String version = ServiceTestCase.ZEBRAFISH_VERSION;
		DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();


		adapter.setVocabulary(codingScheme, version, null);

		String code = "ZFA_0000010";//cardiovascular_system

		//Boolean hasChildren(String codingScheme, String code);
		Boolean bool_obj = adapter.hasChildren(codingScheme, code);
		assertTrue("hasChildren(String,String)", bool_obj);

		//Boolean hasParents(String codingScheme, String code)
		bool_obj = adapter.hasParents(codingScheme, code);
		assertTrue("hasParents(String,String)", bool_obj);
	}
    
    /**
     * Test parent and child_ meta thesaurus.
     */
    public void testParentAndChild_MetaThesaurus()
    {
		String codingScheme = ServiceTestCase.META_SCHEME;
		String version = ServiceTestCase.META_VERSION;
		DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();


		adapter.setVocabulary(codingScheme, version, null);
		String code = "C0475628";//

		//Boolean hasChildren(String codingScheme, String code);
		Boolean bool_obj = adapter.hasChildren(codingScheme, code);
		assertTrue("hasChildren(String,String)", bool_obj);

		//Boolean hasParents(String codingScheme, String code)
		bool_obj = adapter.hasParents(codingScheme, code);
		assertTrue("hasParents(String,String)", bool_obj);

    }
    
    /**
     * Test get related concepts.
     */
    public void testGetRelatedConcepts_StringbooleanStringString()
    {
    	String conceptCode ="C0009393";// "C37927";
    	try {
    		EVSApplicationService evsSvc =  (EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(serviceUrl, _service);

    		DLBAdapter ladapter = new DLBAdapter(evsSvc);
    		ladapter.setVocabulary(ServiceTestCase.META_SCHEME);
    		Vector<CodedEntry> v = ladapter.getRelatedConcepts(conceptCode, false, DLBAdapter.DEFAULT_ASSOC_FWD, ServiceTestCase.ncitMeta);
    		assertNotNull("getRelatedConcepts("+conceptCode+","+ncitMeta+") is not null", v);
    		assertTrue("getRelatedConcepts("+conceptCode+","+ncitMeta+") has values", v.size()>0);

    	}
    	catch (Exception e){
    		fail("Exception thrown :" + e.getMessage());
    	}
    }
    
}
