package gov.nih.nci.lexbig.ext.utils;

import static org.junit.Assert.fail;
import junit.framework.TestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.lexbig.ext.DLBAdapter;
import gov.nih.nci.lexbig.ext.NavigationUtils;
import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.NavigationUtils;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.evs.domain.MetaThesaurusConcept;


import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;

import java.util.HashMap;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.commonTypes.Source;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Presentation;


import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;


public class NavigationUtilsTest extends ServiceTestCase {

	final static String testID = "NavigationUtilsTest";
	NavigationUtils navUtils = new NavigationUtils();
	CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
	DLBAdapter adapter = AdapterServiceHolder.instance().getDLBAdapter();
	
    /* (non-Javadoc)
     * @see gov.nih.nci.evs.EVSQueryServiceTestCase#getTestID()
     */
    @Override
    protected String getTestID()
    {
        return testID;
    }
	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.NavigationUtils()'
	 */
	public void testNavigationUtils() {
    	csvt.setTag("PRODUCTION");
    	csvt.setVersion(META_VERSION);
	}


    public void testGetRootNodes() throws LBException  {


	    try {
		    ResolvedConceptReferenceList rcrl = NavigationUtils.getRootNodes(AdapterServiceHolder.instance().getEVSsvc(), ncitMeta, META_SCHEME,  null);
		    assertNotNull("null",rcrl);
		    ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
		    assertTrue("Has entries",rcra.length>20);
//            int j = 0;
//			for (int i=0; i<rcra.length; i++)
//			{
//				 ResolvedConceptReference rcr = rcra[i];
//				 CodedEntry ce = rcr.getReferencedEntry();
//				 j = i + 1;
//				 System.out.println("\t(" + j + ") " + ce.getConceptCode() + " " + ce.getEntityDescription().getContent());
//			}

		} catch (Exception ex) {
//			ex.printStackTrace();
			fail("Exception thrown");
		}
	}


	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getChildNodes(LexBIGService, String, String, CodingSchemeVersionOrTag, String)'
	 */
	public void testGetChildNodes() {
		String code = "C0344722";
	    HashMap hm = NavigationUtils.getChildNodes(AdapterServiceHolder.instance().getEVSsvc(), META_SCHEME, ncitMeta, csvt, code);
	    assertNotNull("null", hm);
	    assertTrue("Has values", hm.size() > 0);
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getParentNodes(LexBIGService, String, String, CodingSchemeVersionOrTag, String)'
	 */
	public void testGetParentNodes() {
		String code = "C0344722";
	    HashMap hm = NavigationUtils.getParentNodes(AdapterServiceHolder.instance().getEVSsvc(), META_SCHEME, ncitMeta, csvt, code);
	    assertNotNull("null", hm);
	    assertTrue("Has values", hm.size() > 0);
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getAssociatedNodes(LexBIGService, String, String, CodingSchemeVersionOrTag, String, boolean)'
	 *
	@SuppressWarnings("static-access")
	public void testGetAssociatedNodes_LexBIGServiceStringStringCSVTStringboolean()
	{
		NavigationUtils navUtils = new NavigationUtils();
		LexBIGService lbSvc = AdapterServiceHolder.instance().getEVSsvc();
		String version = META_VERSION;
		String sab = ncitMeta;
		String scheme = META_SCHEME;
		String code = "C0344722";
		CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
		csvt.setVersion(version);

        try {
			HashSet hset = new HashSet();
			HashMap hmap = navUtils.getAssociatedNodes(lbSvc,
											  scheme,
											  null,
											  csvt,
											  code, // Abnormality of atrial septum
											  true); //true = get children

			ResolvedConceptReferenceList list = new ResolvedConceptReferenceList();
			Iterator it = hmap.keySet().iterator();
			int j = 0;
			while (it.hasNext())
			{
				String associationNm = (String) it.next();
				System.out.println("associationNm: " + associationNm);

				ResolvedConceptReferenceList rcrl = (ResolvedConceptReferenceList) hmap.get(associationNm);
				for (Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference(); nodes.hasNext(); )
				{
					ResolvedConceptReference ref = nodes.next();
					String conceptcode = ref.getConceptCode();
					if (!hset.contains(conceptcode))
					{
						list.addResolvedConceptReference(ref);
						j++;
						System.out.println("\t(" + j + ") " + conceptcode + " " + ref.getEntityDescription().getContent());
					}
				}
			}



			hset = new HashSet();
			hmap = navUtils.getAssociatedNodes(lbSvc,
									  scheme,
									  null,
									  csvt,
									  code, // Abnormality of atrial septum
									  false);

			list = new ResolvedConceptReferenceList();
			it = hmap.keySet().iterator();
			j = 0;
			while (it.hasNext())
			{
				String associationNm = (String) it.next();
				System.out.println("associationNm: " + associationNm);

				ResolvedConceptReferenceList rcrl = (ResolvedConceptReferenceList) hmap.get(associationNm);
				for (Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference(); nodes.hasNext(); )
				{
					ResolvedConceptReference ref = nodes.next();
					String conceptcode = ref.getConceptCode();
					if (!hset.contains(conceptcode))
					{
						list.addResolvedConceptReference(ref);
						j++;
						System.out.println("\t(" + j + ") " + conceptcode + " " + ref.getEntityDescription().getContent());
					}
				}
			}



		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	*/

	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringBoolean_Meta_true()
	{
//		NavigationUtils navUtils = new NavigationUtils();
//		CodingSchemeVersionOrTag csvortag = new CodingSchemeVersionOrTag();

		ResolvedConceptReferenceList rcrl = NavigationUtils.getAssociatedNodes(AdapterServiceHolder.getEVSsvc(), META_SCHEME, ncitMeta, null, DOMAIN_CUI,"", true); //true = get children
        assertNotNull("null",rcrl);
        assertTrue("size",rcrl.getResolvedConceptReferenceCount()>0);
        
        //TODO write test that passes in an actual association
	}

	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringBoolean_Meta_false()
	{
//		NavigationUtils navUtils = new NavigationUtils();
		CodingSchemeVersionOrTag csvortag = new CodingSchemeVersionOrTag();
		csvortag.setTag("PRODUCTION");
		csvortag.setVersion(META_VERSION);


		ResolvedConceptReferenceList rcrl = NavigationUtils.getAssociatedNodes(AdapterServiceHolder.getEVSsvc(), META_SCHEME, ncitMeta, csvortag, DOMAIN_CUI,"", false); //true = get children
        assertNotNull("null",rcrl);
        assertTrue("size",rcrl.getResolvedConceptReferenceCount()>0);
        
      //TODO write test that passes in an actual association
	}

	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringBoolean_NCIT_true()
	{
//		NavigationUtils navUtils = new NavigationUtils();
		CodingSchemeVersionOrTag csvortag = new CodingSchemeVersionOrTag();
		

		ResolvedConceptReferenceList rcrl= NavigationUtils.getAssociatedNodes(AdapterServiceHolder.getEVSsvc(), THES_SCHEME, null, csvortag, "C12434","", true); //true = get children
        assertNotNull("null",rcrl);
        assertTrue("size",rcrl.getResolvedConceptReferenceCount()>0);
        
      //TODO write test that passes in an actual association
	}
	
	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringARRBoolean_NCIT_true()
	{
//		NavigationUtils navUtils = new NavigationUtils();
		CodingSchemeVersionOrTag csvortag = new CodingSchemeVersionOrTag();
		String[] assocs;

		ResolvedConceptReferenceList rcrl= NavigationUtils.getAssociatedNodes(AdapterServiceHolder.getEVSsvc(), THES_SCHEME, null, csvortag, "C12434","", true); //true = get children
        assertNotNull("null",rcrl);
        assertTrue("size",rcrl.getResolvedConceptReferenceCount()>0);
        
      //TODO write test that passes in an actual association array
	}
	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getChildrenBySource(LexBIGService, LexBIGServiceConvenienceMethods, String, String, String, String, String, CodingSchemeVersionOrTag, boolean)'
	 */
	public void testGetChildrenBySource() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getParentsBySource(LexBIGService, LexBIGServiceConvenienceMethods, String, String, String, String, String, CodingSchemeVersionOrTag, boolean)'
	 */
	public void testGetParentsBySource() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.isBlank(String)'
	 */
	public void testIsBlank() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.isNotBlank(String)'
	 */
	public void testIsNotBlank() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getTopNodes(LexBIGService, String, CodingSchemeVersionOrTag, String, String, String)'
	 */
	public void testGetTopNodes() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getEndNodes(LexBIGService, String, CodingSchemeVersionOrTag, String, String, String)'
	 */
	public void testGetEndNodes() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.openPrintWriter(String)'
	 */
	public void testOpenPrintWriter() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.closeWriter(PrintWriter)'
	 */
	public void testCloseWriter() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getAssociationSourceBySAB(LexBIGService, LexBIGServiceConvenienceMethods, String, String, String, String, String, CodingSchemeVersionOrTag, boolean)'
	 */
	public void testGetAssociationSourceBySAB() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getAssociationTargetBySAB(LexBIGService, LexBIGServiceConvenienceMethods, String, String, String, String, String, CodingSchemeVersionOrTag, boolean)'
	 */
	public void testGetAssociationTargetBySAB() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getAssociatedNodes(LexBIGService, String, String, CodingSchemeVersionOrTag, String, String[], boolean)'
	 */
	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringStringArrayBoolean() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getAssociatedNodes(LexBIGService, String, String, CodingSchemeVersionOrTag, String, String, boolean)'
	 */
	public void testGetAssociatedNodes_LexBIGServiceStringStringCodingSchemeVersionOrTagStringStringBoolean() {
		fail("Not yet implemented"); // TODO
	}

	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.NavigationUtils.getNavigationDirection(LexBIGService, String, CodingSchemeVersionOrTag, String)'
	 */
	public void testGetNavigationDirection() {
		fail("Not yet implemented"); // TODO
	}

}
