package gov.nih.nci.lexbig.ext.utils;

import static org.junit.Assert.fail;
import junit.framework.TestCase;
import gov.nih.nci.lexbig.ext.AdapterServiceHolder;
import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.NavigationUtils;
import gov.nih.nci.lexbig.ext.SearchUtil;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.evs.domain.MetaThesaurusConcept;

import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import gov.nih.nci.system.applicationservice.EVSApplicationService;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;

import java.util.HashMap;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.commonTypes.Source;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Presentation;

import org.LexGrid.LexBIG.Utility.Iterators.ResolvedConceptReferencesIterator;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class SearchUtilTest extends ServiceTestCase {

	public static final int SEARCH_STARTS_WITH = 1;
	public static final int SEARCH_ENDS_WITH = 2;
	public static final int SEARCH_CONTAINS = 3;
	public static final int SEARCH_EXACT_MATCH = 4;

	private LexBIGService lbSvc = null;
	private String url = serviceUrl;
	private String codingSchemeName = THES_SCHEME;
	private String version = THES_VERSION;
	private String tag = THES_TAG;

	final static String testID = "SearchUtilTest";

    /* (non-Javadoc)
     * @see gov.nih.nci.evs.EVSQueryServiceTestCase#getTestID()
     */
    @Override
    protected String getTestID()
    {
        return testID;
    }
	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.SearchUtil'
	 */


	public SearchUtilTest()
	{

	}


/***************************************************************
 ** searchNodesForName(java.util.List,int,int,boolean,boolean)
 ***************************************************************/

     public void testSearchNodesForName_Listintintbooleanboolean() {//List names, int count, int algorithm, boolean soundsLike, boolean includeObsolete){

		 try {
			 ArrayList names = new ArrayList();
			 int algorithm = SEARCH_STARTS_WITH;
			 int count = 1000;
			 boolean soundsLike = false;
			 boolean includeObsolete = false;
			 //GPX1 Gene
			 names.add("GPX1");

			 SearchUtil searchUtil = new SearchUtil(codingSchemeName, version, url);
			 ResolvedConceptReferencesIterator matchIterator = null;
			 matchIterator = searchUtil.searchNodesForName(names, count, algorithm, soundsLike, includeObsolete);

			 if (matchIterator != null)
			 {
				 try {
					 int maxToReturn = 1000;

					 ResolvedConceptReferenceList rcrl = matchIterator.next(maxToReturn);
					 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
					 for (int i=0; i<rcra.length; i++)
					 {
						 ResolvedConceptReference rcr = rcra[i];
						 CodedEntry ce = rcr.getReferencedEntry();
						 int j=i+1;
						 System.out.println("\t(" + j + ") " + ce.getConceptCode() + " (" + ce.getEntityDescription().getContent() + ")");
					 }
					 assertTrue("correct source found in results", rcra.length > 0);

				 } catch (Exception e) {
					 e.printStackTrace();
			 	 }
			 }
			 else
			 {
				 System.out.println("\nNo matches.");
			 }
		 } catch (Exception ex) {
              ex.printStackTrace();
		 }
	 }



/***************************************************************
 ** searchNodesForName(java.util.List,int,boolean,boolean,java.lang.String,java.lang.String,java.lang.String,java.lang.String)
 ***************************************************************/


   public void testSearchNodesForName_ListintbooleanbooleanStringStringString() {/*List names,@SuppressWarnings("unused") int count,
	    boolean soundsLike, boolean searchInactive,
	    String searchSource, String roleToSearchIn, String associationToSearchIn){*/

        ArrayList names = new ArrayList();
        names.add("Cell");
        names.add("Aging");

        int count = 1000;
        boolean soundsLike = false;
        boolean searchInactive = false;
        String searchSource = null;
        String roleToSearchIn = "Gene_Plays_Role_In_Process";//"Anatomic_Structure_Has_Location";
        String associationToSearchIn = null;

        SearchUtil searchUtil = new SearchUtil(codingSchemeName, version, url);
        ResolvedConceptReferencesIterator rcri = searchUtil.searchNodesForName(names, count, soundsLike, searchInactive,
               searchSource, roleToSearchIn, associationToSearchIn);

		if (rcri != null)
		{
			 try {
				 int maxToReturn = 1000;

				 ResolvedConceptReferenceList rcrl = rcri.next(maxToReturn);
				 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
				 for (int i=0; i<rcra.length; i++)
				 {
					 ResolvedConceptReference rcr = rcra[i];
					 CodedEntry ce = rcr.getReferencedEntry();
					 int j=i+1;
					 System.out.println("\t" + "(" + j + ") " + ce.getConceptCode() + " (" + ce.getEntityDescription().getContent() + ")");
				 }
				 assertTrue("correct source found in results", rcra.length > 0);
			 } catch (Exception e) {
				 e.printStackTrace();
			 }
		}
		else
		{
			System.out.println("\nNo matches.");
		}
	}




/***************************************************************
 ** searchNodesForDefinition(java.util.List,int,boolean,boolean)
 ***************************************************************/

	public void testSearchNodesForDefinition() {//List names, int count, boolean soundsLike, boolean searchInactive){

		 try {
			 ArrayList names = new ArrayList();
			 String criteria = "Cavity";
			 int count = 10;
			 boolean soundsLike = false;
			 boolean includeObsolete = false;
			 boolean searchInactive = false;
			 names.add(criteria);
			 SearchUtil searchUtil = new SearchUtil(codingSchemeName, version, url);
			 ResolvedConceptReferencesIterator matchIterator = null;
             matchIterator = searchUtil.searchNodesForDefinition(names, count, soundsLike, searchInactive);
			 if (matchIterator != null)
			 {
				 try {
					 int maxToReturn = 20;

					 ResolvedConceptReferenceList rcrl = matchIterator.next(maxToReturn);
					 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
					 for (int i=0; i<rcra.length; i++)
					 {
						 ResolvedConceptReference rcr = rcra[i];
						 CodedEntry ce = rcr.getReferencedEntry();
						 int j=i+1;
						 System.out.println("\t(" + j + ") " + ce.getConceptCode() + " (" + ce.getEntityDescription().getContent() + ")");
					 }
					 assertTrue("correct source found in results", rcra.length > 0);
				 } catch (Exception e) {
					 e.printStackTrace();
			 	 }
			 }
			 else
			 {
				 System.out.println("\nNo matches.");
			 }
		 } catch (Exception ex) {
              ex.printStackTrace();
		 }
	}


/***************************************************************
 ** searchNodesForAssociations(java.util.List,int,boolean,boolean,java.lang.String)
 ***************************************************************/

	public void testSearchNodesForAssociations() {//List names, int count,boolean soundsLike,
		//boolean searchInactive, String searchSource, String roleToSearchIn, String associationToSearchIn){

        ArrayList names = new ArrayList();
        names.add("Cell");
        names.add("Aging");

        int count = 1000;
        boolean soundsLike = false;
        boolean searchInactive = false;
        String searchSource = null;

        String roleToSearchIn = "Gene_Plays_Role_In_Process";//"Anatomic_Structure_Has_Location";
        String associationToSearchIn = null;

        SearchUtil searchUtil = new SearchUtil(codingSchemeName, version, url);
        ResolvedConceptReferencesIterator rcri = searchUtil.searchNodesForAssociations(names, count, soundsLike, searchInactive, roleToSearchIn);

		if (rcri != null)
		{
			 try {
				 int maxToReturn = 1000;

				 ResolvedConceptReferenceList rcrl = rcri.next(maxToReturn);
				 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
				 for (int i=0; i<rcra.length; i++)
				 {
					 ResolvedConceptReference rcr = rcra[i];
					 CodedEntry ce = rcr.getReferencedEntry();
					 int j=i+1;
					 System.out.println("\t(" + j + ") " + ce.getConceptCode() + " (" + ce.getEntityDescription().getContent() + ")");
				 }
				 assertTrue("correct source found in results", rcra.length > 0);
			 } catch (Exception e) {
				 e.printStackTrace();
			 }
		}
		else
		{
			System.out.println("\nNo matches.");
		}
	}


/***************************************************************
 ** searchNodesForProperties(java.util.List,int,boolean,boolean,java.lang.String,java.lang.String,java.lang.String)
 ***************************************************************/

	public void testSearchNodesForProperties_ListintbooleanbooleanStringStringString() { /*List names,
											  @SuppressWarnings("unused") int count,
											  boolean soundsLike,
											  boolean searchInactive,
											  String searchSource,
											  String roleToSearchIn,
											  String associationToSearchIn){*/

        ArrayList names = new ArrayList();
        names.add("Cell");
        names.add("Aging");

        int count = 1000;
        boolean soundsLike = false;
        boolean searchInactive = false;
        String searchSource = null;

        String roleToSearchIn = "Gene_Plays_Role_In_Process";//"Anatomic_Structure_Has_Location";
        String associationToSearchIn = null;

        SearchUtil searchUtil = new SearchUtil(codingSchemeName, version, url);
        ResolvedConceptReferencesIterator rcri = searchUtil.searchNodesForProperties(names, count, soundsLike, searchInactive, searchSource, roleToSearchIn, associationToSearchIn);

		if (rcri != null)
		{
			 try {
				 int maxToReturn = 1000;
				 ResolvedConceptReferenceList rcrl = rcri.next(maxToReturn);
				 ResolvedConceptReference[] rcra = rcrl.getResolvedConceptReference();
				 for (int i=0; i<rcra.length; i++)
				 {
					 ResolvedConceptReference rcr = rcra[i];
					 CodedEntry ce = rcr.getReferencedEntry();
					 int j=i+1;
					 System.out.println("\t(" + j + ") " + ce.getConceptCode() + " (" + ce.getEntityDescription().getContent() + ")");
				 }
				 assertTrue("correct source found in results", rcra.length > 0);
			 } catch (Exception e) {
				 e.printStackTrace();
			 }
		}
		else
		{
			System.out.println("\nNo matches.");
		}
	}
	public void testCreateLexBIGService() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetSupportedAssociations() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetCodingScheme() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testRestrictToSource() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetPropertyForCodingSchemeSearch() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testVector2LocalNameList() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForDefinitionListBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForPropertiesListIntBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForPropertiesListBooleanBooleanStringStringString() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForPropertiesListIntBooleanBooleanStringStringString() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForPropertiesListIntStringArrayBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForPropertiesListIntIntStringArrayBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForNameListIntBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetNext() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testDump_matches() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesStringStringStringStringIntStringBooleanBooleanBoolean() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesStringStringStringStringString() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testMatchSynonyms() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testMatchSpell() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testMatchTermCompletion() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testMatchWordCompletion() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testMatchSubquery() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetReferenceWeight() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testGetTextWeight() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testCreateConceptReferenceList() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testToWords() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testRun() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testSearchNodesForNameListIntBooleanBooleanStringStringStringString() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	public void testFindCodedEntriesWithSourceCodeMatching() {
		fail("Not yet implemented"); // TODO
	}

}

