package gov.nih.nci.lexbig.ext.utils;

import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.lexbig.ext.SortUtils;
import gov.nih.nci.lexbig.ext.TreeNode;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.LexGrid.commonTypes.EntityDescription;
import org.LexGrid.concepts.CodedEntry;

public class SortUtilsTest extends ServiceTestCase {

	final static String testID = "SortUtilsTest";

    /* (non-Javadoc)
     * @see gov.nih.nci.evs.EVSQueryServiceTestCase#getTestID()
     */
    @Override
    protected String getTestID()
    {
        return testID;
    }
	/*
	 * Test method for 'gov.nih.nci.lexbig.ext.SearchUtil'
	 */


	public SortUtilsTest()
	{

	}

    private static String getKey(Object c)
    {
        return getKey(c, SortUtils.SORT_BY_NAME);
    }


    private static String getKey(Object c, int sort_option)
    {
	    if (c instanceof CodedEntry)
	    {
			CodedEntry concept = (CodedEntry) c;
			if (sort_option == SortUtils.SORT_BY_NAME) return concept.getEntityDescription().getContent();
			return concept.getConceptCode();
		}
	    else if (c instanceof TreeNode)
	    {
			TreeNode treenode = (TreeNode) c;
			return treenode.getName();
		}
	    else if (c instanceof String)
	    {
			return (String) c;
		}
	    return "NA";
    }



   private boolean validateQuickSort(List A)
   {
	   if (A.size()<=1) return true;
	   for (int i=0; i<A.size()-1; i++)
	   {
		   String key_1 = getKey(A.get(i));
		   String key_2 = getKey(A.get(i+1));
		   if (key_1.compareToIgnoreCase(key_2) > 0) return false;
	   }
	   return true;
   }


   private boolean validateQuickSort(Vector v)
   {
	   if (v.size()<=1) return true;
	   for (int i=0; i<v.size()-1; i++)
	   {
		   String key_1 = (String) v.elementAt(i);
		   String key_2 = (String) v.elementAt(i+1);
		   if (key_1.compareToIgnoreCase(key_2) > 0) return false;
	   }
	   return true;
   }

   private boolean validateQuickSort_ByCode(List A)
   {
	   if (A.size()<=1) return true;
	   for (int i=0; i<A.size()-1; i++)
	   {
		   String key_1 = getKey(A.get(i),SortUtils.SORT_BY_CODE);
		   String key_2 = getKey(A.get(i+1),SortUtils.SORT_BY_CODE);
		   if (key_1.compareToIgnoreCase(key_2) > 0) return false;
	   }
	   return true;
   }
/***************************************************************
 ** quickSort(List A)
 ***************************************************************/


   public void testQuickSort_List()
   {
	   Vector v = new Vector();
	   v.add("D");
	   v.add("B");
	   v.add("C");
	   v.add("A");
//	   System.out.println("\n\nSort CodedEntry:");
	   List A = new ArrayList();
	   for (int i=0; i<v.size(); i++)
	   {
		   String name = (String) v.elementAt(i);
           CodedEntry ce = new CodedEntry();
           EntityDescription entityDescription = new EntityDescription();
           entityDescription.setContent(name);
           ce.setEntityDescription(entityDescription);
           String code = "C000" + i;
           ce.setConceptCode(code);
           A.add(ce);
	   }

	   for (int i=0; i<A.size(); i++)
	   {
		   CodedEntry ce = (CodedEntry) A.get(i);
//		   System.out.println("Before sort: " + ce.getEntityDescription().getContent() + " code: " + ce.getConceptCode());
	   }
//	   System.out.println("\n\n");
	   SortUtils.quickSort(A);

	   for (int i=0; i<A.size(); i++)
	   {
		   CodedEntry ce = (CodedEntry) A.get(i);
//		   System.out.println("After sort by name (default): " + ce.getEntityDescription().getContent() + " code: " + ce.getConceptCode());
	   }
	   assertTrue("testQuickSort_List: ", validateQuickSort(A));

//	   System.out.println("\n\nSort TreeNode:");
	   A = new ArrayList();
	   for (int i=0; i<v.size(); i++)
	   {
		   String name = (String) v.elementAt(i);
           TreeNode node = new TreeNode();
           node.setName(name);
           A.add(node);
	   }

	   for (int i=0; i<A.size(); i++)
	   {
		   TreeNode node = (TreeNode) A.get(i);
//		   System.out.println("Before sort: " + node.getName());
	   }
//	   System.out.println("\n\n");
	   SortUtils.quickSort(A);

	   for (int i=0; i<A.size(); i++)
	   {
		   TreeNode node = (TreeNode) A.get(i);
//		   System.out.println("After sort: " + node.getName());
	   }

	   assertTrue("testQuickSort_List: ", validateQuickSort(A));

   }


/***************************************************************
 ** quickSort(List A, int sort_option)
 ***************************************************************/


   public void testQuickSort_Listint()
   {
	   Vector v = new Vector();
	   v.add("D");
	   v.add("B");
	   v.add("C");
	   v.add("A");
//	   System.out.println("\n\nSort CodedEntry:");
	   List A = new ArrayList();
	   for (int i=0; i<v.size(); i++)
	   {
		   String name = (String) v.elementAt(i);
           CodedEntry ce = new CodedEntry();
           EntityDescription entityDescription = new EntityDescription();
           entityDescription.setContent(name);
           ce.setEntityDescription(entityDescription);
           String code = "C000" + i;
           ce.setConceptCode(code);
           A.add(ce);
	   }

//	   System.out.println("\n\n");
	   SortUtils.quickSort(A, SortUtils.SORT_BY_CODE);
	   for (int i=0; i<A.size(); i++)
	   {
		   CodedEntry ce = (CodedEntry) A.get(i);
//		   System.out.println("After sort by code: " + ce.getEntityDescription().getContent() + " code: " + ce.getConceptCode());
	   }
	   assertTrue("testQuickSort_Listint: ", validateQuickSort_ByCode(A));

//	   System.out.println("\n\n");
	   SortUtils.quickSort(A, SortUtils.SORT_BY_NAME);
	   for (int i=0; i<A.size(); i++)
	   {
		   CodedEntry ce = (CodedEntry) A.get(i);
//		   System.out.println("After sort by name: " + ce.getEntityDescription().getContent() + " code: " + ce.getConceptCode());
	   }
	   assertTrue("testQuickSort_Listint: ", validateQuickSort(A));


//	   System.out.println("\n\nSort TreeNode:");
	   A = new ArrayList();
	   for (int i=0; i<v.size(); i++)
	   {
		   String name = (String) v.elementAt(i);
           TreeNode node = new TreeNode();
           node.setName(name);
           A.add(node);
	   }

//	   System.out.println("\n\n");
	   SortUtils.quickSort(A, SortUtils.SORT_BY_CODE);
	   for (int i=0; i<A.size(); i++)
	   {
		   TreeNode node = (TreeNode) A.get(i);
//		   System.out.println("After sort: " + node.getName());
	   }
	   assertTrue("testQuickSort_Listint: ", validateQuickSort(A));

//	   System.out.println("\n\n");
	   SortUtils.quickSort(A, SortUtils.SORT_BY_NAME);
	   for (int i=0; i<A.size(); i++)
	   {
		   TreeNode node = (TreeNode) A.get(i);
//		   System.out.println("After sort: " + node.getName());
	   }
	   assertTrue("testQuickSort_Listint: ", validateQuickSort(A));
   }


/***************************************************************
 ** Vector quickSort(Vector v)
 ***************************************************************/

   public void testQuickSort_Vector()
   {
	   Vector v = new Vector();
	   v.add("A");
	   v.add("C");
	   v.add("B");
	   v.add("D");
//	   System.out.println("\n\nSort String:");
	   for (int i=0; i<v.size(); i++)
	   {
	   	   String s = (String) v.elementAt(i);
//		   System.out.println("Before sort: (" + i + ") " + s);
	   }
//	   System.out.println("\n\n");
	   Vector w = SortUtils.quickSort(v);
	   for (int i=0; i<w.size(); i++)
	   {
	       String s = (String) w.elementAt(i);
//	       System.out.println("After sort: (" + i + ") " + s);
	   }

	   assertTrue("testQuickSort_Vector: ", validateQuickSort(w));

   }


/***************************************************************
 ** Vector quickSort(Vector v, int sort_option)
 ***************************************************************/

   public void testQuickSort_Vector_Int()
   {
	   Vector v = new Vector();
	   v.add("A");
	   v.add("C");
	   v.add("B");
	   v.add("D");
//	   System.out.println("\n\nSort String:");
	   for (int i=0; i<v.size(); i++)
	   {
	   	   String s = (String) v.elementAt(i);
//		   System.out.println("Before sort: (" + i + ") " + s);
	   }
//	   System.out.println("\n\n");
	   Vector w = SortUtils.quickSort(v, SortUtils.SORT_BY_CODE);
	   for (int i=0; i<w.size(); i++)
	   {
	       String s = (String) w.elementAt(i);
//	       System.out.println("After sort: (" + i + ") " + s);
	   }

	   assertTrue("testQuickSort_Vector_Int: ", validateQuickSort(w));

   }

   
}




