package gov.nih.nci.performance;

import gov.nih.nci.ServiceTestCase;
import gov.nih.nci.evs.query.EVSQuery;
import gov.nih.nci.evs.query.EVSQueryImpl;

import java.util.ArrayList;
import java.util.List;

import org.LexGrid.LexBIG.DataModel.Collections.LocalNameList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.SortOptionList;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.SearchDesignationOption;
import org.LexGrid.LexBIG.Utility.Constructors;

/**
 * The Class SearchQuery.
 */
public class SearchQuery extends ServiceTestCase{
   
    final static String testID = "SearchQuery";
    private Integer sizeReturned = new Integer(50);
	
    protected String getTestID()
    {
        return testID;
    }

    /**
     * Test search nc i_initializer_oncogene.
     */
    public void testSearchNCI_initializer_oncogene()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "oncogene";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    }
    

    /**
     * Test search nc i_initializer_american.
     */
    public void testSearchNCI_initializer_american()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "american";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    }
    
    /**
     * Test search nc i_evsquery1_ blood.
     */
    public void testSearchNCI_evsquery1_Blood()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    }
    
    /**
     * Test search nc i_evsquery2_ blood.
     */
    public void testSearchNCI_evsquery2_Blood()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    } 
    
    /**
     * Test search nc i_dlb_ blood.
     */
    public void testSearchNCI_dlb_Blood()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getDLBInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
    		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
    	}
    	catch (Exception e) {
    		fail("Exception");
    	}

    }
    
    /**
     * Test search nc i_lbs_ blood.
     */
    public void testSearchNCI_lbs_Blood()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getLexBIGInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
    		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;    	}
    	catch (Exception e) {
    		fail("Exception");
    	}
    }
    

    /**
     * Test search meta_evsquery1_ blood.
     */
    public void testSearchMeta_evsquery1_Blood()
    {
    	String vocabName = ServiceTestCase.META_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
    }
    
    /**
     * Test search meta_evsquery2_ blood.
     */
    public void testSearchMeta_evsquery2_Blood()
    {
    	String vocabName = ServiceTestCase.META_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
    }
    
    /**
     * Test search meta_dlb_ blood.
     */
    public void testSearchMeta_dlb_Blood()
    {
    	String vocabName = ServiceTestCase.META_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getDLBInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
      		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;    	
    	}
    	catch (Exception e) {
    		fail("Exception");
    	}

    }
    
    /**
     * Test search meta_lbs_ blood.
     */
    public void testSearchMeta_lbs_Blood()
    {
    	String vocabName = ServiceTestCase.META_SCHEME;
    	String searchTerm = "Blood";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getLexBIGInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
      		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;    	
    	}
    	catch (Exception e) {
    		fail("Exception");
    	}
    }
     
    /**
     * Test search nc i_evsquery1_set2_gene.
     */
    public void testSearchNCI_evsquery1_set2_gene()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "gene";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    }
    
    /**
     * Test search nc i_evsquery2_set2_gene.
     */
    public void testSearchNCI_evsquery2_set2_gene()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "gene";
    	int limit = 50;
    	List evsResults = new ArrayList();
    	//evsquery search
        try {
			EVSQuery evsQuery = new EVSQueryImpl();
			evsQuery.searchDescLogicConcepts(vocabName,searchTerm,limit);
	        evsResults = (List)PerformanceServiceHolder.instance().getQueryInstance().evsSearch(evsQuery);
	        assertNotNull("null",evsResults);
	        Integer size = new Integer(evsResults.size());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
            } catch (Exception ex) {
            	fail("Exception");
            }
         
    } 
    
    /**
     * Test search nc i_dlb_set2_gene.
     */
    public void testSearchNCI_dlb_set2_gene()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "gene";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getDLBInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
    		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;
    	}
    	catch (Exception e) {
    		fail("Exception");
    	}

    }
    
    /**
     * Test search nc i_lbs_set2_gene.
     */
    public void testSearchNCI_lbs_set2_gene()
    {
    	String vocabName = ServiceTestCase.THES_SCHEME;
    	String searchTerm = "gene";
    	int limit = 50;
    	CodedNodeSet cns;
    	try {
    		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
    		cns = PerformanceServiceHolder.instance().getLexBIGInstance().getCodingSchemeConcepts(vocabName, versionOrTag);
    		cns.restrictToMatchingDesignations(searchTerm, SearchDesignationOption.PREFERRED_ONLY, "contains", null);
    		LocalNameList restrictToProperties = new LocalNameList();
    		SortOptionList sortCriteria =
    			Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
    		ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
    				restrictToProperties,
    				null,
    				limit);
    		assertNotNull("null",list);
    		Integer size = new Integer(list.getResolvedConceptReferenceCount());
	        assertTrue("size",size>0);	        
	        assertTrue(size.toString() +" <> "+ sizeReturned.toString(),size.intValue()==sizeReturned.intValue());
	        sizeReturned = size;    	}
    	catch (Exception e) {
    		fail("Exception");
    	}
    }
    
    
   
    


}
