/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl;

import junit.framework.TestCase;

import org.LexGrid.LexBIG.DataModel.Collections.NameAndValueList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Core.AssociatedConcept;
import org.LexGrid.LexBIG.DataModel.Core.Association;
import org.LexGrid.LexBIG.DataModel.Core.ConceptReference;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Impl.testUtility.ServiceHolder;
import org.LexGrid.LexBIG.Impl.LexBIGServiceTestCase;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeGraph;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.LexBIG.Utility.ConvenienceMethods;


/**
 * JUnit test cases for the coded node graph.
 *
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.11 $ checked in on $Date: 2008/05/29 17:08:39 $
 */
public class CodedNodeGraphImplTest extends TestCase
{

//    public void testAreCodesRelated() throws LBException
//    {
//        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
//        
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
//        
//        // A couple of non-transitive tests
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", null), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is technically, illegal.  But its what everyone is going to do... so I bent some rules.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "Automobiles"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is technically illegal as well... bent the rules the same as above.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:11.11.0.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is how hasSubtype is supposed to be specified, if they wanted to fully specify it
//        //If you don't know where this oid came from, you can see why I bent the rules...
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), true).booleanValue());
//
//        
//        //those 4 tests should all work with directOnly set to false as well...
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", null), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is technically, illegal.  But its what everyone is going to do... so I bent some rules.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "Automobiles"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is technically illegal as well... bent the rules the same as above.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:11.11.0.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is how hasSubtype is supposed to be specified, if they wanted to fully specify it
//        //If you don't know where this oid came from, you can see why I bent the rules...
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1"), false).booleanValue());
//
//        
//        //now test something that requires transitivity
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", null), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is technically, illegal.  But its what everyone is going to do... so I bent some rules.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "Automobiles"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is technically illegal as well... bent the rules the same as above.
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:11.11.0.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), false).booleanValue());
//        
//        //This is how hasSubtype is supposed to be specified, if they wanted to fully specify it
//        //If you don't know where this oid came from, you can see why I bent the rules...
//        assertTrue(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), false).booleanValue());
//
//        //and with direct only set to true, they should fail
//       assertFalse(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", null), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is technically, illegal.  But its what everyone is going to do... so I bent some rules.
//        assertFalse(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "Automobiles"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is technically illegal as well... bent the rules the same as above.
//        assertFalse(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:11.11.0.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), true).booleanValue());
//        
//        //This is how hasSubtype is supposed to be specified, if they wanted to fully specify it
//        //If you don't know where this oid came from, you can see why I bent the rules...
//        assertFalse(cng.areCodesRelated(Constructors.createNameAndValue("hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"), Constructors.createConceptReference("005", "Automobiles"), Constructors.createConceptReference("Jaguar", "urn:oid:11.11.0.1"), true).booleanValue());
//    }
    
    /**
 * Test intersection.
 * 
 * @throws LBException the LB exception
 */
public void testIntersection() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
        ConvenienceMethods cm = new ConvenienceMethods(lbsi);
        
        assertNotNull("ConvenienceMethods returned", cm);
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
        CodedNodeGraph cng = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");
        
        NameAndValueList nv = Constructors.createNameAndValueList("hasSubtype");
        //create a graph that contains everything....
//        cng = cng.restrictToAssociations(Constructors.createNameAndValueList("hasSubtype"), null);
        cng = cng.restrictToAssociations(nv, null);
        cng = cng.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C25377"}, LexBIGServiceTestCase.THES_SCHEME, null));
//      ResolvedConceptReferenceList rcrl_Temp0 = cng.resolveAsList(Constructors.createConceptReference("C25377",LexBIGServiceTestCase.THES_SCHEME), true, false, -1, -1, null, null, null, 0);//appearance
//      ResolvedConceptReference[] rcr_Temp0 = rcrl_Temp0.getResolvedConceptReference();

        //create a graph that contains 005 -> Ford2
//        CodedNodeGraph cng2 = lbsi.getNodeGraph("Automobiles", null, "Relations");
//        cng2 = cng2.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"005"}, "Automobiles", null));
//        cng2 = cng2.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"Ford"}, "Automobiles", null));

        CodedNodeGraph cng2 = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");
        cng2 = cng2.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C37927","C25377","C54453","C48326"}, LexBIGServiceTestCase.THES_SCHEME, null));//color
//        cng2 = cng2.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"C48329","C48326"}, LexBIGServiceTestCase.THES_SCHEME, null));//green, red
//        cng2 = cng2.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C25377"}, LexBIGServiceTestCase.THES_SCHEME, null));//appearance
//        cng2 = cng2.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"C37927"}, LexBIGServiceTestCase.THES_SCHEME, null));
        //intersect them
        
//        ResolvedConceptReferenceList rcrl_Temp = cng2.resolveAsList(null, true, false, -1, -1, null, null, null, 50);
//        ResolvedConceptReference[] rcr_Temp = rcrl_Temp.getResolvedConceptReference();
        cng.intersect(cng2);
        
        //result should only contain 005 -> Ford
        //result should be 005 -> Ford -> Jaguar 
        
        ResolvedConceptReferenceList rcrl = cng.resolveAsList(null, true, false, -1, -1, null, null, null, 50);
        ResolvedConceptReference[] rcr = rcrl.getResolvedConceptReference();
        
        assertNotNull("0",rcr);
//        int len = rcr.length;
        assertTrue("1",rcr.length == 1);
        //top node
//        assertTrue(rcr[0].getConceptCode().equals("005"));
        assertTrue("2",rcr[0].getConceptCode().equals("C25377"));

        //no uplink
        assertTrue("3",rcr[0].getTargetOf() == null ||  rcr[0].getTargetOf().getAssociation().length == 0);
        
        //1 down link
        Association[] assn = rcr[0].getSourceOf().getAssociation();
        assertTrue("4",assn.length == 1);
        AssociatedConcept[] ac = assn[0].getAssociatedConcepts().getAssociatedConcept();
        assertTrue("5",ac.length == 1);
//        assertTrue(ac[0].getConceptCode().equals("Ford"));
        assertTrue("6",ac[0].getConceptCode().equals("C37927"));
        //no more below it 
        assertTrue("7",ac[0].getSourceOf() == null);
    }
    
    /**
     * Test is code in graph.
     * 
     * @throws LBException the LB exception
     */
    public void testIsCodeInGraph() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
        ConvenienceMethods cm = new ConvenienceMethods(lbsi);
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
        CodedNodeGraph cng = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");
         
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("Ford", "Automobiles")).booleanValue());
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("Ford", "urn:oid:11.11.0.1")).booleanValue());
        assertTrue("1",cng.isCodeInGraph(Constructors.createConceptReference("C12434", LexBIGServiceTestCase.THES_SCHEME)).booleanValue());//Blood
        assertTrue("2",cng.isCodeInGraph(Constructors.createConceptReference("C12434", LexBIGServiceTestCase.THES_URN)).booleanValue());

        //this is an odd local name we assigned to automobiles.
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("Ford", "SomeOtherValue")).booleanValue());
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("Jaguar", "Automobiles")).booleanValue());
        assertTrue("3",cng.isCodeInGraph(Constructors.createConceptReference("C12727", LexBIGServiceTestCase.THES_LOCAL)).booleanValue());//Heart
        assertTrue("4",cng.isCodeInGraph(Constructors.createConceptReference("C12905", LexBIGServiceTestCase.THES_LOCAL_NSDI)).booleanValue());//Thoracic Cavity
        
//        cng = cng.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"005"}, "Automobiles", null));
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("Ford", "Automobiles")).booleanValue());
        //this will be false because of the restrict to source codes...
//        assertFalse(cng.isCodeInGraph(Constructors.createConceptReference("Jaguar", "Automobiles")).booleanValue());
        cng = cng.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C12727"}, LexBIGServiceTestCase.THES_LOCAL, null));//Heart
        assertTrue("5",cng.isCodeInGraph(Constructors.createConceptReference("C12727", LexBIGServiceTestCase.THES_LOCAL)).booleanValue());
        //this will be false because of the restrict to source codes...
  //      assertFalse("6",cng.isCodeInGraph(Constructors.createConceptReference("C13018", LexBIGServiceTestCase.THES_LOCAL)).booleanValue());//Organ
        
//        cng = cng.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"GM"}, "Automobiles", null));
//        assertFalse(cng.isCodeInGraph(Constructors.createConceptReference("Ford", "Automobiles")).booleanValue());
//        assertTrue(cng.isCodeInGraph(Constructors.createConceptReference("GM", "Automobiles")).booleanValue());
        cng = cng.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"C12905"}, LexBIGServiceTestCase.THES_SCHEME, null));
        assertFalse("7",cng.isCodeInGraph(Constructors.createConceptReference("C13018", LexBIGServiceTestCase.THES_SCHEME)).booleanValue());
        assertTrue("8",cng.isCodeInGraph(Constructors.createConceptReference("C12905", LexBIGServiceTestCase.THES_SCHEME)).booleanValue());
    }
    
    /**
     * Test list code relationships.
     * 
     * @throws LBException the LB exception
     */
    public void testListCodeRelationships() throws  LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
        CodedNodeGraph cng = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");

        ConceptReference[] cr = cng.listCodeRelationships(Constructors.createConceptReference("C13018",LexBIGServiceTestCase.THES_SCHEME), Constructors.createConceptReference("C12727",LexBIGServiceTestCase.THES_SCHEME), true).getConceptReference();
//        ConceptReference[] cr = cng.listCodeRelationships(Constructors.createConceptReference("C13018",LexBIGServiceTestCase.THES_SCHEME), null, true).getConceptReference();
        assertTrue("1",cr.length == 1);
        //Yes, has subtype should come back with the global oid for hasSubtype.
        assertTrue("2",contains(cr, "hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"));
        
        //transitive flag switched
//        cr = cng.listCodeRelationships(Constructors.createConceptReference("005","Automobiles"), Constructors.createConceptReference("Ford","Automobiles"), true).getConceptReference();
        cr = cng.listCodeRelationships(Constructors.createConceptReference("C13018",LexBIGServiceTestCase.THES_SCHEME), Constructors.createConceptReference("C12727",LexBIGServiceTestCase.THES_SCHEME), true).getConceptReference();

        assertTrue("3",cr.length == 1);
        //Yes, has subtype should come back with the global oid for hasSubtype.
        assertTrue("4",contains(cr, "hasSubtype", "urn:oid:1.3.6.1.4.1.2114.108.1.8.1"));
        
        cr = cng.listCodeRelationships(Constructors.createConceptReference("C12727",LexBIGServiceTestCase.THES_SCHEME), Constructors.createConceptReference("C12905",LexBIGServiceTestCase.THES_SCHEME), false).getConceptReference();

        assertTrue("5",cr.length == 1);
        //Yes, has subtype should come back with the global oid for hasSubtype.
        assertTrue("6",contains(cr, "Anatomic_Structure_Has_Location", "http://ncicb.nci.nih.gov/xml/owl/EVS/Thesaurus.owl#R81"));
        
        //transitive flag switched
        cr = cng.listCodeRelationships(Constructors.createConceptReference("C12727",LexBIGServiceTestCase.THES_SCHEME), Constructors.createConceptReference("C12799",LexBIGServiceTestCase.THES_SCHEME), true).getConceptReference();

        assertTrue("7",cr.length == 0);

    }
    
    /**
     * Test union.
     * 
     * @throws LBException the LB exception
     */
    public void testUnion() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
        ConvenienceMethods cm = new ConvenienceMethods(lbsi);
        
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
        CodedNodeGraph cng = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");
        
        //create a graph that contains 005 -> Ford
        cng = cng.restrictToAssociations(Constructors.createNameAndValueList("hasSubtype"), null);
//        cng = cng.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"005"}, "Automobiles", null));
//        cng = cng.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"Ford"}, "Automobiles", null));
        cng = cng.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C13018"}, LexBIGServiceTestCase.THES_SCHEME, null));//Organ
        cng = cng.restrictToTargetCodes(cm.createCodedNodeSet(new String[] {"C12727"}, LexBIGServiceTestCase.THES_SCHEME, null));//Heart
        
        //create a graph that contains Ford -> * (* is actually Jaguar, in this code system)
//        CodedNodeGraph cng2 = lbsi.getNodeGraph("Automobiles", null, "Relations");
//        cng2 = cng2.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"Ford"}, "Automobiles", null));
        CodedNodeGraph cng2 = lbsi.getNodeGraph(LexBIGServiceTestCase.THES_SCHEME, null, "roles");
        cng2 = cng2.restrictToSourceCodes(cm.createCodedNodeSet(new String[] {"C12727"}, LexBIGServiceTestCase.THES_SCHEME, null));

        //join them
        cng.union(cng2);
        
        //result should be 005 -> Ford -> Jaguar 
        ResolvedConceptReference[] rcr = cng.resolveAsList(null, true, false, -1, -1, null, null, null, 50).getResolvedConceptReference();

        assertTrue("1",rcr.length == 1);
        //top node
        assertTrue("2",rcr[0].getConceptCode().equals("C13018"));

        //no uplink
        assertTrue("3",rcr[0].getTargetOf() == null ||  rcr[0].getTargetOf().getAssociation().length == 0);
        
        //1 down link
        Association[] assn = rcr[0].getSourceOf().getAssociation();
        assertTrue("4",assn.length == 1);
        AssociatedConcept[] ac = assn[0].getAssociatedConcepts().getAssociatedConcept();
        assertTrue("5",ac.length == 1);
        assertTrue("6",ac[0].getConceptCode().equals("C12727"));
        
        //ford has a downlink to Jaguar
        assn = ac[0].getSourceOf().getAssociation();
        assertTrue("7",assn.length == 2);
        ac = assn[0].getAssociatedConcepts().getAssociatedConcept();
        assertTrue("8",ac.length == 1);
        assertTrue("9",ac[0].getConceptCode().equals("C12905"));//Thoracic Cavity
    }
    
//    public void testFilter() throws LBException
//    {
//        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
//        ConvenienceMethods cm = new ConvenienceMethods(lbsi);
//        
//        CodedNodeGraph cng = lbsi.getNodeGraph("Automobiles", null, "Relations");
//        
//        //If I add a filter which only allows 'r' or 'm' as the third letter in entity description
//        //this should cut the full tree down to ...
//        //    Domestic Auto Makers
//        //    Ford (subclass of Auto Makers)
//        //    Car
//        
//        try
//        {
//            TestFilter2.register();
//        }
//        catch (LBParameterException e)
//        {
//            //can happen if is already registered by another test.
//        }
//        
//        ResolvedConceptReference[] rcr =  cng.resolveAsList(null, true, false, -1, -1, null, null, null, cm.createLocalNameList(TestFilter2.name_), -1).getResolvedConceptReference();
//        
//        assertTrue(rcr.length == 1);
//        ResolvedConceptReference ref = null;
//        for (int i = 0; i < rcr.length && ref == null; i++)
//        	if (rcr[i].getConceptCode().equals("005"))
//        		ref = rcr[i];
//        assertTrue(ref != null);
//        assertTrue(ref.getConceptCode().equals("005"));
//        
//        assertTrue(ref.getSourceOf().getAssociation()[0].getAssociatedConcepts().getAssociatedConceptCount() == 1);
//        assertTrue(ref.getSourceOf().getAssociation()[0].getAssociatedConcepts().getAssociatedConcept(0).getConceptCode().equals("Ford"));
//        
//        assertTrue(ref.getSourceOf().getAssociation()[0].getAssociatedConcepts().getAssociatedConcept(0).getSourceOf() == null);
//    }
    
    
    private boolean contains(ConceptReference[] cr, String code, String codeSystem)
    {
        boolean contains = false;
        for (int i = 0; i < cr.length; i++)
        {
            if (cr[i].getConceptCode().equals(code) && cr[i].getCodingScheme().equals(codeSystem))
            {
                contains = true;
                break;
            }
        }
        return contains;
    }
}