/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl;

import java.lang.reflect.Field;

import gov.nih.nci.system.client.proxy.EVSProxyHelperImpl;
import junit.framework.TestCase;

import org.LexGrid.LexBIG.DataModel.Collections.ModuleDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ModuleDescription;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.RestrictToMatchingDesignations;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.RestrictToMatchingProperties;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.RestrictToProperties;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.Union;
import org.LexGrid.LexBIG.Impl.testUtility.ServiceHolder;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.concepts.CodedEntry;
import org.springframework.aop.framework.Advised;

/**
 * JUnit Tests for the CodedNodeSetImpl
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.8 $ checked in on $Date: 2008/05/29 17:08:39 $
 */
public class CodedNodeSetImplTest extends TestCase
{

    /**
     * Test optimize pending ops order.
     * 
     * @throws Exception the exception
     */
    public void testOptimizePendingOpsOrder() throws Exception
    {//This might be a real error related to remoting.  Should return 5 results for first search.
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
        getMatchingAlgorithms(lbsi);

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null).restrictToMatchingDesignations("India", CodedNodeSet.SearchDesignationOption.PREFERRED_ONLY, "LuceneQuery", null);
        
        CodedNodeSet cns2 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null).restrictToMatchingProperties(null, new PropertyType[]{PropertyType.PRESENTATION}, "France", "startsWith",
                null);
        
        cns.union(cns2);

        cns = cns.restrictToProperties(null, new PropertyType[]{PropertyType.DEFINITION});
        ResolvedConceptReferenceList rcrl = cns.resolveToList(null, null, null, 30);
        ResolvedConceptReference[] rcr = rcrl.getResolvedConceptReference();


        assertNotNull("0",rcr);
        assertTrue("1",rcr.length < 10);
        //top node

        assertTrue("2", contains(rcr,"C16727",LexBIGServiceTestCase.THES_SCHEME));
        ResolvedConceptReference temp = getConcept(rcr,"C16727");
        
        CodedNodeSet realObject = getObjectFromCallback(cns);
        CodedNodeSet realObject2 = getObjectFromCallback(cns2);
        
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(1) instanceof RestrictToMatchingDesignations);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(2) instanceof Union);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(3) instanceof RestrictToProperties);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.size() == 4);
//
        assertTrue(((CodedNodeSetImpl) realObject2).pendingOperations_.get(1) instanceof RestrictToMatchingProperties);
        assertTrue(((CodedNodeSetImpl) realObject2).pendingOperations_.size() == 2);

        ((CodedNodeSetImpl) realObject).optimizePendingOpsOrder();

        // restrictions should be above union.
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(1) instanceof RestrictToMatchingDesignations);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(2) instanceof RestrictToProperties);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.get(3) instanceof Union);
        assertTrue(((CodedNodeSetImpl) realObject).pendingOperations_.size() == 4);

        // cns2 shouldn't change, because it is cloned. - would be confusing if it changed.
        assertTrue(((CodedNodeSetImpl) realObject2).pendingOperations_.get(1) instanceof RestrictToMatchingProperties);
        assertTrue(((CodedNodeSetImpl) realObject2).pendingOperations_.size() == 2);
     
        
        // but the one inside of the cns union op should have changed.
        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) realObject).pendingOperations_.get(3))).getCodes()).pendingOperations_
                .get(1) instanceof RestrictToMatchingProperties);
        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) realObject).pendingOperations_.get(3))).getCodes()).pendingOperations_
                .get(2) instanceof RestrictToProperties);
        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) realObject).pendingOperations_.get(3))).getCodes()).pendingOperations_
                .size() == 3);

        // so far so good, lets make it more complex.

        CodedNodeSet cns3 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.META_SCHEME, null);
        cns3 = cns3.restrictToMatchingDesignations("India", null, "startsWith", null);

        cns.union(cns3);

        cns = cns.restrictToMatchingProperties(Constructors.createLocalNameList("Semantic_Type"), null, "Population Group",
                                         "contains", null);
        rcrl = cns.resolveToList(null, null, null, 30);
        rcr = rcrl.getResolvedConceptReference();

        assertNotNull("8",rcr);
        assertTrue("9",rcr.length <= 30);
        //top node
 //       assertTrue("10",rcr[0].getConceptCode().equals("C16727"));//India
        assertTrue("10",contains(rcr,"C16727",LexBIGServiceTestCase.META_SCHEME));
       
        CodedNodeSet cns4 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.ZEBRAFISH_SCHEME, null);
        cns.intersect(cns4);
        cns.restrictToProperties(Constructors.createLocalNameList("definition"), null);
        rcrl = cns.resolveToList(null, null, null, 30);
        rcr = rcrl.getResolvedConceptReference();

        assertNotNull("16",rcr);
        assertTrue("17",rcr.length == 1);
        //top node
//        assertTrue("18",rcr[0].getConceptCode().equals("C16727"));//India
        assertTrue("18",contains(rcr,"C16727",LexBIGServiceTestCase.META_SCHEME));
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(1) instanceof RestrictToMatchingDesignations);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(2) instanceof RestrictToProperties);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(3) instanceof Union);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(4) instanceof Union);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(5) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(6) instanceof Intersect);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(7) instanceof RestrictToProperties);
//
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.size() == 8);

//        ((CodedNodeSetImpl) cns).optimizePendingOpsOrder();

        // one of the restrictions should have moved up, but nothing changes below the intersection.
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(1) instanceof RestrictToMatchingDesignations);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(2) instanceof RestrictToProperties);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(3) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(4) instanceof Union);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(5) instanceof Union);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(6) instanceof Intersect);
//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.get(7) instanceof RestrictToProperties);

//        assertTrue(((CodedNodeSetImpl) cns).pendingOperations_.size() == 8);
//
//        // cns2 shouldn't change, because it is cloned. - would be confusing if it changed.
//        assertTrue(((CodedNodeSetImpl) cns2).pendingOperations_.get(1) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) cns2).pendingOperations_.size() == 2);
//
//        // cns3 shouldn't change, because it is cloned. - would be confusing if it changed.
//        assertTrue(((CodedNodeSetImpl) cns3).pendingOperations_.get(1) instanceof RestrictToMatchingDesignations);
//        assertTrue(((CodedNodeSetImpl) cns3).pendingOperations_.size() == 2);

        // both existing unions should have had one restriction added.
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(4))).getCodes()).pendingOperations_
//                .get(1) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(4))).getCodes()).pendingOperations_
//                .get(2) instanceof RestrictToProperties);
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(4))).getCodes()).pendingOperations_
//                .get(3) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(4))).getCodes()).pendingOperations_
//                .size() == 4);
//
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(5))).getCodes()).pendingOperations_
//                .get(1) instanceof RestrictToMatchingDesignations);
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(5))).getCodes()).pendingOperations_
//                .get(2) instanceof RestrictToMatchingProperties);
//        assertTrue(((CodedNodeSetImpl) ((Union) (((CodedNodeSetImpl) cns).pendingOperations_.get(5))).getCodes()).pendingOperations_
//                .size() == 3);

    }

    /**
     * Test union.
     * 
     * @throws LBException the LB exception
     */
    public void testUnion() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns = cns.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48328"}, LexBIGServiceTestCase.THES_SCHEME));
//        ResolvedConceptReference[] rcr1 = cns.resolveToList(null, null, null, 50).getResolvedConceptReference();
        CodedNodeSet cns2 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns2 = cns2.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48327"}, LexBIGServiceTestCase.THES_SCHEME));
//        ResolvedConceptReference[] rcr2 = cns2.resolveToList(null, null, null, 50).getResolvedConceptReference();
        cns.union(cns2);

        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 50).getResolvedConceptReference();
//        int len = rcr.length;
        assertTrue("1",rcr.length == 2);
        assertTrue("2",contains(rcr, "C48328", LexBIGServiceTestCase.THES_SCHEME));
        assertTrue("3",contains(rcr, "C48327", LexBIGServiceTestCase.THES_SCHEME));
    }

    /**
     * Test intersection.
     * 
     * @throws LBException the LB exception
     */
    public void testIntersection() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns = cns.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48328", "C48327"}, LexBIGServiceTestCase.THES_SCHEME));

        CodedNodeSet cns2 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns2 = cns2.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48327", "C48326"}, LexBIGServiceTestCase.THES_SCHEME));

        cns.intersect(cns2);

        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 50).getResolvedConceptReference();
//        int length = rcr.length;
        assertTrue("1",rcr.length == 1);
        assertTrue("2",contains(rcr, "C48327", LexBIGServiceTestCase.THES_SCHEME));
    }

    /**
     * Test difference.
     * 
     * @throws LBException the LB exception
     */
    public void testDifference() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns = cns.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48328", "C48327", "C48326"},
        		LexBIGServiceTestCase.THES_SCHEME));

        CodedNodeSet cns2 = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns2 = cns2.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48328", "C48329"}, LexBIGServiceTestCase.THES_SCHEME));

        cns.difference(cns2);

        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 50).getResolvedConceptReference();
//        int len = rcr.length;
        assertTrue("1",rcr.length == 2);
        assertTrue("2",contains(rcr, "C48327", LexBIGServiceTestCase.THES_SCHEME));
        assertFalse("3",contains(rcr, "C48329", LexBIGServiceTestCase.THES_SCHEME));
    }

    /**
     * Test union to self.
     * 
     * @throws LBException the LB exception
     */
    public void testUnionToSelf() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        cns = cns.restrictToCodes(Constructors.createConceptReferenceList(new String[]{"C48327"}, LexBIGServiceTestCase.THES_SCHEME));

        cns.union(cns);

        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 50).getResolvedConceptReference();
//        int len = rcr.length;
        assertTrue("1",rcr.length == 1);
        assertTrue("2",contains(rcr, "C48327", LexBIGServiceTestCase.THES_SCHEME));
    }

    /**
     * Test result limit.
     * 
     * @throws LBException the LB exception
     */
    public void testResultLimit() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);

        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 1).getResolvedConceptReference();

        assertTrue("1",rcr.length == 1);
    }

//    public void testFilters() throws LBException
//    {
//        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();
//
//        CodedNodeSet cns = lbsi.getCodingSchemeConcepts("Automobiles", null);
//
//        ResolvedConceptReference[] rcr = cns.resolveToList(null, null, null, 0).getResolvedConceptReference();
//        // no filters - length of 9
//        assertTrue(rcr.length == 9);
//
//        try
//        {
//            TestFilter.register();
//        }
//        catch (LBParameterException e1)
//        {
//            // may have been registered by a previous test - thats ok.
//        }
//
//        rcr = cns.resolveToList(null, Constructors.createLocalNameList(TestFilter.name_), null, null, 0)
//                .getResolvedConceptReference();
//        // filter for second letter of entity description being 'a'- length of 2
//        assertTrue(rcr.length == 2);
//
//        assertTrue(contains(rcr, "C0001", "Automobiles"));
//        assertTrue(contains(rcr, "Jaguar", "Automobiles"));
//
//        // test with iterator
//
//        ResolvedConceptReferencesIterator rcri = cns.resolve(null, Constructors.createLocalNameList(TestFilter.name_),
//                                                             null, null);
//        // filter for second letter of entity description being 'a'- length of 2
//        assertTrue(rcri.numberRemaining() == -1); // number remaining isn't calculated with filters
//
//        // there should be 2 results.
//
//        assertTrue(rcri.hasNext());
//        rcr = new ResolvedConceptReference[]{rcri.next()};
//        assertTrue(contains(rcr, "C0001", "Automobiles") || contains(rcr, "Jaguar", "Automobiles"));
//        assertTrue(rcri.hasNext());
//        rcr = new ResolvedConceptReference[]{rcri.next()};
//        assertTrue(contains(rcr, "C0001", "Automobiles") || contains(rcr, "Jaguar", "Automobiles"));
//
//        // another call to next should return null - they all got removed by the filter.
//        assertTrue(rcri.next() == null);
//
//        assertFalse(rcri.hasNext());
//
//        // yet another call should throw an exception
//
//        try
//        {
//            rcri.next();
//            fail("Didn't throw LBResourceUnavailableException");
//        }
//        catch (LBResourceUnavailableException e)
//        {
//            // expected path
//        }
//
//        // test with iterator and get(start, end) method. also sort, so we know what order they should be in
//
//        rcri = cns.resolve(Constructors.createSortOptionList(new String[]{"code"}, new Boolean[]{new Boolean(true)}),
//                           Constructors.createLocalNameList(TestFilter.name_), null, null);
//        // filter for second letter of entity description being 'a'- length of 2
//        assertTrue(rcri.numberRemaining() == -1); // number remaining isn't calculated with filters
//
//        // there should be 2 results.
//
//        rcr = rcri.get(0, 1).getResolvedConceptReference();
//        assertTrue(rcr.length == 1);
//        assertTrue(rcrEquals(rcr[0], "C0001", "Automobiles"));
//
//        rcr = rcri.get(1, 2).getResolvedConceptReference();
//        assertTrue(rcr.length == 1);
//        assertTrue(rcrEquals(rcr[0], "Jaguar", "Automobiles"));
//
//        try
//        {
//            rcr = rcri.get(2, 3).getResolvedConceptReference();
//            fail("Didn't throw LBParameterException");
//        }
//        catch (LBParameterException e)
//        {
//            // expected path
//        }
//
//        // go backwards
//        rcr = rcri.get(0, 1).getResolvedConceptReference();
//        assertTrue(rcr.length == 1);
//        assertTrue(rcrEquals(rcr[0], "C0001", "Automobiles"));
//
//        // get both
//        rcr = rcri.get(0, 2).getResolvedConceptReference();
//        assertTrue(rcr.length == 2);
//        assertTrue(rcrEquals(rcr[0], "C0001", "Automobiles"));
//        assertTrue(rcrEquals(rcr[1], "Jaguar", "Automobiles"));
//
//        rcri.release();
//
//        // try an invalid filter...
//        try
//        {
//            rcr = cns.resolveToList(null, Constructors.createLocalNameList("invalid"), null, null, 0)
//                    .getResolvedConceptReference();
//            fail("Didn't throw LBParameterException");
//        }
//        catch (LBParameterException e)
//        {
//            // expected path
//        }
//
//        // try two filters.
//
//        try
//        {
//            TestFilter2.register();
//        }
//        catch (LBParameterException e)
//        {
//            // can happen if is already registered by another test.
//        }
//
//        rcr = cns.resolveToList(null,
//                                Constructors.createLocalNameList(new String[]{TestFilter.name_, TestFilter2.name_}),
//                                null, null, 0).getResolvedConceptReference();
//        // only 1 should pass both filters
//        assertTrue(rcr.length == 1);
//
//        assertTrue(contains(rcr, "C0001", "Automobiles"));
//    }

    /**
 * Test restrict property type returns.
 * 
 * @throws LBException the LB exception
 */
public void testRestrictPropertyTypeReturns() throws LBException
    {
        LexBIGService lbsi = ServiceHolder.instance().getLexBIGService();

        CodedNodeSet cns = lbsi.getCodingSchemeConcepts(LexBIGServiceTestCase.THES_SCHEME, null);
        cns = cns.restrictToCodes(Constructors.createConceptReferenceList(new String[] {"C48327"}, LexBIGServiceTestCase.THES_SCHEME));

        // no type restriction
        ResolvedConceptReference[] rcrs = cns.resolveToList(null, null, null, 0).getResolvedConceptReference();
        assertTrue(rcrs.length == 1);
        CodedEntry ce = rcrs[0].getReferencedEntry();

        assertTrue("1",ce.getCommentCount() == 0);
        assertTrue("2",ce.getConceptPropertyCount() == 6);
        assertTrue("3",ce.getDefinitionCount() == 2);
        assertTrue("4",ce.getInstructionCount() == 0);
        assertTrue("5",ce.getPresentationCount() == 5);

        // restrict to a couple of presentation types
        rcrs = cns.resolveToList(
                                 null,
                                 null,
                                 new PropertyType[]{PropertyType.COMMENT, PropertyType.PRESENTATION,
                                         PropertyType.DEFINITION}, 0).getResolvedConceptReference();
        assertTrue("6",rcrs.length == 1);
        ce = rcrs[0].getReferencedEntry();

        assertTrue("7",ce.getCommentCount() == 0);
        assertTrue("8",ce.getConceptPropertyCount() == 0);
        assertTrue("9",ce.getDefinitionCount() == 2);
        assertTrue("10",ce.getInstructionCount() == 0);
        assertTrue("11",ce.getPresentationCount() == 5);
        
        assertTrue("12",ce.getPresentation()[0].getText().getContent().equals("Purple"));
        assertTrue("13",ce.getDefinition()[0].getText().getContent().equals("Any of a group of colors with a hue between that of violet and red."));
        
       // restrict to one presentation type
        rcrs = cns.resolveToList(
                                 null,
                                 null,
                                 new PropertyType[]{PropertyType.PRESENTATION},
                                 0).getResolvedConceptReference();
        assertTrue("14",rcrs.length == 1);
        ce = rcrs[0].getReferencedEntry();

        assertTrue("15",ce.getCommentCount() == 0);
        assertTrue("16",ce.getConceptPropertyCount() == 0);
        assertTrue("17",ce.getDefinitionCount() == 0);
        assertTrue("18",ce.getInstructionCount() == 0);
        assertTrue("19",ce.getPresentationCount() >= 1);
        
        // restrict to one presentation type (that doesn't have entries)
        rcrs = cns.resolveToList(
                                 null,
                                 null,
                                 new PropertyType[]{PropertyType.INSTRUCTION},
                                 0).getResolvedConceptReference();
        assertTrue("20",rcrs.length == 1);
        ce = rcrs[0].getReferencedEntry();

        assertTrue("21",ce.getCommentCount() == 0);
        assertTrue("22",ce.getConceptPropertyCount() == 0);
        assertTrue("23",ce.getDefinitionCount() == 0);
        assertTrue("24",ce.getInstructionCount() == 0);
        assertTrue("25",ce.getPresentationCount() == 0);
        
        // restrict to one presentation type and one property name
        rcrs = cns.resolveToList(
                                 null,
                                 Constructors.createLocalNameList("definition"),
                                 new PropertyType[]{PropertyType.DEFINITION},
                                 0).getResolvedConceptReference();
        assertTrue("26",rcrs.length == 1);
        ce = rcrs[0].getReferencedEntry();

        assertTrue("27",ce.getCommentCount() == 0);
        assertTrue("28",ce.getConceptPropertyCount() == 0);
        assertTrue("29",ce.getDefinitionCount() == 1);
        assertTrue("30",ce.getInstructionCount() == 0);
        assertTrue("31",ce.getPresentationCount() == 0);
        
        // restrict to one presentation type and one property name (which don't line up)
        rcrs = cns.resolveToList(
                                 null,
                                 Constructors.createLocalNameList("Symomym"),
                                 new PropertyType[]{PropertyType.DEFINITION},
                                 0).getResolvedConceptReference();
        assertTrue("32",rcrs.length == 1);
        ce = rcrs[0].getReferencedEntry();

        assertTrue("33",ce.getCommentCount() == 0);
        assertTrue("34",ce.getConceptPropertyCount() == 0);
        assertTrue("35",ce.getDefinitionCount() == 0);
        assertTrue("36",ce.getInstructionCount() == 0);
        assertTrue("37",ce.getPresentationCount() == 0);

    }

    private boolean contains(ResolvedConceptReference[] rcr, String code, String codeSystem)
    {
        boolean contains = false;
        for (int i = 0; i < rcr.length; i++)
        {
            if (rcrEquals(rcr[i], code, codeSystem))
            {
                contains = true;
                break;
            }
        }
        return contains;
    }

    private boolean rcrEquals(ResolvedConceptReference rcr, String code, String codeSystem)
    {
        if (rcr.getConceptCode().equals(code) && rcr.getCodingScheme().equals(codeSystem))
        {
            return true;
        }
        return false;
    }
    
    private ResolvedConceptReference getConcept(ResolvedConceptReference[] inRef, String conceptCode)
    {
    	ResolvedConceptReference rcr = null;
    	for (int i=0; i< inRef.length; i++)
    	{
    		if (inRef[i].getConceptCode().compareTo(conceptCode) == 0)
    		{
    			rcr = inRef[i];
    		}   		
    	}
    	
    	return rcr;
    }
    
    private void getMatchingAlgorithms(LexBIGService lbsi)
    {
   	
    	ModuleDescriptionList mdl = lbsi.getMatchAlgorithms();
    	ModuleDescription md = null;
    	for (int i=0;i< mdl.getModuleDescriptionCount();i++)
    	{
    		md = mdl.getModuleDescription(i);
    		md.getName();
    	}
    }
    
    private CodedNodeSet getObjectFromCallback(CodedNodeSet cns) throws Exception
    {
        Object interceptor = null;
        int i = 0;
        
        try{
        while (true) {
            Field field = cns.getClass().getDeclaredField("CGLIB$CALLBACK_"+i);
            field.setAccessible(true);
            Object value = field.get(cns);
            String testValue = value.getClass().getName();
            if (value.getClass().getName().contains("EqualsInterceptor")) {
                interceptor = value;
                break;
            }
            i++;
        }
        Field field = interceptor.getClass().getDeclaredField("advised");
        field.setAccessible(true);
        Advised advised = (Advised)field.get(interceptor);
        Object realObject = advised.getTargetSource().getTarget();
        CodedNodeSet cns_out = (CodedNodeSet) realObject;
        return cns_out;
        }
        catch (Exception e){
        	throw new Exception("Class not convertable");
        }
 
    }
}