/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl.dataAccess;

import java.util.Date;

import junit.framework.TestCase;

import org.LexGrid.LexBIG.DataModel.Core.types.CodingSchemeVersionStatus;
import org.LexGrid.LexBIG.Impl.dataAccess.Registry.DBEntry;
import org.LexGrid.LexBIG.Impl.dataAccess.Registry.HistoryEntry;
import org.LexGrid.LexBIG.Utility.ConvenienceMethods;
import org.LexGrid.util.sql.DBUtility;

/**
 * JUnit test cases for the registry system.
 *
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.2 $ checked in on $Date: 2007/12/15 00:07:47 $
 */
public class RegistryTest extends TestCase
{

    /**
     * Test registry.
     * 
     * @throws Exception the exception
     */
    @SuppressWarnings("null")
    public void testRegistry() throws Exception
    {
        Registry registry = ResourceManager.instance().getRegistry();
        
        int preDbCount = registry.getDBEntries().length;
        int preHistCount = registry.getHistoryEntries().length;
        
        String id = registry.getNextDBIdentifier();
        
        registry.addNewItem("2.3.4", "1.0", CodingSchemeVersionStatus.ACTIVE.toString(), "<JUNIT_IGNORE>jdbc:test", "production", id + "", "prefix");
        
        String historyId = registry.getNextHistoryIdentifier();
        registry.addNewHistory("1.5.7", "<JUNIT_IGNORE>testUrl", historyId + "theDBName", "thePrefix");
        
        Date date = registry.getLastUpdateTime();
        
        registry = null;
        ResourceManager.reInit(null);
        registry = ResourceManager.instance().getRegistry();
        
        if (ResourceManager.instance().getSystemVariables().getAutoLoadSingleDBMode())
        {
            assertTrue(registry.getNextDBIdentifier().equals(DBUtility.computeNextIdentifier(id)));
            assertTrue(registry.getNextHistoryIdentifier().equals(DBUtility.computeNextIdentifier(historyId)));
        }
        else
        {
            assertTrue(registry.getNextDBIdentifier().equals((Integer.parseInt(id) + 1) + ""));
            assertTrue(registry.getNextHistoryIdentifier().equals((Integer.parseInt(historyId) + 1) + ""));
        }
        
        assertTrue(registry.getLastUpdateTime().getTime() >= date.getTime());
        
        DBEntry[] dbe = registry.getDBEntries();
        
        assertTrue(dbe != null);
        assertTrue(dbe.length == preDbCount + 1);
        
        assertTrue(dbe[preDbCount].dbURL.equals("<JUNIT_IGNORE>jdbc:test"));
        assertTrue(dbe[preDbCount].status.equals(CodingSchemeVersionStatus.ACTIVE.toString()));
        assertTrue(dbe[preDbCount].tag.equals("production"));
        assertTrue(dbe[preDbCount].urn.equals("2.3.4"));
        assertTrue(dbe[preDbCount].version.equals("1.0"));
        assertTrue(dbe[preDbCount].dbName.equals(id + ""));
        assertTrue(dbe[preDbCount].prefix.equals("prefix"));
        assertTrue(dbe[preDbCount].deactiveDate == 0);
        assertTrue(dbe[preDbCount].lastUpdateDate > 0);
        
        
        
        HistoryEntry[] history = registry.getHistoryEntries();
        assertTrue(history != null);
        assertTrue(history.length == preHistCount + 1);
        assertTrue(history[preHistCount].urn.equals("1.5.7"));
        assertTrue(history[preHistCount].dbURL.equals("<JUNIT_IGNORE>testUrl"));
        assertTrue(history[preHistCount].dbName.equals(historyId + "theDBName"));
        assertTrue(history[preHistCount].prefix.equals("thePrefix"));
        assertTrue(history[preHistCount].lastUpdateDate <= registry.getLastUpdateTime().getTime());
        
        registry.remove(ConvenienceMethods.createAbsoluteCodingSchemeVersionReference("2.3.4", "1.0"));
        registry.removeHistoryEntry("1.5.7");
    }
}