/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl.function.metadata;

import java.util.HashSet;

import junit.framework.TestCase;

import org.LexGrid.LexBIG.DataModel.Core.AbsoluteCodingSchemeVersionReference;
import org.LexGrid.LexBIG.DataModel.Core.MetadataProperty;
import org.LexGrid.LexBIG.Impl.testUtility.ServiceHolder;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.LexBIG.Impl.LexBIGServiceTestCase;

/**
 * Class to test the MetaData search capabilities.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.4 $ checked in on $Date: 2007/12/15 00:07:15 $
 */
public class TestMetaDataSearch extends TestCase
{
    
    /**
     * Test list coding schemes.
     * 
     * @throws Exception the exception
     */
    public void testListCodingSchemes() throws Exception
    {
        LexBIGServiceMetadata md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        AbsoluteCodingSchemeVersionReference[] acsvrl = md.listCodingSchemes()
                .getAbsoluteCodingSchemeVersionReference();

        assertTrue(acsvrl.length >= 2);
//        assertTrue(contains(acsvrl, Constructors.createAbsoluteCodingSchemeVersionReference("test.1", "1.0")));
//        assertTrue(contains(acsvrl, Constructors.createAbsoluteCodingSchemeVersionReference("test.2", "2.0")));
        assertTrue(contains(acsvrl, Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION)));
        assertTrue(contains(acsvrl, Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.META_SCHEME, LexBIGServiceTestCase.META_VERSION)));
    }

    private boolean contains(AbsoluteCodingSchemeVersionReference[] acsvr, AbsoluteCodingSchemeVersionReference acsvr2)
    {
        for (int i = 0; i < acsvr.length; i++)
        {
            if (acsvr[i].getCodingSchemeURN().equals(acsvr2.getCodingSchemeURN())
                    && acsvr[i].getCodingSchemeVersion().equals(acsvr2.getCodingSchemeVersion()))
            {
                return true;
            }
        }
        return false;
    }

    private boolean contains(MetadataProperty[] mdp, String urn, String version, String name, String value)
    {
        for (int i = 0; i < mdp.length; i++)
        {
            if (mdp[i].getCodingSchemeURN().equals(urn)
                    && mdp[i].getCodingSchemeVersion().equals(version)
                    && mdp[i].getName().equals(name)
                    && mdp[i].getValue().equals(value))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Test container restriction search.
     * 
     * @throws Exception the exception
     */
    public void testContainerRestrictionSearch() throws Exception
    {
        LexBIGServiceMetadata md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToValue("owl-full", "LuceneQuery");
        md = md.restrictToCodingScheme(Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION));

        MetadataProperty[] result = md.resolve().getMetadataProperty();
        assertTrue(result.length >= 2);
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "format", "OWL-FULL"));
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "download_format", "OWL-FULL"));

        md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToCodingScheme(Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION));
        md = md.restrictToPropertyParents(new String[]{"core_format"});
        md = md.restrictToValue("owl-full", "LuceneQuery");
        result = md.resolve().getMetadataProperty();
        assertTrue(result.length >= 1);
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "format", "OWL-FULL"));

    }
    
    /**
     * Test property restriction search.
     * 
     * @throws Exception the exception
     */
    public void testPropertyRestrictionSearch() throws Exception
    {
        LexBIGServiceMetadata md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToValue("owl-full", "LuceneQuery");
        md = md.restrictToCodingScheme(Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION));

        MetadataProperty[] result = md.resolve().getMetadataProperty();
        assertTrue(result.length >= 2);
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "format", "OWL-FULL"));
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "download_format", "OWL-FULL"));

        md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToCodingScheme(Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION));
        md = md.restrictToProperties(new String[]{"format"});
        md = md.restrictToValue("owl-full", "LuceneQuery");
        result = md.resolve().getMetadataProperty();
        assertTrue(result.length >= 1);
        assertTrue(contains(result, LexBIGServiceTestCase.THES_LOCAL, LexBIGServiceTestCase.THES_VERSION, "format", "OWL-FULL"));

    }
    
    /**
     * Test coding scheme restriction search.
     * 
     * @throws Exception the exception
     */
    public void testCodingSchemeRestrictionSearch() throws Exception
    {
        LexBIGServiceMetadata md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToValue("English", "LuceneQuery");
        MetadataProperty[] result = md.resolve().getMetadataProperty();
        
        HashSet<String> temp = new HashSet<String>();
        for (int i = 0; i < result.length; i++)
        {
            temp.add(result[i].getCodingSchemeURN() + ":" + result[i].getCodingSchemeVersion());
        }
        
        //should be more than 1 unique code system.
        assertTrue(temp.size() >= 2);
        
        //should contain this
        assertTrue(temp.contains(LexBIGServiceTestCase.META_SCHEME + ":" + LexBIGServiceTestCase.META_VERSION));
        
        //now do the restriction, and retest.
        
        md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToValue("English", "LuceneQuery");
        md = md.restrictToCodingScheme(Constructors.createAbsoluteCodingSchemeVersionReference(LexBIGServiceTestCase.META_SCHEME , LexBIGServiceTestCase.META_VERSION));
        result = md.resolve().getMetadataProperty();
        
        temp = new HashSet<String>();
        for (int i = 0; i < result.length; i++)
        {
            temp.add(result[i].getCodingSchemeURN() + ":" + result[i].getCodingSchemeVersion());
        }
        
        //should be more than 1 unique code system.
        assertTrue(temp.size() >= 1);
        
        //should contain this
        assertTrue(temp.contains(LexBIGServiceTestCase.META_SCHEME + ":" + LexBIGServiceTestCase.META_VERSION));
    }
    
    /**
     * Test value restriction.
     * 
     * @throws Exception the exception
     */
    public void testValueRestriction() throws Exception
    {
        LexBIGServiceMetadata md = ServiceHolder.instance().getLexBIGService().getServiceMetadata();
        md = md.restrictToValue(".*core.*", "RegExp");
 
        MetadataProperty[] result = md.resolve().getMetadataProperty();
        assertTrue(result.length >= 3);
        assertTrue(contains(result, LexBIGServiceTestCase.META_SCHEME, LexBIGServiceTestCase.META_VERSION, "homepage", "http://ncicb.nci.nih.gov/core/EVS"));
    }
}