/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl.helpers;

import java.util.Arrays;

import junit.framework.TestCase;

import org.LexGrid.LexBIG.DataModel.InterfaceElements.SortOption;
import org.LexGrid.LexBIG.Utility.Constructors;

/**
 * JUnit Tests for the CodeToReturn sorter.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.2 $ checked in on $Date: 2007/12/15 00:06:59 $
 */
public class CodeToReturnTest extends TestCase
{

    /**
     * Test code to return sorter.
     * 
     * @throws Exception the exception
     */
    public void testCodeToReturnSorter() throws Exception
    {
        CodeToReturn[] testData = new CodeToReturn[4];

        // basic test - sort by code.

        testData[0] = new CodeToReturn("a", "1", "1.0", (float) 0.9);
        testData[1] = new CodeToReturn("z", "1", "1.0", (float) 0.8);
        testData[2] = new CodeToReturn("bb", "1", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("q", "1", "1.0", (float) 0.6);

        SortOption[] sortOrder = new SortOption[]{Constructors.createSortOption("code", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("a"));
        assertTrue(testData[1].getConceptCode().equals("bb"));
        assertTrue(testData[2].getConceptCode().equals("q"));
        assertTrue(testData[3].getConceptCode().equals("z"));
        
        //now try reverse
        sortOrder = new SortOption[]{Constructors.createSortOption("code", new Boolean(false))};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[3].getConceptCode().equals("a"));
        assertTrue(testData[2].getConceptCode().equals("bb"));
        assertTrue(testData[1].getConceptCode().equals("q"));
        assertTrue(testData[0].getConceptCode().equals("z"));
        
        

        // sort by code, break ties with score.

        testData[0] = new CodeToReturn("a", "1", "1.0", (float) 0.9);
        testData[1] = new CodeToReturn("a", "1", "1.0", (float) 1.0);
        testData[2] = new CodeToReturn("bb", "1", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("q", "1", "1.0", (float) 0.6);

        sortOrder = new SortOption[]{Constructors.createSortOption("code", new Boolean(true)),
                Constructors.createSortOption("matchToQuery", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("a"));
        assertTrue(testData[0].getScore() == (float) 1.0);
        assertTrue(testData[1].getConceptCode().equals("a"));
        assertTrue(testData[1].getScore() == (float) 0.9);
        assertTrue(testData[2].getConceptCode().equals("bb"));
        assertTrue(testData[3].getConceptCode().equals("q"));

        // sort by score

        testData[0] = new CodeToReturn("a", "1", "1.0", (float) .1);
        testData[1] = new CodeToReturn("b", "1", "1.0", (float) .5);
        testData[2] = new CodeToReturn("c", "1", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("d", "1", "1.0", (float) 0.996);

        sortOrder = new SortOption[]{Constructors.createSortOption("matchToQuery", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("d"));
        assertTrue(testData[1].getConceptCode().equals("c"));
        assertTrue(testData[2].getConceptCode().equals("b"));
        assertTrue(testData[3].getConceptCode().equals("a"));

        // sort by score - break ties with code (reverse order on sort by score)

        testData[0] = new CodeToReturn("b", "1", "1.0", (float) .5);
        testData[1] = new CodeToReturn("a", "1", "1.0", (float) .5);
        testData[2] = new CodeToReturn("d", "1", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("c", "1", "1.0", (float) 0.7);

        sortOrder = new SortOption[]{Constructors.createSortOption("matchToQuery", new Boolean(false)),
                Constructors.createSortOption("code", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("a"));
        assertTrue(testData[1].getConceptCode().equals("b"));
        assertTrue(testData[2].getConceptCode().equals("c"));
        assertTrue(testData[3].getConceptCode().equals("d"));

        // sort by codesystem

        testData[0] = new CodeToReturn("a", "1.4", "1.0", (float) .5);
        testData[1] = new CodeToReturn("b", "1.3", "1.0", (float) .5);
        testData[2] = new CodeToReturn("c", "1.2", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("d", "1.1", "1.0", (float) 0.7);

        sortOrder = new SortOption[]{Constructors.createSortOption("codeSystem", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("d"));
        assertTrue(testData[1].getConceptCode().equals("c"));
        assertTrue(testData[2].getConceptCode().equals("b"));
        assertTrue(testData[3].getConceptCode().equals("a"));

        // group by code system, sort by score then code

        testData[0] = new CodeToReturn("a", "1.1", "1.0", (float) 0.9);
        testData[1] = new CodeToReturn("c", "1.1", "1.0", (float) 0.7);
        testData[2] = new CodeToReturn("b", "1.1", "1.0", (float) 0.7);
        testData[3] = new CodeToReturn("z", "1.0", "1.0", (float) 0.1);

        sortOrder = new SortOption[]{Constructors.createSortOption("codeSystem", null),
                Constructors.createSortOption("matchToQuery", null), Constructors.createSortOption("code", null)};

        Arrays.sort(testData, new CodeToReturnComparator(sortOrder));

        assertTrue(testData[0].getConceptCode().equals("z"));
        assertTrue(testData[1].getConceptCode().equals("a"));
        assertTrue(testData[2].getConceptCode().equals("b"));
        assertTrue(testData[3].getConceptCode().equals("c"));

    }
}