/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl.helpers;

import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ExtensionDescription;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Exceptions.LBParameterException;
import org.LexGrid.LexBIG.Extensions.Query.Filter;
import org.LexGrid.LexBIG.Impl.Extensions.ExtensionRegistryImpl;
import org.LexGrid.commonTypes.Source;


/**
 * Filter to use for JUnit test cases. This one only allows items that have an 'a' as their second letter in
 * the entity descrition.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.2 $ checked in on $Date: 2007/12/15 00:06:59 $
 */
public class TestFilter implements Filter
{
    
    /** The provider_. */
    public static String provider_    = "Mayo Clinic";
    
    /** The Constant role_. */
    final static String role_        = "author";           // 2006 model change
    
    /** The Constant providerSource. */
    final static Source providerSource     = new Source(); // 2006 model change
    
    /** The description_. */
    public static String description_ = "Filter for JUnit tests";
    
    /** The name_. */
    public static String name_        = "JUnit Test Filter";
    
    /** The version_. */
    public static String version_     = "1.0";

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.Extensions.Query.Filter#match(org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference)
     */
    public boolean match(ResolvedConceptReference ref)
    {
        return ref.getEntityDescription().getContent().charAt(1) == 'a';
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.Extensions.Extendable#getDescription()
     */
    public String getDescription()
    {
        return description_;
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.Extensions.Extendable#getName()
     */
    public String getName()
    {
        return name_;
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.Extensions.Extendable#getProvider()
     */
    public String getProvider()
    {
        return provider_;
    }

    /* (non-Javadoc)
     * @see org.LexGrid.LexBIG.Extensions.Extendable#getVersion()
     */
    public String getVersion()
    {
        return version_;
    }

    /**
     * Register.
     * 
     * @throws LBParameterException the LB parameter exception
     * @throws LBException the LB exception
     */
    public static void register() throws LBParameterException, LBException
    {
        ExtensionDescription ed = new ExtensionDescription();
        ed.setExtensionBaseClass(TestFilter.class.getInterfaces()[0].getName());
        ed.setExtensionClass(TestFilter.class.getName());
        ed.setDescription(description_);
        ed.setName(name_);
        ed.setVersion(version_);
        // Update for 2006 Model
        providerSource.setContent(provider_);    // 2006 model change
        providerSource.setRole(role_);           // 2006 model change
        ed.setExtensionProvider(providerSource); // 2006 model change
        //ed.setExtensionProvider(provider_);

        // I'm registering them this way to avoid the lexBig service manager API.
        // If you are writing an add-on extension, you should register them through the
        // proper interface.
        ExtensionRegistryImpl.instance().registerFilterExtension(ed);
    }

}