/*
 * Copyright: (c) 2004-2006 Mayo Foundation for Medical Education and
 * Research (MFMER).  All rights reserved.  MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, the trade names, 
 * trademarks, service marks, or product names of the copyright holder shall
 * not be used in advertising, promotion or otherwise in connection with
 * this Software without prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.LexGrid.LexBIG.Impl.testUtility;

import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.client.ApplicationServiceProvider;

import java.io.File;
import java.util.Properties;

import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Impl.LexBIGServiceImpl;
import org.LexGrid.LexBIG.Impl.LexBIGServiceTestCase;
import org.LexGrid.LexBIG.Impl.dataAccess.ResourceManager;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;

/**
 * Singleton class for getting a LexBIGService for the JUnit test cases.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 1.8 $ checked in on $Date: 2008/05/30 19:31:39 $
 */
public class ServiceHolder 
{
    private static final String serviceUrl = LexBIGServiceTestCase.serviceUrl;

    // GF# 14545
    private static final String _service = "EvsServiceInfo";
    
    private static Properties[]                configs_;
    private static int                         currentConfig_ = 0;
//    private static ArrayList<TestServerConfig> serverConfigs;
    private static ServiceHolder               sh_;
    private File                               testConfigFolder_;
    private boolean                            singleConfigMode_ = true;

    private LexBIGService                      lbsi_;

    /**
     * Use this to get an instance of the ServiceHolder. If 'configureForSingleConfig' has not been called,
     * this will run the tests on all configured databases in the testConfig.props file.
     * 
     * @return the service holder
     */
    public static ServiceHolder instance()
    {
        if (sh_ == null)
        {
            throw new RuntimeException("ServiceHolder has not been configured");
            //sh_ = new ServiceHolder(true, false);
        }
        return sh_;

    }

    /**
     * Use this to set up the tests for end user enviroment validation. Only runs the tests once, using their
     * normal config file.
     * 
     * @param useCaCore boolean whether to use caCORE
     */
    public static void configureForSingleConfig(boolean useCaCore)
    {

//        sh_ = new ServiceHolder(false, useCaCore);
        sh_ = new ServiceHolder(useCaCore);

    }

    /**
     * Configure next.
     * 
     * @throws LBException the LB exception
     */
    public void configureNext() throws LBException
    {
        ResourceManager.reInit(configs_[currentConfig_++]);
        lbsi_ = new LexBIGServiceImpl();
    }

//    protected ArrayList<TestServerConfig> getServerConfigs()
//    {
//        return serverConfigs;
//    }

    /**
 * Removes the current config test folder.
 */
public void removeCurrentConfigTestFolder()
    {
        String path = (String) configs_[0].get("REGISTRY_FILE");
        File temp = new File(path).getParentFile();
        deleteFolder(temp);
        temp.mkdir();
    }

    /**
     * Removes the test folder.
     */
    public void removeTestFolder()
    {
        deleteFolder(testConfigFolder_);
    }

    private ServiceHolder(boolean useCaCore)
    {
    	try
    	{   
    		singleConfigMode_ = true;
            if (useCaCore) {
                lbsi_ =  (EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(serviceUrl, _service);
            }
            else {
                lbsi_ = new LexBIGServiceImpl();
            }
    	}
        catch (Exception e)
        {
            System.err.println("Problem reading Test config file");
            e.printStackTrace();
            System.exit(-1);
        }
    }
    
//    private ServiceHolder(boolean multiConfig, boolean useCaCore)
//    {   
//        multiConfig = false;
//        try
//        {
//            if (multiConfig)
//            {
//                singleConfigMode_ = false;
//                testConfigFolder_ = new File(System.getProperty("java.io.tmpdir")
//                        + System.getProperty("file.separator")
//                        + "LexBIGTest-"
//                        + UUID.randomUUID().toString());
//
//                // get rid of whatever is there.
//                deleteFolder(testConfigFolder_);
//
//                testConfigFolder_.mkdir();
//
//                serverConfigs = TestServerConfigReader.getServerConfigs();
//
//                configs_ = new Properties[serverConfigs.size()];
//
//                for (int i = 0; i < serverConfigs.size(); i++)
//                {
//                    configs_[i] = createPropertiesObject(serverConfigs.get(i), i, testConfigFolder_);
//                }
//
//                if (configs_.length > 0)
//                {
//                    // clear the log files from the last run.
//                    File temp = new File(configs_[0].getProperty("LOG_FILE_LOCATION"), "LexBIG_full_log.txt");
//                    temp.delete();
//                    temp = new File(configs_[0].getProperty("LOG_FILE_LOCATION"), "LexBIG_load_log.txt");
//                    temp.delete();
//                }
//            }
//            else
////            {
//                singleConfigMode_ = true;
//                if (useCaCore) {
//                    lbsi_ =  (EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(serviceUrl);
//                }
//                else {
//                    lbsi_ = new LexBIGServiceImpl();
//                }
////            }
//
//        }
//        catch (Exception e)
//        {
//            System.err.println("Problem reading Test config file");
//            e.printStackTrace();
//            System.exit(-1);
//        }
//
//    }

    /**
     * Gets the lex big service.
     * 
     * @return the lex big service
     */
    public LexBIGService getLexBIGService()
    {
        return lbsi_;
    }

    /**
     * Gets the single config mode.
     * 
     * @return the single config mode
     */
    public boolean getSingleConfigMode()
    {
        return singleConfigMode_;
    }


    private static void deleteFolder(File file)
    {
        if (file.isDirectory())
        {
            File[] temp = file.listFiles();
            for (int i = 0; i < temp.length; i++)
            {
                deleteFolder(temp[i]);
            }
        }
        file.delete();
    }
    
    /**
     * Gets the uRL.
     * 
     * @return the uRL
     */
    public static String getURL()
    {
    	return serviceUrl;
    }
}