package gov.nih.nci.lexbig.ext;

import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.client.ApplicationServiceProvider;

import java.io.PrintWriter;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.util.List;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Category;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import org.LexGrid.LexBIG.DataModel.Collections.AbsoluteCodingSchemeVersionReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.MetadataPropertyList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Core.AbsoluteCodingSchemeVersionReference;
import org.LexGrid.LexBIG.DataModel.Core.AssociatedConcept;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeSummary;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.MetadataProperty;
import org.LexGrid.LexBIG.DataModel.Core.NameAndValue;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.CodingSchemeRendering;
import org.LexGrid.LexBIG.DataModel.NCIHistory.NCIChangeEvent;
import org.LexGrid.LexBIG.DataModel.NCIHistory.types.ChangeType;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.codingSchemes.CodingSchemeVersion;
import org.LexGrid.commonTypes.PropertyQualifier;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Comment;
import org.LexGrid.concepts.Definition;
import org.LexGrid.concepts.Instruction;
import org.LexGrid.concepts.Presentation;
import org.LexGrid.naming.SupportedAssociation;
import org.LexGrid.naming.SupportedProperty;
import org.LexGrid.relations.Relations;

import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.DataModel.Core.Association;
import org.LexGrid.LexBIG.Utility.ConvenienceMethods;
import org.LexGrid.LexBIG.DataModel.Collections.AssociationList;
import org.LexGrid.LexBIG.DataModel.Collections.AssociatedConceptList;
import org.LexGrid.LexBIG.DataModel.Collections.LocalNameList;

import org.LexGrid.LexBIG.DataModel.InterfaceElements.RenderingDetail;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeTagList;
import org.LexGrid.LexBIG.Exceptions.LBParameterException;


/**
 * <!-- LICENSE_TEXT_START -->
 * Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
 * and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
 * in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
 * "This product includes software developed by NGIT and the National Cancer Institute."
 * If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
 * wherever such third-party acknowledgments normally appear.
 * 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
 * 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
 * the recipient to use any trademarks owned by either NCI or NGIT
 * 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
 * NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * <!-- LICENSE_TEXT_END -->
 */

/**
 * DLBAdapter
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */
public final class DLBAdapter extends DLBWrapper {
	/** The Constant DEFAULT_ASSOC_FWD. {"CHD", "hasSubtype", "hasSubclass", "inverse_isa", "inverse_is_a"}*/
	public final static String[] DEFAULT_ASSOC_FWD = new String[] {"CHD", "hasSubtype", "hasSubclass", "inverse_isa", "inverse_is_a"};

	/** The Constant DEFAULT_ASSOC_REV. {"PAR", "RB", "isa", "is_a", "branch_of", "part_of", "tributary_of"}*/
	public final static String[] DEFAULT_ASSOC_REV = new String[] {"PAR", "RB", "isa", "is_a", "branch_of", "part_of", "tributary_of"};

	/** a static constant, 0. */
	public static final int MATCH_BY_NAME = 0;

	/** a static constant, 1. */
	public static final int MATCH_BY_ROLE = 1;

	/** a static constant, 2. */
	public static final int MATCH_BY_PROPERTY = 2;

	/** a static constant, 3. */
	public static final int MATCH_BY_SILO = 3;

	/** a static constant, 4. */
	public static final int MATCH_BY_ASSOCIATION = 4;

	/** a static constant, 5. */
	public static final int MATCH_BY_SYNONYM = 5;

	/** a static constant, 6. */
	public static final int MATCH_BY_INVERSE_ROLE = 6;

	/** a static constant, 7. */
	public static final int MATCH_BY_INVERSE_ASSOCIATION = 7;

	/** a static constant, 8. */
	public static final int MATCH_BY_ROLE_PROPERTY = 8;

	/** a static constant, 9. */
	public static final int MATCH_BY_SOURCE_CODE = 9;



	/** A static constant, 0. */
	public static final int WITH_NO_ATTRIBUTES = 0;

	/** A static constant, 1. */
	public static final int WITH_ALL_ATTRIBUTES = 1;

	/** A static constant, 2. */
	public static final int WITH_ALL_ROLES = 2;

	/** A static constant, 3. */
	public static final int WITH_ALL_PROPERTIES = 3;

	/** A static constant, 4. */
	public static final int WITH_ALL_ASSOCIATIONS = 4;


    /** a constant 1000. */
    final static int MAX_LEVEL = 1000;

    /** a namespace id. */
    int namespaceId = -1;

    /** default search algorithm name. */
    private String defaultAlgorithm = "LuceneQuery";

    /** The default tag. */
    private String defaultTag = "PRODUCTION";

    /**
     * default Namespace Id.
     */
    //private int defaultNamespaceId = 0;

    /** An instance of Category.
     */
    //Category cat;


///////////////////////
//  Constructors
///////////////////////

    /** Default constructor
     *
     * @since DLBExt 1.0
     */
    public DLBAdapter()
    {
		super();
	}


    /**
     * Instantiates a new dLB adapter.
     *
     * @param lbSvc the EVSApplicationService
     */
    public DLBAdapter(EVSApplicationService lbSvc)
    {
		super(lbSvc);
	}

    /**
     * Gets the current coding scheme name.
     *
     * @return the current coding scheme name
     */
    public String getCurrentCodingSchemeName()
    {
		return this.codingScheme;
	}


    /**
     * Gets the current version.
     *
     * @return the current version
     */
    public String getCurrentVersion()
    {
		return this.version;
	}


    /**
     * Gets the current tag.
     *
     * @return the current tag
     */
    public String getCurrentTag()
    {
		return this.tag;
	}



  /**
   * Gets the adapter hash map.
   *
   * @param url the url of server
   *
   * @return the adapter hash map
   */
  public static HashMap getAdapterHashMap(String url)
  {
	  HashMap<String,DLBAdapter> hmap = new HashMap<String,DLBAdapter>();
	  DLBAdapter adapter0 = new DLBAdapter();
      Vector<String> name_vec = adapter0.getVocabularyNames();

      //GF# 14545
      String _service = "EvsServiceInfo";
      
      for (int i=0; i<name_vec.size(); i++)
      {
		  DLBAdapter adapter = null;
		  if (url != null)
		  {
                try {
		  		    adapter = new DLBAdapter((EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(url, _service));
			    } catch (Exception e) {
					e.printStackTrace();
				}

		  } else {
			  adapter = new DLBAdapter();
		  }
		  String name = (String) name_vec.elementAt(i);
		  String version = null;
		  String tag = null;

		  Boolean canConnect = null;

		  Vector<String> v = adapter.getAvailableCodingSchemeTags(name, null);
		  if (v.size() == 0 || !v.contains("PRODUCTION"))
		  {
			  version = adapter.getVocabularyVersionByTag(name, tag);
			  //GF9072
			  //adapter.setVocabulary(name, version, tag);
			  canConnect = adapter.setVocabulary(name, version, tag);
		  }
		  else
		  {
		      if (v.contains("PRODUCTION"))
		      {
				  tag = "PRODUCTION";
				  version = adapter.getVocabularyVersionByTag(name, tag);
				  //GF9072
				  //adapter.setVocabulary(name, version, tag);
				  canConnect = adapter.setVocabulary(name, version, tag);
			  }
			  else
			  {
				  version = adapter.getVocabularyVersionByTag(name, tag);
				  //GF9072
				  //adapter.setVocabulary(name, version, tag);
				  canConnect = adapter.setVocabulary(name, version, tag);

			  }
		  }
		  //GF9072
		  if (canConnect != null && canConnect.equals(Boolean.TRUE))
		  {
		  	  hmap.put(name, adapter);
		  }
	  }
	  return hmap;
  }




	/**
	 * Gets the name of the default search algorithm.
	 *
	 * @return the name of the default search algorithm.
	 *
	 * @since DLBExt 1.0
	 */
	public String getDefaultAlgorithm()
	{
		return defaultAlgorithm;
	}

	/**
	 * Sets the name of the default search algorithm.
	 *
	 * @param algorithm the name of the default search algorithm
	 *
	 * @since DLBExt 1.0
	 */
	public void setDefaultAlgorithm(String algorithm)
	{
		defaultAlgorithm = algorithm;
	}

	/**
	 * Sets vocabulary name.
	 *
	 * @param name vocabulary name
	 *
	 * @return boolean variable, true if successful; false, otherwise
	 *
	 * @since DLBExt 1.0
	 */
    private String localName2FormalName(String name)
	{
        //return localName2FormalName(name, version, tag);
        return localName2FormalName(name, null, null);
	}

	/**
	 * Local name2 formal name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 *
	 * @return the string
	 */
	private String localName2FormalName(String codingSchemeName, String vers, String ltag)
	{
		try {
			if (lbSvc == null)
			{
				return null;
			}

			CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
			String formalName = cs.getFormalName();
			if (formalName.compareTo(codingSchemeName) == 0) return codingSchemeName;

			Vector localnames = array2Vector(cs.getLocalName());
			for (int j=0; j<localnames.size(); j++)
			{
				 String localname = (String) localnames.elementAt(j);
				 if (localname.compareTo(codingSchemeName) == 0)
				 {
					 return cs.getFormalName();
				 }
			}

		} catch (Exception e) {
			getLogger().warn("Error in localName2FormalName");
		}
		return null;
	}

//	used
    /**
 * Sets vocabulary (coding scheme) name.
 *
 * @param name coding scheme name
 *
 * @return a boolean variable - true, if successful; false, if otherwise.
 *
 * @since DLBExt 1.0
 */
    public Boolean setVocabulary(String name)
    {
		String defaultVersion = getVocabularyVersionByTag(name, defaultTag);
        Boolean ret_obj = setVocabulary(name, defaultVersion, defaultTag);
        return ret_obj;
	}


	/**
	 * Sets vocabulary name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 *
	 * @return boolean variable; set to true, if successful; false, otherwise
	 *
	 * @since DLBExt 1.0
	 */
	public Boolean setVocabulary(String codingSchemeName, String vers, String ltag)
	{
		// GF9072
		Boolean isActive = isCodingSchemeActive(lbSvc, codingSchemeName, vers);
		if (isActive == null || isActive.equals(Boolean.FALSE)) return Boolean.FALSE;

        if (ltag != null && ltag.compareTo("") != 0)
        {
		    Vector tags = getAvailableCodingSchemeTags(codingSchemeName, vers);
			if (tags.size() == 0)
			{
				getLogger().info("No tag is found for coding scheme " + codingSchemeName);
				return Boolean.FALSE;
			}
			else
			{
				if (!tags.contains(ltag))
				{
					getLogger().warn("Tag " + ltag + " is not found.");
					return Boolean.FALSE;
				}
		    }
		}

        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
        defaultCodingScheme = null;
        try {
         	defaultCodingScheme = lbSvc.resolveCodingScheme(codingSchemeName, versionOrTag);
         	codingScheme = defaultCodingScheme.getCodingScheme();

 		} catch (Exception e) {
 			e.printStackTrace();
		}
        //if (codingScheme == null || codingScheme.compareTo("") == 0)
        if (defaultCodingScheme == null) return Boolean.FALSE;

        //this.codingScheme = localName2FormalName(name, version, tag);//name;
        this.version = vers;
        this.tag = ltag;

		return Boolean.TRUE;
	}


    /**
     * Gets the formal coding scheme name.
     *
     * @param name the name
     *
     * @return the formal coding scheme name
     */
    public String getFormalCodingSchemeName(String name)
    {
		return localName2FormalName(name);
	}


    /**
     * Gets the vocabulary description.
     *
     * @param name the name
     *
     * @return the vocabulary description
     */
    public String getVocabularyDescription(String name)
    {
		return localName2FormalName(name);
	}


    /**
     * Gets all local names of a coding scheme.
     *
     * @param codingSchemeName coding scheme name
     *
     * @return all local names of a coding scheme.
     *
     * @since DLBExt 1.0
     */
    public Vector getLocalNames(String codingSchemeName)
    {
		CodingScheme cs = getCodingScheme(codingSchemeName, null, null);
		if (cs == null) return null;
		return array2Vector(cs.getLocalName());
	}


	/**
	 * Gets the namespace ID corresponding to a given coding scheme or vocabulary name.
	 *
	 * @param namespaceName a given coding scheme or vocabulary name
	 *
	 * @return the namespace ID corresponding to a given coding scheme or
	 * vocabulary name
	 *
	 * @since DLBExt 1.0
	 * @deprecated - evsapi4 does not have namespace ID's
	 */@Deprecated
	public int getNamespaceId(String namespaceName)
	{
        Integer int_obj = getNamespaceId(namespaceName, null, null);
        if (int_obj == null) return -1;
        return int_obj.intValue();
	}

	/**
	 * Finds formal name of a coding scheme for a given namespace code.
	 * @param namespaceCode namespace code (a local name of a coding scheme)
	 *
	 * @return formal name of a coding scheme for a given namespace code.
	 *
	 * @since DLBExt 1.0
	 * @deprecated - evsapi4 no longer supports namespace ID's
	 */@Deprecated
	public String findNamespaceByCode(String namespaceCode)
	{
		return localName2FormalName(namespaceCode);
	}

	/**
	 * Gets namespace id (local name of a vocabulary name).
	 *
	 * @param namespaceName coding scheme or vocabulary name
	 * @param vers coding scheme or vocabulary version
	 * @param ltag coding scheme or vocabulary tag
	 *
	 * @return namespace id (local name of a vocabulary name)
	 *
	 * @since DLBExt 1.0
	 * @deprecated - evsapi4 no longer supports namespace id's
	 */@Deprecated
	public Integer getNamespaceId(String namespaceName, String vers, String ltag)
	{
		CodingScheme cs = getCodingScheme(namespaceName, vers, ltag);
		if (cs == null) return null;
		String[] names = cs.getLocalName();

		if (names.length == 0)
		{
			return null;
		}

		String name = null;
		for (int i=0; i<names.length; i++)
		{
			name = names[i];
			boolean isInteger = true;
			for (int j=0; j<names.length; j++)
			{
				char ch = name.charAt(j);
				if (!Character.isDigit(ch))
				{
					isInteger = false;
					break;
				}
			}
			if (isInteger)
			{
				int id = Integer.parseInt(name);
				return new Integer(id);
			}
		}
		return null;
	}


    /**
 * Checks if a coded entry (concept) has any subconcepts.
 *
 * @param conceptCode concept code
 *
 * @return boolean TRUE if it has; FALSE, if otherwise
     *
     * @since DLBExt 1.0
     *
     * @deprecated - use hasChildren(String)
     */@Deprecated
    public boolean hasSubconcepts(String conceptCode)
    {
		Boolean Bool_obj = hasChildren(conceptCode);
		if (Bool_obj == null) return false;
		return Bool_obj.booleanValue();
	}

//  used
	/**
 * Checks if a coded entry (concept) has any subconcepts.
 *
 * @param conceptCode concept code
 *
 * @return Boolean TRUE if it has; FALSE, if otherwise
 *
 * @since DLBExt 1.0
 */
	public Boolean hasChildren(String conceptCode)
    {
//		return getFetchedHasSubs(conceptCode);

	    if (conceptCode == null)
	    {
			return Boolean.FALSE;
		}

        //String associationName = IS_A;
		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);

		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

        Boolean bVal = hasAssociatedConcepts(codingScheme,
        		                     conceptCode,
									 direction,
									 is_a);

		return bVal;
    }


/**
	 * Check if the first concept is a subconcept of the second concept.
	 *
	 * @param code1 the code1
	 * @param code2 the code2

      * @return a boolean. True if concept1 is a subconcept of concept2, False
      * if concept1 isn't a sub concept of concept2;
      *
      * @since DLBExt 1.0
      * @deprecated - use isChild(String,String)
      */@Deprecated
     public boolean isSubConcept(java.lang.String code1,
		                         java.lang.String code2)
	 {
		Boolean Bool_obj = isChild(code1, code2);
		if (Bool_obj == null) return false;
		return Bool_obj.booleanValue();
	 }


//   used
	/**
 * Check if the first concept is a subconcept of the second concept.
 *
 * @param code1 the code1
 * @param code2 the code2
 *
 * @return Boolean TRUE if it is; FALSE, if otherwise
 *
 * @since DLBExt 1.0
 */
	public Boolean isChild(java.lang.String code1,
	                       java.lang.String code2)

    {
		try{

//			String code1 = nameToCode(codingScheme, version, tag, concept1);
			if (code1 == null)
			{
//				getLogger().warn("Concept " + concept1 + " not found.");
				return null;
			}

//			String code2 = nameToCode(codingScheme, version, tag, concept2);
			if (code2 == null)
			{
//				getLogger().warn("Concept " + concept2 + " not found.");
				return null;
			}

			//String associationName = IS_A;
			Boolean isValidCode = isConceptCodeValid(code1);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code1);
				return null;
			}
			isValidCode = isConceptCodeValid(code2);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code2);
				return null;
			}
		boolean direction = false;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

			//String associationName = getHasSubtype(codingScheme);
			return areAssociatedConcepts(codingScheme, version, tag,
										code1,
										code2,
										direction,
										is_a);
		}
		catch (Exception e){
			getLogger().error("Exception in DLBAdapter.isChild() " + e.getStackTrace());
			return Boolean.FALSE;
		}
    }


	/**
	 * Checks if concept with code1 is a subconcept of the concept with code2.
	 *
	 * @param code1 the first concept code
	 * @param code2 the second concept code
	 *
	 * @return boolean, true, if it is; false, otherwise
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use isSubConcept(String,String)
	 */@Deprecated
	public boolean isSubConceptByCode(java.lang.String code1, java.lang.String code2)
	{
	    Boolean Bool_val = isChildByCode(code1, code2);
	    if (Bool_val == null) return false;

	    if (Bool_val.equals(Boolean.TRUE)) return true;
	    return false;
	}


	/**
	 * Checks if concept with code1 is a subconcept of the concept with code2.
	 *
	 * @param code1 the first concept code
	 * @param code2 the second concept code
	 *
	 * @return boolean, true, if it is; false, otherwise
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use isChild(String,String)
	 */@Deprecated
	public Boolean isChildByCode(java.lang.String code1,
								 java.lang.String code2)
	{
		try {
			Boolean isValidCode = isConceptCodeValid(code1);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code1);
				return null;
			}
			isValidCode = isConceptCodeValid(code2);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code2);
				return null;
			}

			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;


			return areAssociatedConcepts(codingScheme,
										version, tag,
										code1,
										code2,
										direction,
										is_a);
		} catch (Exception e) {
			getLogger().error("Exception in DLBAdapter.isChildByCode() " + e.getStackTrace());
			return Boolean.FALSE;
		}
	}



    /**
     * Gets all CodedEntry instances corresponding to subconcepts of a given concept.
     *
     * @param code a concept code
     *
     * @return a Vector containing all CodedEntry instances corresponding to
     * subconcepts of a given concept.
     *
     * @since DLBExt 1.0
     */
    private Vector<CodedEntry> getAllSubconcepts(String code)
	{

		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0 ) direction = !direction;


//        return getAssociatedCodedEntries(codingScheme,
//										 version,
//										 tag,
//	                                     code,
//										 null,
//										 direction,
//										 //IS_A,
//										 is_a,
//										 -1,
//										 -1
//										 );

        return getAssociatedCodedEntries(codingScheme,
        		version,
        		tag,
        		code,
        		direction,
        		is_a);
	}

//  used
	/**
 * Gets the code of all subconcepts of the conceptcode argument.
 * @param code the code
 *
 * @return an array of Concept code which are sub concepts of conceptcode;
 * return null, if connection fails or an exception is thrown.
 *
 * @since DLBExt 1.0
 * @deprecated - evsapi4 no longer supports name as an identifier.  Use getAllSubConcepts(String)
 */@Deprecated
	public Vector getAllSubConceptNames(java.lang.String code)
    {
//		String code = nameToCode(codingScheme, version, tag, conceptName);

		if (code == null) return null;
        Vector<CodedEntry> w = getAllSubconcepts(code);
        if (w == null) return null;

        try {
			Vector<String> v = new Vector<String>();
			for (int i=0; i<w.size(); i++)
			{
				CodedEntry ce = (CodedEntry) w.elementAt(i);
				//v.add(codeToName(codingScheme, ce.getConceptCode()));
//				v.add(codeToName(codingScheme, version, ce.getConceptCode()));
				v.add(ce.getConceptCode());
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
    }


    /**
     * Gets the codes, or the names, of all subconcepts of the specified concept.
     *
     * @param inputvalue input value
     * @param inputcodeObj Boolean TRUE if the input value is a concept
     * code; FALSE, if it's a name
     * @param outputcodeObj Boolean TRUE if the output value is a
     * concept code; FALSE, if it's a name
     *
     * @return a vector containing the codes, or the names, of all subconcepts
     * of the specified concept; return null, if connection fails or an
     * exception is thrown.
     *
     * @since DLBExt 1.0
     * @deprecated - evsapi4 no longer supports name as identifier. Use getSubConcepts(String)
     */@Deprecated
    public Vector<String> getSubConcepts(String inputvalue, Boolean inputcodeObj, Boolean outputcodeObj)
    {
		return getSubConcepts(inputvalue, inputcodeObj.booleanValue(),
		                      outputcodeObj.booleanValue());
	}

    /**
     * Gets the sub concepts.
     *
     * @param conceptCode the concept code
     *
     * @return the sub concept codes
     */
    public Vector<String> getSubConcepts(String conceptCode)
    {
    	try{
    		boolean direction = true;
    		String is_a = getHasSubtype(codingScheme);
    		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0)
    		{
				direction = !direction;   //TODO is this always true?
			}


    		Vector<String> w = findRelatedCodes(codingScheme, version, tag, conceptCode, is_a, direction);

    		Vector<String> v = new Vector<String>();

    		if (w == null)
    		{
    			getLogger().warn("findRelatedCodes returns null.");
    			return null;
    		}
    		else if (w.size() == 0)
    		{
    			return v;
    		}
    		return w;
    	} catch (Exception e) {
    		getLogger().error("getSubConcepts threw exception.");
    		return null;
    	}
    }

//  used
    /**
 * Gets the codes, or the names, of all subconcepts of the specified concept.
 *
 * @param inputvalue input value
 * @param inputcode parameter not used
 * @param outputcode parameter not used
 *
 * @return a vector containing the codes, or the names, of all subconcepts
 * of the specified concept; return null, if connection fails or an
 * exception is thrown.
 *
 * @deprecated - evsapi4 no longer supports name as identifier. Use getSubConcepts(String)
 */@Deprecated
    public Vector<String> getSubConcepts(String inputvalue,
    		@SuppressWarnings("unused")	boolean inputcode,
    		@SuppressWarnings("unused")	boolean outputcode)
	{
		try {
			String code = inputvalue;

//			if (!inputcode)
//			{
//				code = nameToCode(codingScheme, version, tag, inputvalue);
//			}

			//String relation = IS_A;
			//String relation = getHasSubtype(codingScheme);
            boolean direction = true;
            String is_a = getHasSubtype(codingScheme);

            if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

			Vector<String> w = findRelatedCodes(codingScheme, version, tag, code, is_a, direction);

			Vector<String> v = new Vector<String>();

			if (w == null)
			{
				getLogger().warn("findRelatedCodes returns null.");
				return null;
			}
			else if (w.size() == 0)
			{
				return v;
			}

//			if (outputcode)
//			{
				return w;
//			}

//			for (int i=0; i<w.size(); i++)
//			{
//				String t = (String) w.elementAt(i);
//
//				t = findConceptNameByCode(codingScheme,
//									   version,
//									   t);
//				v.add(t);
//			}
//			return v;
	    } catch (Exception e) {
			getLogger().error("findRelatedCodes threw exception.");
			return null;
		}
	}

    /**
     * Checks if a concept has any superconcept.
     *
     * @param conceptCode the concept code
     *
     * @return a Boolean. true - conceptCode has any superconcept; false - it
     * has no superconcept; null, if connection fails or an exception is
     * thrown.
     *
     * @since DLBExt 1.0
     * @deprecated - use hasPatents(String)
     */@Deprecated
    public boolean hasSuperconcepts(String conceptCode)
    {
	    Boolean Bool_val = hasParents(conceptCode);
	    if (Bool_val == null) return false;
	    return Bool_val.booleanValue();
	}

//  used
	/**
 * Checks if a concept has any super concepts.
 *
 * @param conceptCode concept code
 *
 * @return Boolean TRUE if the concept has; FALSE, if otherwise
 *
 * @since DLBExt 1.0
 */
	public Boolean hasParents(String conceptCode)
    {
//		return getFetchedHasSups(conceptCode);

	    if (conceptCode == null) return Boolean.FALSE;
        //String associationName = IS_A;
        //String associationName = getHasSubtype(codingScheme);
			boolean direction = false;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;


        Boolean bVal = hasAssociatedConcepts(codingScheme,
        		                     conceptCode,
									 direction,
									 is_a);

		if (bVal == null)
		{
			getLogger().warn("WARNING: ConvenienceMethods getFetchedHasSups returns null.");
		}

		return bVal;
    }


    /**
     * Checks if a concept has any super concepts.
     *
     * @param inputvalue concept name
     * @param isCode concept name
     *
     * @return boolean TRUE if the concept has; FALSE, if otherwise
     *
     * @deprecated - evsapi4 does not support name as an identifier. Use hasParents(String)
     */@Deprecated
    public boolean hasSuperconcepts(String inputvalue, boolean isCode)
    {
	    Boolean Bool_val = hasParents(inputvalue, isCode);
	    if (Bool_val == null) return false;
	    return Bool_val.booleanValue();
	}

	/**
	 * Checks if a concept has any super concepts.
	 *
	 * @param inputvalue input value
	 * @param isCode boolean TRUE if the input value is a concept code;
	 * FALSE, if it's a name
	 *
	 * @return Boolean TRUE if the concept has; FALSE, if otherwise
	 *
	 * @since DLBExt 1.0
	 * @deprecated - evsapi4 does not support name as an identifier. Use hasParents(String)
	 */@Deprecated
	public Boolean hasParents(String inputvalue, boolean isCode)
    {
		return getFetchedHasSups(inputvalue, isCode);

    }

	   /**
   	 * Gets the codes of all superconcepts of the specified concepts.
   	 *
   	 * @param code the code
   	 *
   	 * @return a vector containing the codes of all
   	 * superconcepts of the specified concepts; return null, if an
   	 * exception is thrown.
   	 *
   	 * @since DLBExt 1.0
   	 */
	public Vector<String> getSuperConcepts(String code)
	{
		try{


			if (code == null)
			{
				System.out.println("WARNING: getSuperConcepts -- Code " + code + " not identifiable.");
			}


			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);

			if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

			Vector<String> w = findRelatedCodes(codingScheme, version, tag, code, is_a, direction);
			Vector<String> v = new Vector<String>();

			if (w == null)
			{
				return null;
			}
			else if (w.size() == 0)
			{
				return v;
			}

			return w;

		} catch (Exception e) {
			return null;
		}

	}


   /**
    * Gets the codes (or the names) of all superconcepts of the specified concepts.
    *
    * @param inputvalue code or the specified concept name
    * @param inputcodeObj parameter not used
    * @param outputcodeObj parameter not used
    *
    * @return a vector containing the codes (or the names) of all
    * superconcepts of the specified concepts; return null, if an
    * exception is thrown.
    *
    * @since DLBExt 1.0
    * @deprecated - evsapi4 does not support name as an identifier.  Use getSuperconcepts(String)
    */@Deprecated
//	used
   public Vector<String> getSuperConcepts(String inputvalue,
		   @SuppressWarnings("unused")Boolean inputcodeObj,
		   @SuppressWarnings("unused")Boolean outputcodeObj)
   {
	   return getSuperConcepts(inputvalue);
//	   return getSuperConcepts(inputvalue, inputcodeObj.booleanValue(), outputcodeObj.booleanValue());
   }
// used
   /**
 * Gets the codes (or the names) of all superconcepts of the specified concepts.
 *
 * @param inputvalue code or the specified concept name
 * @param inputcode parameter not used
 * @param outputcode parameter not used
 * code; FALSE, if it's a name
 *
 * @return the codes (or the names) of all superconcepts of the specified
 * concepts.
 *
 * @since DLBExt 1.0
 * @deprecated - evsapi4 does not support name as an identifier.  Use getSuperconcepts(String)
 */@Deprecated
   public Vector<String> getSuperConcepts(String inputvalue,
		   @SuppressWarnings("unused")boolean inputcode,
		   @SuppressWarnings("unused")boolean outputcode)
   {
	   return getSuperConcepts(inputvalue);
//		try {
//			String code = inputvalue;
////			if (!inputcode)
////			{
////				code = nameToCode(codingScheme, version, tag, inputvalue);
////			}
//
//			if (code == null)
//			{
//				System.out.println("WARNING: getSuperConcepts -- Code " + inputvalue + " not identifiable.");
//			}
//
//
//            boolean direction = false;
//            String is_a = getHasSubtype(codingScheme);
//
//            if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;
//
//			Vector<String> w = findRelatedCodes(codingScheme, version, tag, code, is_a, direction);
//			Vector<String> v = new Vector<String>();
//
//			if (w == null)
//			{
//				return null;
//			}
//			else if (w.size() == 0)
//			{
//				return v;
//			}
//
//			if (outputcode)
//			{
//				return w;
//			}
//
//			for (int i=0; i<w.size(); i++)
//			{
//				String t = (String) w.elementAt(i);
//				//t = findConceptNameByCode(codingScheme, version, tag, t);
//				t = codeToName(codingScheme, version, tag, t);
//				v.add(t);
//			}
//			return v;
//	    } catch (Exception e) {
//			return null;
//		}
	}


//	/**
//	 * Gets associated concepts.
//	 *
//	 * @param code concept code
//	 * @param relation association name
//	 *
//	 * @return a Vector of AssociatedConcept objects
//	 *
//	 * @since DLBExt 1.0
//	 */
//	private Vector getAssociatedConcepts( String code, String relation)
//    {
//        return getAssociatedConcepts(
//			                               codingScheme,
//			                               version,
//			                               tag,
//	                                       code,
//	                                       relation);
//    }


	/**
	 * Gets tree data from the server (method for supporting DTS Browser development).
	 *
	 * @param code the code of the root concept
	 * @param directionB - parameter not used
	 * @param levels - the depth of the tree
	 * @param propertyname - parameter not used
	 *
	 * @return The root (DefaultMutableTreeNode) of the tree
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use getTree(String)
	 */@Deprecated
	// For browser use
	public DefaultMutableTreeNode getTree(String code,
							@SuppressWarnings("unused")	boolean directionB,
							int levels,
							@SuppressWarnings("unused")	String propertyname
							)
	{
//		String code = nameToCode(codingScheme, version, tag, rootname);

		if (code == null) return null;

		int ASD_index = 0;

		CodedEntry ce = getConcept(code, ASD_index);
		TreeNode node = new TreeNode();
		node.setConcept(ce);
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(node);

		dmtn = populateTreeNodes(dmtn);
		return dmtn;

	}

	/**
	 * Gets tree data from the server (method for supporting DTS Browser development).
	 *
	 * @param code the code of the root concept
	 *
	 * @return The root (DefaultMutableTreeNode) of the tree
	 *
	 * @since DLBExt 1.0
	 */
	// For browser use
	public DefaultMutableTreeNode getTree(String code)
	{
//		String code = nameToCode(codingScheme, version, tag, rootname);

		if (code == null) return null;

		int ASD_index = 0;

//		CodedEntry ce = getConcept(code, ASD_index);
		CodedEntry ce = getConcept(code);
		TreeNode node = new TreeNode();
		node.setConcept(ce);
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(node);

		dmtn = populateTreeNodes(dmtn);
		return dmtn;

	}

	/**
	 * Populate the content of a TreeNode instance (the UserObject instance of a DefaultMutableTreeNode).
	 *
	 * @param dmtn a  DefaultMutableTreeNode instance
	 *
	 * @return a  DefaultMutableTreeNode instance
	 *
	 * @since DLBExt 1.0
	 */
	private DefaultMutableTreeNode populateTreeNodes(DefaultMutableTreeNode dmtn)
	{
		TreeNode nd = (TreeNode) dmtn.getUserObject();
		CodedEntry ce = nd.getConcept();
		String code = ce.getConceptCode();
		boolean buildReferencedEntries = true;

		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

        Vector v = getAssociatedCodedEntries(codingScheme,
                                           version, //null
                                           is_a,
                                           code,
                                           direction,
                                           buildReferencedEntries);

        if (v == null || v.size() == 0)
        {
			return dmtn;
		}

        //int ASD_index = 0;
        for (int i=0; i<v.size(); i++)
        {
			CodedEntry c = (CodedEntry) v.elementAt(i);
			if (c != null)
			{
				String subconcept_code = c.getConceptCode();
				//String subconcept_name = codeToName(codingScheme, version, subconcept_code);

				CodedEntry child_concept = new CodedEntry();
				child_concept.setConceptCode(subconcept_code);

				TreeNode node = new TreeNode();
				node.setConcept(child_concept);

				DefaultMutableTreeNode childnode = new DefaultMutableTreeNode(node);
				childnode = populateTreeNodes(childnode);
				dmtn.add(childnode);
		    }
		}
		return dmtn;
	}



	//////////////////////////////////////////////////////////////////////////////////////////
	//  SORT
	//////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * A private method for performing quick sort.
	 *
	 * @param a input vector of strings
	 * @param lo0 initial low value
	 * @param hi0 initial high value //081403
	 *
	 * @since DLBExt 1.0
	 */
	@SuppressWarnings("unused")
	private void QuickSort(Vector<String> a, int lo0, int hi0)
	{
		int lo = lo0;
		int hi = hi0;
		String mid;

		if ( hi0 > lo0)
		{
			mid = (String) a.elementAt( ( lo0 + hi0 ) / 2 );
			while( lo <= hi )
			{
				while(lo < hi0)
				{
					String s_lo = (String) a.elementAt(lo);
					if (s_lo.compareTo(mid) < 0)
						++lo;
					else
						break;
				}

				while(hi > lo0)
				{
					String s_hi = (String) a.elementAt(hi);
					if (s_hi.compareTo(mid) > 0)
						--hi;
					else
						break;
				}

				if( lo <= hi )
				{
					swap(a, lo, hi);
					++lo;
					--hi;
				}
			}
			if( lo0 < hi )
				QuickSort( a, lo0, hi );

			if( lo < hi0 )
				QuickSort( a, lo, hi0 );

		}
	}

	/**
	 * A private method for swapping two string elements in a vector.
	 *
	 * @param a input vector of strings
	 * @param i first position
	 * @param j second position
	 *
	 * @since DLBExt 1.0
	 */
	private void swap(Vector<String> a, int i, int j)
	{
		String T;
		T = (String) a.elementAt(i);
		a.setElementAt(a.elementAt(j), i);
		a.setElementAt(T, j);
	}

//	/**
//	 * A private method for QuickSort.
//	 *
//	 * @param a an array of strings
//	 *
//	 * @since DLBExt 1.0
//	 */
//	private void sort(Vector<String> a)
//	{
//		if (a.size() == 0)
//			return;
//
//		QuickSort(a, 0, a.size() - 1);
//	}

	//112603
//	/**
//	 * Convert an object to a byte array.
//	 *
//	 * @param ob an Object instance
//	 *
//	 * @return a byte array
//	 *
//	 * @since DLBExt 1.0
//	 * @since.lexrpc 1.3.3
//	 */
//	private byte[] objectToByteArray(Object ob)
//	{
//		if (ob == null) return null;
//
//		byte[] byteNode= null;
//		try	{
//			ByteArrayOutputStream baos=new ByteArrayOutputStream();
//			ObjectOutputStream oos=new ObjectOutputStream(baos);
//			oos.writeObject(ob); // Serialize the Class object
//			byteNode=baos.toByteArray();
//			oos.close();
//		}catch (Exception e){
//			e.printStackTrace();
//		}
//		return byteNode;
//	}




	/**
	 * Gets the Concept intance corresponding to a specific concept code.
	 *
	 * @param code concept code
	 * @param ASD_Index parameter not used
	 *
	 * @return the Concept intance corresponding to a specific concept code
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use getConcept(String)
	 */@Deprecated
	public CodedEntry getConcept(String code,@SuppressWarnings("unused") int ASD_Index)
	{
//        return getCodedEntry(codingScheme,
//                                    version,
//	                                tag,
//									code,
//									ASD_Index);
        return getCodedEntry(codingScheme,version,tag,code);

	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//  Utility
	//////////////////////////////////////////////////////////////////////////////////////////


//	/**
//	 * Converts a java.util.date to a java.sql.date
//	 *
//	 * @param utildate a java.util.date object
//	 *
//	 * @return java.sql.Date
//	 */
//	private java.sql.Date toSqldate(Date utildate)
//	{
//		if (utildate == null) return null;
//
//		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//		String s = sdf.format(utildate);
//		return java.sql.Date.valueOf(s);
//	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//  HISTORY
	//////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Checks if a concept code is valid.
	 *
	 * @param code concept code
	 *
	 * @return boolean true - concept code is valid; false - if it is not.
	 */
	public Boolean isConceptCodeValid(String code){
		try {
            return isCodeInGraph( codingScheme,
								          version,
								          tag,
								          code);

       }catch (Exception e) {
			getLogger().error("ConvenienceMethods isConceptCodeValid "+e);
		}
		return null;
	}

	/**
	 * Checks if a concept name is valid.
	 *
	 * @param conceptname concept name
	 *
	 * @return boolean true - concept name is valid; false - if it is not.
	 * @deprecated - evsapi4 does not support name as an identifier
	 */@Deprecated
	public Boolean isConceptNameValid(String conceptname){
		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null)
		{
			return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

	/**
	 * Checks if a concept history is available.
	 *
	 * @return boolean true - if concept history is available; false - if it is
	 * not.
	 */
	public boolean isHistoryAvailable(){
		//if (lbSvc != null) return true;
		//072807
		Vector v = getHistoryDates();
		if (v == null || v.size() == 0) return false;
		return true;
	}


	/**
	 * Checks if is history available.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return true, if is history available
	 */
	public boolean isHistoryAvailable(String codingSchemeName){
		//if (lbSvc != null) return true;
		//072807
		Vector v = getHistoryBaselineDates(codingSchemeName);
		if (v == null || v.size() == 0) return false;
		return true;
	}

	/**
	 * Checks if a concept in a namespace has a specific role
	 * and if so whether the role has a specific value.
	 *
	 * @param role_name role name
	 * @param role_value role value
	 * @param Concept_in the name of the specific ontylog concept
	 * @param isCode whether the concept_in is a name or a code
	 *
	 * @return A Boolean variable with value set to TRUE if it has; FALSE,
	 * otherwise
	 * @deprecated - evsapi does not support name as identifier. Use containsRole(String,String,String)
	 */@Deprecated
    public Boolean containsRole(String role_name, String role_value,
					            String Concept_in, boolean isCode)
	{
		if (!hasRelation(codingScheme, ROLE_RELATION)) return Boolean.FALSE;

		Vector v = getFetchedRoles(Concept_in, isCode);
		if (v == null) return null;

		//Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			Role r = (Role) v.elementAt(i);
			String s = r.getName() + "$" + r.getValue();
			if (s.compareTo(role_name + "$" + role_value) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}


//Used
	/**
 * Contains role.
 *
 * @param role_name the role_name
 * @param role_value the role_value
 * @param Concept_in the conceptCode
 *
 * @return the boolean
 */
public Boolean containsRole(String role_name, String role_value,
					String Concept_in)
	{
//		return containsRole(role_name, role_value, Concept_in, false);
		if (!hasRelation(codingScheme, ROLE_RELATION)) return Boolean.FALSE;

		Vector v = getFetchedRoles(Concept_in);
		if (v == null) return null;

		//Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			Role r = (Role) v.elementAt(i);
			String s = r.getName() + "$" + r.getValue();
			if (s.compareTo(role_name + "$" + role_value) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}


	/**
	 * Checks if a ontylog concept in a namespace has a specific inverse role
	 * and if so whether the role has a specific value.
	 *
	 * @param role_name role name
	 * @param role_value role value
	 * @param Concept_in the name of the specific ontylog concept
	 * @param isCode the is code
	 *
	 * @return A Boolean variable with value set to TRUE if it has; FALSE,
	 * otherwise
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use containsInverseRole(String,String,String)
	 */@Deprecated
	public Boolean containsInverseRole(String role_name, String role_value,
					                   String Concept_in, boolean isCode)
	{
		if (!hasRelation(codingScheme, ROLE_RELATION)) return Boolean.FALSE;

		Vector v = getFetchedInverseRoles(Concept_in, isCode);
		if (v == null) return null;

		//Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			Role r = (Role) v.elementAt(i);
			String s = r.getName() + "$" + r.getValue();
			if (s.compareTo(role_name + "$" + role_value) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;

		/*
		String s = role_name + "$" + role_value;
		if (v.contains(s))
		{
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
		*/
	}


//Used
	/**
 * Contains inverse role.
 *
 * @param role_name the role_name
 * @param role_value the role_value
 * @param Concept_in the concept_in
 *
 * @return the boolean
 */
public Boolean containsInverseRole(String role_name, String role_value,
					                   String Concept_in)
	{
		return containsInverseRole(role_name, role_value, Concept_in, false);
	}


	/**
	 * Finds the code of the specified concept.
	 *
	 * @param conceptname the specified concept name
	 *
	 * @return the specified concept code; return null, if connection fails or an
	 * exception is thrown.
	 *
	 * @deprecated - evsapi4 does not support name as an identifier
	 * @since DLBExt 1.0
	 */@Deprecated
	public String getConceptCodeByName(java.lang.String conceptname)
    {
		return nameToCode(codingScheme, version, tag, conceptname);
    }
 /**
 * Finds the name of the specified concept.
 *
 * @param conceptCode concept code
 *
 * @return the specified concept entity description; return null, if connection fails or an
 * exception is thrown.
 *
 * @since DLBExt 1.0
 */
	public String getConceptNameByCode(java.lang.String conceptCode)
    {
//		return codeToName(codingScheme, version, tag, conceptCode);
//		return getConceptName(codingScheme,version,tag,conceptCode);
//      GF13917
        return findDescription(codingScheme, null, conceptCode);
    }

	//////////////////////////////////////////////////////////////////////////////////////////
	//  ROLE RELATIONSHIPS
	//////////////////////////////////////////////////////////////////////////////////////////
    /**
	 * Gets the names of all role types defined in the vocabulary.
	 *
	 * @return a vector containing the names of all role types defined in the
	 * vocabulary; null, if connection fails or an exception is thrown.
	 *
	 * @since DLBExt 1.0
	 */
//	used
    public Vector getAllRoleNames()
	{
		return getSupportedRoleNames(codingScheme);
	}

    /**
     * Gets the all role names.
     *
     * @param codingSchemeName the coding scheme name
     *
     * @return the all role names
     */
    public Vector getAllRoleNames(String codingSchemeName)
	{
		return getSupportedRoleNames(codingSchemeName);
	}


	/**
	 * Gets the values of a particular role owned by the specified concept.
	 *
	 * @param code the specified concept code
	 * @param rolename the specified role name
	 *
	 * @return the values of a particular role owned by the specified concept;
	 * null, if connection fails or an exception is thrown.
	 *
	 * @since DLBExt 1.0
	 */
	public Vector getRoleValues(String code,
	                            String rolename)
    {
//		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null) return null;
//		String ltag = null;

  	    Vector<Role> w = getRoles(codingScheme,
										  null,
										  null,
										  code,
										  rolename);

        if (w == null) return null;
        Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
		    NameAndValue nv = w.elementAt(i).toNameAndValue();
	        v.add(nv.getContent());
		}
		return v;
    }

//	used
	/**
 * Gets the names and values of roles owned by the specified concept.
 * Each name and value pair is returned as a string delimited by a '$' sign (i.e., name$value).
 *
 * @param conceptname the specified concept name
 *
 * @return a vector containing the names and values of roles owned by a
 * specified concept; null, if connection fails or an exception is
 * thrown.
 *
 * @since DLBExt 1.0
 * @deprecated use getRolesByConceptCode(String conceptcode)
 */@Deprecated
	public Vector<Role> getRolesByConceptName(String conceptname)
    {
		return getFetchedRoles(conceptname);
	}

//	used
	/**
 * Gets the names and values of roles owned by the specified concept.
 * Each name and value pair is returned as a string delimited by a '$' sign (i.e., name$value).
 *
 * @param conceptcode the specified concept code
 *
 * @return a vector containing the names and values of roles owned by a
 * specified concept; null, if connection fails or an exception is
 * thrown.
 *
 * @since DLBExt 1.0
 *
 */
	public Vector<Role> getRolesByConceptCode(String conceptcode)
    {
		return getFetchedRoles(conceptcode);
	}
	//////////////////////////////////////////////////////////////////////////////////////////
	//  PROPERTIES
	//////////////////////////////////////////////////////////////////////////////////////////

    /**
	 * Gets the names of all property types defined in the vocabulary.
	 *
	 * @return a vector containing the names of all property types defined in
	 * the vocabulary; null, if connection fails or an exception is
	 * thrown.
	 *
	 * @since DLBExt 1.0
	 */
    public Vector<String> getPropertyTypes() // a CodingScheme
	{
		/*
		//org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType[] propertytypes = getSupportedPropertyNames(codingScheme, version, null);
		Vector v = new Vector();
		for (int i=0; i<propertytypes.length; i++)
		{
			v.add(propertytypes[i]);
		}
		return v;
		*/
		return getSupportedPropertyNames(codingScheme, version, tag);
	}


	/**
	 * Gets the names and values of roles owned by the specified concept.
	 *
	 * @param conceptcode the specified concept code
	 *
	 * @return a vector containing the names and values of properties owned by a
	 * specified concept; null, if connection fails or an exception is
	 * thrown.
	 *
	 * @since DLBExt 1.0
	 * @deprecated use getPropertiesByCode(String conceptCode)
	 */@Deprecated
//  used
	public Vector<Property> getPropertiesByConceptName(String conceptcode)
	{
//		return getPropertiesByConceptName(codingScheme, version, tag, conceptname);
		return getPropertiesByCode(codingScheme, version, conceptcode);

	}


	/**
	 * Gets the properties by code.
	 *
	 * @param conceptcode the conceptcode
	 *
	 * @return the properties for the given concept code
	 */
	public Vector<Property> getPropertiesByCode(String conceptcode)
	{
		return getPropertiesByCode(codingScheme, version, conceptcode);
	}

//	used
	/**
 * Gets the values of a particular role owned by the specified concept.
 *
 * @param conceptcode the specified concept code
 * @param propertyname property name
 *
 * @return the values of a particular role owned by the specified concept;
 * null, if connection fails or an exception is thrown.
 *
 * @since DLBExt 1.0
 */
	public Vector<String> getPropertyValues(String conceptcode, String propertyname)
    {
        return getPropertyValues(codingScheme,
	                                     version,
									     tag,
									     conceptcode,
									     propertyname);
    }


		/**
		 * Determines if concept1 is a subconcept of concept2 in the specified
		 * namespace. Concept1 and concept2 are concept codes
		 *
		 *
		 * @param concept1 - a concept code
		 * @param concept2 - a concept code
		 *
		 * @return true if concept1 is a subconcept of concept2, false if concept1
		 * isn't a subconcept of concept2 Returns null, if there is an
		 * exception.
		 *
		 * @since DLBExt 1.0
		 * @deprecated - use isSubConcept(String,String)
		 */@Deprecated
		public Boolean isSubConceptByName(String concept1,
						String concept2)
		{
			try{
//				String code1 = nameToCode(codingScheme, version, tag, concept1);
//				String code2 = nameToCode(codingScheme, version, tag, concept2);
				String code1 = concept1;
				String code2 = concept2;

				return isSubConceptByCode(code1, code2);
			}
			catch (Exception e){
				return Boolean.FALSE;
			}
		}



		/**
		 * Determines if concept1 is a subconcept of a collection of concepts in the specified
		 * namespace.
		 *
		 * @param concepts_in a list of concept codes
		 * @param code1 the code1
		 *
		 * @return a vector of Boolean variables indicating whether concept1 is a
		 * subconcept of each concept in the list
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<Boolean> isSubConceptByName(String code1,
						Vector concepts_in)
		{
//            String code1 = nameToCode(codingScheme, version, tag, concept1);
            Vector<Boolean> v = new Vector<Boolean>();
            for (int i=0; i<concepts_in.size(); i++)
            {
//				String name = (String) concepts_in.elementAt(i);
//				String code2 = nameToCode(codingScheme, version, tag, name);
				String code2 = (String) concepts_in.elementAt(i);
		        v.add(isSubConceptByCode(code1, code2));
			}
			return v;
		}



		/**
		 * "@since DLBExt 1.0
		 *
		 * @param associationtypeName the associationtype name
		 *
		 * @return the supported association
		 */
//		private SupportedAssociation findAssociationTypeByName(String associationtypeName) {
//            CodingScheme cs = getCodingScheme(codingScheme, version);
//		    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
//		    for (int i=0; i<assos.length; i++)
//		    {
//				if (assos[i].getContent().compareTo(associationtypeName) == 0) return assos[i];
//			}
//			return null;
//	    }


		/**
		 * Gets an array of Concept objects that represent the same concepts as the argument
		 * array of concept codes.
		 *
		 * @param conceptCodes_in a vector of concept codes
		 *
		 * @return a vector of Concept corresponding to the argument concept codes,
		 * with all attributes filled; it returns null, if there is an
		 * exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<CodedEntry> findConceptsByCode(Vector<String> conceptCodes_in)
		{
			if (conceptCodes_in == null) return null;
            Vector<CodedEntry> v = new Vector<CodedEntry>();
            try {

				for (int i=0; i<conceptCodes_in.size(); i++)
				{
					String code = (String) conceptCodes_in.elementAt(i);
					CodedEntry ce = getCodedEntry(codingScheme, version, tag, code); // asd??? (to be implemented)
					if (ce == null)
					{
						getLogger().warn("WARNING: findConceptsByCode ce is null.");
						v.add(null);
					}
					else
					{
						v.add(ce);
					}
				}

			} catch (Exception e) {
				getLogger().error("ConvenienceMethods findConceptsByCode exception thrown.");
				return null;
			}
			return v;
		}

//		used
		/**
 * Retrieves all concept property types.
 *
 * @return a vector of all supported property types. Returns an empty
 * vector if there are no supported property type. Returns null, if
 * there is an exception.
 *
 * @since DLBExt 1.0
 */
		public Vector getAllConceptPropertyTypes()
		{
			return getConceptPropertyTypes();
		}


		/**
		 * Retrieves all concept property types within a given namespace.
		 *
		 * @return a vector of PropertyType in the namespace. Returns an empty
		 * vector if there are none. Returns null, if
		 * there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<String> getConceptPropertyTypes()
		{
			Vector<SupportedProperty> w = getSupportedProperties(codingScheme, version, tag);
			if (w == null) return null;

			Vector<String> v = new Vector<String>();
			for (int i=0; i<w.size(); i++)
			{
				SupportedProperty sp = (SupportedProperty) w.elementAt(i);
				//v.add(sp.getContent());
				v.add(sp.getLocalId());
			}
			return v;
		}


//////////////////
// History
//////////////////

		/**
 * Gets all distinct dates in concept_history table.
 *
 * @return a vector of Date All distinct dates in concept_history table.
 * Returns null, if there is an exception.
 *
 * @since DLBExt 1.0
 */


		public Vector<Date> getHistoryDates()
		{
			return getHistoryBaselineDates(codingScheme);
		}



		/**
		 * Gets the baseline date of creation of the specified concept.
		 *
		 * @param code concept code
		 *
		 * @return Date The baseline date of creation of inCode from either "Create"
		 * or "Split" edit actions. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */


		public java.util.Date getConceptCreationDate(String code)
		{
			return getConceptCreationDate(codingScheme, code);
		}


		/**
		 * Gets the creation date.
		 *
		 * @param code the code
		 *
		 * @return the creation date
		 */
		public java.util.Date getCreationDate(String code)
		{
			return getConceptCreationDate(codingScheme, code);
		}

//		used
		/**
 * Gets the descendant concept codes of the specified concept. The search extends to a final
 * baseline date starting from an initial baseline date. The search is based on a boolean value. If
 * the value is true, the method only searches for the active concepts at the final baseline date. If
 * the value is false, the method searches for all descendant concepts, whether active or retired.
 * The search is only applied to "split/merge" actions and the specified concept could be the
 * descendant concept of itself.
 *
 * @param inCode The specified concept's code.
 * @param atBaseline The boolean value to specify the search type.
 * @param iBaselineDate_in initial baseline date
 * @param fBaselineDate_in final baseline date
 *
 * @return a vector of String The returned concept codes which are the
 * descendant concepts of inCode. Returns null, if there is an
 * exception.
 *
 * @since DLBExt 1.0
 */
		public Vector<String> getDescendantCodes(
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		    return getDescendantCodes(
				        codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in);
		}



		/**
		 * Gets the descendant concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * not retired in the specified baseline date.
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 *
		 * @return a vector of String The returned concept codes are the descendant
		 * concepts of inCode which are involved in "split or merge" actions
		 * that occurred in baselineDate. Returns null, if there is an
		 * exception.
		 *
		 * @deprecated use getDescendantCodes(String,Date)
		 * @since DLBExt 1.0
		 */@Deprecated
		public Vector<String> getCodeActionChildren(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getCodeActionChildren(
				        codingScheme,
						inCode,
						baselineDate_in);
 		}

		/**
		 * Gets the descendant concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * not retired in the specified baseline date.
		 *
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 *
		 * @return a vector of String The returned concept codes are the descendant
		 * concepts of inCode which are involved in "split or merge" actions
		 * that occurred in baselineDate. Returns null, if there is an
		 * exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<String> getDescendantCodes(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getDescendantCodes(
				        codingScheme,
						inCode,
						baselineDate_in);
 		}


		/**
		 * Gets the descendant concepts' CodeActionDate of the specified concept. A CodeActionDate
		 * contains a concept code and a baseline date. The method searches for all descendant
		 * concepts of the specified concept, whether active or retired. The search is only applied to the
		 * "split/merge" actions and the specified concept could be the descendant concept of itself.
		 *
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 *
		 * @return a vector of CodeActionDate The returned CodeActionDates contain
		 * the descendant concepts of inCode and the correspond baseline dates.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector getDescendantCodeActionDates(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		     return getDescendantCodeActionDates(
				        codingScheme,
						inCode,
						iBaselineDate_in,
						fBaselineDate_in);

		}


//		used
		/**
 * Gets the ancestor concept codes of the specified concept. The search extends to a final
 * baseline date starting from an initial baseline date. The search is based on a boolean value. If
 * the value is true, the method only searches for the active concepts at the initial baseline date.
 * If the value is false, the method searches for all ancestor concepts, whether active or retired.
 * The search is only applied to "split/merge" actions and the specified concept could be the
 * ancestor concept of itself.
 *
 * @param inCode - The specified concept's code.
 * @param atBaseline - The boolean value to specify the search type.
 * @param iBaselineDate_in initial baseline date
 * @param fBaselineDate_in final baseline date
 *
 * @return a vector of String The returned concept codes which are the
 * ancestor concepts of inCode. Returns null, if there is an exception.
 *
 * @since DLBExt 1.0
 */
		public Vector<String> getAncestorCodes(
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		    return getAncestorCodes(
				        codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in);

		}

		// 073006 (v1rc6 addition)
		/**
		 * Gets the ancestors.
		 *
		 * @param inCode the in code
		 *
		 * @return the ancestors
		 */
		public Vector getAncestors(String inCode)
		{
		    return getAncestors(
				        codingScheme,
						inCode);
		}

 		// 073006 (v1rc6 addition)
		/**
		  * Gets the descendants.
		  *
		  * @param inCode the in code
		  *
		  * @return the descendants
		  */
		 public Vector getDescendants(String inCode)
		{
		    return getDescendants(
				        codingScheme,
						inCode);
		}


		/**
		 * Gets the ancestor concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * an ancestor of itself.
		 *
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 *
		 * @return a vector of String The returned concept codes are the ancestor
		 * concepts of inCode which are involved in "split or merge" actions
		 * that occurred in baselineDate. Returns null, if there is an
		 * exception.
		 *
		 * @deprecated - use getAncestorCodes(String,Date)
		 * @since DLBExt 1.0
		 */@Deprecated
		public Vector getCodeActionParents(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getCodeActionParents(
				        codingScheme,
						inCode,
						baselineDate_in);

		}

		/**
		 * Gets the ancestor concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * an ancestor of itself.
		 *
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 *
		 * @return a vector of String The returned concept codes are the ancestor
		 * concepts of inCode which are involved in "split or merge" actions
		 * that occurred in baselineDate. Returns null, if there is an
		 * exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<String> getAncestorCodes(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getAncestorCodes(
				        codingScheme,
						inCode,
						baselineDate_in);

		}


		/**
		 * Gets the ancestor concepts' CodeActionDate of the specified concept. A CodeActionDate
		 * contains a concept code and a baseline date. The method searches for all ancestor concepts
		 * of the specified concept, whether active or retired. The search is only applied to
		 * "split/merge" actions and the specified concept could be the ancestor concept of itself.
		 *
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 *
		 * @return a vector of CodeActionDate The returned CodeActionDates contain
		 * the ancestor concepts of inCode and the correspond baseline dates.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<String> getAncestorCodeActionDates(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
	         return getAncestorCodeActionDates(
		                codingScheme,
						inCode,
						iBaselineDate_in,
						fBaselineDate_in);
		}



		/**
		 * Gets all dates in which a specified action takes place on the specified concept. The returned
		 * dates might not be distinct when the specified action is "split/merge".
		 *
		 * @param code concept code
		 * @param action the action
		 *
		 * @return a vector of Date All dates in which iAction takes place on
		 * inCode. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */

//		used
		public Vector getConceptEditActionDates(
						String code,
						String action)
		{
			 int iAction = toActionIndex(action);
             return getConceptEditActionDates(codingScheme, code, iAction);
		}


		/**
		 * Gets the concept edit action dates.
		 *
		 * @param code the code
		 * @param iAction the i action
		 *
		 * @return the concept edit action dates
		 */
		public Vector<Date> getConceptEditActionDates(
						String code,
						int iAction)
		{
             return getConceptEditActionDates(codingScheme, code, iAction);
		}


		/**
		 * Gets all EditAction of the specified concept. A EditAction contains a edit date and an editing
		 * action. The method searches for all editing actions and the corresponding editing dates on the
		 * specified concept.
		 *
		 * @param inCode - The specified concept's code.
		 *
		 * @return a vector of EditAction The returned EditActions contain all
		 * editing actions and the corresponding editing dates on inCode.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
//		used
		public Vector<EditActionDate> getConceptEditAction(String inCode)
		{
			return getConceptEditAction(codingScheme, inCode);
		}

		/**
		 * Checks if the specified concept is retired or not.
		 *
		 * @param inCode - The specified concept's code.
		 *
		 * @return a boolean True - inCode is retired. False - inCode is active.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 * @deprecated - use isRetired()
		 */@Deprecated
		public boolean isConceptRetired(String inCode)
		{
			Boolean Bool_val = isRetired(inCode);
			if (Bool_val == null) return false;
			return Bool_val.booleanValue();
		}

//		used
		/**
 * Checks if is retired.
 *
 * @param inCode the in code
 *
 * @return the boolean
 */
public Boolean isRetired(String inCode)
		{
            return isCodeRetired(codingScheme, inCode);
		}


       /**
        * Checks if the specified concept is edited or not during a date range. The check is depended
        * on a boolean value. If the boolean value is true, the method only checks if the specified
        * concept was merged or split in the date range. If the boolean value is false, the method
        * checks if the specified concept was edited in any way in the date range.
        *
        * @param inCode - The specified concept's code.
        * @param mrg_splt - True - only check for "merge or split" editing action.
        * False - check for any editing action.
        * @param iBaselineDate_in - The initial baseline date.
        * @param fBaselineDate_in - The final baseline date.
        *
        * @return a boolean True - inCode is edited. False - inCode isn't edited.
        * Returns null, if there is an exception.
        *
        * @since DLBExt 1.0
        * @deprecated - use isEdited
        */@Deprecated
       public boolean isConceptEdited(String inCode,
			boolean mrg_splt,
			java.util.Date iBaselineDate_in,
			java.util.Date fBaselineDate_in)
	   {
           Boolean Bool_val = isEdited(inCode,
			 mrg_splt,
			 iBaselineDate_in,
			 fBaselineDate_in);
		   return Bool_val.booleanValue();
	   }



		/**
        * Checks if the specified concept is edited or not during a date range. The check is depended
        * on a boolean value. If the boolean value is true, the method only checks if the specified
        * concept was merged or split in the date range. If the boolean value is false, the method
        * checks if the specified concept was edited in any way in the date range.
        *
        * @param inCode - The specified concept's code.
        * @param mrg_splt - True - only check for "merge or split" editing action.
        * False - check for any editing action.
        * @param iBaselineDate_in - The initial baseline date.
        * @param fBaselineDate_in - The final baseline date.
        *
        * @return a boolean True - inCode is edited. False - inCode isn't edited.
        * Returns null, if there is an exception.
		 */
		public Boolean isEdited(
						String inCode,
						boolean mrg_splt,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
			return isConceptEdited(
		                codingScheme,
						inCode,
						mrg_splt,
						iBaselineDate_in,
						fBaselineDate_in);
		}


		/**
		 * Gets the number of editing actions on the specified concept registered in the specified
		 * baseline date.
		 *
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in - The specified baseline date.
		 *
		 * @return an int The number of editing actions on inCode registered in
		 * baselineDate. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public int getNumberEditActions(
						String inCode,
						java.util.Date baselineDate_in)
		{
            Integer int_obj = getNumberEditActions(codingScheme, inCode, baselineDate_in);
            return int_obj.intValue();
		}


		/**
		 * Gets the number of editing actions on the specified concept registered in the specified
		 * baseline date.
		 *
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 *
		 * @return an int The number of editing actions on inCode registered in
		 * baselineDate. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public int getNumberEditActions(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
            Integer int_obj = getNumberEditActions(codingScheme, inCode, iBaselineDate_in,
                fBaselineDate_in);
            return int_obj.intValue();
		}


		/**
		 * Gets all EditActionDate of the specified concept. A EditAction contains a edit date and an editing
		 * action. The method searches for all editing actions and the corresponding editing dates on the
		 * specified concept.
		 *
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 *
		 * @return a vector of EditActionDate The returned EditActionDates contain
		 * all editing actions and the corresponding editing dates on inCode.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector<EditActionDate> getConceptEditAction(
						String inCode,
						java.util.Date baselineDate_in)
		{
			Vector<EditActionDate> w = new Vector<EditActionDate>();
			Vector v = getConceptHistory(codingScheme,
										inCode,
										baselineDate_in,
										baselineDate_in);
			for (int i=0; i<v.size(); i++)
			{
				NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
				EditActionDate e = new EditActionDate();
				e.setEditDate(nce.getEditDate());
				e.setAction(nce.getEditaction().getType());
				w.add(e);
			}
			return w;
	    }


/////////////////////////////////////////////////////
//// API Class: License
/////////////////////////////////////////////////////

		/**
 * Gets all licenses from the server. If no condition exists, then pass in a empty
 * String.
 *
 * @return Vector a vector of Strings describing the licenses Returns null,
 * if there is an exception.
 *
 * @since DLBExt 1.0
 */

		public String getLicense()
		{
			return getCopyRight(codingScheme);
		}


		/**
		 * Gets all licenses for the set coding Scheme
		 *
		 * @return Vector a vector of Strings describing the licenses Returns null,
		 * if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
//		used
		public Vector<String> getAllLicenses()
		{
			Vector<String> v = new Vector<String>();
			v.add(getCopyRight(codingScheme));
			return v;
		}


		/**
		 * Retrieves an array of ContentVersions which are the list of versions for
		 * the given namespace Id in the knowledgebase.
		 *
		 * @param name the name
		 *
		 * @return the ContentVersions of the given namespace Returns null, if there
		 * is an exception.
		 *
		 * @since DLBExt 1.0
		 */
//		used
		public String getVocabularyVersion(String name)
		{
			try {
				if (lbSvc == null)
				{
				   getLogger().warn("WARNING: lbSvc is null.");
				   return null;
				}

//				HashSet hset = new HashSet();
				CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
				CodingSchemeRendering[] csrs = lcsrl.getCodingSchemeRendering();
				for (int i=0; i<csrs.length; i++)
				{
					CodingSchemeRendering csr = csrs[i];
					CodingSchemeSummary css = csr.getCodingSchemeSummary();
					String formalname = css.getFormalName();
					String localname = css.getLocalName();
					if (formalname.compareTo(name) == 0 || localname.compareTo(name) == 0)
					{
						return css.getRepresentsVersion();
					}
                }
			} catch (Exception e) {
				return null;
		    }
			return null;
		}




		/**
		 * Gets the versions.
		 *
		 * @return the versions
		 */
		public Vector getVersions()
		{
			return getAvailableCodingSchemeVersions(codingScheme);
		}



      /**
       * Retrieves a vector of root concepts in the default coding system.
       *
       * @param v the v
       *
       * @return a vector of root concepts in the default coding system with
       * attributes specified by ASD_index
       *
       * @since DLBExt 1.0
       */


      private CodedEntry[] toArray(Vector<CodedEntry> v)
      {
		  if (v == null) return null;
		  CodedEntry[] a = new CodedEntry[v.size()];
		  for (int i=0; i<v.size(); i++)
		  {
			  a[i] = (CodedEntry) v.elementAt(i);
		  }
		  return a;
	  }

	  /**
  	 * Gets the concept child roots.
  	 *
  	 * @return the concept child roots
  	 */
  	/*
  	public CodedEntry[] getConceptChildRoots()
	  {
		Vector w = getRootCodedEntries(codingScheme, version, tag, null, getHasSubtype(codingScheme));

        return toArray(w);
	  }
	*/

    public CodedEntry[] getConceptChildRoots()
    {
        try {
         	CodingScheme cs = getCodingScheme(codingScheme, version, null);
         	ResolvedConceptReferenceList rcrl = getTopLevelNodes(cs);
			 if (rcrl.getResolvedConceptReferenceCount() > 0)
			 {
				 CodedEntry[] a = new CodedEntry[rcrl.getResolvedConceptReferenceCount()];
				 int lcv = 0;
				 try {
					Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference();
					while (nodes.hasNext())
					{
						ResolvedConceptReference ref = nodes.next();
						CodedEntry entry = ref.getReferencedEntry();
						if (entry == null)
						{
							entry = getCodedEntryByCode(codingScheme, version, null, entry.getConceptCode());
						}
						a[lcv] = entry;
						lcv++;
					}
					return a;
				 } catch (Exception ex) {
                    ex.printStackTrace();
				 }
			 }

 		} catch (Exception e) {
 			e.printStackTrace();
 			return null;
		}
		return null;
	}


	  /**
  	 * Gets the concept child roots.
  	 *
  	 * @param ASD_index parameter not used
  	 *
  	 * @return the concept child roots
  	 * @deprecated - use getConceptChildRoots()
  	 */@Deprecated

  	public CodedEntry[] getConceptChildRoots(@SuppressWarnings("unused")int ASD_index)
	{
		/*
			Vector w = getRootCodedEntries(codingScheme, version, tag, null, getHasSubtype(codingScheme));

            return toArray(w);
        */
        return getConceptChildRoots();
	}

	  /**
  	 * Retrieves a vector of root concepts in the default coding system.
  	 *
  	 * @param ASD_index parameter not used
  	 * @param relationName relation name
  	 * @param associationName association name
  	 *
  	 * @return a vector of root concepts in the default coding system with
  	 * attributes specified by ASD_index
  	 *
  	 * @since DLBExt 1.0
  	 * @deprecated - use getConceptChildRoots(String, String)
  	 */@Deprecated
	  public CodedEntry[] getConceptChildRoots(@SuppressWarnings("unused")int ASD_index, String relationName, String associationName)
	  {
			Vector w = getRootCodedEntries(codingScheme, version, tag, relationName, associationName);
            return toArray(w);
	  }

	  /**
	  	 * Retrieves a vector of root concepts in the default coding system.
	  	 *
	  	 * @param relationName relation name
	  	 * @param associationName association name
	  	 *
	  	 * @return a vector of root concepts in the default coding system with
	  	 * attributes specified by ASD_index
	  	 *
	  	 * @since DLBExt 1.0
	  	 */
	  public CodedEntry[] getConceptChildRoots(String relationName, String associationName)
	  {
			Vector w = getRootCodedEntries(codingScheme, version, tag, relationName, associationName);
            return toArray(w);
	  }


        /**
         * Concept association2 role.
         *
         * @param asso the asso
         *
         * @return the role
         */
        private Role conceptAssociation2Role(gov.nih.nci.lexbig.ext.ConceptAssociation asso)
        {
			Role r = new Role(asso.getName(), asso.getValue());
			Vector v = asso.getQualifiers();
			for (int i=0; i<v.size(); i++)
			{
				Qualifier q = (Qualifier) v.elementAt(i);
				r.addQualifier(q);
			}
			return r;
		}

		/**
		 * Gets the fetched associations.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 * @param isCode parameter not used
		 * @param relation the relation
		 *
		 * @return the fetched associations
		 * @deprecated - use getFetchedAssociations(String,String,String)
		 */@Deprecated
		private Vector<Role> getFetchedAssociations(String codingSchemeName, String Concept_in, @SuppressWarnings("unused")boolean isCode, String relation)
		{
			String code = Concept_in;
			String vers = null;
			String ltag = null;
			boolean direction = true;
			String relationContainer = relation;
			Vector asso_names = null;

	        Vector<ConceptAssociation> w = getAssociations(
		                                   codingSchemeName,
										   vers,
										   ltag,
										   code,
										   direction,
										   relationContainer,
										   asso_names
										   );

/*
//			if (!isCode)
//			{
//				code = super.nameToCode(codingSchemeName, version, tag, Concept_in);
//				if (code == null)
//				{
//					getLogger().warn("Invalid code: " + code);
//					return null;
//				}
//			}
            //String is_a = IS_A;
            boolean direction = true;
            String is_a = getHasSubtype(codingSchemeName);
            if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


            Vector<Role> v = new Vector<Role>();
    		Vector<ConceptAssociation> w = getConceptAssociations(
			                                   codingSchemeName,
	                                           null,
	                                           null,
	                                           code,
	                                           true,
	                                           //DLBWrapper.ROLE_RELATION);
	                                           relation);

*/
			if (w == null)
			{
				getLogger().error("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}
/*
            Vector<Role> v = new Vector<Role>();
			//Vector roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				//if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
*/
            Vector<Role> v = new Vector<Role>();
            for (int i=0; i<w.size(); i++)
            {
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (nv.getName().compareToIgnoreCase(DLBWrapper.IS_A) != 0)
				{
					v.add(conceptAssociation2Role(nv));
				}
            }
            return v;
		}

		/**
		 * Gets the fetched associations.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 * @param relation the relation
		 *
		 * @return the fetched associations
		 */
		public Vector<Role> getFetchedAssociations(String codingSchemeName, String Concept_in,  String relation)
		{
			String code = Concept_in;
//			if (!isCode)
//			{
//				code = super.nameToCode(codingSchemeName, version, tag, Concept_in);
//				if (code == null)
//				{
//					getLogger().warn("Invalid code: " + code);
//					return null;
//				}
//			}
            //String is_a = IS_A;
            boolean direction = true;
            String is_a = getHasSubtype(codingSchemeName);
            if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

			Vector<Role> v = new Vector<Role>();

    		Vector<ConceptAssociation> w = getConceptAssociations(
			                                   codingSchemeName,
	                                           null,
	                                           null,
	                                           code,
	                                           true,
	                                           //DLBWrapper.ROLE_RELATION);
	                                           relation);


			if (w == null)
			{
				getLogger().error("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}


			Vector roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
            return v;
		}

		/**
		 * Gets the fetched roles.
		 *
		 * @param Concept_in the concept_in
		 *
		 * @return the fetched roles
		 */
		public Vector<Role> getFetchedRoles(String Concept_in)
		{
//			return getFetchedRoles(Concept_in, false);
			return getFetchedAssociations(codingScheme, Concept_in, DLBWrapper.ROLE_RELATION);
	    }

		/**
		 * Gets the fetched roles.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 *
		 * @return the fetched roles
		 */
		public Vector<Role> getFetchedRoles(String codingSchemeName,String Concept_in)
		{
//			return getFetchedRoles(Concept_in, false);
			return getFetchedAssociations(codingSchemeName, Concept_in, DLBWrapper.ROLE_RELATION);
	    }

		/**
		 * Gets the roles which have been retrieved for this concept.
		 *
		 * @param Concept_in the name of a concept, if isCode is false; it is the
		 * code of a concept, if isCode is true;
		 * @param isCode a boolean variable indicating whether Concept_in is a
		 * name or code
		 *
		 * @return a vector of the roles Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 * @deprecated - evsapi does not support name as an identifier.  Use getFetchedRoles(String)
		 */@Deprecated
		public Vector<Role> getFetchedRoles(String Concept_in, boolean isCode)
		{
			return getFetchedRoles(codingScheme, Concept_in, isCode);
		}

		/**
		 * Gets the fetched roles.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 * @param isCode the is code
		 *
		 * @return the fetched roles
		 * @deprecated - evsapi does not support name as an identifier.  Use getFetchedRoles(String,String)
		 */@Deprecated
		public Vector<Role> getFetchedRoles(String codingSchemeName, String Concept_in, boolean isCode)
		{
			return getFetchedAssociations(codingSchemeName, Concept_in, isCode, DLBWrapper.ROLE_RELATION);
		}

		/**
		 * Get the number of roles that are available of the types specified in the
		 * ConceptAttributeSetDescriptor used to obtain this Concept.
		 *
		 * @param Concept_in the name of an ontylog concept
		 *
		 * @return number of roles of the specified types Returns null, if there is
		 * an exception.
		 *
		 * @since DLBExt 1.0
		 */

		//public Integer getNumberOfSpecifiedRoles(String Concept_in)
		public int getNumberOfSpecifiedRoles(String Concept_in)
		{
		     Vector v = getFetchedRoles(Concept_in);
		     if (v == null) return -1;
		     //return new Integer(v.size());
		     return v.size();
		}


		/**
		 * Gets the inverse roles which have been retrieved for this concept, i.e. the
		 * roles of other concepts which have this concept as a value.
		 *
		 * @param Concept_in the name of an ontylog concept
		 *
		 * @return a vector of the inverse roles Returns null, if there is an
		 * exception.
		 *
		 * @since DLBExt 1.0
		 */

		public Vector getFetchedInverseRoles(String Concept_in)
		{
			return getFetchedInverseRoles(Concept_in, false);
		}


		/**
		 * Gets the fetched inverse roles.
		 *
		 * @param Concept_in the concept_in
		 * @param isCode the is code
		 *
		 * @return the fetched inverse roles
		 * @deprecated - use getFetchedInverseRoles(String)
		 */@Deprecated
		public Vector getFetchedInverseRoles(String Concept_in, boolean isCode)
		{
             return getFetchedInverseRoles(codingScheme, Concept_in, isCode);
        }


		/**
		 * Gets the fetched inverse roles.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 * @param isCode parameter not used
		 *
		 * @return the fetched inverse roles
		 * @deprecated - use getFetchedInverseRoles(String,String)
		 */@Deprecated
		public Vector<Role> getFetchedInverseRoles(String codingSchemeName, String Concept_in, @SuppressWarnings("unused")boolean isCode)
		{
			String code = Concept_in;
//			if (!isCode)
//			{
//				code = nameToCode(codingSchemeName, version, tag, Concept_in);
//				if (code == null)
//				{
//					getLogger().error("Invalid concept name: " + Concept_in);
//					return null;
//				}
//			}
            //String is_a = IS_A;
            String is_a = getHasSubtype(codingSchemeName);
            //boolean direction = false;
            //if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


			Vector<Role> v = new Vector<Role>();
	        Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
	                                       null,
	                                       null,
	                                       code,
	                                       false);

			if (w == null)
			{
				getLogger().warn("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}


			Vector<String> roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				//NameAndValue nv = (NameAndValue) w.elementAt(i);
				//String assoname = nv.getName();
				/*
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						//v.add(nv.getName() + "$" + nv.getContent());
						v.add(new Role(nv.getName(), nv.getContent()));
					}
			    }
			    */
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
            return v;
		}


		/**
		 * Gets the fetched inverse roles.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param Concept_in the concept_in
		 *
		 * @return the fetched inverse roles
		 */
		public Vector<Role> getFetchedInverseRoles(String codingSchemeName, String Concept_in)
		{
			String code = Concept_in;
//			if (!isCode)
//			{
//				code = nameToCode(codingSchemeName, version, tag, Concept_in);
//				if (code == null)
//				{
//					getLogger().error("Invalid concept name: " + Concept_in);
//					return null;
//				}
//			}
            //String is_a = IS_A;
            String is_a = getHasSubtype(codingSchemeName);
            //boolean direction = false;
            //if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


			Vector<Role> v = new Vector<Role>();
	        Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
	                                       null,
	                                       null,
	                                       code,
	                                       false);

			if (w == null)
			{
				getLogger().warn("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}


			Vector<String> roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				//NameAndValue nv = (NameAndValue) w.elementAt(i);
				//String assoname = nv.getName();
				/*
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						//v.add(nv.getName() + "$" + nv.getContent());
						v.add(new Role(nv.getName(), nv.getContent()));
					}
			    }
			    */
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
            return v;
		}


		/**
		 * Get the number of inverse roles that are available of the types specified in
		 * the ConceptAttributeSetDescriptor used to obtain this
		 * Concept.
		 *
		 * @param Concept_in the name of an ontylog concept
		 *
		 * @return number of inverse roles of the specified types Returns null, if
		 * there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		//public Integer getNumberOfSpecifiedInverseRoles(String Concept_in)
		public int getNumberOfSpecifiedInverseRoles(String Concept_in)
		{
		     Vector v = getFetchedInverseRoles(Concept_in);
		     if (v == null) return -1;
		     return v.size();
		}


		/**
		 * Checks if the concept has subconcepts.
		 *
		 * @param code the code
		 *
		 * @return If the concept has subconcepts, return true. Otherwise, return
		 * false. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 * @deprecated - use hasChildren(String)
		 */@Deprecated
		private Boolean getFetchedHasSubs(String code)
		{
//		    String code = nameToCode(codingScheme, version, tag, name);
		    if (code == null)
		    {
				return Boolean.FALSE;
			}

            //String associationName = IS_A;
			boolean direction = true;
			String is_a = getHasSubtype(codingScheme);

			if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			return bVal;
		}


		/**
		 * Checks if the concept has superconcepts.
		 *
		 * @param code the code
		 *
		 * @return If the concept has superconcepts, return true. Otherwise, return
		 * false. Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 * @deprecated - use hasParents(String)
		 */@Deprecated
		private Boolean getFetchedHasSups(String code)
		{
//		    String code = nameToCode(codingScheme, version, tag, name);
		    if (code == null) return Boolean.FALSE;
            //String associationName = IS_A;
            //String associationName = getHasSubtype(codingScheme);
 			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;


            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			if (bVal == null)
			{
				getLogger().warn("WARNING: ConvenienceMethods getFetchedHasSups returns null.");
			}

			return bVal;
		}



		/**
		 * Gets the fetched has sups.
		 *
		 * @param value the value
		 * @param isCode the is code
		 *
		 * @return the fetched has sups
		 * @deprecated - use getFetchedHasSups(String)
		 */@Deprecated
		private Boolean getFetchedHasSups(String value,@SuppressWarnings("unused")boolean isCode)
		{
			String code = value;
//			if (!isCode)
//			{
//		    	code = nameToCode(codingScheme, version, tag, value);
//		    }
		    if (code == null) return Boolean.FALSE;
            //String associationName = getHasSubtype(codingScheme);
  			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			if (bVal == null)
			{
				getLogger().warn("WARNING: ConvenienceMethods getFetchedHasSups returns null.");
			}

			return bVal;
		}


	    /**
    	 * Searchs concepts by string.  Does not search the actual name, as evsapi4 no longer supports name as an identifier.
    	 *
    	 * @param pattern match pattern
    	 * @param searchLimit search limit
    	 *
    	 * @return matched concepts in a Vector<CodedEntry>
    	 *
    	 * @since DLBExt 1.0
    	 */
		//public byte[] searchConcepts(String pattern, int max, String matchAlgorithm)
	    public Vector<CodedEntry> findConceptsWithNameMatching(String pattern, int searchLimit) throws LBParameterException
	    {
			try {
				CodedEntry[] a = searchConcepts(pattern, searchLimit, defaultAlgorithm);
				return toVector(a);
		    } catch (Exception ex) {
				throw new LBParameterException(ex.getMessage());
			}
		}


		/**
		 * Searchs concepts by string.  Does not search the actual name, as evsapi4 no longer supports name as an identifier.
		 *
		 * @param matchText search target
		 * @param maxToReturn maximum number of matched concepts to return
		 * @param synonyms true, if synonyms are returned; false, otherwise
		 *
		 * @return a vector of Concept objects with names that match the search
		 * pattern. An empty vector is returned when there are no matches.
		 * Returns null, if there is an exception.
		 *
		 * @since DLBExt 1.0
		 */
		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						boolean synonyms) throws LBParameterException
		{
			 try {
		    	 return findConceptsWithNameMatching(matchText,
						maxToReturn,
						synonyms,
						defaultAlgorithm);
		     } catch (Exception ex) {
				 throw new LBParameterException(ex.getMessage());
			 }
		}

		/**
		 * Find concepts with name matching.
		 *
		 * @param matchText the match text
		 * @param maxToReturn the max to return
		 * @param matchAlgorithm the match algorithm
		 *
		 * @return the vector
		 */
		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						String matchAlgorithm) throws LBParameterException
		{
			 try {
		        return findConceptsWithNameMatching(matchText,
						maxToReturn,
						true,
						// defaultAlgorithm); //#2823
						matchAlgorithm);
		     } catch (Exception ex) {
				 throw new LBParameterException(ex.getMessage());
			 }
		}


		/**
		 * Find concepts with name matching.
		 *
		 * @param matchText the match text
		 * @param maxToReturn the max to return
		 * @param synonyms parameter not used
		 * @param matchAlgorithm the match algorithm
		 *
		 * @return the vector
		 * @deprecated - use findConceptsWithNameMatching(String,int,String)
		 */@Deprecated
		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						@SuppressWarnings("unused")boolean synonyms,
						String matchAlgorithm)

						throws LBParameterException
		{
//			boolean preferredOnly = true;
//			if (synonyms) preferredOnly = false;


//            Vector sortByProperty_vec = new Vector();
            Vector source_vec = new Vector();
			Vector w = null;
			try {
				w = findCodedEntriesWithDesignationMatching(
												codingScheme,
												version,
												matchText,
												source_vec,
												matchAlgorithm,
												maxToReturn);
			} catch (Exception e) {
				throw new LBParameterException(e.getMessage());
			}

            return w;
		}




//////////////////////////////////////////////////////////////////////////////////////////

    /**
 * Find concept with source code matching.
 *
 * @param source the source
 * @param code the code
 * @param limit the limit
 *
 * @return the coded entry[]
 */
public CodedEntry[] findConceptWithSourceCodeMatching(String source, String code, int limit)
    {
        int matchOption = 9;
        int ASDIndex = 3;

        CodedEntry[] concepts = searchConcepts(code,
					 limit,
					 matchOption,
					 source,
				 	 ASDIndex);
        return concepts;
	}

	/**
	 * Retrieves each concept with a property value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @param propType_in the prop type_in
	 * @param pattern - the concept name pattern for searching
	 * @param searchLimit - the maximum numbe of matched concepts
	 *
	 * @return a vector of Concept objects with names that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 */
//  used
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String propType_in,
		String pattern,
		int searchLimit)
	{
        return findConceptsWithPropertyMatching(propType_in, pattern, searchLimit, defaultAlgorithm);
	}



	/**
	 * Find concepts with property matching.
	 *
	 * @param propType_in the prop type_in
	 * @param pattern the pattern
	 * @param searchLimit the search limit
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String propType_in,
		String pattern,
		int searchLimit,
		String matchAlgorithm)

	{
		Vector<String> property_vec = new Vector<String>();
		property_vec.add(propType_in);
		Vector<String> restrictToProperties_vec = new Vector<String>();
		String displayPropertyName = "";
	    return findConceptsWithPropertyMatching(
	                                            pattern,
	                                            property_vec,
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
                                                searchLimit);

	}



	/**
	 * Find concepts with property matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param propType_in the prop type_in
	 * @param pattern the pattern
	 * @param searchLimit the search limit
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String codingSchemeName, String propType_in,
		String pattern,
		int searchLimit,
		String matchAlgorithm)

	{
		Vector<String> property_vec = new Vector<String>();
		property_vec.add(propType_in);
		Vector<String> restrictToProperties_vec = new Vector<String>();
		String displayPropertyName = "";
	    return findConceptsWithPropertyMatching(codingSchemeName,
	                                            pattern,
	                                            property_vec,
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
                                                searchLimit);

	}



	/**
	 * Retrieves each concept with a property value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @param matchText search target
	 * @param property_vec the property_vec
	 * @param restrictToProperties_vec the restrict to properties_vec
	 * @param displayPropertyName the display property name
	 * @param matchAlgorithm matching algorithm used
	 * @param maxToReturn the max to return
	 *
	 * @return a vector of Concept objects with names that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 */


	public Vector<CodedEntry> findConceptsWithPropertyMatching(
	                                            String matchText,
	                                            Vector property_vec, // target property
                                                Vector restrictToProperties_vec,
                                                String displayPropertyName,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
	    return findConceptsWithPropertyMatching(codingScheme,
	                                            matchText,
	                                            property_vec, // target property
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
	                                            maxToReturn);

	}


	/**
	 * Find concepts with property matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param matchText the match text
	 * @param property_vec the property_vec
	 * @param restrictToProperties_vec the restrict to properties_vec
	 * @param displayPropertyName - parameter not used
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 *
	 * @return the vector< coded entry>
	 * @deprecated - use findConceptsWithPropertyMatching(String,String,Vector,Vector,String,int)
	 */@Deprecated
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String codingSchemeName,
	                                            String matchText,
	                                            Vector property_vec, // target property
                                                Vector restrictToProperties_vec,
                                                @SuppressWarnings("unused")	String displayPropertyName,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
		if (property_vec.size() == 0)
		{
			property_vec = getSupportedPropertyNames(codingSchemeName, null, null);
		}


		Vector<CodedEntry> w = findCodedEntriesWithPropertyMatching(
											codingSchemeName,
											null,
											matchText,
											property_vec,
											restrictToProperties_vec,
											matchAlgorithm,
											maxToReturn);
        return w;
	}

	/**
	 * Find concepts with property matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param matchText the match text
	 * @param property_vec the property_vec
	 * @param restrictToProperties_vec the restrict to properties_vec
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String codingSchemeName,
	                                            String matchText,
	                                            Vector property_vec, // target property
                                                Vector restrictToProperties_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
		if (property_vec.size() == 0)
		{
			property_vec = getSupportedPropertyNames(codingSchemeName, null, null);
		}


		Vector<CodedEntry> w = findCodedEntriesWithPropertyMatching(
											codingSchemeName,
											null,
											matchText,
											property_vec,
											restrictToProperties_vec,
											matchAlgorithm,
											maxToReturn);
        return w;
	}

	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @param matchText search target
	 * @param maxToReturn maximum number of matched concepts to return
	 * @param displayPropertyName parameter not used
	 * @param includeSynonyms false, if match preferred name only; false,
	 * otherwise
	 * @param matchAlgorithm matching algorithm used
	 *
	 * @return a vector of Concept objects with names that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use findConceptsWithDesignationMatching(String,int,boolean,String)
	 */@Deprecated
	public Vector<CodedEntry> findConceptsWithDesignationMatching(String matchText,
												int maxToReturn,
												@SuppressWarnings("unused")String displayPropertyName,
												boolean includeSynonyms,
												String matchAlgorithm) throws LBParameterException
	{
		boolean preferredOnly = true;
		if (includeSynonyms) preferredOnly = false;
		Vector sortByProperty_vec = new Vector();
		Vector restrictToProperties_vec = new Vector();

        try {
		    return findConceptsWithDesignationMatching(
	                                            matchText,
												preferredOnly,
	                                            sortByProperty_vec,
                                                restrictToProperties_vec,
                                                matchAlgorithm,
	                                            maxToReturn);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}
	}


	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @param matchText search target
	 * @param maxToReturn maximum number of matched concepts to return
	 * @param includeSynonyms false, if match preferred name only; false,
	 * otherwise
	 * @param matchAlgorithm matching algorithm used
	 *
	 * @return a vector of Concept objects with names that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 */


	public Vector<CodedEntry> findConceptsWithDesignationMatching(String matchText,
												int maxToReturn,
												boolean includeSynonyms,
												String matchAlgorithm) throws LBParameterException
	{
		boolean preferredOnly = true;
		if (includeSynonyms) preferredOnly = false;
		Vector sortByProperty_vec = new Vector();
		Vector restrictToProperties_vec = new Vector();

        try {
		    return findConceptsWithDesignationMatching(
	                                            matchText,
												preferredOnly,
	                                            sortByProperty_vec,
                                                restrictToProperties_vec,
                                                matchAlgorithm,
	                                            maxToReturn);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}
	}

	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept presentations.
	 *
	 * @param matchText search target
	 * @param displayPropertyName parameter not used
	 * @param preferredOnly -- this parameter ignored, treated as true
	 * @param sortByProperty_vec -- this parameter ignored, sorts by "matchToQuery" and "code"
	 * @param source_vec the set of sources to search within  -- this paramter ignored, all sources searched
	 * @param matchAlgorithm matching algorithm to be used
	 * @param maxToReturn maximum number of matched concepts to return
	 *
	 * @return a vector of CodedEntry objects with presentations that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use findConceptsWithDesignationMatching(String,boolean,Vector,Vector,String,int)
	 */@Deprecated
	public Vector<CodedEntry> findConceptsWithDesignationMatching(
	                                            String matchText,
	                                            @SuppressWarnings("unused")String displayPropertyName,
												boolean preferredOnly,
	                                            Vector sortByProperty_vec,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn) throws LBParameterException
	{
		if (source_vec != null && source_vec.size() > 0)
		{
			Vector<String> sources = getSupportedSources();
        	for (int i=0; i<source_vec.size(); i++)
        	{
				String source = (String) source_vec.elementAt(i);
				if (!sources.contains(source))
				{
					getLogger().error("Source " + source + " is not identifiable.");
					return null;
				}
			}
 	    }

        Vector<CodedEntry> w = null;
        try {
	    	w = findCodedEntriesWithDesignationMatching(
			codingScheme, version, matchText, preferredOnly, sortByProperty_vec,
			source_vec, matchAlgorithm, maxToReturn);
		} catch (Exception ex) {
			//ex.printStackTrace();
			throw new LBParameterException(ex.getMessage());
		}

		return w;
	}


	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept presentations.
	 *
	 * @param matchText search target
	 * @param preferredOnly -- parameter not used
	 * @param sortByProperty_vec -- parameter not used
	 * @param source_vec the set of sources to search within  -- this paramter ignored, all sources searched
	 * @param matchAlgorithm matching algorithm to be used
	 * @param maxToReturn maximum number of matched concepts to return
	 *
	 * @return a vector of CodedEntry objects with presentations that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use findConceptsWithDesignationMatching(String,Vector,String,int)
	 */@Deprecated
	public Vector<CodedEntry> findConceptsWithDesignationMatching(
	                                            String matchText,
												@SuppressWarnings("unused")boolean preferredOnly,
												@SuppressWarnings("unused")Vector sortByProperty_vec,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)

	                                            throws LBParameterException
	{
		if (source_vec != null && source_vec.size() > 0)
		{
			Vector<String> sources = getSupportedSources();
        	for (int i=0; i<source_vec.size(); i++)
        	{
				String source = (String) source_vec.elementAt(i);
				if (!sources.contains(source))
				{
					getLogger().error("Source " + source + " is not identifiable.");
					return null;
				}
			}
 	    }


    	Vector<CodedEntry> w = null;
    	try {
			w = findCodedEntriesWithDesignationMatching(
			codingScheme, version, matchText,
			source_vec, matchAlgorithm, maxToReturn);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}

		return w;
	}


	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept presentations.
	 *
	 * @param matchText search target
	 * @param source_vec the set of sources to search within  -- this paramter ignored, all sources searched
	 * @param matchAlgorithm matching algorithm to be used
	 * @param maxToReturn maximum number of matched concepts to return
	 *
	 * @return a vector of CodedEntry objects with presentations that match the search
	 * pattern. An empty vector is returned when there are no matches.
	 * Returns null, if there is an exception.
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<CodedEntry> findConceptsWithDesignationMatching(
	                                            String matchText,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)

	                                            throws LBParameterException
	{
		if (source_vec != null && source_vec.size() > 0)
		{
			Vector<String> sources = getSupportedSources();
        	for (int i=0; i<source_vec.size(); i++)
        	{
				String source = (String) source_vec.elementAt(i);
				if (!sources.contains(source))
				{
					getLogger().error("Source " + source + " is not identifiable.");
					return null;
				}
			}
 	    }


    	Vector<CodedEntry> w = null;
    	try {
			w = findCodedEntriesWithDesignationMatching(
			codingScheme, version, matchText,
			source_vec, matchAlgorithm, maxToReturn);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}

		return w;
	}

//////////////////////////////////////////////////////////////////////////////////////////

	/**
 * "@since DLBExt 1.0
 *
 * @param associationName the association name
 * @param matchText the match text
 * @param maxToReturn the max to return
 *
 * @return the vector< coded entry>
 */
	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn)
	{
		return findConceptsWithAssociationMatching(associationName,
												   matchText,
												   maxToReturn,
												   defaultAlgorithm);

	}


	/**
	 * Find concepts with association matching.
	 *
	 * @param associationName the association name
	 * @param matchText the text to search on
	 * @param maxToReturn the max to return
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm)
    {
		return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   null,
												   getRelation(codingScheme, associationName));
												   //null);
    }



	/**
	 * Find concepts with association matching.
	 *
	 * @param associationName the association name
	 * @param matchText the text to search on
	 * @param maxToReturn the max to return
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the vector of property names to return
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec)
    {
		return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec);


    }




	/**
	 * Find concepts with association matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param associationName the association name
	 * @param matchText the text to search on
	 * @param maxToReturn the max to return
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the vector of property names to return
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithAssociationMatching(String codingSchemeName,
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec)
    {
	     return findConceptsWithAssociationMatching(codingSchemeName,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec,
												   getRelation(codingSchemeName, associationName));
												   //null);

    }


	/**
	 * Find concepts with association matching.
	 *
	 * @param associationName the association name
	 * @param matchText the text to search on
	 * @param maxToReturn the max to return
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the vector of property names to return
	 * @param relationName the relation name - the name of the relation group the association belongs to
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec,
												   String relationName)
    {
	     return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec,
												   relationName);

    }



    // Use this method if the association is known to be contained in a partition called "roles", or "associations".
	/**
     * Find concepts with association matching.
     *
     * @param codingSchemeName the coding scheme name
     * @param associationName the association name
     * @param matchText the text to search on
     * @param maxToReturn the max to return
     * @param matchAlgorithm the match algorithm
     * @param restricted_property_vec the vector of property names to return
     * @param relationName the relation name - the name of the relation group the association belongs to
     *
     * @return the vector< coded entry>
     */
    public Vector<CodedEntry> findConceptsWithAssociationMatching(String codingSchemeName,
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec,
												   String relationName)
	{
		Vector<CodedEntry> w = findCodedEntriesWithAssociationMatching(
			codingSchemeName, null, matchText, associationName, matchAlgorithm, maxToReturn, RESTRICT_TO_TARGET,
			restricted_property_vec, relationName);

		return w;
	}


    /**
     * Gets the relation.
     *
     * @param codingSchemeName the coding scheme name
     * @param associationName the association name
     *
     * @return the relation - the name of the relation group the association belongs to
     */
    public String getRelation(String codingSchemeName, String associationName)
    {
		if (associationName == null) return null;//"relations";
		CodingScheme cs = getCodingScheme(codingSchemeName, null, null);
		Relations[] relations = cs.getRelations();
		for (int i=0; i<relations.length; i++)
		{
			Relations r = relations[i];
			org.LexGrid.relations.Association[] associations = r.getAssociation();
			for (int j=0; j<associations.length; j++)
			{
				org.LexGrid.relations.Association association = associations[j];
				String name = association.getAssociation();
				if (name.compareToIgnoreCase(associationName) == 0) return r.getDc();
				String forwardName = association.getForwardName();
				if (forwardName.compareToIgnoreCase(associationName) == 0) return r.getDc();
				String inverseName = association.getInverse();
				if (inverseName != null)
				{
					if (inverseName.compareToIgnoreCase(associationName) == 0) return r.getDc();
				}
		    }
		}
		return null;
	}
//////////////////////////////////////////////////////////////////////////////////////////


		/**
 * Retrieves each concept with a selected DTSPropertyType, if the concept DTSPropertyType
 * value contains the argument pattern.
 *
 * @param propertyName the property name
 * @param matchText the match text
 * @param searchLimit the maximum number of matches to be returned
 *
 * @return a vector of CodedEntries that have a property value that contains the
 * search pattern. An empty vector is returned when there are no
 * property values that contain the pattern. Returns null, if there is
 * an exception.
 *
 * @since DLBExt 1.0
 */
		public Vector<CodedEntry> findConceptsWithPropertyContaining(String propertyName,
						String matchText,
						int searchLimit)
		{
 			String matchAlgorithm = "contains";
 			return findConceptsWithPropertyMatching(
				                                propertyName,
	                                            matchText,
                                                searchLimit,
                                                matchAlgorithm);

		}




		/**
		 * Find concepts with role matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the search text
		 * @param searchLimit the max number of values to return
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithRoleMatching(String roleType_in,
						String pattern,
						int searchLimit)
		{
		    return findConceptsWithRoleMatching(roleType_in,
						pattern,
						searchLimit,
						defaultAlgorithm);
		}



		/**
		 * Find concepts with role matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the search text
		 * @param searchLimit the max number of values to return
		 * @param matchAlgorithm the match algorithm
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithRoleMatching(String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm)
		{
			return findConceptsWithAssociationMatching(
				                                   roleType_in,
												   pattern,
												   searchLimit,
												   matchAlgorithm);
		}




		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(String roleType_in,
						String pattern,
						int searchLimit)
		{
		    return findConceptsWithInverseAssociationMatching(roleType_in,
						pattern,
						searchLimit,
						defaultAlgorithm);
		}

		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 * @param matchAlgorithm the match algorithm
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm)
		{
			return findConceptsWithInverseAssociationMatching(roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						null);
		}


		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 * @param matchAlgorithm the match algorithm
		 * @param restricted_property_vec the list of properties to be returned
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec)
		{
		    return findConceptsWithInverseAssociationMatching(
				        codingScheme,
			            roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						restricted_property_vec);

		}


		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 * @param matchAlgorithm the match algorithm
		 * @param restricted_property_vec the list of properties to be returned
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String codingSchemeName,
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec)
		{
		     return findConceptsWithInverseAssociationMatching(
			            codingSchemeName,
			            roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						restricted_property_vec,
						null);
		}



		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 * @param matchAlgorithm the match algorithm
		 * @param restricted_property_vec the list of properties to be returned
		 * @param relationName the relation name
		 *
		 * @return the vector< coded entry>
		 */
		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec,
						String relationName)
		{
			return findConceptsWithInverseAssociationMatching(
							codingScheme,
							roleType_in,
							pattern,
							searchLimit,
							matchAlgorithm,
							restricted_property_vec,
							relationName);
		}



		/**
		 * Find concepts with inverse association matching.
		 *
		 * @param codingSchemeName the coding scheme name
		 * @param roleType_in the name of the role to search
		 * @param pattern the pattern
		 * @param searchLimit the search limit
		 * @param matchAlgorithm the match algorithm
		 * @param restricted_property_vec the list of properties to be returned
		 * @param relationName the relation name
		 *
		 * @return the vector< coded entry>
		 */
        public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String codingSchemeName,
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec,
						String relationName)
		{
			Vector<CodedEntry> w = findCodedEntriesWithAssociationMatching(
				 codingSchemeName, null, pattern, roleType_in, matchAlgorithm, searchLimit,
				 RESTRICT_TO_SOURCE, restricted_property_vec, relationName);

			return w;
		}


/////////////////////////////////////////////////////
//// API for Supporting Term Browser Development
/////////////////////////////////////////////////////

	/**
 * Gets all PropertyType in a given namespace.
 *
 * @return all PropertyType objects for the coding scheme
 *
 * @since DLBExt 1.0
 */
//		used
	public PropertyType[] getAllPropertyTypes()
	{
		//byte[] byteNode= null;
		Vector w = getSupportedProperties(codingScheme, version, tag);
		if (w == null) return null;

        PropertyType[] a = new PropertyType[w.size()];
		//Vector v = new Vector();
		for (int i=0; i<w.size(); i++)
		{
			SupportedProperty sp = (SupportedProperty) w.elementAt(i);
			int j = i+1;

			PropertyType propertytype
			   = new PropertyType(
				        sp.getLocalId(),
						//sp.getContent(),
						j, // property id (integer valued) is not supported by LexBIG
						"");
			//v.add(propertytype);
			a[i] = propertytype;
		}
		return a;//objectToByteArray(v);
	}


	/**
	 * Gets all RoleType.
	 *
	 * @return all RoleType objects for the coding scheme
	 *
	 * @since DLBExt 1.0
	 */
	public RoleType[] getAllRoleTypes()
	{
		//101606
		Vector roles = getSupportedRoleNames();
	    try {
			Vector<RoleType> v = new Vector<RoleType>();
			Vector<SupportedAssociation> assos = getSupportedAssociations(codingScheme);
			for (int j=0; j<assos.size(); j++)
			{
				SupportedAssociation sa = (SupportedAssociation) assos.elementAt(j);
				String assoname = sa.getLocalId();

				if (roles.contains(assoname) && assoname.compareTo(IS_A) != 0
				    && assoname.compareTo(IS_A_OBO) != 0)
				{
					RoleType roletype = new RoleType(
						        sa.getLocalId(),
								//sa.getContent(),
								j+1,//-1,
								"");
					v.add(roletype);
			    }
			}
			if (v.size() == 0) return null;
			RoleType[] a = new RoleType[v.size()];

			for (int i=0; i<v.size(); i++)
			{
				a[i] = (RoleType) v.elementAt(i);
			}
			return a;

		} catch (Exception e) {
		    e.printStackTrace();
			getLogger().error("getAllRoleTypes failed.");
			return null;
		}
	}


	/**
	 * Gets concepts matching a given sets of concept names in a byte array.
	 *
	 * @param conceptCodes_in concept codes
	 * @param ASD_index attribute set descriptor index
	 *
	 * @return concepts matching a given sets of concept names in a byte array
	 *
	 * @since DLBExt 1.0
	 */
	public CodedEntry[] getConcepts(Vector<String> conceptCodes_in, int ASD_index)
	{
        if (conceptCodes_in == null) return null;
        if (conceptCodes_in.size() == 0) return new CodedEntry[0];

//        Vector properties = null;
//        if (ASD_index == 0)
//        {
//			properties = new Vector();
//			properties.add(NONE);
//		}

		try {
			Vector<CodedEntry> v = new Vector<CodedEntry>();
			for (int i=0; i<conceptCodes_in.size(); i++)
			{
				String conceptcode = (String) conceptCodes_in.elementAt(i);
			//	CodedEntry ce = getConceptByName(conceptcode, ASD_index);
				CodedEntry ce = getConcept(conceptcode, ASD_index);
				v.add(ce);
			}

			return toArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}
	}


	/**
	 * Gets RoleType by name
	 * .
	 *
	 * @param rolename role name
	 *
	 * @return RoleType matches with a given role type name
	 *
	 * @since DLBExt 1.0
	 */
	public RoleType getRoleTypeByName(String rolename)
	{
		Vector v = getSupportedRoleNames();
		for (int i=0; i<v.size(); i++)
		{
			String assoname = (String) v.elementAt(i);
			if (assoname.compareTo(rolename) == 0)
			{
				RoleType rt = new RoleType(
						 rolename,
						 -1,
						 "");

				return rt;//objectToByteArray(rt);
			}
		}
		return  null;
	}


	/**
	 * Gets PropertyType by name.
	 *
	 * @param propertyName property name
	 *
	 * @return PropertyType matches with a given property type name
	 *
	 * @since DLBExt 1.0
	 */
	public PropertyType getPropertyTypeByName(String propertyName)
	{
		Vector<String> v = getSupportedPropertyNames(codingScheme, version, null);
		for (int i=0; i<v.size(); i++)
		{
			String propname = (String) v.elementAt(i);
			if (propname.compareTo(propertyName) == 0)
			{
				 PropertyType pt = new PropertyType(
						 propertyName,
						 -1,
						 "");
				return pt;//objectToByteArray(pt);
			}
		}
		return  null;
	}


    /**
     * Finds concept by name.
     * @param conceptname concept name
     *
     * @return a CodedEntry instance in a byte array
     *
     * @since DLBExt 1.0
     * @deprecated - evsapi4 does not support name as an identifier
     */@Deprecated
//	used
    public CodedEntry findConcept(String conceptname)
    {
		return findConceptByName(conceptname, 1);
	}


    /**
     * Find concept.
     * @param conceptname the conceptname
     * @param allAttributes the all attributes
     *
     * @return the coded entry
     *
     * @deprecated - evsapi4 does not support name as an identifier
     */@Deprecated
    public CodedEntry findConcept(String conceptname, boolean allAttributes)
    {
		int ASD_index = 0;
		if (allAttributes) ASD_index = 1;
		return findConceptByName(conceptname, ASD_index);
	}


    /**
     * Find concept by name.
     *
     * @param code the code
     * @param ASD_index the aS d_index
     *
     * @return the coded entry
     *
     * @deprecated - evsapi4 does not support name as an identifier
     */@Deprecated
    public CodedEntry findConceptByName(String code, int ASD_index)
	{
//		String code = nameToCode(codingScheme, version, tag, conceptname);

		if (code == null)
		{
			getLogger().error("Unable to find code for name " + code);
			return null;
		}
		return findConceptByCode(code, ASD_index);
	}


    /**
     * "@since DLBExt 1.0
     *
     * @param code the code
     * @param displayPropertyName the display property name
     *
     * @return the coded entry
     *
     * @deprecated - evsapi4 does not support name as an identifier
     */@Deprecated
    public CodedEntry findConceptByName(String code, String displayPropertyName)
	{
//		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null) return null;
		return findConceptByCode(code, displayPropertyName);
	}


//    /**
//     * "@since DLBExt 1.0
//     *
//     * @param code the code
//     * @param ASD_index the aS d_index
//     *
//     * @return the concept by name
//     *
//     * @deprecated - evsapi4 does not support name as an identifier
//     */@Deprecated
//    private CodedEntry getConceptByName(String code, int ASD_index)
//	{
////		String code = nameToCode(codingScheme, version, tag, conceptname);
//		if (code == null) return null;
//		CodedEntry ce = getCodedEntry(codingScheme, version, tag, code, ASD_index);
//		return ce;
//	}

//////////////////////////////////////////////////////////////////////


	 /**
 * getFetched Concept Associations of an Concept instance.
 *
 * @param code the code
 *
 * @return a vector of ConceptAssociations
 *
 * @since DLBExt 1.0
 */
	 public Vector<ConceptAssociation> getFetchedConceptAssociations(String code)
     {
          return getFetchedConceptAssociations(codingScheme, code);
     }



	 /**
 	 * Gets the fetched concept associations.
 	 *
 	 * @param codingSchemeName the coding scheme name
 	 * @param code the concept code
 	 *
 	 * @return the fetched concept associations
 	 */
 	public Vector<ConceptAssociation> getFetchedConceptAssociations(
										  String codingSchemeName,
										  String code)
	 {
		//Vector associations = getSupportedAssociationNames(codingSchemeName);
		//if (associations.size() == 0) return new Vector();

		//String is_a = IS_A;
		String is_a = getHasSubtype(codingSchemeName);
		Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();

		Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
									   null,
									   null,
									   code,
									   true);

		if (w == null)
		{
			getLogger().warn("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
			return null;
		}

        Vector<String> roles = getSupportedRoleNames(codingSchemeName);
		if (w == null) return null;
		for (int i=0; i<w.size(); i++)
		{
			gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			String assoname = nv.getName();
			if (!roles.contains(assoname))
			{
				if (nv.getName().compareToIgnoreCase(is_a) != 0)
				{
					v.add(nv);
				}
			}
		}
		return v;
	 }


    /**
     * "@since DLBExt 1.0
     * Searches through the vector passed in and returns any properties that are valid
     *
     * @param propertynames the propertynames
     *
     * @return the valid property names vector
     */
    public Vector<String> getValidPropertyNames(Vector<String> propertynames)
	{
        Vector<String> v = new Vector<String>();
		if (propertynames == null) return null;
		if (propertynames.size() == 0) return v;
		Vector<String> propnames = getSupportedPropertyNames(codingScheme, version, null);
		for (int i=0; i<propertynames.size(); i++)
		{
			String s = (String) propertynames.elementAt(i);
			boolean valid = false;
			for (int j=0; j<propnames.size(); j++)
            {
				String t = (String) propnames.elementAt(j);
				if (s.compareTo(t) == 0)
				{
					valid = true;
					break;
				}
			}
			if (valid)
			{
				v.add(s);
			}
		}
		return v;
	}



//  used
    /**
 * Gets the concept.
 *
 * @param inputvalue the inputvalue
 * @param inputcode true for code, false for name
 * @param ASD_index the AttributeSetDescriptor index
 *
 * @return the CodedEntry
 *
 * @deprecated - evsapi4 does not support name as an identifier.  Use getConcept(String,int)
 */
@Deprecated
public CodedEntry getConcept(String inputvalue,
                             boolean inputcode,
                             int ASD_index)
	{
		if (inputcode)
		{
			return findConceptByCode(inputvalue, ASD_index);
		}
		return findConceptByName(inputvalue, ASD_index);
	}



    /**
     * getConcept.
     *
     * @param inputvalue the inputvalue
     * @param inputcode true for code, false for name
     * @param displayPropertyName the display property name
     *
     * @return the CodedEntry
     *
     * @since DLBExt 1.0
     * @deprecated evsapi4 does not support name as an identifier. Use getConcept(String,String)
     */
    @Deprecated
	public CodedEntry getConcept(String inputvalue,
                             boolean inputcode,
                             String displayPropertyName)
	{
		if (inputcode)
		{
			return findConceptByCode(inputvalue, displayPropertyName);
		}
		return findConceptByName(inputvalue, displayPropertyName);
	}



     /**
      * Gets concepts matching a given sets of concept code in a string vector.
      *
      * @param conceptCodes_in the vector of concept codes
      *
      * @return concepts matching a given sets of concept names in CodedEntry array
      *
      * @since DLBExt 1.0
      */
     public CodedEntry[] getConcepts(Vector<String> conceptCodes_in)
     {
          return getConcepts(conceptCodes_in, true);
	 }


     /**
      * Gets the concepts.
      *
      * @param conceptCodes_in the vector of concept codes
      * @param allAttributes whether to retrieve all concept attributes
      *
      * @return the CodedEntry array
      */
     public CodedEntry[] getConcepts(Vector<String> conceptCodes_in, boolean allAttributes)
     {
		  if (allAttributes)
		  {
          	return getConcepts(conceptCodes_in,
                             true,
                             1); // all attributes
		  }
		  else
		  {
          	return getConcepts(conceptCodes_in,
                             true,
                             0); // no attributes

		  }
	 }



     /**
      * Gets the concepts.
      *
      * @param inputvalues the vector of concept codes
      * @param inputcode ttrue if code, false if name
      * @param ASD_index the AttributeSetDescriptor
      *
      * @return the CodedEntry array
      *
      * @deprecated - evsapi4 no longer supports name as an identifier.  Use getConcepts(Vector,int)
      */
     @Deprecated
	public CodedEntry[] getConcepts(Vector<String> inputvalues,
                              boolean inputcode,
                              int ASD_index)
	{
        if (!inputcode)
        {
			return getConcepts(inputvalues, ASD_index);
	    }

		Vector<CodedEntry> v = new Vector<CodedEntry>();
		for (int i=0; i<inputvalues.size(); i++)
		{
			String conceptcode = (String) inputvalues.elementAt(i);
			// to be modified (ASD_index);

			CodedEntry ce = getCodedEntryByCode(codingScheme, version, tag, conceptcode);
			v.add(ce);
		}
		return toArray(v);
	}



    /**
     * Gets subconcepts of a given Concept.
     *
     * @param conceptcode the conceptcode
     * @param allAttributes - parameter not used
     *
     * @return CodedEntry array
     *
     * @since DLBExt 1.0
     * @deprecated - use getDirectSubs(String)
     */@Deprecated
//   used
    public CodedEntry[] getDirectSubs(String conceptcode, boolean allAttributes)
    {
		int ASD_index = 0;
		if (allAttributes) ASD_index = 1;
		return getDirectSubs(conceptcode, true, ASD_index);

	}

    /**
     * Gets subconcepts of a given Concept.
     *
     * @param conceptcode the conceptcode
     *
     * @return CodedEntry array
     *
     * @since DLBExt 1.0
     */
    public CodedEntry[] getDirectSubs(String conceptcode)
    {
		return getDirectSubs(conceptcode, true, 1);
	}

    /**
     * Gets subconcepts of a given Concept.
     *
     * @param conceptcode the conceptcode
     * @param ASD_index - parameter not used
     *
     * @return CodedEntry array
     *
     * @since DLBExt 1.0
     * @deprecated - use getDirectSubs(String)
     */@Deprecated
    public CodedEntry[] getDirectSubs(String conceptcode, int ASD_index)
    {
		return getDirectSubs(conceptcode, true, ASD_index);
    }


    /**
     * Gets subconcepts of a given Concept.
     *
     * @param inputvalue the inputvalue
     * @param inputcode parameter not used
     * @param ASD_index the AttributeSetDescriptor
     *
     * @return the direct subs
     *
     * @deprecated - evsapi4 does not support name as an identifier. Use getDirectSubs(String, int)
     */
    @Deprecated
	public CodedEntry[] getDirectSubs(String inputvalue,
			                    @SuppressWarnings("unused") boolean inputcode,
                                int ASD_index)
	{
		try {
			String code = inputvalue;
//			if (!inputcode)
//			{
//				code = nameToCode(codingScheme, version, tag, inputvalue);
//			}
			if (code == null) return null;

			//String relation = IS_A;
			String relation = getHasSubtype(codingScheme);

			boolean direction = true;
			// deal with ASD_Index later

			//Vector v = new Vector();
			Vector<String> codes = findRelatedCodes(codingScheme,
	                               version,
	                               tag,
	                               code,
	                               relation,
	                               direction);
	        if (codes == null)
	        {
			    getLogger().warn("findRelatedCodes returns null.");
			    return null;//objectToByteArray(v);
			}

            CodedEntry[] a = new CodedEntry[codes.size()];
			for (int i=0; i<codes.size(); i++)
			{
				String concept_code = (String) codes.elementAt(i);
		        CodedEntry c = getConcept(concept_code, ASD_index);
		        //v.add(c);
		        a[i] = c;
			}
			return a;//objectToByteArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}

	}


    /**
     * Gets direct superconcepts of the given concept.
     *
     * @param conceptcode the conceptcode
     * @param allAttributes - whether to return all attributes of the concept
     *
     * @return CodedEntry array
     *
     * @since DLBExt 1.0
     */
    public CodedEntry[] getDirectSups(String conceptcode, boolean allAttributes)
	{
		if (allAttributes)
		{
        	return getDirectSups(conceptcode, true, 1);
		}
		else
		{
			return getDirectSups(conceptcode, true, 0);
		}
	}


    /**
     * Gets direct superconcepts of the given concept.
     *
     * @param conceptcode the conceptcode
     *
     * @return CodedEntry array
     */
    public CodedEntry[] getDirectSups(String conceptcode)
	{
        return getDirectSups(conceptcode, false, 1);
	}


    /**
     * Gets direct superconcepts of the given concept.
     *
     * @param conceptcode the conceptcode
     * @param ASD_index the AttributeSetDescriptor - not used
     *
     * @return CodedEntry array
     * @deprecated - use getDirectSups(String)
     */@Deprecated
    public CodedEntry[] getDirectSups(String conceptcode, int ASD_index)
	{
        return getDirectSups(conceptcode, false, ASD_index);
	}


    /**
     * Gets direct superconcepts of the given concept.
     *
     * @param inputvalue the inputvalue
     * @param inputcode parameter not used
     * @param ASD_index the AttributeSetDescriptor - not used
     *
     * @return CodedEntry array
     *
     * @deprecated - evsapi4 does not support name as an identifier. Use getDirectSups(String,int)
     */@Deprecated
    public CodedEntry[] getDirectSups(String inputvalue,
    		                    @SuppressWarnings("unused")boolean inputcode,
                                int ASD_index)
	{
		try {
			String code = inputvalue;
//			if (!inputcode)
//			{
//				code = nameToCode(codingScheme, version, tag, inputvalue);
//			}
			if (code == null) return null;

			//String relation = IS_A;
			String relation = getHasSubtype(codingScheme);

			boolean direction = false;
			// deal with ASD_Index later

            //Vector v = new Vector();
			Vector codes = findRelatedCodes(codingScheme,
	                               version,
	                               tag,
	                               code,
	                               relation,
	                               direction);

	        if (codes == null)
	        {
			    getLogger().warn("findRelatedCodes returns null.");
			    return null;//objectToByteArray(v);
			}

            CodedEntry[] a = new CodedEntry[codes.size()];
			for (int i=0; i<codes.size(); i++)
			{
				String concept_code = (String) codes.elementAt(i);
		        CodedEntry c = getConcept(concept_code, ASD_index);
		        //v.add(c);
		        a[i] = c;
			}
			return a;//objectToByteArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}
	}



//	/**
//	 * converts Hashtable into CodedEntry array.
//	 *
//	 * @param visited_node Hashtable of visited nodes.
//	 *
//	 * @return the CodedEntry[]
//	 */
//	private CodedEntry[] sortVisitedNodes(Hashtable visited_node)
//    {
//		CodedEntry[] concepts = new CodedEntry[visited_node.size()];
//		int lcv = 0;
//		for (Enumeration e = visited_node.keys(); e.hasMoreElements();)
//		{
//			CodedEntry concept = (CodedEntry) e.nextElement();
//			concepts[lcv] = concept;
//			lcv++;
//		}
//		return concepts;//sortConcepts(concepts);
//	}


	/**
	 * Checks if a concept has any subconcepts.
	 *
	 * @param codingSchemeName coding scheme name
	 * @param code concept code
	 *
	 * @return Boolean variable, set to TRUE if it is; to FALSE, otherwise
	 *
	 * @since DLBExt 1.0
	 */
	public Boolean hasChildren(String codingSchemeName, String code)
	{
		boolean direction = true;
		String is_a = getHasSubtype(codingSchemeName);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

        return hasAssociatedConcepts(codingSchemeName,
	                                     code,
	                                     direction,
										 is_a);
	}

	/**
	 * Checks if a concept has any superconcepts.
	 *
	 * @param codingSchemeName coding scheme name
	 * @param code coding scheme code
	 *
	 * @return Boolean variable, set to TRUE if it is; to FALSE, otherwise
	 */
	public Boolean hasParents(String codingSchemeName, String code)
	{
		boolean direction = false;
		String is_a = getHasSubtype(codingSchemeName);
		if (is_a.compareTo(IS_A_OBO) == 0 || is_a.compareTo(PAR) == 0) direction = !direction;

        return hasAssociatedConcepts(codingSchemeName,
	                                     code,
										 direction,
										 is_a);
	}


     /**
      * Gets the property qualifiers for the given property.
      *
      * @param p the Property
      *
      * @return the property qualifiers
      */
     Vector<PropertyQualifier> getPropertyQualifiers(org.LexGrid.commonTypes.Property p)
     {
		 Vector<PropertyQualifier> v = new Vector<PropertyQualifier>();
		 PropertyQualifier[] qualifiers = p.getPropertyQualifier();
		 for (int i=0; i<qualifiers.length; i++)
		 {
			 v.add(qualifiers[i]);
		 }
		 return v;
	 }


	 /**
 	 * Finds the Concept instance for a given concept code.
 	 *
 	 * @param conceptcode the conceptcode
 	 * @param allAttributes - parameter not used
 	 *
 	 * @return the CodedEntry instance for a given concept code
 	 *
 	 * @since DLBExt 1.0
 	 * @deprecated - use getConcept(String)
 	 */@Deprecated
	 public CodedEntry findConceptByCode(String conceptcode, boolean allAttributes)
	 {
		 int ASD_index = 0;
		 if (allAttributes) ASD_index = 1;
		 return findConceptByCode(conceptcode, ASD_index);
	 }

//used
	 /**
 * Find concept by code.
 *
 * @param code the code
 *
 * @return the CodedEntry
 	 * @deprecated - use getConcept(String)
 	 */@Deprecated
	 public CodedEntry findConceptByCode(String code)
	 {
//		 return findConceptByCode(code, 1);
		 return getConcept(code);
     }

	 /**
 	 * findConceptByCode.
 	 *
 	 * @param code concept code
 	 * @param ASD_index parameter not used
 	 *
 	 * @return the CodedEntry
 	 * @deprecated - use getConcept(String)
 	 */@Deprecated
	 public CodedEntry findConceptByCode(String code,@SuppressWarnings("unused") int ASD_index)
	 {
//		 return getConcept(code, ASD_index);
		 return getConcept(code);
	 }


	 /**
 	 * findConceptByCode.
 	 *
 	 * @param code concept code
 	 * @param displayPropertyName parameter not used
 	 *
 	 * @return the CodedEntry
 	 * @deprecated - use getConcept(String)
 	 */@Deprecated
	 public CodedEntry findConceptByCode(String code, @SuppressWarnings("unused")String displayPropertyName)
	 {
//		 return getConcept(code, displayPropertyName);
		 return getConcept(code);
		 }


	 /**
 	 * getConcept.
 	 *
 	 * @param codingSchemeName the coding scheme name
 	 * @param code concept code
 	 * @param isCode - parameter not used
 	 * @param ASD_index - parameter not used
 	 *
 	 * @return the CodedEntry
 	 *
 	 * @deprecated - evsapi4 does not support name as an identifier.  Use getConcept(String,String)
 	 */@Deprecated
	 public CodedEntry getConcept(String codingSchemeName, String code,@SuppressWarnings("unused") boolean isCode, @SuppressWarnings("unused") int ASD_index)
	 {
		 CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
		 if (ce == null)
		 {
			 //getLogger().warn("Cannot find CodedEntry for code " + code);
			 return null;
		 }
	     return ce;
	 }

	 /**
 	 * getConcept.
 	 *
 	 * @param codingSchemeName the coding scheme name
 	 * @param code concept code
 	 * @param ASD_index - parameter not used
 	 *
 	 * @return the CodedEntry
 	 * @deprecated - use getConcept(String,String)
 	 */@Deprecated
	 public CodedEntry getConcept(String codingSchemeName, String code, @SuppressWarnings("unused")int ASD_index)
	 {
//		 CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code, ASD_index);
		 CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);

		 if (ce == null)
		 {
			 //getLogger().warn("Cannot find CodedEntry for code " + code);
			 return null;
		 }
	     return ce;
	 }

	 /**
	 	 * getConcept.
	 	 *
	 	 * @param codingSchemeName the coding scheme name
	 	 * @param code concept code
	 	 *
	 	 * @return the CodedEntry
	 	 */
		 public CodedEntry getConcept(String codingSchemeName, String code)
		 {
//			 CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code, ASD_index);
			 CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);

			 if (ce == null)
			 {
				 //getLogger().warn("Cannot find CodedEntry for code " + code);
				 return null;
			 }
		     return ce;
		 }

//	 /**
// 	 * getConcept.
// 	 *
// 	 * @param code concept code
// 	 * @param displayPropertyName parameter not used
// 	 *
// 	 * @return the CodedEntry
// 	 * @deprecated - use getConcept(String)
// 	 */  	@Deprecated
//	 private CodedEntry getConcept(String code, @SuppressWarnings("unused")	String displayPropertyName)
//	 {
//
//         CodedEntry ce = getCodedEntry(codingScheme,
//	                                version,
//									tag,
//									code,
//									3);
//
//		 if (ce == null)
//		 {
//			 getLogger().warn("Cannot find CodedEntry for code " + code);
//			 return null;
//		 }
//	     return ce;
//	 }

	 /**
	 	 * getConcept.
	 	 *
	 	 * @param code concept code
	 	 *
	 	 * @return the CodedEntry
	 	 */
		 public CodedEntry getConcept(String code)
		 {

	         CodedEntry ce = getCodedEntry(codingScheme,
		                                version,
										tag,
										code);

			 if (ce == null)
			 {
				 //getLogger().warn("Cannot find CodedEntry for code " + code);
				 return null;
			 }
		     return ce;
		 }


	 /**
 	 * get the set of valid status that a concept can fall into for the codingScheme.
 	 *
 	 * @return the supported concept status as a String Vector
 	 */
	 public Vector<String> getSupportedConceptStatus()
	 {
		 return getSupportedConceptStatus(codingScheme);
	 }



/**
 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
 * The tree contains the child concepts to the level supplied.
 *
 * @param rootcode the concept code for the root node of the tree
 * @param direction true for down, false for up
 * @param levels depth of the tree
 * @param asdIndex parameter not used
 * @param isaflag true for use is-a relationship, false not to
 * @param rolenames the rolenames to build the tree from
 *
 * @return the DefaultMutableTreeNode
 */


	public DefaultMutableTreeNode getTree(
		                  String rootcode,
	                      boolean direction,
	                      int levels,
	                      @SuppressWarnings("unused") int asdIndex,
		                  boolean isaflag,
		                  Vector rolenames)
	{
			return getTree(
				  rootcode,
				  direction,
				  levels,
				  asdIndex,
				  isaflag,
				  rolenames,
				  false);
	}



/**
 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
 * The tree contains the child concepts to the level supplied.
 *
 * @param rootcode the concept code for the root node of the tree
 * @param direction true for down, false for up
 * @param levels depth of the tree
 * @param asdIndex parameter not used
 * @param isaflag true for use is-a relationship, false not to
 * @param rolenames the rolenames to build the tree from
 * @param direction_role traverse direction for roles; true for forward, false for backward
 *
 * @return the DefaultMutableTreeNode
 */

	public DefaultMutableTreeNode getTree(
		                  String rootcode,
	                      boolean direction,
	                      int levels,
	                      int asdIndex,
		                  boolean isaflag,
		                  Vector rolenames,
		                  boolean direction_role)
	{
		if (rootcode == null) return null;
        CodedEntry c = getConcept(rootcode);
		if (c == null)
		{
			System.out.println("Root concept " + rootcode + " not found -- tree generation aborts.");
			return null;
		}

        CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
        if (version != null) csvt.setVersion(version);
        String sab = null;
        boolean navigationDir = NavigationUtils.getNavigationDirection(lbSvc, codingScheme, csvt, sab);
        if (!navigationDir) direction = !direction;

		String is_a = null;
		if (isaflag)
		{
			is_a = getHasSubtype(codingScheme);
		}

		TreeNode treenode = new TreeNode();
		treenode.setTraverseDown(direction);
		treenode.setTraverseForward(direction_role);
		treenode.setIsA(isaflag);
		treenode.setConcept(c);
		treenode.setName(c.getEntityDescription().getContent());

		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(treenode);
		dmtn = populateTreeNodes(dmtn, isaflag, is_a, rolenames, direction, direction_role, 0, levels);
		return dmtn;
	}



/**
 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
 * The tree contains the child concepts to the level supplied.
 *
 * @param rootcode the concept code for the root node of the tree
 * @param direction true for down, false for up
 * @param levels depth of the tree
 * @param asdIndex parameter not used
 * @param is_a the name of the relationship which defines the taxomomy, or hierarchy
 * @param rolenames the rolenames to build the tree from
 *
 * @return the DefaultMutableTreeNode
 */

	public DefaultMutableTreeNode getTree(
		                  String rootcode,
	                      boolean direction,
	                      int levels,
	                      @SuppressWarnings("unused")int asdIndex,
		                  String is_a,
		                  Vector rolenames)
    {

		 return getTree(  rootcode,
	                      direction,
	                      levels,
	                      asdIndex,
		                  is_a,
		                  rolenames,
		                  false); // opposite direction
    }



/**
 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
 * The tree contains the child concepts to the level supplied.
 *
 * @param rootcode the concept code for the root node of the tree
 * @param direction traverse direction for hierarchical relationship; true for down, false for up
 * @param levels parameter not used
 * @param asdIndex parameter not used
 * @param is_a the name of the relationship which defines the taxomomy, or hierarchy
 * @param rolenames the rolenames to build the tree from
 * @param direction_role traverse direction for roles; true for forward, false for backward
 *
 * @return the DefaultMutableTreeNode
 */
	public DefaultMutableTreeNode getTree(
		                  String rootcode,
	                      boolean direction,
	                      int levels,
	                      @SuppressWarnings("unused")int asdIndex,
		                  String is_a,
		                  Vector rolenames,
		                  boolean direction_role)
	{
		if (levels < -1)
		{
		    System.out.println("Error input -- level must be greater than or equals to -1.");
		    return null;
		}

		if (rootcode == null) return null;
        CodedEntry c = getConcept(rootcode);
		if (c == null)
		{
			return null;
		}

		if (is_a == null)
		{
	        return getTree(
		                  rootcode,
	                      direction,
	                      levels,
	                      asdIndex,
		                  true,
		                  rolenames,
		                  direction_role);
		}

		boolean isaflag = true;

        CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
        if (version != null) csvt.setVersion(version);
        String sab = null;
        boolean navigationDir = NavigationUtils.getNavigationDirection(lbSvc, codingScheme, csvt, sab);
        if (!navigationDir) direction = !direction;

		TreeNode treenode = new TreeNode();
		treenode.setTraverseDown(direction);
		treenode.setTraverseForward(direction_role);
		treenode.setIsA(isaflag);
		treenode.setConcept(c);
		treenode.setName(c.getEntityDescription().getContent());

		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(treenode);
		dmtn = populateTreeNodes(dmtn, isaflag, is_a, rolenames, direction, direction_role, 0, levels);

		return dmtn;
	}


	/**
	 * Populate tree nodes surrounding a given tree node.
	 *
	 * @param dmtn the DefaultMutableTreeNode
	 * @param isaflag true to use is-a relation, false not to
	 * @association_name the name of the associaation
	 * @param rolenames the list of rolenames to build the tree from
	 * @param direction true for down, false for up
	 * @param direction of role relationship, true for forward, false for backward
	 * @param curr_level current level
	 * @param max_level depth of the tree
	 *
	 * @return the DefaultMutableTreeNode
	 *
	 * @since DLBExt 1.0
	 */
	private DefaultMutableTreeNode populateTreeNodes(
		                                            DefaultMutableTreeNode dmtn,
	    	                                        boolean isaflag,
	    	                                        String association_name,
	    	                                        Vector rolenames,
	    	                                        boolean direction,
	    	                                        boolean direction_role,
	    	                                        int curr_level,
	    	                                        int max_level)
	{
		if (max_level != -1)
		{
			if (curr_level > max_level) return null;
		}
        TreeNode node = (TreeNode) dmtn.getUserObject();
        if (node == null) return null;

		CodedEntry c = node.getConcept();
		String code = c.getConceptCode();

	    HashMap hashMap = getTreeNodes(
								code,
								isaflag,
								association_name,
								rolenames,
								direction,
								direction_role);

        Set ref_codes = hashMap.keySet();

        Vector w = new Vector();
        Iterator it = ref_codes.iterator();
        while (it.hasNext())
        {
			String subconcept_code = (String) it.next();
			TreeNode treenode = (TreeNode) hashMap.get(subconcept_code);
			CodedEntry child_concept = new CodedEntry();
			child_concept.setConceptCode(subconcept_code);
            treenode.setConcept(child_concept);
            CodedEntry ce = getConcept(subconcept_code, 0);
            String concept_name = ce.getEntityDescription().getContent();
            treenode.setName(concept_name);
            treenode.setConcept(ce);
            w.add(treenode);
		}

        if (w.size() > 0)
        {
			w = SortUtils.quickSort(w);

			for (int i=0; i<w.size(); i++)
			{
				TreeNode treenode = (TreeNode) w.elementAt(i);
				DefaultMutableTreeNode child_dmtd = new DefaultMutableTreeNode(treenode);
				child_dmtd = populateTreeNodes(child_dmtd, isaflag, association_name, rolenames, direction, direction_role, curr_level+1, max_level);
				if (child_dmtd != null) dmtn.add(child_dmtd);
			}
	    }

		return dmtn;
	}






	/**
	 * getTreeNodes.
	 *
	 * @param root_code concept code
	 * @param isaflag true to use is-a hierarchy, false not to.
	 * @param association_name the name of the association.
	 * @param rolenames the list of rolenames to build the tree from
	 * @param direction true for down, false for up
	 * @param direction of role relationship, true for forward, false for backward
	 *
	 * @return the tree nodes
	 *
	 * @since DLBExt 1.0
	 */
	private HashMap<String,TreeNode> getTreeNodes(
								 String root_code,
								 boolean isaflag,
								 String association_name,
								 Vector rolenames,
								 boolean direction,
								 boolean direction_role)
	{
	//  code to vector of role names mapping
		HashMap<String,TreeNode> code2RolenamesMap = new HashMap<String,TreeNode>();
		if ((rolenames == null || rolenames.size() == 0) && !isaflag)
		{
			return code2RolenamesMap;
		}

        Vector<String> links = new Vector<String>();
        if (rolenames != null)
        {
			for (int i=0; i<rolenames.size(); i++)
			{
				String rolename = (String) rolenames.elementAt(i);
				links.add(rolename);
			}
	    }

        Vector<NameAndValue> w = new Vector<NameAndValue>();
        if (isaflag)
        {
	        Vector concepts = getAssociations(lbSvc,
		                        codingScheme,
								version,
								null,
								root_code,
								direction,
								null,
								association_name); // hierarchical relationship

			if (concepts != null)
			{
				for (int i=0; i<concepts.size(); i++)
				{
					CodedEntry c = (CodedEntry) concepts.elementAt(i);
					if (c!= null)
					{
						NameAndValue nv = createNameAndValue(association_name, c.getConceptCode());
						w.add(nv);
				    }
				}
		    }
		}

		if (rolenames != null)
		{
			for (int i=0; i<rolenames.size(); i++)
			{
				String associationName = (String) rolenames.elementAt(i);
				//Vector concepts = getRelatedConcepts(root_code, direction, associationName, 0);
	            Vector concepts = getAssociations(lbSvc,
		                        codingScheme,
								version,
								null,
								root_code,
								direction_role,
								null,
								associationName);

				for (int j=0; j<concepts.size(); j++)
				{
					CodedEntry c = (CodedEntry) concepts.elementAt(j);
					if (c != null)
					{
						NameAndValue nv = createNameAndValue(associationName, c.getConceptCode());
						w.add(nv);
					}
				}
			}
		}

		if (w.size() == 0) // an orphan, no needs to continue
		{
			return code2RolenamesMap;
		}

		for (int i=0; i<w.size(); i++)
		{
			NameAndValue nv = (NameAndValue) w.elementAt(i);
			String rolename = nv.getName();

            if (links != null && rolename != null)
            {
				if (links.contains(rolename) || (isaflag && rolename.equals(IS_A)) )
				{
					String ref_code = nv.getContent();
					if (code2RolenamesMap.containsKey(ref_code))
					{
						TreeNode node = (TreeNode) code2RolenamesMap.get(ref_code);
						node.setIsA(isaflag);
						if (rolename.equals(IS_A))
						{
							node.setTraverseDown(direction);
						}
						else
						{
							node.setTraverseForward(direction_role);
						}

						if (!node.getLinks().contains(ref_code))
						{
							node.addLink(rolename);
						}
					}
					else
					{
						TreeNode node = new TreeNode();
						node.setIsA(isaflag);

						//node.setTraverseDown(direction);
						if (rolename.equals(IS_A))
						{
							node.setTraverseDown(direction);
						}
						else
						{
							node.setTraverseForward(direction_role);
						}

						node.addLink(rolename);
						code2RolenamesMap.put(ref_code, node);
					}
				}
		    }
		}
		return code2RolenamesMap;
	}




	/**
	 * getAssociations.
	 *
	 * @param lbSvc instance of LexBIGService
	 * @param codingSchemeName coding scheme name
	 * @param vers version
	 * @param ltag tag
	 * @param code concept code
	 * @param direction direction
	 * @param relationContainer the name of the relations container
	 * @param asso_name association name
	 *
	 * @return a Vector of CodedEntry
	 *
	 * @since DLBExt 1.0
	 */
	protected Vector<CodedEntry> getAssociations(
		                                  LexBIGService lbSvc,
		                                  String codingSchemeName,
										  String vers,
										  String ltag,
										  String code,
										  boolean direction,
										  String relationContainer,
										  String asso_name
										  )
	{
		if (asso_name == null) return null;
		Vector asso_names = new Vector();
		asso_names.add(asso_name);
	    Vector<ConceptAssociation> assocs = getAssociations(
		                                   lbSvc,
		                                   codingSchemeName,
										   vers,
										   ltag,
										   code,
										   direction,
										   relationContainer,
										   asso_names
		            					   );
		if (assocs == null) return null;
		Vector<CodedEntry> w = new Vector();
		for (int i=0; i<assocs.size(); i++)
		{
			ConceptAssociation asso = (ConceptAssociation) assocs.elementAt(i);
			CodedEntry ce = getConcept(asso.getValue(), 0);
			w.add(ce);
		}

		return w;
	}


	/**
	 * getAssociations.
	 *
	 * @param lbSvc instance of LexBIGService
	 * @param codingSchemeName coding scheme name
	 * @param vers version
	 * @param ltag tag
	 * @param code concept code
	 * @param direction direction
	 * @param relationContainer the name of the relations container
	 * @param asso_name association name
	 *
	 * @return a Vector of ConceptAssociation
	 *
	 * @since DLBExt 1.0
	 */
	protected Vector<ConceptAssociation> getAssociations(
		                                  LexBIGService lbSvc,
		                                  String codingSchemeName,
										  String vers,
										  String ltag,
										  String code,
										  boolean direction,
										  String relationContainer,
										  Vector asso_names
										  )
	{
		//LexBIGService lbSvc = getLexBIGService();
        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
        CodingScheme cs = null;
        try {
        	cs = lbSvc.resolveCodingScheme(codingSchemeName, versionOrTag);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
        if (cs == null) return null;

        if (relationContainer != null)
        {
			Relations[] relations = cs.getRelations();
			boolean containerDefined = false;
			for (int i=0; i<relations.length; i++)
			{
				Relations relation = (Relations) relations[i];
				String dc = relation.getDc();
				if (dc.compareTo(relationContainer) == 0)
				{
					containerDefined = true;
					if (asso_names == null)
					{
						asso_names = new Vector();
						org.LexGrid.relations.Association[] associations = relation.getAssociation();
						for (int j=0; j<associations.length; j++)
						{
							org.LexGrid.relations.Association association = (org.LexGrid.relations.Association) associations[j];
							asso_names.add(association.getAssociation());
						}
						break;
				    }
				}
			}
			if (!containerDefined) return new Vector();
	    }

		Vector<ConceptAssociation> v = new Vector();
		ResolvedConceptReferenceList matches = null;
		try {
		   matches = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, null)
				.resolveAsList(
					ConvenienceMethods.createConceptReference(code, codingSchemeName),
					direction, !direction, 1, 1, new LocalNameList(), null, null, 1024);
		} catch (Exception ex) {
			return null;
		}

		if (matches.getResolvedConceptReferenceCount() > 0) {
			Enumeration<ResolvedConceptReference> refEnum = matches.enumerateResolvedConceptReference();

			while (refEnum.hasMoreElements()) {
				ResolvedConceptReference ref = refEnum.nextElement();
				if (ref != null)
				{
					AssociationList sourceof = ref.getSourceOf();
					if (!direction)
					{
						sourceof = ref.getTargetOf();
					}
					org.LexGrid.LexBIG.DataModel.Core.Association[] associations = sourceof.getAssociation();
					if (associations != null)
					{
						for (int i = 0; i < associations.length; i++) {
							org.LexGrid.LexBIG.DataModel.Core.Association assoc = associations[i];

							assoc = processForAnonomousNodes(assoc);

							if (asso_names == null)
							{
								AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
								for (int j = 0; j < acl.length; j++) {
									AssociatedConcept ac = acl[j];
									if (ac.getConceptCode() != null)
									{
										v.add(new ConceptAssociation(assoc.getAssociationName(), ac.getConceptCode()));
									}
								}
							}
							else if (asso_names.contains(assoc.getAssociationName()))
							{
								AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
								for (int j = 0; j < acl.length; j++) {
									AssociatedConcept ac = acl[j];
									if (ac.getConceptCode() != null)
									{
										v.add(new ConceptAssociation(assoc.getAssociationName(), ac.getConceptCode()));
									}
								}
							}
						}
					}
			    }
			}
		}
		return v;
	}



    /**
     * Code exists.
     *
     * @param code concept code
     *
     * @return true, if code exists
     *
     * @since DLBExt 1.0
     */
    public Boolean codeExists(String code)
    {
	    return codeExists(codingScheme, code);
    }


    /**
     * Checks if two codes are related through a particular association.
     *
     * @param code1 concept code1
     * @param code2 concept code2
     * @param associationName association name
     *
     * @return Boolean variable, sets to TRUE if related; to FALSE, otherwise
     *
     * @since DLBExt 1.0
     */
    public Boolean areCodesRelated(
								 String code1, // source
								 String code2, // target
								 String associationName)
	{
		/*
    	return areCodesRelated(codingScheme,
								 version,
								 null,
								 code1, // source
								 code2, // target
								 associationName);
		*/
		Vector v = getRelatedConcepts(code1, associationName);
		if (v == null) return Boolean.FALSE;
		for (int i=0; i<v.size(); i++)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(i);
			if (ce.getConceptCode().compareTo(code2) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;
    }


///////////////////////
//  History
///////////////////////

     /**
 * Dumps dumpNCIChangeEvent (for debug use only).
 *
 * @param pw the pw
 * @param e the e
 *
 * @since DLBExt 1.0
 */
     @SuppressWarnings("unused")
	private void dumpNCIChangeEvent(PrintWriter pw, NCIChangeEvent e)
     {
		 String code = e.getConceptcode();
		 String name = e.getConceptName();
		 int actionId = e.getEditaction().getType();
		 String action = "";
		 if (actionId == ChangeType.CREATE_TYPE)
		 {
			 action = "Create";
		 }
		 else if (actionId == ChangeType.MERGE_TYPE)
		 {
			 action = "Merge";
		 }
		 else if (actionId == ChangeType.MODIFY_TYPE)
		 {
			 action = "Modify";
		 }
		 else if (actionId == ChangeType.RETIRE_TYPE)
		 {
			 action = "Retire";
		 }
		 else if (actionId == ChangeType.SPLIT_TYPE)
		 {
			 action = "Split";
		 }

		 java.util.Date d = e.getEditDate();
		 String referenceCode = e.getReferencecode();
		 String referenceName = e.getReferencename();
		 pw.println("\n\tCode: " + code);
		 pw.println("\tName: " + name);
		 pw.println("\tAction: " + action);
		 pw.println("\tEdit Date: " + d.toString());
		 pw.println("\tReference Code: " + referenceCode);
		 pw.println("\tReference Name: " + referenceName);
 	 }


    /**
     * Converts a NCICHangeEvent instance to a HistoryRecord instance.
     *
     * @param nsId namespace id
     * @param e a NCICHangeEvent instance
     *
     * @return a HistoryRecord instance
     *
     * @since DLBExt 1.0
     */
    private HistoryRecord NCIChangeEvent2HistoryRecord(
		int nsId, NCIChangeEvent e)
    {
		if (e == null) return null;
		HistoryRecord rec = new HistoryRecord();
		rec.setCode(e.getConceptcode());
		rec.setNamespaceId(nsId);
//		int editaction = e.getEditaction().getType();
		String action = "";
		int actionId = e.getEditaction().getType();
		if (actionId == ChangeType.CREATE_TYPE)
		{
		    action = "Create";
		}
		else if (actionId == ChangeType.MERGE_TYPE)
		{
		    action = "Merge";
		}
		else if (actionId == ChangeType.MODIFY_TYPE)
		{
		    action = "Modify";
		}
		else if (actionId == ChangeType.RETIRE_TYPE)
		{
		    action = "Retire";
		}
		else if (actionId == ChangeType.SPLIT_TYPE)
		{
		    action = "Split";
		}
		rec.setEditAction(action);
		rec.setEditDate(e.getEditDate());
		rec.setReference(e.getReferencecode());
		return rec;
	}


//  used
    //public byte[] getConceptHistory(
	/**
 * Gets HistoryRecords corresponding to a given concept code spanning the given dates.
 *
 * @param code concept code
 * @param beginDate begin date
 * @param endDate end date
 *
 * @return array of HistoryRecord
 *
 * @since DLBExt 1.0
 */
	public HistoryRecord[] getConceptHistory(
                                    String code,
                                    java.util.Date beginDate,
                                    java.util.Date endDate)
    {
        Vector<NCIChangeEvent> w = getConceptHistory(codingScheme,
                                    code,
                                    beginDate,
                                    endDate);

        Vector<HistoryRecord> v = new Vector<HistoryRecord>();

		for (int i=0; i<w.size(); i++)
		{
			NCIChangeEvent nce = (NCIChangeEvent) w.elementAt(i);
			if (nce != null)
			{
				HistoryRecord rec = NCIChangeEvent2HistoryRecord(namespaceId, nce);
				v.add(rec);
		    }
		}
		HistoryRecord[] a = new HistoryRecord[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			a[i] = (HistoryRecord) v.elementAt(i);
		}
		return a;
    }


    /**
     * Gets concept change version Dates.
     *
     * @param code concept code
     *
     * @return a vector containing all concept change version Dates
     *
     * @since DLBExt 1.0
     */
//    private Vector<Date> getConceptChangeVersionDates(String code)
//	{
//		Vector<Date> v = new Vector<Date>();
//        Vector csvs = getConceptChangeVersions(codingScheme, code);
//		for (int i=0;i<csvs.size(); i++)
//		{
//			CodingSchemeVersion csv = (CodingSchemeVersion) csvs.elementAt(i);
//			java.util.Date d = csv.getVersionDate();
//            v.add(d);
//		}
//		return v;
//    }


    /**
     * Gets concept create version Date.
     *
     * @param code concept code
     *
     * @return the date concept was created
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getConceptCreateVersionDate(String code)
    {
		CodingSchemeVersion csv = getConceptCreateVersion(codingScheme, code);
		if (csv == null) return null;
		return csv.getVersionDate();

    }

    /**
     * return the first history date available for the coding scheme.
     *
     * @return the history start
     *
     * @deprecated - use getHistoryStartDate
     */@Deprecated
    public java.util.Date getHistoryStart()
    {
    	return getHistoryStartBaselineDate(codingScheme);
    }


    /**
     * return the first history date available for the coding scheme.
     *
     * @return the history start date
     */
    public java.util.Date getHistoryStartDate()
    {
		return getHistoryStartBaselineDate(codingScheme);
    }


    /**
     * return the first history date available for the coding scheme.
     *
     * @return the history start baseline date
     *
     * @deprecated - use getHistoryStartDate
     */@Deprecated
    public java.util.Date getHistoryStartBaselineDate()
    {
		return getHistoryStartBaselineDate(codingScheme);
    }


    /**
     * return the last history date available for the coding scheme.
     *
     * @return the history end
     *
     * @deprecated - use getHistoryEndDate
     */@Deprecated
    public java.util.Date getHistoryEnd()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }


    /**
     * return the last history date available for the coding scheme.
     *
     * @return the history end date
     */
    public java.util.Date getHistoryEndDate()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }


    /**
     * return the last history date available for the coding scheme.
     *
     * @return the history end baseline date
     *
     * @deprecated - use getHistoryEndDate
     */@Deprecated
    public java.util.Date getHistoryEndBaselineDate()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }




	/**
	 * Gets the available published vocabulary versions.
	 *
	 * @param vocabularyname the vocabularyname
	 *
	 * @return the vocabulary versions
	 *
	 * @since DLBExt 1.0
	 */
	public Vector getVocabularyVersions(String vocabularyname)
	{
        return getAvailableCodingSchemeVersions(vocabularyname);
	}


	/**
	 * Gets the available coding scheme tags.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 *
	 * @return the available coding scheme tags
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<String> getAvailableCodingSchemeTags(String codingSchemeName_in, String vers)
	{
		 Vector<String> v = new Vector<String>();
		//[#14274] getVocabularyTags returns a vector that is NOT greater than zero (KLO, 05/19/2008)
		 String codingSchemeName = localName2FormalName(codingSchemeName_in, vers, null);
		 if (codingSchemeName == null) return v;
		 try {
			 CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			 CodingSchemeRendering[] csra = lcsrl.getCodingSchemeRendering();
			 for (int i=0; i<csra.length; i++)
			 {
				CodingSchemeRendering csr = csra[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();

				if ((css.getFormalName() != null && css.getFormalName().compareTo(codingSchemeName) == 0) ||
				    (css.getLocalName() != null && css.getLocalName().compareTo(codingSchemeName) == 0))
				{
					if (vers == null)
					{
						RenderingDetail rd = csr.getRenderingDetail();
						CodingSchemeTagList cstl = rd.getVersionTags();
						java.lang.String[] tags = cstl.getTag();
						for (int j=0; j<tags.length; j++)
						{
							String ltag = (String) tags[j];
							v.add(ltag);
						}
					}
					else if (vers.compareTo(css.getRepresentsVersion()) == 0)
					{
						RenderingDetail rd = csr.getRenderingDetail();
						CodingSchemeTagList cstl = rd.getVersionTags();
						java.lang.String[] tags = cstl.getTag();
						for (int j=0; j<tags.length; j++)
						{
							String ltag = (String) tags[j];
							v.add(ltag);
						}
					}
				}
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return v;
	 }


	/**
	 * Gets available coding scheme tags for a given coding scheme name and version.
	 *
	 * @param codingSchemeName coding scheme name
	 * @param vers coding scheme version
	 *
	 * @return the vocabulary tags
	 *
	 * @since DLBExt 1.0
	 */
	public Vector getVocabularyTags(String codingSchemeName, String vers)
	{
        return getAvailableCodingSchemeTags(codingSchemeName, vers);
	}

	/**
	 * Gets available coding scheme names.
	 *
	 * @return a Vector containing all available coding scheme names.
	 */
//	private Vector getAvailableCodingSchemes()
//	{
//		 return getSupportedCodingSchemes();
//	}

//	used
	/**
 * Gets available vocabulary (i.e., coding scheme) names.
 *
 * @return a Vector containing all available vocabulary (i.e., coding scheme)
 * names.
 *
 * @since DLBExt 1.0
 */
	//080307
	public Vector<String> getVocabularyNames()
	{
        return getSupportedCodingSchemeNames();//getAvailableCodingSchemes();
	}

	/**
	 * Checks if a coding scheme exists.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return true, if the coding scheme exists; false, otherwise
	 *
	 * @since DLBExt 1.0
	 */
	public boolean codingSchemeExists(String codingSchemeName)
	{
		//Vector v = getSupportedCodingSchemes();
		Vector v = getSupportedCodingSchemeNames();
		return (v.contains(codingSchemeName));
	}


     /**
      * dump history records (for debugging use).
      *
      * @param pw an instance of PrintWriter
      * @param e an instance of HistoryRecord
      *
      * @since DLBExt 1.0
      */
     public void dumpHistoryRecord(PrintWriter pw, HistoryRecord e)
     {
		 String code = e.getCode();
		 int namespace_Id = e.getNamespaceId();
		 String editaction = e.getEditAction();
		 java.util.Date editdate = e.getEditDate();
		 String reference = e.getReference();
         pw.println("\n");
		 pw.println("\tCode: " + code);
		 pw.println("\tNamespaceId: " + namespace_Id);
		 pw.println("\tAction: " + editaction);
		 pw.println("\tEdit Date: " + editdate.toString());
		 pw.println("\tReference Code: " + reference);
 	 }


	/**
	 * Print a tree (UserObject is a Concept instance). for debugging purposes
	 *
	 * @param treeWriter an instance of PrintWriter
	 * @param root The root (DefaultMutableTreeNode) of the tree
	 *
	 * @since DLBExt 1.0
	 */
	public void printTree(PrintWriter treeWriter, DefaultMutableTreeNode root)
	{
		if (treeWriter == null)
		{
		    //getLogger().warn("PrintWriter is null.");
			return;
		}

		if (root == null)
		{

		    //getLogger().warn("printTree -- root is null.");
			return;
		}

        CodedEntry con = null;
        TreeNode treenode = null;
		try {
			treenode = (TreeNode) root.getUserObject();
			con = (CodedEntry) treenode.getConcept();
	    } catch (Exception e) {
			e.printStackTrace();
			//getLogger().warn("ERROR: printTree con == null " );
			return;
		}

        //treeWriter.println(getConceptName(con) + " (" + con.getConceptCode() + ")");
        treeWriter.println(treenode.getName() + " (" + con.getConceptCode() + ")");
		treeWriter.flush();
        printTree(treeWriter, root, 1);
	}

	/**
	 * Print a tree (UserObject is a Concept instance).
	 *
	 * @param treeWriter an instance of PrintWriter
	 * @param root the root (DefaultMutableTreeNode) of the tree
	 * @param level tree level
	 *
	 * @since DLBExt 1.0
	 */
	private void printTree(PrintWriter treeWriter,
	                       DefaultMutableTreeNode root,
						   int level)
	{
		int length = root.getChildCount();
		if (length ==0) return;

		for (int i=0; i<length; i++) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)root.getChildAt(i);
			printNode(treeWriter, node, level);
			treeWriter.flush();
			printTree(treeWriter, node, level+1);
		}
	}

	/**
	 * Print a tree node (UserObject is a Concept instance).
	 *
	 * @param treeWriter an instance of PrintWriter
	 * @param node tree node
	 * @param level tree level
	 *
	 * @since DLBExt 1.0
	 */
	private void printNode(PrintWriter treeWriter,
	                       DefaultMutableTreeNode node,
						   int level)
	{
		TreeNode treenode = (TreeNode) node.getUserObject();
		CodedEntry concept = (CodedEntry) treenode.getConcept();
		String conceptName = treenode.getName();
		boolean traverseDown = treenode.getTraverseDown();
		boolean traverseForward = treenode.getTraverseForward();
		boolean isA = treenode.getIsA();
		HashSet links = treenode.getLinks();

		if (isA && (links.contains(IS_A)))
		{
   		    for (int j=0; j<level; j++)
			{
				treeWriter.print("\t");
				treeWriter.flush();
			}

			if (traverseDown)
				treeWriter.println("[inverse is-a] "+conceptName + " (" + concept.getConceptCode() + ")");
			else
				treeWriter.println("[is-a] "+conceptName + " (" + concept.getConceptCode() + ")");

            treeWriter.flush();
		}
		else if (isA && (links.contains(IS_A_OBO)))
		{
   		    for (int j=0; j<level; j++)
			{
				treeWriter.print("\t");
				treeWriter.flush();
			}

			if (traverseDown)
				treeWriter.println("[is-a] "+conceptName + " (" + concept.getConceptCode() + ")");
			else
				treeWriter.println("[inverse is-a] "+conceptName + " (" + concept.getConceptCode() + ")");

            treeWriter.flush();
		}

		Iterator it = links.iterator();
		while (it.hasNext())
		{
			String linkname = (String) it.next();
			if (linkname.compareTo(IS_A) != 0 && linkname.compareTo(IS_A_OBO) != 0)
            {
				for (int j=0; j<level; j++)
				{
					treeWriter.print("\t");
					treeWriter.flush();
				}

				if (traverseForward)
				    treeWriter.println("["+linkname+"] "+conceptName + " (" + concept.getConceptCode() + ")");
				else
					treeWriter.println("[inverse "+linkname+"] "+conceptName + " (" + concept.getConceptCode() + ")");

				treeWriter.flush();
		    }
		}
	}


	/**
	 * Ouptut a tree to a PrintWriter.
	 *
	 * @param pw an instance of PrintWriter
	 * @param dmtn an instance of DefaultMutableTreeNode
	 *
	 * @since DLBExt 1.0
	 */
	public void outputTree(PrintWriter pw, DefaultMutableTreeNode dmtn)
	{
         outputTree(pw, dmtn, 0);
    }


	/**
	 * Ouptut a tree to a print write.
	 *
	 * @param pw an instance of PrintWriter
	 * @param dmtn an instance of DefaultMutableTreeNode
	 * @param level limit the levels of tree to be printed
	 *
	 * @since DLBExt 1.0
	 */
	public void outputTree(PrintWriter pw, DefaultMutableTreeNode dmtn, int level)
	{
		if (dmtn == null) return;
		if (pw == null) return;

		TreeNode node = (TreeNode) dmtn.getUserObject();
		CodedEntry c = node.getConcept();

        StringBuffer sb = new StringBuffer();
			for (int i = 0; i < level; i++)
				sb.append('\t');
			String prefix = sb.toString();

		pw.println(prefix +
			c.getConceptCode() +
				"/" + node.getName());

        pw.flush();
		Enumeration enumeration = dmtn.children();
		while (enumeration.hasMoreElements())
		{
			DefaultMutableTreeNode child_node = (DefaultMutableTreeNode) enumeration.nextElement();
			outputTree(pw, child_node, level+1);
		}
	}

	/**
	 * Output properties of a given CodedEntry.
	 *
	 * @param pw the PrintWriter
	 * @param ce the CodedEntry
	 */
	public void outputProperties(PrintWriter pw, CodedEntry ce)
	{
        if (ce == null)
        {
 			getLogger().error("wrapper.getProperties invalid input -- ce is null.");
 			return;
		}

		Definition[] definitions = ce.getDefinition();
		for (int i=0; i<definitions.length; i++)
		{
		    Definition d = definitions[i];
		    pw.println(d.getProperty() + ": " + d.getText().getContent());
		}

		Comment[] comments = ce.getComment();
		for (int i=0; i<comments.length; i++)
		{
		    Comment c = comments[i];
		    pw.println(c.getProperty() + ": " + c.getText().getContent());
		}

		Instruction[] instructions = ce.getInstruction();
		for (int i=0; i<instructions.length; i++)
		{
		    Instruction inst = instructions[i];
		    pw.println(inst.getProperty() + ": " + inst.getText().getContent());

		}

		Presentation[] presentations = ce.getPresentation();
		if (presentations != null)
		{
			for (int i=0; i<presentations.length; i++)
			{
				Presentation p = presentations[i];
				Vector source_vec = getPropertySources(p);
				if (source_vec.size() == 0)
				{
//					java.lang.String prop_name = p.getProperty();
//					java.lang.String prop_value = p.getText().getContent();

				    pw.println(p.getProperty() + ": " + p.getText().getContent());
				}
			}
		}


 	    org.LexGrid.commonTypes.Property[] properties = ce.getConceptProperty();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			if (p != null)
			{
				pw.println(p.getProperty() + ": " + p.getText().getContent());
		    }
		}
	}


	/**
	 * Gets root concepts.
	 *
	 * @return an array containing root concept instances for a coding Scheme
	 *
	 * @since DLBExt 1.0
	 */
	public CodedEntry[] getRootConcepts()
	{
		/*
		boolean allAttributes = true;
		return getRootConcepts(allAttributes);
		*/
		return getConceptChildRoots();
	}


//	used
	/**
 * Gets the root concepts.
 *
 * @param allAttributes whether to return all attributes of the retrieved concepts
 *
 * @return an array containing root concept instances for a coding Scheme
 */
public CodedEntry[] getRootConcepts(boolean allAttributes)
	{
	    if (allAttributes)
		{
			return getRootConcepts(1);
		}
		return getRootConcepts(0);
	}



    /**
     * Gets the root concepts.
     *
     * @param ASD_index the AttributeSetDescriptor
     *
     * @return an array containing root concept instances for a coding Scheme
     */
    public CodedEntry[] getRootConcepts(int ASD_index)
    {
		return getConceptChildRoots();
	}



	/**
	 * To vector.
	 *
	 * @param a the a
	 *
	 * @return the vector< coded entry>
	 */
	private Vector<CodedEntry> toVector(CodedEntry[] a)
	{
		if (a == null) return null;
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		for (int i=0; i<a.length; i++)
		{
		    v.add(a[i]);
		}
		return v;
	}


	/**
	 * Gets the Coded Entries that match the search term.
	 *
	 * @param pattern the search text
	 * @param matchLimit the max number of matches to return
	 *
	 * @return the CodedEntry vector
	 */
	public Vector<CodedEntry> getMatches(
		String pattern,
		int matchLimit) throws LBParameterException
	{
		try {
			return findConceptsWithNameMatching(pattern, matchLimit);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}
	}

	 /**
 	 * Gets the concept status.
 	 *
 	 * @param conceptcode the conceptcode
 	 *
 	 * @return the concept status
 	 */
 	public String getConceptStatus(String conceptcode)
	 {
//		 String code = nameToCode(codingScheme, version, tag, conceptname);
		 String status = getConceptStatus(codingScheme, version, tag, conceptcode);
		 if (status == null) return "Unknown";
		 return status;
	 }


    /**
     * Checks for status.
     *
     * @param conceptcode the conceptcode
     *
     * @return boolean whether the concept has an assigned status
     */
    public Boolean hasStatus(String conceptcode)
    {
//		String code = nameToCode(codingScheme, version, tag, conceptname);
		String status = getConceptStatus(codingScheme, version, tag, conceptcode);
		if (status == null) return Boolean.FALSE;
		return Boolean.TRUE;
	}


	/**
	 * Checks if is valid history date.
	 *
	 * @param historydate the historydate
	 *
	 * @return true, if is valid history date
	 */
	public boolean isValidHistoryDate(Date historydate)
	{
		return isValidHistoryDate(codingScheme, historydate);
	}


    /**
     * Gets the property names for a given concept.
     *
     * @param conceptcode the concept code
     *
     * @return the property names
     */
    public Vector getPropertyNames(String conceptcode)
    {
        return getPropertyNames(codingScheme,
	                            version,
								tag,
								conceptcode);

	}


	/**
	 * Find concepts related to the given concept.
	 *
	 * @param code concept code
	 * @param relation relationship name
	 * @param direction true: code is source; false: code is target
	 *
	 * @return String Vector or related concept codes
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<String> findRelatedCodes(String code,
                                   String relation,
	                               boolean direction)
	{
		return findRelatedCodes(codingScheme,
	                                    version,
	                                    tag,
	                                    code,
	                                    relation,
	                                    direction);
	}

//	used
	/**
 * Gets the all qualifier types.
 *
 * @return all qualifier types for the coding scheme
 */
	public Vector<String> getAllQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}

//	used
	/**
 * Gets the all concept property qualifier types.
 *
 * @return the concept property qualifier types for the coding scheme
 */
	public Vector<String> getAllConceptPropertyQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}

	/**
	 * Gets the concept property qualifier types.
	 *
	 * @return the concept property qualifier types for the coding scheme
	 */
	public Vector<String> getConceptPropertyQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}

//	used
	/**
 * Gets the all concept association qualifier types.
 *
 * @return the all concept association qualifier types for the coding scheme
 */
public Vector<String> getAllConceptAssociationQualifierTypes()
	{
		return getSupportedAssociationQualifier(codingScheme);
	}

	/**
	 * Gets the concept association qualifier types.
	 *
	 * @return the concept association qualifier types for the coding scheme
	 */
	public Vector<String> getConceptAssociationQualifierTypes()
	{
		return getSupportedAssociationQualifier(codingScheme);
	}


    /**
     * Gets the all role types in namespace.
     *
     * @return the all role types in namespace for the coding scheme
     */
    public RoleType[] getAllRoleTypesInNamespace()
    {
		Vector v = getSupportedRoleNames();

		RoleType[] roletypes = new RoleType[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			String name = (String) v.elementAt(i);
			RoleType rt = new RoleType(name, -1, "");
			roletypes[i] = rt;
		}
		return roletypes;
	}

    /**
     * NCI change event2 edit action date.
     *
     * @param nce the nce
     *
     * @return the edits the action date
     */
    private EditActionDate NCIChangeEvent2EditActionDate(NCIChangeEvent nce)
    {
		if (nce == null) return null;
		EditActionDate ead = new EditActionDate();
		ead.setEditDate(nce.getEditDate());
		ChangeType ct =	nce.getEditaction();
		ead.setAction(ct.getType());
		return ead;
	}

//  used
    /**
 * Gets the concept history for the given concept.
 *
 * @param conceptcode the conceptcode
 *
 * @return the concept history
 */
public ConceptHistory getConceptHistory(String conceptcode)
    {
		ConceptHistory history = new ConceptHistory();
		history.setCode(conceptcode);
		history.setName(getConceptNameByCode(conceptcode));
		history.setNamespaceId(getNamespaceId(codingScheme));

        Vector v = getConceptHistory(codingScheme, conceptcode);
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			history.addEditActionDate(NCIChangeEvent2EditActionDate(nce));
		}

		return history;
	}


	/**
	 * Search concepts.
	 *
	 * @param pattern the search text
	 * @param searchLimit the max number of terms to return
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(String pattern, int searchLimit) throws LBParameterException
	{
		try {
			return searchConcepts(pattern, searchLimit, defaultAlgorithm);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}
	}


	/**
	 * Search concepts.
	 *
	 * @param pattern the search text
	 * @param matchAlgorithm the match algorithm
	 * @param max the max
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(String pattern, int max, String matchAlgorithm) throws LBParameterException
	{

		Vector w = null;
		try {
			w = findCodedEntriesWithDesignationMatching(
					  codingScheme,
					  version,
					  pattern,
					  null,
					  matchAlgorithm,
					  max);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}

		return toArray(w);
	}


//	/**
//	 * Searches concepts that match with a specific search criteria (complex search).
//	 *
//	 * @param namespaceId namespace id
//	 * @param pattern search pattern
//	 * @param limit maximum matched concepts
//	 * @param matchOption (by_name, by_property,by_role,etc)
//	 * @param matchType match type (name of property, role, etc., if applicable)
//	 * @param ASDIndex AttributeSetDescriptor index
//	 *
//	 * @return concepts that match with a specific search criteria in a byte
//	 * array
//	 *
//	 * @deprecated - evsapi4 does not support namespaceid
//	 */@Deprecated
//	private CodedEntry[] searchConcepts(
//	             int namespaceId,
//				 String pattern,
//				 int limit,
//				 int matchOption,
//				 String matchType, // role/property name, or source name (if search by name)
//				 int ASDIndex
//				 )
//	{
//        String codingSchemeName = "";
//        if (namespaceId <= 0)
//        {
//			codingSchemeName = codingScheme;
//		}
//		else
//		{
//			Integer nsid_obj = new Integer(namespaceId);
//			String nsid_str = nsid_obj.toString();
//			codingSchemeName = localName2FormalName(nsid_str);
//	    }
//
//	    return searchConcepts(
//	             codingSchemeName,
//				 pattern,
//				 limit,
//				 matchOption,
//				 matchType,
//				 ASDIndex
//				 );
//	}


    /**
     * Search concepts.
     *
     * @param pattern the search pattern
     * @param limit max concepts to return
     * @param matchOption (by_name, by_property,by_role,etc)
     * @param matchType match type (name of property, role, etc., if applicable)
     *
     * @return the coded entry[]
     */
    public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType)
	{
        return searchConcepts(
			     codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType);
	}


//  used
    /**
 * Search concepts.
 *
 * @param pattern the pattern
 * @param limit the max number of concepts to return
 * @param matchOption (by_name, by_property,by_role,etc)
 * @param matchType match type (name of property, role, etc., if applicable)
 * @param ASDIndex parameter not used
 *
 * @return the coded entry[]
 * @deprecated - use searchConcepts(String,int,int,String)
 */@Deprecated
    public CodedEntry[] searchConcepts(
    		String pattern,
    		int limit,
    		int matchOption,
    		String matchType,
    		@SuppressWarnings("unused")int ASDIndex)
    {
    	return searchConcepts(
    			codingScheme,
    			pattern,
    			limit,
    			matchOption,
    			matchType,
    			defaultAlgorithm,
    			null);
	}

	/**
	 * Search concepts.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param ASDIndex parameter not used
	 *
	 * @return the coded entry[]
	 * @deprecated - use searchConcepts(String,String,int,int,String)
	 */@Deprecated
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 @SuppressWarnings("unused")int ASDIndex)
	{
		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 defaultAlgorithm,
				 null);
	}

	/**
	 * Search concepts.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType)
	{
		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 defaultAlgorithm,
				 null);
	}

//	used
	/**
 * Search concepts.
 *
 * @param pattern the search pattern
 * @param limit max number of concepts to return
 * @param matchOption (by_name, by_property,by_role,etc)
 * @param matchType match type (name of property, role, etc., if applicable)
 * @param ASDIndex parameter not used
 * @param matchAlgorithm the match algorithm
 *
 * @return the coded entry[]
 * @deprecated - use searchConcept(String,int,int,String,String)
 */@Deprecated
	public CodedEntry[] searchConcepts(
			String pattern,
			int limit,
			int matchOption,
			String matchType,
			@SuppressWarnings("unused")int ASDIndex,
			String matchAlgorithm)
    {
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 null);
    }

	/**
	 * Search concepts.
	 *
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the coded entry[]
	 */
		public CodedEntry[] searchConcepts(
				String pattern,
				int limit,
				int matchOption,
				String matchType,
				String matchAlgorithm)
	    {
			return searchConcepts(
		             codingScheme,
					 pattern,
					 limit,
					 matchOption,
					 matchType,
					 matchAlgorithm,
					 null);
	    }
	/**
	 * Search concepts.
	 *
	 * @param pattern the search pattern
	 * @param codingSchemeName the coding scheme name
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param ASDIndex parameter not used
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the coded entry[]
	 * @deprecated - use searchConcepts(String,String,int,int,String,String)
	 */@Deprecated
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 @SuppressWarnings("unused")int ASDIndex,
				 String matchAlgorithm)
	{

		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 null); // restricted_property_vec

	}
	/**
	 * Search concepts.
	 *
	 * @param pattern the search pattern
	 * @param codingSchemeName the coding scheme name
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm)
	{

		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 null); // restricted_property_vec

	}
	/**
	 * Search concepts.
	 *
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param ASDIndex parameter not used
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the coded entry[]
	 * @deprecated - use searchConcepts(String,int,int,String,String,Vector)
	 */@Deprecated
	public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 @SuppressWarnings("unused")int ASDIndex,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 restricted_property_vec);

	}
	/**
	 * Search concepts.
	 *
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 restricted_property_vec);

	}


	/**
	 * Search concepts.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param ASDIndex parameter not used
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the coded entry[]
	 * @deprecated - use searchConcepts(String,String,String,int,int,String,String,Vector)
	 */@Deprecated
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 @SuppressWarnings("unused")int ASDIndex,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (matchAlgorithm == null || matchAlgorithm.compareTo("") == 0)
		{
			matchAlgorithm = defaultAlgorithm;
		}
	    try {

			if (matchOption == MATCH_BY_NAME)
			{
                Vector<String> source_vec = null;
                if (matchType != null && matchType.compareTo("") != 0)
                {
					source_vec = new Vector<String>();
					source_vec.add(matchType);
				}

				v = findCodedEntriesWithDesignationMatching(
							  codingSchemeName,
							  null,
							  pattern,
							  false, // preferredOnly
							  null, //  sortByProperty_vec
							  source_vec, // source_vec
							  matchAlgorithm,
							  limit);

		        return toArray(v);
			}

			else if (matchOption == MATCH_BY_ROLE)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_PROPERTY)
			{
				 v = findConceptsWithPropertyMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm);
			}

			else if (matchOption == MATCH_BY_SILO)
			{
                 // not supported by LexBIG
			}

			else if (matchOption == MATCH_BY_ASSOCIATION)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ROLE)
			{
			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ASSOCIATION)
			{

			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_SOURCE_CODE)
			{
				// search by an source code
				// if no code is specified, then all concepts have the specified source will be retrieved
				v = findConceptsWithSourceCodeMatching(codingSchemeName, matchType, pattern, limit);
			}

		} catch (Exception e) {
	        e.printStackTrace();
		}

		return toArray(v);
	}


	/**
	 * Search concepts.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param pattern the search pattern
	 * @param limit max number of concepts to return
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the coded entry[]
	 */
	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (matchAlgorithm == null || matchAlgorithm.compareTo("") == 0)
		{
			matchAlgorithm = defaultAlgorithm;
		}
	    try {

			if (matchOption == MATCH_BY_NAME)
			{
                Vector<String> source_vec = null;
                if (matchType != null && matchType.compareTo("") != 0)
                {
					source_vec = new Vector<String>();
					source_vec.add(matchType);
				}

				v = findCodedEntriesWithDesignationMatching(
							  codingSchemeName,
							  null,
							  pattern,
							  source_vec, // source_vec
							  matchAlgorithm,
							  limit);

		        return toArray(v);
			}

			else if (matchOption == MATCH_BY_ROLE)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_PROPERTY)
			{
				 v = findConceptsWithPropertyMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm);
			}

			else if (matchOption == MATCH_BY_SILO)
			{
                 // not supported by LexBIG
			}

			else if (matchOption == MATCH_BY_ASSOCIATION)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ROLE)
			{
			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ASSOCIATION)
			{

			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_SOURCE_CODE)
			{
				// search by an source code
				// if no code is specified, then all concepts have the specified source will be retrieved
				v = findConceptsWithSourceCodeMatching(codingSchemeName, matchType, pattern, limit);
			}

		} catch (Exception e) {
	        e.printStackTrace();
		}

		return toArray(v);
	}

	/**
	 * Gets the matched concept count.
	 *
	 * @param pattern the search pattern
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the matched concept count
	 */
	public int getMatchedConceptCount(
				 String pattern,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		return getMatchedConceptCount(
	             codingScheme,
				 pattern,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 restricted_property_vec);
	}




	/**
	 * Gets the matched concept count.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param pattern the search pattern
	 * @param matchOption (by_name, by_property,by_role,etc)
	 * @param matchType match type (name of property, role, etc., if applicable)
	 * @param matchAlgorithm the match algorithm
	 * @param restricted_property_vec the restricted_property_vec
	 *
	 * @return the matched concept count
	 */
	public int getMatchedConceptCount(
	             String codingSchemeName,
				 String pattern,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		CodedEntry[] concepts = searchConcepts(
	             codingSchemeName,
				 pattern,
				 100000,
				 matchOption,
				 matchType,
				 0,//ASDIndex,
				 matchAlgorithm,
				 restricted_property_vec);
		return concepts.length;
	}


    /**
     * Gets the roles pointing from a given concept.
     *
     * @param ce the CodedEntry
     *
     * @return the roles
     */
    public Vector<Role> getRoles(CodedEntry ce)
    {
		return getRoles(codingScheme, version, tag, ce);
	}

    /**
     * Gets the roles pointing from a given concept.
     *
     * @param ce the CodedEntry
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     *
     * @return the roles
     */
    public Vector<Role> getRoles(String cScheme, String vers, String ltag, CodedEntry ce)
    {
		if (ce == null) return null;
		return getRoles(cScheme, vers, ltag, ce.getConceptCode());
	}


    /**
     * Gets the roles pointing from a given concept.
     *
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param code the concept code
     *
     * @return the roles
     */
    public Vector<Role> getRoles(String cScheme, String vers, String ltag, String code)
    {
		Vector<Role> roles = new Vector<Role>();
		Vector<ConceptAssociation> w1 = null;
		try {
			w1 = getConceptAssociations(   cScheme,
					                       vers,
					                       ltag,
										   code,
										   true,
										   ROLE_RELATION);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}


		if (w1 != null)
		{
			for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						roles.add(conceptAssociation2Role(nv));
					}
				}
			}

		}
		return roles;
	}


    /**
     * Gets the roles pointed to a given concept
     *
     * @param ce the CodedEntry
     *
     * @return the inverse roles
     */
    public Vector<Role> getInverseRoles(CodedEntry ce)
    {
		return getInverseRoles(codingScheme, version, tag, ce);
	}

    /**
     *  Gets the roles pointed to a given concept
     *
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param ce the ce
     *
     * @return the inverse roles
     */
    public Vector<Role> getInverseRoles(String cScheme, String vers, String ltag,CodedEntry ce)
    {
		if (ce == null) return null;
		return getInverseRoles(cScheme, vers, ltag, ce.getConceptCode());
	}


    /**
     *  Gets the roles pointed to a given concept
     *
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param code the concept code
     *
     * @return the inverse roles
     */
    public Vector<Role> getInverseRoles(String cScheme, String vers, String ltag, String code)
    {
    	Vector<Role> roles = new Vector<Role>();
    	Vector<ConceptAssociation> w2 = null;
    	try {
    		w2 = getConceptAssociations(
    				cScheme,
    				vers,
    				ltag,
    				code,
    				false,
    				ROLE_RELATION);
    	} catch (Exception e) {
    		e.printStackTrace();
    		return null;
		}

		if (w2 != null)
		{
			for (int i=0; i<w2.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w2.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						roles.add(conceptAssociation2Role(nv));
					}
				}
			}
		}
		return roles;
	}


    /**
     * Gets the associations pointed from a given concept
     *
     * @param ce the CodedEntry
     *
     * @return the concept associations
     */
    public Vector<ConceptAssociation> getConceptAssociations(CodedEntry ce)
    {
		return getConceptAssociations(codingScheme, version, tag, ce);
	}

    /**
     * Gets the associations pointed from a given concept
     *
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param ce the CodedEntry
     *
     * @return the concept associations
     */
    public Vector<ConceptAssociation> getConceptAssociations(String cScheme, String vers, String ltag,CodedEntry ce)
    {
		if (ce == null) return null;
		return getConceptAssociations(cScheme, vers, ltag, ce.getConceptCode());
	}

    /**
    * Gets the associations pointed from a given concept
    *
    * @param cScheme the coding scheme
    * @param vers the version
    * @param ltag the tag
    * @param code the concept code
    *
    * @return the concept associations
    */
    public Vector<ConceptAssociation> getConceptAssociations(String cScheme, String vers, String ltag, String code)
    {
		Vector<ConceptAssociation> associations = new Vector<ConceptAssociation>();
		Vector<ConceptAssociation> w1 = null;

		String relationName = null;
		if (hasRelation(cScheme, vers, ltag, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;
		try {
			w1 = getConceptAssociations(
					cScheme,
					vers,
					ltag,
					code,
					true,
					relationName);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		if (w1 != null)
		{
			for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						associations.add(nv);
					}
				}
			}

		}
		return associations;
	}



    /**
    * Gets the associations pointed from a given concept
    *
    * @param cScheme the coding scheme
    * @param vers the version
    * @param ltag the tag
    * @param code the concept code
    *
    * @return the concept associations
    */
    public Vector<ConceptAssociation> getInverseConceptAssociations(String cScheme, String vers, String ltag, String code)
    {
		Vector<ConceptAssociation> associations = new Vector<ConceptAssociation>();
		Vector<ConceptAssociation> w1 = null;

		String relationName = null;
		if (hasRelation(cScheme, vers, ltag, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;
		try {
			w1 = getConceptAssociations(
					cScheme,
					vers,
					ltag,
					code,
					false,
					relationName);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		if (w1 != null)
		{
			for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						associations.add(nv);
					}
				}
			}

		}
		return associations;
	}



    /**
     * Gets the associations pointed to a given concept
     *
     * @param ce the ce
     *
     * @return the inverse concept associations
     */
    public Vector getInverseConceptAssociations(CodedEntry ce)
    {
		return getInverseConceptAssociations(codingScheme, version, tag, ce);
	}

    /**
     * Gets the associations pointed to a given concept
     *
    * @param cScheme the coding scheme
    * @param vers the version
    * @param ltag the tag
     * @param ce the CodedEntry
     *
     * @return the inverse concept associations
     */
    public Vector getInverseConceptAssociations(String cScheme, String vers, String ltag, CodedEntry ce)
    {
		if (ce == null) return null;
		return getInverseConceptAssociations(cScheme, vers, ltag, ce.getConceptCode());
	}


    /**
     * Gets the associations pointed to a given concept
     *
     * @param cScheme the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param code the concept code
     *
     * @return the inverse concept associations
     */
     // GForge 13294, KLO 032808
     /*
    public Vector<ConceptAssociation> getInverseConceptAssociations(String cScheme, String vers, String ltag, String code)
    {
    	Vector<ConceptAssociation> associations = new Vector<ConceptAssociation>();
    	Vector<ConceptAssociation> w1 = null;

    	String relationName = null;
    	if (hasRelation(cScheme, vers, ltag, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;
    	try {
    		w1 = getConceptAssociations(
    				cScheme,
    				vers,
    				ltag,
    				code,
    				false,
    				//ASSOCIATION_RELATION);
    				relationName);

    	} catch (Exception ex) {
    		//ex.printStackTrace();
    		return null;
    	}

    	if (w1 != null)
    	{
    		for (int i=0; i<w1.size(); i++)
    		{
    			gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						associations.add(nv);
					}
				}
			}

		}
		return associations;
	}
	*/


    /**
     * Output concept to PrintWriter, for debug purposes
     *
     * @param pw the PrintWriter
     * @param ce the CodedEntry
     */
    public void outputConcept(PrintWriter pw, CodedEntry ce)
	{
		if (pw == null)
		{
			System.out.println("outputConcept -- PrintWriter is null.");
			return;
		}
		if (ce == null)
		{
			System.out.println("outputConcept -- ce is null.");
			return;
		}

        String conceptName = getConceptName(ce);
//        String code = ce.getConceptCode();
		pw.println("\nConcept: " + conceptName);
		pw.println("Code: " + ce.getConceptCode());
		//pw.println("Concept Status: " + ce.getConceptStatus());
		pw.println("Is Concept Active: " + ce.getIsActive());

		pw.println("\nProperties:");
		Vector properties = getProperties(ce);
		for (int i=0; i<properties.size(); i++)
		{
			gov.nih.nci.lexbig.ext.Property p = (gov.nih.nci.lexbig.ext.Property) properties.elementAt(i);
			pw.println("\t" + p.getName() + ": " + p.getValue());
			Vector qualifier_vec = p.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}
		}


		pw.println("\nRoles:");
		Vector roles = getRoles(ce);
		for (int i=0; i<roles.size(); i++)
		{
			Role role = (Role) roles.elementAt(i);
            CodedEntry c = getConcept(role.getValue(), 0);
			pw.println("\t["
				+  conceptName
				+  "]-->(" + role.getName()
				+")-->["
				+ getConceptName(c) + " (" + role.getValue() + ")"
				+ "]");

			Vector qualifier_vec = role.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (roles.size() == 0)
		{
			pw.println("\tNone");
		}

        pw.println("\nInverse Roles:");
		roles = getInverseRoles(ce);
		for (int i=0; i<roles.size(); i++)
		{
			Role role = (Role) roles.elementAt(i);

//			CodedEntry c = getConcept(role.getValue(), 0);

			pw.println("\t["
				+  role.getValue()
				+  "]-->(" + role.getName()
				+")-->["
				+ conceptName + " (" + ce.getConceptCode() + ")"
				+ "]");

			Vector qualifier_vec = role.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (roles.size() == 0)
		{
			pw.println("\tNone");
		}

		pw.println("\nAssociations:");
		Vector associations = getConceptAssociations(ce);
		for (int i=0; i<associations.size(); i++)
		{
			gov.nih.nci.lexbig.ext.ConceptAssociation asso = (ConceptAssociation) associations.elementAt(i);
			CodedEntry c = getConcept(asso.getValue(), 0);
			pw.println("\t["
				+  conceptName
				+  "]-->(" + asso.getName()
				+")-->["
				+ getConceptName(c) + " (" + asso.getValue() + ")"
				+ "]");

			Vector qualifier_vec = asso.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (associations.size() == 0)
		{
			pw.println("\tNone");
		}

        pw.println("\nInverse Associations:");
		associations = getInverseConceptAssociations(ce);
		for (int i=0; i<associations.size(); i++)
		{
		    gov.nih.nci.lexbig.ext.ConceptAssociation asso = (ConceptAssociation) associations.elementAt(i);
//		    CodedEntry c = getConcept(asso.getValue(), 0);

			pw.println("\t["
				+  asso.getValue()
				+  "]-->(" + asso.getName()
				+")-->["
				+ conceptName + " (" + ce.getConceptCode() + ")"
				+ "]");

			Vector qualifier_vec = asso.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}
		}
		if (associations.size() == 0)
		{
			pw.println("\tNone");
		}
		pw.println("------------------\n\n");
	}


/*
(C) id
*/

	/**
 * Gets the role type by role id.
 *
 * @param roleId the role id
 *
 * @return the RoleType
 */
public RoleType getRoleTypeById(int roleId)
	{
		Vector v = getSupportedRoleNames();
		if (roleId <0 || roleId >= v.size()) return null;
		String rolename = (String) v.elementAt(roleId);
		return new RoleType(rolename, roleId, "");
	}


	/**
	 * Gets the property type by property id.
	 *
	 * @param propertyId the property id
	 *
	 * @return the property type
	 */
	public PropertyType getPropertyTypeById(int propertyId)
	{
		PropertyType[] properties = getAllPropertyTypes();
		if (propertyId >= 0 && propertyId<properties.length)
		{
			String name = properties[propertyId].getName();
			return new PropertyType(name, propertyId, "");
		}
		return null;
	}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Methods for supporting metathesaurus queries
/////////////////////////////////////////////////////////////////////////////////////////////////
//used
	/**
 * Gets the supported sources.
 *
 * @return the supported sources
 */
public Vector<String> getSupportedSources()
	{
		return getSupportedSourceNames(codingScheme, version, tag);
	}


	/**
	 * Returns a vector of the names of org.LexGrid.naming.SupportedContext for the default coding scheme.
	 *
	 * @return a vector of the names of org.LexGrid.naming.SupportedContext for
	 * the default coding scheme.
	 *
	 * @since DLBExt 1.0
	 */
	public Vector getSupportedContext()
	{
		return getSupportedContext(codingScheme, version, tag);
	}

	/**
	 * Gets the supported context count.
	 *
	 * @return the supported context count
	 */
	public int getSupportedContextCount()
	{
		return getSupportedContextCount(codingScheme, version, tag);
	}

//used
    /**
 * Find concepts with source code matching.
 *
 * @param sourceAbbr the source abbr
 * @param code the code
 * @param limit the max number of results to return
 *
 * @return the vector< coded entry>
 */
public Vector<CodedEntry> findConceptsWithSourceCodeMatching(String sourceAbbr, String code, int limit)
    {
         return findConceptsWithSourceCodeMatching(codingScheme, sourceAbbr, code, limit);
	}


    /**
     * Find concepts with source code matching.
     *
     * @param codingSchemeName the coding scheme name
     * @param sourceAbbr the source abbr
     * @param code the code
     * @param limit the max number of results to return
     *
     * @return the vector< coded entry>
     */
    public Vector<CodedEntry> findConceptsWithSourceCodeMatching(String codingSchemeName, String sourceAbbr, String code, int limit)
    {
		/*
        Vector sources = getSupportedSourceNames(codingSchemeName, null, null);
        if (!sources.contains(sourceAbbr))
        {
			getLogger().warn("Source " + sourceAbbr + " is not identifiable.");
			return null;
		}
		*/

		Vector<CodedEntry> v = findCodedEntriesWithSourceCodeMatching(codingSchemeName, sourceAbbr, code, limit);
		return v;
	}


    /**
     * Gets the related concepts.
     *
     * @param conceptCode the concept code
     * @param associationName the association name
     *
     * @return the related concepts
     */
    public Vector getRelatedConcepts(String conceptCode, String associationName)
    {
		return getRelatedConceptsBySource(
		 								 conceptCode,
		 								 associationName,
		 								 null); // sourceAbbr
	}

//  used
    /**
 * Gets the related concepts.
 *
 * @param conceptCode the concept code
 * @param direction true for down, false for up
 * @param associationName the association name
 *
 * @return the related concepts
 */
public Vector getRelatedConcepts(String conceptCode, boolean direction, String associationName)
    {
		return getRelatedConceptsBySource(
		 								 conceptCode,
		 								 direction,
		 								 associationName,
		 								 null); // sourceAbbr
	}


    // 031407
    /**
     * Gets the related concepts.
     *
     * @param conceptCode the concept code
     * @param direction the direction
     * @param associationName the association name
     * @param ASD_Index parameter not used
     *
     * @return the related concepts
     *
     * @deprecated - use getRelatedConcepts(String, boolean, String)
     */@Deprecated
    public Vector<CodedEntry> getRelatedConcepts(String conceptCode, boolean direction, String associationName, @SuppressWarnings("unused")int ASD_Index)
    {
        Vector<CodedEntry> v = getRelatedCodedEntries(codingScheme,
 								 version,
 								 tag,
 								 conceptCode,
 								 direction,
 								 associationName);

		return v;
	}



    /**
     * Gets the related concepts by source.
     *
     * @param conceptCode the concept code
     * @param associationName the association name
     * @param sourceAbbr the source abbr
     *
     * @return the related concepts by source
     */
    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 String associationName,
 								 String sourceAbbr)
	{
	    if (sourceAbbr != null)
	    {
			Vector sources = getSupportedSources();
			if (!sources.contains(sourceAbbr))
			{
				getLogger().error("Source " + sourceAbbr + " is not identifiable.");
				return null;
			}
	    }

	    /*
        Vector v1 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 true,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        Vector v2 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 false,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        */

         Vector<CodedEntry> v1 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 true,//boolean direction,
 										 associationName,
 										 sourceAbbr
 										 );
         Vector<CodedEntry> v2 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 false,//boolean direction,
 										 associationName,
 										 sourceAbbr
										 );


        // 031507
        if (v1 == null && v2 == null) return new Vector<CodedEntry>();
        else if (v1 != null && v2 == null) return v1;
        else if (v1 == null && v2 != null) return v2;
        else
        {
			for (int i=0; i<v2.size(); i++)
			{
				CodedEntry ce = (CodedEntry) v2.elementAt(i);
				v1.add(ce);
			}
			return v1;
		}

	}

    // No direction is specified == both direction applied:
    /**
     * Gets the related concepts by source.
     *
     * @param conceptCode the concept code
     * @param associationName the association name
     * @param sourceAbbr the source abbr
     * @param ASD_Index parameter not used
     *
     * @return the related concepts by source
     *
     * @deprecated - use getRelatedConceptsBySource(String,String,String)
     */@Deprecated
    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 String associationName,
 								 String sourceAbbr,
 								@SuppressWarnings("unused")int ASD_Index)
   {
	    if (sourceAbbr != null)
	    {
			Vector sources = getSupportedSources();
			if (!sources.contains(sourceAbbr))
			{
				getLogger().error("Source " + sourceAbbr + " is not identifiable.");
				return null;
			}
	    }

	    /*
        Vector v1 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 true,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        Vector v2 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 false,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        */

         Vector<CodedEntry> v1 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 new Vector(),//Vector properties,
 										 true,//boolean direction,
 										 associationName,
 										 1,//int resolveAssociationDepth,
 										 1,//int resolveCodedEntryDepth,
 										 sourceAbbr
 										 );
         Vector<CodedEntry> v2 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 new Vector(),//Vector properties,
 										 false,//boolean direction,
 										 associationName,
 										 1,//int resolveAssociationDepth,
 										 1,//int resolveCodedEntryDepth,
 										 sourceAbbr
										 );


        // 031507
        if (v1 == null && v2 == null) return new Vector<CodedEntry>();
        else if (v1 != null && v2 == null) return v1;
        else if (v1 == null && v2 != null) return v2;
        else
        {
			for (int i=0; i<v2.size(); i++)
			{
				CodedEntry ce = (CodedEntry) v2.elementAt(i);
				v1.add(ce);
			}
			return v1;
		}
	}

//  used
    /**
 * Gets the related concepts by source.
 *
 * @param conceptCode the concept code
 * @param direction true for down, false for up
 * @param associationName the association name
 * @param sourceAbbr the source abbr
 *
 * @return the related concepts by source
 */
public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 boolean direction,
 								 String associationName,
 								 String sourceAbbr)
	{
	if (sourceAbbr != null)
	{
		Vector sources = getSupportedSources();

		if (!sources.contains(sourceAbbr))
		{
			getLogger().error("Source " + sourceAbbr + " is not identifiable.");
			return null;
		}
    }
/*
    Vector v = getAssociatedCodedEntriesBySource(codingScheme,
									 null,//String version,
									 null,//String tag,
                                     conceptCode,
									 new Vector(),//Vector properties,
									 //true,//boolean direction, 050807
									 direction,
									 associationName,
									 1,//int resolveAssociationDepth,
									 1,//int resolveCodedEntryDepth,
									 sourceAbbr
									 );
*/

    Vector<CodedEntry> v = getAssociatedCodedEntriesBySource(codingScheme,
									 version,
									 tag,
                                     conceptCode,
									 direction,
									 associationName,
									 sourceAbbr
									 );

    // 031507
    if (v == null) return new Vector<CodedEntry>();
	return v;

	}

    /**
     * Gets the related concepts by source.
     *
     * @param conceptCode the concept code
     * @param direction true for down, false for up
     * @param associationName the association name
     * @param sourceAbbr the source abbr
     * @param ASD_Index parameter not used
     *
     * @return the related concepts by source
     *
     * @deprecated - use getRelatedConceptBySource(String,boolean,String,String)
     */@Deprecated
    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 boolean direction,
 								 String associationName,
 								 String sourceAbbr,
 								@SuppressWarnings("unused")int ASD_Index)
    {
		if (sourceAbbr != null)
		{
			Vector sources = getSupportedSources();

			if (!sources.contains(sourceAbbr))
			{
				getLogger().error("Source " + sourceAbbr + " is not identifiable.");
				return null;
			}
	    }
/*
        Vector v = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 //true,//boolean direction, 050807
										 direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
*/

        Vector<CodedEntry> v = getAssociatedCodedEntriesBySource(codingScheme,
										 version,
										 tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 //true,//boolean direction, 050807
										 direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );

        // 031507
        if (v == null) return new Vector<CodedEntry>();
		return v;
	}


    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //static String ROOT_ASSOCIATION = "owl:AnnotationProperty";
    //static String ROOT_ROLE = "owl:ObjectProperty";

	/**
     * Gets the supported association names for the coding scheme
     *
     * @return a String Vector of the supported association names
     *
     * @deprecated - use getAllAssociationTypes
     */@Deprecated
    public Vector<String> getSupportedAssociationNames()
	{
         return getSupportedAssociationNames(codingScheme);
	}

	/**
	 * Gets the supported concept association names.
	 *
	 * @return a String Vector of the supported concept association names
	 *
	 * @deprecated - use getConceptAssociationTypes
	 */@Deprecated
	public Vector getSupportedConceptAssociationNames()
	{
         return getConceptAssociationTypes();
	}

	/**
	 * Gets the supported concept association names.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return the supported concept association names
	 */
	public Vector getSupportedConceptAssociationNames(String codingSchemeName)
	{
         return getAssociationNamesByRelation(codingSchemeName, DLBWrapper.ASSOCIATION_RELATION);
	}

	/**
	 * Gets the supported concept association names.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 *
	 * @return the supported concept association names
	 */
	public Vector getSupportedConceptAssociationNames(String codingSchemeName, String vers)
	{
         return getAssociationNamesByRelation(codingSchemeName, vers, DLBWrapper.ASSOCIATION_RELATION);
	}

	/**
	 * Gets the supported role names for the coding scheme
	 *
	 * @return the supported role names
	 * @deprecated - use getAllRoleNames()
	 */@Deprecated
	public Vector<String> getSupportedRoleNames()
	{
         return getSupportedRoleNames(codingScheme);
	}

    /**
     * Gets the concept association types for the coding scheme
     *
     * @return the concept association types
     */
    public Vector getConceptAssociationTypes()
    {
		if (hasRelation(codingScheme, ASSOCIATION_RELATION))
		{
			return getAssociationNamesByRelation(codingScheme, version, ASSOCIATION_RELATION);
		}
		else
		{
			return getAssociationNamesByRelation(codingScheme, version, null);
		}
    }

//  used
    /**
     * Gets the all association types for the coding scheme
     *
     * @return the all association types
     *
     *
     */
    public Vector getAllAssociationTypes()
    {
//  	return getSupportedAssociationNames();
    	return getSupportedAssociationNames(codingScheme);
    }

//  used
    /**
     * Gets the all concept association types for the coding scheme
     * @return the all concept association types
     *
     * @deprecated - use getConceptAssociationTypes
     */@Deprecated
    public Vector getAllConceptAssociationTypes()
    {
    	return getConceptAssociationTypes();
    }



    /**
     * Gets the supported property qualifiers for the coding scheme
	 *
	 * @return the supported property qualifiers
	 */
	public Vector getSupportedPropertyQualifiers()
	{
         return getSupportedPropertyQualifiers(codingScheme);
	}


	/**
	 * Gets the definition collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the definition collectionby code
	 */
	public Vector getDefinitionCollectionbyCode(String codingSchemeName, String code) {
		return getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_DEFINITION, null);
	}

	/**
	 * Gets the semantic type collection for the given concept.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the semantic type collectionby code
	 */
	public Vector getSemanticTypeCollectionbyCode(String codingSchemeName, String code) {
		return getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_PROPERTY, "Semantic_Type");
	}

	/**
	 * Gets the property collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the property collectionby code
	 */
	public Vector getPropertyCollectionbyCode(String codingSchemeName, String code) {
		Vector v = new Vector();
		Vector u = getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_PROPERTY, null);
		for (int i=0; i<u.size(); i++)
		{
			v.add(u.elementAt(i));
		}
		u = getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_DEFINITION, null);
		for (int i=0; i<u.size(); i++)
		{
			v.add(u.elementAt(i));
		}
		u = getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_COMMENT, null);
		for (int i=0; i<u.size(); i++)
		{
			v.add(u.elementAt(i));
		}
		u = getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_INSTRUCTION, null);
		for (int i=0; i<u.size(); i++)
		{
			v.add(u.elementAt(i));
		}

		//KLO, 04/30/08
		u = getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_PRESENTATION, null);
		for (int i=0; i<u.size(); i++)
		{
			v.add(u.elementAt(i));
		}


		return v;
	}


	/**
	 * Gets the atom collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the atom collectionby code
	 */
	public Vector getAtomCollectionbyCode(String codingSchemeName, String code) {
		return getConceptProperties(codingSchemeName, code, DLBWrapper.TYPE_PRESENTATION, null);
	}


	/**
	 * Gets the role collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the role collectionby code
	 */
	public Vector getRoleCollectionbyCode(String codingSchemeName, String code) {
		CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
		return getRoles(ce);
	}

	/**
	 * Gets the inverse role collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the inverse role collectionby code
	 */
	public Vector getInverseRoleCollectionbyCode(String codingSchemeName, String code) {
		CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
		return getInverseRoles(ce);
	}

	/**
	 * Gets the association collection for the given concept
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the association collectionby code
	 */
	public Vector getAssociationCollectionbyCode(String codingSchemeName, String code) {
		CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
		return getConceptAssociations(ce);
    }

	/**
	 * Gets the inverse association collection for the given concept.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the inverse association collectionby code
	 */
	public Vector getInverseAssociationCollectionbyCode(String codingSchemeName, String code) {
		// GForge 13294, KLO 032808
		/*
		CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
	    return getInverseConceptAssociations(ce);
	    */
		return getInverseConceptAssociations(codingSchemeName, null, null, code);
    }


	/**
	 * Gets the history collection for the given conceptfor the given concept.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the history collectionby code
	 */
	public Vector<EditActionDate> getHistoryCollectionbyCode(String codingSchemeName, String code) {
		// 072807
	    Vector<NCIChangeEvent> v = getConceptHistory(codingSchemeName, code);
	    Vector<EditActionDate> w = new Vector<EditActionDate>();
		for (int i=0; i<v.size(); i++)
		{
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			EditActionDate e = new EditActionDate();
			e.setEditDate(nce.getEditDate());
			e.setAction(nce.getEditaction().getType());
			w.add(e);
		}
		return w;
	}

    /**
     * Checks if association supported in the coding scheme
     *
     * @param name the association name
     *
     * @return true, if is association supported
     */
    public boolean isAssociationSupported(String name)
	{
		Vector v = getSupportedAssociationNames(codingScheme);
		return v.contains(name);
	}
/*
    public boolean isAssociationSupported(String codingSchemeName, String name)
	{
		Vector v = getSupportedAssociationNames(codingSchemeName, version, null);
		return v.contains(name);
	}
*/



    /**
     * Match by source code.
     *
     * @param codingSchemeName the coding scheme name
     * @param pattern the pattern
     * @param limit the limit
     * @param source the source
     * @param ASDIndex parameter not used
     *
     * @return the coded entry[]
     * @deprecated - use searchConcepts(String,String,int,String)
     */@SuppressWarnings("unused")
	@Deprecated
    private CodedEntry[] matchBySourceCode(String codingSchemeName, String pattern, int limit, String source,@SuppressWarnings("unused") int ASDIndex)
    {
    	// match by code
    	int matchOption = MATCH_BY_SOURCE_CODE;
		String matchType = source;
		CodedEntry[] concepts = searchConcepts(
					  codingSchemeName,
					  pattern,
					  limit,
					  matchOption,
					  matchType);


        HashSet<String> hset = new HashSet<String>();

        // match by name
		Vector<String> source_vec = new Vector<String>();
		source_vec.add(source);
		String matchAlgorithm = "LuceneQuery";

        Vector<CodedEntry> v = null;
        try {
			v = findCodedEntriesWithDesignationMatching(
					  codingSchemeName,
					  null,
					  pattern,
					  false, // preferredOnly
					  null, //  sortByProperty_vec
					  source_vec, // source_vec
					  matchAlgorithm,
					  limit);
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		for (int j=0; j<v.size(); j++)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(j);
			hset.add(ce.getConceptCode());
		}

		for (int i=0; i<concepts.length; i++)
		{
			CodedEntry ce = concepts[i];
			if (!hset.contains(ce.getConceptCode()))
			{
				hset.add(ce.getConceptCode());
				v.add(ce);
			}
		}

		concepts = new CodedEntry[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			concepts[i] = (CodedEntry) v.elementAt(i);
		}
		return concepts;
	}



 // boolean shortResult, boolean score not used.


    /**
     * Search by source code.
     *
     * @param codingSchemeName the coding scheme name
     * @param pattern the pattern
     * @param limit the limit
     * @param source the source
     *
     * @return the coded entry[]
     */
    private CodedEntry[] searchConcepts(String codingSchemeName,
    		String pattern,
    		int limit,
    		String source) throws LBParameterException
    {
    	// match by code
    	int matchOption = MATCH_BY_SOURCE_CODE;
		String matchType = source;
		CodedEntry[] concepts = searchConcepts(
					  codingSchemeName,
					  pattern,
					  limit,
					  matchOption,
					  matchType);


        HashSet<String> hset = new HashSet<String>();

        // match by name
		Vector<String> source_vec = new Vector<String>();
		source_vec.add(source);
		String matchAlgorithm = "LuceneQuery";

		Vector<CodedEntry> v = null;
		try {
			v = findCodedEntriesWithDesignationMatching(
					  codingSchemeName,
					  null,
					  pattern,
					  source_vec, // source_vec
					  matchAlgorithm,
					  limit);
		} catch (Exception ex) {
			throw new LBParameterException(ex.getMessage());
		}

		for (int j=0; j<v.size(); j++)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(j);
			hset.add(ce.getConceptCode());
		}

		for (int i=0; i<concepts.length; i++)
		{
			CodedEntry ce = concepts[i];
			if (!hset.contains(ce.getConceptCode()))
			{
				hset.add(ce.getConceptCode());
				v.add(ce);
			}
		}

		concepts = new CodedEntry[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			concepts[i] = (CodedEntry) v.elementAt(i);
		}
		return concepts;
	}


	/**
  * Search concepts.
  *
  * @param scheme the coding scheme
  * @param searchTerm the search term
  * @param limit the max number of results to return
  * @param source the source
  * @param cui the cui
  * @param shortResult - parameter not used
  * @param score - parameter not used
  * @param ASDIndex parameter not used
  *
  * @return the coded entry[]
  * @deprecated - use searchConcepts(String,String,int,String,boolean)
  */@Deprecated
 public CodedEntry[] searchConcepts(String scheme,
		 String searchTerm,
		 int limit,
		 String source,
						 boolean cui,
						 @SuppressWarnings("unused")boolean shortResult,
						 @SuppressWarnings("unused")boolean score,
						 @SuppressWarnings("unused")int ASDIndex) throws LBParameterException
    {
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (cui)
		{
//			CodedEntry concept  = findConceptByCode(searchTerm, ASDIndex);
			CodedEntry concept = getConcept(searchTerm);
			if (concept != null)
			{
				v.add(concept);
			}
		}
		else
		{
			if (source != null && source.compareTo("*") != 0) // search by source
			{
				/*
			    int matchOption = MATCH_BY_SOURCE_CODE;
			    String matchType = source;
			    return searchConcepts(
				              scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType,
				              ASDIndex);
				*/
//				return matchBySourceCode(scheme, searchTerm, limit, source, ASDIndex);
                try {
					return searchConcepts(scheme, searchTerm, limit, source);
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
		        }
			}
			else
			{
			    int matchOption = MATCH_BY_NAME;
			    String matchType = null;
			    try {
			    	return searchConcepts(
				              scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType);
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
		        }
 		    }
	    }
	    return toArray(v);
	}


	/**
* Search concepts.
*
* @param scheme the coding scheme
* @param searchTerm the search term
* @param limit the max number of results to return
* @param source the source
* @param cui the cui

*
* @return the coded entry[]
*/
public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
						 boolean cui) throws LBParameterException
 {
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (cui)
		{
//			CodedEntry concept  = findConceptByCode(searchTerm, ASDIndex);
			CodedEntry concept = getConcept(searchTerm);
			if (concept != null)
			{
				v.add(concept);
			}
		}
		else
		{
			if (source != null && source.compareTo("*") != 0) // search by source
			{
				try {
					return searchConcepts(scheme, searchTerm, limit, source);
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
				}
			}
			else
			{
			    int matchOption = MATCH_BY_NAME;
			    String matchType = null;
			    try {
			    	return searchConcepts(
				              scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType);
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
				}
		    }
	    }
	    return toArray(v);
	}


	/**
	 * Search concepts.
	 *
	 * @param scheme the coding scheme
	 * @param searchTerm the search term
	 * @param limit the max number of results to return
	 * @param source the source
	 * @param cui the cui
	 * @param shortResult - parameter not used
	 * @param score - parameter not used
	 *
	 * @return the coded entry[]
	 * @deprecated - use searchConcepts(String,String,int,String,boolean)
	 */@Deprecated
	public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
						 boolean cui,
						 @SuppressWarnings("unused")boolean shortResult,
						 @SuppressWarnings("unused")boolean score)
    {
	   try {
	       return searchConcepts(scheme, searchTerm, limit, source, cui);
	   } catch (Exception ex) {
	      ex.printStackTrace();
	      return null;
	   }
	}


    /**
     * Gets the available meta data urn's.
     *
     * @return the available meta data urn's
     */
    public Vector getAvailableMetaDataURNs()
    {
		Vector<String> v = new Vector<String>();
		try {
			AbsoluteCodingSchemeVersionReferenceList acsvrl = metadataService.listCodingSchemes();
			AbsoluteCodingSchemeVersionReference[] acdvra =	acsvrl.getAbsoluteCodingSchemeVersionReference();
			for (int i=0; i<acdvra.length; i++)
			{
				AbsoluteCodingSchemeVersionReference acsvr = acdvra[i];
				String urn = acsvr.getCodingSchemeURN();
//				String version = acsvr.getCodingSchemeVersion();
				v.add(urn);
			}
	    } catch (Exception e) {
			e.printStackTrace();

		}
		return v;
	}


	/**
	 * Gets the metadata properties.
	 *
	 * @param codingSchemeURN the coding scheme urn
	 *
	 * @return the metadata properties
	 */
	public NameAndValue[] getMetadataProperties(String codingSchemeURN)
	{
        String formalName = localName2FormalName(codingSchemeURN);
		Vector<NameAndValue> v = new Vector<NameAndValue>();
		try {
			LexBIGServiceMetadata svc = lbSvc.getServiceMetadata();

			AbsoluteCodingSchemeVersionReferenceList acsvrl = svc.listCodingSchemes();
			AbsoluteCodingSchemeVersionReference[] acdvra =	acsvrl.getAbsoluteCodingSchemeVersionReference();
			for (int i=0; i<acdvra.length; i++)
			{
				AbsoluteCodingSchemeVersionReference acsvr = acdvra[i];
				String urn = acsvr.getCodingSchemeURN();
				if (urn.equals(formalName))
				{
					//100807 KLO
					svc = svc.restrictToCodingScheme(acsvr);
					MetadataPropertyList mdpl = svc.resolve();
					MetadataProperty[] properties = mdpl.getMetadataProperty();
					for (int j=0; j<properties.length; j++)
					{
						MetadataProperty property = properties[j];
						NameAndValue nv = createNameAndValue(property.getName(), property.getValue());
						v.add(nv);
					}
				}
			}

			if (v.size() > 0)
			{
				NameAndValue[] nv_array = new NameAndValue[v.size()];
				for (int i=0; i<v.size(); i++)
				{
					NameAndValue nv = (NameAndValue) v.elementAt(i);
					nv_array[i] = nv;
			    }
			    return nv_array;
			}

	    } catch (Exception e) {
			e.printStackTrace();

		}
		return new NameAndValue[0];

	}


	/**
	 * Converts String to date.
	 *
	 * @param aString the a string
	 *
	 * @return the date
	 */
	public static Date stringToDate(String aString)
    {
		return stringToDate(aString, "MM/dd/yyyy");
	}

	/**
	 * Converts String to date.
	 *
	 * @param aString the a string
	 * @param format the format
	 *
	 * @return the date
	 */
	public static Date stringToDate(String aString, String format)
	{
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(format);
			ParsePosition pos = new ParsePosition(0);
 			Date theDate = sdf.parse(aString, pos);
 			return theDate;
 		}catch(Exception e){
 			e.printStackTrace();
 			return null;
 		}
	}

	/**
 * Find concepts with silo matching.
 * @param searchTerm parameter not used
 * @param limit parameter not used
 *
 * @return returns an empty vector
 *
 * @deprecated - evsapi4 does not support silos
 */@Deprecated
public Vector findConceptsWithSiloMatching(@SuppressWarnings("unused")String searchTerm,@SuppressWarnings("unused") int limit)
	{
		return new Vector();
	}

	//used
	/**
	 * Fetch term associations.
	 *
	 * @param code the code
	 *
	 * @return the vector of ConceptAssociation
	 */
	public Vector<ConceptAssociation> fetchTermAssociations(String code)
	{
		return getConceptAssociations(codingScheme, version, tag, code);
	}


    /**
     * Gets the related concept codes.
     *
     * @param code the code
     * @param associationName the association name
     * @param sourceAbbr the source abbr
     *
     * @return the String vector of related concept codes
     */
    public Vector<String> getRelatedConceptCodes(String code, String associationName, String sourceAbbr)
    {
		return getRelatedConceptCodes(code, null, associationName, sourceAbbr);
	}

    /**
     * Gets the related concept codes.
     *
     * @param code the code
     * @param direction the direction
     * @param associationName the association name
     * @param sourceAbbr the source abbr
     *
     * @return the related concept codes
     */
    public Vector<String> getRelatedConceptCodes(String code,  Boolean direction, String associationName, String sourceAbbr)
    {
		Vector<String> v = new Vector<String>();
		Vector<AssociatedConcept> v1 = new Vector<AssociatedConcept>();
		Vector<AssociatedConcept> v2 = new Vector<AssociatedConcept>();
		try {
			if (direction == null)
			{
				v1 = getSources(codingScheme, version, tag, code, associationName, sourceAbbr);
				v2 = getTargets(codingScheme, version, tag, code, associationName, sourceAbbr);
			}
			else if (direction.equals(Boolean.TRUE))
			{
				v2 = getTargets(codingScheme, version, tag, code, associationName, sourceAbbr);
			}
			else if (direction.equals(Boolean.FALSE))
			{
				v2 = getSources(codingScheme, version, tag, code, associationName, sourceAbbr);
			}

			for (int i=0; i<v1.size(); i++)
			{
				AssociatedConcept ac = (AssociatedConcept) v1.elementAt(i);
				v.add(ac.getConceptCode());
			}

			for (int i=0; i<v2.size(); i++)
			{
				AssociatedConcept ac = (AssociatedConcept) v2.elementAt(i);
				v.add(ac.getConceptCode());
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return v;
		}
		return v;
	}


//==========================================================================================================================================
// GF10442-10448, 10450: KLO, 121207
	 /**
     * Gets the parent concepts.
     *
     * @param code the concept code
     * @param source the source
     *
     * @return the parent concepts
     */
    public Vector<CodedEntry> getParentConcepts(String code, String source)
         {
    		 boolean direction = false;
    		 Vector<CodedEntry> v = getRelatedConcepts(code, direction, DEFAULT_ASSOC_FWD, source);
             if (v == null || v.size() == 0)
             {
    			 v = getRelatedConcepts(code, true, DEFAULT_ASSOC_REV, source);
    		 }
    		 return v;
    	 }

	 /**
 * Gets the parent concepts.
 *
 * @param code the concept code
 * @param source the source
 * @param ASD_Index parameter not used
 *
 * @return the parent concepts
 * @deprecated - use getParentConcepts(String,String)
 */@Deprecated
public Vector<CodedEntry> getParentConcepts(String code, String source, int ASD_Index)
     {
		 boolean direction = false;
		 Vector<CodedEntry> v = getRelatedConcepts(code, direction, DEFAULT_ASSOC_FWD, source, ASD_Index);
         if (v == null || v.size() == 0)
         {
			 v = getRelatedConcepts(code, true, DEFAULT_ASSOC_REV, source, ASD_Index);
		 }
		 return v;
	 }

 /**
	 * Gets the child concepts.
	 *
* @param code the concept code
* @param source the source
	 *
	 * @return the child concepts
	 */
	public Vector<CodedEntry> getChildConcepts(String code, String source)
  {
		 boolean direction = true;
		 Vector<CodedEntry> v = getRelatedConcepts(code, direction, DEFAULT_ASSOC_FWD, source);
      if (v == null || v.size() == 0)
      {
			 v = getRelatedConcepts(code, false, DEFAULT_ASSOC_REV, source);
		 }
		 return v;
	 }


	 /**
 	 * Gets the child concepts.
 	 *
 * @param code the concept code
 * @param source the source
 * @param ASD_Index parameter not used
 	 *
 	 * @return the child concepts
 	 * @deprecated - use getChildConcepts(String,String)
 	 */@Deprecated
 	public Vector<CodedEntry> getChildConcepts(String code, String source, int ASD_Index)
     {
		 boolean direction = true;
		 Vector<CodedEntry> v = getRelatedConcepts(code, direction, DEFAULT_ASSOC_FWD, source, ASD_Index);
         if (v == null || v.size() == 0)
         {
			 v = getRelatedConcepts(code, false, DEFAULT_ASSOC_REV, source, ASD_Index);
		 }
		 return v;
	 }
  	/**
  	 * Gets the related concepts.
  	 *
  	 * @param code the concept code
  	 * @param assocs the association
  	 * @param source the source
  	 *
  	 * @return the related concepts
  	 */
  	public Vector<CodedEntry> getRelatedConcepts(String code, String[] assocs, String source)
      {
 		 boolean direction = true;
 	     return getRelatedConcepts(code, direction, assocs, source);
 	 }

 	/**
 	 * Gets the related concepts.
 	 *
 	 * @param code the concept code
 	 * @param assocs the association
 	 * @param source the source
 	 * @param ASD_Index parameter not used
 	 *
 	 * @return the related concepts
 	 * @deprecated - use getRelatedConcepts(String,String[],String)
 	 */@Deprecated
 	public Vector<CodedEntry> getRelatedConcepts(String code, String[] assocs, String source, int ASD_Index)
     {
		 boolean direction = true;
	     return getRelatedConcepts(code, direction, assocs, source, ASD_Index);
	 }

	 /**
 	 * Gets the related concepts.
 	 *
 	 * @param code the concept code
 	 * @param direction the direction
 	 * @param assocs the associations
 	 * @param source the source
 	 * @param ASD_Index parameter not used
 	 *
 	 * @return the related concepts
 	 *
 	 * @deprecated - use getRelatedConcepts(String,boolean,String[],String)
 	 */@Deprecated
 	public Vector<CodedEntry> getRelatedConcepts(String code, boolean direction, String[] assocs, String source, @SuppressWarnings("unused")int ASD_Index)
	 {
		 CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
		 csvt.setVersion(this.version);
		 CodingScheme cs = getCodingScheme(this.codingScheme, this.version);

		 if (cs == null)
		 {
			 //println("Unable to find CodingScheme.");
			 return null;
		 }


         CodedEntry entry = getCodedEntryByCode(codingScheme, version, null, code);

		 //CodedEntry entry = getCodedEntryByCode(lbSvc, cs.getFormalName(), null, null, code);
		 if (entry == null)
		 {
			 //println("CodedEntry for " + code + " does not exist.\n");
			 return null;
		 }
//		 else
//		 {
//			 String name = entry.getEntityDescription().getContent();
//			 //println(name + " (" + code + ")\n");
//		 }

		 boolean navigateForward = direction;
		 boolean forward = NavigationUtils.getNavigationDirection(lbSvc, cs.getFormalName(), null, source);

		 if (forward)
		 {
			 navigateForward = !direction;
		 }

		 HashMap asso_map = null;
		 try {
			 asso_map = NavigationUtils.getAssociatedNodes(
				lbSvc,
				cs.getFormalName(),
				source,
				csvt,
				code,
				assocs,
				!navigateForward);

		 } catch (Exception ex) {
			 ex.printStackTrace();
			 return new Vector<CodedEntry>();
		 }

		 Set keyset = asso_map.keySet();

		 //println("Number of relations: " + keyset.size());

         Vector<CodedEntry> v = new Vector<CodedEntry>();
		 Iterator it = keyset.iterator();
		 while (it.hasNext()) {
			 String rel = (String) it.next();
			 //println("Relation: " + rel);
             ResolvedConceptReferenceList rcrl = (ResolvedConceptReferenceList) asso_map.get(rel);

			 if (rcrl.getResolvedConceptReferenceCount() > 0)
			 {
				 try {
					Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference();
					while (nodes.hasNext())
					{
						ResolvedConceptReference ref = nodes.next();

						String parentcode = ref.getConceptCode();

						entry = ref.getReferencedEntry();
						if (entry == null)
						{
							//entry = getCodedEntryByCode(lbSvc, cs.getFormalName(), null, null, parentcode);
							entry = getCodedEntryByCode(codingScheme, version, null, parentcode);

//							String name = entry.getEntityDescription().getContent();
							//println("\t" + name + " (" + parentcode + ")");
							v.add(entry);
						}
					}
				 } catch (Exception ex) {

				 }
			 }
		 }
         return v;
     }


	 /**
	 * Gets the related concepts.
	 *
	 * @param code the concept code
	 * @param direction the direction
	 * @param assocs the associations
	 * @param source the source
	 *
	 * @return the related concepts
	 */
	public Vector<CodedEntry> getRelatedConcepts(String code, boolean direction, String[] assocs, String source)
	 {
		 CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
		 csvt.setVersion(this.version);
		 CodingScheme cs = getCodingScheme(this.codingScheme, this.version);

		 if (cs == null)
		 {
			 //println("Unable to find CodingScheme.");
			 return null;
		 }


        CodedEntry entry = getCodedEntryByCode(codingScheme, version, null, code);

		 //CodedEntry entry = getCodedEntryByCode(lbSvc, cs.getFormalName(), null, null, code);
		 if (entry == null)
		 {
			 //println("CodedEntry for " + code + " does not exist.\n");
			 return null;
		 }
//		 else
//		 {
//			 String name = entry.getEntityDescription().getContent();
//			 //println(name + " (" + code + ")\n");
//		 }

		 boolean navigateForward = direction;
		 boolean forward = NavigationUtils.getNavigationDirection(lbSvc, cs.getFormalName(), null, source);

		 if (forward)
		 {
			 navigateForward = !direction;
		 }

		 HashMap asso_map = null;
		 try {
			 asso_map = NavigationUtils.getAssociatedNodes(
				lbSvc,
				cs.getFormalName(),
				source,
				csvt,
				code,
				assocs,
				!navigateForward);

		 } catch (Exception ex) {
			 ex.printStackTrace();
			 return new Vector<CodedEntry>();
		 }

		 Set keyset = asso_map.keySet();

		 //println("Number of relations: " + keyset.size());

        Vector<CodedEntry> v = new Vector<CodedEntry>();
		 Iterator it = keyset.iterator();
		 while (it.hasNext()) {
			 String rel = (String) it.next();
			 //println("Relation: " + rel);
            ResolvedConceptReferenceList rcrl = (ResolvedConceptReferenceList) asso_map.get(rel);

			 if (rcrl.getResolvedConceptReferenceCount() > 0)
			 {
				 try {
					Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference();
					while (nodes.hasNext())
					{
						ResolvedConceptReference ref = nodes.next();

						String parentcode = ref.getConceptCode();

						entry = ref.getReferencedEntry();
						if (entry == null)
						{
							//entry = getCodedEntryByCode(lbSvc, cs.getFormalName(), null, null, parentcode);
							entry = getCodedEntryByCode(codingScheme, version, null, parentcode);

//							String name = entry.getEntityDescription().getContent();
							//println("\t" + name + " (" + parentcode + ")");
							v.add(entry);
						}
					}
				 } catch (Exception ex) {

				 }
			 }
		 }
        return v;
    }


	 /**
 	 * Gets the related concepts.
 	 *
 	 * @param code the concept code
 	 * @param associationName the association name
 	 * @param source the source
 	 * @param ASD_Index the AttributeSetDescriptor
 	 *
 	 * @return the related concepts
 	 */
 	public Vector<CodedEntry> getRelatedConcepts(String code, String associationName, String source, int ASD_Index)
	 {
		 return getRelatedConcepts(code, true, associationName, source, ASD_Index);
	 }

	 /**
 	 * Gets the related concepts.
 	 *
 	 * @param code the concept code
 	 * @param direction the direction
 	 * @param associationName the association name
 	 * @param source the source
 	 * @param ASD_Index the AttributeSetDescriptor
 	 *
 	 * @return the related concepts
 	 */
 	public Vector<CodedEntry> getRelatedConcepts(String code, boolean direction, String associationName, String source, int ASD_Index)
	 {
		 String[] assocs = null;
		 if (associationName != null)
		 {
			 assocs = new String[1];// getStringArray(associations);
			 assocs[0] = associationName;
		 }
		 return getRelatedConcepts(code, direction, assocs, source, ASD_Index);
	 }

//==========================================================================================================================================

    // GF13130, 13212, 13213
    public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source, boolean cui, String matchAlgorithm)
           throws LBParameterException
    {
		if (cui)
		{
			CodedEntry concept = getConcept(searchTerm);
			if (concept != null)
			{
				CodedEntry[] concepts = new CodedEntry[1];
				concepts[0] = concept;
				return concepts;
			}
		}
		else
		{
			if (source != null && source.compareTo("*") != 0) // search by source
			{
				// match by code
				int matchOption = MATCH_BY_SOURCE_CODE;
				String matchType = source;
				CodedEntry[] concepts = null;
				try {
					concepts = searchConcepts(
							  scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType,
							  matchAlgorithm,
							  null); // restricted_property_vec
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
				}

				HashSet<String> hset = new HashSet<String>();
				// match by name
				Vector<String> source_vec = new Vector<String>();
				source_vec.add(source);
				Vector<CodedEntry> v = null;
				try {
				   v = findCodedEntriesWithDesignationMatching(
							  scheme,
							  null,
							  searchTerm,
							  source_vec, // source_vec
							  matchAlgorithm,
							  limit);
				} catch (Exception ex) {
					throw new LBParameterException(ex.getMessage());
				}

				if (v != null)
				{
					for (int j=0; j<v.size(); j++)
					{
						CodedEntry ce = (CodedEntry) v.elementAt(j);
						hset.add(ce.getConceptCode());
					}
				}
				if (concepts != null)
				{
					for (int i=0; i<concepts.length; i++)
					{
						CodedEntry ce = concepts[i];
						if (!hset.contains(ce.getConceptCode()))
						{
							hset.add(ce.getConceptCode());
							v.add(ce);
						}
					}
				}

				CodedEntry[] entries = new CodedEntry[v.size()];
				for (int i=0; i<v.size(); i++)
				{
					entries[i] = (CodedEntry) v.elementAt(i);
				}
				return entries;
			}
			else // not by source code
			{
			    int matchOption = MATCH_BY_NAME;
			    String matchType = null;
				CodedEntry[] concepts = searchConcepts(
						 scheme,
						 searchTerm,
						 limit,
						 matchOption,
						 matchType,
						 matchAlgorithm,
						 null); // restricted_property_vec

			    return concepts;
		    }
	    }
	    //return toArray(v);
	    return new CodedEntry[0];
	}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// getTree overloading
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	/**
	 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
	 *
	 * @param rootcode - the concept code to use as the root of the tree.
	 * @param direction - true for down, false for up
	 * @param asdIndex paramter not used
	 *
	 * @return the root node byte array of a tree containing a collection of
	 * children
	 *
	 * @deprecated - use getTree(String,boolean)
	 */@Deprecated
	public DefaultMutableTreeNode getTree(String rootcode,
					boolean direction,
					@SuppressWarnings("unused")int asdIndex)
	{

		return getTree(rootcode,
					direction,
					-1,
					asdIndex,
					true,
					new Vector());
	}


	/**
	 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
	 * The tree contains the child concepts to the level supplied.
	 *
	 * @param rootcode the rootcode
	 * @param direction true for down, false for up
	 *
	 * @return the tree
	 *
	 */

	public DefaultMutableTreeNode getTree(
		            String rootcode,
					boolean direction)
	{
		return getTree(rootcode,
					direction,
					-1,
					0,
					true,
					new Vector());
	}


	/**
	 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
	 * The tree contains the child concepts to the level supplied.
	 *
	 * @param rootcode the rootcode
	 * @param direction true for down, false for up
	 * @param level the depth of the tree
	 * @param asdIndex parameter not used
	 *
	 * @return the tree
	 * @deprecated - use getTree(String,boolean)
	 */
	public DefaultMutableTreeNode getTree(String rootcode,
					boolean direction,
					int level,
					@SuppressWarnings("unused") int asdIndex)
	{
		return getTree(rootcode,
					direction,
					level,
					asdIndex,
					true,
					new Vector());
	}



 /**
 * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
 * The tree contains the child concepts to the level supplied.
 *
 * @param rootcode the concept code for the root node of the tree
 * @param direction true for down, false for up
 * @param isaflag true for use is-a relationship, false not to
 * @param rolenames the rolenames to build the tree from
 *
 * @return the DefaultMutableTreeNode
 */
	public DefaultMutableTreeNode getTree(String rootcode,
	                      boolean direction,
		                  boolean isaflag,
		                  Vector rolenames)
	{
		  return getTree(rootcode,
	                      direction,
	                      -1,
	                      0,
		                  isaflag,
		                  rolenames);
	}



  /**
  * Returns a DefaultMutableTreeNode based on the concept provided and limits the number of levels of child concepts returned.
  * The tree contains the child concepts to the level supplied.
  *
  * @param rootcode the rootcode
  * @param direction true for down, false for up
  * @param isaName the name of the is-a relation
  * @param rolenames the rolenames to build the tree from
  *
  * @return the tree
  */
    public DefaultMutableTreeNode getTree(String rootcode,
 	                      boolean direction,
 		                  String isaName,
 		                  Vector rolenames)
 	{
	               return getTree(
		                  rootcode,
	                      direction,
	                      -1,
	                      0,
		                  isaName,
		                  rolenames);
    }

}

