/*
 *
 */
package gov.nih.nci.lexbig.ext;

import gov.nih.nci.system.applicationservice.EVSApplicationService;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import org.LexGrid.LexBIG.DataModel.Collections.AssociatedConceptList;
import org.LexGrid.LexBIG.DataModel.Collections.AssociationList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeTagList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeVersionList;
import org.LexGrid.LexBIG.DataModel.Collections.ConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.ExtensionDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.LocalNameList;
import org.LexGrid.LexBIG.DataModel.Collections.ModuleDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.NCIChangeEventList;
import org.LexGrid.LexBIG.DataModel.Collections.NameAndValueList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.SortOptionList;
import org.LexGrid.LexBIG.DataModel.Collections.SystemReleaseList;
import org.LexGrid.LexBIG.DataModel.Core.AssociatedConcept;
import org.LexGrid.LexBIG.DataModel.Core.Association;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeSummary;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ConceptReference;
import org.LexGrid.LexBIG.DataModel.Core.NameAndValue;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.Core.types.CodingSchemeVersionStatus;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.CodingSchemeRendering;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ExtensionDescription;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ModuleDescription;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.RenderingDetail;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.SortOption;
import org.LexGrid.LexBIG.DataModel.NCIHistory.NCIChangeEvent;
import org.LexGrid.LexBIG.DataModel.NCIHistory.types.ChangeType;
import org.LexGrid.LexBIG.Exceptions.LBParameterException;
import org.LexGrid.LexBIG.History.HistoryService;
import org.LexGrid.LexBIG.Impl.LexBIGServiceImpl;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeGraph;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.SearchDesignationOption;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.LexBIG.Utility.ConvenienceMethods;
import org.LexGrid.LexBIG.Utility.Iterators.ResolvedConceptReferencesIterator;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.codingSchemes.CodingSchemeVersion;
import org.LexGrid.codingSchemes.Versions;
import org.LexGrid.commonTypes.PropertyQualifier;
import org.LexGrid.commonTypes.Source;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Comment;
import org.LexGrid.concepts.Definition;
import org.LexGrid.concepts.Instruction;
import org.LexGrid.concepts.Presentation;
import org.LexGrid.naming.SupportedAssociation;
import org.LexGrid.naming.SupportedAssociationQualifier;
import org.LexGrid.naming.SupportedConceptStatus;
import org.LexGrid.naming.SupportedContext;
import org.LexGrid.naming.SupportedProperty;
import org.LexGrid.naming.SupportedPropertyQualifier;
import org.LexGrid.naming.SupportedRepresentationalForm;
import org.LexGrid.naming.SupportedSource;
import org.LexGrid.relations.Relations;
import org.LexGrid.versions.SystemRelease;
import org.apache.log4j.Category;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import org.LexGrid.commonTypes.EntityDescription;
import org.LexGrid.LexBIG.Exceptions.LBException;

import org.LexGrid.LexBIG.Impl.Extensions.GenericExtensions.LexBIGServiceConvenienceMethodsImpl;
import org.LexGrid.LexBIG.Impl.dataAccess.ResourceManager;

/**
 * DLBWrapper
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */
public class DLBWrapper {

// to be activated later
	//public EVSApplicationService lbSvc = null;

    protected org.LexGrid.LexBIG.LexBIGService.LexBIGService lbSvc = null;
	/** an instance of org.LexGrid.LexBIG.Utility.ConvenienceMethods
	 */
 	public org.LexGrid.LexBIG.Utility.ConvenienceMethods lbConvMethods = null;

 	//LexBIGCM lbConvMethods = null;

    /** an instance of CodingSchemeRenderingList
     */
    public CodingSchemeRenderingList csrl = null;

    /** A vector containing the name of all supported coding schemes.
     */
    protected Vector<CodingScheme> supportedCodingSchemes = null;


	// NCI Unique name for concept code
	//static final String NC_NAME = "Display_Name";
	/** A static string "KIND".
	 */
	static final String KIND = "KIND";

	/** A static string "CONCEPT_NAME".
	 */
	static final String CONCEPT_NAME = "CONCEPT_NAME";

	/** A constant String "Preferred_Name".
	 */
	static final String PREFERRED_NAME = "Preferred_Name";

	/** A constant String "hasSubtype".
	 */
	static final String IS_A = "hasSubtype";

	/** A constant String "PAR".
	 */
	static final String PAR = "PAR";


	/** A constant String "is_a".
	 */
	static final String IS_A_OBO = "is_a";

	/** A constant String "is_a".
	 */
	static final String HAS_SUBCLASS = "hasSubclass";


	/** A constant String "NONE".
	 */
	static final String NONE = "NONE";

	/** A constant String "exactMatch".
	 */
	static final String EXACT_MATCH = "exactMatch";

    /** A constant equaling 1.
     */
    public static final int RESTRICT_TO_TARGET = 1;
	/** A constant equaling 2.
	 */
	public static final int RESTRICT_TO_SOURCE = 2;

    /** A constant String with a value "owl:AnnotationProperty"
     */
    public static String ROOT_ASSOCIATION = "owl:AnnotationProperty";
    /** A constant String with a value "owl:ObjectProperty"
     */
    public static String ROOT_ROLE = "owl:ObjectProperty";

    /** A constant String with a value "associations"
     */
    public static String ASSOCIATION_RELATION = "associations";
    /** A constant String with a value "roles"
     */
    public static String ROLE_RELATION = "roles";


    //static final String SEMANTIC_NET = "SemanticNet";

    /** A constant String with a value "urn:lsid:nlm.nih.gov:semnet"
     */
    static final String SEMANTIC_NET = "urn:lsid:nlm.nih.gov:semnet";

    /** A static vector containing all available semantic types.
     */
    static Vector<String> semanticType_vec;
    /** A static hashmap mapping semantic type to code.
     */
    static HashMap<String,String> semanticType_hashmap;
    static HashMap<String,String> semanticTypeName2Code_hashmap;

    /** A static variable.
     */
    static Category cat;

    // property type:
    /** A static String with a value "definition"
     */
    public static String TYPE_DEFINITION = "definition";
    /** A static String with a value "comment"
     */
    public static String TYPE_COMMENT = "comment";
    /** A static String with a value "property"
     */
    public static String TYPE_PROPERTY = "property";
    /** A static String with a value "instruction"
     */
    public static String TYPE_INSTRUCTION = "instruction";
    /** A static String with a value "presentation"
     */
    public static String TYPE_PRESENTATION = "presentation";


//    private boolean isRemote = false;

    protected String version = null;
    protected String tag = null;

	/** active coding scheme name
	 */
	protected String codingScheme;// = NSID2codingSchemeName(namespaceId);

    protected CodingScheme defaultCodingScheme = null;

	/** The Constant BROADENING_HIER_ASSOC. {"CHD", "hasSubtype"}*/
	final static String[] BROADENING_HIER_ASSOC = new String[] {"CHD", "hasSubtype"};

	/** The Constant NARROWING_HIER_ASSOC. {"PAR", "isa", "branch_of", "part_of", "tributary_of"}*/
	final static String[] NARROWING_HIER_ASSOC = new String[] {"PAR", "isa", "is_a", "branch_of", "part_of", "tributary_of"};

	/** The Constant HIER_ASSOC. {"hasSubtype", "hasSubclass", "is_a", "isa", "PAR", "CHD", "branch_of", "part_of", "tributary_of"}*/
	final static String[] HIER_ASSOC = new String[] {"hasSubtype", "hasSubclass", "is_a", "isa", "PAR", "CHD", "branch_of", "part_of", "tributary_of"};

	/** An instance of LocalNameList.
	 */
	static LocalNameList noPropertiesList = new LocalNameList();
	{
		// temporarily commented out
		//ncNameList.addEntry(NONE);
	}

	/** A static LocalNameList.
	 */
	static LocalNameList kindList = new LocalNameList();
	{
		//ncNameList.addEntry(KIND);
		kindList.addEntry(KIND);
	}

    LexBIGServiceMetadata metadataService = null;

///////////////////////
//  Constructors
///////////////////////

	/** Instantiates a new DLBWrapper.
	 *
	 * @since DLBExt 1.0
	 */
    // Local
	public DLBWrapper()
    {
		try {
			this.lbSvc = new LexBIGServiceImpl();
			lbConvMethods = new ConvenienceMethods(this.lbSvc);
			metadataService = this.lbSvc.getServiceMetadata();
			csrl = this.lbSvc.getSupportedCodingSchemes();
			supportedCodingSchemes = getSupportedCodingSchemes();

			this.codingScheme = null;
			this.version = null;
			this.tag = null;

		} catch (Exception e) {
			e.printStackTrace();
		}
    }


    // Remote
    /**
     * Instantiates a new DLBWrapper.
     *
     * @param lbSvc the LexBIGService
     */
    public DLBWrapper(EVSApplicationService lbSvc)
    {
		this.lbSvc = lbSvc;

//		isRemote = true;

		try {
			lbConvMethods = new ConvenienceMethods(this.lbSvc);
			metadataService = this.lbSvc.getServiceMetadata();

			this.codingScheme = null;
			this.version = null;
			this.tag = null;

			csrl = this.lbSvc.getSupportedCodingSchemes();
			supportedCodingSchemes = getSupportedCodingSchemes();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }



    /**
     * Gets the logger.
     *
     * @return the logger
     */
    public Logger getLogger()
    {
//		if (cat == null)
//		{
			// 012607 add appender programmatically
			Logger logger = Logger.getLogger(this.getClass());
			PatternLayout layout = new PatternLayout("%d [%t] %-5p %c - %m%n");

			FileAppender appender = null;
			try {
				appender = new FileAppender(layout,"c:/logfile",true);
			}catch(Exception e){}

			Logger.getRootLogger().addAppender(appender);
			logger.setLevel(Level.WARN);

//			Logger.getLogger("DLBWrapper");
//			cat = Category.getInstance("DLBWrapper");
//		}
		return logger;
	}


///////////////////////
//  Utility
///////////////////////

	/** Converts an array of objects to a vector of objects
	 *
	 * @since DLBExt 1.0
	 * @param objects an array of objects
	 * @return a vector of objects; return null, if connection fails or an
	 *     exception is thrown
	 */@SuppressWarnings("unchecked")
	protected Vector arrayToVector(Object[] objects)
	{
		Vector v = new Vector();
		if (objects == null)
			return v;

		for (int i=0; i<objects.length; i++)
			v.add(objects[i]);
		return v;
	}

	/** Converts a HashSet to a Vector
	 *
	 * @param hset a HashSet instance
	 * @return a Vector instance
	 */
	protected Vector<String> hashSet2Vector(HashSet hset)
	{
	    Vector<String> v = new Vector<String>();
	    Iterator it = hset.iterator();
	    while (it.hasNext())
		{
		    v.add((String) it.next());
		}
		return v;
	}

	/** Converts a LocalNameList to a Vector
	 *
	 * @since DLBExt 1.0
	 * @param v a Vector
	 * @return a LocalNameList
	 */
	public LocalNameList vector2LocalNameList(Vector v)
	{
	    if (v == null) return null;
	    LocalNameList list = new LocalNameList();
		for (int i=0; i<v.size(); i++)
		{
		    String vEntry = (String) v.elementAt(i);
	        list.addEntry(vEntry);
		}
        return list;
	}


	/** Converts a LocalNameList to a HashSet
	 *
	 * @since DLBExt 1.0
	 * @param list an instance of LocalNameList
	 * @return a LocalNameList
	 */
	public HashSet<String> LocalNameList2HashSet(LocalNameList list)
	{
	    if (list == null) return null;
	    HashSet<String> hset = new HashSet<String>();
		java.lang.String[] entries = list.getEntry();


		for (int i=0; i<entries.length; i++)
		{
		    String vEntry = (String) entries[i];
	        if (!hset.contains(vEntry))
			{
                hset.add(vEntry);
			}
		}
        return hset;
	}





	/**
	 * Creates a ConceptReference instance corresponding to a given coding scheme name and a concept code.
	 *
	 * @param code the concept code
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return a ConceptReference instance
	 *
	 * @since DLBExt 1.0
	 */
    public static ConceptReference createConceptReference(String code, String codingSchemeName)
    {
        ConceptReference cr = new ConceptReference();
        cr.setCodingScheme(codingSchemeName);
        cr.setConceptCode(code);
        return cr;
    }


	/** Creates a CodingSchemeVersionOrTag instance for a given version and a tag
	 *
	 * @since DLBExt 1.0
	 * @param vers coding scheme version
	 * @param ltag coding scheme tag
	 *
	 * @return a CodingSchemeVersionOrTag instance

	 */
	public CodingSchemeVersionOrTag createCodingSchemeVersionOrTag(
	                                     String vers, String ltag)
    {
        CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
        if (vers != null && vers.compareTo("") != 0) versionOrTag.setVersion(vers);
		if (ltag != null && ltag.compareTo("") != 0) versionOrTag.setTag(ltag);
		return versionOrTag;
    }


	/** Creates a NameAndValue instance for a given name and value pair.
	 *
	 * @since DLBExt 1.0
	 * @param name a name
	 * @param value a value
	 * @return a NameAndValue instance
	 */
	public NameAndValue createNameAndValue(String name, String value)
	{
        NameAndValue nv = new NameAndValue();
		nv.setName(name);
		nv.setContent(value);
		return nv;
	}




///////////////////////
//  Meta Data
///////////////////////

    /*
     * Discover Vocabularies (and Versions)
     */

    /** Gets all supported coding schemes
     *
     * @since DLBExt 1.0
     * @return a Vector containing all supported CodingScheme instances
     */
    public Vector<CodingScheme> getSupportedCodingSchemes()
	{
		if (supportedCodingSchemes != null)
		{
		    return supportedCodingSchemes;
		}

		Vector<CodingScheme> v = new Vector<CodingScheme>();

        try {
			if (lbSvc == null)
			{
			   getLogger().error("ERROR: lbSvc is null.");
			   return null;
			}
			CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = lcsrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalname = css.getFormalName();
//				String localname = css.getLocalName();
				String representsVersion = css.getRepresentsVersion();
				CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
				vt.setVersion(representsVersion);
				CodingScheme scheme = lbSvc.resolveCodingScheme(formalname, vt);
				v.add(scheme);

			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;
	}


	/** Gets all supported coding scheme names
	 *
	 * @since DLBExt 1.0
	 * @return a Vector containing all supported coding scheme names
	 */
	public Vector<String> getSupportedCodingSchemeNames()
	{
		Vector<String> v = new Vector<String>();
        CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
		for (int i=0; i<csrs.length; i++)
		{
		    CodingSchemeRendering csr = csrs[i];
		    CodingSchemeSummary css = csr.getCodingSchemeSummary();
			v.add(css.getFormalName());
		}
		return v;
	}

    /** Gets CodingScheme instance for a given coding scheme name.
     *
     * @since DLBExt 1.0
     * @param codingSchemeName a coding scheme name
     *
     * @return a CodingScheme instance
     */
    public CodingScheme getCodingScheme(String codingSchemeName)
	{
		/*
		if (!Name2CodingSchemeMap.containsKey(codingScheme)) return null;
		return (CodingScheme) Name2CodingSchemeMap.get(codingScheme);
		*/
		return getCodingScheme(codingSchemeName, null, null);
	}


    /** Gets CodingScheme instance for a given coding scheme name and version.
     *
     * @since DLBExt 1.0
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     *
     * @return a CodingScheme instance
     */
    public CodingScheme getCodingScheme(String codingSchemeName, String vers)
	{
		/*
		if (!Name2CodingSchemeMap.containsKey(codingScheme)) return null;
		return (CodingScheme) Name2CodingSchemeMap.get(codingScheme);
		*/
		return getCodingScheme(codingSchemeName, vers, null);
	}


    /** Gets CodingScheme instance for given coding scheme name, version, and tag
     *
     * @since DLBExt 1.0
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     *
     * @return a CodingScheme instance
     */
    public CodingScheme getCodingScheme(String codingSchemeName, String vers, String ltag)
	{
		CodingSchemeVersionOrTag vt = null;
		if (vers != null || ltag != null)
		{
			vt = createCodingSchemeVersionOrTag(vers, ltag);
		}
		CodingScheme scheme = null;
		try {
			scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return scheme;
	}


    /** Gets all supported search techniques
     *
     * @since DLBExt 1.0
     * @return a String Vector containing all supported search techniques
     */
    public Vector<String> getSupportedSearchTechniques()
	{
		Vector<String> v = new Vector<String>();
		try {

			//ExtensionDescriptionList list = lbSvc.getMatchAlgorithms();
			ModuleDescriptionList list = lbSvc.getMatchAlgorithms();

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSupportedSearchTechnique lbSvc.getIndexExtensions returns null.");
			    return v;
			}

			//ExtensionDescription[] eda = list.getExtensionDescription();
			ModuleDescription[] eda = list.getModuleDescription();
			if (eda == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSupportedSearchTechnique list.getIndexgetExtensionDescription returns null.");
			    return v;
			}

			for (int i=0; i<eda.length; i++)
			{
				//ExtensionDescription ed = (ExtensionDescription) eda[i];
				ModuleDescription ed = (ModuleDescription) eda[i];

				//v.add(ed.getExtensionName());
				v.add(ed.getName());
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;
    }

    /** Gets the description of search technique for a given coding scheme name.
     *
     * @since DLBExt 1.0
     * @param name search technique name
     * @return the description of search technique
     */
    public String getSearchTechniqueDescription(String name)
	{
		try {
			//ExtensionDescriptionList list = lbSvc.getMatchAlgorithms(); //lbSvc.getIndexExtensions();
			ModuleDescriptionList list = lbSvc.getMatchAlgorithms();

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSearchTechniqueDescription lbSvc.getMatchAlgorithms returns null.");
			    return null;
			}

			//ExtensionDescription[] eda = list.getExtensionDescription();
			ModuleDescription[] eda = list.getModuleDescription();
			if (eda == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSearchTechniqueDescription list.getIndexgetExtensionDescription returns null.");
			    return null;
			}

			for (int i=0; i<eda.length; i++)
			{
				//ExtensionDescription ed = (ExtensionDescription) eda[i];
				ModuleDescription ed = (ModuleDescription) eda[i];
				//if (ed.getExtensionName().compareTo(name) == 0)
				if (ed.getName().compareTo(name) == 0)
				{
					//String desc = ed.getExtensionDescription();
					String desc = ed.getDescription();
					return desc;
				}
			}
			getLogger().error("Algorithm " + name + " not found.");
	    } catch (Exception e) {
			e.printStackTrace();
		}

		return null;
    }


    /*
     * Discover Vocabulary Versions
     */

	/** Gets available coding scheme versions for a given CodingScheme
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName
	 * @return a Vector containing all available coding scheme versions
	 */
 	public Vector<String> getAvailableCodingSchemeVersions(String codingSchemeName) //CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
		try {
			CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = lcsrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					v.add(css.getRepresentsVersion());
				}
			}
	    } catch (Exception e) {
			return new Vector<String>();
		}
		return v;
	}


	/** Gets copy right statement for a given CodingScheme
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return copy right statement
	 */
	public String getCopyRight(CodingScheme cs)
	{
		 if (cs == null) return null;
		 return cs.getCopyright().getContent();
	}


	/** Gets copy right statement for a given coding scheme name
	 *
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 *
	 * @return copy right statement
	 */
	public String getCopyRight(String codingSchemeName)
	{
         CodingScheme cs = getCodingScheme(codingSchemeName);
		 if (cs == null)
		 {
			 return null;
		 }

		 if (cs.getCopyright() == null)
		 {
			getLogger().warn("cs.getCopyright() returns null" );
		 }

		 return cs.getCopyright().getContent();
	}

	/** Gets sources of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return a Vector containing sources
	 */@SuppressWarnings("unchecked")
	public Vector<Source> getSources(CodingScheme cs)
	{
		 if (cs == null) return null;
		 return  arrayToVector(cs.getSource());
    }


	/** Gets sources of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 * @return a Vector containing sources
	 */@SuppressWarnings("unchecked")
	public Vector<Source> getSources(String codingSchemeName)
	{
         CodingScheme cs = getCodingScheme(codingSchemeName);
		 if (cs == null) return null;
		 return arrayToVector(cs.getSource());
	}

	/** Gets versions of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return versions of a given CodingScheme
	 */
	public Vector<String> getVersions(CodingScheme cs)
    {
		 if (cs == null) return null;
		 Vector<String> v = new Vector<String>();
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  v.add(csv.getVersion());
		 }
		 return v;
	}

	/** Gets versions of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 * @return versions of a given CodingScheme
	 */
	public Vector<String> getVersions(String codingSchemeName)
    {
         CodingScheme cs = getCodingScheme(codingSchemeName);
		 if (cs == null) return null;
		 Vector<String> v = new Vector<String>();
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  v.add(csv.getVersion());
		 }
		 return v;
	}


	/** Gets the instance of CodingSchemeVersion given an instance of CodingScheme and a version number.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @param vers a coding scheme version
	 * @return the instance of CodingSchemeVersion given an instance of
	 *     CodingScheme and a version number
	 */
	protected CodingSchemeVersion getCodingSchemeVersion(CodingScheme cs, String vers)
    {
		 if (cs == null) return null;

		 // this method fails:
         Versions versions = cs.getVersions();

		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  if (csv.getVersion().compareToIgnoreCase(vers) == 0)
			  {
			      return csv;
			  }
		 }
		 return null;
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 */
	protected CodingSchemeVersion getCodingSchemeVersion(String codingSchemeName, String vers)
    {
         CodingScheme cs = getCodingScheme(codingSchemeName);
		 if (cs == null) return null;
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  if (csv.getVersion().compareToIgnoreCase(vers) == 0)
			  {
			      return csv;
			  }
		 }
		 return null;
	}

	/**
	 * Gets the version date.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 *
	 * @return the version date
	 *
	 * @since DLBExt 1.0
	 */
	public java.util.Date getVersionDate(String codingSchemeName, String vers)
    {
         CodingSchemeVersion csv = getCodingSchemeVersion(codingSchemeName, vers);
		 if (csv == null)  return null;
		 return csv.getVersionDate();
	}

	/**
	 * Checks if is version complete.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 *
	 * @return true, if version complete
	 *
	 * @since DLBExt 1.0
	 */
	public Boolean isVersionComplete(String codingSchemeName, String vers)
    {
         CodingSchemeVersion csv = getCodingSchemeVersion(codingSchemeName, vers);
		 return csv.getIsComplete();
	}




	/**
	 * Gets the vocabulary version by tag.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param ltag the tag
	 *
	 * @return the vocabulary version by tag
	 */
	public String getVocabularyVersionByTag(String codingSchemeName, String ltag)
	{
		 if (codingSchemeName == null) return null;
		 try {
			 CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			 CodingSchemeRendering[] csra = lcsrl.getCodingSchemeRendering();
			 for (int i=0; i<csra.length; i++)
			 {
				CodingSchemeRendering csr = csra[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				if (css.getFormalName().compareTo(codingSchemeName) == 0 || css.getLocalName().compareTo(codingSchemeName) == 0)
				{
					if (ltag == null) return css.getRepresentsVersion();
					RenderingDetail rd = csr.getRenderingDetail();
					CodingSchemeTagList cstl = rd.getVersionTags();
					java.lang.String[] tags = cstl.getTag();
					for (int j=0; j<tags.length; j++)
					{
						String version_tag = (String) tags[j];
						if (version_tag.compareToIgnoreCase(ltag) == 0)
						{
							return css.getRepresentsVersion();
						}
					}
				}
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 getLogger().warn("Version corresponding to tag " + ltag + " is not found " + " in " + codingSchemeName);
		 return null;
	 }

    /**
     * Gets the supported properties.
     *
     * @param cs the CodingScheme
     *
     * @return the supported properties
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedProperty> getSupportedProperties(CodingScheme cs)
	{
		if (cs == null) return null;
        Vector<SupportedProperty> v = new Vector<SupportedProperty>();
	    SupportedProperty[] properties = cs.getMappings().getSupportedProperty();
		for (int i=0; i<properties.length; i++)
		{
		     SupportedProperty sp = (SupportedProperty) properties[i];
		     v.add(sp);
		}
        return v;
	}


    /**
     * Gets the supported sources.
     *
     * @param cs the CodingScheme
     *
     * @return the supported sources
     */
    public Vector<SupportedSource> getSupportedSources(CodingScheme cs)
	{
		if (cs == null) return null;
        Vector<SupportedSource> v = new Vector<SupportedSource>();
	    SupportedSource[] sources = cs.getMappings().getSupportedSource();


		for (int i=0; i<sources.length; i++)
		{
		     SupportedSource src = (SupportedSource) sources[i];
		     v.add(src);
		}
        return v;
	}



    /**
     * Gets the supported properties.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param ltag the tag
     *
     * @return the supported properties
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedProperty> getSupportedProperties(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedProperties(cs);
	}


    /**
     * Gets the supported property names.
     *
     * @param cs the cs
     *
     * @return the supported property names
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedPropertyNames(CodingScheme cs)
	{
        Vector w = getSupportedProperties(cs);
		if (w == null) return null;

        Vector<String> v = new Vector<String>();
		for (int i=0; i<w.size(); i++)
		{
		     SupportedProperty sp = (SupportedProperty) w.elementAt(i);

		     if (sp == null)
		     {
				 getLogger().warn("DLBWrapper getSupportedPropertyNames SupportedProperty == null");
			 }
			 else if (sp.getLocalId() == null)
			 {
			     getLogger().warn("DLBWrapper getSupportedPropertyNames SupportedProperty.getLocalId() == null");
			 }


		     //v.add(sp.getContent());
		     v.add(sp.getLocalId());
		}
        return v;
	}


    /**
     * Gets the supported property names.
     *
     * @return the supported property names
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedPropertyNames()
	{
		Vector<String> v = new Vector<String>();
		Vector cs_vec = getSupportedCodingSchemes();
		for (int i=0; i<cs_vec.size(); i++)
		{
		    CodingScheme cs = (CodingScheme) cs_vec.elementAt(i);
		    Vector w = getSupportedProperties(cs);

			if (w == null)
			{
			   getLogger().warn("WARNING: getSupportedProperties(cs) returns null");
			   return null;
			}

			for (int j=0; j<w.size(); j++)
			{
				 SupportedProperty sp = (SupportedProperty) w.elementAt(j);
				 String propertyname = sp.getLocalId();

				 if (!v.contains(propertyname))
				 {
					 v.add(propertyname);
				 }
			}
		}

        return v;
	}




    /**
     * Gets the supported property names.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag the tag
     *
     * @return the supported property names
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedPropertyNames(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedPropertyNames(cs);
	}


    /**
     * Gets the supported representational forms.
     *
     * @param cs the CodingScheme
     *
     * @return the supported representational forms
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedRepresentationalForms(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedRepresentationalForm[] forms = cs.getMappings().getSupportedRepresentationalForm();
		for (int i=0; i<forms.length; i++)
		{
		     SupportedRepresentationalForm sf = (SupportedRepresentationalForm) forms[i];
		     v.add(sf.getContent());
		}
        return v;
	}

    /**
     * Gets the supported representational forms.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag the ltag
     *
     * @return the supported representational forms
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedRepresentationalForms(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedRepresentationalForms(cs);
	}

    /**
     * Gets the supported concept status.
     *
     * @param codingSchemeName the coding scheme name
     *
     * @return the supported concept status
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedConceptStatus(String codingSchemeName)
	{
//	    CodingScheme cs = getCodingScheme(codingSchemeName, version, tag);
	    CodingScheme cs = getCodingScheme(codingSchemeName, null, null);
	    return getSupportedConceptStatus(cs);
    }

    /**
     * Gets the supported concept status.
     *
     * @param cs the CodingScheme
     *
     * @return the supported concept status
     */
    public Vector<String> getSupportedConceptStatus(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedConceptStatus[] scs = cs.getMappings().getSupportedConceptStatus();
	    if (scs == null)
	    {
			getLogger().warn("WARNING: DLBWrapper getSupportedConceptStatus returns null.");
			return null;
		}
		else if (scs.length == 0)
		{
			getLogger().warn("WARNING: DLBWrapper getSupportedConceptStatus returns 0 entries.");
		}

		for (int i=0; i<scs.length; i++)
		{
		     SupportedConceptStatus status = (SupportedConceptStatus) scs[i];
		     //v.add(status.getContent());
		     v.add(status.getLocalId());
		}
		return v;
	}


    /**
     * Gets the supported concept status.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag the ltag
     *
     * @return the supported concept status
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedConceptStatus(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedConceptStatus(cs);
	}




///////////////////////////////////////////////////////////////////////////////////

    /**
 * Resolve concept reference.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param ltag a coding scheme tag
 * @param code the concept code
 *
 * @return the resolved concept reference
 *
 * @since DLBExt 1.0
 */
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingSchemeName,
	                                String vers,
		                            String ltag,
									String code)
	{
        //return resolveConceptReference(codingScheme, version, null, code, null, 1, 1);
//        return resolveConceptReference(codingSchemeName, vers, ltag, code, null, 1, 1);
        return resolveConceptReference(codingSchemeName,vers,ltag,code,true);
	}


    /**
     * Resolve concept reference.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param resolveAssociationDepth parameter not used
     * @param resolveCodedEntryDepth parameter not used
     *
     * @return the resolved concept reference
     *
     * @since DLBExt 1.0
     * @deprecated - use resolveConceptReference(String,String,String,String)
     */@Deprecated
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingSchemeName,
	                                String vers,
	                                String ltag,
									String code,
									Vector properties,
									int resolveAssociationDepth,
                                    int resolveCodedEntryDepth)
	{
    	return resolveConceptReference(
    			codingSchemeName,
    			vers,
    			ltag,
									code,
									properties,
									resolveAssociationDepth,
                                    resolveCodedEntryDepth,
                                    true);
	}


    /**
     * Resolve concept reference.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param resolveAssociationDepth parameter not used
     * @param resolveCodedEntryDepth parameter not used
     * @param direction the direction
     *
     * @return the resolved concept reference
     *
     * @since DLBExt 1.0
     * @deprecated - use resolveConceptReference(String,String,String,String,boolean)
     */@Deprecated
    protected ResolvedConceptReference resolveConceptReference(
    		String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		Vector properties,
    		int resolveAssociationDepth,
    		int resolveCodedEntryDepth,
    		boolean direction
    )
    {
    	return resolveConceptReference(
    			codingSchemeName,
    			vers,
    			ltag,
    			code,
    			properties,
    			resolveAssociationDepth,
    			resolveCodedEntryDepth,
    			direction,
    			null);

    }



    /**
     * Resolve concept reference.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param resolveAssociationDepth parameter not used
     * @param resolveCodedEntryDepth parameter not used
     * @param direction the direction
     * @param relation parameter not used
     *
     * @return the resolved concept reference
     * @deprecated - use resolveConceptReferenct(String,String,String,String,Vector,boolean)
     */@Deprecated
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingSchemeName,
	                                String vers,
									String ltag,
									String code,
									@SuppressWarnings("unused")Vector properties,
									@SuppressWarnings("unused")int resolveAssociationDepth,
									@SuppressWarnings("unused")int resolveCodedEntryDepth,
                                    boolean direction,
                                    @SuppressWarnings("unused")String relation
									)
	{
//        String relationName = relation;
//        // 031407
//        if (relation != null)
//        {
//			if (!hasRelation(codingSchemeName, vers, ltag, relation))
//			{
//				relationName = null;
//			}
//	    }

		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
		ResolvedConceptReference rcr = null;
		ResolvedConceptReferenceList rcrl = null;

		boolean b1 = true;
		boolean b2 = false;

		if (!direction)
		{
			b1 = false;
			b2 = true;
		}

		try {

			ConceptReference ref = createConceptReference(code, codingSchemeName);
			if (ref == null)
			{
				getLogger().warn("WARNING: createConceptReference(" + code + ") returns null.");
				return null;
			}

			rcrl = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null)
					.resolveAsList(
						ref,
						b1,
						b2,
						1,
						1,
						null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
						null,
						null,
						1024);


			if (rcrl == null)
			{
				getLogger().error("lbSvc.getNodeGraph returns null");
				return null;
			}

			// Analyze the result ...
			if (rcrl.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
				if (rcrs == null)
				{
					getLogger().error("rcrl.getResolvedConceptReference returns null");
					return null;
				}
				if (rcrs.length == 0)
				{
					getLogger().error("rcrs.length == 0");
					return null;
				}
				rcr = rcrs[0];
			}

	    } catch (Exception e) {
			getLogger().error("ResolvedConceptReference threw exception.");
			//e.printStackTrace();
		}
        return rcr;
	}


    /**
     * Resolve concept reference.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction the direction
     *
     * @return the resolved concept reference
     */
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingSchemeName,
	                                String vers,
									String ltag,
									String code,
                                    boolean direction)
	{
//        String relationName = relation;
//        // 031407
//        if (relation != null)
//        {
//			if (!hasRelation(codingSchemeName, vers, ltag, relation))
//			{
//				relationName = null;
//			}
//	    }

		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
		ResolvedConceptReference rcr = null;
		ResolvedConceptReferenceList rcrl = null;

		boolean b1 = true;
		boolean b2 = false;

		if (!direction)
		{
			b1 = false;
			b2 = true;
		}

		try {

			ConceptReference ref = createConceptReference(code, codingSchemeName);
			if (ref == null)
			{
				getLogger().warn("WARNING: createConceptReference(" + code + ") returns null.");
				return null;
			}

			rcrl = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null)
					.resolveAsList(
						ref,
						b1,
						b2,
						1,
						1,
						null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
						null,
						null,
						1024);


			if (rcrl == null)
			{
				getLogger().error("lbSvc.getNodeGraph returns null");
				return null;
			}

			// Analyze the result ...
			if (rcrl.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
				if (rcrs == null)
				{
					getLogger().error("rcrl.getResolvedConceptReference returns null");
					return null;
				}
				if (rcrs.length == 0)
				{
					getLogger().error("rcrs.length == 0");
					return null;
				}
				rcr = rcrs[0];
			}

	    } catch (Exception e) {
			getLogger().error("ResolvedConceptReference threw exception.");
			//e.printStackTrace();
		}
        return rcr;
	}


	/*
	 * Test Membership in Vocabulary
	 */

	 /**
	 * Concept exists.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param code the concept code
	 *
	 * @return true, if concept exists
	 *
	 * @since DLBExt 1.0
	 */
	 public boolean conceptExists(String codingSchemeName, String code)
     {
//	     Boolean Bool_val = conceptExists(codingSchemeName, null, null, true, code);
		 Boolean Bool_val = conceptExists(codingSchemeName, null, null, code);
	     if (Bool_val == null || Bool_val.equals(Boolean.FALSE)) return false;
	     return true;
     }


	 /**
 	 * Test if concept exists within the given coding scheme
 	 *
 	 * @param codingSchemeName the coding scheme name
 	 * @param vers the coding scheme version
 	 * @param ltag the coding scheme tag
 	 * @param activeOnly parameter not used
 	 * @param code the concept code
 	 *
 	 * @return true if concept exists, false otherwise
 	 * @deprecated - use conceptExists(String,String,String,String)
 	 */@Deprecated
 	protected Boolean conceptExists(String codingSchemeName, String vers, String ltag,
	                               @SuppressWarnings("unused")boolean activeOnly, String code)
	 {
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
	 	 try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, activeOnly);
			 //ConceptReference cr = createConceptReference(codingScheme, code);

			 ConceptReference cr = createConceptReference(code, codingSchemeName);


			 return cns.isCodeInSet(cr);
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }


	 /**
	 * Test if concept exists within the given coding scheme
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return true if concept exists, false otherwise
	 */
	protected Boolean conceptExists(String codingSchemeName, String vers, String ltag,
	                               String code)
	 {
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
	 	 try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, activeOnly);
			 //ConceptReference cr = createConceptReference(codingScheme, code);

			 ConceptReference cr = createConceptReference(code, codingSchemeName);


			 return cns.isCodeInSet(cr);
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }



 	/**
 	 * Creates the name and value list.
 	 *
 	 * @param names the names
 	 * @param values the values
 	 *
 	 * @return the name and value list
 	 */
 	protected NameAndValueList createNameAndValueList(String[] names, String[] values)
 	{
 		NameAndValueList nvList = new NameAndValueList();
 		for (int i=0; i<names.length; i++)
 		{
 			NameAndValue nv = new NameAndValue();
 			nv.setName(names[i]);
 			if (values != null)
 			{
 				nv.setContent(values[i]);
 			}
 			nvList.addNameAndValue(nv);
 		}
 		return nvList;
 	}



	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingSchemeName a coding scheme name
	  * @param vers a coding scheme version
	  * @param ltag
	  * @param associationName
	  * @param associationQualiferName
	  * @param code
	  */
	 protected Boolean isCodeInGraph(String codingSchemeName, String vers, String ltag,
	                               String associationName, String associationQualiferName, String code)
	 {
		 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
	 	 try {
			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null);

			 NameAndValueList arl_asso =
				createNameAndValueList(
					new String[] {associationName}, null);

			 NameAndValueList arl_qualifier =
				createNameAndValueList(
					new String[] {associationQualiferName}, null);

//			 LocalNameList qualifiers = null;//new LocalNameList();

			 cng = cng.restrictToAssociations(arl_asso, arl_qualifier);
			 //ConceptReference targetCode = createConceptReference(codingSchemeName, code);

			 ConceptReference targetCode = createConceptReference(code, codingSchemeName);

			 return cng.isCodeInGraph(targetCode);
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }


    /*
     * Enumerate All Concepts
     */

	 /**
     * Gets the all concepts.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param activeOnly - parameter not used
     * @param sortproperties - parameter not used
     * @param properties - vector of properties to retrict the search to
     * @param maxToReturn the max to return
     *
     * @return the all concepts
     *
     * @since DLBExt 1.0
     * @deprecated - use getAllConcepts(String,String,String,Vector,int)
     */@Deprecated
	 protected Vector<CodedEntry> getAllConcepts(String codingSchemeName, String vers, String ltag, @SuppressWarnings("unused")boolean activeOnly,
			 @SuppressWarnings("unused")Vector sortproperties, Vector properties, int maxToReturn
								   )
     {
	     Vector<CodedEntry> v = new Vector<CodedEntry>();
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
         try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, activeOnly);

	//		 LocalNameList sortByProperty = vector2LocalNameList(sortproperties);
			 LocalNameList restrictToProperties = vector2LocalNameList(properties);

// temporary fix
			// Sort by search engine recommendation & code ...
			SortOptionList sortCriteria =
			    //Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
			    Constructors.createSortOptionList(new String[]{"entityDescription"});


//			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortByProperty, restrictToProperties, maxToReturn);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, restrictToProperties, null, maxToReturn);

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }



	 /**
 	 * Gets the all concepts.
 	 *
 	 * @param codingSchemeName a coding scheme name
 	 * @param vers a coding scheme version
 	 * @param ltag a coding scheme tag
 	 * @param properties - vector of properties to retrict the search to
 	 * @param maxToReturn the max to return
 	 *
 	 * @return the all concepts
 	 *
 	 * @since DLBExt 1.0
 	 */
	 protected Vector<CodedEntry> getAllConcepts(String codingSchemeName, String vers, String ltag,
	                                Vector properties, int maxToReturn
								   )
     {
	     Vector<CodedEntry> v = new Vector<CodedEntry>();
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
         try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, activeOnly);

	//		 LocalNameList sortByProperty = vector2LocalNameList(sortproperties);
			 LocalNameList restrictToProperties = vector2LocalNameList(properties);

// temporary fix
			// Sort by search engine recommendation & code ...
			SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});


//			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortByProperty, restrictToProperties, maxToReturn);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, restrictToProperties, null, maxToReturn);

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }




    /*
     * Enumerate Properties
	 * Retrieve Concept and Attributes by Code
     */

	 /**
     * Gets the concept properties.
     *
     * @param ce the CodedEntry
     *
     * @return the concept properties
     *
     * @since DLBExt 1.0
     */
	 protected Vector<NameAndValue> getConceptProperties(CodedEntry ce)
     {
	    Vector<NameAndValue> v = new Vector<NameAndValue>();
        java.util.Enumeration enumeration = ce.enumerateComment();
		while (enumeration.hasMoreElements())
		{
		    Comment property = (Comment) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateDefinition();
		while (enumeration.hasMoreElements())
		{
		    Definition property = (Definition) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateInstruction();
		while (enumeration.hasMoreElements())
		{
		    Instruction property = (Instruction) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateConceptProperty();
		while (enumeration.hasMoreElements())
		{
		    org.LexGrid.commonTypes.Property property = (org.LexGrid.commonTypes.Property) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));

		}
		return v;
    }



    /**
     * Find coded entry with name matching.
     * Will return 1 entry, which will be the first one that has
     * an entityDescription that exactly matches the search term.
     *
     * Since entityDescriptions are not unique, the returned result might not
     * be the only, or the best, match.  The method might return different results
     * at different times.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param name the search term
     *
     * @return the coded entry
     * @deprecated - evsapi does not support name as an identifier
     */@Deprecated
    protected CodedEntry findCodedEntryWithNameMatching(String codingSchemeName,
                                                String vers,
	                                            String name)
	{
        Vector v = null;
        try {
	        v = findCodedEntriesWithDesignationMatching(codingSchemeName,
 	                                            vers,
 	                                            name,
 												false,
 	                                            new Vector(),
                                                null,
 												"exactMatch",
												1);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		if (v != null && v.size() > 0)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(0);
			return ce;
		}
		return null;

    }


    /*
     * Map Synonyms to Preferred Names
     */

    /**
     *
     * @since DLBExt 1.0
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param matchText
     * @param preferredOnly
     * @param sortByProperty_vec
     * @param maxToReturn
     * @param matchAlgorithm
     */
    protected Vector<String> mapSynonymsToPreferredNames(String codingSchemeName,
                                                String vers,
	                                            String matchText,
												boolean preferredOnly,
	                                            Vector sortByProperty_vec,
	                                            int maxToReturn,
												String matchAlgorithm)
    {
	    Vector<String> restrictToProperties_vec = new Vector<String>();
		restrictToProperties_vec.add("Synonym");


	    Vector<CodedEntry> v = null;
	    try {
	    	v = findCodedEntriesWithDesignationMatching(codingSchemeName,
	                                            vers,
	                                            matchText,
												preferredOnly,
	                                            sortByProperty_vec,
                                                restrictToProperties_vec,
												matchAlgorithm,
												maxToReturn);
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}

        // Get perferred names for each element
		String propertyName = PREFERRED_NAME;
		Vector<String> names = new Vector<String>();
		for (int i=0; i<v.size(); i++)
		{
//             ResolvedConceptReference rcr = (ResolvedConceptReference) v.elementAt(i);
			 Vector<String> u = getPropertyValues(v.elementAt(i), propertyName);
			 for (int j=0; j<u.size(); j++)
			 {
			     names.add((String) u.elementAt(j));
			 }
		}
		return names;
	}




    /*
     * Test for Current/Obsolete Concept
     */


	 /**
     * Gets the concept status.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag the ltag
     * @param code the code
     *
     * @return the concept status
     *
     * @since DLBExt 1.0
     */
	 public String getConceptStatus(String codingSchemeName, String vers, String ltag, String code)
	 {
         CodedEntry ce = getCodedEntry(codingSchemeName,
	                                vers,
									ltag,
									code);
         if (ce == null)
		 {
			  getLogger().warn("Concept with code " + code + " not found in " + codingSchemeName
			      + ".");
			  return null;
		 }
		 return ce.getConceptStatus();
	 }

     /**
      * Checks if is concept active.
      *
      * @param codingSchemeName a coding scheme name
      * @param code the code
      *
      * @return true, if checks if is concept active
      *
      * @since DLBExt 1.0
      */
     public Boolean isConceptActive(String codingSchemeName,
									 String code)
	 {
//          return isConceptActive(codingSchemeName, version, tag, code);
          return isConceptActive(codingSchemeName, null, null, code);
	 }



     /**
      * Checks if is concept active.
      *
      * @param codingSchemeName a coding scheme name
      * @param vers a coding scheme version
      * @param ltag the ltag
      * @param code the code
      *
      * @return true, if checks if is concept active
      *
      * @since DLBExt 1.0
      */
     public Boolean isConceptActive(String codingSchemeName,
	                                 String vers,
									 String ltag,
									 String code)

     {
         CodedEntry ce = getCodedEntry(codingSchemeName,
	                                vers,
									ltag,
									code);
         if (ce == null)
		 {
			  getLogger().warn("Concept with code " + code + " not found in " + codingSchemeName
			      + ".");
			  return null;
		 }
         return new Boolean(ce.getIsActive());
	 }




     /**
      * Checks if is code retired.
      *
      * @param codingSchemeName the coding scheme name
      * @param code the code
      *
      * @return the boolean
      */
     public Boolean isCodeRetired(String codingSchemeName, String code)
     {
		 Boolean retvalue = isConceptActive(codingSchemeName, null, null, code);
		 if (retvalue != null)
		 {
			 if (retvalue.equals(Boolean.TRUE)) return Boolean.FALSE;
			 return Boolean.TRUE;
		 }

		 try {
			 Vector v = getConceptEditActionDates(codingSchemeName, code, toActionIndex("retire"));
			 if (v != null && v.size() > 0) return Boolean.TRUE;

	     } catch (Exception e) {


			 return null;
		 }
		 return Boolean.FALSE;
	 }


    /*
     * Enumerate Kinds
     */

	 /**
     * Gets the concepts by kind.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param activeOnly parameter not used
     * @param kind the kind
     * @param maxToReturn the max to return
     *
     * @return the concepts by kind
     *
     * @since DLBExt 1.0
     * @deprecated - use getConceptsByKind(String,String,String,String,int)
     */@Deprecated
	 protected Vector<CodedEntry> getConceptsByKind(String codingSchemeName, String vers, String ltag,
			 @SuppressWarnings("unused") boolean activeOnly, String kind, int maxToReturn)
     {
		 /*
		 CodedNodeSet 	restrictToMatchingProperties(LocalNameList propertyNames,
		                                             CodedNodeSet.PropertyType[] propertyTypes,
		                                             java.lang.String matchText,
		                                             java.lang.String matchAlgorithm,
		                                             java.lang.String language)
		 */

		 Vector<CodedEntry> v = new Vector<CodedEntry>();
		 try {
			 CodedNodeSet cns =	lbSvc.getCodingSchemeConcepts(codingSchemeName, createCodingSchemeVersionOrTag(vers, ltag));//, activeOnly);
			 cns = cns.restrictToMatchingProperties(kindList, null, kind, "ExactMatch", null);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(null,kindList,null,maxToReturn);
			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     }
	     catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }

	 /**
     * Gets the concepts by kind.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag

     * @param kind the kind
     * @param maxToReturn the max to return
     *
     * @return the concepts by kind
     *
     * @since DLBExt 1.0
     */
	 protected Vector<CodedEntry> getConceptsByKind(String codingSchemeName, String vers, String ltag,
			  String kind, int maxToReturn)
     {
		 /*
		 CodedNodeSet 	restrictToMatchingProperties(LocalNameList propertyNames,
		                                             CodedNodeSet.PropertyType[] propertyTypes,
		                                             java.lang.String matchText,
		                                             java.lang.String matchAlgorithm,
		                                             java.lang.String language)
		 */

		 Vector<CodedEntry> v = new Vector<CodedEntry>();
		 try {
			 CodedNodeSet cns =	lbSvc.getCodingSchemeConcepts(codingSchemeName, createCodingSchemeVersionOrTag(vers, ltag));//, activeOnly);
			 cns = cns.restrictToMatchingProperties(kindList, null, kind, "ExactMatch", null);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(null,kindList,null,maxToReturn);
			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     }
	     catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }


///////////////////////
//  Relationships
///////////////////////

    /**
 * Gets the relation names.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 *
 * @return the relation names
 *
 * @since DLBExt 1.0
 */
    public Vector<String> getRelationNames(String codingSchemeName, String vers)
	{
	    Vector<String> v = new Vector<String>();
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, null);

		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
			v.add(rel.getDc());
		}
		return v;
	}


    /**
     * Gets the inverse name of a given association
     *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param relationName the relation container name
     * @param associationName the association name
     *
     * @return the inverse name
     */
    public String getInverseName(String codingSchemeName, String vers, String relationName, String associationName)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, null);

		Relations[] rels = cs.getRelations();
		Relations relation = rels[0];
		if (relationName != null)
		{
			for (int i=0; i<rels.length; i++)
			{
				Relations rel = (Relations) rels[i];
				if (rel.getDc().compareTo(relationName) == 0)
				{
					relation = rel;
					break;
				}
			}
		}

		org.LexGrid.relations.Association[] assos = relation.getAssociation();
		for (int j=0; j<assos.length; j++)
		{
			org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
		    if (asso.getForwardName().compareTo(associationName) == 0)
		    {
				return asso.getReverseName();
			}
			else if (asso.getReverseName() != null)
			{
				if (asso.getReverseName().compareTo(associationName) == 0)
				{
					return asso.getForwardName();
				}
			}
		}
		return null;
	}



    /**
     * Checks for existence of relation in a coding scheme.
     *
     * @param codingSchemeName the coding scheme name
     * @param relationName the relation name
     *
     * @return true, if successful
     */
    public boolean hasRelation(String codingSchemeName, String relationName)
    {
		return hasRelation(codingSchemeName, null, null, relationName);
	}

    /**
     * Checks for relation.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param ltag the tag
     * @param relationName the relation name
     *
     * @return true, if successful
     */
    public boolean hasRelation(String codingSchemeName, String vers, String ltag, String relationName)
    {
		try
		{
			CodingSchemeVersionOrTag vt = createCodingSchemeVersionOrTag(vers, ltag);
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return hasRelation(scheme, relationName);
    	} catch (Exception e) {
			return false;
		}
	}


    /**
     * Checks for relation.
     *
     * @param cs the CodingScheme
     * @param relationName the relation name
     *
     * @return true, if successful
     */
    public boolean hasRelation(CodingScheme cs, String relationName)
    {
		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
		    String name = rel.getDc();
		    if (name.compareTo(relationName) == 0)
		    {
				return true;
			}
		}
		return false;
	}


   /**
    * Gets the supported association names for the given coding scheme.
    *
    * @param codingSchemeName the coding scheme name
    *
    * @return the association names
    */
   private Vector<String> getAssociationNames(String codingSchemeName)
    {
	    return getAssociationNames(codingSchemeName, null, null);
    }


    /**
     * Gets the supported association names for the given coding scheme.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param ltag the tag
     *
     * @return the association names
     */
    private Vector<String> getAssociationNames(String codingSchemeName, String vers, String ltag)
    {
		Vector<String> v = new Vector<String>();
		try
		{
			CodingSchemeVersionOrTag vt = createCodingSchemeVersionOrTag(vers, ltag);
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return getAssociationNames(scheme);
    	} catch (Exception e) {
			return v;
		}
	}


	/**
	 * Gets the supported association names for the given coding scheme.
	 *
	 * @param cs the CodingScheme
	 *
	 * @return the association names
	 */
	private Vector<String> getAssociationNames(CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
		Relations[] relations = cs.getRelations();
		for (int i=0; i<relations.length; i++)
		{
			Relations r = relations[i];
			org.LexGrid.relations.Association[] associations = r.getAssociation();
			for (int j=0; j<associations.length; j++)
			{
				org.LexGrid.relations.Association association = associations[j];
				String name = association.getAssociation();
				v.add(name);
		    }
		}
		return v;
	}


    /**
     * Gets the association names by relation.
     *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param relationName the relation name
     *
     * @return the association names by relation
     */
    public Vector<String> getAssociationNamesByRelation(String codingSchemeName, String vers, String relationName)
	{
		CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
		vt.setVersion(vers);
		try
		{
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return getAssociationNamesByRelation(scheme, relationName);
    	} catch (Exception e) {
			return null;
		}
	}


    /**
     * Gets the association names by relation.
     *
     * @param codingSchemeName the coding scheme
     * @param relationName the relation name
     *
     * @return the association names by relation
     */
    public Vector<String> getAssociationNamesByRelation(String codingSchemeName,  String relationName)
	{
		CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
		try
		{
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return getAssociationNamesByRelation(scheme, relationName);
    	} catch (Exception e) {
			return null;
		}
	}

    /**
     * Gets the association names by relation.
     *
     * @param cs the CodingScheme
     * @param relation the relation
     *
     * @return the association names by relation
     */
    public Vector<String> getAssociationNamesByRelation(CodingScheme cs, String relation)
    {
		//031507
	    String relationName = relation;
        if (relation != null)
        {
			if (!hasRelation(cs, relation))
			{
				relationName = null;//"Relations";//null;
			}
	    }

		Vector<String> v = new Vector<String>();
	    if (relation == null)
	    {
			// return all association names
			return getAssociationNames(cs);
		}


		Relations[] rels = cs.getRelations();
		/*
		if (relationName == null || relationName.compareTo("") == 0)
		{
		    Relations rel = (Relations) rels[0];
			org.LexGrid.relations.Association[] assos = rel.getAssociation();
			for (int j=0; j<assos.length; j++)
			{
				org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
				//v.add(asso.getForwardName());
				v.add(asso.getAssociation());
			}
			return v;
		}
		*/

		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
		    String name = rel.getDc();
		    if (name.compareTo(relationName) == 0)
		    {
                org.LexGrid.relations.Association[] assos = rel.getAssociation();
				for (int j=0; j<assos.length; j++)
				{
					org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
					//v.add(asso.getForwardName());
					v.add(asso.getAssociation());
				}
		    }
		}
		return v;
	}


    /**
     * Gets the native relation name.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     *
     * @return the native relation name
     *
     * @since DLBExt 1.0
     */
    protected String getNativeRelationName(String codingSchemeName, String vers)
	{
//	    Vector v = new Vector();
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, null);
		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
			if (rel.getIsNative()) return rel.getDc();
		}
		return "";
	}



    /**
     * Checks for associated concepts.
     *
     * @param codingSchemeName the coding scheme
     * @param code the code
     * @param direction the direction
     * @param associationName the association name
     *
     * @return true, if checks for associated concepts
     *
     * @since DLBExt 1.0
     */
    public Boolean hasAssociatedConcepts(String codingSchemeName,
	                                     String code,
										 boolean direction,
										 String associationName)
	{
		Vector<String> v = findRelatedCodes(codingSchemeName, null, null,
	                                code,
									associationName,
									direction);

		if (v == null) return null;
		if (v.size() == 0) return Boolean.FALSE;
		return Boolean.TRUE;
	}

    /*
     * Traverse Graph via Role Links (by recursion using getAssociatedConcepts)
     */

    /*
     * Enumerate Source Concepts for Relation and Target
     */

    /**
     * Gets the association sources.
     *
     * @param codingSchemeName the coding scheme name
     * @param code the concept code
     * @param associationName the association name
     *
     *
     * @return the association sources
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getAssociationSources(String codingSchemeName,
	                                    String code,
										String associationName)
	{
         return getAssociatedCodedEntries(codingSchemeName,
	                                  code,
									  false,
									  associationName);
	}


    /**
     * Gets the association sources.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param associationName the association name
     *
     * @return the association sources
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getAssociationSources(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		Vector properties,
    		String associationName)
    {
    	return getAssociatedCodedEntries(codingSchemeName,
    			vers,
    			ltag,
    			code,
    			properties,
    			true,
    			associationName);
    }


    /*
     * Enumerate Target Concepts for Relation and Source
     */

    /**
     * Gets the association targets.
     *
     * @param codingSchemeName a coding scheme name
     * @param code a concept code
     * @param associationName the association name
     *
     * @return the association targets
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getAssociationTargets(String codingSchemeName,
	                                    String code,
										String associationName)
	{
         return getAssociatedCodedEntries(codingSchemeName,
	                                  code,
									  true,
									  associationName);
	}


    /**
     * Gets the association targets.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param ltag the tag
     * @param code the concept code
     * @param properties parameter not used
     * @param associationName the association name
     *
     * @return the association targets
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getAssociationTargets(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		Vector properties,
    		String associationName)
    		{
    	return getAssociatedCodedEntries(codingSchemeName,
    			vers,
    			ltag,
    			code,
    			properties,
    			false,
    			associationName);
    		}

////////////////////////////////////////////////////////////////////////////////////

    /**
 * Gets the associated coded entries.
 *
 * @param codingSchemeName a coding scheme name
 * @param code a concept code
 * @param associationName the association name
 *
 * @return the associated coded entries
 *
 * @since DLBExt 1.0
 */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
	                                    String code,
										String associationName)
    {
         return getAssociatedCodedEntries(codingSchemeName,
        		                         null,
        		                         null,
	                                     code,
										 true,
										 associationName);
	}

    /**
     * Gets the associated coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param code a concept code
     * @param direction - true to find target, false to find sources
     * @param associationName the association name
     *
     * @return the associated coded entries
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
	                                    String code,
										boolean direction,
										String associationName)
	{

//         return getAssociatedCodedEntries(codingSchemeName,
//										 null,
//										 null,
//	                                     code,
//										 null,
//										 direction,
//										 associationName);

         return getAssociatedCodedEntries(codingSchemeName,
				 null,
				 null,
                 code,
				 direction,
				 associationName);
	}


    /**
     * Gets the associated coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties - parameter not used
     * @param direction - true to find target, false to find sources
     * @param associationName the association name
     *
     * @return the associated coded entries
     *
     * @deprecated - use getAssociatiedCodedEntries(String,String,String,String,boolean,String)
     */@Deprecated
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		Vector properties,
    		boolean direction,
    		String associationName)
    		{
    	return getAssociatedCodedEntries(codingSchemeName,
    			vers,
    			ltag,
    			code,
    			properties,
    			direction,
    			associationName, 1, 1);
    		}




////Associated Concepts////////////////////////////////////////////////////////////////////////////////////
	/**
 * Gets the associated concepts.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param ltag the coding scheme tag
 * @param code the concept code
 * @param relation the relation name
 *
 *
 * @return the associated concepts
 *
 * @since DLBExt 1.0
 */
    public Vector<AssociatedConcept> getAssociatedConcepts(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		String relation)
    {
    	return getAssociatedConcepts(codingSchemeName,
    			vers,
    			ltag,
    			code,
    			relation,
    			true);
    }


	/**
	 * Gets the associated concepts.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param ltag a coding scheme tag
	 * @param code a concept code
	 * @param relation a relation name
	 * @param direction - true to find target, false to find sources
	 *
	 * @return the associated concepts
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<AssociatedConcept> getAssociatedConcepts(String codingSchemeName,
	                                       String vers,
	                                       String ltag,
	                                       String code,
	                                       String relation,
	                                       boolean direction)
    {
		Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
        try {

			// Perform the query ...
			boolean b1 = true;
			boolean b2 = false;

			if (!direction)
			{
				b1 = false;
				b2 = true;
			}

            ResolvedConceptReferenceList matches = null;

            SortOptionList sortOptions = Constructors.createSortOptionList(new String[]{"entityDescription"});

            if (relation == null)
            {
				matches =
					lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null)
						.resolveAsList(
							createConceptReference(code, codingSchemeName),
							b1, b2, 1, 1,
							null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
							null,
							sortOptions, 1024);
			}
			else
			{
			    NameAndValueList rel_list =
				    createNameAndValueList(new String[] {relation}, null);

			    CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null);
				cng = cng.restrictToAssociations(
						rel_list,
						null);

/*
 ResolvedConceptReferenceList 	resolveAsList(ConceptReference graphFocus, boolean resolveForward,
        boolean resolveBackward, int resolveCodedEntryDepth, int resolveAssociationDepth,
        LocalNameList propertyNames, CodedNodeSet.PropertyType[] propertyTypes,
        SortOptionList sortOptions, int maxToReturn)
*/



				matches = cng
					.resolveAsList(
						createConceptReference(code, codingSchemeName),
						b1, b2, 1, 1,
						null,
						null,
						sortOptions, 1024);

			}

			// Analyze the result ...

			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

                AssociationList assolist = null;
                if (direction)
                {
				    assolist = ref.getSourceOf();
			    }
			    else
			    {
					assolist = ref.getTargetOf();
				}
				if (assolist == null)
				{
					return v;
				}

				Association[] associations = assolist.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();

					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
					v = SortUtils.quickSort(v);
				}
			}

	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;

	}


	/**
	 * Gets the association names.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param ltag a coding scheme tag
	 * @param code a concept code
	 * @param direction - true to find target, false to find sources
	 *
	 * @return the association names
	 *
	 * @since DLBExt 1.0
	 */
	// including roles and associations
	public Vector<String> getAssociationNames(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			boolean direction)
			{
		HashSet<String> hset = new HashSet<String>();
		Vector<String> v = new Vector<String>();
		Vector<ConceptAssociation> w = getConceptAssociations(
				codingSchemeName,
				vers,
				ltag,
				code,
				direction);

		String is_a = getHasSubtype(codingSchemeName);

         for (int i=0; i<w.size(); i++)
         {
			  gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			  if (nv.getName().compareToIgnoreCase(is_a) != 0)
			  {
			      if (!hset.contains(nv.getName()));
			      {
					  hset.add(nv.getName());
					  v.add(nv.getName());
				  }
              }
		 }

		 return v;
	}


    /**
     * Checks for associated concepts.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code a concept code
     * @param properties - parameter not used
     * @param direction  true to find target, false to find sources
     * @param associationName the association name
     *
     * @return true, if has associated concepts
     *
     * @since DLBExt 1.0
     */
	public Boolean hasAssociatedConcepts(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			Vector properties,
			boolean direction,
			String associationName
	)
	{
//		Vector v = new Vector();
		int resolveAssociationDepth = 1; // distance
		int resolveCodedEntryDepth = 1;
		ResolvedConceptReference rcr = resolveConceptReference(
				codingSchemeName,
				vers,
				ltag,
				code,
				properties,
				resolveAssociationDepth,
				resolveCodedEntryDepth);

		if (rcr == null)
		{
			return null;
		}
        AssociationList ass0list = null;
		if (direction) // associations of which this code is source
		{
		   ass0list = rcr.getSourceOf();
		}
		else // associations of which this code is target
		{
           ass0list = rcr.getTargetOf();
		}

		if (ass0list == null)
		{
			getLogger().warn("WARNING: ass0list == null. ");
			return Boolean.FALSE;
		}

        Association[] assoArray;
        try {
			assoArray = ass0list.getAssociation();
			if (assoArray == null)
			{
				getLogger().warn("hasAssociatedConcepts assoArray == null. -- return FALSE");
				return Boolean.FALSE;
			}
	    } catch (Exception e) {
			getLogger().error("hasAssociatedConcepts Exception thrown.");
			return Boolean.FALSE;
		}
		// Check association type (is-a, part-of, etc)
		/* 050707
        for (int k=0; k<assoArray.length; k++)
		{
		    String name = assoArray[k].getAssociationName();
			if (name.compareTo(associationName) == 0)
			{
			    return Boolean.TRUE;
			}
		}
		*/

        if (associationName != null)
        {
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();
				if (name.compareTo(associationName) == 0)
				{
					return Boolean.TRUE;
				}
			}
     	}
     	else
     	{
			if (assoArray.length > 0) return Boolean.TRUE;
		}

        return Boolean.FALSE;
    }


    /**
     * Are associated concepts.
     *
     * @param codingSchemeName a coding scheme name
     * @param code1 the code1
     * @param code2 the code2
     * @param direction true if code1 source and code2 target, false otherwise
     * @param associationName the association name
     *
     * @return true, if are associated concepts
     *
     * @since DLBExt 1.0
     */
    public Boolean areAssociatedConcepts(String codingSchemeName,
	                                     String code1,
										 String code2,
										 boolean direction,
										 String associationName)
	{
		return areAssociatedConcepts(codingSchemeName,
									null,
									null,
	                                code1,
									code2,
									direction,
									associationName);
	}



    /**
     * Are associated concepts.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code1 the code1
     * @param code2 the code2
     * @param direction true if code1 source and code2 target, false otherwise
     * @param associationName the association name
     *
     * @return true, if are associated concepts
     *
     * @since DLBExt 1.0
     */
    public Boolean areAssociatedConcepts(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code1,
    		String code2,
    		boolean direction,
    		String associationName)
    {
    	try {
    		Vector assos = getAssociatedConcepts(codingSchemeName,
    				vers,
    				ltag,
    				code1,
    				associationName,
    				direction);

    		if (assos == null) return Boolean.FALSE;
			 for (int i=0; i<assos.size(); i++)
			 {
				 AssociatedConcept ac = (AssociatedConcept) assos.elementAt(i);
				 if (ac == null)
				 {
					 getLogger().warn("WARNING: AssociatedConcept is null");
				 }
				 else if (ac.getConceptCode() == null)
				 {
					 getLogger().warn("WARNING: ac.getConceptCode() == null");
				 }
				 else if (ac.getConceptCode().compareTo(code2) == 0)
				 {
					 return Boolean.TRUE;
				 }
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return Boolean.FALSE;
	 }



    /**
     * Checks if is code in graph.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code a concept code
     *
     * @return true, if checks if is code in graph
     *
     * @since DLBExt 1.0
     */
    public Boolean isCodeInGraph(String codingSchemeName,
								 String vers,
								 String ltag,
								 String code)
	{

		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
		try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null);
			//ConceptReference ref = createConceptReference(codingScheme, code);

			ConceptReference ref = createConceptReference(code, codingSchemeName);
			if (ref == null) return Boolean.FALSE;
            try {
				return cng.isCodeInGraph(ref);
			} catch (Exception ex) {
				return Boolean.FALSE;
			}
	    } catch (Exception e) {
			e.printStackTrace();
			getLogger().warn("DLBWrapper isCodeInGraph cng.isCodeInGraph(ref) returns null");
			return Boolean.FALSE;
		}
	}


    /**
     * Are codes related.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code1 the code1
     * @param code2 the code2
     * @param associationName the association name
     *
     * @return true, if are codes related
     *
     * @since DLBExt 1.0
     */
    public Boolean areCodesRelated(String codingSchemeName,
								 String vers,
								 String ltag,
								 String code1, // source
								 String code2, // target
								 String associationName)
	{
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
		try {

			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, tagOrVersion, null);

 	         NameAndValue association = createNameAndValue(associationName, null);


             ConceptReference ref1 = createConceptReference(code1, codingSchemeName);
             ConceptReference ref2 = createConceptReference(code2, codingSchemeName);

             Boolean bool_obj = Boolean.FALSE;
             try {
             	bool_obj = cng.areCodesRelated(association, ref1, ref2, true);
			 } catch (Exception ex) {
				 //ex.printStackTrace();
				 getLogger().warn("WARNING: cng.areCodesRelated throws exceptions");
				 return null;
			 }
			 return bool_obj;
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
    }


///////////////////////////////////////////////////////////////////////////////////////////////




	/**
 * Gets the related coded entries.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param ltag a coding scheme tag
 * @param code the code
 * @param direction true if code1 source and code2 target, false otherwise
 * @param associationName the association name
 *
 * @return the related coded entries
 *
 * @since DLBExt 1.0
 */
	protected Vector<CodedEntry> getRelatedCodedEntries(
		                 String codingSchemeName,
						 String vers,
						 String ltag,
						 String code, // source
						 boolean direction, // true, if code is source
						 String associationName)
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
//		String relation = null;
		CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(vers, ltag);

		// Perform the query ...
//		NameAndValue nv = new NameAndValue();
		//NameAndValueList nvList = new NameAndValueList();

//		NameAndValueList nvList = null;

        try {

			boolean b1 = true;
			if (!direction)
			{
				b1 = false;
			}
			boolean b2 = false;
			if (!direction)
			{
				b2 = true;
			}

			ResolvedConceptReferenceList matches =
				lbSvc.getNodeGraph(codingSchemeName, csvt, null)
					.resolveAsList(
						createConceptReference(code, codingSchemeName),
						b1, b2, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList al = null;
				if (direction)
				{
					al = ref.getSourceOf();
				}
				else
				{
					al = ref.getTargetOf();
				}


				Association[] associations = al.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					if (associationName != null)
					{
						if (assoc.getAssociationName().compareTo(associationName) == 0)
						{
							AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
							if (acl != null)
							{
								for (int j = 0; j < acl.length; j++) {

									AssociatedConcept ac = acl[j];
									if (ac != null)
									{
										String concept_code = ac.getConceptCode();

										CodedEntry ce = getCodedEntry(codingSchemeName,
											vers, ltag, concept_code);
										v.add(ce);
									}
								}
							}
						}
				    }
				    else
				    {
						AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
						if (acl != null)
						{
							for (int j = 0; j < acl.length; j++) {
								AssociatedConcept ac = acl[j];
								if (ac != null)
								{
									String concept_code = ac.getConceptCode();

									CodedEntry ce = getCodedEntry(codingSchemeName,
										vers, null, concept_code);
									v.add(ce);
								}
							}
						}
					}
				}

			}
			return v;
	    } catch (Exception e) {
			return null;
		}

	}


    /**
     * Gets the codes in coded entries.
     *
     * @param w the CodedEntry Vector
     *
     * @return the codes in coded entries
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getCodesInCodedEntries(Vector<CodedEntry> w)
	{
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
		{
			  CodedEntry ce = (CodedEntry) w.elementAt(i);
              v.add(ce.getConceptCode());
		}
        return v;
	}



//======================================================================================================================
    /*
     * Retrieve Relations for Concept
	 *
     */


    /**
 * Checks if is valid role in the coding scheme.
 *
 * @param codingSchemeName the coding scheme name
 * @param associationName the association name
 *
 * @return true, if is role
 */
protected boolean isRole(String codingSchemeName, String associationName)
    {
		Vector associations = getSupportedRoleNames(codingSchemeName);
		if (associations == null || associations.size() == 0) return false;
		if (associations.contains(associationName)) return true;
		return false;
	}

    /**
     * converts ConceptAssociation to Role.
     *
     * @param asso a ConceptAssociation
     *
     * @return the vector< role>
     */
    public static Vector<Role> conceptAssociations2Roles(Vector<ConceptAssociation> asso)
    {

    	ConceptAssociation assoInstance = new ConceptAssociation();
    	Vector<Role> roles = new Vector<Role>();
    	for (int i = 0; i < asso.size(); i++)
    	{
    		assoInstance = asso.elementAt(i);
    		Role r = new Role(assoInstance.getName(), assoInstance.getValue());
    		Vector v = assoInstance.getQualifiers();
    		for (int j=0; j<v.size(); j++)
    		{
    			Qualifier q = (Qualifier) v.elementAt(i);
    			r.addQualifier(q);
    		}
    		roles.add(r);

    	}


		return roles;
	}

	/**
	 * Gets the roles.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 *
	 * @return the roles
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<Role> getRoles(String codingSchemeName, String code)
    {
	     return getRoles(codingSchemeName, null, null, code);
    }


	/**
	 * Gets the concept associations.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param code the code
	 *
	 * @return the concept associations
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<ConceptAssociation> getConceptAssociations(String codingSchemeName, String code)
    {
		 return getConceptAssociations(codingSchemeName, null, null, code);
    }



	/**
	 *  Gets Role objects for given association name
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 * @param associationName the association name
	 *
	 * @return the roles
	 */
	public Vector<Role> getRoles(String codingSchemeName,
						   String code,
						   String associationName)
    {
	   		Vector<ConceptAssociation> ca =  getConceptAssociations(codingSchemeName,
					  null,
					  null,
					  code,
					  associationName);


	   		return DLBWrapper.conceptAssociations2Roles(ca);
    }


	/**
	 * Gets ConceptAssociation objects for given association name
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param code a concept code
	 * @param associationName the association name
	 *
	 * @return the concept associations
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<ConceptAssociation> getConceptAssociations(String codingSchemeName,
										 String code,
										 String associationName)
    {
	     return getConceptAssociations(codingSchemeName,
										  null,
										  null,
										  code,
										  associationName);
    }

   	/**
	    * Gets Role objects for given association name
	    *
	    * @param codingSchemeName the coding scheme name
	    * @param vers the coding scheme version
	    * @param ltag the coding scheme tag
	    * @param code the concept code
	    * @param associationName the association name
	    *
	    * @return the roles
	    */
	   public Vector<Role> getRoles(String codingSchemeName,
										  String vers,
										  String ltag,
										  String code,
										  String associationName)
	{
   		Vector<ConceptAssociation> ca =  getConceptAssociations(codingSchemeName, vers, ltag, code, associationName);
   		return DLBWrapper.conceptAssociations2Roles(ca);
	}

	   	/**
	    * Gets ConceptAssociation objects for given association name
	    *
	    * @param codingSchemeName the coding scheme name
	    * @param vers the coding scheme version
	    * @param ltag the coding scheme tag
	    * @param code the concept code
	    * @param associationName the association name
	    *
	    * @return the roles
	    */
	   public Vector<ConceptAssociation> getConceptAssociations(String codingSchemeName,
			   String vers,
			   String ltag,
			   String code,
			   String associationName)
	   {
		   /*
		   Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
				   vers,
				   ltag,
				   code,
				   true);
		   if (w == null) return null;
		   if (associationName == null) return w;
		   Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
		   for (int i=0; i<w.size(); i++)
		   {
			   gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			   if (nv.getName().compareTo(associationName) == 0)
			   {
				   v.add(nv);
			   }
	       }
	  	   return v;
	  	   */
	  	   boolean direction = true;
	  	   String relationContainer = null;
	  	   Vector asso_names = null;
	  	   if (associationName != null)
	  	   {
	  	   	   asso_names = new Vector();
			   asso_names.add(associationName);
		   }
	       return getAssociations(codingSchemeName, vers, ltag, code, direction, relationContainer, asso_names);
	   }


		private gov.nih.nci.lexbig.ext.ConceptAssociation convertConceptAssociation(String codingSchemeName,
		                                                                    String vers,
		                                                                    String associationName,
		                                                                    AssociatedConcept ac,
		                                                                    boolean returncode)
		{
			if (ac == null) return null;
			gov.nih.nci.lexbig.ext.ConceptAssociation concept_association = null;
			String asso_code = ac.getConceptCode();
			if (returncode)
			{
				concept_association = new gov.nih.nci.lexbig.ext.ConceptAssociation(associationName, asso_code);
			}
			else
			{
				String asso_name = codeToName(codingSchemeName, vers, asso_code);
				concept_association = new gov.nih.nci.lexbig.ext.ConceptAssociation(associationName, asso_name);
	        }
	        NameAndValueList nvl = ac.getAssociationQualifiers();
	        if (nvl != null)
	        {
				NameAndValue[] nva = nvl.getNameAndValue();
				for (int i=0; i<nva.length; i++)
				{
					NameAndValue nv = nva[i];
					concept_association.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(nv.getName(), nv.getContent()));
				}
			}
			return concept_association;
		}

	/**
	 * Gets the concept associations.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return the concept associations
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<ConceptAssociation> getConceptAssociations(String codingSchemeName,
			String vers,
			String ltag,
			String code)
	{
		Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
		String relationName = null;
		if (hasRelation(codingSchemeName, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;

        boolean direction = true;
        String relationContainer = relationName;
        Vector asso_names = null;
	    return getAssociations(codingSchemeName, vers, ltag, code, direction, relationContainer, asso_names);

/*

		ResolvedConceptReference rcr = resolveConceptReference(codingSchemeName,
				vers,
				ltag,
				code,
				null,
				1,
				1,
				true,
				//ASSOCIATION_RELATION);
		relationName);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getSourceOf();

		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingSchemeName);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			{
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), vers, ltag, cr.getConceptCode());
					v.add(new ConceptAssociation(assoname, getConceptName(codingSchemeName, codedentry_ref)));
				}
		    }
		}
		return v;
*/
	}


    // 031207
    /**
     * Gets the concept associations.
     *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
     * @param direction true if concept is source of association, false if target
     * @param returncode true to return association code, false for name
     * @param forTreeTraversal the for tree traversal
     *
     * @return the concept associations
     */

     /*
	public Vector<ConceptAssociation> getConceptAssociations(
			String codingSchemeName,
			String vers,
			String ltag,
			String code,
			boolean direction,
			boolean returncode,
			boolean forTreeTraversal)
			{
		return getConceptAssociations(
				codingSchemeName,
				vers,
				ltag,
				code,
				direction,
				returncode,
				forTreeTraversal,
				null);
			}

	*/

    /**
     * Gets the concept associations.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction true if concept is source of association, false if target
     *
     * @return the concept associations
     *
     * @since DLBExt 1.0
     */
    public Vector<ConceptAssociation> getConceptAssociations(
    		String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		boolean direction
    )
    {
    	Vector<ConceptAssociation> v = getConceptAssociations(
    			codingSchemeName,
    			vers,
    			ltag,
    			code,
    			direction,
    			//false);
    			true); // returncode?
    	return v;
    }


    // 031007
    /**
     * Gets the concept associations.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction true if concept is source of association, false if target
     * @param relation the name of the relation container
     *
     * @return the concept associations
     */
    public Vector<ConceptAssociation> getConceptAssociations(
    		String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		boolean direction,
    		String relation)
    {
		/*
    	Vector<ConceptAssociation> v = getConceptAssociations(
    			codingSchemeName,
    			vers,
    			ltag,
    			code,
    			direction,
    			true, //080307
    			false,
    			relation);
    	return v;
    	*/
    	Vector asso_names = null;
	    return getAssociations(codingSchemeName,
							   vers,
							   ltag,
							   code,
							   direction,
							   relation,
							   asso_names);
    }


    /**
     * Gets the concept associations.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction true if concept is source of association, false if target
     * @param returncode true to return association code, false for name
     *
     * @return the concept associations
     *
     * @since DLBExt 1.0
     */
    public Vector<ConceptAssociation> getConceptAssociations(
    		String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		boolean direction,
    		boolean returncode)
    {
		/*
    	boolean forTreeTraversal = false;
    	return getConceptAssociations(
    			codingSchemeName,
    			vers,
    			ltag,
    			code,
    			direction,
    			returncode,
    			forTreeTraversal
    	);
    	*/

	  	String relationContainer = null;
	  	Vector asso_names = null;
	    Vector v = getAssociations(codingSchemeName, vers, ltag, code, direction, relationContainer, asso_names);
	    if (!returncode)
	    {
			for (int i=0; i<v.size(); i++)
			{
				ConceptAssociation asso = (ConceptAssociation) v.elementAt(i);
				asso.setValue(getConceptName(codingSchemeName, vers, asso.getValue()));
			}
		}
		return v;
    }



    // 090506
    /**
     * Gets the concept associations.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction true if concept is source of association, false if target
     * @param returncode true to return association code, false for name
     * @param forTreeTraversal the for tree traversal
     * @param relation name of the relation container
     *
     * @return the concept associations
     */

/*
    public Vector<ConceptAssociation> getConceptAssociations(
    		String codingSchemeName,
    		String vers,
    		String ltag,
    		String code,
    		boolean direction,
    		boolean returncode,
    		boolean forTreeTraversal,
    		String relation)
    {
    	if (relation != null)
    	{
    		if (relation.compareToIgnoreCase(ROLE_RELATION) == 0 && !hasRelation(codingSchemeName, ROLE_RELATION))
    		{
    			return new Vector<ConceptAssociation>();
    		}
    	}

    	String relationName = relation;
    	if (relation != null)
    	{
    		if (!hasRelation(codingSchemeName, vers, ltag, relation))
    		{
    			relationName = null;//"Relations";//null;
    		}
    	}

    	CodingSchemeVersionOrTag versionOrTag
    	= createCodingSchemeVersionOrTag(vers, ltag);

    	//boolean broader_to_narrower = direction; // traverse_down
    	//if (!direction) broader_to_narrower = false;

    	boolean traverse_dir_isa = direction;
    	boolean traverse_dir_role = direction;

    	if (forTreeTraversal)
    	{
    		traverse_dir_isa = direction;
    		traverse_dir_role = !direction;
    	}

    	boolean traverse_dir;
    	Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
    	try {
    		CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, versionOrTag);
    		if (scheme == null)
    		{
    			getLogger().warn("WARNING: Unable to resolve coding scheme,");
    			return null;
    		}

    		// IS_A
    		//String associationname = DLBWrapper.IS_A;
    		String is_a = getHasSubtype(codingSchemeName);
    		String associationname = is_a;
    		traverse_dir = traverse_dir_isa;

    		Association asso = null;
    		if (traverse_dir)
    		{
    			asso = lbConvMethods.getChildrenOf(code, relationName, associationname,
    					codingSchemeName, createCodingSchemeVersionOrTag(vers, ltag), false, null);
    		}
    		else
    		{
    			asso = lbConvMethods.getParentsOf(code, relationName, associationname,
     					codingSchemeName, createCodingSchemeVersionOrTag(vers, ltag), false, null);
    		}

    		if (asso != null)
    		{
    			AssociatedConceptList acl =	asso.getAssociatedConcepts();
    			if (acl != null)
    			{
    				AssociatedConcept[] acs = acl.getAssociatedConcept();
    				if (acs != null)
    				{
    					for (int k=0; k<acs.length; k++)
    					{
    						if (acs[k] != null)
    						{
    							gov.nih.nci.lexbig.ext.ConceptAssociation concept_asso = convertConceptAssociation(codingSchemeName,
    									vers,
    									associationname,
    									acs[k],
    									returncode);
    							v.add(concept_asso);

    						}
    					}
    				}
    			}
    		}

    		// Other associations
    		Association[] associations = null;
    		traverse_dir = traverse_dir_role;
    		if (traverse_dir)
    		{
    			associations = getChildrenOf(code, relationName, codingSchemeName, vers, ltag, false);
    		}
    		else
    		{
    			associations = getParentsOf(code, relationName, codingSchemeName, vers, ltag, false);
    		}

    		if (associations != null)
    		{
    			for (int i=0; i<associations.length; i++)
    			{
    				asso = associations[i];
    				associationname = asso.getAssociationName();
    				//if (associationname.compareTo(DLBWrapper.IS_A) != 0)

    				if (is_a != null)
    				{
    					if (associationname.compareTo(is_a) != 0)
    					{
    						AssociatedConceptList acl =	asso.getAssociatedConcepts();
    						if (acl != null)
    						{
    							AssociatedConcept[] acs = acl.getAssociatedConcept();
    							if (acs != null)
    							{
    								for (int k=0; k<acs.length; k++)
    								{
    									if (acs[k] != null)
    									{
    										gov.nih.nci.lexbig.ext.ConceptAssociation concept_asso = convertConceptAssociation(codingSchemeName,
    												vers,
    												associationname,
    												acs[k],
    												returncode);
    										v.add(concept_asso);

    									}
    								}
    							}
    						}
    					}
    				}
    			}
    		}
    	} catch (Exception e) {
    		//e.printStackTrace();
    		getLogger().error("Error encountered when calling getConceptAssociations.");
    	}
    	return v;
   	}
*/


	//030707
    /**
	 * Gets the supported role names.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return the supported role names
	 */
	public Vector<String> getSupportedRoleNames(String codingSchemeName)
	{
		if (!hasRelation(codingSchemeName, ROLE_RELATION)) return new Vector<String>();

		return getAssociationNamesByRelation(codingSchemeName, version, ROLE_RELATION);
    }

    /**
     * Gets the supported role names.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     *
     * @return the supported role names
     */
    public Vector<String> getSupportedRoleNames(String codingSchemeName, String vers)
	{
		if (!hasRelation(codingSchemeName, vers, null, ROLE_RELATION)) return new Vector<String>();
		return getAssociationNamesByRelation(codingSchemeName, vers, ROLE_RELATION);
    }


	/**
	 * Gets the roles.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return the roles
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<Role> getRoles(String codingSchemeName,
						   String vers,
						   String ltag,
					  	   String code)
	{
		Vector<String> associations = getSupportedRoleNames(codingSchemeName, vers);
		if (associations == null || associations.size() == 0) return new Vector<Role>();

	    Vector<Role> v = new Vector<Role>();
        ResolvedConceptReference rcr = resolveConceptReference(codingSchemeName, vers,
        		ltag, code);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getSourceOf();

		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingSchemeName);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (associations.contains(assoname))
			{
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), vers, ltag, cr.getConceptCode());
					v.add(new Role(assoname, getConceptName(codingSchemeName, codedentry_ref)));
				}
		    }
		}
		return v;
	}


    /*
     * Enumerate Target Concepts for Relation and Source
     */

	/**
	 * Gets the inverse roles.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param code the code
	 *
	 * @return the inverse roles
	 *
	 * @since DLBExt 1.0
	 */
	public Vector getInverseRoles(String codingSchemeName,
			String code)
	{
		return getInverseRoles(codingSchemeName, null, null, code);
	}

	/**
	 * Gets the inverse roles.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return the inverse roles
	 */
	public Vector<Role> getInverseRoles(String codingSchemeName,
			String vers,
			String ltag,
			String code)
	{


		Vector<ConceptAssociation> w = getConceptAssociations(
				codingSchemeName,
				vers,
				ltag,
				code,
				false);
		return conceptAssociations2Roles(w);
//		return nameAndValues2Roles(w);
	}


	/**
	 * Gets the inverse concept associations.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param code the code
	 *
	 * @return the inverse concept associations
	 */
	public Vector<ConceptAssociation> getInverseConceptAssociations(String codingSchemeName,
										  String code)
    {
         return getInverseConceptAssociations(codingSchemeName, null, null, code);
    }



	/**
	 * Gets the inverse concept associations.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return the inverse concept associations
	 *
	 * @since DLBExt 1.0
	 */
/*
	public Vector<ConceptAssociation> getInverseConceptAssociations(String codingSchemeName,
										  String vers,
										  String ltag,
										  String code)
	{
		//Vector<String> associations = getSupportedAssociationNames(codingScheme, version, ROOT_ASSOCIATION);
		//Vector associations = getSupportedAssociationNames(codingScheme, version, ROOT_ASSOCIATION);
		//getSupportedConceptAssociationNames();
		//KLO 082307
		Vector associations = getAssociationNamesByRelation(codingSchemeName, vers, ASSOCIATION_RELATION);

		if (associations == null || associations.size() == 0) return new Vector<ConceptAssociation>();

	    Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
        ResolvedConceptReference rcr = resolveConceptReference(codingSchemeName, vers,
        		ltag, code);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getInverseConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getTargetOf();
		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingSchemeName);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (associations.contains(assoname))
			{
				//if (!assoname.equals(IS_A))
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), vers, ltag, cr.getConceptCode());
					v.add(new ConceptAssociation(assoname, getConceptName(codingSchemeName, codedentry_ref)));
				}
		    }
		}

		return v;
	}
*/


    // GForge 13294, KLO 032808
	public Vector<ConceptAssociation> getInverseConceptAssociations(String codingSchemeName,
										  String vers,
										  String ltag,
										  String code)
	{
		LexBIGService lbSvc = getLexBIGService();
        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
        CodingScheme cs = null;
        try {
        	cs = lbSvc.resolveCodingScheme(codingSchemeName, versionOrTag);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
        if (cs == null) return null;
        Relations[] relations = cs.getRelations();
        boolean assocContainerDefined = false;
        Vector asso_names = new Vector();
        for (int i=0; i<relations.length; i++)
        {
			Relations relation = (Relations) relations[i];
			String dc = relation.getDc();
			if (dc.compareTo("associations") == 0)
			{
				assocContainerDefined = true;
				//System.out.println("Relation: " + dc);
				org.LexGrid.relations.Association[] associations = relation.getAssociation();
				for (int j=0; j<associations.length; j++)
				{
					org.LexGrid.relations.Association association = (org.LexGrid.relations.Association) associations[j];
					//System.out.println("\tAssociation: " +  association.getAssociation());
					asso_names.add(association.getAssociation());
				}
				break;
			}
		}
		if (!assocContainerDefined) return new Vector();


		Vector<ConceptAssociation> v = new Vector();
		ResolvedConceptReferenceList matches = null;
		try {
		   matches = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, null)
				.resolveAsList(
					ConvenienceMethods.createConceptReference(code, codingSchemeName),
					false, true, 1, 1, new LocalNameList(), null, null, 1024);
		} catch (Exception ex) {
			//ex.printStackTrace();
			return null;
		}

		if (matches.getResolvedConceptReferenceCount() > 0) {
			Enumeration<ResolvedConceptReference> refEnum = matches.enumerateResolvedConceptReference();

			while (refEnum.hasMoreElements()) {
				ResolvedConceptReference ref = refEnum.nextElement();
				if (ref != null)
				{
					AssociationList targetof = ref.getTargetOf();
					org.LexGrid.LexBIG.DataModel.Core.Association[] associations = targetof.getAssociation();
					if (associations != null)
					{
						for (int i = 0; i < associations.length; i++) {
							org.LexGrid.LexBIG.DataModel.Core.Association assoc = associations[i];
							if (asso_names.contains(assoc.getAssociationName()))
							{
								AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
								for (int j = 0; j < acl.length; j++) {
									AssociatedConcept ac = acl[j];
									//EntityDescription ed = ac.getEntityDescription();
									//v.add(new ConceptAssociation(assoc.getAssociationName(), ed.getContent()));
									v.add(new ConceptAssociation(assoc.getAssociationName(), ac.getConceptCode()));
								}
							}
						}
					}
			    }
			}
		}
		return v;
	}



//
//    /**
//     * Converts NameAndValue to Role.
//     *
//     * @param v the vector <NameAndValue>
//     *
//     * @return the vector< Role>
//     */
//    private Vector<Role> nameAndValues2Roles(Vector<NameAndValue> v)
//    {
//
//		if (v == null) return new Vector<Role>();
//		Vector<Role> u = new Vector<Role>();
//		for (int i=0; i<v.size(); i++)
//		{
//			NameAndValue nv = (NameAndValue) v.elementAt(i);
//		    u.add(new Role(nv.getName(), nv.getContent()));
//		}
//		return u;
//	}
//


    /*
     * Enumerate Relation Types for Concept
     */

	/**
     * Gets the relation types for concept.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the coding scheme version
     * @param ltag the coding scheme tag
     * @param code the concept code
     *
     * @return the relation types for concept
     *
     * @since DLBExt 1.0
     */
	public HashSet<String> getRelationTypesForConcept(String codingSchemeName,
										  String vers,
										  String ltag,
										  String code)
	{
	    HashSet<String> hset = new HashSet<String>();
        ResolvedConceptReference rcr = resolveConceptReference(codingSchemeName, vers,
        		ltag, code);

        AssociationList al = rcr.getSourceOf();
		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (!hset.contains(assoname))
			{
                hset.add( assoname);
			}
		}
		return hset;
	}


///////////////////////
//  Search
///////////////////////



	/**
 * Find coded entries with designation matching.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param matchText - the text to search for
 * @param preferredOnly - parameter not used
 * @param sortByProperty_vec - parameter not used
 * @param source_vec - list of properties to search
 * @param matchAlgorithm - "exactMatch", "contains", etc
 * @param maxToReturn - max number of results to return
 *
 * @return the vector< coded entry>
 *
 * @since DLBExt 1.0
 * @deprecated - use findCodedEntriesWithDesignationMatching(String,String,String,Vector,String,int)
 */@Deprecated
	public Vector<CodedEntry> findCodedEntriesWithDesignationMatching(
		                                        String codingSchemeName,
	                                            String vers,
	                                            String matchText,
	                                            @SuppressWarnings("unused")boolean preferredOnly,
	                                            @SuppressWarnings("unused")Vector sortByProperty_vec,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	                                            throws LBParameterException
	{

//	    CodedNodeSet cns_1;
//	    CodedNodeSet cns_2;
	    CodedNodeSet cns;
        Vector<CodedEntry> v = new Vector<CodedEntry>();
        try {
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, null);
			cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);//, false);
//			cns_1 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			cns_2 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			if (cns_1 == null || cns_2 == null)
			if (cns == null)
			{
				getLogger().warn("WARNING: DLBWrapper getCodingSchemeConcepts returns null");
				return v;
			}

			//Vector property_vec = new Vector();

//            if (hasProperty(codingScheme, "CONCEPT_NAME"))
//            {
//				property_vec.add("CONCEPT_NAME");
//			}
//
//            if (hasProperty(codingScheme, "textualPresentation"))
//            {
//				property_vec.add("textualPresentation");
//			}

			//property_vec.add("textualPresentation");

			//LocalNameList propertyList = vector2LocalNameList(property_vec);
			LocalNameList sourceList = new LocalNameList();

//			LocalNameList contextList = null;
//			NameAndValueList qualifierList = null;

			//cns.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
			cns = cns.restrictToMatchingDesignations(matchText, SearchDesignationOption.PREFERRED_ONLY, matchAlgorithm, null);

//			cns_1 = cns_1.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
//			cns_2	= cns_2.restrictToMatchingDesignations(matchText, preferredOnly, matchAlgorithm, null);
//			cns = cns_1.union(cns_2);


			if (source_vec != null && source_vec.size() > 0)
			{
				// 050507
			    sourceList = vector2LocalNameList(source_vec);
			    //cns = cns.restrictToProperties(propertyList, null, sourceList, null, null);
			    CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			    //cns = cns.restrictToProperties(null, null, sourceList, null, null);
			    cns = cns.restrictToProperties(null, types, sourceList, null, null);
			}


			LocalNameList restrictToProperties = new LocalNameList();//vector2LocalNameList(restrictToProperties_vec);

//			LocalNameList sortByProperty = vector2LocalNameList(sortByProperty_vec);
			//LocalNameList restrictToProperties = vector2LocalNameList(restrictToProperties_vec);

			// Sort by search engine recommendation & code .
			SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

			//ResolvedConceptReferenceList list = cns.resolveToList(sortByProperty,
/*
			ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
																  restrictToProperties,
																  null,
																  maxToReturn);
			return v;

*/
////[#14262] Does not throw exception when exceed search limit - testMetaSearch_overLimit
			ResolvedConceptReferenceList list = null;
			/*
			int max = getSystemMaxResultSize();
			if (maxToReturn < max) max = maxToReturn;

			try {
				list = cns.resolveToList(sortCriteria,
									  restrictToProperties,
									  null,
									  max);
			*/
			try {
				list = cns.resolveToList(sortCriteria,
									  restrictToProperties,
									  null,
									  maxToReturn);
			} catch (Exception ex) {
				throw new LBParameterException(ex.getMessage());
			}

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper resolveToList returns null");
			}
            else
            {
				ResolvedConceptReference[] rcrArray = list.getResolvedConceptReference();
				if (rcrArray == null)
				{
					getLogger().warn("WARNING: DLBWrapper getResolvedConceptReference returns null");
				}
				else
				{
					for (int i=0; i<rcrArray.length; i++)
					{
						ResolvedConceptReference rcr = (ResolvedConceptReference) rcrArray[i];
						v.add(rcr.getReferencedEntry());
					}
			    }
		    }

	    } catch (Exception e) {
			e.printStackTrace();
			return v;
	    }
		return v;
	}


	/**
 * Find coded entries with designation matching.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param matchText - the text to search for
 * @param source_vec - list of properties to search
 * @param matchAlgorithm - "exactMatch", "contains", etc
 * @param maxToReturn - max number of results to return
 *
 * @return the vector< coded entry>
 *
 * @since DLBExt 1.0
 */
	public Vector<CodedEntry> findCodedEntriesWithDesignationMatching(
		                                        String codingSchemeName,
	                                            String vers,
	                                            String matchText,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)

	                                            throws LBParameterException
	{

//	    CodedNodeSet cns_1;
//	    CodedNodeSet cns_2;
	    CodedNodeSet cns;
        Vector<CodedEntry> v = new Vector<CodedEntry>();
        try {
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, null);
			cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);//, false);
//			cns_1 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			cns_2 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			if (cns_1 == null || cns_2 == null)
			if (cns == null)
			{
				getLogger().warn("WARNING: DLBWrapper getCodingSchemeConcepts returns null");
				return v;
			}

			//Vector property_vec = new Vector();

//            if (hasProperty(codingScheme, "CONCEPT_NAME"))
//            {
//				property_vec.add("CONCEPT_NAME");
//			}
//
//            if (hasProperty(codingScheme, "textualPresentation"))
//            {
//				property_vec.add("textualPresentation");
//			}

			//property_vec.add("textualPresentation");

			//LocalNameList propertyList = vector2LocalNameList(property_vec);
			LocalNameList sourceList = new LocalNameList();

//			LocalNameList contextList = null;
//			NameAndValueList qualifierList = null;

			//cns.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
			//cns = cns.restrictToMatchingDesignations(matchText, SearchDesignationOption.PREFERRED_ONLY, matchAlgorithm, null);
			cns = cns.restrictToMatchingDesignations(matchText, SearchDesignationOption.ALL, matchAlgorithm, null);

//			cns_1 = cns_1.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
//			cns_2	= cns_2.restrictToMatchingDesignations(matchText, preferredOnly, matchAlgorithm, null);
//			cns = cns_1.union(cns_2);


			if (source_vec != null && source_vec.size() > 0)
			{
				// 050507
			    sourceList = vector2LocalNameList(source_vec);
			    //cns = cns.restrictToProperties(propertyList, null, sourceList, null, null);
			    CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			    //cns = cns.restrictToProperties(null, null, sourceList, null, null);
			    cns = cns.restrictToProperties(null, types, sourceList, null, null);
			}


			LocalNameList restrictToProperties = new LocalNameList();//vector2LocalNameList(restrictToProperties_vec);

//			LocalNameList sortByProperty = vector2LocalNameList(sortByProperty_vec);
			//LocalNameList restrictToProperties = vector2LocalNameList(restrictToProperties_vec);

			// Sort by search engine recommendation & code .
			SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

			//ResolvedConceptReferenceList list = cns.resolveToList(sortByProperty,

////[#14262] Does not throw exception when exceed search limit - testMetaSearch_overLimit
			ResolvedConceptReferenceList list = null;
			/*
			int max = getSystemMaxResultSize();
			if (maxToReturn < max) max = maxToReturn;

			try {
			   list = cns.resolveToList(sortCriteria,
									  restrictToProperties,
									  null,
									  max);
			*/

			try {
			   list = cns.resolveToList(sortCriteria,
									  restrictToProperties,
									  null,
									  maxToReturn);
			} catch (Exception ex) {
				//ex.printStackTrace();
				throw new LBParameterException(ex.getMessage());

			}

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper resolveToList returns null");
			}
			else
			{
				ResolvedConceptReference[] rcrArray = list.getResolvedConceptReference();
				if (rcrArray == null)
				{
					getLogger().warn("WARNING: DLBWrapper getResolvedConceptReference returns null");
				}

				for (int i=0; i<rcrArray.length; i++)
				{
					ResolvedConceptReference rcr = (ResolvedConceptReference) rcrArray[i];
					v.add(rcr.getReferencedEntry());
				}
	    	}

	    } catch (Exception e) {
			e.printStackTrace();
			return v;
	    }
		return v;
	}

//////////////////////////////////////////////////////////////////////////////////////////
	/**
 * Find coded entries with association.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param associationName the association name
 * @param matchAlgorithm the match algorithm
 * @param maxToReturn the max to return
 *
 * @return the vector< coded entry>
 *
 * @since DLBExt 1.0
 */
	public Vector<CodedEntry> findCodedEntriesWithAssociation(String codingSchemeName,
	                                              String vers,
	                                              String associationName,
	                                              String matchAlgorithm,
	                                              int maxToReturn)
	{ //throws ParameterException, InternalErrorException {
	    return findCodedEntriesWithAssociationMatching(codingSchemeName,
	    		                                vers,
	                                            "*",
												associationName,
												matchAlgorithm,
												maxToReturn);
	}

//////////////////////////////////////////////////////////////////////////////////////


	/**
 * Find coded entries with association matching.
 *
 * @param codingSchemeName a coding scheme name
 * @param vers a coding scheme version
 * @param matchText the match text
 * @param associationName the association name
 * @param matchAlgorithm the match algorithm
 * @param maxToReturn the max to return
 *
 * @return the vector< coded entry>
 *
 * @since DLBExt 1.0
 */
	public Vector<CodedEntry> findCodedEntriesWithAssociationMatching(String codingSchemeName,
	                                            String vers,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn)
	{
	     return findCodedEntriesWithAssociationMatching(codingSchemeName,
	    		 vers,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												RESTRICT_TO_TARGET);
    }

	/**
	 * Find coded entries with association matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers a coding scheme version
	 * @param matchText the match text
	 * @param associationName the association name
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 * @param restriction_direction RESTRICT_TO_TARGET or RESTRICT_TO_SOURCE
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findCodedEntriesWithAssociationMatching(String codingSchemeName,
	                                            String vers,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction)
	{
		Vector restricted_property_vec = null;
		return findCodedEntriesWithAssociationMatching(codingSchemeName,
				vers,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												restriction_direction,
												restricted_property_vec);
	}


	/**
	 * Find coded entries with association matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers a coding scheme version
	 * @param matchText the match text
	 * @param associationName the association name
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 * @param restriction_direction RESTRICT_TO_TARGET or RESTRICT_TO_SOURCE
	 * @param restricted_property_vec list of properties to search
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findCodedEntriesWithAssociationMatching(String codingSchemeName,
	                                            String vers,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction,
												Vector restricted_property_vec)
	{
	     return findCodedEntriesWithAssociationMatching(codingSchemeName,
	    		 vers,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												restriction_direction,
												restricted_property_vec,
												null);
	}



	/**
	 * Find coded entries with association matching.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers a coding scheme version
	 * @param matchText the match text
	 * @param associationName the association name
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 * @param restriction_direction RESTRICT_TO_TARGET or RESTRICT_TO_SOURCE
	 * @param restricted_property_vec list of properties to search
	 * @param relationName the relation name
	 *
	 * @return the vector< coded entry>
	 */
	public Vector<CodedEntry> findCodedEntriesWithAssociationMatching(String codingSchemeName,
	                                            String vers,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction,
												Vector restricted_property_vec,
												String relationName)
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
        CodedNodeSet cns = null;
        CodedNodeGraph cng = null;

        CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
        versionOrTag.setVersion(vers);

        try {
			cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);//, false);
			if (cns == null) return v;
//			boolean preferredOnly = false;
			String language = null;
			//cns = cns.restrictToMatchingDesignations(matchText, preferredOnly, matchAlgorithm, language);

			cns = cns.restrictToMatchingDesignations(matchText,
			               CodedNodeSet.SearchDesignationOption.ALL,
			               matchAlgorithm, language);

            LocalNameList propertyList = null;
            if (restricted_property_vec != null)
            {
				propertyList = vector2LocalNameList(restricted_property_vec);
				cns = cns.restrictToProperties(propertyList, null);
			}

            cng = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, relationName);

			NameAndValueList asso_list =
				createNameAndValueList(
					new String[] {associationName}, null);

			//NameAndValueList qualifier_list = createNameAndValueList(new String[] {"someValuesFrom"}, null);

            cng = cng.restrictToAssociations(asso_list, null);
            //cng = cng.restrictToAssociations(asso_list, qualifier_list);

            ResolvedConceptReferenceList rcrl = null;

            if (restriction_direction == RESTRICT_TO_TARGET)
            {
				cng = cng.restrictToTargetCodes(cns);
			    rcrl = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				   1, 1, null, null, null, maxToReturn);
			}
			else if (restriction_direction == RESTRICT_TO_SOURCE)
			{
				cng = cng.restrictToSourceCodes(cns);
			    rcrl = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				   1, 1, null, null, null, maxToReturn);
			}

 			ResolvedConceptReference[] rcrArray = rcrl.getResolvedConceptReference();
 			CodedEntry ce = null;

			for (int i=0; i<rcrArray.length; i++)
			{
				if (restriction_direction == RESTRICT_TO_TARGET)
				{
				    ce = rcrArray[i].getReferencedEntry();
				    if (ce != null)
				    {
						v.add(ce);
					}
				}
				else if (restriction_direction == RESTRICT_TO_SOURCE)
				{
                    AssociationList al = rcrArray[i].getSourceOf();
					Association[] assos = al.getAssociation();
					if (assos != null)
					{
						for (int j=0; j<assos.length; j++)
						{
							Association asso = (Association) assos[j];
                            AssociatedConceptList acl = asso.getAssociatedConcepts();
                            AssociatedConcept[] ac_array = acl.getAssociatedConcept();
                            for (int k=0; k<ac_array.length; k++)
                            {
								AssociatedConcept ac = (AssociatedConcept) ac_array[k];
								String code = ac.getConceptCode();
								ce = getCodedEntry(codingSchemeName, vers, null, code);
								if (ce != null) v.add(ce);
						    }
						}
					}
					else
					{
						getLogger().warn("assos == null");
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return v;
	}

    /**
     * Gets the CodedNodeSet size.
     *
     * @param cns the CodedNodeSet
     *
     * @return the CodedNodeSet size
     */
    protected int getCNSSize(CodedNodeSet cns)
    {
         try {
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(null, null, null, 1000);
			 ResolvedConceptReference[] rcrArray = rcrl.getResolvedConceptReference();

			 return rcrArray.length;
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return -1;
	}


    /**
     * Gets the CodedNodeGraph size.
     *
     * @param cng the CodedNodeGraph
     *
     * @return the CodedNodeGraph size
     */
    protected int getCNGSize(CodedNodeGraph cng)
    {
		 boolean resolveForward = true;
		 boolean resolveBackward = false;
		 int resolveAssociationDepth = 1;
		 int maxToReturn = 1000;
         try {
             CodedNodeSet cns = cng.toNodeList(null, resolveForward, resolveBackward, resolveAssociationDepth, maxToReturn);

			 return getCNSSize(cns);
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return -1;

	}

///////////////////////
//  Tree
///////////////////////


	/**
	 * Concepts tree data from the server.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code parameter not used
	 * @param relationName - name of relations container to search
	 * @param properties - parameter not used
	 * @param sourcecodes - parameter not used
	 * @param associations - the association names to search on
	 * @param direction - parameter not used
	 * @param depth - parameter not used

	 *
	 * @return The CodedNodeGraph of the tree, right now will always return null
	 *
	 * @since DLBExt 1.0
	 * @deprecated
	 */@Deprecated
    protected CodedNodeGraph getTreeFromServer(String codingSchemeName,
    		String vers,
    		String ltag,
    		@SuppressWarnings("unused")String code, // root concept code

    		String relationName,

    		@SuppressWarnings("unused")Vector properties,
    		@SuppressWarnings("unused")Vector sourcecodes,
    		@SuppressWarnings("unused")Vector associations,

    		@SuppressWarnings("unused")boolean direction,
    		@SuppressWarnings("unused")int depth)

    {
	     CodedNodeGraph cng = null;
	     try {
			 CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
			 cng = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, relationName);

            String[] asso_ary = new String[associations.size()];
            for (int i=0; i<associations.size(); i++)
            {
				asso_ary[i] = (String) associations.elementAt(i);
			}

            NameAndValueList restrictToAssociations = createNameAndValueList(asso_ary, null);

			NameAndValueList associationQualifiers = new NameAndValueList();

			cng = cng.restrictToAssociations(restrictToAssociations,
											 associationQualifiers);


			//ConceptReference focusConcept = createConceptReference(codingScheme, code);

//			ConceptReference focusConcept = createConceptReference(code, codingScheme);


			// to be implemented:
			Boolean ret = null;

			 //Boolean ret = cng.setFocus(focusConcept);

			 if (ret == null || ret == Boolean.FALSE) return null;

		 } catch (Exception e) {
			e.printStackTrace();
			return null;
		 }

         return cng;
    }


	/**
	 * Gets the number of instances of key within the HashMap
	 *
	 * @param HashMap the hash map
	 * @param key the key
	 *
	 * @return the child count
	 *
	 * @since DLBExt 1.0
	 */
	protected int getChildCount(HashMap HashMap, String key)
	{
		if (HashMap.containsKey(key))
		{
			Vector v = (Vector) HashMap.get(key);
			return v.size();
		}
	    return 0;
	}




	/////////////////////////////////////////////////////////////////////////////////////////



	/**
	 * Convert an NCI concept code to a concept name.  Note that this could be simplified if everyone
	 * agrees that the entityDescription node for the concept code will always be the NC_Name.
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 * @param code Code to be converted
	 * @return name or null if code not located
	 *
	 * @deprecated - evsapi4 does not support name as an identifier
	 */@Deprecated
	public String codeToName(String codingSchemeName, String code)
	{
        return codeToName(codingSchemeName, null, null, code);
	}


	/**
	 * Code to name.
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param code the concept code
	 *
	 * @return the name
	 *
	 * @deprecated - evsapi4 does not support name as an identifier
	 * @since DLBExt 1.0
	 */@Deprecated
	public String codeToName(String codingSchemeName, String vers, String code) {
		return codeToName(codingSchemeName, vers, null, code);
	}

	/**
	 * Code to name.
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @deprecated - evsapi4 does not support name as an identifier
	 * @return the string
	 */@Deprecated
	public String codeToName(String codingSchemeName, String vers, String ltag, String code) {
		try {
//			String version_param = vers;
//			if (version_param == null) version_param = version;
//
//			String tag_param = ltag;
//			if (tag_param == null) tag_param = tag;

			CodingSchemeVersionOrTag vtag = createCodingSchemeVersionOrTag(vers, ltag);
			String name = new ConvenienceMethods(lbSvc).codeToName(code, codingSchemeName, vtag);
			return name;
		} catch (Exception ex) {
			getLogger().warn("Exception." + ex);
		}
		return null;
    }


	/**
	 * Gets the coded entry by code.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param code the concept code
	 *
	 * @return the coded entry by code
	 *
	 * @since DLBExt 1.0
	 */
	public CodedEntry getCodedEntryByCode(String codingSchemeName, String vers, String ltag, String code)
	{
//        ResolvedConceptReference rcr = null;
        try {
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
			ConceptReferenceList crefs =
				createConceptReferenceList(
					new String[] {code}, codingSchemeName);

			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);
			cns = cns.restrictToCodes(crefs);
			ResolvedConceptReferenceList matches = cns.resolveToList(null, null, null, 1);


			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				CodedEntry entry = ref.getReferencedEntry();
				return entry;
			}
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return null;
	}


	/**
	 * Gets the concept name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param code the concept code
	 *
	 * @return the concept name
	 *
	 * @since DLBExt 1.0
	 */
	public String getConceptName(String codingSchemeName, String vers, String code)
	{
//		return codeToName(codingSchemeName, vers, code);
		return findDescription(codingSchemeName, vers, code);
	}

	/**
	 * Gets the concept name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 *
	 * @return the concept name
	 *
	 * @since DLBExt 1.0
	 */
	public String getConceptName(String codingSchemeName, String code)
	{
		return findDescription(codingSchemeName,code);
	}


	/**
	 * Return the concept code that corresponds to the name.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param name concept name to look up
	 *
	 * @return corresponding code or null if problems
	 *
	 * @deprecated - evsapi4 does not support name as an identifier
	 */@Deprecated
	public String nameToCode(String codingSchemeName, String name)
	{
        return nameToCode(codingSchemeName, null, null, name);
	}


	/**
	 * Name to code.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param ltag the coding scheme tag
	 * @param name concept name
	 *
	 * @return the string
	 *
	 * @since DLBExt 1.0
	 * @deprecated - evsapi4 does not support name as an identifier
	 */@Deprecated
	public String nameToCode(String codingSchemeName, String vers, String ltag, String name) {
		try {
//			String version_param = vers;
//			if (version_param == null) version_param = version;
//
//			String tag_param = ltag;
//			if (tag_param == null) tag_param = tag;

			CodingSchemeVersionOrTag vtag = createCodingSchemeVersionOrTag(vers, ltag);

			String code = new ConvenienceMethods(lbSvc).nameToCode(name, codingSchemeName, vtag);
			return code;
		} catch (Exception ex) {
			getLogger().warn("Exception." + ex);
		}
		return null;
    }



	/**
	 * Find coded entries with property matching.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param matchText - term to search for
	 * @param property_vec - list of properties to search
	 * @param restrictToProperties_vec - list of properties to return
	 * @param matchAlgorithm the match algorithm
	 * @param maxToReturn the max to return
	 *
	 * @return the vector< coded entry>
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<CodedEntry> findCodedEntriesWithPropertyMatching(String codingSchemeName,
	                                                   String vers,
	                                                   String matchText,
	                                                   Vector property_vec,
	                                                   Vector restrictToProperties_vec,
	                                                   String matchAlgorithm,
	                                                   int maxToReturn)
    {
	    return findCodedEntriesWithPropertyMatching(codingSchemeName,
	    		                                       vers,
	                                                   matchText,
	                                                   property_vec,
	                                                   restrictToProperties_vec,
	                                                   matchAlgorithm,
	                                                   maxToReturn,
	                                                   null);

    }



	/**
	 * Find coded entries with property matching.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param matchText - term to search for
	 * @param property_vec - list of properties to search
	 * @param restrictToProperties_vec - list of properties to return
	 * @param matchAlgorithm - "exactMatch", "contains", etc.
	 * @param maxToReturn - max number of results to return
	 * @param language the language
	 *
	 * @return the vector< coded entry>
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<CodedEntry> findCodedEntriesWithPropertyMatching(String codingSchemeName,
	                                                   String vers,
	                                                   String matchText,
	                                                   Vector property_vec,
	                                                   Vector restrictToProperties_vec,
	                                                   String matchAlgorithm,
	                                                   int maxToReturn,
	                                                   String language)
	{

//		for (int i=0; i<property_vec.size(); i++)
//		{
//			String s = (String)property_vec.elementAt(i);
//		}

//         String tag = null;
//         boolean activeOnly = true;
         Vector<CodedEntry> v = new Vector<CodedEntry>();
		 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, null);
	 	 try {
	         //CodedNodeSet cns = new CodedNodeSetImpl(codingScheme, tagOrVersion, true);
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, activeOnly);

//	         boolean preferredOnly = false;
             LocalNameList propertyList = new LocalNameList();
	         if (property_vec != null || property_vec.size() > 0)
             {
 	             propertyList = vector2LocalNameList(property_vec);
			 }
			 else
			 {
				 propertyList = vector2LocalNameList(getSupportedPropertyNames(getCodingScheme(codingSchemeName, vers, null)));
			 }
             cns = cns.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, language);

//             LocalNameList sortByProperty = null;
             LocalNameList restrictToProperties = vector2LocalNameList(restrictToProperties_vec);

			 // Sort by search engine recommendation & code ...
			 SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

             ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, restrictToProperties,
                  null, maxToReturn);

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			 ResolvedConceptReference rcr = null;
			 for (int i=0; i<rcrs.length; i++)
			 {
				 rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
			 return v;

		 } catch (Exception e) {
			 e.printStackTrace();

		 }
		 return v;
	}



	/**
	 * Returns the entity description for a given code, or the code
	 * if no description can be located.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param code the code
	 *
	 * @return String
	 *
	 * @since DLBExt 1.0
	 */
	protected String findDescription(String codingSchemeName, String code) {
		// Perform the query .
//		try {
//			ConceptReferenceList crefs =
//				createConceptReferenceList(
//					new String[] {code}, codingSchemeName);
//
//			CodedNodeSet nodes = lbSvc.getCodingSchemeConcepts(codingSchemeName, createCodingSchemeVersionOrTag(null, null));
//			nodes = nodes.restrictToCodes(crefs);
//
//			// Restrict the properties ...
//			ResolvedConceptReferenceList matches =
//				nodes.resolveToList(null, new LocalNameList(), null, 1);
//
//			// Analyze the result ...
//			if (matches.getResolvedConceptReferenceCount() > 0) {
//				ResolvedConceptReference ref =
//					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
//				return ref.getReferencedEntry().getEntityDescription().getContent();
//			}
//	    } catch (Exception e) {
//			e.printStackTrace();
//		}
//		return code;

		return findDescription(codingSchemeName, null, code);
	}

	/**
	 * Returns the entity description for a given code, or the code
	 * if no description can be located.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param code the code
	 *
	 * @return String
	 *
	 * @since DLBExt 1.0
	 */
	protected String findDescription(String codingSchemeName, String vers, String code) {
		// Perform the query .
		try {
			ConceptReferenceList crefs =
				createConceptReferenceList(
					new String[] {code}, codingSchemeName);

			CodedNodeSet nodes = lbSvc.getCodingSchemeConcepts(codingSchemeName, createCodingSchemeVersionOrTag(vers, null));
			nodes = nodes.restrictToCodes(crefs);

			// Restrict the properties ...
			ResolvedConceptReferenceList matches =
				nodes.resolveToList(null, new LocalNameList(), null, 1);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
				return ref.getReferencedEntry().getEntityDescription().getContent();
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return code;
	}


	/**
	 * Find concept name by code.
	 * @param codingSchemeName the coding scheme name
	 * @param vers the coding scheme version
	 * @param code the concept code
	 *
	 * @return the concept name
	 *
	 * @since DLBExt 1.0
	 * @deprecated - use getConceptName(String,String,String)
	 */@Deprecated
	// GF13918
	public String findConceptNameByCode(String codingSchemeName,
	                               String vers,
	                               String code)
	{
		//GF#13917
        //return codeToName(codingSchemeName, vers, code);
        return findDescription(codingSchemeName, vers, code);
    }

	/**
	 * Find concepts related to the given concept.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param ltag a coding scheme tag
	 * @param code the concept code
	 * @param relation name of association to search on
	 * @param direction true if concept is source of association, false if target
	 *
	 * @return the Vector of concept codes for related concepts
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<String> findRelatedCodes(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			String relation,
			boolean direction)
			{
		Vector<AssociatedConcept> w = getAssociatedConcepts(codingSchemeName,
				vers,
				ltag,
				code,
				relation,
				direction);
		if (w == null) return null;
		Vector<String> v = new Vector<String>();
		for (int i=0; i<w.size(); i++)
		{
			AssociatedConcept cs = (AssociatedConcept) w.elementAt(i);
			v.add(cs.getConceptCode());
		}
		return v;
			}



	/**
	 * Gets all associated concepts.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param code the concept code
	 * @param direction true if concept is source of association, false if target
	 *
	 * @return the associated concepts
	 *
	 * @since DLBExt 1.0
	 */
	public Vector<AssociatedConcept> getAllAssociatedConcepts(String codingSchemeName, String vers,
	                              String code, boolean direction)
	{
	    return getAssociatedConcepts(codingSchemeName,
	    		                           vers,
	                                       null,
	                                       code,
	                                       null,
	                                       direction);
	}






///////////////////////
//  Concept
///////////////////////

    /**
 * Gets the concept name.
 *
 * @param codingSchemeName the coding scheme
 * @param ce the CodedEntry
 *
 * @return the concept name
 *
 *
 * @since DLBExt 1.0
 */
    public String getConceptName(String codingSchemeName, CodedEntry ce)
	{
	    if (ce == null) return null;
//	    return codeToName(codingScheme, ce.getConceptCode());
	    return findDescription(codingSchemeName,ce.getConceptCode());
	}

///////////////////////////////////////////////////////////////////////////////////
    /**
     * Convert AtributeSetDescriptor index to LocalNameList.
     *
     * @param ASD_Index the aS d_ index
     *
     * @return the local name list
     */
    protected LocalNameList atributeSetDescriptorIndex2LocalNameList(int ASD_Index)
    {
		 if (ASD_Index == 1 || ASD_Index == 3) return null;
		 return new LocalNameList();
	 }

    /**
     * Gets the coded entry.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     *
     * @return the coded entry
     */
    public CodedEntry getCodedEntry(String codingSchemeName,
    		String vers,
    		String ltag,
    		String code)
    {
		if (code == null || code.compareTo("") == 0)
		{
			getLogger().error("getCodedEntry getCodedEntry code = null");
			return null;
		}

        ConceptReferenceList crefs = new ConceptReferenceList();
		ConceptReference cr = new ConceptReference();
		cr.setCodingScheme(codingSchemeName);
		cr.setConceptCode(code);
		crefs.addConceptReference(cr);


		// Perform the query ...
        ResolvedConceptReferenceList matches = null;
		try {
			CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, false)
			// KLO, 04/30/08
			//cns.restrictToCodes(crefs);
			cns = cns.restrictToCodes(crefs);
			matches = cns.resolveToList(null, null, null, 1);

			if (matches == null)
			{
				getLogger().warn("WARNING: getCodedEntry ResolvedConceptReferenceList returns matches = 0)");
				return null;
			}

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
				CodedEntry entry = ref.getReferencedEntry();

				return entry;
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return null;
    }



    /**
     * Gets the coded entry.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param ASD_Index parameter not used
     *
     * @return the coded entry
     *
     * @since DLBExt 1.0
     * @deprecated - use getCodedEntry(String,String,String,String)
     */@Deprecated
    public CodedEntry getCodedEntry(String codingSchemeName,
	                                String vers,
									String ltag,
									String code,
									@SuppressWarnings("unused")int ASD_Index)

	{
		if (code == null || code.compareTo("") == 0)
		{
			getLogger().error("getCodedEntry getCodedEntry code = null");
			return null;
		}

        ConceptReferenceList crefs = new ConceptReferenceList();
		ConceptReference cr = new ConceptReference();
		cr.setCodingScheme(codingSchemeName);
		cr.setConceptCode(code);
		crefs.addConceptReference(cr);


		// Perform the query ...
        ResolvedConceptReferenceList matches = null;
		try {
			CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, tagOrVersion);//, false)
			cns.restrictToCodes(crefs);
			matches = cns.resolveToList(null, null, null, 1);

			if (matches == null)
			{
				getLogger().warn("WARNING: getCodedEntry ResolvedConceptReferenceList returns matches = 0)");
				return null;
			}

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
				CodedEntry entry = ref.getReferencedEntry();

				return entry;
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}


    /**
     * Gets the root coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     *
     * @return the root coded entries
     *
     * @since DLBExt 1.0
     */
    public Vector<CodedEntry> getRootCodedEntries(String codingSchemeName, String vers, String ltag)
    {
    	return getRootCodedEntries(codingSchemeName,
    			vers,
    			ltag,
    			null, // relation
    			getHasSubtype(codingSchemeName));
    }



    /**
     * Gets the referenced entry.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param rcr the ResolvedConceptReference
     *
     * @return the referenced entry
     */
    protected CodedEntry getReferencedEntry(String codingSchemeName, String vers, String ltag, ResolvedConceptReference rcr)
    {
		if (rcr == null) return null;

		CodedEntry ce = rcr.getReferencedEntry();
		if (ce != null) return ce;
		ce = getCodedEntryByCode(codingSchemeName, vers, ltag, rcr.getConceptCode());
		if (ce == null)
		{
			getLogger().warn("CodedEntry is null");
		}
		return ce;
	}


	/**
	 * Gets the root coded entries.
	 *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
	 * @param relation the relation container name
	 * @param association the association name
	 *
	 * @return the root coded entries
	 *
	 * @since DLBExt 1.0
	 */





    public ResolvedConceptReferenceList getTopLevelNodes(CodingScheme cs){
	   return getTopLevelNodes(cs, null);
    }

	public ResolvedConceptReferenceList getTopLevelNodes(CodingScheme cs, String source){
	   try{
		   CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();
		   csvt.setVersion(cs.getRepresentsVersion());

		   return getTopNodesInSource(lbSvc, cs, cs.getFormalName(), csvt, null, null, source);

		}catch(Exception e){
		   e.printStackTrace();
		}
		return null;
	}


	//protected List getTopNodesInSource(LexBIGService lbSvc, CodingScheme cs, String scheme, CodingSchemeVersionOrTag csvt,
	protected ResolvedConceptReferenceList getTopNodesInSource(LexBIGService lbSvc, CodingScheme cs, String scheme, CodingSchemeVersionOrTag csvt,
	                                   String relation, String association, String source)
			throws LBException
    {
		ResolvedConceptReferenceList rootNodes = null;
		try {
			String src = source;
			if (!scheme.equals("NCI MetaThesaurus")) src = null;
			rootNodes = NavigationUtils.getRootNodes(
				    lbSvc,
					src,
					scheme,
					csvt);

		} catch (Exception e) {
            e.printStackTrace();
		}
		return rootNodes;
	}






	public Vector<CodedEntry> getRootCodedEntries(String codingSchemeName,
								   String vers,
								   String ltag,
								   String relation,
								   String association)
    {
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(vers, ltag);
        try {
            // Refer to GF#14361
			LexBIGServiceConvenienceMethodsImpl lbscm = (LexBIGServiceConvenienceMethodsImpl)
				 lbSvc.getGenericExtension("LexBIGServiceConvenienceMethods");
			lbscm.setLexBIGService(lbSvc);

			ResolvedConceptReferenceList rcrl = null;
			try {
				//rcrl = lbConvMethods.getTopNodes(codingSchemeName, tagOrVersion, relation, association, null); //"hasSubtype");
				rcrl = lbscm.getTopNodes(codingSchemeName, tagOrVersion, relation, association, null); //"hasSubtype");
			} catch (Exception ex) {
				ex.printStackTrace();
				getLogger().warn("LexBIGServiceConvenienceMethodsImpl.getTopNodes threws exceptions");
				return null;
			}

			ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			if (rcrs == null)
			{
				getLogger().warn("WARNING: getResolvedConceptReference returns rcrs == null");
			}
			else if (rcrs.length == 0)
			{
				getLogger().warn("WARNING: getResolvedConceptReference returns 0 entries.");
			}
			else
			{
				for (int i=0; i<rcrs.length; i++)
				{
					ResolvedConceptReference rcr = rcrs[i];
					CodedEntry ce = getReferencedEntry(codingSchemeName, vers, ltag, rcr);
					if (ce != null)
					{
						v.add(ce);
					}
			    }
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return v;
	}

	   /**
     * Gets the associated coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param direction true if concept is source of association, false if target
     * @param associationName the association name
     *
     * @return the associated coded entries
     *
     * @since DLBExt 1.0
     */
	public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			boolean direction,
			String associationName)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		try {
//			ResolvedConceptReference rcr = resolveConceptReference(
//					codingSchemeName,
//					vers,
//					ltag,
//					code,
//					properties,
//					resolveAssociationDepth,
//					resolveCodedEntryDepth,
//					direction);

			ResolvedConceptReference rcr = resolveConceptReference(
					codingSchemeName,
					vers,
					ltag,
					code);

			if (rcr == null)
			{
				getLogger().warn("WARNING: resolveConceptReference returns null.");
				return null;
			}



			AssociationList ass0list = null;
			if (direction) // associations of which this code is source
			{
			   ass0list = rcr.getSourceOf();
			}
			else // associations of which this code is target
			{
			   ass0list = rcr.getTargetOf();
			}

			if (ass0list == null)
			{
				return v;
			}

			Association[] assoArray = ass0list.getAssociation();

			if (assoArray == null)
			{
				return v;
			}

			// Check association type (is-a, part-of, etc)
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();
				// 050707
				if (associationName != null)
				{
					if (name.compareTo(associationName) == 0)
					{
						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
						if (acl != null)
						{
							AssociatedConcept[] acArray = acl.getAssociatedConcept();
							if (acArray != null && acArray.length > 0)
							{
								for (int i=0; i<acArray.length; i++)
								{
									if (acArray[i] != null)
									{
										CodedEntry ce = null;
										try {
											ce = acArray[i].getReferencedEntry();
										} catch (Exception ex) {

										}

										if (ce != null)
										{
											v.add(ce);
										}
									}
								}
							}
						}
					}
			    }
			    else
			    {
					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
					if (acl != null)
					{
						AssociatedConcept[] acArray = acl.getAssociatedConcept();
						if (acArray != null && acArray.length > 0)
						{
							for (int i=0; i<acArray.length; i++)
							{
								if (acArray[i] != null)
								{
									CodedEntry ce = null;
									try {
										ce = acArray[i].getReferencedEntry();
									} catch (Exception ex) {

									}

									if (ce != null)
									{
										v.add(ce);
									}
								}
							}
						}
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        return v;
    }


    /**
     * Gets the associated coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param direction true if concept is source of association, false if target
     * @param associationName the association name
     * @param resolveAssociationDepth parameter not used
     * @param resolveCodedEntryDepth parameter not used
     *
     * @return the associated coded entries
     *
     * @deprecated - use getAssociatedCodedEntries(String,String,String,String,boolean,String)
     */@Deprecated
	public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			@SuppressWarnings("unused")Vector properties,
			boolean direction,
			String associationName,
			@SuppressWarnings("unused")int resolveAssociationDepth,
			@SuppressWarnings("unused")int resolveCodedEntryDepth
	)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		try {
//			ResolvedConceptReference rcr = resolveConceptReference(
//					codingSchemeName,
//					vers,
//					ltag,
//					code,
//					properties,
//					resolveAssociationDepth,
//					resolveCodedEntryDepth,
//					direction);

			ResolvedConceptReference rcr = resolveConceptReference(
					codingSchemeName,
					vers,
					ltag,
					code);

			if (rcr == null)
			{
				getLogger().warn("WARNING: resolveConceptReference returns null.");
				return null;
			}



			AssociationList ass0list = null;
			if (direction) // associations of which this code is source
			{
			   ass0list = rcr.getSourceOf();
			}
			else // associations of which this code is target
			{
			   ass0list = rcr.getTargetOf();
			}

			if (ass0list == null)
			{
				return v;
			}

			Association[] assoArray = ass0list.getAssociation();

			if (assoArray == null)
			{
				return v;
			}

			// Check association type (is-a, part-of, etc)
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();
				// 050707
				if (associationName != null)
				{
					if (name.compareTo(associationName) == 0)
					{
						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
						if (acl != null)
						{
							AssociatedConcept[] acArray = acl.getAssociatedConcept();
							if (acArray != null && acArray.length > 0)
							{
								for (int i=0; i<acArray.length; i++)
								{
									if (acArray[i] != null)
									{
										CodedEntry ce = null;
										try {
											ce = acArray[i].getReferencedEntry();
										} catch (Exception ex) {

										}

										if (ce != null)
										{
											v.add(ce);
										}
									}
								}
							}
						}
					}
			    }
			    else
			    {
					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
					if (acl != null)
					{
						AssociatedConcept[] acArray = acl.getAssociatedConcept();
						if (acArray != null && acArray.length > 0)
						{
							for (int i=0; i<acArray.length; i++)
							{
								if (acArray[i] != null)
								{
									CodedEntry ce = null;
									try {
										ce = acArray[i].getReferencedEntry();
									} catch (Exception ex) {

									}

									if (ce != null)
									{
										v.add(ce);
									}
								}
							}
						}
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        return v;
    }



     // 120406
     /**
      * Gets the associated coded entries by source.
      *
      * @param codingSchemeName a coding scheme name
      * @param vers a coding scheme version
      * @param ltag a coding scheme tag
      * @param code the concept code
      * @param direction true if concept is source of association, false if target
      * @param associationName the association name
      * @param sourceAbbr the source abbreviation
      *
      * @return the associated coded entries by source
      */
 	public Vector<CodedEntry> getAssociatedCodedEntriesBySource(String codingSchemeName,
 			String vers,
 			String ltag,
 			String code,
 			boolean direction,
 			String associationName,
 			String sourceAbbr
 	)
 	{
 		Vector<CodedEntry> v = new Vector<CodedEntry>();
 		try {
 			ResolvedConceptReference rcr = resolveConceptReference(
 					codingSchemeName,
 					vers,
 					ltag,
 					code,
 					direction);

 			if (rcr == null)
 			{
 				getLogger().warn("WARNING: resolveConceptReference returns null.");
 				//050807
 				//return null;
 				return v;
 			}

 			AssociationList ass0list = null;
 			if (direction) // associations of which this code is source
 			{
 			   ass0list = rcr.getSourceOf();
 			}
 			else // associations of which this code is target
 			{
 			   ass0list = rcr.getTargetOf();
 			}

 			if (ass0list == null)
 			{
 				return v;
 			}

 			Association[] assoArray = ass0list.getAssociation();

 			if (assoArray == null)
 			{
 				return v;
 			}

 			// Check association type (is-a, part-of, etc)
 			for (int k=0; k<assoArray.length; k++)
 			{
 				String name = assoArray[k].getAssociationName();

 				// 050707
 				if (associationName != null)
 				{
 					if (name.compareTo(associationName) == 0)
 					{
 						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
 						if (acl != null)
 						{
 							AssociatedConcept[] acArray = acl.getAssociatedConcept();
 							if (acArray != null && acArray.length > 0)
 							{
 								for (int i=0; i<acArray.length; i++)
 								{
 									boolean matched = false;
 									if (acArray[i] != null)
 									{
 										if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
 										{
 											// modified to be compatible with version 2
 											//LocalNameList lnl =	acArray[i].getAssociationQualifiers();
 											NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
 											NameAndValue[] nvArray = lnl.getNameAndValue();
 											for (int k2=0; k2<nvArray.length; k2++)
 											{
 												NameAndValue nv = nvArray[k2];
 												if (nv.getName().compareTo(sourceAbbr) == 0)
 												{
 													matched = true;
 													break;
 												}
 											}


 										}
 										else
 										{
 											matched = true;
 										}

 										if (matched)
 										{
 											CodedEntry ce = null;
 											try {
 												ce = acArray[i].getReferencedEntry();
 											} catch (Exception ex) {
                                                 ex.printStackTrace();
 											}

 											if (ce != null)
 											{
 												v.add(ce);
 											}
 										}
 									}
 								}
 							}
 						}
 				    }
 				}
 				else
 				{
 					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
 					if (acl != null)
 					{
 						AssociatedConcept[] acArray = acl.getAssociatedConcept();
 						if (acArray != null && acArray.length > 0)
 						{
 							for (int i=0; i<acArray.length; i++)
 							{
 								boolean matched = false;
                                 if (acArray[i] != null)
                                 {
 									if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
 									{
 										// check if source matches
 										// Modified to be compatible with version 2
 										// LocalNameList lnl =	acArray[i].getAssociationQualifiers();
 										NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
 										NameAndValue[] nvArray = lnl.getNameAndValue();
 										for (int k2=0; k2<nvArray.length; k2++)
 										{
 											NameAndValue nv = nvArray[k2];
 											if (nv.getName().compareTo(sourceAbbr) == 0)
 											{
 												matched = true;
 												break;
 											}
 										}
 									}
 									else
 									{
 										matched = true;
 									}

 									if (matched)
 									{
 										CodedEntry ce = null;
 										try {
 											ce = acArray[i].getReferencedEntry();
 										} catch (Exception ex) {
                                             ex.printStackTrace();
 										}

 										if (ce != null)
 										{
 											v.add(ce);
 										}
 								    }
 							    }
 							}
 					    }
 				    }
 				}
 			}
 	    } catch (Exception e) {
 			e.printStackTrace();
 			return null;
 		}
         return v;
     }



    // 120406
    /**
     * Gets the associated coded entries by source.
     *
     * @param codingSchemeName a coding scheme name
     * @param vers a coding scheme version
     * @param ltag a coding scheme tag
     * @param code the concept code
     * @param properties parameter not used
     * @param direction true if concept is source of association, false if target
     * @param associationName the association name
     * @param resolveAssociationDepth parameter not used
     * @param resolveCodedEntryDepth parameter not used
     * @param sourceAbbr the source abbreviation
     *
     * @return the associated coded entries by source
     * @deprecated - use getAssociatedCodedEntriedBySources(String,String,String,String,boolean,String,String)
     */@Deprecated
	public Vector<CodedEntry> getAssociatedCodedEntriesBySource(String codingSchemeName,
			String vers,
			String ltag,
			String code,
			Vector properties,
			boolean direction,
			String associationName,
			int resolveAssociationDepth,
			int resolveCodedEntryDepth,
			String sourceAbbr
	)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		try {
			ResolvedConceptReference rcr = resolveConceptReference(
					codingSchemeName,
					vers,
					ltag,
					code,
					properties,
					resolveAssociationDepth,
					resolveCodedEntryDepth,
					direction);

			if (rcr == null)
			{
				getLogger().warn("WARNING: resolveConceptReference returns null.");
				//050807
				//return null;
				return v;
			}

			AssociationList ass0list = null;
			if (direction) // associations of which this code is source
			{
			   ass0list = rcr.getSourceOf();
			}
			else // associations of which this code is target
			{
			   ass0list = rcr.getTargetOf();
			}

			if (ass0list == null)
			{
				return v;
			}

			Association[] assoArray = ass0list.getAssociation();

			if (assoArray == null)
			{
				return v;
			}

			// Check association type (is-a, part-of, etc)
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();

				// 050707
				if (associationName != null)
				{
					if (name.compareTo(associationName) == 0)
					{
						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
						if (acl != null)
						{
							AssociatedConcept[] acArray = acl.getAssociatedConcept();
							if (acArray != null && acArray.length > 0)
							{
								for (int i=0; i<acArray.length; i++)
								{
									boolean matched = false;
									if (acArray[i] != null)
									{
										if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
										{
											// modified to be compatible with version 2
											//LocalNameList lnl =	acArray[i].getAssociationQualifiers();
											NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
											NameAndValue[] nvArray = lnl.getNameAndValue();
											for (int k2=0; k2<nvArray.length; k2++)
											{
												NameAndValue nv = nvArray[k2];
												if (nv.getName().compareTo(sourceAbbr) == 0)
												{
													matched = true;
													break;
												}
											}


										}
										else
										{
											matched = true;
										}

										if (matched)
										{
											CodedEntry ce = null;
											try {
												ce = acArray[i].getReferencedEntry();
											} catch (Exception ex) {
                                                ex.printStackTrace();
											}

											if (ce != null)
											{
												v.add(ce);
											}
										}
									}
								}
							}
						}
				    }
				}
				else
				{
					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
					if (acl != null)
					{
						AssociatedConcept[] acArray = acl.getAssociatedConcept();
						if (acArray != null && acArray.length > 0)
						{
							for (int i=0; i<acArray.length; i++)
							{
								boolean matched = false;
                                if (acArray[i] != null)
                                {
									if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
									{
										// check if source matches
										// Modified to be compatible with version 2
										// LocalNameList lnl =	acArray[i].getAssociationQualifiers();
										NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
										NameAndValue[] nvArray = lnl.getNameAndValue();
										for (int k2=0; k2<nvArray.length; k2++)
										{
											NameAndValue nv = nvArray[k2];
											if (nv.getName().compareTo(sourceAbbr) == 0)
											{
												matched = true;
												break;
											}
										}
									}
									else
									{
										matched = true;
									}

									if (matched)
									{
										CodedEntry ce = null;
										try {
											ce = acArray[i].getReferencedEntry();
										} catch (Exception ex) {
                                            ex.printStackTrace();
										}

										if (ce != null)
										{
											v.add(ce);
										}
								    }
							    }
							}
					    }
				    }
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        return v;
    }





    /**
     * Checks for associated coded entries.
     *
     * @param codingSchemeName a coding scheme name
     * @param relationship the name of the relations container
     * @param association the association name
     * @param conceptCode the concept code
     * @param direction true if concept is source of association, false if target
     *
     * @return true, if checks for associated coded entries
     *
     * @since DLBExt 1.0
     */
    public Boolean hasAssociatedCodedEntries(String codingSchemeName,
                                           String relationship,
                                           String association,
                                           String conceptCode,
                                           boolean direction)
    {
        Vector<CodedEntry> v = getAssociatedCodedEntries(codingSchemeName,
                                           relationship,
                                           association,
                                           conceptCode,
                                           direction, true);
        if (v == null) return Boolean.FALSE;
        if (v.size() > 0) return Boolean.TRUE;
        return Boolean.FALSE;
    }




     /**
      * Gets the associated coded entries.
      *
      * @param codingSchemeName a coding scheme name
      * @param relation - the name of the relations container
      * @param association the association
      * @param conceptCode the concept code
      * @param direction the direction
      * @param buildReferencedEntries the build referenced entries
      *
      * @return the associated coded entries
      *
      * @since DLBExt 1.0
      */

    /*
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
                                           String relation,
                                           String association,
                                           String conceptCode,
                                           boolean direction,
                                           boolean buildReferencedEntries)
    {
	    Vector<CodedEntry> v = new Vector<CodedEntry>();
	    try {
			Association asso = null;
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(null, null);
			if (direction)
			{
				//asso = lbConvMethods.getChildrenOf(conceptCode, relation, association,
						codingSchemeName, versionOrTag, buildReferencedEntries, null);
			}
			else
			{
				//asso = lbConvMethods.getParentsOf(conceptCode, relation, association,
                        codingSchemeName, versionOrTag, buildReferencedEntries, null);
			}
			if (asso != null)
			{
				AssociatedConceptList acl =	asso.getAssociatedConcepts();
				if (acl != null)
				{
					AssociatedConcept[] acs = acl.getAssociatedConcept();
					if (acs != null)
					{
						for (int i=0; i<acs.length; i++)
						{
							CodedEntry ce = acs[i].getReferencedEntry();
							if (ce == null)
							{
								getLogger().warn("WARNING: getReferencedEntry returns null");
							}
							else
							{
								v.add(ce);
							}
						}
					}
				}
		    }
	    } catch (Exception e) {
			e.printStackTrace();
			getLogger().error("Error encountered when calling getAssociatedCodedEntries.");
		}
		return v;
    }
    */



     public Vector<CodedEntry> getAssociatedCodedEntries(String codingSchemeName,
                                           String relation,
                                           String association,
                                           String conceptCode,
                                           boolean direction,
                                           boolean buildReferencedEntries)
    {
	    Vector<CodedEntry> v = new Vector<CodedEntry>();
	    String version = null;
	    String ltag = null;
	    String relationContainer = null;
	    Vector asso_names = null;
	    Vector<ConceptAssociation> w = getAssociations(
		                                  codingSchemeName, null, null,
										  conceptCode,
										  direction,
										  relationContainer,
										  asso_names
										  );
	    for (int i=0; i<w.size(); i++)
	    {
			ConceptAssociation asso = (ConceptAssociation) w.elementAt(i);
            CodedEntry ce = getCodedEntry(codingSchemeName,
	                                version,
									ltag,
									asso.getValue());
			v.add(ce);
		}
		return v;

	}




    /**
     * Code exists.
     *
     * @param codingSchemeName a coding scheme name
     * @param code the code
     *
     * @return true, if code exists
     *
     * @since DLBExt 1.0
     */
    public Boolean codeExists(String codingSchemeName, String code)
    {
        try
        {
            CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, createCodingSchemeVersionOrTag(null, null));
            return cns.isCodeInSet(Constructors.createConceptReference(code, codingSchemeName));
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }


///////////////////////
//  History
///////////////////////

    /**
 * Checks if is valid history date.
 *
 * @param codingSchemeName the coding scheme
 * @param historydate the history date
 *
 * @return true, if is valid history date
 */
public boolean isValidHistoryDate(String codingSchemeName, Date historydate)
    {
		Vector<Date> v = getHistoryBaselineDates(codingSchemeName);
		for (int i=0;i<v.size(); i++)
		{
			java.util.Date d = (java.util.Date) v.elementAt(i);
			if (d.equals(historydate)) return true;
		}
		return false;
	}

    /**
     * Gets the history baseline dates.
     *
     * @param codingSchemeName a coding scheme name
     *
     * @return the history baseline dates
     *
     * @since DLBExt 1.0
     */
    public Vector<Date> getHistoryBaselineDates(String codingSchemeName)
    {
		Vector<Date> v = new Vector<Date>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
            java.util.Date releasedAfter = null;
            java.util.Date releasedBefore = null;
 			SystemReleaseList srl = historySvc.getBaselines(releasedAfter,
                                releasedBefore);

            SystemRelease[] srs = srl.getSystemRelease();
            for (int i=0;i<srs.length; i++)
            {
				SystemRelease sr = srs[i];
				java.util.Date d = sr.getReleaseDate();
				v.add(d);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     * Gets the concept creation date.
     *
     * @param codingSchemeName the coding scheme name
     * @param code the code
     *
     * @return the concept creation date
     */
    public java.util.Date getConceptCreationDate(String codingSchemeName, String code)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
            CodingSchemeVersion csv = historySvc.getConceptCreationVersion(
				 Constructors.createConceptReference(code, null));

            return csv.getVersionDate();
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
	}


    /**
     * Gets the concept history.
     *
     * @param codingSchemeName a coding scheme name
     * @param code the code
     *
     * @return the concept history
     *
     * @since DLBExt 1.0
     */
    public Vector<NCIChangeEvent> getConceptHistory(String codingSchemeName,
                                    String code)
	{
        return getConceptHistory(codingSchemeName,
                                 code,
                                 getHistoryStartBaselineDate(codingSchemeName),
                                 getHistoryEndBaselineDate(codingSchemeName));
	}



    /**
     * Gets the concept history.
     *
     * @param codingSchemeName the coding scheme name
     * @param code the code
     * @param beginDate the begin date
     * @param endDate the end date
     *
     * @return the concept history
     */
    public Vector<NCIChangeEvent> getConceptHistory(String codingSchemeName,
                                    String code,
                                    java.util.Date beginDate,
                                    java.util.Date endDate)
    {
		Vector<NCIChangeEvent> v = new Vector<NCIChangeEvent>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				v.add(nce);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     * Gets the concept history.
     *
     * @param codingSchemeName a coding scheme name
     * @param code the code
     * @param csv the CodingSchemeVersion
     *
     * @return the concept history
     *
     * @since DLBExt 1.0
     */
    public Vector<NCIChangeEvent> getConceptHistory(String codingSchemeName,
                                  String code,
                                  CodingSchemeVersion csv)
    {
		Vector<NCIChangeEvent> v = new Vector<NCIChangeEvent>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), csv);
            NCIChangeEvent[] nces = cel.getEntry();
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				v.add(nce);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     * Gets the concept change versions.
     *
     * @param codingSchemeName a coding scheme name
     * @param code the code
     *
     * @return the concept change versions
     *
     * @since DLBExt 1.0
     */
    public Vector<CodingSchemeVersion> getConceptChangeVersions(String codingSchemeName, String code)
    {
		Vector<CodingSchemeVersion> v = new Vector<CodingSchemeVersion>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
            java.util.Date beginDate = null;
            java.util.Date endDate = null;

            ConceptReference ref = Constructors.createConceptReference(code, null);
			CodingSchemeVersionList csvl = historySvc.getConceptChangeVersions(
				ref, beginDate, endDate);

			CodingSchemeVersion[] csvs = csvl.getEntry();
            for (int i=0;i<csvs.length; i++)
            {
				CodingSchemeVersion csv = csvs[i];
				v.add(csv);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     * Gets the concept create version.
     *
     * @param codingSchemeName a coding scheme name
     * @param code the code
     *
     * @return the concept create version
     *
     * @since DLBExt 1.0
     */
    public CodingSchemeVersion getConceptCreateVersion(String codingSchemeName, String code)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
//            java.util.Date beginDate = null;
//            java.util.Date endDate = null;

            ConceptReference ref = Constructors.createConceptReference(code, null);
            if (ref == null)
            {
				getLogger().warn("getConceptCreateVersion Constructors returns null");
				return null;
			}
            CodingSchemeVersion csv = null;
            try {
            	csv = historySvc.getConceptCreationVersion(ref);
			} catch (Exception ex) {
				ex.printStackTrace();
				return null;
			}
            return csv;

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
    }


    /**
     * Gets the history start baseline date.
     *
     * @param codingSchemeName a coding scheme name
     *
     * @return the history start baseline date
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getHistoryStartBaselineDate(String codingSchemeName)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			SystemRelease sr = historySvc.getEarliestBaseline();
			java.util.Date d = sr.getReleaseDate();
			return d;
        }
        catch (Exception e)
        {
 			e.printStackTrace();
 		}
 		return null;
    }


    /**
     * Gets the history end baseline date.
     *
     * @param codingSchemeName a coding scheme name
     *
     * @return the history end baseline date
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getHistoryEndBaselineDate(String codingSchemeName)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			SystemRelease sr = historySvc.getLatestBaseline();
			java.util.Date d = sr.getReleaseDate();
			return d;
        }
        catch (Exception e)
        {
 			e.printStackTrace();
 		}
 		return null;
    }


	/**
	 * Gets the concept edit action dates.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 * @param iAction the history ChangeType index (retire, split, etc)
	 *
	 * @return the concept edit action dates
	 */
	public Vector<Date> getConceptEditActionDates(String codingSchemeName, String code, int iAction)
	{
		Vector<Date> date_vec = new Vector<Date>();
        Vector<NCIChangeEvent> v = getConceptHistory(codingSchemeName,
                                    code,
                                    getHistoryStartBaselineDate(codingSchemeName),
                                    getHistoryEndBaselineDate(codingSchemeName));
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			int actionId = nce.getEditaction().getType();
			if (actionId == iAction)
			{
				date_vec.add(nce.getEditDate());
			}
		}

		return date_vec;
	}


	/**
	 * Extracts an edit action string (for example, 'CREATE') from a
	 * edit action constant string (form example, 'EDIT_ACTION_CREATE_STR') corresponding
	 * to the specified edit action code.
	 *
	 * @param actionId
	 * @return an edit action string (for example, 'CREATE')
	 * @since dlbExt 1.0
	 */
	protected String iActiontoStr(int actionId)
	{
		String action = "";
		if (actionId == ChangeType.CREATE_TYPE)
		{
		    action = "Create";
		}
		else if (actionId == ChangeType.MERGE_TYPE)
		{
		    action = "Merge";
		}
		else if (actionId == ChangeType.MODIFY_TYPE)
		{
		    action = "Modify";
		}
		else if (actionId == ChangeType.RETIRE_TYPE)
		{
		    action = "Retire";
		}
		else if (actionId == ChangeType.SPLIT_TYPE)
		{
		    action = "Split";
		}
		return action;
	}


	/**
	 * To action index.
	 *
	 * @param action the edit action string (for example, 'CREATE')
	 *
	 * @return the ChangeType index
	 */
	public int toActionIndex(String action)
	{
		int iAction = -1;
		if (action.compareToIgnoreCase("Create") == 0)
		{
		    iAction = ChangeType.CREATE_TYPE;
		}
		else if (action.compareToIgnoreCase("Merge") == 0)
		{
		    iAction = ChangeType.MERGE_TYPE;
		}
		else if (action.compareToIgnoreCase("Modify") == 0)
		{
		    iAction = ChangeType.MODIFY_TYPE;
		}
		else if (action.compareToIgnoreCase("Retire") == 0)
		{
		    iAction = ChangeType.RETIRE_TYPE;
		}
		else if (action.compareToIgnoreCase("Split") == 0)
		{
		    iAction = ChangeType.SPLIT_TYPE;
		}
		return iAction;
	}



	/**
	 * Converts Date to string.
	 *
	 * @param aDate the a date
	 *
	 * @return the string in "yyyy-MM-dd" format
	 */
	protected String dateToString(java.util.Date aDate)
	{
		if (aDate == null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String theDate = sdf.format(aDate);
		return theDate;
	}


	/**
	 * Gets the concept edit action.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the code
	 *
	 * @return the concept EditActionDate

	 */
	public Vector<EditActionDate> getConceptEditAction(String codingSchemeName, String code)
	{
		Vector<EditActionDate> w = new Vector<EditActionDate>();
        Vector<NCIChangeEvent> v = getConceptHistory(codingSchemeName,
                                    code,
                                    getHistoryStartBaselineDate(codingSchemeName),
                                    getHistoryEndBaselineDate(codingSchemeName));
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			EditActionDate e = new EditActionDate();
			e.setEditDate(nce.getEditDate());
			e.setAction(nce.getEditaction().getType());
			w.add(e);
		}
		return w;
	}


	/**
	 * Checks if is concept edited.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 * @param mrg_splt true if checking for merge or split, false if for create
	 * @param beginDate the begin date
	 * @param endDate the end date
	 *
	 * @return the boolean
	 */
	public Boolean isConceptEdited(
		                String codingSchemeName,
						String code,
						boolean mrg_splt,
						java.util.Date beginDate,
						java.util.Date endDate)
	{
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
//            int knt = 0;
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				int actionId = nce.getEditaction().getType();
				if (mrg_splt)
				{
					if (actionId == ChangeType.SPLIT_TYPE ||
					    actionId == ChangeType.MERGE_TYPE)
					{
						return Boolean.TRUE;
					}
				}
				else
				{
					if (actionId != ChangeType.CREATE_TYPE) return Boolean.TRUE;
				}
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return Boolean.FALSE;
	}


	/**
	 * Gets the number edit actions.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 * @param baselineDate_in the baseline date_in
	 *
	 * @return the number edit actions
	 */
	public Integer getNumberEditActions(
		                String codingSchemeName,
						String code,
						java.util.Date baselineDate_in)
	{
		return getNumberEditActions(
				        codingSchemeName,
						code,
						baselineDate_in,
						baselineDate_in);
	}


	/**
	 * Gets the number edit actions.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 * @param beginDate the begin date
	 * @param endDate the end date
	 *
	 * @return the number edit actions
	 */
	public Integer getNumberEditActions(
		                String codingSchemeName,
						String code,
						java.util.Date beginDate,
						java.util.Date endDate)
	{
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            return new Integer(nces.length);
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
	}


	/**
	 * Gets the ancestor code action dates.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 *
	 * @return the ancestor code action dates
	 */
	public Vector<String> getAncestorCodeActionDates(
		                String codingSchemeName,
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		Vector<String> v = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            boolean isRetired = false;
            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
            java.util.Date retire_date = null;
            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();
                if (code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					isRetired = true;
					retire_date = nce.getEditDate();
				}
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.MERGE_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					retire_thru_merge = true;
//					merge_into = ref_code;
					merge_into_date = nce.getEditDate();
					String s = dateToString(merge_into_date);
					v.add(ref_code+"|"+s);
					return v;
				}
                else if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
//					create_thru_split = true;
//					split_from = code;
//					split_from_date = nce.getEditDate();
				}
			}

			if (isRetired && !retire_thru_merge)
			{
	            for (int i=0;i<nces.length; i++)
                {
					NCIChangeEvent nce = nces[i];
					String code = nce.getConceptcode();
					String ref_code = nce.getReferencecode();
					int actionId =	nce.getEditaction().getType();
					if (code.compareTo(inCode) == 0 &&
						actionId == ChangeType.RETIRE_TYPE)
					{
						if (ref_code != null && ref_code.compareTo("") != 0)
						{
							retire_date = nce.getEditDate();
							String s = dateToString(retire_date);
							v.add(ref_code+"|"+s);
						}
					}
				}
				return v;
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;

	}

	/**
	 * Gets the code action parents.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param baselineDate_in the baseline date_in
	 *
	 * @return the code action parents
	 *
	 * @deprecated use getAncestorCodes(String,String,Date)
	 */@Deprecated
	public Vector<String> getCodeActionParents(
		                String codingSchemeName,
						String inCode,
						java.util.Date baselineDate_in)
	{
				 return getAncestorCodes(
						 codingSchemeName,
						inCode,
						true,
						baselineDate_in,
						baselineDate_in,
						true);
	}


	/**
	 * Gets the ancestor codes.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param inCode the concept code to pull history on
	 * @param baselineDate_in the baseline date_in
	 *
	 * @return the ancestor codes
	 */
	public Vector<String> getAncestorCodes(
			String codingSchemeName,
			String inCode,
			java.util.Date baselineDate_in)
	{
		 return getAncestorCodes(
				 codingSchemeName,
				inCode,
				true,
				baselineDate_in,
				baselineDate_in,
				true);
	}


	/**
	 * Gets the ancestor codes.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param inCode the concept code to pull history on
	 * @param atBaseline the at baseline
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 *
	 * @return the ancestor codes
	 */
	public Vector<String> getAncestorCodes(
		                String codingSchemeName,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		boolean split_merge_only = false;
				 return getAncestorCodes(
						 codingSchemeName,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in,
						split_merge_only);

	}


	/**
	 * Gets the ancestor codes.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param inCode the concept code to pull history on
	 * @param atBaseline the at baseline
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 * @param split_merge_only the split_merge_only
	 *
	 * @return the ancestor codes
	 */
	public Vector<String> getAncestorCodes(
		                String codingSchemeName,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in,
						boolean split_merge_only)
	{
		Vector<String> codes = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;
			if (atBaseline) endDate = iBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            boolean isRetired = false;
            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
//            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();

                if (code.compareTo(inCode) == 0 && actionId == ChangeType.RETIRE_TYPE)
                {
					isRetired = true;
//					retire_date = nce.getEditDate();
				}
				else if (actionId == ChangeType.MERGE_TYPE)
				{
					retire_thru_merge = true;
					if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) != 0)
					{
						codes.add(code);
						codes.add(ref_code);
					}
				}

				/*
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.MERGE_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					retire_thru_merge = true;
					merge_into = ref_code;
					merge_into_date = nce.getEditDate();
					codes.add(ref_code);
					return codes;
				}
                else if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					create_thru_split = true;
					split_from = code;
					split_from_date = nce.getEditDate();
				}
				*/
				else if (actionId == ChangeType.SPLIT_TYPE)
				{
					if (ref_code != null && ref_code.compareTo(inCode) != 0)
					{
						codes.add(inCode);
					}
					else if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) != 0)
					{
						codes.add(code);
					}
				}
			}

			if (isRetired && !retire_thru_merge && !split_merge_only)
			{
	            for (int i=0;i<nces.length; i++)
                {
					NCIChangeEvent nce = nces[i];
					String code = nce.getConceptcode();
					String ref_code = nce.getReferencecode();
					int actionId =	nce.getEditaction().getType();
					if (code.compareTo(inCode) == 0 && actionId == ChangeType.RETIRE_TYPE)
					{
						if (ref_code != null && ref_code.compareTo("") != 0)
						{
							codes.add(ref_code);
						}
					}
				}
				return codes;
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return codes;
	}





	/**
	 * Gets the descendant code action dates.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 *
	 *
	 * @return the descendant code action dates
	 */
	public Vector<String> getDescendantCodeActionDates(
		                String codingSchemeName,
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		Vector<String> v = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
//            boolean isRetired = false;
//            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();
                if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					retire_date = nce.getEditDate();
					String s = dateToString(retire_date);
					v.add(ref_code+"|"+s);
				}
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					Date split_date = nce.getEditDate();
					String s = dateToString(split_date);
					v.add(ref_code+"|"+s);
				}
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
	}


	/**
	 * Gets the code action children.
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param baselineDate_in the baseline date_in
	 *
	 * @return the code action children
	 *
	 * @deprecated use getDescendantCodes(String,String,Date)
	 */@Deprecated
	public Vector<String> getCodeActionChildren(
		                String codingSchemeName,
						String inCode,
						java.util.Date baselineDate_in)
	{
				 return getDescendantCodes(
		                codingSchemeName,
						inCode,
						true,
						baselineDate_in,
						baselineDate_in,
						true);
	}


	/**
	 * Gets the descendant codes.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param inCode the concept code to pull history on
	 * @param baselineDate_in the baseline date_in
	 *
	 * @return the descendant codes
	 */
	public Vector<String> getDescendantCodes(String codingSchemeName, String inCode, java.util.Date baselineDate_in)
	{
		 return getDescendantCodes(
				    codingSchemeName,
					inCode,
					true,
					baselineDate_in,
					baselineDate_in,
					true);
	}

	/**
	 * Gets the descendant codes.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param atBaseline the at baseline
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 *
	 * @return the descendant codes
	 */
	public Vector<String> getDescendantCodes(
		                String codingSchemeName,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
         boolean merge_split_only = false;
 	     return getDescendantCodes(
 	    		codingSchemeName,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in,
						merge_split_only);

	}


	/**
	 * Gets the descendant codes.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the concept code to pull history on
	 * @param atBaseline the at baseline
	 * @param iBaselineDate_in the initial baseline date_in
	 * @param fBaselineDate_in the final baseline date_in
	 * @param merge_split_only the merge_split_only
	 *
	 * @return the descendant codes
	 */
	public Vector<String> getDescendantCodes(
		                String codingSchemeName,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in,
						boolean merge_split_only)
	{
		Vector<String> codes = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;
			if (atBaseline) endDate = iBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);

            NCIChangeEvent[] nces = cel.getEntry();

//            boolean isRetired = false;
//            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
//            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();
                if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					if (!merge_split_only)
					{
						codes.add(code);
				    }
				}

				else if (actionId == ChangeType.SPLIT_TYPE)
				{
					if (ref_code != null && code.compareTo(inCode) == 0)
					{
						codes.add(code);
						codes.add(ref_code);
					}
				}

				else if (actionId == ChangeType.MERGE_TYPE)
				{
					if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) == 0)
					{
						codes.add(code);
					}
					else if (ref_code != null && code.compareTo(inCode) == 0)
					{
						codes.add(ref_code);
					}
				}


			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return codes;
	}





    // 073006 (v1rc6 addition)
 	/**
     * Gets the ancestors.
     *
     * @param codingSchemeName the coding scheme
     * @param inCode the in code
     *
     * @return the ancestors
     */
    public Vector<String> getAncestors(String codingSchemeName, String inCode)
 	{
 		 Vector<String> codes = new Vector<String>();
         try
         {
 			 HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
 			 NCIChangeEventList cel = historySvc.getAncestors(Constructors
                     .createConceptReference(inCode, null));

             NCIChangeEvent[] nces = cel.getEntry();
             for (int i=0;i<nces.length; i++)
             {
 				 NCIChangeEvent nce = nces[i];
                 String ref_code = nce.getReferencecode();
                 codes.add(ref_code);
			 }
			 return codes;

         }
         catch (Exception e)
         {
 			e.printStackTrace();
         }
         return codes;
	}

	// 073006 (v1rc6 addition)
 	/**
	 * Gets the descendants.
	 *
	 * @param codingSchemeName the coding scheme
	 * @param inCode the in code
	 *
	 * @return the descendants
	 */
	public Vector<String> getDescendants(String codingSchemeName, String inCode)
 	{
 		 Vector<String> codes = new Vector<String>();
         try
         {
 			 HistoryService historySvc = lbSvc.getHistoryService(codingSchemeName);
 			 NCIChangeEventList cel = historySvc.getDescendants(Constructors
                     .createConceptReference(inCode, null));

             NCIChangeEvent[] nces = cel.getEntry();
             for (int i=0;i<nces.length; i++)
             {
 				 NCIChangeEvent nce = nces[i];
                 String ref_code = nce.getReferencecode();
                 codes.add(ref_code);
			 }
			 return codes;

         }
         catch (Exception e)
         {
 			e.printStackTrace();
         }
         return codes;
	}



	/**
	 * Gets the supported property qualifier.
	 *
	 * @param codingSchemeName the coding scheme
	 *
	 * @return the supported property qualifier
	 */
	public Vector<String> getSupportedPropertyQualifier(String codingSchemeName)
	{
		if (codingSchemeName == null) return null;
        Vector<String> v = new Vector<String>();
        try {
			CodingScheme cs = getCodingScheme(codingSchemeName);
			org.LexGrid.naming.SupportedPropertyQualifier[] supportedPropertyQualifiers
			    = cs.getMappings().getSupportedPropertyQualifier();
			if (supportedPropertyQualifiers == null) return null;
			for (int i=0; i<supportedPropertyQualifiers.length; i++)
			{
				SupportedPropertyQualifier q = supportedPropertyQualifiers[i];
				//v.add(q.getContent());
				v.add(q.getLocalId());
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
    }

	/**
	 * Gets the supported association qualifier.
	 *
	 * @param codingSchemeName the coding scheme
	 *
	 * @return the supported association qualifier
	 */
	public Vector<String> getSupportedAssociationQualifier(String codingSchemeName)
	{
		if (codingSchemeName == null) return null;
        Vector<String> v = new Vector<String>();
        try {
			CodingScheme cs = getCodingScheme(codingSchemeName);
			org.LexGrid.naming.SupportedAssociationQualifier[] supportedAssociationQualifiers
			    = cs.getMappings().getSupportedAssociationQualifier();
			if (supportedAssociationQualifiers == null) return null;
			for (int i=0; i<supportedAssociationQualifiers.length; i++)
			{
				SupportedAssociationQualifier q = supportedAssociationQualifiers[i];
				//v.add(q.getContent());
				v.add(q.getLocalId());
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
	}




    /**
     * Gets the supported context count.
     *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     *
     * @return the supported context count
     */
    public int getSupportedContextCount(String codingSchemeName, String vers, String ltag)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return -1;
        return getSupportedContextCount(cs);
	}


    /**
     * Gets the supported context count.

     *
     * @param cs the CodingScheme
     *
     *
     * @return the supported context count
     */
    public int getSupportedContextCount(CodingScheme cs)
 	{
 		if (cs == null) return -1;
//        Vector v = new Vector();
 	    int n = cs.getMappings().getSupportedContextCount();
        return n;
	}




    /**
	 * Gets the supported context.
	 *
	 * @param codingSchemeName a coding scheme name
	 * @param vers a coding scheme version
	 * @param ltag the tag
	 *
	 * @return the supported context
	 *
	 * @since DLBExt 1.0
	 */
    public Vector getSupportedContext(String codingSchemeName, String vers, String ltag)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedContextNames(cs);
	}



    /**
     * Gets the supported context.
     *
     * @param cs the CodingScheme
     *
     * @return the supported context
     */
    public Vector<SupportedContext> getSupportedContext(CodingScheme cs)
 	{
 		if (cs == null) return null;
        Vector<SupportedContext> v = new Vector<SupportedContext>();
 	    SupportedContext[] contexts = cs.getMappings().getSupportedContext();
 		for (int i=0; i<contexts.length; i++)
 		{
 		     SupportedContext sc = (SupportedContext) contexts[i];
 		     v.add(sc);
 		}
        return v;
	}



    /**
	 * Gets the supported context names.
	 *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
	 *
	 * @return the supported context
	 *
	 * @since DLBExt 1.0
	 */
    public Vector<String> getSupportedContextNames(String codingSchemeName, String vers, String ltag)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedContextNames(cs);
	}

    /**
     * Gets the supported context names.
     *
     * @param cs the cs
     *
     * @return the supported context names
     */
    public Vector<String> getSupportedContextNames(CodingScheme cs)
 	{
        Vector<SupportedContext> w = getSupportedContext(cs);
 		if (w == null) return null;

        Vector<String> v = new Vector<String>();
 		for (int i=0; i<w.size(); i++)
 		{
 		     SupportedContext sc = (SupportedContext) w.elementAt(i);
 		     v.add(sc.getContent());
 		}
        return v;
	}


    /**
     * Gets the supported sources.
     *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     *
     * @return the supported sources
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedSource> getSupportedSource(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedSource(cs);
	}


    /**
     * Gets the supported source.
     *
     * @param cs the CodingScheme
     *
     * @return the supported source
     */
    public Vector<SupportedSource> getSupportedSource(CodingScheme cs)
 	{
 		if (cs == null) return null;
        Vector<SupportedSource> v = new Vector<SupportedSource>();
 	    SupportedSource[] sources = cs.getMappings().getSupportedSource();
 		for (int i=0; i<sources.length; i++)
 		{
 		     SupportedSource src = (SupportedSource) sources[i];
 		     v.add(src);
 		}
        return v;
	}


    /**
     * Gets the supported source names.
     *
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     *
     * @return the supported source names
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedSourceNames(String codingSchemeName, String vers, String ltag)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName, vers, ltag);
        if (cs == null) return null;
        return getSupportedSourceNames(cs);
	}

    /**
     * Gets the supported source names.
     *
     * @param cs the CodingScheme
     *
     * @return the supported source names
     */
    public Vector<String> getSupportedSourceNames(CodingScheme cs)
 	{
        Vector<SupportedSource> w = getSupportedSource(cs);
 		if (w == null) return null;

        Vector<String> v = new Vector<String>();
 		for (int i=0; i<w.size(); i++)
 		{
 		     SupportedSource src = (SupportedSource) w.elementAt(i);
 		     //v.add(src.getContent());
 		     v.add(src.getLocalId());
 		}
        return v;
	}


   /**
    * Find coded entries with source code matching.
    *
    * @param codingSchemeName the coding scheme name
    * @param sourceAbbr the source abbr
    * @param code the concept code
    * @param maxToReturn the max to return
    *
    * @return the vector< coded entry>
    */
   public Vector<CodedEntry> findCodedEntriesWithSourceCodeMatching(String codingSchemeName, String sourceAbbr, String code, int maxToReturn)
   {
        return findCodedEntriesWithSourceCodeMatching(codingSchemeName, null, null, sourceAbbr, code, maxToReturn);
   }

   /**
    * Find coded entries with source code matching.
    *
    * @param codingSchemeName the coding scheme
    * @param vers the version
    * @param ltag the tag
    * @param sourceAbbr the source abbr
    * @param code the concept code
    * @param maxToReturn the max to return
    *
    * @return the vector< coded entry>
    */
   public Vector<CodedEntry> findCodedEntriesWithSourceCodeMatching(String codingSchemeName, String vers, String ltag,
                                                        String sourceAbbr, String code, int maxToReturn)
   {
		CodingScheme cs = getCodingScheme(codingSchemeName);
        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);

        LocalNameList contextList = null;
        NameAndValueList qualifierList = null;
        Vector<String> supported_qualifiers = getSupportedPropertyQualifiers(cs);
        if (supported_qualifiers.contains("source-code"))
        {
			if (code != null && code.compareTo("") != 0)
			{
				qualifierList = new NameAndValueList();
				qualifierList.addNameAndValue(createNameAndValue("source-code", code));
			}
		}

     // propertyLnL
        LocalNameList propertyLnL = null;
     /*
        Vector<String> w1 = new Vector<String>();

        String prop_name = "FULL_SYN";
        if (hasProperty(codingSchemeName, prop_name))
        {
	        w1.add(prop_name);
		}
		prop_name = "textualPresentation";
		if (hasProperty(codingSchemeName, prop_name))
		{
			w1.add(prop_name);
		}
        LocalNameList propertyLnL = vector2LocalNameList(w1);
     */

     // sourceLnL
        Vector<String> w2 = new Vector<String>();
        w2.add(sourceAbbr);
        LocalNameList sourceLnL = vector2LocalNameList(w2);

        if (sourceAbbr.compareTo("*") == 0)
        {
			sourceLnL = null;
		}

        Vector<CodedEntry> v = new Vector<CodedEntry>();

		SortOptionList sortCriteria =
			    //Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
			    Constructors.createSortOptionList(new String[]{"entityDescription", "code"});

		try {
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);//, false);
			if (cns == null)
			{
				return new Vector<CodedEntry>();
			}
			CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			cns = cns.restrictToProperties(propertyLnL, types, sourceLnL, contextList, qualifierList);

            //ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, propertyLnL, types, maxToReturn);
            ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, null, types, maxToReturn);
			ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			for (int i=0; i<rcrs.length; i++)
			{
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			}

		} catch (Exception e) {
			 getLogger().error("ERROR: Exception in findCodedEntriesWithSourceCodeMatching.");
			 e.printStackTrace();
			 return null;
		}
		return v;
    }

    /**
     * Creates the production tag.
     *
     * @return the coding scheme version or tag
     */
    public static CodingSchemeVersionOrTag createProductionTag()
    {
        CodingSchemeVersionOrTag tag = new CodingSchemeVersionOrTag();
        //tag.setTag(LBConstants.KnownTags.PRODUCTION.toString());
        tag.setTag("PRODUCTION");
        return tag;
    }

   /**
    * Creates the sort option.
    *
    * @param sortExtensionName the sort extension name
    * @param ascending true for ascending, false for descending
    *
    * @return the sort option
    */
   public static SortOption createSortOption(String sortExtensionName, Boolean ascending)
    {
        SortOption result = new SortOption();
        result.setExtensionName(sortExtensionName);
        result.setAscending(ascending);
        return result;
    }

    /**
     * Creates the sort option list.
     *
     * @param sortExtensionNames the sort extension names
     * @param ascending true for ascending, false for descending
     *
     * @return the sort option list
     *
     * @throws LBParameterException the LB parameter exception
     */
    public static SortOptionList createSortOptionList(String[] sortExtensionNames, Boolean[] ascending) throws LBParameterException
    {
        SortOptionList result = new SortOptionList();
        if (sortExtensionNames == null || ascending == null || sortExtensionNames.length != ascending.length)
        {
            throw new LBParameterException(
                    "Invalid Parameters - sortExtensionNames array must be the same length as the ascending array - and neither can be null");
        }
        for (int i = 0; i < ascending.length; i++)
        {
            SortOption temp = new SortOption();
            temp.setExtensionName(sortExtensionNames[i]);
            temp.setAscending(ascending[i]);
            result.addEntry(temp);
        }

        return result;
    }

    /**
     * Creates the sort option list.
     *
     * @param sortExtensionNames the sort extension names
     *
     * @return the sort option list
     */
    public static SortOptionList createSortOptionList(String[] sortExtensionNames)
    {
        SortOptionList result = new SortOptionList();
        if (sortExtensionNames == null)
        {
            return null;
        }
        for (int i = 0; i < sortExtensionNames.length; i++)
        {
            SortOption temp = new SortOption();
            temp.setExtensionName(sortExtensionNames[i]);
            temp.setAscending(null);
            result.addEntry(temp);
        }

        return result;
    }


    /**
     * Gets the semantic type code.
     *
     * @param name the name of the Semantic Type
     *
     * @return the semantic type code
     */
    public String getSemanticTypeCode(String name)
    {
		if (semanticTypeName2Code_hashmap == null)
		{
			initializeSemanticTypes();
		}
		if (semanticTypeName2Code_hashmap.containsKey(name))
		{
			String code = (String) semanticTypeName2Code_hashmap.get(name);
			return code;
		}
		else
		{
			return "Unknown";
		}
	}

    /**
     * Gets the semantic type codes.
     *
     * @param names the names of the Semantic Type
     *
     * @return the semantic type codes
     */
    public Vector<String> getSemanticTypeCodes(Vector<String> names)
    {
		Vector<String> u = new Vector<String>();
		for (int i=0; i<names.size(); i++)
		{
			String name = (String) names.elementAt(i);
			u.add(getSemanticTypeCode(name));
		}
		return u;
	}

	/**
	 * Initialize semantic types.
	 *
	 * @return true, if successful
	 */
	public boolean initializeSemanticTypes()
	{
		//LexBIGService lbSvc = new LexBIGServiceImpl();
		semanticType_vec = new Vector<String>();
		semanticType_hashmap = new HashMap<String,String>();
		semanticTypeName2Code_hashmap = new HashMap<String,String>();
		try {

			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(
					SEMANTIC_NET, createProductionTag());//, true);
            if (cns == null)
            {
				getLogger().error("SEMANTIC_NET data are either not available or accessible.");
				return false;
			}
			for (
				ResolvedConceptReferencesIterator concepts = cns.resolve(
					createSortOptionList(new String[] {"code"}),
					new LocalNameList(), null);
				concepts.hasNext(); )
			{
				ResolvedConceptReference ref = concepts.next();
				if (ref != null)
				{
					String semantictype = ref.getEntityDescription().getContent();
					if (semantictype.compareTo("topThing") != 0)
					{
						semanticType_vec.add(ref.getEntityDescription().getContent());
						semanticType_hashmap.put(ref.getConceptCode(), ref.getEntityDescription().getContent());
						semanticTypeName2Code_hashmap.put(ref.getEntityDescription().getContent(), ref.getConceptCode());
					}
			    }

			}
			return true;
	    } catch (Exception e) {
			//e.printStackTrace();
			getLogger().error("ERROR: Unable to initialize semantic types.");
			return false;
		}
		//return v;
	}

	/**
	 * Gets the semantic types.
	 *
	 * @return the semantic types
	 */
	public HashMap<String,String> getSemanticTypes()
	{
		if (semanticType_hashmap == null)
		{
			boolean retval = initializeSemanticTypes();
			if (!retval) return null;
		}
		return semanticType_hashmap;
	}

    /**
     * Gets the change type number.
     *
     * @param s the change type name ("CREATE","MODIFY", etc)
     *
     * @return the change type number
     */
    public int getChangeTypeNumber(String s)
    {
		if (s.compareToIgnoreCase("CREATE") == 0)
		{
			return ChangeType.CREATE_TYPE;
		}
		else if (s.compareToIgnoreCase("MERGE") == 0)
		{
			return ChangeType.MERGE_TYPE;
		}
		else if (s.compareToIgnoreCase("MODIFY") == 0)
		{
			return ChangeType.MODIFY_TYPE;
		}
		else if (s.compareToIgnoreCase("RETIRE") == 0)
		{
			return ChangeType.RETIRE_TYPE;
		}
		else if (s.compareToIgnoreCase("SPLIT") == 0)
		{
			return ChangeType.SPLIT_TYPE;
		}
		return -1;
	}

	/**
	 * Checks for existance of property in coding scheme.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param property the property
	 *
	 * @return true, if property exists in coding scheme
	 */
	protected boolean hasProperty(String codingSchemeName, String property)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName);
		SupportedProperty[] properties = cs.getMappings().getSupportedProperty();
		for (int i=0; i<properties.length; i++)
		{
			//String name = properties[i].getContent();
			String name = properties[i].getLocalId();
			if (name.compareTo(property) == 0) return true;
		}
		return false;

	}



    // 090506
    /**
     * Creates the code node set.
     *
     * @param conceptCodes the concept codes
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     *
     * @return the coded node set
     */
    protected CodedNodeSet createCodeNodeSet(String[] conceptCodes, String codingSchemeName,
        String vers, String ltag)
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
        try {
			ConceptReferenceList crl = createConceptReferenceList(conceptCodes, codingSchemeName);
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, versionOrTag);//, false);
			cns = cns.restrictToCodes(crl);
			return cns;
	    } catch (Exception e) {
			return null;
		}
    }

    // 090506
    /**
     * Gets the children of the given concept
     *
     * @param conceptCode the concept code
     * @param relationship the name of the relation container
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param buildReferencedEntries whether to build referenced entries
     *
     * @return the children of
     */
    protected Association[] getChildrenOf(String conceptCode, String relationship, String codingSchemeName,
            String vers, String ltag, boolean buildReferencedEntries) //throws LBException
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
	 	try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, relationship);
			cng = cng.restrictToSourceCodes(createCodeNodeSet(new String[]{conceptCode}, codingSchemeName, vers, ltag));
			int buildRefEntries = -1;
			if (buildReferencedEntries)
			{
				buildRefEntries = 1;
			}
			ResolvedConceptReferenceList rcrl = cng.resolveAsList(createConceptReference(conceptCode, codingSchemeName),
																  true, false, buildRefEntries, 1, null, null, null, 0);
			if (rcrl != null)
			{
				if (rcrl.getResolvedConceptReferenceCount() > 0)
				{
					AssociationList al = rcrl.getResolvedConceptReference(0).getSourceOf();
					Association[] associations = al.getAssociation();
					return associations;

				}
		    }
		} catch (Exception e) {
            getLogger().warn("WARNING: DLBWrapper getChildrenOf throws exception.");
	    }
        return null;
    }

    // 090506
    /**
     * Gets the parents of given concept.
     *
     * @param conceptCode the concept code
     * @param relationship the name of the relation container
     * @param codingSchemeName the coding scheme
     * @param vers the version
     * @param ltag the tag
     * @param buildReferencedEntries whether to build referenced entries
     *
     * @return the parents of
     */
    protected Association[] getParentsOf(String conceptCode, String relationship, String codingSchemeName,
                                     String vers, String ltag, boolean buildReferencedEntries)
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
	 	try {
			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, relationship);
             cng = cng.restrictToTargetCodes(createCodeNodeSet(new String[]{conceptCode}, codingSchemeName, vers, ltag));
			int buildRefEntries = -1;
			if (buildReferencedEntries)
			{
				buildRefEntries = 1;
			}
            ResolvedConceptReferenceList rcrl = null;
			rcrl = cng.resolveAsList(createConceptReference(conceptCode,codingSchemeName),
																  false, true, buildRefEntries, 1, null, null, null, 0);
			if (rcrl != null)
			{
				if (rcrl.getResolvedConceptReferenceCount() > 0)
				{
					AssociationList al = rcrl.getResolvedConceptReference(0).getTargetOf();
					Association[] associations = al.getAssociation();
					return associations;
				}
		    }
		} catch (Exception e) {
            getLogger().warn("WARNING: DLBWrapper getParentsOf throws exception.");
	    }

		return null;
    }



///////////////////////
//  Associations
///////////////////////

    /*
     * Enumerate Relationships (i.e., supported relationships in vocabulary)
     */

    /**
 * Gets the supported associations.
 *
 * @return the supported associations
 *
 * @since DLBExt 1.0
 */
    public Vector<SupportedAssociation> getSupportedAssociations()
	{
		Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();
		HashSet<String> hset = new HashSet<String>();
		Vector<CodingScheme> codingschemes = getSupportedCodingSchemes();
		for (int i=0; i<codingschemes.size(); i++)
		{
			CodingScheme cs = (CodingScheme) codingschemes.elementAt(i);
			SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
			for (int j=0; j<assos.length; j++)
			{
				 SupportedAssociation sa = (SupportedAssociation) assos[j];
				 if (!hset.contains(sa.getContent()))
				 {
				     v.add(sa);
				     hset.add(sa.getContent());
				 }
			}
	    }
        return v;
	}


    /**
     * Gets the supported associations.
     *
     * @param cs the CodingScheme
     *
     * @return the supported associations
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedAssociation> getSupportedAssociations(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();
	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa);
		}
        return v;
	}



    /**
     * Gets the supported associations.
     *
     * @param codingSchemeName a coding scheme name
     *
     * @return the supported associations
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedAssociation> getSupportedAssociations(String codingSchemeName)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName);
		if (cs == null) return null;

        Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();

	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa);
		}
        return v;
	}



    /**
     * Gets the supported association names.
     *
     * @param codingSchemeName a coding scheme name
     *
     * @return the supported association names
     *
     * @since DLBExt 1.0
     */
     // include roles and associations
    public Vector<String> getSupportedAssociationNames(String codingSchemeName)
	{
        CodingScheme cs = getCodingScheme(codingSchemeName);
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa.getLocalId());
		}
        return v;
	}

    /**
     * Gets the target names.
     *
     * @param rcrl the ResolvedConceptReferenceList
     *
     * @return the target names
     */@SuppressWarnings("unchecked")
    protected Set<String> getTargetNames(ResolvedConceptReferenceList rcrl) {
    	Set<String> targetNames = new TreeSet();
    	for (Enumeration rcrEnum = rcrl.enumerateResolvedConceptReference(); rcrEnum.hasMoreElements(); ) {
    		ResolvedConceptReference rcr = (ResolvedConceptReference) rcrEnum.nextElement();
    		AssociationList targetList = rcr.getSourceOf();
    		if (targetList.getAssociationCount() > 0)
	    		for (Enumeration targetEnum = targetList.enumerateAssociation(); targetEnum.hasMoreElements(); ) {
	    			AssociatedConceptList conceptList = ((Association) targetEnum.nextElement()).getAssociatedConcepts();
	    			for (Enumeration conceptEnum = conceptList.enumerateAssociatedConcept(); conceptEnum.hasMoreElements(); ) {
						Object obj = conceptEnum.nextElement();
						if (obj != null)
						{
							AssociatedConcept asso_concept = (AssociatedConcept) obj;
							if (asso_concept.getEntityDescription() != null)
							{
                            	targetNames.add(asso_concept.getEntityDescription().getContent());
							}
					    }
	    				//targetNames.add(((AssociatedConcept) conceptEnum.nextElement()).getEntityDescription().getContent());
	    			}
	    		}
    	}
    	return targetNames;
    }


    /**
     * Gets the supported association names.
     *
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     * @param type the association type (ROOT_ASSOCIATION or ROOT_ROLE)
     *
     * @return the supported association names
     */
    public Vector<String> getSupportedAssociationNames(String codingSchemeName, String vers, String type)
    {
		if (type == null)
		{
		    return getAssociationNamesByRelation(codingSchemeName, vers, ASSOCIATION_RELATION);
		}
		if (type.compareTo(ROOT_ASSOCIATION) != 0 && type.compareTo(ROOT_ROLE) != 0)
		{
		    return null;
	    }
		else if (type.compareTo(ROOT_ROLE) == 0)
		{
			return getAssociationNamesByRelation(codingSchemeName, vers, ROLE_RELATION);
		}
	    else if (type.compareTo(ROOT_ASSOCIATION) == 0)
	    {
			return getAssociationNamesByRelation(codingSchemeName, vers, ASSOCIATION_RELATION);
		}
		return null;

	}

	/**
	 * Gets the supported property qualifiers.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return the supported property qualifiers
	 */
	public Vector<String> getSupportedPropertyQualifiers(String codingSchemeName)
	{
		CodingScheme cs = getCodingScheme(codingSchemeName);
		return getSupportedPropertyQualifiers(cs);
	}

	/**
	 * Gets the supported property qualifiers.
	 *
	 * @param cs the CodingScheme
	 *
	 * @return the supported property qualifiers
	 */
	public Vector<String> getSupportedPropertyQualifiers(CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
        SupportedPropertyQualifier[] supported_qualifiers = cs.getMappings().getSupportedPropertyQualifier();
        for (int i=0; i<supported_qualifiers.length; i++)
        {
			//v.add(supported_qualifiers[i].getContent());
			v.add(supported_qualifiers[i].getLocalId());
		}
		return v;
	}


    /**
     * Convert LexBIG property array to evs Property object Vector.
     *
     * @param properties the LexBIG Properties object array
     *
     * @return the vector of evs Property objects
     */
    public Vector<Property> convertProperties(org.LexGrid.commonTypes.Property[] properties)
    {
		Vector<Property> v = new Vector<Property>();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			gov.nih.nci.lexbig.ext.Property prop
			   = new gov.nih.nci.lexbig.ext.Property(p.getProperty(), p.getText().getContent());
			PropertyQualifier[] qualifiers = p.getPropertyQualifier();
			for (int j=0; j<qualifiers.length; j++)
			{
				prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(qualifiers[j].getPropertyQualifierId(), qualifiers[j].getContent()));
			}
			//090107
			org.LexGrid.commonTypes.Source[] sources = p.getSource();
			for (int j=0; j<sources.length; j++)
			{
				org.LexGrid.commonTypes.Source src = (org.LexGrid.commonTypes.Source) sources[j];
				prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("Source", src.getContent()));
			}


			if (p instanceof org.LexGrid.concepts.Presentation)
			{
				Presentation presentation = (Presentation) p;
				java.lang.String form = presentation.getRepresentationalForm();
				if (form != null)
				{
					prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("term-group", form));
				}
				java.lang.String format = p.getPresentationFormat();
				if (format != null)
				{
					prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("format", format));
				}

			}

			v.add(prop);
		}
		return v;
	}


	/**
	 * Gets the properties for a coded entry.
	 *
	 * @param ce the CodedEntry
	 *
	 * @return the properties
	 */
	public Vector<Property> getProperties(CodedEntry ce)
	{
        if (ce == null)
        {
 			cat.error("wrapper.getProperties invalid input -- ce is null.");
			return null;
		}
        Vector<Property> v = new Vector<Property>();
		Vector<Property> w = convertProperties(ce.getDefinition());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getComment());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getInstruction());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getPresentation());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getConceptProperty());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		return v;
	}


	/*

	public Vector getProperties(CodedEntry ce)
    {
        if (ce == null)
        {
 			cat.error("wrapper.getProperties invalid input -- ce is null.");
			return null;
		}

        Vector v = new Vector();
        // Properties
		Definition[] definitions = ce.getDefinition();
		for (int i=0; i<definitions.length; i++)
		{
		    Definition d = definitions[i];
			String s = "definition" + "$" + d.getText().getContent();
			v.add(s);
		}

		Comment[] comments = ce.getComment();
		for (int i=0; i<comments.length; i++)
		{
		    Comment c = comments[i];
			String s = "comment" + "$" + c.getText().getContent();
			v.add(s);
		}

		Instruction[] instructions = ce.getInstruction();
		for (int i=0; i<instructions.length; i++)
		{
		    Instruction inst = instructions[i];
			String s = "instruction" + "$" + inst.getText().getContent();

			v.add(s);
		}

		Presentation[] presentations = ce.getPresentation();
		if (presentations != null)
		{
			for (int i=0; i<presentations.length; i++)
			{
				Presentation p = presentations[i];
				Vector source_vec = getPropertySources(p);
				if (source_vec.size() == 0)
				{
					//gov.nih.nci.lexrpc.client.Property property
					//   = new gov.nih.nci.lexrpc.client.Property();
					java.lang.String prop_name = p.getProperty();
					java.lang.String prop_value = p.getText().getContent();

					String s = prop_name + "$" + prop_value;
					v.add(s);
				}
			}
		}


 	    org.LexGrid.commonTypes.Property[] properties = ce.getConceptProperty();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			if (p != null)
			{
				String s = p.getProperty() + "$" + p.getText().getContent();
				v.add(s);
		    }
		}
		return v;
	}
	*/


	/**
	 * Gets the properties by code.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param code the code
	 *
	 * @return the properties by code
	 */
	public Vector<Property> getPropertiesByCode(String codingSchemeName, String vers, String code)
    {
		 if (code == null)
		 {
			 cat.error("WARNING: code for " + code + " not found.");
			 return null;
		 }

	    CodedEntry ce = getCodedEntry(codingSchemeName, vers, null, code);//, restrictproperties);
	    return getProperties(ce);
	}


	/**
	 * Gets the properties by code.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 * @param code the code
	 *
	 * @return the properties by code
	 */
	public Vector<Property> getPropertiesByCode(String codingSchemeName, String vers, String ltag,String code)
    {
		 if (code == null)
		 {
			 cat.error("WARNING: code for " + code + " not found.");
			 return null;
		 }

	    CodedEntry ce = getCodedEntry(codingSchemeName, vers, ltag, code);//, restrictproperties);
	    return getProperties(ce);
	}
//	public Vector<Property> getPropertiesByConceptName(String codingSchemeName, String versionName, String tagName, String conceptname)
//    {
////		Vector<Property> restrictproperties = null;
//        String code = nameToCode(codingSchemeName, versionName, tagName, conceptname);
//
//		 if (code == null)
//		 {
//			 cat.error("WARNING: code for " + conceptname + " not found.");
//			 return null;
//		 }
//
//	    CodedEntry ce = getCodedEntry(codingSchemeName, versionName, tagName, code);//, restrictproperties);
//	    return getProperties(ce);
//	}

	/**
 * Gets the property names.
 *
 * @param ce the CodedEntry
 *
 * @return the property names
 */
    public Vector<String> getPropertyNames(CodedEntry ce)
    {
		Vector<Property> w = getProperties(ce);
	    HashSet<String> hset = new HashSet<String>();
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
			String s = (String) w.elementAt(i).getName();
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			if (!hset.contains(name))
			{
				hset.add(name);
				v.add(name);
			}
		}
		hset.clear();
		return v;
    }

	/**
	 * Gets the property names.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag parameter not used
	 * @param conceptcode the concept code
	 *
	 * @return the property names
	 */
    public Vector<String> getPropertyNames(String codingSchemeName,
	                                 String vers,
									 String ltag,
									 String conceptcode)
    {
//		Vector<Property> w = getPropertiesByConceptName(codingScheme, version, tag, conceptname);
		Vector<Property> w = getPropertiesByCode(codingSchemeName, vers, ltag,conceptcode);
		if (w == null)
		{
			return null;
		}
	    HashSet<String> hset = new HashSet<String>();
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			String name = property.getName();
			/*
			String s = (String) w.elementAt(i);
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			*/
			if (!hset.contains(name))
			{
				hset.add(name);
				v.add(name);
			}
		}
		hset.clear();
		return v;
    }

	/**
	 * Gets the property values.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag parameter not used
	 * @param conceptcode the conceptcode
	 * @param propertyname the propertyname
	 *
	 * @return the property values
	 */
    public Vector<String> getPropertyValues(String codingSchemeName,
	                                 String vers,
									 String ltag,
									 String conceptcode,
									 String propertyname
									 )
    {
        Vector<String> v = new Vector<String>();
//        Vector<Property> w = getPropertiesByConceptName(codingScheme, version, tag, conceptname);
		Vector<Property> w = getPropertiesByCode(codingSchemeName, vers,ltag, conceptcode);
        if (w == null) return null;
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			if (property.getName().compareTo(propertyname) == 0)
			{
				v.add(property.getValue());
			}
			//080307
			/*
			String s = (String) w.elementAt(i);
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			String value = s.substring(pos+1, s.length());
			if (name.compareTo(propertyname) == 0) v.add(value);
			*/

		}
		return v;
    }

    /**
     * Gets the property values.
     *
     * @param ce the CodedEntry
     * @param propertyname the property name
     *
     * @return the property values
     */
    public Vector<String> getPropertyValues(CodedEntry ce, String propertyname)
    {
        Vector<String> v = new Vector<String>();
        Vector<Property> w = getProperties(ce);
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			if (property.getName().compareTo(propertyname) == 0)
			{
				v.add(property.getValue());
			}
		}
		return v;
    }

    /**
     * Gets property sources.
     *
     * @param prop the LexGrid Property object
     *
     * @return a Vector containing property sources
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getPropertySources(org.LexGrid.commonTypes.Property prop)
	{
		Vector<String> source_vec = new Vector<String>();
		/*
		java.lang.String[] sources = prop.getSource();
		if (sources != null)
		{
			for (int i=0; i<sources.length; i++)
			{
				source_vec.add(sources[i]);
			}
		}
		*/
		org.LexGrid.commonTypes.Source[] sources = prop.getSource();
		if (sources != null)
		{
			for (int i=0; i<sources.length; i++)
			{
				source_vec.add(sources[i].getContent());
			}
		}

		return source_vec;
 	}

    /**
     * Converts Array of Strings to Vector<String>.
     *
     * @param a the String array
     *
     * @return the vector< string>
     */
    public Vector<String> array2Vector(String[] a)
    {
		if (a == null) return null;
		Vector<String> v = new Vector<String>();
		if (a.length == 0) return v;
		for (int i=0; i<a.length; i++)
		{
			Object obj = (Object) a[i];
			v.add((String) obj);
		}
		return v;
	}


    // Version 2 beta
    /**
     * Gets the filter extensions.
     *
     * @return the filter extensions
     */
    public Vector<String> getFilterExtensions()
    {
		Vector<String> v = new Vector<String>();
		ExtensionDescriptionList edl = lbSvc.getFilterExtensions();
		ExtensionDescription[] ed_array = edl.getExtensionDescription();
		for (int i=0; i<ed_array.length; i++)
		{
     		ExtensionDescription ed = ed_array[i];
     		v.add(ed.getExtensionClass());
 		}
 		return v;
	}

	/**
	 * Gets the concept properties.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param code the concept code
	 * @param propertytype the propertytype
	 * @param propertyname the propertyname
	 *
	 * @return the concept properties
	 */
	public Vector<Property> getConceptProperties(String codingSchemeName, String code, String propertytype, String propertyname) {

		Vector<Property> v = new Vector<Property>();
        CodedEntry ce = getCodedEntry(codingSchemeName,
	                                null,
									null,
									code);
		if (ce == null)
		{
   			getLogger().error("Concept code " + code + " not found in " + codingSchemeName);
		    return null;
		}

		if (propertyname != null && propertyname.compareTo("*") == 0) propertyname = null;

		org.LexGrid.commonTypes.Property[] properties = null;
		if (propertytype.compareToIgnoreCase(TYPE_DEFINITION) == 0)
		{
			properties = ce.getDefinition();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_COMMENT) == 0)
		{
			properties = ce.getComment();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_PROPERTY) == 0)
		{
			properties = ce.getConceptProperty();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_INSTRUCTION) == 0)
		{
			properties = ce.getInstruction();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_PRESENTATION) == 0)
		{
			properties = ce.getPresentation();
		}

		if (properties == null)
		{
			getLogger().error("getConceptProperties returns null");
			return null;
		}

		else
		{
			for (int i=0; i<properties.length; i++)
			{
				org.LexGrid.commonTypes.Property p = properties[i];
				String prop_name = p.getProperty();
				if (propertyname == null || prop_name.compareTo(propertyname) == 0)
				{
					gov.nih.nci.lexbig.ext.Property prop
					   = new gov.nih.nci.lexbig.ext.Property(prop_name, p.getText().getContent());
                    PropertyQualifier[] qualifiers = p.getPropertyQualifier();
					//for (int j=0; j<qualifiers.length; j++)
					for (int j=0; j<qualifiers.length; j++)
					{
						prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(qualifiers[j].getPropertyQualifierId(), qualifiers[j].getContent()));
					}

					org.LexGrid.commonTypes.Source[] sources = p.getSource();
					for (int j=0; j<sources.length; j++)
					{
						org.LexGrid.commonTypes.Source src = (org.LexGrid.commonTypes.Source) sources[j];
						prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("Source", src.getContent()));
					}
					v.add(prop);
			    }
			}
		}
		return v;
	}

/*
    public String getHasSubtype(String codingSchemeName)
    {
//		Vector v = getSupportedRoleNames(codingSchemeName);

		Vector v = getAssociationNames(codingSchemeName);


		//073007
		if (v == null) return IS_A; // hasSubtype

		if (v.contains(IS_A))
		{
			return IS_A;
		}
		else if (v.contains(IS_A_OBO)) // is-a
		{
			return IS_A_OBO;
		}
		else if (v.contains(HAS_SUBCLASS))
		{
			return HAS_SUBCLASS;
		}

		else if (v.contains("PAR"))
		{
			return "PAR";
		}

		else
		{
			cat.error("hasSubtype association is not defined.");
			return null;
		}
	}
	*/
    //GF9458
    /**
 * Gets the checks for subtype.
 *
 * @param codingSchemeName the coding scheme name
 *
 * @return the checks for subtype
 */
public String getHasSubtype(String codingSchemeName)
    {
		Vector<String> v = getAssociationNames(codingSchemeName);
		if (v == null) return IS_A;
		for (int i = 0; i < HIER_ASSOC.length; i++)
		{
			String asso = HIER_ASSOC[i];
			if (v.contains(asso)) return asso;
		}
		System.out.println("WARNING: Hierarchical relationship for " + codingSchemeName + " cannot be identified.");
		return null;
	}

///////////////////////////////////////////////////////////////////////////////////////////////////
	/**
 * Search coded entries.
 *
 * @param codingSchemeName the coding scheme name
 * @param vers the version
 * @param ltag the tag
 * @param searchTerm the search term
 * @param limit parameter not used
 * @param source the source to search
 * @param cui parameter not used
 * @param shortResult parameter not used
 * @param score parameter not used
 *
 * @return the vector< coded entry>
 * @deprecated - use searchCodedEntries(String,String,String,String,String)
 */@Deprecated
@SuppressWarnings("unused")
public Vector<CodedEntry> searchCodedEntries(String codingSchemeName, String vers, String ltag, String searchTerm, int limit, String source,
						 boolean cui, boolean shortResult, boolean score)
	{
		SearchUtil searchUtil = new SearchUtil(codingSchemeName, vers, lbSvc);
	    return searchUtil.searchNodes(codingSchemeName, vers, ltag, searchTerm, limit, source,
						 cui, shortResult, score);
	}

/**
 * Search coded entries.
 *
 * @param codingSchemeName the coding scheme name
 * @param vers the version
 * @param ltag the tag
 * @param searchTerm the search term
 * @param source the source to search
 *
 * @return the vector< coded entry>
 */
public Vector<CodedEntry> searchCodedEntries(String codingSchemeName, String vers, String ltag, String searchTerm, String source)
	{
		SearchUtil searchUtil = new SearchUtil(codingSchemeName, vers, lbSvc);

	    return searchUtil.searchNodes(codingSchemeName, vers, ltag, searchTerm, source);
	}

	/**
	 * Restrict to source.
	 *
	 * @param CodedNodeSet the cns
	 * @param source the source
	 *
	 * @return the restricted coded node set
	 */
	protected CodedNodeSet restrictToSource(CodedNodeSet cns, String source)
	{
		if (source == null || source.compareTo("*") == 0) return cns;
		Vector<String> v = new Vector<String>();
		v.add(source);
	    LocalNameList sourceList = vector2LocalNameList(v);
		CodedNodeSet.PropertyType[] types = new CodedNodeSet.PropertyType[] {CodedNodeSet.PropertyType.PRESENTATION};
		try {

			cns = cns.restrictToProperties(null, types, sourceList, null, null);

		} catch (Exception e) {
   			getLogger().error("restrictToSource error. Source: " + source);
		    return null;
		}
		return cns;
	}

	/**
	 * Gets the preferred name.
	 *
	 * @param ce the CodedEntry
	 *
	 * @return the preferred name
	 */
	public String getPreferredName(CodedEntry ce)
	{
		 if (ce == null)
		 {
			 return null;
		 }
		 Presentation[] presentations = ce.getPresentation();
		 if (presentations == null) return ce.getConceptCode();
//		 String name = null;
		 for (int i=0; i<presentations.length; i++)
		 {
			 if (presentations[i].getIsPreferred().equals(Boolean.TRUE))
			 {
				 return presentations[i].getText().getContent();
			 }
		 }
		 return null;
	}

	/**
	 * Gets the name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 * @param code the concept code
	 *
	 *
	 * @return the name
	 */
	public String getConceptName(String codingSchemeName, String vers, String ltag, String code)
	{
		CodedEntry ce = getCodedEntryByCode(codingSchemeName, vers, ltag, code);
		String name = getName(ce);
		if (name != null) return name;
		return getConceptName(codingSchemeName, vers, code);
	}

	/**
	 * Gets the name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 * @param code the concept code
	 *
	 * @deprecated use getConceptName(String,String,String,String)
	 * @return the name
	 */@Deprecated
	public String getName(String codingSchemeName, String vers, String ltag, String code)
	{
		CodedEntry ce = getCodedEntryByCode(codingSchemeName, vers, ltag, code);
		String name = getName(ce);
		if (name != null) return name;
		return codeToName(codingSchemeName, vers, code);
	}

	/**
	 * Gets the name.
	 *
	 * @param ce the CodedEntry
	 *
	 * @return the name
	 *
	 * @deprecated use getConceptName(CodedEntry)
	 */@Deprecated
	public String getName(CodedEntry ce)
	{
		 if (ce == null)
		 {
			 return null;
		 }
		 String pt = getPreferredName(ce);
		 //KLO, 081107
		 if (pt != null) return pt;
		 Presentation[] presentations = ce.getPresentation();
		 if (presentations == null || presentations.length == 0) return ce.getConceptCode();
		 return presentations[0].getText().getContent();
	}

	/**
	 * Gets the concept name.
	 *
	 * @param ce the CodedEntry
	 *
	 * @return the concept name
	 */
	public String getConceptName(CodedEntry ce)
	{
		 if (ce == null)
		 {
			 return null;
		 }
		 String pt = getPreferredName(ce);
		 //KLO, 081107
		 if (pt != null) return pt;
		 Presentation[] presentations = ce.getPresentation();
		 if (presentations == null || presentations.length == 0) return ce.getConceptCode();
		 return presentations[0].getText().getContent();
	}

    /**
     * Creates the concept reference list.
     *
     * @param codes the list of codes to get concept references for
     * @param codingSchemeName the coding scheme name
     *
     * @return the concept reference list
     */
    public static ConceptReferenceList createConceptReferenceList(String[] codes, String codingSchemeName)
    {
        if (codes == null)
        {
            return null;
        }
        ConceptReferenceList list = new ConceptReferenceList();
        for (int i = 0; i < codes.length; i++)
        {
            ConceptReference cr = new ConceptReference();
            cr.setCodingScheme(codingSchemeName);
            cr.setConceptCode(codes[i]);
            list.addConceptReference(cr);
        }
        return list;
    }





	 /**
 	 * Gets the coded entry sources.
 	 *
 	 * @param ce the CodedEntry
 	 *
 	 * @return the coded entry sources
 	 */
 	protected Vector<String> getCodedEntrySources(CodedEntry ce)
     {
	    Vector<String> v = new Vector<String>();
	    HashSet<String> hset = new HashSet<String>();
        java.util.Enumeration enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
		    Source[] sources = property.getSource();
		    for (int i=0; i<sources.length; i++)
		    {
				Source src = (Source) sources[i];
				String name = src.getContent();
				if (!hset.contains(name))
				{
					v.add(name);
					hset.add(name);
				}
			}
		}

		return v;
    }

    // KLO, 080307
	/**
     * Gets the source collectionby code.
     *
     * @param codingSchemeName the coding scheme name
     * @param code the concept code
     *
     * @return the source collection by code
     */
    public Vector<String> getSourceCollectionbyCode(String codingSchemeName, String code) {
	    CodedEntry ce = getCodedEntry(codingSchemeName, null, null, code);
	    if (ce == null)
	    {
			//System.out.println("Concept " + code + " not found.");
			return null;
		}
	    Vector<String> v = new Vector<String>();
	    HashSet<String> hset = new HashSet<String>();
        java.util.Enumeration enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
		    Source[] sources = property.getSource();
		    for (int i=0; i<sources.length; i++)
		    {
				Source src = (Source) sources[i];
				String name = src.getContent();
				if (!hset.contains(name))
				{
					v.add(name);
					hset.add(name);
				}
			}
		}
		return v;
	}


	/**
	 * Checks if is forward name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param directionalName the directional name
	 *
	 * @return the boolean
	 */


	public Boolean isForwardName(java.lang.String codingSchemeName, java.lang.String vers, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(vers);
		try {
			LexBIGServiceConvenienceMethodsImpl lbscm = (LexBIGServiceConvenienceMethodsImpl)
				 lbSvc.getGenericExtension("LexBIGServiceConvenienceMethods");
			lbscm.setLexBIGService(lbSvc);

            //[#14277] GetHasSubtype throws an exception
		    //return new Boolean(lbConvMethods.isForwardName(codingSchemeName, versionOrTag, directionalName));
		    return new Boolean(lbscm.isForwardName(codingSchemeName, versionOrTag, directionalName));
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}


	/**
	 * Checks if is reverse name.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param directionalName the directional name
	 *
	 * @return the boolean
	 */

	public Boolean isReverseName(java.lang.String codingSchemeName, java.lang.String vers, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(vers);
		try {
			LexBIGServiceConvenienceMethodsImpl lbscm = (LexBIGServiceConvenienceMethodsImpl)
				 lbSvc.getGenericExtension("LexBIGServiceConvenienceMethods");
			lbscm.setLexBIGService(lbSvc);

            //[#14277] GetHasSubtype throws an exception
		    //return new Boolean(lbConvMethods.isReverseName(codingSchemeName, versionOrTag, directionalName));
		    return new Boolean(lbscm.isReverseName(codingSchemeName, versionOrTag, directionalName));
		} catch (Exception e) {
			return null;
		}
	}

/*
	public Boolean isForwardName(java.lang.String codingSchemeName, java.lang.String vers, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(vers);
		try {
		    return lbConvMethods.isForwardName(codingSchemeName, versionOrTag, directionalName);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}


	public Boolean isReverseName(java.lang.String codingSchemeName, java.lang.String vers, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(vers);
		try {
		    return lbConvMethods.isReverseName(codingSchemeName, versionOrTag, directionalName);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
*/

 	/**
	  * Gets the coding scheme summary.
	  *
	  * @param codingSchemeName the coding scheme name
	  * @param vers the version
	  *
	  * @return the coding scheme summary
	  */
	 public CodingSchemeSummary getCodingSchemeSummary(String codingSchemeName, String vers)
	{
		try {
			CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = lcsrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					if (css.getRepresentsVersion().compareTo(vers) == 0)
					{
						return css;
					}
				}
			}
	    } catch (Exception e) {
			return null;
		}
		return null;
	}

 	/**
	  * Gets the coding scheme summary.
	  *
	  * @param codingSchemeName the coding scheme name
	  *
	  * @return the coding scheme summary
	  */
	 public CodingSchemeSummary getCodingSchemeSummary(String codingSchemeName)
	{
		try {
			CodingSchemeRenderingList lcsrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = lcsrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					return css;
				}
			}
	    } catch (Exception e) {
			return null;
		}
		return null;
	}

	/**
	 * Gets the represents version.
	 *
	 * @param codingSchemeName the coding scheme name
	 *
	 * @return the version
	 */
	public String getRepresentsVersion(String codingSchemeName)
	{
		CodingSchemeSummary css = getCodingSchemeSummary(codingSchemeName);
		if (css == null) return null;
		return css.getRepresentsVersion();
	}


	/**
	 * Gets the sources.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 * @param code the code
	 * @param relation the relation
	 * @param source the source
	 *
	 * @return the sources
	 */
	protected Vector<AssociatedConcept> getSources(String codingSchemeName, String vers, String ltag, String code, String relation, String source)
	{
        Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
        CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(vers, ltag);

		try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, csvt, null);
			if (relation != null)
			{
				NameAndValue nv = new NameAndValue();
				NameAndValueList nvList = new NameAndValueList();
				nv.setName(relation);
				nvList.addNameAndValue(nv);
				cng = cng.restrictToAssociations(nvList, null);
			}
			if (source != null && source.compareTo("*") != 0)
			{
				CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, csvt);
				cns = restrictToSource(cns, source);
				cng = cng.restrictToSourceCodes(cns);
			}

			ResolvedConceptReferenceList matches =
				cng.resolveAsList(
					ConvenienceMethods.createConceptReference(code, codingSchemeName),
					false, true, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList targetof = ref.getTargetOf();
				Association[] associations = targetof.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return new Vector<AssociatedConcept>();
		}
		return v;
	}


	/**
	 * Gets the targets.
	 *
	 * @param codingSchemeName the coding scheme name
	 * @param vers the version
	 * @param ltag the tag
	 * @param code the code
	 * @param relation the relation
	 * @param source the source
	 *
	 * @return the targets
	 */
	protected Vector<AssociatedConcept> getTargets(String codingSchemeName, String vers, String ltag, String code, String relation, String source)
	{
        Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
        CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(vers, ltag);

        try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingSchemeName, csvt, null);
			if (relation != null)
			{
				NameAndValue nv = new NameAndValue();
				NameAndValueList nvList = new NameAndValueList();
				nv.setName(relation);
				nvList.addNameAndValue(nv);
				cng = cng.restrictToAssociations(nvList, null);
			}
			if (source != null && source.compareTo("*") != 0)
			{
				CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingSchemeName, csvt);
				cns = restrictToSource(cns, source);
				cng = cng.restrictToTargetCodes(cns);
			}


			ResolvedConceptReferenceList matches =
					cng.resolveAsList(
						ConvenienceMethods.createConceptReference(code, codingSchemeName),
						true, false, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList sourceof = ref.getSourceOf();
				Association[] associations = sourceof.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return new Vector<AssociatedConcept>();
		}
		return v;
	}


    // GF9072
    /**
     * Checks if is coding scheme active.
     *
     * @param local_lbSvc the LexBIGService
     * @param codingSchemeName the coding scheme name
     * @param vers the version
     *
     * @return the boolean
     */
    public Boolean isCodingSchemeActive(LexBIGService local_lbSvc, String codingSchemeName, String vers)
    {
		try {
			Enumeration<CodingSchemeRendering> schemes = local_lbSvc.getSupportedCodingSchemes().enumerateCodingSchemeRendering();
			while (schemes.hasMoreElements()) {
				CodingSchemeRendering rendering = schemes.nextElement();
				CodingSchemeSummary summary = rendering.getCodingSchemeSummary();
				if (codingSchemeName.equalsIgnoreCase(summary.getCodingSchemeURN()) ||
						codingSchemeName.equalsIgnoreCase(summary.getFormalName()) ||
						codingSchemeName.equalsIgnoreCase(summary.getLocalName()) )
				{
					if (vers != null)
					{
						if (vers.equalsIgnoreCase(summary.getRepresentsVersion()))
						{
							boolean isActive = rendering.getRenderingDetail().getVersionStatus().equals(CodingSchemeVersionStatus.ACTIVE);
							return new Boolean(isActive);
						}
					}
					else
					{
						boolean isActive = rendering.getRenderingDetail().getVersionStatus().equals(CodingSchemeVersionStatus.ACTIVE);
						return new Boolean(isActive);
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
	    return null;
	}


    /**
     * Gets the related concepts.
     *
     * @param codingSchemeName the coding scheme name
     * @param conceptCode the concept code
     * @param sourceAbbr the source abbr
     * @param relation the relation
     *
     * @return the related concepts
     *
     * @throws Exception the exception
     */
    public ArrayList<String> getRelatedConcepts(String codingSchemeName, String conceptCode, String sourceAbbr, String relation) throws Exception
    {
		return getRelatedConcepts(codingSchemeName, conceptCode, sourceAbbr, relation, true);
    }

    /**
     * Gets the related concepts.
     *
     * @param codingSchemeName the coding scheme name
     * @param conceptCode the concept code
     * @param sourceAbbr the source abbr
     * @param relation the name of the relation container
     * @param navigateForward true if concept is source of association, false if target
     *
     * @return the related concepts
     *
     * @throws Exception the exception
     */
    public ArrayList<String> getRelatedConcepts(String codingSchemeName, String conceptCode, String sourceAbbr, String relation,
        boolean navigateForward) throws Exception
    {
		ArrayList<String> list = new ArrayList<String>();
		CodingSchemeVersionOrTag csvt = new CodingSchemeVersionOrTag();

	    ResolvedConceptReferenceList rcrl = NavigationUtils.getAssociatedNodes(
		     this.lbSvc,
		     codingSchemeName,
			 sourceAbbr,
			 csvt,
			 conceptCode,
			 relation,
			 navigateForward);

		if (rcrl != null && rcrl.getResolvedConceptReferenceCount() > 0)
		{
			for (Iterator<ResolvedConceptReference> nodes = rcrl.iterateResolvedConceptReference(); nodes.hasNext(); )
			{
				ResolvedConceptReference ref = nodes.next();
				String code = ref.getConceptCode();
				list.add(code);
			}
		}
        return list;
   }

   // 112507
   /**
    * Gets the lex big service.
    *
    * @return the LexBIGService
    */
   public LexBIGService getLexBIGService() {
	   return this.lbSvc;
   }


    /**
     * Gets the property collection by codes.
     *
     * @param vocabularyName the vocabulary name
     * @param propertyNames a list of property names (i.e., "Synonym", "FULL_SYN")
     * @param conceptCodes a list of concept codes
     * @return a list of CodedEntry with at least one of the specified properties
     *
     */

	public Vector<CodedEntry> fetchPropertyCollectionByCodes(
		                                        String codingScheme,
		                                        ArrayList propertyNameList,
		                                        ArrayList conceptCodes)

	{
         Vector<CodedEntry> v = new Vector<CodedEntry>();
		 CodingSchemeVersionOrTag tagOrVersion = new CodingSchemeVersionOrTag();
		 //if (version != null) tagOrVersion.setVersion(version);
	 	 try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, null);//, activeOnly);
			 String[] codes = new String[conceptCodes.size()];
			 for (int i=0; i<conceptCodes.size(); i++)
			 {
				codes[i] = (String) conceptCodes.get(i);
			 }

             ConceptReferenceList codeList = createConceptReferenceList(codes, codingScheme);
             cns = cns.restrictToCodes(codeList);

             LocalNameList propNameList = null;
			 if (propertyNameList != null && propertyNameList.size() > 0)
			 {
				propNameList = new LocalNameList();
				for (int i=0; i<propertyNameList.size(); i++)
				{
					String vEntry = (String) propertyNameList.get(i);
					propNameList.addEntry(vEntry);
				}
			 }

			 // Sort by search engine recommendation & code ...
			 SortOptionList sortCriteria =
			    //Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});
			    Constructors.createSortOptionList(new String[]{"entityDescription"});

             ResolvedConceptReferenceList rcrl = null;
             try {
				 rcrl = cns.resolveToList(sortCriteria, propNameList, null, codes.length);
			 } catch (Exception ex) {
				 //System.out.println("WARNING: resolveToList throws exception.");
				 return v;
			 }

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			 ResolvedConceptReference rcr = null;
			 for (int i=0; i<rcrs.length; i++)
			 {
				 rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
			 return v;

		 } catch (Exception e) {
			 e.printStackTrace();

		 }
		 return v;
	}

	protected Vector<ConceptAssociation> getAssociations(
		                                  String codingSchemeName,
										  String vers,
										  String ltag,
										  String code,
										  boolean direction,
										  String relationContainer,
										  Vector asso_names
										  )
	{
		LexBIGService lbSvc = getLexBIGService();
        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(vers, ltag);
        CodingScheme cs = null;
        try {
        	cs = lbSvc.resolveCodingScheme(codingSchemeName, versionOrTag);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
        if (cs == null) return null;

        if (relationContainer != null)
        {
			Relations[] relations = cs.getRelations();
			boolean containerDefined = false;
			for (int i=0; i<relations.length; i++)
			{
				Relations relation = (Relations) relations[i];
				String dc = relation.getDc();
				if (dc.compareTo(relationContainer) == 0)
				{
					containerDefined = true;
					if (asso_names == null)
					{
						asso_names = new Vector();
						org.LexGrid.relations.Association[] associations = relation.getAssociation();
						for (int j=0; j<associations.length; j++)
						{
							org.LexGrid.relations.Association association = (org.LexGrid.relations.Association) associations[j];
							asso_names.add(association.getAssociation());
						}
						break;
				    }
				}
			}
			if (!containerDefined) return new Vector();
	    }

		Vector<ConceptAssociation> v = new Vector();
		ResolvedConceptReferenceList matches = null;
		try {
		   matches = lbSvc.getNodeGraph(codingSchemeName, versionOrTag, null)
				.resolveAsList(
					ConvenienceMethods.createConceptReference(code, codingSchemeName),
					direction, !direction, 1, 1, new LocalNameList(), null, null, 1024);
		} catch (Exception ex) {
			return null;
		}

		if (matches.getResolvedConceptReferenceCount() > 0) {
			Enumeration<ResolvedConceptReference> refEnum = matches.enumerateResolvedConceptReference();

			while (refEnum.hasMoreElements()) {
				ResolvedConceptReference ref = refEnum.nextElement();
				if (ref != null)
				{
					AssociationList sourceof = ref.getSourceOf();
					if (!direction)
					{
						sourceof = ref.getTargetOf();
					}
					org.LexGrid.LexBIG.DataModel.Core.Association[] associations = sourceof.getAssociation();
					if (associations != null)
					{
						for (int i = 0; i < associations.length; i++) {
							org.LexGrid.LexBIG.DataModel.Core.Association assoc = associations[i];

							assoc = processForAnonomousNodes(assoc);

							if (asso_names == null)
							{
								AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
								for (int j = 0; j < acl.length; j++) {
									AssociatedConcept ac = acl[j];
									if (ac.getConceptCode() != null)
									{
										//if (!isAnonymous(codingSchemeName, vers, ltag, ac.getConceptCode()))
                                        {
											v.add(new ConceptAssociation(assoc.getAssociationName(), ac.getConceptCode()));
										}
									}
								}
							}
							else if (asso_names.contains(assoc.getAssociationName()))
							{
								AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
								for (int j = 0; j < acl.length; j++) {
									AssociatedConcept ac = acl[j];
									if (ac.getConceptCode() != null)
									{
										//if (!isAnonymous(codingSchemeName, vers, ltag, ac.getConceptCode()))
                                        {
											v.add(new ConceptAssociation(assoc.getAssociationName(), ac.getConceptCode()));
										}
									}
								}
							}
						}
					}
			    }
			}
		}
		return v;
	}


    protected Association processForAnonomousNodes(Association assoc){
		//clone Association except associatedConcepts
		Association temp = new Association();
		temp.setAssociatedData(assoc.getAssociatedData());
		temp.setAssociationName(assoc.getAssociationName());
		temp.setAssociationReference(assoc.getAssociationReference());
		temp.setDirectionalName(assoc.getDirectionalName());
		temp.setAssociatedConcepts(new AssociatedConceptList());


		for(int i = 0; i < assoc.getAssociatedConcepts().getAssociatedConceptCount(); i++)
		{
			//Conditionals to deal with anonymous nodes and UMLS top nodes "V-XXX"
			//The first three allow UMLS traversal to top node.
			//The last two are specific to owl anonymous nodes which can act like false
			//top nodes.
			if(
				assoc.getAssociatedConcepts().getAssociatedConcept(i).getReferencedEntry() != null &&
				assoc.getAssociatedConcepts().getAssociatedConcept(i).getReferencedEntry().getIsAnonymous() != null &&
				assoc.getAssociatedConcepts().getAssociatedConcept(i).getReferencedEntry().getIsAnonymous() != false &&
				!assoc.getAssociatedConcepts().getAssociatedConcept(i).getConceptCode().equals("@") &&
				!assoc.getAssociatedConcepts().getAssociatedConcept(i).getConceptCode().equals("@@")
				)
			{
				//do nothing
			}
			else{
				temp.getAssociatedConcepts().addAssociatedConcept(assoc.getAssociatedConcepts().getAssociatedConcept(i));
			}
		}
		return temp;
	}

    /*
	public int getSystemMaxResultSize()
	{
    	return ResourceManager.instance().getSystemVariables().getMaxResultSize();
	}
	*/


}

